#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO .
# -----------
# Genera una imagen de captcha y devuelve un arreglo JSON con los datos para usarla.
# --------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibs = $Bin;
    unshift(@INC, $pathLibs);
    $pathLibs =~ s/\/[^\/]+$/\/cgi\-cpn/;
    unshift(@INC,$pathLibs);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::SHA;
use File::Copy;
use GD::Image;
use JSON;
use POSIX;
use glib_cgi_05;
use glib_fildir_03;
use glib_html_03;
use glib_str_03;
use lib_captcha2;
use lib_ipcheck;
use lib_lang;
use lib_maxrunning;
use lib_utils;
##/libs##

main: {
    lib_utils::setUp();
    # Soporta un maximo de n copias corriendo.
    if (lib_maxrunning::maxExcedido(200)) {
        glib_html_03::print_json_result_hash({'img' => '', 'path' => '', 'code' => '', 'msg' => lib_lang::get_message('captcha_msg_busy')});
    }

    glib_cgi_05::new();

    # Tipo de captcha
    my $captcha_type = glib_cgi_05::param('_type');
    if(! lib_captcha2::validar_tipo($captcha_type)) {
        lib_stdlog::print_err(lib_lang::get_interpolated_message("bin_captcha_invalid_captcha_type", $captcha_type)."\n");
        glib_html_03::print_json_result_hash({'img' => '', 'path' => '', 'code' => '', 'msg' => lib_lang::get_message('captcha_msg_type')});
    }

    # Validacion y gestion de ip bloqueada
    my $user_ip = $ENV{'REMOTE_ADDR'};

    my $maxrequest_por_ip = 100;
    my $bloqueoip_interval = 60;
    my $bloquear_ip = lib_ipcheck::check_bloqueo_ip($prontus_varglb::DIR_IP_CONTROL, $user_ip, $maxrequest_por_ip, $bloqueoip_interval);
    if ($bloquear_ip) {
        glib_html_03::print_json_result_hash({'img' => '', 'path' => '', 'code' => '', 'msg' => lib_lang::get_message('captcha_msg_blocked')});
    }

    # largo por default
    my $captcha_length = glib_cgi_05::param('_captcha_length') || 4;

    my $path_to_img = "$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type/";
    my $full_path_to_img = "$prontus_varglb::DIR_SERVER$path_to_img";

    # Garbage collector, borra archivos siempre y cuando el numero de archivos a sido superado.
    lib_captcha2::launch_garbage_collector($full_path_to_img);

    # Crear directorio de imagenes si no existe.
    glib_fildir_03::check_dir($full_path_to_img);

    my $captcha_code_clear = glib_str_03::random_string($captcha_length);
    my $captcha_code_hash = lib_captcha2::make_hash($captcha_code_clear);

    my $captcha_img_name = lib_captcha2::get_img_name($captcha_code_hash);

    my $make_captcha_img_resp = lib_captcha2::make_captcha_img($captcha_code_clear, "$full_path_to_img/$captcha_img_name", $captcha_length);
    if ($make_captcha_img_resp ne '') {
        glib_html_03::print_json_result_hash({'img' => '', 'path' => '', 'code' => '', 'msg' => lib_lang::get_message('captcha_' .  $make_captcha_img_resp)});
    } else {
        glib_html_03::print_json_result_hash({'img' => $captcha_img_name, 'path' => $path_to_img, 'code' => $captcha_code_hash, 'msg' => 'captcha_ok'});
    }
}
