#!/usr/bin/perl
#
# Control por señales de procesos prontus
#
#

use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    # Para dejar disponibles las librerias de prontus
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use glib_fildir_03;
use glib_hrfec_03;
use lib_lang;
use lib_utils;
##/libs##

main: {
    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_use_instructions');
        print "perl $0 <stop|reload_conf|clear_vars|reset_url_cache>\n";
    }

    my %conf = %{lib_utils::getHash("$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/fcgi_fpm/fcgi.cfg")};

    my $proc_name = $prontus_varglb::FCGI_CONFIG{'fcgi_proc_name'};
    # Asigna valor de configuracion si existe
    if (defined($conf{'procs_id'})) {
        $proc_name = $prontus_varglb::FCGI_CONFIG{'fcgi_proc_name'} . '-' .$conf{'procs_id'};
    }

    my $bin_proc = 0;
    if (defined($conf{'front_proc'})) {
        $bin_proc = $conf{'front_proc'};
    }

    my $proc_name_bin = '';
    if ($bin_proc) {
        $proc_name_bin = $proc_name . '-front';
    }

    my $proc_name_f5 = $prontus_varglb::FCGI_CONFIG{'f5_proc_name'};
    # Asigna valor de configuracion si existe
    if (defined($conf{'procs_id'})) {
        $proc_name_f5 = $prontus_varglb::FCGI_CONFIG{'f5_proc_name'} . '-' .$conf{'procs_id'};
    }

    my $type = $ARGV[0] || '';
    # 10) SIGUSR1
    my $sigusr1 = '-s 10';
    # 12) SIGUSR2
    my $sigusr2 = '-s 12';

    my $signal_master_fpm = 0;
    my $signal_worker_fpm = 0;
    my $signal_master_f5 = 0;
    my $signal_worker_f5 = 0;
    my $signal = '';
    if ($type eq 'stop') {
        $signal_master_fpm = 1;
        $signal_master_f5 = 1;
    } elsif ($type eq 'reload_conf') {
        $signal_worker_fpm = 1;
        $signal_worker_f5 = 1;
        $signal = $sigusr1;
    } elsif ($type eq 'clear_vars') {
        $signal_worker_fpm = 1;
        $signal_worker_f5 = 1;
        $signal = $sigusr2;
    } elsif ($type eq 'reset_url_cache') {
        $signal_worker_f5 = 1;
        $signal = $sigusr2;
    }
    print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "][$type]\n";
    print STDERR "[$proc_name] [$proc_name_bin] [$proc_name_f5]\n";

    my $res;
    # señales a masters
    if ($signal_master_f5) {
        $res = `ps xww | grep -v grep | grep $proc_name_f5 | awk -F " " '{system("kill $signal "\$1)}'`;
    }

    if ($signal_master_fpm) {
        if ($bin_proc) {
            $res = `ps xww | grep -v grep | grep $proc_name_bin | awk -F " " '{system("kill $signal "\$1)}'`;
        }
        $res = `ps xww | grep -v grep | grep $proc_name | awk -F " " '{system("kill $signal "\$1)}'`;
    }

    # señales a workers
    $proc_name = $prontus_varglb::FCGI_CONFIG{'f5_worker_name'};
    # Asigna valor de configuracion si existe
    if (defined($conf{'procs_id'})) {
        $proc_name = $prontus_varglb::FCGI_CONFIG{'f5_worker_name'} . '-' .$conf{'procs_id'};
    }

    $proc_name_bin = '';
    if ($bin_proc) {
        $proc_name_bin = $proc_name . '-front';
    }

    $proc_name_f5 = $prontus_varglb::FCGI_CONFIG{'fcgi_worker_name'};
    # Asigna valor de configuracion si existe
    if (defined($conf{'procs_id'})) {
        $proc_name_f5 = $prontus_varglb::FCGI_CONFIG{'fcgi_worker_name'} . '-' .$conf{'procs_id'};
    }

    if ($signal_worker_f5) {
        $res = `ps xww | grep -v grep | grep $proc_name_f5 | awk -F " " '{system("kill $signal "\$1)}'`;
    }

    if ($signal_worker_fpm) {
        if ($bin_proc) {
            $res = `ps xww | grep -v grep | grep $proc_name_bin | awk -F " " '{system("kill $signal "\$1)}'`;
        }
        $res = `ps xww | grep -v grep | grep $proc_name | awk -F " " '{system("kill $signal "\$1)}'`;
    }
}
