#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Manipulación general de archivos y directorios locales .
#---------------------------------------------------------------
# DECLARACIONES GLOBALES.
#------------------------

package glib_fildir_03;
use utf8;
use strict;
use File::Copy;

# reemplaza instrucciones SSI por lectura del archivo usando read_file();
sub parse_ssi {
    my ($dir_server, $buffer) = @_;
    $buffer =~ s/<!--#include virtual="([^"]+?)" *-->/&read_file($dir_server . $1)/sge;
    return $buffer;
}

# Recibe ruta y nombre del archivo a buscar (sin extension).
# Devuelve solo extension (sin punto) del primer archivo que encuentre. Vacio si no lo encuentra.
sub get_extension {
    my ($dir, $nom_file) = @_;
    my (@files, $i, $ext);

    $ext = '';

    if (-d $dir) {     # Verifica si existe directorio.
        @files = &lee_dir($dir);    # Obtiene lista de archivos del directorio solicitado.

        for ($i = 0 ; $#files >= $i ; $i++) {
            if ($files[$i] =~ /^$nom_file/) {    # Busca coincidencia del archivo.
                $files[$i] =~ /\.(.*)$/g;
                $ext = $1;                       # Asigna extension.
                last;                            # Sale del for.
            }
        }
    }
    return $ext;
}

# Rutina RECURSIVA. Copia un directorio completo con archivos y subdirs.
# Param. de entrada :
# 0) Dir padre del dir origen, path completo sin / al final
# 1) Nombre del Dir. a copiar sin path (este es el dir origen).
# 2) Dir donde en su interior se creara el dir a copiar, path completo sin / al final
# 3) Nuevo nombre que se le va a poner al Dir. al copiarlo, sin path. Este param. tiene
#        efecto solo la primera vez que se invoca a la rutina.
sub copy_tree {
    my ($padre_origen, $df_origen, $df_destino, $nom_destino) = @_;

    my $ret = 0;
    if (-d $padre_origen . '/' . $df_origen) {
        $ret = opendir(dir_handle, $padre_origen . '/' . $df_origen);
    }
    # si es un directorio
    if ($ret) {
        if ($nom_destino ne '') {
            mkdir($df_destino . '/' . $nom_destino, 493);
        } else {
            mkdir($df_destino . '/' . $df_origen, 493);
        }

        my @list_dir = readdir dir_handle;
        close dir_handle;
        foreach my $item (@list_dir) {
            next if (($item eq '.') || ($item eq '..'));
            if ($nom_destino ne '') {
                &copy_tree($padre_origen . '/' . $df_origen, $item, $df_destino . '/' . $nom_destino);
            } else {
                &copy_tree($padre_origen . '/' . $df_origen, $item, $df_destino . '/' . $df_origen);
            }

        }
    } else {    # Es un archivo
        if (-l $padre_origen . '/' . $df_origen) {
            system("cp -a $padre_origen/$df_origen $df_destino/$df_origen");
        } else {
            File::Copy::copy($padre_origen . '/' . $df_origen, $df_destino . '/' . $df_origen);
        }
    }
}

# Rutina RECURSIVA. Copia un directorio completo con archivos y subdirs.
# Param. de entrada :
# 0) Dir padre del dir origen, path completo sin / al final
# 1) Nombre del Dir. a copiar sin path (este es el dir origen).
# 2) Dir donde en su interior se creara el dir a copiar, path completo sin / al final
# 3) Nuevo nombre que se le va a poner al Dir. al copiarlo, sin path. Este param. tiene
#        efecto solo la primera vez que se invoca a la rutina.
# 4) patron de directorios para excluir, no deben llevar / o .
# 5) patron de extensiones de archivo a excluir, solo pueden llevar .
sub copy_tree_exclude_ext {
    my ($padre_origen, $df_origen, $df_destino, $nom_destino, $dir_excluir, $ext_excluir) = @_;

    my $ret = 0;
    $ret = opendir(dir_handle, $padre_origen . '/' . $df_origen);
    # si es un directorio
    if ($ret) {
        my @exlude_dir = split(',', $dir_excluir);

        if ($nom_destino ne '') {
            foreach my $excluir (@exlude_dir) {
                if ($nom_destino =~ /$excluir/) {
                    return;
                }
            }
            mkdir($df_destino . '/' . $nom_destino, 493);
            # print "mkdir [$df_destino/$nom_destino]\n\n";
        } else {
            foreach my $excluir (@exlude_dir) {
                if ($df_origen =~ /$excluir/) {
                    return;
                }
            }
            mkdir($df_destino . '/' . $df_origen, 493);
            # print "mkdir [$df_destino/$df_origen]\n\n";
        }

        my @list_dir = readdir dir_handle;
        close dir_handle;
        foreach my $item (@list_dir) {
            next if (($item eq '.') || ($item eq '..'));
            if ($nom_destino ne '') {
                &copy_tree_exclude_ext($padre_origen . '/' . $df_origen, $item, $df_destino . '/' . $nom_destino, '', $dir_excluir, $ext_excluir);
            } else {
                &copy_tree_exclude_ext($padre_origen . '/' . $df_origen, $item, $df_destino . '/' . $df_origen, '', $dir_excluir, $ext_excluir);
            }
        }
    } else {    # Es un archivo
        my @exlude_file = split(',', $ext_excluir);
        foreach my $excluir (@exlude_file) {
            # escapeamos los caracteres "meta" que pueda tener el patron
            $excluir = quotemeta $excluir;
            if ($df_origen =~ /$excluir/) {
                return;
            }
        }
        File::Copy::copy($padre_origen . '/' . $df_origen, $df_destino . '/' . $df_origen);
        # print "copy [$df_destino/$df_origen]\n\n";
    }
}

# Crea un subdirectorio desde un dir. dado.

# Param. de entrada :
# 0) Dir desde donde se creara el subdir
# 1) Nombre del subdir.
sub crea_dir {
    my ($dir_desde, $nom_dir) = @_;

    if (-d "$dir_desde/$nom_dir") {
        return 1;
    }
    if (-f "$dir_desde/$nom_dir") {
        lib_stdlog::print_err("[crea_dir] Error [$dir_desde/$nom_dir] is a file\n");
        return 0;
    }

    if (-d $dir_desde) {
        if(! mkdir("$dir_desde/$nom_dir", 493) ) {
            lib_stdlog::print_err("[crea_dir] Error making directory [$dir_desde/$nom_dir] [$!]\n");
            return 0;
        }
        return 1;
    } elsif (-e $dir_desde) { # existe pero no es directorio
        lib_stdlog::print_err("[crea_dir] Error: is not a directory [$dir_desde]\n");
        return 0;
    } else { # no existe
        lib_stdlog::print_err("[crea_dir] Error: base directory does not exist [$dir_desde]\n");
        return 0;
    }
}

# Lee una cantidad de caracteres desde el ppio de un archivo.
# Param. de entrada :
# 0) Path real del archivo.
# 1) Nro. de bytes a leer.
# Retorna : El texto leido | '' en caso que el archivo no exista.
sub read_file_bytes {
    my ($archivo, $numbytes) = @_;
    my $buffer;
    $archivo =~ s|\.+[/\\]+||g;

    if (-f $archivo) {
        if (open(my $fh, '<', $archivo)) {
            binmode $fh;
            read $fh, $buffer, $numbytes;
            close $fh;
        } else {
            lib_stdlog::print_err("read_file_bytes: Fail Open file [$archivo]\n[$!]\n");
            lib_stdlog::print_err(join(';', caller), "\n");
            print "Content-Type: text/plain\n\nError 1\n";
            exit;
        }
    }

    return $buffer;
}

# Lee un archivo por completo.
# Param. de entrada :
# 0) Path real del archivo.
# Retorna : El texto leido | '' en caso que el archivo no exista.
sub read_file_bin {
    my ($archivo, $create) = @_;
    my $buffer = '';
    $archivo =~ s|\.+[/\\]+||g;

    if (-f $archivo) {
        if (open(my $fh, '<', $archivo)) {
            binmode $fh;
            read $fh, $buffer, -s $archivo;
            close $fh;
        } else {
            lib_stdlog::print_err("read_file_bin: Fail Open file [$archivo]\n[$!]\n");
            lib_stdlog::print_err(join(';', caller), "\n");
            print "Content-Type: text/plain\n\nError 2\n";
            exit;
        }
    } else {
        lib_stdlog::print_err("read_file_bin: file not found [$archivo]\n");
        lib_stdlog::print_err(join(';', caller), "\n");
        if ($create){
            open my $fh, '>', $archivo;
            print $fh "{}\n";
            close $fh;
        }
    }

    return $buffer;
}

# Lee un archivo por completo.
# Param. de entrada :
# 0) Path real del archivo.
# Retorna : El texto leido | '' en caso que el archivo no exista.
sub read_file {
    my ($archivo, $create) = @_;
    my $buffer = '';
    $archivo =~ s|\.+[/\\]+||g;

    if (-f $archivo) {
        if (open(my $fh, '<', $archivo)) {
            binmode $fh, ':encoding(UTF-8)';
            read $fh, $buffer, -s $archivo;
            close $fh;
        } else {
            lib_stdlog::print_err("read_file: Fail Open file [$archivo]\n[$!]\n");
            lib_stdlog::print_err(join(';', caller), "\n");
            print "Content-Type: text/plain\n\nError 3\n";
            exit;
        }
    } else {
        lib_stdlog::print_err("read_file: file not found [$archivo]\n");
        lib_stdlog::print_err(join(';', caller), "\n");
        if ($create){
            open my $fh, '>', $archivo;
            print $fh "dummy\n";
            close $fh;
        }
    }

    return $buffer;
}

# Escribe un archivo binario
# Param. de entrada :
# 0) Path real del archivo.
# 1) Texto a escribir.
sub write_file_bin {
    my ($archivo, $buffer) = @_;
    $archivo =~ s|\.+[/\\]+||g;

    if (open(my $fh, '>', $archivo)) {
        binmode $fh;
        #Escribe buffer completo
        if (!(print $fh $buffer)) {
            close $fh;
            lib_stdlog::print_err("write_file_bin: Fail Write File [$archivo]\n[$!]\n");
            lib_stdlog::print_err(join(';', caller), "\n");
            print "Content-Type: text/plain\n\nError 5\n";
            exit;
        }
        close $fh;
    } else {
        lib_stdlog::print_err("write_file_bin: Fail Open file [$archivo]\n[$!]\n");
        lib_stdlog::print_err(join(';', caller), "\n");
        print "Content-Type: text/plain\n\nError 6\n";
        exit;
    }
}

# Escribe un archivo.
# Param. de entrada :
# 0) Path real del archivo.
# 1) Texto a escribir.
# 2) (opcional) Describe el encoding del texto a usar para escribir el archivo.
#   Asume que el texto está correctamente encodeado.
sub write_file {
    my ($archivo, $buffer, $encoding) = @_;
    $archivo =~ s|\.+[/\\]+||g;

    if (open(my $fh, '>', $archivo)) {
        if (defined($encoding) && $encoding eq 'iso-8859-1') {
            binmode $fh, ":encoding(latin1)";
        } else {
            binmode $fh, ':encoding(UTF-8)';
        }
        #Escribe buffer completo
        if (!(print $fh $buffer)) {
            close $fh;
            lib_stdlog::print_err("write_file: Fail Write File [$archivo]\n[$!]\n");
            lib_stdlog::print_err(join(';', caller), "\n");
            print "Content-Type: text/plain\n\nError 7\n";
            exit;
        }
        close $fh;
    } else {
        lib_stdlog::print_err("write_file: Fail Open file [$archivo]\n[$!]\n");
        lib_stdlog::print_err(join(';', caller), "\n");
        print "Content-Type: text/plain\n\nError 8\n";
        exit;
    }
}

sub append_file {
# Escribe datos al final de un archivo.
# Param. de entrada :
# 0) Path real del archivo.
# 1) Texto a escribir.
    my ($file, $data) = @_;
    $file =~ s/\.+[\/\\]+//g;

    if (open(my $fh, ">>:encoding(UTF-8)", $file)) {
        print $fh $data;
        close $fh;
    } else {
        lib_stdlog::print_err("append_file: Error al abrir [$file]\n[$!]\n");
    }
}

# Chequea directorio. Si no existe, lo crea.
# Param. de entrada :
# 0) Path real del directorio.
# Retorna : 0 si hubo error, y 1 si no lo hubo.
sub check_dir {
    my $dir = $_[0];
    return 0 if (($dir eq '') || ($dir eq '/'));
    return 1 if (-d $dir);

    # si no existe intentamos crear el path
    if (!-e $dir) {
        my $absolute = 0;
        $absolute = 1 if (substr($dir, 0, 1) eq '/');
        # quita / inicial o final
        $dir =~ s/^\/|\/$//sg;
        my @path_data = split(/\//, $dir);
        my $total = $#path_data + 1;
        my $path = '';
        for (my $i = 0; $i < $total; $i++) {
            if (!$absolute && $i == 0) {
                $path .= $path_data[$i];
            } else {
                $path .= '/' . $path_data[$i];
            }

            next if (-d $path);
            # si existe no es directorio
            if (-e $path) {
                # si no es directorio, es un error
                lib_stdlog::print_err("[check_dir][0] Error: path is a file [$path]\n", join(';', caller, "\n"));
                return 0;
            }
            # no existe, intentamos crear el directorio
            elsif (! mkdir($path, 493)) {
                # si falla la creacion imprime error y retorna
                lib_stdlog::print_err("[check_dir][1] Error making directory [$path] [$!]\n", join(';', caller, "\n"));
                return 0;
            }
        }
    } else {
        # existe un archivo
        lib_stdlog::print_err("[check_dir][2] Error: path is a file [$dir]\n");
        return 0;
    }
    return 1;
}

# Recorre directorio borrandolo todo. Al final tambien borra el
# directorio itself.

# Param. de entrada :
# 0) Path real del directorio.
sub borra_dir {
    my $dir = $_[0];
    my @entries;

    $dir =~ s/\.+[\/\\]+//g;
    if (-e $dir && opendir(my $DIR, $dir)) {
        # Abre directorio.
        @entries = readdir($DIR);
        closedir $DIR;

        foreach my $entry (@entries) {
            if (($entry ne '.') and ($entry ne '..')) {
                if (-d "$dir/$entry") {
                    &borra_dir("$dir/$entry");
                } else {
                    unlink "$dir/$entry";
                }
            }
        }

        rmdir $dir;
    } else {
        lib_stdlog::print_err("rmdir: Can't opendir [$dir] [$!] \n");
    }
}

# Lee un directorio y entrega la lista ordenada de entries en bruto.
# Param. de entrada :
# 0) Path real del directorio.
# Retorna : Arreglo ordenado de entradas en bruto del directorio.
sub lee_dir {
    my $eldir = $_[0];
    $eldir =~ s/\.+[\/\\]+//g;

    my $DIR;
    # Abre directorio.
    if (!opendir($DIR, $eldir)) {
        lib_stdlog::print_err("Can't opendir [$eldir] [$!]\n");
        return ();
    }
    my @entries = readdir($DIR);
    closedir $DIR;
    # Ordena entries alfabeticamente.
    @entries = sort @entries;
    return @entries;
}

# Retorna la antiguedad del archivo en segundos con respecto a la fecha de hoy.
sub get_antiguedad_archivo {
    my $mtime = (stat($_[0]))[9];
    my $diff  = time - $mtime;

    return $diff;
}

# Elimina los archivos de mas de 10 minutos de antiguedad de un directorio
sub garbage_collection {
    my $path_answer_dir = $_[0];
    # lib_stdlog::print_err("garbage collection dir [$path_answer_dir]\n");
    return unless(-d $path_answer_dir);
    if (opendir(my $DIR, $path_answer_dir)) {
        my @entries = readdir($DIR);
        closedir $DIR;
        foreach my $file (@entries) {
            if (-f "$path_answer_dir/$file") {
                if ((stat("$path_answer_dir/$file"))[9] < (time - 600)) {
                    unlink "$path_answer_dir/$file";
                }
            }
        }
    }
} # garbage_collection

1;
