#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Manipulacion general de strings
#---------------------------------------------------------------

package glib_str_03;
use utf8;
use strict;

# Recibe numero real y lo formatea con separadores de miles y cant. de decimales deseada.
sub format_miles_dec {
    my ($num, $d_digits) = @_;

    my $dec = 0;
    if ($d_digits) {
        $num = sprintf("%.${d_digits}f", $num);
        # rescata solo decimales
        $dec = substr($num, index($num, '.')+1);
    }
    # deja solo parte entera
    $num = int($num);

    my $formatted = '';
    while($num >= 1000) {
        $formatted = sprintf('.%03d', $num%1000) . $formatted;
        $num = int($num/1000);
    }

    if ($d_digits) {
        return "$num$formatted,$dec";
    } else {
        return "$num$formatted";
    }
}

# formatea numero en bytes
sub bytes2kb {
    my $bytes = $_[0];
    $bytes = $bytes / 1024;
    $bytes = 0 if ($bytes < 0);
    $bytes = &format_miles_dec($bytes, 0);
    $bytes .= ' KiB';
    return $bytes;
}

# Valida nombres de directorios ingresados por el usuario.
# Param :
# 0) Nombre del directorio (sin path) a validar
# Retornos : 1 si esta ok, 0 en caso contrario
sub chars_dir_ok {
    if ($_[0] =~ /[^a-z0-9_~]/) {
        return 0;
    } else {
        return 1;
    }
}

# Valida que el dato ingresado contenga sólo digitos.
# Param :
# 0) String a validar
# Retornos : 1 si esta ok, 0 en caso contrario
sub is_digit {
    if ($_[0] =~ /[^0-9]/) {
        return 0;
    } else {
        return 1;
    }

}

# Devuelve solo los digitos de un string.
# Param :
# 0) String de entrada.
# Retorna : string con todos los caracteres que no eran digitos
# eliminados.
sub solo_digitos {
    my $texto = $_[0];
    $texto =~ s/[^0-9]//g;
    return $texto;
}

sub trim {
    my $valor = $_[0];
    $valor =~ s/^\s+//;
    $valor =~ s/\s+$//;
    return $valor;
}

# Funcion que calcula el digito verificador de un rut dado.
# Parametro:
# 0) rut sin dv.
sub devuelve_dv_rut {
    my $r;
    for ($_ = $_[0] ; s/\d$// || ($_ = (11 - $a % 11) % 10) * 0 ; $a += $& * (++$r - 7 ? $r : ($r = 1)) + $&) {
        # nada.
    }
    return $_ || "K";
}

# Retorna un string aleatorio de n letras.
sub random_string {
    my $n       = $_[0];
    my @charset = ('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'R', 'T', 'U', 'V', 'W', 'X', 'Y', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'g', 'h', 'i', 'j', 'k', 'm', 'n', 'o', 'p', 'r', 's', 'u', 'v', 'w', 'x', 'y', 'z' );
    # 51 caracteres
    my $i;
    my $salida = '';
    for ($i = 0 ; $i < $n ; $i++) {
        $salida .= $charset[int(rand(51))];
    }
    return $salida;
}

sub solo_texto {
    my $str = $_[0];
    $str =~ s/[^\wáéíóúÁÉÍÓÚüÜñÑ]/ /sg;    # solo permite nombres normales
    return $str;
}

sub correct_encode_utf8 {
    my $hashref = $_[0];
    foreach my $key (keys %{$hashref}) {
        if (!utf8::is_utf8($hashref->{$key})) {
            utf8::decode($hashref->{$key});
        }
    }
}

sub correct_encode_utf8_string {
    if (!utf8::is_utf8($_[0])) {
        utf8::decode($_[0]);
    }
}

1;
