#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# Funciones para utilizar captcha sin utilizar sesiones.
package lib_captcha2;

use strict;
use utf8;

# NRO. MAXIMO DE ARCHIVOS QUE PUEDEN EXISTIR SIMULTANEAMENTE
our $MAX_FILES = 2000;
# 600 segundos, 10 minutos.
our $TIME_OLD_FILES = 600;
# COLOR LETRAS
our @FORECOLOR = (0, 0, 0);
# COLOR FONDO
our @BACKCOLOR = (255, 255, 255);

our %TYPE = ('form' => 1, 'posting' => 1, 'enviar' => 1, 'cpan' => 1);
our $PATH_CAPTCHA_IMG = '/imag/prontus_captcha';

sub validar_tipo {
    if (defined($TYPE{$_[0]})) {
        return 1;
    } else {
        return 0;
    }
}

sub valida_captcha {
    my ($captcha_input, $captcha_code, $captcha_type, $captcha_img) = @_;
    $captcha_input =~ s/[^\w]//sg;

    if (! &validar_tipo($captcha_type)) {
        lib_stdlog::print_err("Invalid captcha_type[$captcha_type]\n");
        return 'msg_badtype';
    }

    if ($captcha_img && $captcha_code) {
        # El codigo de validacion no puede ser vacio
        if ($captcha_input eq '') {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('captcha_lib2_missing_code', $captcha_input) , "\n");
            return 'msg_void';
        }

        # Se revisa que la imagen exista
        if (! -f "$prontus_varglb::DIR_SERVER$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type/$captcha_img") {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('captcha_lib2_missing_image', "$prontus_varglb::DIR_SERVER$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type/$captcha_img"), "\n");
            return 'msg_expired';
        } else {
            # Si la imagen existe, se borra para evitar nueva invocacion
            if (-f "$prontus_varglb::DIR_SERVER$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type/$captcha_img") {
                unlink "$prontus_varglb::DIR_SERVER$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type/$captcha_img";
            }
        }

        # Finalmente se valida todo
        my $encoded = &make_hash($captcha_input);
        if($encoded ne $captcha_code) {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('captcha_lib2_invalid_codes', $encoded, $captcha_code),"\n");
            return 'msg_invalid';
        }
        return '';

    } else {
        lib_stdlog::print_err(lib_lang::get_interpolated_message('captcha_lib2_missing_code', $captcha_input) , "\n");
        return 'msg_void';
    }
}

sub make_hash {
    my $str = $_[0];
    my $pass1 = Digest::SHA::sha1_hex($str);
    my $pass2 = Digest::SHA::sha1_base64($pass1) . $str;
    my $pass3 = Digest::SHA::sha1_hex($pass2);
    return $pass3;
}

sub get_img_name {
    my $hash = $_[0];
    my $str_parts = join ' ', unpack '(A4)*', $hash;
    my @parts = split(" ", $str_parts);
    my @new_order = (9,6,3,0,5,8,2,7,1,4);
    my $new_str = '';

    foreach my $num (@new_order) {
        $new_str .= $parts[$num];
    }

    return $new_str.".jpg";
}

sub make_captcha_img {
    my ($string, $filename_path, $char_length) = @_;
    my $default_char_length = 4;
    $char_length = $char_length || $default_char_length;

    my $trueTypeFont = "$prontus_varglb::DIR_SERVER/prontus/cpan/fonts/fontcaptcha.ttf";
    if (! -f $trueTypeFont) { # verfica archivo ttf
        lib_stdlog::print_err(lib_lang::get_interpolated_message('captcha_lib2_font_not_found', $trueTypeFont), "\n");
        return 'msg_system';
    }

     # glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$lib_captcha2::PATH_CAPTCHA_IMG/$captcha_type");
    my @string = split(//,$string);
    my $imgHeight = 45; # Alto de las imágenes de trabajo.
    my $imgWidth = 85;  # Ancho de las imagenes de trabajo.
    my $salidaHeight = 35; # Alto de la imagen de salida.
    my $salidaWidth = 80; # Alto de la imagen de salida por defecto
    # si el captcha debe ser más largo, extendemos el ancho de la imagen.
    if ($char_length != $default_char_length) {
        $salidaWidth = POSIX::ceil($char_length * $salidaWidth / $default_char_length);
        $imgWidth = POSIX::ceil($char_length * $imgWidth / $default_char_length);
    }
    my $fontSize = 18; # Tamano del font en puntos.
    my $posMargin = 2; # Margen que se le da a la posicion de la letra.
    my $maxAngle = 0.2; # Maxima inclinacion, en radianes.
    my $letra = GD::Image->new($imgWidth, $imgHeight, 1); # Imagen para dibujar cada letra.
    my $image = GD::Image->new($imgWidth, $imgHeight, 1); # La imagen de trabajo.
    my $salida = GD::Image->new($salidaWidth, $salidaHeight, 1); # La imagen de salida.

    my $black = $image->colorAllocate($FORECOLOR[0], $FORECOLOR[1], $FORECOLOR[2]);
    my $white = $image->colorAllocate($BACKCOLOR[0], $BACKCOLOR[1], $BACKCOLOR[2]);

    my ($dstX,$dstY,$srcX,$srcY,$width,$height,$miny,$maxy,$percent,$ancho);
    # Limpia imagen de trabajo.
    $image->filledRectangle(0, 0, $imgWidth+1, $imgHeight+1, $white);
    # Lugar de copiado de la letra en la imagen de trabajo.
    $dstX = 1;
    # Coordenadas Y minima y maxima.
    $miny = $salidaHeight;
    $maxy = 0;
    for(my $i=0; $i<=$#string; $i++) {
        # Escribe una letra en la imagen para escribir letras.
        # Primero, borra la imagen.
        $letra->filledRectangle(0, 0, $imgWidth+1, $imgHeight+1, $white);
        # @bounds = $image->stringFT($fgcolor,$fontname,$ptsize,$angle,$x,$y,$string)
        # @bounds[0,1]  Lower left corner (x,y)
        # @bounds[2,3]  Lower right corner (x,y)
        # @bounds[4,5]  Upper right corner (x,y)
        # @bounds[6,7]  Upper left corner (x,y)
        # Dibuja la letra. x e y se aumentan en $posMargin para compensar el efecto de la inclinacion aleatoria.
        my @bounds = $letra->stringFT($black, $trueTypeFont, $fontSize, $maxAngle*2*(0.5-rand), $posMargin, $fontSize+$posMargin, $string[$i]);
        $ancho = $bounds[2] - $bounds[0];
        # Copia la letra en la imagen de salida.
        $dstY = 1 + int(rand($posMargin*2));
        $srcX = 1;
        $srcY = 1;
        $width = $salidaWidth;
        $height = $salidaHeight;
        $percent = 40 + int(rand(20)); # % de merge es 50 en promedio.
        $image->copyMerge($letra, $dstX, $dstY, $srcX, $srcY, $width, $height, $percent);
        $dstX += $ancho + int(rand(int($posMargin/2))) - $posMargin;
        if (($bounds[7]+$dstY) < $miny) {
            $miny = $bounds[7] + $dstY;
        }
        if (($bounds[1]+$dstY) > $maxy) {
            $maxy = $bounds[1] + $dstY;
        }
    }
    # Borra imagen de salida.
    $salida->filledRectangle(0, 0, $salidaWidth+1, $salidaHeight+1, $white);
    # $image->copy($sourceImage,$dstX,$dstY,$srcX,$srcY,$width,$height);
    # Calcula parametros para centrar la imagen en la salida.
    $width = $dstX + $posMargin*2;
    $dstX = int(($salidaWidth - $dstX - $posMargin)/2);
    $dstY = int(($salidaHeight - ($maxy - $miny))/2) + 2;
    $srcX = 1;
    $srcY = $miny;
    $height = $salidaHeight;
    $salida->copy($image, $dstX, $dstY, $srcX, $srcY, $width, $height);
    # print "dstX=$dstX dstY=$dstY srcX=$srcX srcY=$srcY width=$width height=$height\n"; # denug

    open (my $FILE, '>', $filename_path);
    binmode $FILE;
    print $FILE $salida->jpeg(50);
    close $FILE;
    return '';
}

sub launch_garbage_collector {
    my $path = $_[0];
    $path =~ s/[#;&<>|~]+//g;
    my $cmd = "/usr/bin/perl $prontus_varglb::DIR_SERVER/cgi-cpn/prontus_garbage_collector.pl $path $TIME_OLD_FILES $MAX_FILES >/dev/null 2>&1 &";
    system($cmd);
}

1;
