#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Funciones para convertir entidades xml en escapeo hexadecimal compatible html.
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------
package lib_encodes;
use strict;
use utf8;

# Detecta y pone en ISO los caracteres escapeados en html.
sub unescape_html {
    my($toencode) = $_[0];

    $toencode=~s/&quot;/\"/g;
    $toencode=~s/&Yacute;/\x86/g;
    $toencode=~s/&brvbar;/\x87/g;
    $toencode=~s/&eth;/\x8B/g;
    $toencode=~s/&middot;/\x95/g;
    $toencode=~s/&shy;/\x96/g;

    $toencode=~s/&#166;/\xA6/g;
    $toencode=~s/&#175;/\xAF/g;
    $toencode=~s/&#178;/\xB2/g;
    $toencode=~s/&#179;/\xB3/g;
    $toencode=~s/&#185;/\xB9/g;
    $toencode=~s/&#188;/\xBC/g;
    $toencode=~s/&#189;/\xBD/g;
    $toencode=~s/&#190;/\xBE/g;
    $toencode=~s/&#215;/\xD7/g;

    $toencode=~s/&nbsp;/\xA0/g;
    $toencode=~s/&iexcl;/\xA1/g;
    $toencode=~s/&cent;/\xA2/g;
    $toencode=~s/&pound;/\xA3/g;
    $toencode=~s/&curren;/\xA4/g;
    $toencode=~s/&yen;/\xA5/g;
    $toencode=~s/&sect;/\xA7/g;
    $toencode=~s/&uml;/\xA8/g;
    $toencode=~s/&copy;/\xA9/g;
    $toencode=~s/&ordf;/\xAA/g;
    $toencode=~s/&laqno/\xAB;/g;
    $toencode=~s/&not;/\xAC/g;
    $toencode=~s/&shy;/\xAD/g;
    $toencode=~s/&reg;/\xAE/g;
    $toencode=~s/&deg;/\xB0/g;
    $toencode=~s/&plusmn;/\xB1/g;
    $toencode=~s/&acute;/\xB4/g;
    $toencode=~s/&micro;/\xB5/g;
    $toencode=~s/&para;/\xB6/g;
    $toencode=~s/&middot;/\xB7/g;
    $toencode=~s/&cedil;/\xB8/g;
    $toencode=~s/&ordm;/\xBA/g;
    $toencode=~s/&raquo;/\xBB/g;
    $toencode=~s/&iquest;/\xBF/g;
    $toencode=~s/&Agrave;/\xC0/g;
    $toencode=~s/&Aacute;/\xC1/g;
    $toencode=~s/&Acirc;/\xC2/g;
    $toencode=~s/&Atilde;/\xC3/g;
    $toencode=~s/&Auml;/\xC4/g;
    $toencode=~s/&Aring;/\xC5/g;
    $toencode=~s/&AElig;/\xC6/g;
    $toencode=~s/&Ccedil;/\xC7/g;
    $toencode=~s/&Egrave;/\xC8/g;
    $toencode=~s/&Eacute;/\xC9/g;
    $toencode=~s/&Ecirc;/\xCA/g;
    $toencode=~s/&Euml;/\xCB/g;
    $toencode=~s/&Igrave;/\xCC/g;
    $toencode=~s/&Iacute;/\xCD/g;
    $toencode=~s/&Icirc;/\xCE/g;
    $toencode=~s/&Iuml;/\xCF/g;
    $toencode=~s/&Ntilde;/\xD1/g;
    $toencode=~s/&Ograve;/\xD2/g;
    $toencode=~s/&Oacute;/\xD3/g;
    $toencode=~s/&Ocirc;/\xD4/g;
    $toencode=~s/&Otilde;/\xD5/g;
    $toencode=~s/&Ouml;/\xD6/g;

    $toencode=~s/&Oslash;/\xD8/g;
    $toencode=~s/&Ugrave;/\xD9/g;
    $toencode=~s/&Uacute;/\xDA/g;
    $toencode=~s/&Ucirc;/\xDB/g;
    $toencode=~s/&Uuml;/\xDC/g;
    $toencode=~s/&Yacute;/\xDD/g;
    $toencode=~s/&THORN;/\xDE/g;
    $toencode=~s/&szlig;/\xDF/g;
    $toencode=~s/&agrave;/\xE0/g;
    $toencode=~s/&aacute;/\xE1/g;
    $toencode=~s/&acirc;/\xE2/g;
    $toencode=~s/&atilde;/\xE3/g;
    $toencode=~s/&auml;/\xE4/g;
    $toencode=~s/&aring;/\xE5/g;
    $toencode=~s/&aelig;/\xE6/g;
    $toencode=~s/&ccedil;/\xE7/g;
    $toencode=~s/&egrave;/\xE8/g;
    $toencode=~s/&eacute;/\xE9/g;
    $toencode=~s/&ecirc;/\xEA/g;
    $toencode=~s/&euml;/\xEB/g;
    $toencode=~s/&igrave;/\xEC/g;
    $toencode=~s/&iacute;/\xED/g;
    $toencode=~s/&icirc;/\xEE/g;
    $toencode=~s/&iuml;/\xEF/g;
    $toencode=~s/&eth;/\xF0/g;
    $toencode=~s/&ntilde;/\xF1/g;
    $toencode=~s/&ograve;/\xF2/g;
    $toencode=~s/&oacute;/\xF3/g;
    $toencode=~s/&ocirc;/\xF4/g;
    $toencode=~s/&otilde/\xF5;/g;
    $toencode=~s/&ouml;/\xF6/g;
    $toencode=~s/&divide;/\xF7/g;
    $toencode=~s/&oslash;/\xF8/g;
    $toencode=~s/&ugrave/\xF9;/g;
    $toencode=~s/&uacute;/\xFA/g;
    $toencode=~s/&ucirc;/\xFB/g;
    $toencode=~s/&uuml;/\xFC/g;
    $toencode=~s/&brvbar;/\xFD/g;
    $toencode=~s/&thorn;/\xFE/g;
    $toencode=~s/&yuml;/\xFF/g;

    $toencode=~s/&#131;/\x83/g;

    $toencode=~s/&#133;/\x85/g;

    $toencode=~s/&#136;/\x88/g;
    $toencode=~s/&#137;/\x89/g;
    $toencode=~s/&#138;/\x8A/g;

    $toencode=~s/&#140;/\x8C/g;
    $toencode=~s/&#141;/\x8D/g;
    $toencode=~s/&#142;/\x8E/g;
    $toencode=~s/&#143;/\x8F/g;
    $toencode=~s/&#144;/\x90/g;
    $toencode=~s/&#145;/\x91/g;
    $toencode=~s/&#146;/\x92/g;

    $toencode=~s/&#151;/\x97/g;

    $toencode=~s/&#153;/\x99/g;
    $toencode=~s/&#154;/\x9A/g;

    $toencode=~s/&#156;/\x9C/g;
    $toencode=~s/&#157;/\x9D/g;
    $toencode=~s/&#158;/\x9E/g;
    $toencode=~s/&#159;/\x9F/g;

    $toencode=~s/&amp;/&/g;  # Al final, traduce los ampersands.

    return $toencode;
} # unescape_html

# texto de entrada se codifica para ser usado como url
sub urlencode_text {
    my $toencode = $_[0];
    $toencode =~ s/[\r\n]+/ /sg;
    $toencode =~ s/<.*?>/ /sg;
    $toencode =~ s/ {2,}/ /sg;
    $toencode =~ s/ $//sg;
    $toencode =~ s/^ //sg;
    $toencode = URI::Escape::uri_escape_utf8($toencode);
    $toencode =~ s/'/%27/g;
    $toencode =~ s/"/%22/g;
    return $toencode;
}

# Reemplaza tildes por letras normales.
sub no_accent_mark {
    my $to_encode = $_[0];

    $to_encode =~ s/&(.)acute;/$1/g;
    $to_encode =~ s/&(.)tilde;/$1/g;
    $to_encode =~ s/&(.)uml;/$1/g;
    $to_encode =~ s/&(.)circ;/$1/g;
    $to_encode =~ s/&(.)grave;/$1/g;

    $to_encode =~ s/\xC0/A/g;
    $to_encode =~ s/\xC1/A/g;
    $to_encode =~ s/\xC2/A/g;
    $to_encode =~ s/\xC3/A/g;
    $to_encode =~ s/\xC4/A/g;
    $to_encode =~ s/\xC5/A/g;
    $to_encode =~ s/\xC6/AE/g;
    $to_encode =~ s/\xC7/C/g;
    $to_encode =~ s/\xC8/E/g;
    $to_encode =~ s/\xC9/E/g;
    $to_encode =~ s/\xCA/E/g;
    $to_encode =~ s/\xCB/E/g;
    $to_encode =~ s/\xCC/I/g;
    $to_encode =~ s/\xCD/I/g;
    $to_encode =~ s/\xCE/I/g;
    $to_encode =~ s/\xCF/I/g;
    $to_encode =~ s/\xD0/D/g;
    $to_encode =~ s/\xD1/N/g;
    $to_encode =~ s/\xD2/O/g;
    $to_encode =~ s/\xD3/O/g;
    $to_encode =~ s/\xD4/O/g;
    $to_encode =~ s/\xD5/O/g;
    $to_encode =~ s/\xD6/O/g;
    $to_encode =~ s/\xD7/X/g;
    $to_encode =~ s/\xD8/O/g;
    $to_encode =~ s/\xD9/U/g;
    $to_encode =~ s/\xDA/U/g;
    $to_encode =~ s/\xDB/U/g;
    $to_encode =~ s/\xDC/U/g;
    $to_encode =~ s/\xDD/Y/g;
    $to_encode =~ s/\xDE/TH/g;
    $to_encode =~ s/\xDF/SZ/g;
    $to_encode =~ s/\xE0/a/g;
    $to_encode =~ s/\xE1/a/g;
    $to_encode =~ s/\xE2/a/g;
    $to_encode =~ s/\xE3/a/g;
    $to_encode =~ s/\xE4/a/g;
    $to_encode =~ s/\xE5/a/g;
    $to_encode =~ s/\xE6/ae/g;
    $to_encode =~ s/\xE7/c/g;
    $to_encode =~ s/\xE8/e/g;
    $to_encode =~ s/\xE9/e/g;
    $to_encode =~ s/\xEA/e/g;
    $to_encode =~ s/\xEB/e/g;
    $to_encode =~ s/\xEC/i/g;
    $to_encode =~ s/\xED/i/g;
    $to_encode =~ s/\xEE/i/g;
    $to_encode =~ s/\xEF/i/g;

    $to_encode =~ s/\xF1/n/g;
    $to_encode =~ s/\xF2/o/g;
    $to_encode =~ s/\xF3/o/g;
    $to_encode =~ s/\xF4/o/g;
    $to_encode =~ s/\xF5/o/g;
    $to_encode =~ s/\xF6/o/g;

    $to_encode =~ s/\xF8/o/g;
    $to_encode =~ s/\xF9/u/g;
    $to_encode =~ s/\xFA/u/g;
    $to_encode =~ s/\xFB/u/g;
    $to_encode =~ s/\xFC/u/g;
    $to_encode =~ s/\xFD/y/g;
    $to_encode =~ s/\xFE/th/g;
    $to_encode =~ s/\xFF/y/g;

    return $to_encode;
}

sub escape_xml {
    my $to_encode = $_[0];
    # reemplaza entidades no numericas que no son compatibles con xml
    $to_encode =~ s/&acute;/&#180;/g;
    $to_encode =~ s/&Alpha;/&#913;/g;
    $to_encode =~ s/&alpha;/&#945;/g;
    $to_encode =~ s/&amp;/&#38;/g;
    $to_encode =~ s/&and;/&#8743;/g;
    $to_encode =~ s/&ang;/&#8736;/g;
    $to_encode =~ s/&asymp;/&#8776;/g;
    $to_encode =~ s/&bdquo;/&#8222;/g;
    $to_encode =~ s/&Beta;/&#914;/g;
    $to_encode =~ s/&beta;/&#946;/g;
    $to_encode =~ s/&brvbar;/&#166;/g;
    $to_encode =~ s/&bull;/&#8226;/g;
    $to_encode =~ s/&cap;/&#8745;/g;
    $to_encode =~ s/&cedil;/&#184;/g;
    $to_encode =~ s/&cent;/&#162;/g;
    $to_encode =~ s/&Chi;/&#935;/g;
    $to_encode =~ s/&chi;/&#967;/g;
    $to_encode =~ s/&circ;/&#710;/g;
    $to_encode =~ s/&clubs;/&#9827;/g;
    $to_encode =~ s/&cong;/&#8773;/g;
    $to_encode =~ s/&copy;/&#169;/g;
    $to_encode =~ s/&crarr;/&#8629;/g;
    $to_encode =~ s/&cup;/&#8746;/g;
    $to_encode =~ s/&curren;/&#164;/g;
    $to_encode =~ s/&dagger;/&#8224;/g;
    $to_encode =~ s/&Dagger;/&#8225;/g;
    $to_encode =~ s/&darr;/&#8595;/g;
    $to_encode =~ s/&deg;/&#176;/g;
    $to_encode =~ s/&Delta;/&#916;/g;
    $to_encode =~ s/&delta;/&#948;/g;
    $to_encode =~ s/&diams;/&#9830;/g;
    $to_encode =~ s/&divide;/&#247;/g;
    $to_encode =~ s/&empty;/&#8709;/g;
    $to_encode =~ s/&emsp;/&#8195;/g;
    $to_encode =~ s/&ensp;/&#8194;/g;
    $to_encode =~ s/&Epsilon;/&#917;/g;
    $to_encode =~ s/&epsilon;/&#949;/g;
    $to_encode =~ s/&equiv;/&#8801;/g;
    $to_encode =~ s/&Eta;/&#919;/g;
    $to_encode =~ s/&eta;/&#951;/g;
    $to_encode =~ s/&euro;/&#8364;/g;
    $to_encode =~ s/&exist;/&#8707;/g;
    $to_encode =~ s/&fnof;/&#402;/g;
    $to_encode =~ s/&forall;/&#8704;/g;
    $to_encode =~ s/&frac12;/&#189;/g;
    $to_encode =~ s/&frac14;/&#188;/g;
    $to_encode =~ s/&frac34;/&#190;/g;
    $to_encode =~ s/&Gamma;/&#915;/g;
    $to_encode =~ s/&gamma;/&#947;/g;
    $to_encode =~ s/&ge;/&#8805;/g;
    $to_encode =~ s/&gt;/&#62;/g;
    $to_encode =~ s/&harr;/&#8596;/g;
    $to_encode =~ s/&hearts;/&#9829;/g;
    $to_encode =~ s/&hellip;/&#8230;/g;
    $to_encode =~ s/&iexcl;/&#161;/g;
    $to_encode =~ s/&infin;/&#8734;/g;
    $to_encode =~ s/&int;/&#8747;/g;
    $to_encode =~ s/&Iota;/&#921;/g;
    $to_encode =~ s/&iota;/&#953;/g;
    $to_encode =~ s/&iquest;/&#191;/g;
    $to_encode =~ s/&isin;/&#8712;/g;
    $to_encode =~ s/&Kappa;/&#922;/g;
    $to_encode =~ s/&kappa;/&#954;/g;
    $to_encode =~ s/&Lambda;/&#923;/g;
    $to_encode =~ s/&lambda;/&#955;/g;
    $to_encode =~ s/&laquo;/&#171;/g;
    $to_encode =~ s/&larr;/&#8592;/g;
    $to_encode =~ s/&lceil;/&#8968;/g;
    $to_encode =~ s/&ldquo;/&#8220;/g;
    $to_encode =~ s/&le;/&#8804;/g;
    $to_encode =~ s/&lfloor;/&#8970;/g;
    $to_encode =~ s/&lowast;/&#8727;/g;
    $to_encode =~ s/&loz;/&#9674;/g;
    $to_encode =~ s/&lrm;/&#8206;/g;
    $to_encode =~ s/&lsaquo;/&#8249;/g;
    $to_encode =~ s/&lsquo;/&#8216;/g;
    $to_encode =~ s/&lt;/&#60;/g;
    $to_encode =~ s/&macr;/&#175;/g;
    $to_encode =~ s/&mdash;/&#8212;/g;
    $to_encode =~ s/&micro;/&#181;/g;
    $to_encode =~ s/&minus;/&#8722;/g;
    $to_encode =~ s/&Mu;/&#924;/g;
    $to_encode =~ s/&mu;/&#956;/g;
    $to_encode =~ s/&nabla;/&#8711;/g;
    $to_encode =~ s/&nbsp;/&#160;/g;
    $to_encode =~ s/&ndash;/&#8211;/g;
    $to_encode =~ s/&ne;/&#8800;/g;
    $to_encode =~ s/&ni;/&#8715;/g;
    $to_encode =~ s/&not;/&#172;/g;
    $to_encode =~ s/&notin;/&#8713;/g;
    $to_encode =~ s/&nsub;/&#8836;/g;
    $to_encode =~ s/&Nu;/&#925;/g;
    $to_encode =~ s/&nu;/&#957;/g;
    $to_encode =~ s/&OElig;/&#338;/g;
    $to_encode =~ s/&oelig;/&#339;/g;
    $to_encode =~ s/&oline;/&#8254;/g;
    $to_encode =~ s/&Omega;/&#937;/g;
    $to_encode =~ s/&omega;/&#969;/g;
    $to_encode =~ s/&Omicron;/&#927;/g;
    $to_encode =~ s/&omicron;/&#959;/g;
    $to_encode =~ s/&oplus;/&#8853;/g;
    $to_encode =~ s/&or;/&#8744;/g;
    $to_encode =~ s/&ordf;/&#170;/g;
    $to_encode =~ s/&ordm;/&#186;/g;
    $to_encode =~ s/&otimes;/&#8855;/g;
    $to_encode =~ s/&para;/&#182;/g;
    $to_encode =~ s/&part;/&#8706;/g;
    $to_encode =~ s/&permil;/&#8240;/g;
    $to_encode =~ s/&perp;/&#8869;/g;
    $to_encode =~ s/&Phi;/&#934;/g;
    $to_encode =~ s/&phi;/&#966;/g;
    $to_encode =~ s/&Pi;/&#928;/g;
    $to_encode =~ s/&pi;/&#960;/g;
    $to_encode =~ s/&piv;/&#982;/g;
    $to_encode =~ s/&plusmn;/&#177;/g;
    $to_encode =~ s/&pound;/&#163;/g;
    $to_encode =~ s/&prime;/&#8242;/g;
    $to_encode =~ s/&Prime;/&#8243;/g;
    $to_encode =~ s/&prod;/&#8719;/g;
    $to_encode =~ s/&prop;/&#8733;/g;
    $to_encode =~ s/&Psi;/&#936;/g;
    $to_encode =~ s/&psi;/&#968;/g;
    $to_encode =~ s/&radic;/&#8730;/g;
    $to_encode =~ s/&raquo;/&#187;/g;
    $to_encode =~ s/&rarr;/&#8594;/g;
    $to_encode =~ s/&rceil;/&#8969;/g;
    $to_encode =~ s/&rdquo;/&#8221;/g;
    $to_encode =~ s/&reg;/&#174;/g;
    $to_encode =~ s/&rfloor;/&#8971;/g;
    $to_encode =~ s/&Rho;/&#929;/g;
    $to_encode =~ s/&rho;/&#961;/g;
    $to_encode =~ s/&rlm;/&#8207;/g;
    $to_encode =~ s/&rsaquo;/&#8250;/g;
    $to_encode =~ s/&rsquo;/&#8217;/g;
    $to_encode =~ s/&sbquo;/&#8218;/g;
    $to_encode =~ s/&Scaron;/&#352;/g;
    $to_encode =~ s/&scaron;/&#353;/g;
    $to_encode =~ s/&sdot;/&#8901;/g;
    $to_encode =~ s/&sect;/&#167;/g;
    $to_encode =~ s/&shy;/&#173;/g;
    $to_encode =~ s/&Sigma;/&#931;/g;
    $to_encode =~ s/&sigma;/&#963;/g;
    $to_encode =~ s/&sigmaf;/&#962;/g;
    $to_encode =~ s/&sim;/&#8764;/g;
    $to_encode =~ s/&spades;/&#9824;/g;
    $to_encode =~ s/&sub;/&#8834;/g;
    $to_encode =~ s/&sube;/&#8838;/g;
    $to_encode =~ s/&sum;/&#8721;/g;
    $to_encode =~ s/&sup;/&#8835;/g;
    $to_encode =~ s/&sup1;/&#185;/g;
    $to_encode =~ s/&sup2;/&#178;/g;
    $to_encode =~ s/&sup3;/&#179;/g;
    $to_encode =~ s/&supe;/&#8839;/g;
    $to_encode =~ s/&Tau;/&#932;/g;
    $to_encode =~ s/&tau;/&#964;/g;
    $to_encode =~ s/&there4;/&#8756;/g;
    $to_encode =~ s/&Theta;/&#920;/g;
    $to_encode =~ s/&theta;/&#952;/g;
    $to_encode =~ s/&thetasym;/&#977;/g;
    $to_encode =~ s/&thinsp;/&#8201;/g;
    $to_encode =~ s/&tilde;/&#732;/g;
    $to_encode =~ s/&times;/&#215;/g;
    $to_encode =~ s/&trade;/&#8482;/g;
    $to_encode =~ s/&uarr;/&#8593;/g;
    $to_encode =~ s/&uml;/&#168;/g;
    $to_encode =~ s/&upsih;/&#978;/g;
    $to_encode =~ s/&Upsilon;/&#933;/g;
    $to_encode =~ s/&upsilon;/&#965;/g;
    $to_encode =~ s/&Xi;/&#926;/g;
    $to_encode =~ s/&xi;/&#958;/g;
    $to_encode =~ s/&yen;/&#165;/g;
    $to_encode =~ s/&Yuml;/&#376;/g;
    $to_encode =~ s/&Zeta;/&#918;/g;
    $to_encode =~ s/&zeta;/&#950;/g;
    $to_encode =~ s/&zwj;/&#8205;/g;
    $to_encode =~ s/&zwnj;/&#8204;/g;

    # elimina entidades no numericas que no fueron reemplazadas y las reemplaza por espacios
    $to_encode =~ s/&[a-zA-Z]+;/ /g;
    # reemplaza espacios dobles por uno simple
    $to_encode =~ s/ {2,}/ /g;
    # Ampersands no seguidos por "amp;" o "38;"
    $to_encode =~ s/&(?!amp;)/&#38;/g;
    $to_encode =~ s/&(?!#38)/&#38;/g;
    # entidades reconocidas por xml
    # >
    $to_encode =~ s/>/&#62;/g;
    # <
    $to_encode =~ s/</&#60;/g;
    # "
    $to_encode =~ s/"/&#34;/g;
    # '
    $to_encode =~ s/'/&#39;/g;
    return $to_encode;
}

sub unescape_xml {
    my $to_decode = $_[0];
    # >
    $to_decode =~ s/&gt;/>/g;
    $to_decode =~ s/&#62;/>/g;
    # <
    $to_decode =~ s/&lt;/</g;
    $to_decode =~ s/&#60;/</g;
    # "
    $to_decode =~ s/&quot;/"/g;
    $to_decode =~ s/&#34;/"/g;

    $to_decode =~ s/&#39;/'/g;
    $to_decode =~ s/&apos;/'/g;

    $to_decode =~ s/&amp;/&/g;
    $to_decode =~ s/&#38;/&/g;

    # reemplaza entidades xml con su correspondiente entidad html
    $to_decode =~ s/&#180;/&acute;/g;
    $to_decode =~ s/&#913;/&Alpha;/g;
    $to_decode =~ s/&#945;/&alpha;/g;
    $to_decode =~ s/&#38;/&amp;/g;
    $to_decode =~ s/&#8743;/&and;/g;
    $to_decode =~ s/&#8736;/&ang;/g;
    $to_decode =~ s/&#8776;/&asymp;/g;
    $to_decode =~ s/&#8222;/&bdquo;/g;
    $to_decode =~ s/&#914;/&Beta;/g;
    $to_decode =~ s/&#946;/&beta;/g;
    $to_decode =~ s/&#166;/&brvbar;/g;
    $to_decode =~ s/&#8226;/&bull;/g;
    $to_decode =~ s/&#8745;/&cap;/g;
    $to_decode =~ s/&#184;/&cedil;/g;
    $to_decode =~ s/&#162;/&cent;/g;
    $to_decode =~ s/&#935;/&Chi;/g;
    $to_decode =~ s/&#967;/&chi;/g;
    $to_decode =~ s/&#710;/&circ;/g;
    $to_decode =~ s/&#9827;/&clubs;/g;
    $to_decode =~ s/&#8773;/&cong;/g;
    $to_decode =~ s/&#169;/&copy;/g;
    $to_decode =~ s/&#8629;/&crarr;/g;
    $to_decode =~ s/&#8746;/&cup;/g;
    $to_decode =~ s/&#164;/&curren;/g;
    $to_decode =~ s/&#8224;/&dagger;/g;
    $to_decode =~ s/&#8225;/&Dagger;/g;
    $to_decode =~ s/&#8595;/&darr;/g;
    $to_decode =~ s/&#176;/&deg;/g;
    $to_decode =~ s/&#916;/&Delta;/g;
    $to_decode =~ s/&#948;/&delta;/g;
    $to_decode =~ s/&#9830;/&diams;/g;
    $to_decode =~ s/&#247;/&divide;/g;
    $to_decode =~ s/&#8709;/&empty;/g;
    $to_decode =~ s/&#8195;/&emsp;/g;
    $to_decode =~ s/&#8194;/&ensp;/g;
    $to_decode =~ s/&#917;/&Epsilon;/g;
    $to_decode =~ s/&#949;/&epsilon;/g;
    $to_decode =~ s/&#8801;/&equiv;/g;
    $to_decode =~ s/&#919;/&Eta;/g;
    $to_decode =~ s/&#951;/&eta;/g;
    $to_decode =~ s/&#8364;/&euro;/g;
    $to_decode =~ s/&#8707;/&exist;/g;
    $to_decode =~ s/&#402;/&fnof;/g;
    $to_decode =~ s/&#8704;/&forall;/g;
    $to_decode =~ s/&#189;/&frac12;/g;
    $to_decode =~ s/&#188;/&frac14;/g;
    $to_decode =~ s/&#190;/&frac34;/g;
    $to_decode =~ s/&#915;/&Gamma;/g;
    $to_decode =~ s/&#947;/&gamma;/g;
    $to_decode =~ s/&#8805;/&ge;/g;
    $to_decode =~ s/&#62;/&gt;/g;
    $to_decode =~ s/&#8596;/&harr;/g;
    $to_decode =~ s/&#9829;/&hearts;/g;
    $to_decode =~ s/&#8230;/&hellip;/g;
    $to_decode =~ s/&#161;/&iexcl;/g;
    $to_decode =~ s/&#8734;/&infin;/g;
    $to_decode =~ s/&#8747;/&int;/g;
    $to_decode =~ s/&#921;/&Iota;/g;
    $to_decode =~ s/&#953;/&iota;/g;
    $to_decode =~ s/&#191;/&iquest;/g;
    $to_decode =~ s/&#8712;/&isin;/g;
    $to_decode =~ s/&#922;/&Kappa;/g;
    $to_decode =~ s/&#954;/&kappa;/g;
    $to_decode =~ s/&#923;/&Lambda;/g;
    $to_decode =~ s/&#955;/&lambda;/g;
    $to_decode =~ s/&#171;/&laquo;/g;
    $to_decode =~ s/&#8592;/&larr;/g;
    $to_decode =~ s/&#8968;/&lceil;/g;
    $to_decode =~ s/&#8220;/&ldquo;/g;
    $to_decode =~ s/&#8804;/&le;/g;
    $to_decode =~ s/&#8970;/&lfloor;/g;
    $to_decode =~ s/&#8727;/&lowast;/g;
    $to_decode =~ s/&#9674;/&loz;/g;
    $to_decode =~ s/&#8206;/&lrm;/g;
    $to_decode =~ s/&#8249;/&lsaquo;/g;
    $to_decode =~ s/&#8216;/&lsquo;/g;
    $to_decode =~ s/&#60;/&lt;/g;
    $to_decode =~ s/&#175;/&macr;/g;
    $to_decode =~ s/&#8212;/&mdash;/g;
    $to_decode =~ s/&#181;/&micro;/g;
    $to_decode =~ s/&#8722;/&minus;/g;
    $to_decode =~ s/&#924;/&Mu;/g;
    $to_decode =~ s/&#956;/&mu;/g;
    $to_decode =~ s/&#8711;/&nabla;/g;
    $to_decode =~ s/&#160;/&nbsp;/g;
    $to_decode =~ s/&#8211;/&ndash;/g;
    $to_decode =~ s/&#8800;/&ne;/g;
    $to_decode =~ s/&#8715;/&ni;/g;
    $to_decode =~ s/&#172;/&not;/g;
    $to_decode =~ s/&#8713;/&notin;/g;
    $to_decode =~ s/&#8836;/&nsub;/g;
    $to_decode =~ s/&#925;/&Nu;/g;
    $to_decode =~ s/&#957;/&nu;/g;
    $to_decode =~ s/&#338;/&OElig;/g;
    $to_decode =~ s/&#339;/&oelig;/g;
    $to_decode =~ s/&#8254;/&oline;/g;
    $to_decode =~ s/&#937;/&Omega;/g;
    $to_decode =~ s/&#969;/&omega;/g;
    $to_decode =~ s/&#927;/&Omicron;/g;
    $to_decode =~ s/&#959;/&omicron;/g;
    $to_decode =~ s/&#8853;/&oplus;/g;
    $to_decode =~ s/&#8744;/&or;/g;
    $to_decode =~ s/&#170;/&ordf;/g;
    $to_decode =~ s/&#186;/&ordm;/g;
    $to_decode =~ s/&#8855;/&otimes;/g;
    $to_decode =~ s/&#182;/&para;/g;
    $to_decode =~ s/&#8706;/&part;/g;
    $to_decode =~ s/&#8240;/&permil;/g;
    $to_decode =~ s/&#8869;/&perp;/g;
    $to_decode =~ s/&#934;/&Phi;/g;
    $to_decode =~ s/&#966;/&phi;/g;
    $to_decode =~ s/&#928;/&Pi;/g;
    $to_decode =~ s/&#960;/&pi;/g;
    $to_decode =~ s/&#982;/&piv;/g;
    $to_decode =~ s/&#177;/&plusmn;/g;
    $to_decode =~ s/&#163;/&pound;/g;
    $to_decode =~ s/&#8242;/&prime;/g;
    $to_decode =~ s/&#8243;/&Prime;/g;
    $to_decode =~ s/&#8719;/&prod;/g;
    $to_decode =~ s/&#8733;/&prop;/g;
    $to_decode =~ s/&#936;/&Psi;/g;
    $to_decode =~ s/&#968;/&psi;/g;
    $to_decode =~ s/&#8730;/&radic;/g;
    $to_decode =~ s/&#187;/&raquo;/g;
    $to_decode =~ s/&#8594;/&rarr;/g;
    $to_decode =~ s/&#8969;/&rceil;/g;
    $to_decode =~ s/&#8221;/&rdquo;/g;
    $to_decode =~ s/&#174;/&reg;/g;
    $to_decode =~ s/&#8971;/&rfloor;/g;
    $to_decode =~ s/&#929;/&Rho;/g;
    $to_decode =~ s/&#961;/&rho;/g;
    $to_decode =~ s/&#8207;/&rlm;/g;
    $to_decode =~ s/&#8250;/&rsaquo;/g;
    $to_decode =~ s/&#8217;/&rsquo;/g;
    $to_decode =~ s/&#8218;/&sbquo;/g;
    $to_decode =~ s/&#352;/&Scaron;/g;
    $to_decode =~ s/&#353;/&scaron;/g;
    $to_decode =~ s/&#8901;/&sdot;/g;
    $to_decode =~ s/&#167;/&sect;/g;
    $to_decode =~ s/&#173;/&shy;/g;
    $to_decode =~ s/&#931;/&Sigma;/g;
    $to_decode =~ s/&#963;/&sigma;/g;
    $to_decode =~ s/&#962;/&sigmaf;/g;
    $to_decode =~ s/&#8764;/&sim;/g;
    $to_decode =~ s/&#9824;/&spades;/g;
    $to_decode =~ s/&#8834;/&sub;/g;
    $to_decode =~ s/&#8838;/&sube;/g;
    $to_decode =~ s/&#8721;/&sum;/g;
    $to_decode =~ s/&#8835;/&sup;/g;
    $to_decode =~ s/&#185;/&sup1;/g;
    $to_decode =~ s/&#178;/&sup2;/g;
    $to_decode =~ s/&#179;/&sup3;/g;
    $to_decode =~ s/&#8839;/&supe;/g;
    $to_decode =~ s/&#932;/&Tau;/g;
    $to_decode =~ s/&#964;/&tau;/g;
    $to_decode =~ s/&#8756;/&there4;/g;
    $to_decode =~ s/&#920;/&Theta;/g;
    $to_decode =~ s/&#952;/&theta;/g;
    $to_decode =~ s/&#977;/&thetasym;/g;
    $to_decode =~ s/&#8201;/&thinsp;/g;
    $to_decode =~ s/&#732;/&tilde;/g;
    $to_decode =~ s/&#215;/&times;/g;
    $to_decode =~ s/&#8482;/&trade;/g;
    $to_decode =~ s/&#8593;/&uarr;/g;
    $to_decode =~ s/&#168;/&uml;/g;
    $to_decode =~ s/&#978;/&upsih;/g;
    $to_decode =~ s/&#933;/&Upsilon;/g;
    $to_decode =~ s/&#965;/&upsilon;/g;
    $to_decode =~ s/&#926;/&Xi;/g;
    $to_decode =~ s/&#958;/&xi;/g;
    $to_decode =~ s/&#165;/&yen;/g;
    $to_decode =~ s/&#376;/&Yuml;/g;
    $to_decode =~ s/&#918;/&Zeta;/g;
    $to_decode =~ s/&#950;/&zeta;/g;
    $to_decode =~ s/&#8205;/&zwj;/g;
    $to_decode =~ s/&#8204;/&zwnj;/g;

    return $to_decode;
}


sub escape_html {
    my $toencode = $_[0];
    # Ampersands no seguidos por "amp;"
    $toencode =~ s/&(?!amp;)/&amp;/g;
    # entidades reconocidas por xml
    # >
    $toencode =~ s/>/&gt;/g;
    # <
    $toencode =~ s/</&lt;/g;
    # "
    $toencode =~ s/"/&quot;/g;
    # '
    $toencode =~ s/'/&apos;/g;
    return $toencode;
}

1;
