#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

package lib_fid;
use strict;
use utf8;

# -------------------------------------------------------------------------#
sub parsea_paneles {
    my $buffer = $_[0];
    my $tpl_panel = lib_utils::load_cpan_core_plt("fid/prontus_fid_panel.html");

    my $output = '';
    my ($title, $id, $macro, $macro_param, $tmp_panel);

    while ($buffer =~ /%%panel\(\s*(.*?)\s*,\s*(.*?)\s*,\s*(.*?)(?:,\s*(.*?))?\)%%/sg) {
        $title = $1;
        $id = $2;
        $macro = $3;
        $macro_param = $4;
        $tmp_panel = $tpl_panel;

        $tmp_panel =~ s/%%panel_id%%/$id/g;
        $tmp_panel =~ s/%%panel_title%%/$title/g;

        if ($macro_param) {
            $tmp_panel =~ s/%%panel_macro%%/$macro,$macro_param/g;
        } else {
            $tmp_panel =~ s/%%panel_macro%%/$macro/g;
        }
        $buffer =~ s/%%panel\(\s*$title\s*,\s*$id.*?\)%%/$tmp_panel/sg;
    }

    return $buffer;
}

# -------------------------------------------------------------------------
# Incluye en el tpl las macros senaladas en el con la marca
# %%macro(<nomfilemacro>)%%
# <nomfilemacro> : Nombre del archivo de la macro (con extension y sin path), ubicado dentro del dir macros

# Version especifica de esta funcion, para macros de fids, para considerar simultaneamente las
# reservadas y las de usuario, permitiendo anidamiento cruzado.
sub add_macros_fid {
    my $textpag     = $_[0];        # buffer de la plantilla conteniendo invocaciones a macros
    my $profundidad = $_[1] || 0;        # solo para invocaciones internas, para manejar recursividad
    my $mostrar     = $_[2];        # para determinar macros que dependan de que el artículo tenga vista o no
    my $pag_aux     = $textpag;
    my @dirs_macros = ( "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/fid/macros",
                        "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/macros",
                        "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/fid/macros_reservadas");
    # Recorre plantillas y parsea macros.
    while ($pag_aux =~ /(%%macro\((.+?)\)%%)/g) {
        my $match = $1;
        my $arg_str = $2;
        $arg_str =~ s/ //g;
        my @args_macro = split(",", $arg_str);

        my $nomfile = $args_macro[0];
        my $id      = $args_macro[1];

        $id = 1 if (!$id);
        #lib_stdlog::print_err("MACRO: $nomfile, $id\n");

        # las macros no pueden contener referencias como ../ para intentar moverse a directorios superiores.
        $nomfile =~ s|\.+[/\\]+||g;

        my $dir_macros = '';
        foreach my $dir (@dirs_macros) {
            if (-f "$dir/$nomfile") {
                $dir_macros = $dir;
            }
        }

        my $buffer_macro;
        # si la macro no existe, marcamos la plantilla y continuamos con la siguiente
        if ($dir_macros eq '') {
            my $relpath_macro = lib_utils::remove_front_string($nomfile, $prontus_varglb::DIR_SERVER);
            $buffer_macro = lib_lang::get_interpolated_message('fid_macro_not_found', $relpath_macro);
            $textpag =~ s/\Q$match\E/$buffer_macro/s;
            next;
        }

        $buffer_macro = glib_fildir_03::read_file("$dir_macros/$nomfile");

        # si es galeria con carga masiva debemos parsear los parametros
        if ($nomfile eq '_galeria_prontus.html') {
            $buffer_macro =~ m|<!--LOOP_GAL-->[\n\r]*(.*?) *?<!--/LOOP_GAL-->|s;
            my $loop_gal = $1;
            my $loop_gal_temp;
            my @temp_array = split(/\|/, $id);
            my $temp_buffer = '';
            if (scalar @temp_array < 2) {
                lib_stdlog::print_err(lib_lang::get_message('fid_no_gallery_conf'), "\n");
                # cargamos valores por defecto
                $id = '1:fotofija_galeria@@_low:65x65|2:fotofija_galeria@@_high:800x600';
                @temp_array = split(/\|/, $id);
            }
            my $counter = 1;
            foreach my $conf_size (@temp_array) {
                $loop_gal_temp = $loop_gal;
                my @data_temp = split(/:/, $conf_size);
                $loop_gal_temp =~ s/##i##/$counter/g;
                # se cambia x por , para la resolucion
                my @resolution = split(/x/, $data_temp[2]);
                $loop_gal_temp =~ s/##w##/$resolution[0]/g;
                $loop_gal_temp =~ s/##h##/$resolution[1]/g;

                $temp_buffer .= $loop_gal_temp;
                $counter++;
            }
            $buffer_macro =~ s|<!--LOOP_GAL-->.*?<!--/LOOP_GAL-->|$temp_buffer|s;
            $temp_buffer = $id;
            $temp_buffer  =~ s/:\d+x\d+//g;
            $buffer_macro =~ s/##_gal_conf##/$temp_buffer/s;
        }
        # Mostrar u ocultar panel de análisis SEO
        elsif ($nomfile eq '_zona_seo.html') {
            if (!$mostrar) {
                $buffer_macro =~ s|<!--SEO_API_HABILITADO-->.*?<!--/SEO_API_HABILITADO-->||sg;
            } else {
                $buffer_macro =~ s|<!--/?SEO_API_HABILITADO-->||g;
            }
        }
        elsif ($nomfile eq '_analisis_previews.html') {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_HABILITADO'} eq 'NO') {
                $buffer_macro =~ s|<!--PREVIEW_GOOGLE-->.*?<!--/PREVIEW_GOOGLE-->||sg;
                $buffer_macro =~ s|<!--PREVIEW_FACEBOOK-->.*?<!--/PREVIEW_FACEBOOK-->||sg;
                $buffer_macro =~ s|<!--PREVIEW_TWITTER-->.*?<!--/PREVIEW_TWITTER-->||sg;
            } else {
                $buffer_macro =~ s|<!--/?PREVIEW_GOOGLE-->||g;
                $buffer_macro =~ s|<!--/?PREVIEW_FACEBOOK-->||g;
                if ($mostrar) { # la macro de twitter depende de que el artículo exista.
                    $buffer_macro =~ s|<!--/?PREVIEW_TWITTER-->||g;
                }
            }
        }
        # Mostrar u ocultar panel de análisis SEO
        elsif ($nomfile eq '_analisis_pagespeed.html') {
            if ($mostrar != 1) {
                $buffer_macro =~ s|<!--PAGESPEED_HABILITADO-->.*?<!--/PAGESPEED_HABILITADO-->||sg;
            } else {
                $buffer_macro =~ s|<!--/?PAGESPEED_HABILITADO-->||g;
            }
        }
        # al menos para el vtxt, no quiero que el vtxt default tenga un 1 adosado.
        elsif ($id == '1' && $nomfile eq '_vtxt.html') {
            $id = '';
        }
        $buffer_macro =~ s/##id##/$id/g;

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_ENABLE'} eq 'SI') {
            $buffer_macro =~ s|<!--ADS-DISABLED-->.*?<!--/ADS-DISABLED-->||sg;
            $buffer_macro =~ s|<!--/?ADS-ENABLED-->||g;
        } else {
            $buffer_macro =~ s|<!--ADS-ENABLED-->.*?<!--/ADS-ENABLED-->||sg;
            $buffer_macro =~ s|<!--/ADS-DISABLED-->||g;
        }

        if ($buffer_macro =~ /%%macro\(.+?\)%%/s) {
            if ($profundidad > 10) {
                my $msg = lib_lang::get_message('fid_macro_over_limit');
                $buffer_macro =~ s/%%macro\(.+?\)%%/$msg/g;
            } else {
                $buffer_macro = &add_macros_fid($buffer_macro, $profundidad + 1, $mostrar);
            }
        }
        $textpag =~ s/\Q$match\E/$buffer_macro/s;
    }

    if (!$profundidad) {
        $textpag = lib_parser::ajusta_crlf($textpag);
        $textpag =~ s/[\n\r]+$//g;
    }

    return $textpag;
}

sub get_child_panels {
    my $id = $_[0];
    my @panels = @{$_[1]};
    my @childs;

    foreach my $panel (@panels) {
        if ($panel->{'parentId'} == $id) {
            push @childs, $panel;
        }
    }

    return @childs;
}

sub generate_fid_menu {
    my $config = $_[0];
    my $plantilla = $_[1];

    # Agrupar la multimedia. Dejarla al final en el menú.
    # La galeria queda antepenultimo.
    my @panels = sort { $a->{order} <=> $b->{order} } @{$config->{'panels'}};
    my @multimedia;
    my $galeria;
    my @items;
    my $menu;

    # %%item(Cabecera, body1)%%%%/item%%
    # %%subitem(Video1, body6)%%%%/subitem%%

    foreach my $panel (@panels) {
        my $type = $panel->{'type'};
        next if ($panel->{'parentId'} > 0);

        if ($type eq 'video' || $type eq 'audio') {
            push @multimedia, $panel;
        } else {
            push @items, $panel;
        }
    }

    foreach my $item (@items) {
        $menu .= "%%item($item->{'name'}, panel$item->{'id'})%%%%/item%%\n";
    }

    if (scalar @multimedia > 0) {
        $menu .= "%%item(Multimedia)%%\n";

        foreach my $item (@multimedia) {
            $menu .= "%%subitem($item->{'name'}, panel$item->{'id'})%%%%/subitem%%\n";
        }

        $menu .= "%%/item%%\n";
    }

    $plantilla =~ s/__menu__/$menu/g;

    return $plantilla;
}

sub generate_tabs {
    my @childs = @{$_[0]};
    my @allFields = @{$_[1]};
    my $nombre_macro = $_[2];
    my $filename = $_[3];
    my $dir_plantillas = "$prontus_varglb::DIR_SERVER/prontus/cpan/plantillas/fid/editor";
    my $file = "$dir_plantillas/panel_tabs.html";
    my $dir_fids = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/fid";
    my $dir_macros = "$dir_fids/macros/$filename"; # directorio especial, con el nombre del fid, para guardar las macros generadas.
    my $buffer = glib_fildir_03::read_file($file);

    $buffer =~ m|<!--TABNAV-->(.*?)<!--/TABNAV-->|s;
    my $tabnav = $1;

    $buffer =~ m|<!--TABCONTENT-->(.*?)<!--/TABCONTENT-->|s;
    my $tabcontent = $1;

    # Parsear menu de tabs.
    my $outputtabnav = '';
    my $outputtabcontent = '';
    my $x = 0;
    foreach my $child (@childs) {
        my $tmp = $tabnav;
        my $tmp2 = $tabcontent;
        my $class = "";

        $class = "active" if ($x == 0);

        $tmp =~ s/__name__/$child->{'name'}/g;
        $tmp =~ s/__id__/tabpanel$child->{'id'}/g;
        $tmp =~ s/__class__/$class/g;

        $outputtabnav .= $tmp;

        my @fields = &get_panel_fields(\@allFields, $child->{'id'});
        my $output;

        foreach my $field (@fields) {
            $output .= &get_field_by_type($field) . "\n";
        }

        $tmp2 =~ s/__content__/$output/g;
        $tmp2 =~ s/__id__/tabpanel$child->{'id'}/g;
        $tmp2 =~ s/__class__/$class/g;

        $outputtabcontent .= $tmp2;

        $x++;
    }

    $buffer =~ s|<!--TABNAV-->.*?<!--/TABNAV-->|$outputtabnav|sg;
    $buffer =~ s|<!--TABCONTENT-->.*?<!--/TABCONTENT-->|$outputtabcontent|sg;

    glib_fildir_03::write_file("$dir_macros/$nombre_macro", $buffer);
}

# Validar que no existe un fid con el mismo nombre.
sub generate_fid {
    my ($filename, $json) = @_;
    my $dir_fids = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/fid";
    my $dir_macros = "$dir_fids/macros/$filename"; # directorio especial, con el nombre del fid, para guardar las macros generadas.
    my $file_plantilla = "$prontus_varglb::DIR_SERVER/prontus/cpan/plantillas/fid/fid_template.html";
    my $file_fid = "$dir_fids/$filename.html";

    utf8::encode($json);
    my $hashref = JSON::decode_json($json);

    my @allFields = @{$hashref->{'fields'}};
    my $buffer_plantilla = glib_fildir_03::read_file($file_plantilla);
    my $paneles;
    my @panels = sort { $a->{order} <=> $b->{order} } @{$hashref->{'panels'}};

    glib_fildir_03::check_dir($dir_macros);

    unlink(glob("$dir_macros/*"));

    foreach my $panel (@panels) {
        my $id = $panel->{'id'};
        my $type = $panel->{'type'};
        my $nombre_macro = "panel_$id.html";
        my $output;
        my @fields = &get_panel_fields(\@allFields, $id);

        # Pasa de largo en los paneles hijos.
        next if ($panel->{'parentId'} > 0);

        if ($type eq 'video') {
            $paneles .= "%%panel($panel->{'name'}, panel$id, _video_xcode.html, $panel->{'videoId'})%%\n";
        } elsif ($type eq 'audio') {
            $paneles .= "%%panel($panel->{'name'}, panel$id, _audio.html, $panel->{'audioId'})%%\n";
        } elsif ($type eq 'galeria') {
            $paneles .= "%%panel(Galería de Fotos, panel$id, _galeria_prontus.html, 1:fotofija_galeria@\@_low:$panel->{'lowWidth'}x$panel->{'lowHeight'}|2:fotofija_galeria@\@_high:$panel->{'highWidth'}x$panel->{'highHeight'})%%\n";
        } elsif ($type eq 'tabs') {
            # Busca los panales hijos.
            my @childs = &get_child_panels($id, \@panels);

            if (scalar @childs > 0) {
                &generate_tabs(\@childs, \@allFields, $nombre_macro, $filename);
                $paneles .= "%%panel($panel->{'name'}, panel$id, $filename/$nombre_macro)%%\n";
            }
        } else {
            foreach my $field (@fields) {
                $output .= &get_field_by_type($field) . "\n";
            }

            glib_fildir_03::write_file("$dir_macros/$nombre_macro", $output);

            $paneles .= "%%panel($panel->{'name'}, panel$id, $filename/$nombre_macro)%%\n";
        }
    }

    $buffer_plantilla =~ s/__paneles__/$paneles/g;
    $buffer_plantilla = &generate_fid_menu($hashref, $buffer_plantilla);

    glib_fildir_03::write_file($file_fid, $buffer_plantilla);
}

sub get_panel_fields {
    my @fields = @{$_[0]};
    my $id = $_[1];
    my @panelFields;

    foreach my $field (@fields) {
        if ($field->{'panelId'} eq $id) {
            push @panelFields, $field;
        }
    }

    my @panelFieldsFinal = sort { $a->{order} <=> $b->{order} } @panelFields;

    return @panelFieldsFinal;
}

sub save_fid_json {
    my ($filename, $json) = @_;
    my $dir = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/fid/editor";
    my $filepath = "$dir/$filename.json";

    glib_fildir_03::check_dir($dir);
    glib_fildir_03::write_file($filepath, $json);

    # lib_stdlog::print_err("filepath[$filepath]\n");

    # Finalmente generar fid.html y actualizar plantilla si es posible
    &generate_fid($filename, $json);
}

sub get_field_by_type {
    my $field = $_[0];
    my $dir_plantillas = "$prontus_varglb::DIR_SERVER/prontus/cpan/plantillas/fid/editor";
    my $file = "$dir_plantillas/field_$field->{'type'}.html";
    my $buffer_field;
    my $type = $field->{'type'};

    if ($type eq 'input') {
        $buffer_field = glib_fildir_03::read_file($file);

        $buffer_field =~ s/__nombre__/$field->{'name'}/g;
        $buffer_field =~ s/__placeholder__/$field->{'placeholder'}/g;
        $buffer_field =~ s/__clases__/$field->{'clases'}/g;

        return $buffer_field;
    } elsif ($type eq 'textarea') {
        $buffer_field = glib_fildir_03::read_file($file);

        $buffer_field =~ s/__nombre__/$field->{'name'}/g;
        $buffer_field =~ s/__placeholder__/$field->{'placeholder'}/g;

        return $buffer_field;
    } elsif ($type eq 'checkbox') {
        $buffer_field = glib_fildir_03::read_file($file);

        $buffer_field =~ s/__nombre__/$field->{'name'}/g;
        $buffer_field =~ s/__placeholder__/$field->{'placeholder'}/g;
        $buffer_field =~ s/__value__/$field->{'value'}/g;

        return $buffer_field;
    } elsif ($type eq 'bajada') {
        $buffer_field = glib_fildir_03::read_file($file);

        return $buffer_field;
    } elsif ($type eq 'vtxt') {
        $buffer_field = glib_fildir_03::read_file($file);

        $buffer_field =~ s/__nombre__/$field->{'name'}/g;
        $buffer_field =~ s/__id__/$field->{'vtxtId'}/g;

        return $buffer_field;
    } elsif ($type eq 'cuerpo') {
        return "%%macro(_vtxt.html)%%";
    } elsif ($type eq 'tax') {
        return "%%_taxonomia%%";
    } elsif ($type eq 'taxmanual') {
        return "%%macro(_tax_manual.html)%%";
    } elsif ($type eq 'tags') {
        return "%%macro(_tags.html)%%";
    } elsif ($type eq 'multitags') {
        return "%%macro(_multitag.html)%%";
    } elsif ($type eq 'combobox') {
        $buffer_field = glib_fildir_03::read_file($file);

        $buffer_field =~ s/__nombre__/$field->{'name'}/g;
        $buffer_field =~ s/__placeholder__/$field->{'placeholder'}/g;

        my @options = @{$field->{'options'}};
        my $str_options;

        foreach my $option (@options) {
            $str_options .= "<option value=\"$option->{'value'}\">$option->{'name'}</option>\n";
        }

        $buffer_field =~ s/__options__/$str_options/g;

        return $buffer_field;
    } elsif ($type eq 'adjunto') {
        return "%%$field->{'placeholder'}($field->{'name'})%%";
    } elsif ($type eq 'macro') {
        return "%%macro($field->{'file'})%%";
    } elsif ($type eq 'foto') {
        return "%%$field->{'placeholder'}($field->{'width'},$field->{'height'},$field->{'name'})%%";
    }
}


sub get_first_fid {
    my %art_tipos = %{$_[0]};
    my $first_fid = (sort(keys %art_tipos))[0];
    if (defined($art_tipos{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'}})) {
        return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'};
    } else {
        return $first_fid;
    }
}

1;
