#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# procesa y parsea marcas y estructuras generales de Prontus
package lib_parser;
use strict;
use utf8;

# usar asi: $buffer =~ s/$CRLF/<p>/sg;
our $CRLF = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
# 12.0 Variables usadas por procesa_ifs.
our $IF_BUFFER = '';
our $IF_INDEX = 0;
our %IF_VARS = ();

# ---------------------------------------------------------------
# procesa condicionales if, nif  de prontus
sub procesa_condicional {
    my ($buffer, $vars_common, $vars_adicionales) = @_;

    # Agrega algunas claves reservadas que no vienen en el XML, para poder usarlas en los if con operadores
    if (ref $vars_adicionales) {
        foreach my $k (keys %$vars_adicionales) {
            $$vars_common{lc $k} = $$vars_adicionales{$k};
        }
    }
    $buffer = &procesa_ifs('', $buffer, $vars_common, $vars_adicionales);

    return $buffer;
}

# ----------------------------------------------------------------------------- #
# 12.0 Rutinas que componen procesa_ifs.

# ----------------------------------------------------------------------------- #
# Parser de ifs, compatible con parser_condicional
sub procesa_ifs {
    my ($sent,$buffer,$vars1,$vars2) = @_;

    my $if_cont_condicionales_begin = 0;
    my $nif_cont_condicionales_begin = 0;
    my $if_cont_condicionales_end = 0;
    my $nif_cont_condicionales_end = 0;

    # se hace utf8::encode y posteriormente utf8::decode, para aumentar la velocidad de procesamiento de las regexp
    # ya que no estamos buscando caracteres extendidos
    utf8::encode($buffer);
    $buffer =~ s/%%if\(/%%if\(/g;
    $buffer =~ s/%%\/if%%/%%\/if%%/g;
    $buffer =~ s/%%nif\(/%%nif\(/g;
    $buffer =~ s/%%\/nif%%/%%\/nif%%/g;

    while ($buffer =~ /%%(n?)if\([^\)]+\)%%/g) {
        if ($1) {
            $nif_cont_condicionales_begin++;
        } else {
            $if_cont_condicionales_begin++;
        }
    }
    while ($buffer =~ /%%\/(n?)if%%/g) {
        if ($1) {
            $nif_cont_condicionales_end++;
        } else {
            $if_cont_condicionales_end++;
        }
    }
    utf8::decode($buffer);

    my $error = 0;
    if ($if_cont_condicionales_begin != $if_cont_condicionales_end) {
        my $err = lib_lang::get_interpolated_message('parser_error_if_tag', $if_cont_condicionales_begin, $if_cont_condicionales_end);
        $buffer = "<h1>$err</h1>". $buffer;
        lib_stdlog::print_err($err, "\n");
        $error = 1;
    }
    if ($nif_cont_condicionales_begin != $nif_cont_condicionales_end) {
        my $err = lib_lang::get_interpolated_message('parser_error_nif_tag', $nif_cont_condicionales_begin, $nif_cont_condicionales_end);
        $buffer = "<h1>$err</h1>". $buffer;
        lib_stdlog::print_err($err, "\n");
        $error = 1;
    }

    return $buffer if ($error);

    # Adapta las variables a lo requerido por el parser.
    # Inicializa el buffer que contiene la plantilla.
    $IF_BUFFER = $buffer;
    $IF_INDEX = 0;
    # Llena el hash de datos.
    %IF_VARS = ();
    foreach my $key (keys %$vars1) {
        # Todas las variables son case insensitive.
        $IF_VARS{lc $key} = $$vars1{$key};
    }
    foreach my $key (keys %$vars2) {
        $IF_VARS{lc $key} = $$vars2{$key};
    }
    # Llama al parser de IFs.
    my $res = &_parser_ifs('start');
    # Avisa si se detectó un desbalanceo de IFs.
    if ($IF_INDEX != length($IF_BUFFER)) {
        $res .= "\n*** ". lib_lang::get_message('parser_error_prontus_tags')." ***\n";
    }
    return $res;
} # procesa_ifs

sub _parser_ifs {
    my ($status) = $_[0];
    my ($begin);
    my $largoBuffer = length($IF_BUFFER);
    # Texto parseado.
    my $resultado = '';

    # Selector de estados.
    if ($status eq 'start') {
        # Estado "sin marca". Busca una marca y pasa al estado corespondiente, retornando
        # el texto de antes de encontrar la marca + el texto que retorna cada marca que
        # encuentra.
        while ($IF_INDEX < $largoBuffer) {
            $begin = $IF_INDEX;
            # Busca una marca.
            &_fwToNextTag();
            # Suma al resultado el trozo de la plantilla sin marcas.
            $resultado .= substr($IF_BUFFER,$begin,($IF_INDEX - $begin));
            # Reconoce el tipo de marca.
            if (substr($IF_BUFFER,$IF_INDEX, 5) eq '%%if(') {
                # Marca IF. Pasa al estado 'if'.
                $IF_INDEX += 5;
                $resultado .= &_parser_ifs('if');
            }elsif (substr($IF_BUFFER,$IF_INDEX, 6) eq '%%nif(') {
                # Marca NIF. Pasa al estado 'nif'.
                $IF_INDEX += 6;
                $resultado .= &_parser_ifs('nif');
            }elsif (substr($IF_BUFFER,$IF_INDEX, 7) eq '%%/if%%') {
                # Fin de marca. Detiene el proceso para retornar el control a la instancia superior.
                # Al no avanzar el indice, la instancia superior reconoce el fin de marca.
                last;
            }elsif (substr($IF_BUFFER,$IF_INDEX, 8) eq '%%/nif%%') {
                # Fin de marca. Detiene el proceso para retornar el control a la instancia superior.
                # Al no avanzar el indice, la instancia superior reconoce el fin de marca.
                last;
            }else {
                # No es un IF ni un NIF, asi que la marca no importa,
                # no se hace nada ya que automaticamente se avanzará
                # hasta el siguiente %% en busca de un if o un nif.
                # Agrega %% al resultado y se salta hasta el cierre de marca.
                if ($IF_INDEX < $largoBuffer) {
                    $begin = $IF_INDEX;
                    $IF_INDEX += 2;
                    &_fwToNextTag();
                    $IF_INDEX += 2;
                    $resultado .= substr($IF_BUFFER,$begin,($IF_INDEX - $begin));
                }
            }
        }
    } elsif ($status eq 'if') {
        # Busca el fin de la condicion.
        my $condicion = &_getTag(')%%');
        # Reconoce y procesa la condicion.
        my $condicionSeCumple = &_conditionParsing($condicion);
        if ($condicionSeCumple) {
            # Procesa plantilla dentro de la marca.
            $resultado .= &_parser_ifs('start');
            # Verifica que la siguiente marca es un %%/if%% y la salta.
            &_endTag('%%/if%%');
        } else {
            # Avanza el indice hasta el fin del IF. No suma nada al resultado.
            &_ignoreTag('%%if(','%%/if%%');
        }
    } elsif ($status eq 'nif') {
        # Reconoce la condicion.
        # Busca el fin de la condicion.
        my $condicion = &_getTag(')%%');
        # Reconoce y procesa la condicion.
        my $condicionSeCumple = &_conditionParsing($condicion);
        if (! $condicionSeCumple) {
            # Procesa plantilla dentro de la marca.
            $resultado .= &_parser_ifs('start');
            # Verifica que la siguiente marca es un %%/nif%% y la salta.
            &_endTag('%%/nif%%');
        }else{
            # Avanza el indice hasta el fin del NIF. No suma nada al resultado.
            &_ignoreTag('%%nif(','%%/nif%%');
        }
    } # if $status
    return $resultado;
} # _parser_ifs

# Procesamiento de una condicion.
# (<variable_prontus>)
# (<variable_prontus> <operador> <literal>)
sub _conditionParsing {
    my $condicion = $_[0];
    my ($valor, $theVar, $comparador, $literal, $opr1, $opr2);
    my $patVar        = '^\s*([^=!<>\s]+)\s*$';
    my $patComparison = '([^=!<>\s]+)\s*(={1,2}|\!=|>|<|>=|<=|eq|ne|lt|gt|le|ge|~|!~)\s*([^=!<>]+)\s*';
    # lib_stdlog::print_err("condicion [$condicion]\n");
    # Procesamiento de la condicion.
    # Si no se detecta un patrón establecido se entiende que la condición no se cumple.
    my $condicionSeCumple = 0;
    if ($condicion =~ /$patVar/) {
        # Es una variable.
        $theVar = $1;
        # lib_stdlog::print_err("theVar [$theVar]\n");
        $valor = $IF_VARS{lc $theVar};
        if ($valor) {
            $condicionSeCumple = 1;
        }
    } elsif ($condicion =~ /$patComparison/) {
        # Se trata de una comparacion.
        $theVar     = $1;
        $comparador = $2;
        $literal    = $3;
        # Trimea el literal.
        $literal =~ s/[\'\"\s]+$//g;
        $literal =~ s/^[\'\"\s]+//g;
        # lib_stdlog::print_err("theVar [$theVar]\n");
        $opr1 = $IF_VARS{lc $theVar};
        $opr2 = $literal;
        if (defined $IF_VARS{lc $opr2}) {
            $opr2 = $IF_VARS{lc $opr2};
        }
        # lib_stdlog::print_err("evaluar [$opr1] [$comparador] [$opr2]\n");

        if      ($comparador eq '=')  { $condicionSeCumple = ($opr1 == $opr2);
        } elsif ($comparador eq '==') { $condicionSeCumple = ($opr1 == $opr2);
        } elsif ($comparador eq '!=') { $condicionSeCumple = ($opr1 != $opr2);
        } elsif ($comparador eq '>=') { $condicionSeCumple = ($opr1 >= $opr2);
        } elsif ($comparador eq '<=') { $condicionSeCumple = ($opr1 <= $opr2);
        } elsif ($comparador eq '<')  { $condicionSeCumple = ($opr1 < $opr2);
        } elsif ($comparador eq '>')  { $condicionSeCumple = ($opr1 > $opr2);
        } elsif ($comparador eq 'eq') { $condicionSeCumple = ($opr1 eq $opr2);
        } elsif ($comparador eq 'ne') { $condicionSeCumple = ($opr1 ne $opr2);
        } elsif ($comparador eq 'gt') { $condicionSeCumple = ($opr1 gt $opr2);
        } elsif ($comparador eq 'ge') { $condicionSeCumple = ($opr1 ge $opr2);
        } elsif ($comparador eq 'lt') { $condicionSeCumple = ($opr1 lt $opr2);
        } elsif ($comparador eq 'le') { $condicionSeCumple = ($opr1 le $opr2);
        } elsif ($comparador eq '~')  { $condicionSeCumple = ($opr1 =~ /$opr2/);
        } elsif ($comparador eq '!~') { $condicionSeCumple = ($opr1 !~ /$opr2/);
        }

    } else {
        lib_stdlog::print_err(lib_lang::get_interpolated_message('parser_unknown_condition_result', $condicion, $condicionSeCumple));
    }
    return $condicionSeCumple;
} # _conditionParsing

# Avanza hasta el proximo tag.
sub _fwToNextTag {
    my ($idx);
    # Busca el delimitador de tag.
    $idx = index($IF_BUFFER,'%%',$IF_INDEX);
    if ($idx >= 0) {
        $IF_INDEX = $idx;
    } else {
        # Si no lo encontró, avanza el índice hasta el final del buffer
        # para terminar el procesamiento.
        $IF_INDEX = length($IF_BUFFER);
    }
} # _fwToNextTag

# Avanza hasta el delimitador, obtiene el tag y avanza el indice.
sub _getTag {
    my $delim = $_[0];
    my ($idx,$theTag);
    my $largo = length($delim);
    my $begin = $IF_INDEX;
    # Busca el fin de tag para completar el tag.
    $idx = index($IF_BUFFER,$delim,$IF_INDEX);
    if ($idx >=0) {
        $theTag = substr($IF_BUFFER,$begin,($idx - $begin));
        $IF_INDEX = $largo + $idx; # Avanza hasta el final de la marca.
    } else {
        # Si no lo encontró, avanza el índice hasta el final del buffer
        # para terminar el procesamiento.
        $IF_INDEX = length($IF_BUFFER);
    }
    return $theTag;
} # _getTag

# Ignora trozo de plantilla correspondiente a inicio y fin de marca.
# Esto se hace avanzando el indice hasta el cierre de la marca e ignorando marcas anidadas.
sub _ignoreTag {
    my ($tagStart,$tagEnd) = @_;
    my ($idxU,$idxL,$startIdx,$endIdx,$startCc,$endCc);
    my $largoBuffer = length($IF_BUFFER);
    my $largoStart = length($tagStart);
    my $largoEnd = length($tagEnd);

    # Busca el fin de tag desde la posición actual del índice.
    $startIdx = $IF_INDEX;
    $endIdx   = $IF_INDEX;
    $startCc = 1;
    $endCc = 0;
    while (($endCc < $startCc) && ($endIdx < $largoBuffer)) {
        # Busca un fin de marca.
        $endIdx = index($IF_BUFFER, $tagEnd, $endIdx);
        if ($endIdx > 0) {
            $endCc++;
            $endIdx++;
        }else{
            # Si no lo encuentra asigna el largo del buffer para que salga del loop.
            $endIdx = $largoBuffer;
            }
        # Busca un inicio de marca uc y luego lc.
        $startIdx = index($IF_BUFFER, $tagStart, $startIdx);
        if (($startIdx > 0) && ($startIdx < $endIdx)) {
            $startIdx++;
            $startCc++;
        }
    }
    $IF_INDEX = $endIdx + $largoEnd - 1;
} # _ignoreTag

# Verifica fin de tag y avanza el indice.
sub _endTag {
    my $tagEnd = $_[0];
    if (substr($IF_BUFFER, $IF_INDEX, length($tagEnd)) eq $tagEnd) {
        $IF_INDEX += length($tagEnd);
    }else{
        # Si no lo encontró, avanza el índice hasta más allá del final del buffer
        # para terminar el procesamiento y que señale el error.
        $IF_INDEX = 10 + length($IF_BUFFER);
    }
} # _endTag

# 12.0 /Rutinas que componen procesa_ifs.

# ---------------------------------------------------------------
# Genera y retorna el parseo de un articulo en una fila
sub generar_fila {
    my ($art_ts, $loop, $loopcounter, $tot_artics, $mv, $nro_pag) = @_;
    $tot_artics = $tot_artics || '';
    my $fila = $loop;

    if ($art_ts ne '') {
        if (!defined($lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts})) {
            unless($lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts} = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $art_ts,
                'no_check_dirs'      => 1)) {
                lib_stdlog::print_err(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");
                return ('');
            };
        }

        my %claves_adicionales = (
            '_nropagina' => $nro_pag,
            '_loopcounter' => $loopcounter,
            '_totartics' => $tot_artics
        );
        # agrega vista a datos de artículo
        $claves_adicionales{'_mv'} = $mv;

        my $art_dirfecha = substr($art_ts, 0, 8);

        $fila = &procesa_loop_artic($fila,'##');
        # procesar loops e if de multitag
        $fila = &procesar_condicional_extra($fila, $loopcounter);
        $fila = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->parse_artic_data($fila, \%claves_adicionales, $mv);
        $fila = lib_prontus::parser_custom_function($fila);

        # Borra marcas no sustituidas
        $fila =~ s/%%.+?%%//g;
    } else {
        # Armar la fila sin datos.
        $fila = '';
    }

    return $fila;
}

# procesa ifv y nifv
sub procesar_condicional_extra {
    my ($buffer, $loopcounter) = @_;
    my $localbuf = $buffer;

    my $error_div = lib_lang::get_message('parser_error_div_by_0');

    while ($buffer =~ /%%ifv\((\d+)\, *(\d+)\)%%.+?%%\/ifv%%/sg) {
        my ($div, $res) = ($1, $2);

        if (!$div) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%/<h1>$error_div<\/h1>/s;
            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod != $res) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%//s;
        }
    }

    while ($buffer =~ /%%nifv\((\d+)\, *(\d+)\)%%.+?%%\/nifv%%/sg) {
        my ($div, $res) = ($1, $2);

        if (!$div) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%/<h1>$error_div<\/h1>/s;
            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod == $res) {
            $localbuf =~ s/%%(nifv\($div\, *$res\))%%(.+?)%%\/(nifv)%%//s;
        }
    }

    return $localbuf;
}


# ---------------------------------------------------------------
# recibe el buffer de texto y una string, e.g. ## ,  que delimita las marcas a reemplazar dentro del loop_artic.
# devuelve el buffer procesado.
sub procesa_loop_artic {
    my ($buffer, $doblegato_string) = @_;

    my $aux = $buffer;
    my %ready;
    while ($buffer =~ /(%%_loop_artic\(\s*(\d+)\s*,\s*(\d+)\s*\)%%(.*?)%%\/_loop_artic%%)/sg) {
        my $loop_artic = $1;
        my $inicio = $2;
        my $fin    = $3;
        my $loop   = $4;
        # salta marcas ya procesadas
        next if ($ready{$loop_artic});

        # lib_stdlog::print_err("_loop_artic($inicio,$fin)\n");
        # lib_stdlog::print_err("loop[$loop]\n");
        my $totloop = '';
        for (my $i = $inicio ; $i <= $fin ; $i++) {
            # lib_stdlog::print_err("for($i)\n");
            my $looptemp = $loop;
            # lib_stdlog::print_err("looptemp = $looptemp\n");
            $looptemp =~ s/\Q$doblegato_string\Ei\Q$doblegato_string\E/$i/g;

            $totloop = $totloop . $looptemp;
        }
        # lib_stdlog::print_err("totloop[$totloop]\n");
        $aux =~ s/\Q$loop_artic\E/$totloop/sg;
    }
    return $aux;
}    #procesa_loop_artic

sub parse_globals {
    my ($buffer, $nom_edic) = @_;

    $prontus_varglb::PRONTUS_ID = 'prontus' if ($prontus_varglb::PRONTUS_ID eq '');

    utf8::encode($buffer);
    $buffer =~ s/%%_nom_edic%%/$nom_edic/g;
    $buffer =~ s/%%_server_name%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}/g;
    $buffer =~ s/%%_frontend_protocol%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'}:\/\//g;
    $buffer =~ s/%%_cpan_server_name%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'}/g;
    $buffer =~ s/%%_cpan_protocol%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'}:\/\//g;
    $buffer =~ s/%%_prontus_id%%/$prontus_varglb::PRONTUS_ID/g;
    $buffer =~ s/%%_prontus_version_corta%%/$prontus_varglb::RAMA_INSTALADA.$prontus_varglb::NRO_REVISION_INSTALADA$prontus_varglb::BETA_REVISION_INSTALADA/g;
    $buffer =~ s/%%_prontus_version%%/$prontus_varglb::VERSION_PRONTUS/g;
    $buffer =~ s/%%_prontus_user_id%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}/g;
    $buffer =~ s/%%_recaptcha_sitekey%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_SITEKEY'}/g;
    $buffer =~ s/%%_seo_twitter_handle%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_TWITTER_HANDLE'}/g;
    $buffer =~ s/%%_seo_fb_id%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_FB_ID'}/g;
    utf8::decode($buffer);
    # seo global id puede contener caracteres utf8
    $buffer =~ s/%%_seo_global_id%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_GLOBAL_ID'}/g;

    return $buffer;
}

# Incluye en el tpl las macros señaladas en el con la marca
# %%macro(<nomfilemacro>)%%
# <nomfilemacro> : Nombre del archivo de la macro (con extension y sin path), ubicado dentro del dir macros
sub add_macros {
    my $textpag     = $_[0];    # buffer de la plantilla conteniendo invocaciones a macros
    my $dir_macros  = $_[1];    # dir. fisico completo, sin slash al final
    my $profundidad = $_[2] || 0;    # solo para invocaciones internas, para manejar recursividad
    my $prefijo     = $_[3];    # optativo, para considerar solo las macros q comiencen con cierto prefijo
    my ($nomfile, $id, $buffer_macro);

    my $pag_aux = $textpag;

    # Recorre tpl y parsea macros.
    while ($pag_aux =~ /(%%macro\((.+?)\)%%)/g) {
        my $tag = $1;
        my $arg_str = $2;
        $arg_str =~ s/ //sg;
        my @args_macro = split(",", $arg_str);

        $nomfile = $args_macro[0];
        $id      = $args_macro[1];
        $id = 1 if (!$id);

        # lib_stdlog::print_err("nomfile[$nomfile] prefijo[$prefijo]\n");

        if ($prefijo ne '') {
            next if ($nomfile !~ /^$prefijo/);
        }

        # las macros no pueden contener referencias como ../ para intentar moverse a directorios superiores.
        $nomfile =~ s|\.+[/\\]+||g;

        if (!-f "$dir_macros/$nomfile") {
            my $relpath_macro = lib_utils::remove_front_string("$dir_macros/$nomfile", $prontus_varglb::DIR_SERVER);
            $buffer_macro = lib_lang::get_interpolated_message('parser_macro_not_found', $relpath_macro);
            $textpag =~ s/\Q$tag\E/$buffer_macro/s;
            next;
        }

        $buffer_macro = glib_fildir_03::read_file("$dir_macros/$nomfile");
        # Parsea id recibido por parametro. Por defecto es 1.
        $buffer_macro =~ s/##id##/$id/sg;

        if ($buffer_macro =~ /%%macro\(.+?\)%%/) {
            if ($profundidad > 10) {
                my $msg = lib_lang::get_message('parser_error_max_lvl_macro');
                $buffer_macro =~ s/%%macro\(.+?\)%%/$msg/g;
            } else {
                $buffer_macro = &add_macros($buffer_macro, $dir_macros, $profundidad+1, $prefijo);
            }
        }

        $textpag =~ s/\Q$tag\E/$buffer_macro/s;
    }

    # parsear globales de prontus en macro cargada
    if (!$profundidad) {
        $textpag = &ajusta_crlf($textpag);
        $textpag = &parse_globals($textpag, '');
        $textpag =~ s/[\n\r]+$//g;
    }

    return $textpag;
}

# ---------------------------------------------------------------
# elimina archivos de relacionados por tags y tax manual
# params:
# directorio a revisar
# timestamp del artículo a tratar
# si hay que borrar todo o solamente los relacionados por taxonomía
sub borrar_relacionados {
    my ($dst_dir, $ts, $solo_tax) = @_;

    if ($solo_tax) {
        my @files = glib_fildir_03::lee_dir($dst_dir);
        foreach my $archivo (@files) {
            if ($archivo !~ /${ts}_tags_.*/) {
                unlink $archivo;
            }
        }
    } else {
        my $pat = $dst_dir . '/' . $ts . '_*.*';
        unlink glob($pat);
    }
}

sub clean_spaces {
    my $buffer = $_[0];
    $buffer =~ s/ {2,}[\n\r]//g;
    return $buffer;
}

sub ajusta_crlf {
    my $buffer = $_[0];
    $buffer =~ s/$CRLF/\x0a/sg;
    return $buffer;
}

1;
