#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

package lib_platform;

use strict;
use utf8;

# clase para resaltar con CSS mensajes de error en la vista web.
our $ERROR_CSS_CLASS = 'check-error';

# Check versions of dependencies.  0 for version = any version acceptable
our $MODULES = [
    {name => 'DBI', version => '1.50'},
    {name => 'DBD::mysql', version => '3.0002'},
    {name => 'GD',version => '2.30'},
    {name => 'JSON',version => '2.14'},
    {name => 'LockFile::Simple',version => '0.205'},
    # email
    {name => 'Email::Simple', version => '2.203'},
    {name => 'Email::MIME', version => '1.926'},
    {name => 'Email::Sender',version => '1.300016'},
    {name => 'IO::All', version => '0.79'},
    {name => 'File::LibMagic', version => '1.02'},
    {name => 'Try::Tiny', version => '0.22'},

    {name => 'XML::Parser',version => '2.34'},
    {name => 'HTTP::Response',version => '1.53',from => 'libwww'},
    {name => 'LWP::UserAgent',version => '2.033',from => 'libwww'},
    {name => 'LWP::Protocol::https',version => '6.06'},
    {name => 'Net::DNS', version => '0.65'},
    {name => 'URI::Escape', version => '3.20'},
    {name => 'Crypt::Eksblowfish::Bcrypt', version => '0.009'},
    {name => 'MIME::Base64', version => '3.15'},
    {name => 'Crypt::JWT', version => '0.023'},
    {name => 'Digest::SHA', version => '5.60'},
    {name => 'MIME::Parser', version => '5'},
    {name => 'Image::ExifTool', version => '9'},
    {name => 'Net::Amazon::S3', version => '0.85'},
];

# vers_cmp is adapted from Sort::Versions 1.3 1996/07/11 13:37:00 kjahds,
# which is not included with Perl by default, hence the need to copy it here.
# Seems silly to require it when this is the only place we need it...
sub vers_cmp {
    if (@_ < 2) { die "not enough parameters for vers_cmp" }
    if (@_ > 2) { die "too many parameters for vers_cmp" }
    my ($a, $b) = @_;
    my (@A) = ($a =~ /(\.|\d+|[^\.\d]+)/g);
    my (@B) = ($b =~ /(\.|\d+|[^\.\d]+)/g);
    my ($A,$B);
    while (@A and @B) {
        $A = shift @A;
        $B = shift @B;
        if ($A eq "." and $B eq ".") {
            next;
        } elsif ( $A eq "." ) {
            return -1;
        } elsif ( $B eq "." ) {
            return 1;
        } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
            return $A <=> $B if $A <=> $B;
        } else {
            $A = uc $A;
            $B = uc $B;
            return $A cmp $B if $A cmp $B;
        }
    }
    @A <=> @B;
}

# ---------------------------------------
# This was originally clipped from the libnet Makefile.PL, adapted here to
# use the above vers_cmp routine for accurate version checking.
sub have_vers {
    my ($web, $pkg, $wanted, $realname) = @_;
    my $error_status = '';
    my ($msg, $vnum, $vstr, $result);
    my $pkg4display = $pkg;
    $pkg4display .= "($realname)" if ($realname);

    my $version = !$wanted?'(any version)':"(v$wanted)";

    $result = sprintf("%28s %-12s ", $pkg4display, !$wanted?'(any)':"(v$wanted)");

    # Modules may change $SIG{__DIE__} and $SIG{__WARN__}, so localise them here
    # so that later errors display 'normally'
    local $::SIG{__DIE__};
    local $::SIG{__WARN__};

    my $file = $pkg . ".pm";
    $file =~ s/::/\//g;
    eval {require $file;};

    # do this twice to avoid a "used only once" error for these vars
    if ($@) {
        $vnum = -1;
    } else {
        $vnum = 0;
        eval { $vnum = $pkg->VERSION; };
        if ($@) {
            eval { $vnum = $pkg->Version; };
        }
    }

    if ($vnum eq "-1") { # string compare just in case it's non-numeric
        $vstr = lib_lang::get_interpolated_message('check_platform_not_found');
    } elsif (vers_cmp($vnum,"0") > -1) {
        $vstr = lib_lang::get_interpolated_message('check_platform_version_found', $vnum);
    } else {
        $vstr = lib_lang::get_message('check_platform_version_unknown');
    }

    my $vok = (vers_cmp($vnum,$wanted) > -1);

    my $pass = lib_lang::get_interpolated_message('check_platform_version_ok', $vstr);
    my $fail = lib_lang::get_interpolated_message('check_platform_version_fail', $vstr);
    $result .= $vok ? $pass : $fail;
    if ($vok) {
        $error_status = '';
    } else {
        $error_status = $ERROR_CSS_CLASS;
    }

    my $datos_gd = '';
    if ($pkg eq 'GD' && $error_status eq '') {
        my $template_name = "core/check_soporte_gd.txt";
        if ($web) {
            $template_name = "core/check_soporte_gd.html";
        }
        $datos_gd = lib_utils::load_cpan_core_plt($template_name);
        $datos_gd = lib_utils::set_coreplt_ppal_simple($datos_gd);

        my $resultado_check;
        ($resultado_check, $error_status) = &check_soporte_gd('gif');
        $datos_gd =~ s/%%soporte_gif%%/$resultado_check/g;
        $datos_gd =~ s/%%error_status_gif%%/$error_status/g;

        ($resultado_check, $error_status) = &check_soporte_gd('jpeg');
        $datos_gd =~ s/%%soporte_jpeg%%/$resultado_check/g;
        $datos_gd =~ s/%%error_status_jpeg%%/$error_status/g;

        ($resultado_check, $error_status) = &check_soporte_gd('png');
        $datos_gd =~ s/%%soporte_png%%/$resultado_check/g;
        $datos_gd =~ s/%%error_status_png%%/$error_status/g;

        ($resultado_check, $error_status) = &check_soporte_gd('webp');
        $datos_gd =~ s/%%soporte_webp%%/$resultado_check/g;
        $datos_gd =~ s/%%error_status_webp%%/$error_status/g;
        $datos_gd .= "\n";
        $error_status = '';
    }
    $result .= $datos_gd;

    return $result, $error_status;
}

# Checks if the object or class has a method called METHOD. If it does then a reference to the sub is returned.
# If it does not then undef is returned. This includes methods inherited or imported by $obj, CLASS, or VAL.
sub check_soporte_gd { # jpeg, libungif y libpng
    my $format = $_[0];
    my $status;
    if (GD::Image->can($format)) {
        return sprintf("%28s ", lib_lang::get_message('check_platform_gd_support')." $format") . ' OK', '';
    } else {
        return sprintf("%28s %-12s ", lib_lang::get_message('check_platform_error'). ' ' . lib_lang::get_message('check_platform_gd_support') . " $format", lib_lang::get_message('check_platform_not_found')), $ERROR_CSS_CLASS;
    }
}

sub check_xcoding {
    my ($web, $path_ffmpeg) = @_;
    my $xcoding_ver = '2.1.0';
    my $result;
    my $error_status = '';
    my $template_name = "core/check_xcoding.txt";
    if ($web) {
        $template_name = "core/check_xcoding.html"
    }
    my $datos_xcoding = lib_utils::load_cpan_core_plt($template_name);
    $datos_xcoding = lib_utils::set_coreplt_ppal_simple($datos_xcoding);
    $datos_xcoding =~ s/%%path_ffmpeg%%/$path_ffmpeg/g;

    # Primero se chequea la version
    my $resp = `$path_ffmpeg -version 2>&1`;
    if($resp =~ /^FFmpeg version ([^\s,]+)/i) {
        my $ver = $1;
        my $origver = $ver;
        $result = sprintf("%28s %-12s ", 'FFmpeg', "($xcoding_ver)");
        $datos_xcoding =~ s/%%result_xcoding_ver%%/$result/g;

        # eliminamos el texto '.git' si es una version de git, lo reemplazamos por un 0
        $ver =~ s/(git)/0/;
        # si es una version sin subversion agregamos 0
        $ver =~ s/^(\d+\.\d+)$/$1.0/;
        # deja solo los 3 numeros de la versión
        $ver =~ s/((\d+\.\d+\.\d+).*)/$2/;

        if($ver =~ /\d+\.\d+\.\d+/) {
            my $vok = (vers_cmp($ver,$xcoding_ver) > -1);
            my $error = lib_lang::get_interpolated_message('check_platform_version_fail', $origver);
            my $pass = lib_lang::get_interpolated_message('check_platform_version_ok', $origver);
            $result = ((($vok) ? $pass : $error));
            if ($vok) {
                $error_status = $ERROR_CSS_CLASS;
            }

        } elsif ($resp =~ /(built on .*?) with/) {
            my $built = $1;
            $result = lib_lang::get_interpolated_message('check_platform_version_error', $built);
            $datos_xcoding =~ s/%%result_xcoding_ver%%/$result/g;
        } else {
            $result = lib_lang::get_interpolated_message('check_platform_version_error', $ver);
        }

        $datos_xcoding =~ s/%%result_comparar_version%%/$result/g;

        # Se comprueba soporte para libx264
        my $libx264_support = lib_lang::get_message('check_platform_libx264_support');
        $result = sprintf("%28s %-12s ", $libx264_support, '');
        if($resp =~ /--enable-libx264/) {
            $result .= lib_lang::get_message('check_platform_ok');
        } else {
            my $resp = `$path_ffmpeg -codecs 2> /dev/null | grep x264`;
            if($resp ne '') {
                $result .= lib_lang::get_message('check_platform_ok');
            } else {
                $result = lib_lang::get_interpolated_message('check_platform_disabled', $result);
            }
        }
        $datos_xcoding =~ s/%%result_soporte_libx264%%/$result/g;

        # Se comprueba soporte para aac
        my $aac_support = lib_lang::get_message('check_platform_aac_support');
        $result = sprintf(" %28s %-12s ", $aac_support, '');
        my $resp = `$path_ffmpeg -codecs 2> /dev/null | grep " aac "`;
        if ($resp =~ /\(encoders:.*?aac.*?\)/ || $resp =~ /DEA.*?aac/) {
            $result .= lib_lang::get_message('check_platform_ok');
        } else {
            $result = lib_lang::get_interpolated_message('check_platform_disabled', $result);
        }
        $datos_xcoding =~ s/%%result_soporte_aac%%/$result/g;

        # Se comprueba soporte para libfdk_aac
        my $libfdk_aac_support = lib_lang::get_message('check_platform_libfdk_aac_support');
        $result = sprintf("%28s %-11s ", $libfdk_aac_support, '');
        if ($resp =~ /--enable-libfdk-aac/) {
            $result .= lib_lang::get_message('check_platform_ok');
        } else {
            $resp = `$path_ffmpeg -codecs 2> /dev/null | grep libfdk_aac`;
            if($resp ne '') {
                $result .= lib_lang::get_message('check_platform_ok');
            } else {
                $result = lib_lang::get_interpolated_message('check_platform_disabled', $result);
            }
        }
        $datos_xcoding =~ s/%%result_soporte_libfdk_aac%%/$result/g;

        # Se comprueba soporte para libmp3lame
        my $libmp3lame_support = lib_lang::get_message('check_platform_libmp3lame_support');
        $result = sprintf("%28s %-11s ", $libmp3lame_support, '');
        $resp = `$path_ffmpeg -codecs 2> /dev/null | grep libmp3lame`;
        if($resp ne '') {
            $result .= lib_lang::get_message('check_platform_ok');
        } else {
            $result = lib_lang::get_interpolated_message('check_platform_disabled', $result);
        }
        $datos_xcoding =~ s/%%result_soporte_libmp3lame%%/$result/g;

        $datos_xcoding =~ s/%%ffmpeg_no_legible%%//g;
    } else {
        $result = lib_lang::get_message('check_platform_ffmpeg_unreadable');
        $datos_xcoding =~ s/%%ffmpeg_no_legible%%/$result/g;
    }

    return $datos_xcoding;
}

sub check_perl {
    my ($pagina, $web) = @_;

    # S.O.
    my $os = uc $^O; # solo esta en algunas plataformas
    $pagina =~ s/%%sistema_operativo%%/$os/g;

    # Perl
    my $perlv = $];
    my $min_perl = 5.020002;
    if ($perlv < $min_perl) {
        my $error_perl = lib_lang::get_interpolated_message('check_platform_perl_fail', $min_perl, $perlv);
        $pagina =~ s/%%error_perl%%/$error_perl/g;
        $pagina =~ s/%%error_status_perl%%/$ERROR_CSS_CLASS/g;
        $pagina =~ s/%%.*%%//gs;
        &print_response($pagina, $web);
    }

    $perlv =~ /(\d+).(\d{3})(\d{3})/;
    my ($version, $subversion, $revision) =  ($1, $2, $3);
    $version =~ s/^0+//;
    $subversion =~ s/^0+//;
    $revision =~ s/^0+//;

    $perlv = lib_lang::get_interpolated_message("check_platform_perl_ok", "$version.$subversion.$revision");

    $pagina =~ s/%%perlv%%/$perlv/g;
    $pagina =~ s/%%error_perl%%//g;
    $pagina =~ s/%%error_status_perl%%//g;

    return $pagina;
}

sub check_modules {
    my ($web, $pagina) = @_;

    my $result;
    my $error_status;
    my ($loop, $looptmp, $looptot);
    $pagina =~ /%%loop_modules%%(.*)%%\/loop_modules%%/g;
    $loop = $1;
    foreach my $module (@{$MODULES}) {
        $looptmp = $loop;

        ($result, $error_status) = &have_vers($web, $module->{name}, $module->{version}, $module->{from});
        $looptmp =~ s/%%module_info%%/$result/g;
        $looptmp =~ s/%%error_status_module%%/$error_status/g;
        $looptot .= $looptmp;
    }
    $pagina =~ s/%%loop_modules%%.*%%\/loop_modules%%/$looptot/gs;

    return $pagina;
}

sub check_unzip {
    my ($web, $pagina) = @_;

    my $resp = `unzip`;
    my $unzip_status;
    my $error_status;
    if ($resp =~ /(unzip) (\d+\.\d+)(.*?)[\n\r]/is) {
        my $ver = $2;
        my $vok = (&vers_cmp($ver,'1.00') > -1);
        if ($vok) {
            $unzip_status = lib_lang::get_interpolated_message('check_platform_version_ok', $ver);
        } else {
            $unzip_status = lib_lang::get_interpolated_message('check_platform_version_fail', $ver);
            $error_status = $ERROR_CSS_CLASS;
        }
    } else {
        $unzip_status = lib_lang::get_message('check_platform_unzip_missing');
        $error_status = $ERROR_CSS_CLASS;
    }

    $pagina =~ s/%%unzip_status%%/$unzip_status/g;
    $pagina =~ s/%%error_status_unzip%%/$error_status/g;

    return $pagina;
}

sub check_ffmpeg {
    my ($web, $pagina) = @_;
    # chequeo de xcoding
    my @paths_ffmpeg;

    # Se busca en las rutas especificadas en la variable de entorno PATH.
    my $str = 'echo $PATH';
    my $ret_path = `$str`;
    $ret_path =~ s/[\n\t\s\r]//sg;

    if ($ret_path) {
        @paths_ffmpeg = split(/:/, $ret_path);
    } else {
        push @paths_ffmpeg, '/usr/local/bin';
        push @paths_ffmpeg, '/usr/bin';
    }

    # ffmpeg
    my $found_path_ffmpeg = 0;
    my $results_check_xcoding = '';
    foreach my $path_ffmpeg (@paths_ffmpeg) {
        if (-f "$path_ffmpeg/ffmpeg") {
            $found_path_ffmpeg = 1;
            $results_check_xcoding .= &check_xcoding($web, "$path_ffmpeg/ffmpeg");
            $results_check_xcoding .= "\n" if !$web;
        }
    }
    $pagina =~ s/%%results_check_xcoding%%/$results_check_xcoding/g;

    my $error_ffmpeg;
    my $error_status;
    if (!$found_path_ffmpeg) {
        my $paths = join(', ', @paths_ffmpeg);
        $error_ffmpeg = lib_lang::get_interpolated_message("check_platform_ffmpeg_missing", $paths);
        $error_status = $ERROR_CSS_CLASS;
    } else {
        $error_ffmpeg = lib_lang::get_message('check_platform_ffmpeg_ok');
        $error_status = '';
    }

    $pagina =~ s/%%error_ffmpeg%%/$error_ffmpeg/g;
    $pagina =~ s/%%error_status_ffmpeg%%/$error_status/g;

    return $pagina;
}

# Chequear escritura de archivos.
#
# - Emular la escritura de un archivo con: html, xml, adjuntos, fotos y mm (todo)
# - Emular art relac
# - Emular tax
# - Emular escritura de portada
# - Escribir en carpeta temp de la cgi
# - Escribir en cpan/data
#
# Crear carpetas si esto no existe.

# Site de articulos
sub check_write {
    my $dirfecha = &glib_hrfec_03::get_date_pack4();
    my $ts = &glib_hrfec_03::get_dtime_pack4();
    my $reldir_pags = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/pags";
    my $reldir_img = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/imag";
    my $reldir_mmedia = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dirfecha$prontus_varglb::DIR_MMEDIA";
    my $reldir_asocfile = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ASOCFILE/$dirfecha/$ts";
    my $reldir_xml = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/xml";

    my $plantilla = lib_utils::load_cpan_core_plt("core/check_write.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    my ($result, $falla);

    ($result, $falla) = &write_file_test($reldir_pags);
    $plantilla =~ s/%%result_reldir_pags%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_img);
    $plantilla =~ s/%%result_reldir_img%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_mmedia);
    $plantilla =~ s/%%result_reldir_media%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_asocfile);
    $plantilla =~ s/%%result_reldir_asocfile%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_xml);
    $plantilla =~ s/%%result_reldir_xml%%/$result/g;

    # art. relac
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXONOMIA");
    $plantilla =~ s/%%result_art_relac%%/$result/g;

    # cpan/data y derivados
    ($result, $falla) = &write_file_test($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'});
    $plantilla =~ s/%%result_dbm%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log");
    $plantilla =~ s/%%result_cpan_log%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/search");
    $plantilla =~ s/%%result_dbm_search%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/user_lock");
    $plantilla =~ s/%%result_dbm_user_lock%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/users");
    $plantilla =~ s/%%result_dbm_users%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs");
    $plantilla =~ s/%%result_dbm_procs%%/$result/g;

    # site ediciones
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC");
    $plantilla =~ s/%%result_dir_edic%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base");
    $plantilla =~ s/%%result_dir_edic_base%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/port");
    $plantilla =~ s/%%result_dir_edic_base_port%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/rss");
    $plantilla =~ s/%%result_dir_edic_base_rss%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/xml");
    $plantilla =~ s/%%result_dir_edic_base_xml%%/$result/g;

    # port. tipo tema
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT");
    $plantilla =~ s/%%result_dir_edic_port_tema%%/$result/g;

    # dir temp
    my $current_cgi = $0;

    $current_cgi =~ s/(\w+\.cgi)$/prontus_temp/;
    $current_cgi =~ s/\\/\//;
    $current_cgi =~ s/$prontus_varglb::DIR_SERVER//;
    ($result, $falla) = &write_file_test($current_cgi);
    $plantilla =~ s/%%result_dir_pesoscero%%/$result/g;

    return $plantilla;
}

sub write_file_test {
    my $reldir = $_[0];
    my $result;
    my $falla = 0;

    if (!(&glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$reldir")) ) {
        $result = lib_lang::get_interpolated_message("check_install_missing_folder", $reldir);
        return $result;
    }

    my $file_test = "$prontus_varglb::DIR_SERVER$reldir/prueba.txt";
    &glib_fildir_03::write_file($file_test, 'test');

    $result = "[$reldir/prueba.txt] ... ";

    if ( (-f $file_test) && (-s $file_test) )  {
        $result .= lib_lang::get_message('check_install_ok');
        $result .= lib_lang::get_message('check_install_deleting');
        if (unlink $file_test) {
            $result .= lib_lang::get_message('check_install_ok');
        } else {
            $result .= lib_lang::get_message('check_install_fail');
            $falla = 1;
        }
    } else {
        $result .= lib_lang::get_message('check_install_fail');
        $falla = 1;
    }

    return ($result, $falla);
}

sub query_table {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_query_table.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);
    my $result;

    # Conectando con BD
    my $msg_db = lib_lang::get_interpolated_message('check_install_connect_db');
    $plantilla =~ s/%%encabezado%%/$msg_db $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    unless($base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}",
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'},
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) ) {
        &salir(lib_lang::get_interpolated_message('check_install_missing_db', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}, DBI->errstr));
    }

    my $sql = "select prueba_id, prueba_titular from prueba";
    my ($id, $nom);
    my $salida = &local_sql_bind($base, $sql, \($id, $nom)) || &salir(lib_lang::get_interpolated_message('check_install_query_error', $sql, DBI->errstr));
    my ($loop, $looptmp, $looptot);
    $plantilla =~ /%%loop_registros%%(.*)%%\/loop_registros%%/g;
    $loop = $1;

    while ($salida->fetch) {
        $looptmp = $loop;
        $looptmp =~ s/%%registro%%/prueba_id[$id] - prueba_titular[$nom]/g;
        $looptot .= $looptmp;
    }

    $salida->finish;
    $base->disconnect;
    $plantilla =~ s/%%loop_registros%%.*%%\/loop_registros%%/$looptot/gis;

    return $plantilla;
}

# ejecuta una sentencia sql y devuelve su handler.
# Param entrada:
#  0) descriptor de la base de datos abierta.
#  1) sentencia SQL propiamente tal.
#  2) nombres de las variables asociadas a los campos.
#  retorno:
#  0) descriptor de la respuesta. (Usar solamente fetch para cargar las variables con las filas siguientes.
sub local_sql_bind {
    my ($dbh) = shift;
    my ($sql) = shift;
    my (@var) = (@_);
    my ($sth);

    $sth = $dbh->prepare($sql) || return 0;

    $sth->execute || return 0;
    $sth->bind_columns(undef, (@var)) || return 0;

    return($sth);
}

sub escribir_diez_reg {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_escribir_diez_reg.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);
    # Conectando con BD
    my $msg_db = lib_lang::get_interpolated_message('check_install_connect_db');
    $plantilla =~ s/%%encabezado%%/$msg_db $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    unless($base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}",
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'},
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) ) {
        &salir(lib_lang::get_interpolated_message('check_install_missing_db', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}, DBI->errstr));
    }

    my $i;
    my ($loop, $looptmp, $looptot);
    $plantilla =~ /%%loop_registros%%(.*)%%\/loop_registros%%/g;
    $loop = $1;
    my $times = 10;
    for ($i = 1; $i <= $times; $i++) {
        $looptmp = $loop;
        my $sql = 'insert into prueba values(NULL, "TEXTO DEL REGISTRO")';
        $base->do($sql) || &salir(lib_lang::get_interpolated_message('check_install_insert_error', $i, $base->errstr));
        my $counter_ok = lib_lang::get_interpolated_message('check_install_insert_count', $i);
        $looptmp =~ s/%%registro%%/$counter_ok/g;
        $looptot .= $looptmp;
    }

    $base->disconnect;
    $plantilla =~ s/%%loop_registros%%.*%%\/loop_registros%%/$looptot/gis;
    my $insert_records = lib_lang::get_interpolated_message('check_install_insert_records', $times);
    $plantilla =~ s/%%resultado%%/$insert_records/g;

    return $plantilla;
}

# Prueba de escribir un registro en la tabla [prueba].
sub escribir_reg {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_insert.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    # Conectando con BD
    my $msg_db = lib_lang::get_interpolated_message('check_install_connect_db');
    $plantilla =~ s/%%encabezado%%/$msg_db $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    unless($base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}",
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'},
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) ) {
        &salir(lib_lang::get_interpolated_message('check_install_missing_db', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}, DBI->errstr));
    }

    my $sql;
    # Inserta registro en la tabla.
    my $msg = lib_lang::get_interpolated_message('check_install_insert_in_table');
    $plantilla =~ s/%%preliminar%%/$msg/g;
    $sql = 'insert into prueba values(NULL, "TEXTO DEL REGISTRO")';
    $base->do($sql) || &salir(lib_lang::get_interpolated_message('check_install_insert_single_record_error', $base->errstr));
    $base->disconnect;
    my $record_msg = lib_lang::get_message("check_install_insert_single_record");
    $plantilla =~ s/%%resultado%%/$record_msg/g;

    return $plantilla;
}

# Crear tabla de pruebas en la BD.
# Tabla: 'prueba'. Columnas: 'prueba_id' y 'prueba_titular'.
# Si no existe, se crea.
# Si ya existe, se borra la que estaba antes.
# Con MySQL si es MySQL, tomando los datos del cfg.
sub crear_tabla {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_create_table.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    # Conectando con BD
    my $msg_db = lib_lang::get_interpolated_message('check_install_connect_db');
    $plantilla =~ s/%%encabezado%%/$msg_db $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    unless($base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}",
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'},
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) ) {
        &salir(lib_lang::get_interpolated_message('check_install_missing_db', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}, DBI->errstr));
    }

    # Chequeando existencia
    my $sql;
    my $table_check = lib_lang::get_message('check_install_table_check');
    $plantilla =~ s/%%preliminar%%/$table_check/g;

    if (&lib_setbd::existe_tabla($base, 'prueba')) {
        my $table_exists = lib_lang::get_message('check_install_table_exists');
        $plantilla =~ s/%%existe_elimina%%/$table_exists/g;

        $sql = 'drop table prueba';
        $base->do($sql) || &salir(lib_lang::get_interpolated_message('check_install_delete_table_error', $base->errstr));
        my $table_deleted = lib_lang::get_message('check_install_table_deleted');
        $plantilla =~ s/%%existe_elimina_result%%/$table_deleted/g;
        $plantilla =~ s/%%check_existencia%%//g;
        $plantilla =~ s/%%\/check_existencia%%//g;
    } else {
        $plantilla =~ s/%%check_existencia%%.*%%\/check_existencia%%//gis;
    }

    # Crear tabla
    $plantilla =~ s/%%crear_tabla%%/Creando tabla 'prueba'.../g;
    $sql = "create table prueba (
            prueba_id int(5) not null auto_increment,
            prueba_titular varchar(128) not null default '',
            primary key  (prueba_id),
            key prueba_titular (prueba_titular))";

    $base->do($sql) || &salir(lib_lang::get_interpolated_message('check_install_create_table_error', $base->errstr));
    $base->disconnect;

    my $table_created = lib_lang::get_message('check_install_table_created');
    $plantilla =~ s/%%result_crear_tabla%%/$table_created/g;

    return $plantilla;
}

sub test_lock {
    my $lock_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}/prueba.lock";
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_insert_lock.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    my $preliminar = lib_lang::get_interpolated_message('check_install_locking', $lock_file);
    my ($status, $fh, $data) = lib_waitlock::lock_file($lock_file);
    if (!$status) {
        $preliminar .= lib_lang::get_message('check_install_locking_error'). lib_lang::get_message($fh). $data;
    } else {
        $preliminar .= lib_lang::get_message('check_install_locking_ok');
    }
    $plantilla =~ s/%%preliminar%%/$preliminar/g;

    my $write_result = &escribir_registro();
    $plantilla =~ s/%%resultado%%/$write_result/g;

    my $postliminar = lib_lang::get_interpolated_message('check_install_unlocking', $lock_file);
    my $result;
    ($status, $result, $data) = lib_waitlock::unlock_file($lock_file, $fh);
    if (!$status) {
        $postliminar .= lib_lang::get_message('check_install_unlocking_error') . lib_lang::get_message($result). $data;
    } else {
        $postliminar .= lib_lang::get_message('check_install_unlocking_ok');
    }

    $plantilla =~ s/%%postliminar%%/$postliminar/g;

    return $plantilla;
}

sub salir {
    lib_utils::print_html($_[0]);
}

sub print_response {
    my ($ambiente_web, $msg) = @_;
    if ($ambiente_web) {
        lib_utils::print_html($msg);
    } else {
        print $msg;
        exit;
    }
}

1;
