#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# funciones para manejo de portadas
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use prontus_varglb;
# use glib_fildir_03;
# use lib_waitlock; en actualizar_portadas_byartic
package lib_ports;

use strict;
use utf8;

# variables para la creacion de portadas
our %AREA = ();
our %ORDEN = ();
our %VB = ();
our %ART_PRONTUS = ();
# no ejecutar preprocesos
our $PREPROC_NO_BG = 0;

# Re-genera todas las portadas de la edicion correspondiente,
# donde el articulo referido este presente.
# Se considera la ed. base, la ultima y la vigente.
sub actualizar_portadas_byartic {
    my $ts     = $_[0];    # Nombre del articulo SIN extension y sin path, a dejar sin VoBo en las secciones donde aparezca.
    my $accion = $_[1];    # ALTA_CONTROL | BORRAR

    # procesa edicion, vigente, la ultima y la base.
    my @ediciones = &get_edics4update();

    foreach my $edic (@ediciones) {
        # lib_stdlog::print_err("edic[$edic]\n");
        # Directorio destino de las portadas de la edicion, en site, sino existe en site no busca el xml asociado
        my $path_site = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . "/$edic" . $prontus_varglb::DIR_PORT;

        my @entries = glib_fildir_03::lee_dir($path_site);

        # Para cada portada.
        foreach my $port (@entries) {
            next if (substr($port, 0, 1) eq '.');
            my $requiere_actualizar = 0;

            # portada en el site
            my $arch_seccion = "$path_site/$port";

            my $arch_xml = $arch_seccion;
            $arch_xml =~ s/\/port\/(\w+?)\.\w*$/\/xml\/$1\.xml/;
            # lib_stdlog::print_err("arch_xml[$arch_xml]\n");

            # template de la portada
            my $path_tsec = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_PORT . "/$port";
            next if ((!-f $arch_xml) || ($port =~ /^preview/));

            my $lock = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/$port";
            $lock =~ s/\.\w+$//;
            $lock = "$lock.mutex";

            my ($status, $fh, $data) = lib_waitlock::lock_file($lock);
            if (!$status) {
                lib_stdlog::print_err(lib_lang::get_message($fh). $data);
            }

            %AREA  = ();
            %ORDEN = ();
            %VB    = ();
            %ART_PRONTUS = ();

            my $buffer_port_xml = glib_fildir_03::read_file($arch_xml);
            my $port_pub = &get_port_pub($buffer_port_xml);
            # por defecto no_write desactivado.
            my $no_write = 0;
            # Si la portada no esta publicada se activa el no_write.
            $no_write = 1 if ($port_pub eq '0');

            # Rescatar la info de c/artic de la portada actual
            # Solo carga la info de portada si el articulo esta publicado en ella
            if ($buffer_port_xml =~ m|<file>$ts</file>|s) {
                my %rowartics = lib_ports::get_rowartics($buffer_port_xml);
                # articulos publicados en la portada
                foreach my $art (keys %rowartics) {
                    # Asocia area al articulo.
                    $AREA{$prontus_varglb::PRONTUS_ID}{$port}{$art} = $rowartics{$art}{'area'};
                    # Asocia prioridad correspondiente.
                    $ORDEN{$prontus_varglb::PRONTUS_ID}{$port}{$art} = $rowartics{$art}{'ord'};
                    # Asocia vobo del articulo en la portada.
                    $VB{$prontus_varglb::PRONTUS_ID}{$port}{$art}   = $rowartics{$art}{'vb'};
                    # prontus al que pertenece el articulo
                    $ART_PRONTUS{$prontus_varglb::PRONTUS_ID}{$port}{$art} = $rowartics{$art}{'prontus'};
                }
                if ($accion eq 'BORRAR') {
                    delete $AREA{$prontus_varglb::PRONTUS_ID}{$port}{$ts};
                    delete $ORDEN{$prontus_varglb::PRONTUS_ID}{$port}{$ts};
                    delete $VB{$prontus_varglb::PRONTUS_ID}{$port}{$ts};
                    delete $ART_PRONTUS{$prontus_varglb::PRONTUS_ID}{$port}{$ts};
                }
                # indico que hay que regenerar la portada
                $requiere_actualizar = 1;
            } # while articulo de esta portada

            # Re-Genera la portada si en ella esta el articulo en cuestion
            if ($requiere_actualizar) {
                my $mv            = '';
                my $sin_regen_xml = 0;
                my $ts_preview    = '';
                my $resp = &make_portada($arch_seccion, $path_tsec, $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID, $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}, $edic, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);

                # Ahora proceso multivistas
                $sin_regen_xml = 1;
                $resp = '';
                foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
                    $resp = &make_portada($arch_seccion, $path_tsec, $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID, $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}, $edic, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);
                }
            }

            my $result;
            ($status, $result, $data) = lib_waitlock::unlock_file($lock, $fh);
            if (!$status) {
                lib_stdlog::print_err(lib_lang::get_message($result). $data, "\n");
            }

            &exec_postproceso($prontus_varglb::PRONTUS_ID, $port, $edic);
        }    # foreach portadas de la edicion
    }    # foreach
}

# Carga un hash con los articulos publicados en las portadas
# y últimas ediciones
sub load_artic_pubs {
    my $prontus_id = $_[0];
    # procesa edicion, vigente, la ultima y la base.
    my (@ediciones) = &get_edics4update();
    my %hash_artics;

    my $pathdir_json = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/data/cache';
    my $path_json = "$pathdir_json/json_artics_in_ports.json";

    if (-f $path_json) {
        my $json_artics = glib_fildir_03::read_file_bin($path_json);
        my $ref_hash_artics;
        # decodifica datos de json
        eval { $ref_hash_artics = JSON::decode_json($json_artics); };
        if ($@) {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('global_json_file_decode_error', $@, $path_json, $json_artics), "\n");
            # si falla, el json es invalido y se borra para que se re-genere
            unlink($path_json);
        }
    }

    # Solo si la edicion es base, se guardan sólo las portadas base
    my %ports_base;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
        foreach my $port_base (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}) {
            $ports_base{$port_base} = 1;
            # lib_stdlog::print_err("Base: $port_base\n");
        }
    } else {
        %ports_base = %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}};
    }

    foreach my $edic (@ediciones) {
        # lib_stdlog::print_err("edic[$edic]\n");
        # Dir destino de las portadas de la ed.
        my $path_site = $prontus_varglb::DIR_SERVER .
                        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} .
                        $prontus_varglb::DIR_EDIC .
                        "/$edic" .
                        $prontus_varglb::DIR_PORT;

        # Para cada port configurada
        foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
                if ($edic eq 'base') {
                    # hay multiedicion y esta procesando edicion base, si portada no es base, la salta
                    next unless ($ports_base{$port});
                } else {
                    # hay multiedicion y esta procesando ediciones que no son la base, si portada es base, la salta
                    next if ($ports_base{$port});
                }
            }

            # portada en el site
            my $arch_seccion = "$path_site/$port";

            my $arch_xml = $arch_seccion;
            $arch_xml =~ s/\/port\/(\w+?)\.\w*$/\/xml\/$1\.xml/;

            next if ((!-f $arch_xml) || ($port =~ /^preview/));

            my $buffer_port_xml = glib_fildir_03::read_file($arch_xml);
            my %rowartics = lib_ports::get_rowartics($buffer_port_xml);

            # Rescatar la info de c/artic de la port actual
            foreach my $art (keys %rowartics) {
                $hash_artics{$rowartics{$art}{'prontus'}}{$art}{$edic}{$port} = 1;
            }
        }
    }

    my $ref_hash_artics = \%hash_artics;
    my $json_artics = JSON::encode_json($ref_hash_artics);
    utf8::decode($json_artics);

    glib_fildir_03::check_dir($pathdir_json);
    glib_fildir_03::write_file($path_json, $json_artics);

    return %hash_artics;
}

# Borra el cache de artículos publicados en portadas para que se refresque automáticamente.
sub borrar_json_artic_pubs {
    my $prontus_id = $_[0];
    my $pathdir_json = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_id}{'DIR_CPAN'} . '/data/cache';
    my $path_json = "$pathdir_json/json_artics_in_ports.json";

    if (-f $path_json) {
        unlink $path_json;
    }
}

# Obtiene un arreglo con las ediciones cuyas portadas es necesario actualizar por haberse modificado o eliminado
# articulos publicados en ellas.
# Estas ediciones corresponden a:
# - ed. base
# - ed. vigente
# - ultima edicion
# - la de hoy
sub get_edics4update {
    my (@ediciones);

    # Siempre procesa la ed. base.
    my $edic_base = $prontus_varglb::DIR_BASEEDIC;
    $edic_base =~ s/^\///;
    push @ediciones, $edic_base;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {

        # Obtener ed. vigente.
        my $ed_vigente = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}/ed_vigente.txt");
        if ($ed_vigente) {
            push @ediciones, $ed_vigente;
        }

        # Obtener ultima edicion
        my $ed_newer = &get_newer_edic();
        if (($ed_newer) && ($ed_newer ne $ed_vigente)) {
            push @ediciones, $ed_newer;
        }

        # Obtener la de hoy
        my $edic_hoy;    # aaaammdd
        my $fecha_hoy = glib_hrfec_03::get_date_pack4();
        $edic_hoy = substr($fecha_hoy, 0, 4) . '_' . substr($fecha_hoy, 4, 2) . '_' . substr($fecha_hoy, 6, 2) . '_1';
        if (-d "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/$edic_hoy") {
            push @ediciones, $edic_hoy if (($edic_hoy ne $ed_newer) && ($edic_hoy ne $ed_vigente));
        }
    }

    return @ediciones;
}

# Lee el xml de la portada y obtiene el campo port_pub
# 1 = portada publicada
# 0 = portada no publicada, no debiese existir html en site.
#
sub get_port_pub {
    my $buffer_port_xml = $_[0];
    # si el campo no esta se entrega por defecto que si esta publicado.
    my $port_pub = 1;

    if ($buffer_port_xml =~ /<port_pub>(\d+)<\/port_pub>/) {
        $port_pub = $1;
    }

    return $port_pub;
}

# Genera la portada de acuerdo al template escogido.
# $dest_file: full path de la portada a generar, incluyendo extension
# $path_tpl: full path de la plantilla de portada, incluyendo extension
sub make_portada {
    my ($dest_file, $path_tpl, $dir_server, $prontus_id, $mv, $public_server_name, $nom_edic, $sin_regen_xml, $control_fecha, $ts_preview, $no_write) = @_;

    # lib_stdlog::print_err("make_portada: path_tpl[$path_tpl] mv[$mv] dst[$dest_file]\n");

    my %plts4port;
    my %port_nom_clones;
    my %port_ext_clones;

    # Agregar la portada principal tambien
    $plts4port{$path_tpl} = $dest_file;

    $path_tpl =~ /(.*)\/([^\/]+\.\w+)$/;
    my $dir_tpl = $1;
    my $nomport = $2;
    # lib_stdlog::print_err("nomport[$nomport]\n");
    # Agregar las plantillas paralelas
    if (exists($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport})) {
        # contiene las paralelas en un arreglo
        my $extra_ports = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport}{'plt_paralela'};
        foreach my $extra_port (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport}{'plt_paralela'}}) {
            next if (!-f "$dir_tpl/$extra_port");
            my $my_dest_file = $dest_file;
            $my_dest_file =~ s|/$nomport$|/$extra_port|;
            $plts4port{"$dir_tpl/$extra_port"} = $my_dest_file;

            my ($nom_clon, $ext) = lib_utils::split_nom_y_extension($extra_port);
            $port_nom_clones{"$dir_tpl/$extra_port"} = $nom_clon;
            $port_ext_clones{"$dir_tpl/$extra_port"} = $ext;
        }
    }

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my $port_seo_file = "$port_seo_dir/$nomport";
    $port_seo_file =~ s/\.\w+$/.json/;

    my %seo_data;
    if (-f $port_seo_file) {
        my $jsonSeoBuffer = glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %seo_data = %{$jsonSeoRef};
    }

    # plantilla para los links a previews.
    my $previews_tpl = '';
    if (index($dest_file, 'preview') > -1) {
        my $previews_plt_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/core/links_previews.html";
        $previews_tpl = glib_fildir_03::read_file($previews_plt_path);
        $previews_tpl = lib_utils::parser_language_function($previews_tpl);
    }

    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    # Parsear la portada principal y sus clones
    foreach my $path_tpl_clon (keys %plts4port) {
        my ($no_procesar, $preproc) = &check_port_no_procesar($path_tpl_clon, $mv);
        # lib_stdlog::print_err("no_procesar[$no_procesar] preproc[$preproc]\n");

        if ($no_procesar == 1) {
            # lib_stdlog::print_err("No se parsea plantilla, no procesar activo!\n");
            $preproc =~ s/[#;&<>|~]+//g;

            if ($preproc ne '') {
                # Se ejecuta el pre-proceso. Esperar que termine o se ejecuta en segudno plano ?
                my $dest_xml = $dest_file;
                $dest_xml =~ s|/port/(\w+)(\.\w+)?$|/xml/$1\.xml|;

                # Incluye path completo hacia el xml y path completo hacia la plantilla que se ignoró.
                my $cmd = "$rutaScript/$preproc $dest_xml $path_tpl_clon >/dev/null 2>&1"; # bg.

                if (!$PREPROC_NO_BG) {
                    $cmd .= " &";
                }

                lib_stdlog::print_err("PRE PROC cmd[$cmd]\n");
                system($cmd);
            }

            next;
        }

        # No se procesa la portada y no se escribe el html.
        if ($no_write == 1) {
            my $dest_file_clon = $plts4port{$path_tpl_clon};
            $dest_file_clon =~ s|/port/|/port-$mv/| if ($mv);

            lib_stdlog::print_err("no_write[$no_write] -> [$dest_file_clon] \n");

            unlink $dest_file_clon if (-e $dest_file_clon);
            next;
        }

        my $dest_file_clon = $plts4port{$path_tpl_clon};
        # lib_stdlog::print_err("path_tpl_clon[$path_tpl_clon]\n");
        # Carga plantilla y realiza parseos normales
        my $buffer = &generic_parse_port($path_tpl_clon, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, $mv, \%seo_data, $nomport);
        # lib_stdlog::print_err("buffer[$buffer]\n");

        # Escribe portada html
        $dest_file_clon =~ s|/port/|/port-$mv/| if ($mv);
        my $destdir = $dest_file_clon;
        $destdir =~ s|/[\w\.]+$||;
        glib_fildir_03::check_dir($destdir);
        my $nom_clon = $port_nom_clones{$path_tpl_clon};
        my $ext_clon = $port_ext_clones{$path_tpl_clon};
        # Si es un clon el que se esta previsualizando, le pongo preview_<nomclon>_<port>.<ext>
        if (($dest_file_clon =~ m|/port(-$mv)?/preview_|) && ($path_tpl ne $path_tpl_clon)) {
            $dest_file_clon =~ s|/port(-$mv)?/preview_|/port$1/preview_$nom_clon\_|;
        }

        # En los preview, poner links a los demas previews
        my $cual_viendo = $path_tpl_clon;
        $cual_viendo =~ /([^\/\\]+\.\w+)$/;
        $cual_viendo = $1;

        $previews_tpl =~ s/%%cual_viendo%%/$cual_viendo/;
        $previews_tpl =~ m|%%loop_previews%%(.*?)%%/loop_previews%|s;
        my $loop_previews = $1;

        if ((%port_nom_clones) && ($dest_file_clon =~ m|/port/preview|)) {
            my $links_previews;
            my $paralela = '';

            foreach my $k (keys %plts4port) {
                my $tmp_preview = $loop_previews;

                my $k_dst = $plts4port{$k};
                if ($path_tpl ne $k) {
                    my $nom_k = $port_nom_clones{$k};
                    $k_dst =~ s|/port/preview_|/port/preview_$nom_k\_|;
                    $paralela = 'paralela';
                }
                $k =~ /([^\/\\]+\.\w+)$/;
                $k = $1;
                $k_dst =~ /([^\/\\]+\.\w+)$/;
                $k_dst = $1;

                $tmp_preview =~ s/%%k_dst%%/$k_dst/g;
                $tmp_preview =~ s/%%paralela%%/$paralela/g;
                $tmp_preview =~ s/%%k%%/$k/g;
                $links_previews = $tmp_preview . $links_previews;
            }

            $previews_tpl =~ s/%%loop_previews%%.*%%\/loop_previews%%/$links_previews/gs;

            if ($buffer =~ /(<body[^>]*?>)/) {
                my $first = $1;
                if ($first =~ /</) {
                    $buffer =~ s/(<div)/$previews_tpl$1/;
                } else {
                    $buffer =~ s/(<body[^>]*?>)/$1$previews_tpl/;
                }
            } else {
                $buffer = $previews_tpl . $buffer;
            }
        }

        $previews_tpl =~ s|%%loop_previews%%.*%%/loop_previews%%||gs;

        $buffer =~ s/%%_vista%%/$mv/g;
        $buffer =~ s/%%_mv%%/$mv/g;
        $buffer =~ s/%%.+?%%//g;
        glib_fildir_03::write_file($dest_file_clon, $buffer);
        lib_prontus::purge_cache($dest_file_clon);
    }

    # Escribe xml de la portada ppal.
    if (!$sin_regen_xml) {
    # eventual respuesta por problemas
        my $resp = &write_xml_port($dest_file, &generar_rows_xml_port($control_fecha, $nomport), $no_write);
        if ($resp) {
            # si hubo problemas, termino acá.
            return $resp;
        }
    }

    # -------------------------------------------------
    # Genera xml para RSS, para todas las vistas
    # -------------------------------------------------

    # Lee archivo de template rss.
    # Se llama igual a la portada pero esta en vez de /port en /rss y tiene extension .xml
    $path_tpl =~ s|/port(\-\w+)?/(\w+)\.\w+?$|/rss$1/$2\.xml|;

    if ((-s $path_tpl) && (-f $path_tpl)) {
        my $buffer = &generic_parse_port($path_tpl, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, '', \%seo_data, $nomport);

        # Parseos especificos
        $buffer =~ s/%%_vista%%/$mv/g;
        $buffer =~ s/%%_mv%%/$mv/g;

        # Escribe RSS.
        $buffer =~ s/%%.+?%%//g;
        &write_rss_port($dest_file, $nom_edic, $buffer);
    }

    # Borra cache de no publicados
    &borrar_json_artic_pubs($prontus_id);

    return '';
}

# Obtiene nombre de la edic mas nueva
sub get_newer_edic {
    my ($dir) = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC;

    my @entries = glib_fildir_03::lee_dir($dir);
    @entries = sort { $b cmp $a } (@entries);
    foreach my $entry (@entries) {
        if (($entry ne 'base') && (-d "$dir/$entry") && ($entry =~ /^[0-9_]+$/)) {
            return $entry;
        }
    }
    return '';
}

sub check_port_no_procesar {
    my $path_tpl = $_[0];
    my $mv = $_[1];
    my $dir_macros = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/plantillas/edic/nroedic/macros";

    my $buffer = lib_prontus::carga_buffer_plt($path_tpl, $dir_macros, $mv);
    my $pproc;
    my $no_procesar = 0;

    # lib_stdlog::print_err("check_port_no_procesar: mv[$mv] path_tpl[$path_tpl]\n");

    if ($buffer =~ /<!--\s*NO_PROCESAR\s*-->/g) {
        $no_procesar = 1;
        if ($buffer =~ /<!--\s*PRE_PROCESO\s*=\s*['|"]?(.+?)['|"]?\s*-->/sg) {
            $pproc = $1;
         }
    }

    return ($no_procesar, $pproc);
}

# carga plantilla de portada y realiza parseos genericos, comunes a port normal y a rss
sub generic_parse_port {
    my ($path_tpl, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, $mv, $seo_ref, $nomport) = @_;
    my %seo_data = %{$seo_ref};

    my $dir_macros = "$dir_server/$prontus_id/plantillas/edic/nroedic/macros";
    my $buffer = lib_prontus::carga_buffer_plt($path_tpl, $dir_macros, $mv);

    my $doblegato_string = glib_str_03::random_string(30);
    $buffer =~ s/##/$doblegato_string/g;

    # Obtener nombre y extensión de la plantilla.
    my @paths = split('/', $path_tpl);
    my $plt_file = $paths[$#paths];
    my ($nom_plt, $ext_plt) = lib_utils::split_nom_y_extension($paths[$#paths]);
    lib_stdlog::print_err("[$plt_file] [$path_tpl]\n");

    $buffer =~ s/%%_plt_nom%%/$nom_plt/g;
    $buffer =~ s/%%_plt_ext%%/$ext_plt/g;
    $buffer =~ s/%%_mv%%/$mv/g;

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_id}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my $port_seo_file = "$port_seo_dir/$nom_plt.json";

    if (-f $port_seo_file) {
        my $jsonSeoBuffer = glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %seo_data = %{$jsonSeoRef};
    }

    # Marcas globales
    $buffer = lib_parser::parse_globals($buffer, $nom_edic);

    # Encuentra los tags de area dentro del template (%%LOOPi%%...%%/LOOP%%),
    # y produce los contenidos.
    my %areas;
    my %area_cont;
    my %area_check;

    while ($buffer =~ /%%loop(\d+)(\([^)]+?\))?%%(.*?)%%\/loop%%/sg) {
        my ($are, $template) = ($1, $3);
        my $pure_are = $are;
        if (!exists $area_cont{$pure_are}) {
            $area_cont{$pure_are} = 1;
            $area_check{$pure_are} = 1;
        }
        if (exists $areas{$are}) {
            $are .= '_' . sprintf("%04d", $area_cont{$pure_are});
            $area_cont{$pure_are}++;
            $area_check{$are} = 1;
        }
        # Parsea el area usando $2 como template parcial.
        $areas{$are} = &parser_area($pure_are, $template, $prontus_id, $control_fecha, $ts_preview, $doblegato_string, $mv, \%seo_data, $nomport);
    }

    # Sustituye las areas parseadas.
    foreach my $key (sort { $a cmp $b } (keys %areas)) {
        my $aux = $areas{$key};
        $key    =~ s/_\d+$//;
        $buffer =~ s/%%loop$key(\([^)]+?\))?%%(.*?)%%\/loop%%/$aux/s;
    }

    # Parseo de nifloop.
    foreach my $key (keys %area_check) {
        if ($areas{$key}) { # el area no esta vacia.
            $key =~ s/_\d+$//;
            $buffer =~ s/%%nifloop$key%%(.*?)%%\/nifloop%%//s;
        }
    }

    # parsea valores de seo
    if (scalar(keys %seo_data)) {
        foreach my $seoKey (keys %seo_data) {
            $buffer =~ s/%%$seoKey%%/$seo_data{$seoKey}/sg;
            $buffer = lib_prontus::parse_tag_prontus($seo_data{$seoKey}, $seoKey, $buffer);
        }

        # Procesar IFs y NIFs fuera de los loops de portada
        $buffer = lib_parser::procesa_condicional($buffer, \%seo_data);
    }

    # Borra todos los tags IFV que quedaron en la pagina.
    $buffer =~ s/%%ifvc?\(\d+\, *\d+\)%%//sg;
    $buffer =~ s/%%\/ifvc?%%//g;

    # Borra todos los tags NIFV que quedaron en la pagina.
    $buffer =~ s/%%nifv\(\d+\, *\d+\)%%//sg;
    $buffer =~ s/%%\/nifv%%//g;

    $buffer =~ s/$doblegato_string/##/g;

    $buffer = lib_prontus::parser_custom_function($buffer);

    $buffer = lib_prontus::parse_includes($buffer);

    $buffer =~ s/%%[^_].+?%%//g;
    $buffer =~ s/<!--POST_PROCESO=.+?-->//g;

    return $buffer;
}

# Verifica si la ed. consultada es o no la vigente, revisando si el nro
# de esta es el q figura en el archivo ed_vigente.txt
sub verifica_ed_vigente {
    my $nombre_ed = $_[0];
    my $pag = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}/ed_vigente.txt");

    if ($pag =~ /$nombre_ed/) {
        return 'SI';
    } else {
        return 'NO';
    }
}

sub generar_rows_xml_port {
    my ($control_fecha,$nomport) = @_;
    my $current_prontus_id = $prontus_varglb::PRONTUS_ID;

    # Ordenar ascendentemente por area / orden / id articulo.
    my @arr_art = sort {$AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$a} <=> $AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$b}
                        || $ORDEN{$prontus_varglb::PRONTUS_ID}{$nomport}{$a} <=> $ORDEN{$prontus_varglb::PRONTUS_ID}{$nomport}{$b}
                        || $b <=> $a}
                        (keys %{$AREA{$prontus_varglb::PRONTUS_ID}{$nomport}});

    my $msg_err_bd;

    my $rowartics_xml = '';
    # plantilla para los rowartics.
    my $rowartics_tpl = "<rowartic>\n".
    "    <prontus>%%prontus_id%%</prontus>\n".
    "    <file>%%art_ts%%</file>\n".
    "    <area>%%area%%</area>\n".
    "    <ord>%%ord%%</ord>\n".
    "    <vb>%%vb%%</vb>\n".
    "    <pub>%%pub%%</pub>\n</rowartic>\n";

    # antes de escribir el nuevo xml, saca una copia de respaldo.
    foreach my $art_ts (@arr_art) {
        my $aux = $rowartics_tpl;
        my $orden = $ORDEN{$prontus_varglb::PRONTUS_ID}{$nomport}{$art_ts};
        my $area  = $AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$art_ts};
        my $vb    = $VB{$prontus_varglb::PRONTUS_ID}{$nomport}{$art_ts};
        my $prontus   = $ART_PRONTUS{$prontus_varglb::PRONTUS_ID}{$nomport}{$art_ts};

        # print STDERR "generar_rows_xml_port [$art_ts] -> [$prontus]\n";

        if (!ref $prontus_varglb::BD_CONN{$prontus}) {
            $prontus_varglb::PRONTUS_ID = $prontus;
            ($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd) = lib_bd::conectar_prontus_bd();

            if (!ref($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID})) {
                print STDERR "ERROR: $msg_err_bd\n";
                return '';
            }
            $prontus_varglb::PRONTUS_ID = $current_prontus_id;
        }

        my $pub = lib_prontus::get_status_pub($art_ts, $control_fecha, $prontus_varglb::BD_CONN{$prontus});

        # print STDERR "ts[$art_ts] pub[$pub] prontus[$prontus] area[$area]\n";

        next if (!$area);
        $orden = 1 if (!$orden);

        $aux =~ s/%%art_ts%%/$art_ts/g;
        $aux =~ s/%%area%%/$area/g;
        $aux =~ s/%%ord%%/$orden/g;
        $aux =~ s/%%vb%%/$vb/g;
        $aux =~ s/%%pub%%/$pub/g;
        $aux =~ s/%%prontus_id%%/$prontus/g;
        $rowartics_xml .= $aux;
    }

    $prontus_varglb::PRONTUS_ID = $current_prontus_id;

    return $rowartics_xml;
}

# Escribe el xml de portada prontus
sub write_xml_port {
    my ($dest_xml, $rowartics_xml, $no_write) = @_;

    if ($rowartics_xml eq '') {
        lib_stdlog::print_err(lib_lang::get_message('ports_error_row_artics_empty'));
    }

    # Validar quota antes de trabajar
    my $msg_err_quota = lib_quota::check_quota_suficiente();
    if ($msg_err_quota) {
        return "Error: $msg_err_quota";
    }
    my $port_pub = 1;

    if ($no_write) {
        $port_pub = 0;
    } else {
        $port_pub = 1;
    }

    if ($dest_xml !~ /\.xml$/) {
        $dest_xml =~ s/\/port\/(\w+)(\.\w+)?$/\/xml\/$1\.xml/;
    }

    if ($rowartics_xml eq '') {
        return "Error al guardar portada";
    }

    # plantilla para los xml.
    my $new_xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<port_data>\n$rowartics_xml<port_pub>$port_pub</port_pub>\n</port_data>\n";

    # antes de escribir el nuevo xml, saca una copia de respaldo.
    my $md5_newXml = Digest::MD5::md5_hex($new_xml);

    my $dobackup = 0;
    my ($nomxml, $dirxml) = ('', '');
    if ($dest_xml =~ /^(.*\/xml)\/(.*\.xml)$/) {
        $dirxml = $1;
        $nomxml = $2;
        glib_fildir_03::check_dir("$dirxml/bak");

        my $curXml     = glib_fildir_03::read_file($dest_xml);
        my $md5_curXml = Digest::MD5::md5_hex($curXml);
        if ($md5_newXml eq $md5_curXml) {
            # portada nueva y antigua son iguales, no hace falta sobre escribir el archivo
            return '';
        } else {
            $dobackup = 1;
        }

        # el respaldo se realiza sólo si los xml son distintos, es decir, sólo si lo que se guarda es distinto a lo guardado
        if ($dobackup) {
            unlink "$dirxml/bak/$nomxml.9" if (-f "$dirxml/bak/$nomxml.9");
            for (my $i = 9 ; $i > 0 ; $i--) {
                File::Copy::move("$dirxml/bak/$nomxml." . ($i - 1), "$dirxml/bak/$nomxml." . ($i));
            }
            File::Copy::move("$dirxml/bak/$nomxml", "$dirxml/bak/$nomxml.0");
        }
    }
    # escribe el nuevo xml
    glib_fildir_03::write_file($dest_xml, $new_xml);
    File::Copy::copy($dest_xml, "$dirxml/bak/$nomxml") if ($dobackup);
    return '';
}

# Escribe el xml del rss de portada prontus
sub write_rss_port {
    my ($dest_rss, $nom_edic, $buffer) = @_;
    $dest_rss =~ s/\/port(\-\w+)?\/(\w+)\.\w+?$/\/rss$1\/$2\.xml/;    # Deduce del path completo de la portada, el del rss.

    # si es una edicion normal, igual se escribe en el dir de la edic base
    # ya que los rss deben estar en una ubicacion fija
    $dest_rss =~ s/$nom_edic/base/g;

    my $destdir_rss = $dest_rss;
    $destdir_rss =~ s/\/[\w\.]+$//;

    glib_fildir_03::check_dir($destdir_rss);
    glib_fildir_03::write_file($dest_rss, $buffer);
    lib_prontus::purge_cache($dest_rss);
}

# Genera un area de acuerdo al template suministrado.
# Retorna el buffer parseado y el nro. de articulos en el area
# %AREA = ();
# %ORDEN = ();
# %VB = ();
sub parser_area {
    my ($area, $theloop, $prontus_id, $control_fecha, $ts_preview, $doblegato_string, $mv, $seo_data, $plt_file) = @_;

    # Obtiene lista de articulos de esta area.
    my $num_artics_in_area = 0;
    my %articulos = ();    # Articulos que pertenecen a esta area.
    foreach my $key (sort { $b cmp $a } (keys %{$AREA{$prontus_id}{$plt_file}})) {
        if ($AREA{$prontus_id}{$plt_file}{$key} eq $area) {
            $articulos{$key} = $AREA{$prontus_id}{$plt_file}{$key};
        }
    }

    # Ordena los art. de esta area por prioridad y timestamp
    my @keys_articulos = sort { $ORDEN{$prontus_id}{$plt_file}{$a} <=> $ORDEN{$prontus_id}{$plt_file}{$b} or $b <=> $a } keys %articulos;

    my (@filtered_keys, $ultimo, %hash_ifvs, $salida_html);
    my $totartics = 0;

    # Aca se determinan los articulos que efectivamente serán publicados
    foreach my $ts_artic (@keys_articulos) {
        # Revisar VoBo de publicacion
        if ($VB{$prontus_id}{$plt_file}{$ts_artic} eq '0') {
            next;
        }

        my $prontus_art = $ART_PRONTUS{$prontus_id}{$plt_file}{$ts_artic};
        if (!defined($lib_prontus::ARTICS{$prontus_art}{$ts_artic})) {
            unless($lib_prontus::ARTICS{$prontus_art}{$ts_artic} = Artic->new(
                                'prontus_id'         => $prontus_art,
                                'public_server_name' => $prontus_varglb::CONFIG{$prontus_art}{'PUBLIC_SERVER_NAME'},
                                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_art}{'CPAN_SERVER_NAME'},
                                'document_root'      => $prontus_varglb::DIR_SERVER,
                                'ts'                 => $ts_artic,
                                'no_check_dirs'      => 1)) {
                lib_stdlog::print_err(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");
                next;
            }
        }

        my %campos_xml = $lib_prontus::ARTICS{$prontus_art}{$ts_artic}->get_xml_content();

        # Filtro por fechap/horap y fechae/horae del artic
        if ($control_fecha eq 'SI' &&
            !lib_utils::fechas_ok($ts_preview, $campos_xml{'_fechap'}, $campos_xml{'_horap'}, $campos_xml{'_fechae'}, $campos_xml{'_horae'})) {
            next;
        }

        # Filtro por alta
        my $alta_key = $campos_xml{'_alta'};
        if (!$alta_key) {    # sin alta
            next;
        }

        # Se guarda ts de articulo valido
        $filtered_keys[$totartics] = $ts_artic;
        $totartics++;
    }

    # Aca se asume que todos los artículos serán parseados
    my $stop_loop   = 0;
    my $loopcounter = 0;
    while (($loopcounter <= $#filtered_keys) && ($stop_loop < 1)) {
        my $ts_artic = $filtered_keys[$loopcounter];

        my $prontus_art = $ART_PRONTUS{$prontus_id}{$plt_file}{$ts_artic};
        if (!defined($lib_prontus::ARTICS{$prontus_art}{$ts_artic})) {
            unless($lib_prontus::ARTICS{$prontus_id}{$ts_artic} = Artic->new(
                                        'prontus_id'         => $prontus_art,
                                        'public_server_name' => $prontus_varglb::CONFIG{$prontus_art}{'PUBLIC_SERVER_NAME'},
                                        'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_art}{'CPAN_SERVER_NAME'},
                                        'document_root'      => $prontus_varglb::DIR_SERVER,
                                        'ts'                 => $ts_artic,
                                        'no_check_dirs'      => 1)) {
                lib_stdlog::print_err(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");
                next;
            }
        }

        my %campos_xml = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_xml_content();

        # agrega vista a datos de artículo
        $campos_xml{'_mv'} = $mv;

        $loopcounter++;
        # Copia sub-template para generar instancia.
        my $localbuf = $theloop;

        # Genera loop_artic
        $localbuf = lib_parser::procesa_loop_artic($localbuf, $doblegato_string);

        # ------------- IFVs.
        # Si el contador de articulos para este loop MOD div != res entonces borrar contenido
        my $buf_aux = $localbuf;
        my ($div, $res, $mod, $instancias);

        if (($buf_aux =~ /%%ifvc\((\d+)\, *(\d+)\)%%.+?%%\/ifvc%%/s) and ($buf_aux !~ /%%ifv\((\d+)\, *(\d+)\)%%.+?%%\/ifv%%/s)) {
            return lib_lang::get_message('ports_error_ifvc'), $loopcounter;
        }

        while ($buf_aux =~ /%%ifv(C?)\((\d+)\, *(\d+)\)%%.+?%%\/ifv\1%%/sg) {
            my $ifvc = $1;
            $div = $2;
            $res = $3;
            if (!$div) {
                $localbuf =~ s/%%(ifv$ifvc\($div\, *$res\))%%(.+?)%%\/(ifv$ifvc)%%/<h1>Error: división por 0<\/h1>/s;
                next;
            }
            $mod = $loopcounter % $div;
            # lib_stdlog::print_err("var[$loopcounter] % div[$div] = mod[$mod] y res[$res] y ifvc[$ifvc]\n"); # DEBUG
            # elimina los segmentos q no corresponden, dejando solo uno de acuerdo a la iteracion q toca.
            if ($mod == $res) {
                if ($ifvc) {
                    $ultimo = 'ifvc';
                } else {
                    $ultimo = 'ifv';
                }
                $hash_ifvs{$loopcounter} = $ultimo;

                $localbuf =~ s/%%(ifv$ifvc\($div\, *$res\))%%(.+?)%%\/(ifv$ifvc)%%/##$1##$2##\/$3##/s;
                last;
            }
            if (($ts_artic eq 'filler') && ($loopcounter == $div)) {
                $stop_loop = 1;
            }
        }  # while %%IFV...

        $localbuf =~ s/%%ifv(C?)\(\d+\, *\d+\)%%.+?%%\/ifv\1%%//sg;
        $localbuf =~ s/##/%%/g;

        # elimino los tags del if que quedo.
        $localbuf =~ s/%%ifvc?\(\d+\, *\d+\)%%//sg;
        $localbuf =~ s/%%\/ifvc?%%//sg;
        # ------------- /IFVs.

        # ------------- NIFVs.
        # Si el contador de articulos para este loop MOD div == res entonces borrar contenido
        $buf_aux = $localbuf;
        ($div, $res, $mod) = '';
        while ($buf_aux =~ /%%nifv\((\d+)\, *(\d+)\)%%.+?%%\/nifv%%/sg) {
            $div = $1;
            $res = $2;
            if (!$div) {
                $localbuf =~ s/%%nifv\($div\, *$res\)%%.+?%%\/nifv%%/<h1>Error: división por 0<\/h1>/s;
                next;
            }
            $mod = $loopcounter % $div;
            # print "var[$loopcounter] % div[$div] = mod[$mod] y res[$res]<br>"; # DEBUG

            if ($mod == $res) {
                $localbuf =~ s/%%nifv\($div\, *$res\)%%.+?%%\/nifv%%//sg;
            }
        }
        # elimino los tags del if que quedo.
        $localbuf =~ s/%%nifv\(\d+\, *\d+\)%%//sg;
        $localbuf =~ s/%%\/nifv%%//sg;
        # ------------- /NIFVs.

        # Parsea datos del artic
        if ($ts_artic eq 'filler') {
            my %nodata;
            my %fillerdata;
            $fillerdata{_ts} = $ts_artic;
            $localbuf = lib_parser::procesa_condicional($localbuf, \%fillerdata, \%nodata);
            $localbuf =~ s/%%.*?%%//sg;
        } elsif ($localbuf ne '') {
            # Procesa condicionales mas algunas variables adicionales
            my %claves_adicionales = (
                '_ts'          => $ts_artic,
                '_loopcounter' => $loopcounter,
                '_totartics'   => $totartics,
                '_area'        => $area
            );

            foreach my $seoKey (keys %{$seo_data}) {
                $claves_adicionales{_area} = $seo_data->{$seoKey};
            }

            $localbuf = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->parse_artic_data($localbuf, \%claves_adicionales, $mv);
        }
        $salida_html .= $localbuf;

        if ($loopcounter >= $totartics) {
            if ($hash_ifvs{$loopcounter} eq 'ifvc') {
                push @filtered_keys, 'filler';
                if ($loopcounter > 10000) {
                    $stop_loop = 1;
                }
            } else {
                $stop_loop = 1;
            }
        }
    }

    return $salida_html;
}   # parserArea

# Lee el xml de una portada prontus y retorna su contenido en un hash
# indexado por el articulo (<ts>.<ext>)
# Los rowartics son de la forma:
#   <rowartic>
#    <prontus>prontus_id</prontus>
#    <file>20070125195234</file>
#    <area>2</area>
#    <ord>2</ord>
#    <vb>1</vb>
#    <pub>0</pub>
#  </rowartic>
sub get_rowartics {
    my $buffer_port_xml = $_[0];
    my %rowartics;
    while ($buffer_port_xml =~ m|<rowartic>(.+?)</rowartic>|sg) {
        my $row = $1;
        my ($prontus, $dirfecha, $art, $area, $prio, $vb, $pub) = ('', '', '', '', '', '', '');

        if ($row =~ m|<file>(\d+)</file>|) {
            my $ts = $1;
            if ($row =~ m|<area>(\d+)</area>|) {
                $rowartics{$ts}{'area'} = $1;
            }
            if ($row =~ m|<ord>(\d+)</ord>|) {
                $rowartics{$ts}{'ord'} = $1;
            }
            if ($row =~ m|<vb>(\d*)</vb>|) {
                $rowartics{$ts}{'vb'} = $1;
            }
            if ($row =~ m|<pub>(\d*)</pub>|) {
                $rowartics{$ts}{'pub'} = $1;
            }
            if ($row =~ m|<prontus>(\w+)</prontus>|) {
                $rowartics{$ts}{'prontus'}  = $1;
            } else {
                $rowartics{$ts}{'prontus'} = $prontus_varglb::PRONTUS_ID;
            }
        }
    }

    return %rowartics;
}

# POST-PROCESO portadas
# Una vez escrito la portada, ubica el nombre del script de post-proceso (con extension y con path absoluto completo) y lo ejecuta.
# Script de postproceso es optativo y puede ir en cualquier parte de la portada
sub exec_postproceso {
    my ($prontus_id, $port, $edic) = @_;

    my $dir_template = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_id}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_PORT;
    my $dst_sec = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_id}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . "/$edic$prontus_varglb::DIR_PORT";

    # carga plantilla
    my $buffer = glib_fildir_03::read_file("$dir_template/$port");
    $buffer =~ s/\r/\n/sg;

    my $dir_macros = "$prontus_varglb::DIR_SERVER/$prontus_id/plantillas/edic/nroedic/macros";
    $buffer = lib_parser::add_macros($buffer, $dir_macros);

    my @postProcesos = ();
    while ($buffer =~ /<!--POST_PROCESO=(.+?)-->/sg) {
        push(@postProcesos, $1);
    }

    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    # Ejecuta en bground el proceso pasandole x param. el path completo a la portada.
    foreach my $pp (@postProcesos) {
        # para que sea un script valido debe ubicarse en el mismo dir. de cgi del prontus
        # quita ../, no se pueden usar directorios relativos hacia arriba
        $pp =~ s/\.\.\///g;
            $pp =~ s/[#;&<>|~]+//g;
        if ($pp =~ /^\w/) {
            my $cmd = "$rutaScript/$pp $dst_sec/$port $prontus_varglb::CONFIG{$prontus_id}{'PUBLIC_SERVER_NAME'} >/dev/null 2>&1 &";
            lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
            system $cmd;
        }
    }

    if ($prontus_varglb::CONFIG{$prontus_id}{'LIST_PORT_PPROC'} eq 'SI') {
        my $pathnice = lib_utils::get_path_nice();
        $pathnice = "$pathnice -n19 " if ($pathnice);
        my $cmd = "$pathnice $rutaScript/prontus_cron_list.pl $prontus_id >/dev/null 2>&1 &";
        lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
        system $cmd;
    }
}

1;
