#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# -------------------------------BEGIN LIBRERIA-------------
# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------
package lib_prontus;

use strict;
use utf8;

# Indica si hay Prontus Function disponibles
our $PF_ENABLED = 0;
eval {require lib_custom;};
if ($@) {
    $PF_ENABLED = 0;
} else {
    $PF_ENABLED = 1;
}

our $ZIP_AVAILABLE = 0;
our $DO_PURGE = 0;
our $DISABLE_PURGE_CACHE = 0;

# cache de artículos, para no leerlos de nuevo
our %ARTICS = ();
# cache de multitag para no cargar de nuevo
our %MULTITAG = ();
# cache de tags para no cargar de nuevo
our %TAGS_HASH = ();

sub check_dirs {
    # TEMPLATES ARTICULOS
    # Dir. de tpl de las pags de los articulos
    my $dir = $prontus_varglb::DIR_SERVER .
           $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} .
           $prontus_varglb::DIR_ARTIC .
           $prontus_varglb::DIR_FECHA .
           $prontus_varglb::DIR_PAG;

    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    if (!(&check_contenido_dir($dir))) {
        lib_stdlog::print_err(lib_lang::get_message("global_error_checkdirs_pltartic_empty"), "[$dir]\n");
        return lib_lang::get_message("global_error_checkdirs_pltartic_empty");
    }

    # DESTINO ARTICULOS -----------------------------
    # Dir a los articulos existentes hasta antes de la fecha
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_ARTIC;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir para respaldos
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_BKP;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir para semaforos mutex
    $dir = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_LOCK";
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir Multimedia
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EXMEDIA;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # DESTINO TAXONOMIA
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} > 0) {
        my $no_relac_dir = 0;
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXONOMIA_OLD;
        my $fail_dir;
        if (!(glib_fildir_03::check_dir($dir))) {
            return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
        }

        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXONOMIA;
        if (!(glib_fildir_03::check_dir($dir))) {
            return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
        }

        # TEMPLATE portada tipo tema
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXPORT;

        if (!(glib_fildir_03::check_dir($dir))) {
            return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
        }

        # DESTINO portadas tipo tema
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXPORT;

        if (!(glib_fildir_03::check_dir($dir))) {
            return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
        }
    }

    # Directorio de cache de multitag
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_MULTITAG;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir de los dbm
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'});
    }

    # dir de las keys para decriptar tokens openid connect
    if ($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} eq 'SI' && !(glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/oidc_keys/"))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/oidc_keys/");
    }

    # Dir de los logs
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/log'))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/log');
    }

    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/procs'))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/procs');
    }

    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . $prontus_varglb::DIR_DATA4FIDS))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . $prontus_varglb::DIR_DATA4FIDS);
    }

    # Dir cpan/fid
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/fid'))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/fid');
    }

    if (!(&check_contenido_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/fid'))) {
        return lib_lang::get_message("global_error_checkdirs_fids_empty");
    }

    # TEMPLATES DE PORTADAS
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_PORT;

    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    if (!(&check_contenido_dir($dir))) {
        return lib_lang::get_message("global_error_checkdirs_pltports_empty");
    }

    # Dir de TEMPLATE de homepages.
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_HPAGES;

    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Archivo Template de homepage de edición.
    my $nom_tpl_home = "$dir/$prontus_varglb::INDEX_EDIC";
    if (!-f $nom_tpl_home) {
        lib_stdlog::print_err(lib_lang::get_message('global_error_home_template_not_found') , "[$nom_tpl_home]\n");
    }

    # DESTINO EDICIONES
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir. destino de Edicion unica
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC;

    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    $dir .= $prontus_varglb::DIR_PORT;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir destino de los xml de portada de la ed. unica
    $dir =~ s/$prontus_varglb::DIR_PORT$/\/xml/;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    # Dir destino de los rss de portada de la ed. unica
    $dir =~ s/\/xml$/\/rss/;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_interpolated_message("global_error_checkdirs_invalid", $dir);
    }

    return '';
}

sub reset_prontus_config {
    %prontus_varglb::CONFIG = ();
    %prontus_varglb::CONFIG_LOADED = ();
}

sub reset_prontus_vars {
    # cache de artículos, para no leerlos de nuevo
    %ARTICS = ();
    # cache de multitag para no cargar de nuevo
    %MULTITAG = ();
    # cache de tags para no cargar de nuevo
    %TAGS_HASH = ();
    # datos de usuarios
    %prontus_varglb::USERS = ();
    %prontus_varglb::USER_DATA = ();
    %prontus_varglb::PROFILES_LOADED = ();
    %prontus_varglb::PROFILES = ();
}

# Chequea que dentro del dir pasado por parametro haya por lo menos 1 archivo.
sub check_contenido_dir {
    my $ruta_dir = $_[0];

    my @lisdir = glib_fildir_03::lee_dir($ruta_dir);
    foreach my $item (@lisdir) {
        # salta directorios . y .., y archivos que empiezan con .
        next if (substr($item, 0, 1) eq '.');

        if (-f "$ruta_dir/$item") {
            # encuentra a lo menos un archivo valido dentro del directorio.
            return 1;
        }
    }
    return 0;
}

# Obtener el nombre del primer archivo que figura en el directorio.
# Param: 0) Path al Directorio
# Retorna : el nombre del archivo correspondiente  sin path y c/extension o '' en caso de que no se encuentre ningun archivo.
sub get_nomfile {
    my $dir = $_[0];
    my @entries = glib_fildir_03::lee_dir($dir);
    foreach my $entry (@entries) {
        next (! -f "$dir/$entry");
        if (substr($entry, 0, 1) ne '.' and substr(lc $entry, 0, 7) ne 'preview') {
            return $entry;
        }
    }
    return '';
}

# chequea la extension contra un string (separados por coma)
sub check_extension {
    my ($nomfile, $str2check) = @_;

    my $file_extension = lc lib_utils::get_file_extension($nomfile);
    return 0 if ($str2check !~ /(^|,)$file_extension($|,)/);
    return 1;
}

# Chequea si el articulo tiene fotos
sub check_fotos_from_ts {
    my ($ts, $buff_xml_data) = @_;

    if (! defined($buff_xml_data)) {
        my $dirfecha = substr($ts, 0, 8);
        my $fullpath_xml = $prontus_varglb::DIR_SERVER .
                           $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} .
                           $prontus_varglb::DIR_ARTIC .
                           "/$dirfecha" .
                           "/xml" .
                           "/$ts.xml";
        $buff_xml_data = &get_xml_data($fullpath_xml);
    }
    if ($buff_xml_data =~ /<_nomfoto_(\d+)>(.+?)<\/_nomfoto_\1>/) {
        return 1;
    }
    return 0;
}

sub get_prontus_sso_dirs {
    my @entries = get_prontus_dirs($prontus_varglb::DIR_SERVER);
    my @core_dirs; # core dirs
    # se agrega el directorio del prontus master
    push(@core_dirs, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});

    # Recorre revisando todas las carpetas de la raiz que sean un prontus valido
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        my $buffer_conf = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$entry/cpan/$entry.cfg");
        if ($buffer_conf  =~ /PRONTUS_SSO\s*=\s*["']SI["']/ && $buffer_conf =~ /PRONTUS_SSO_MASTER_ID\s*=\s*["']\Q$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'}\E["']/) {
            lib_stdlog::print_err("SSO Enabled [$entry]\n");
            push(@core_dirs, $entry);
        }
    }
    return @core_dirs;
}

sub get_prontus_sso_masters {
    my @entries = get_prontus_dirs($prontus_varglb::DIR_SERVER);
    my @core_dirs; # core dirs

    # Recorre revisando todas las carpetas de la raiz que sean un prontus valido
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        my $buffer_conf = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$entry/cpan/$entry.cfg");
        if ($buffer_conf =~ /PRONTUS_SSO\s*=\s*["']SI["']/ && $buffer_conf =~ /PRONTUS_SSO_MASTER\s*=\s*["']SI["']/) {
            lib_stdlog::print_err("SSO Master Enabled [$entry]\n");
            push(@core_dirs, $entry);
        }
    }
    return @core_dirs;
}

sub get_prontus_dirs {
    my @entries = glib_fildir_03::lee_dir($prontus_varglb::DIR_SERVER);
    my @core_dirs; # core dirs

    # Recorre revisando todas las carpetas de la raiz que sean un prontus valido
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        next if (substr($entry, 0, 1) eq '.');
        next if (!-d "$prontus_varglb::DIR_SERVER/$entry");
        if (lib_utils::valida_prontus($entry, 1)) {
            push(@core_dirs, $entry);
        }
    }
    return @core_dirs;
}

# genera jsons con datos de prontus para ser usados via javascript
sub generaJsonProntus {
    # guardamos algunos datos para poder usarlos en el fid
    my %data = ();
    # guardamos algunos datos para poder usarlos en el frontend del sitio
    my %dataFrontend = ();
    $data{'public-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    $dataFrontend{'public-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    $data{'cpan-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    $data{'front-protocol'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'};
    $dataFrontend{'front-protocol'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'};

    $data{'cpan-protocol'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'};

    $data{'player-embed-video'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_VIDEO'};
    $data{'player-embed-audio'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_AUDIO'};

    # datos auth
    $data{'psw-max-length'} = prontus_auth::get_psw_max_length();
    $data{'psw-min-length'} = prontus_auth::get_psw_min_length();

    # datos para multitag
    $data{'multitag-incluir-prontus_id-url'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'INCLUIR_PRONTUS_ID_URL'} eq 'SI') {
        $data{'multitag-incluir-prontus_id-url'} = 1;
        $dataFrontend{'multitag-incluir-prontus_id-url'} = 1;
    }
    $data{'friendly-include-view-name'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
        $data{'friendly-include-view-name'} = 1;
    }

    $data{'friendly-v5-url-prefix'} = '';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URL_PREFIX'} ne '') {
        $data{'friendly-v5-url-prefix'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URL_PREFIX'};
    }

    $data{'oidc-enabled'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} eq 'SI') {
        $data{'oidc-enabled'} = 1;
    }

    $data{'seo-api-enabled'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_HABILITADO'} eq 'SI') {
        $data{'seo-api-enabled'} = 1;
    }

    @{$data{'mv'}} = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};

    $data{'version'} =  "$prontus_varglb::RAMA_INSTALADA.$prontus_varglb::NRO_REVISION_INSTALADA$prontus_varglb::BETA_REVISION_INSTALADA";

    my $jsonString = JSON::encode_json(\%data);
    utf8::decode($jsonString);

    my $jsonStringFrontend = JSON::encode_json(\%dataFrontend);
    utf8::decode($jsonStringFrontend);

    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS");
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/prontus.js", "var ProntusConf = $jsonString");

    # escribimos a frontend lo mínimo indispensable. Creamos el dir [prontus]/site/js si no existe.
    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js");
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js/prontus.js", "var ProntusConf = $jsonStringFrontend");

    my %data_ads = (
        "adEnable"   => ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_ENABLE'} eq 'SI')?1:0,
        "adInfoMsg"  => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_INFO_MSG'},
        "adTag" => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_TAG'});

    my $jsonStringAds = JSON::encode_json(\%data_ads);
    utf8::decode($jsonStringAds);

    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js/playerConf.json", $jsonStringAds);
}

# Carga plantilla de articulo incluyendo las macros que contenga.
sub carga_buffer_plt {
    my ($fullpath_plt, $pathdir_plt_macros, $mv) = @_;

    $fullpath_plt =~ s/(.+)\/([^\/]+)$/$1-$mv\/$2/ if ($mv ne '');
    # warn "cargando plantilla[$fullpath_plt]";
    my $buffer = glib_fildir_03::read_file($fullpath_plt);
    if ($buffer !~ /\w/) {
        return '';
    }
    $buffer = lib_parser::ajusta_crlf($buffer);
    # agrega macros.
    $buffer = lib_parser::add_macros($buffer, $pathdir_plt_macros);
    $buffer =~ s/%25%25/%%/sg;
    return $buffer;
}

sub replace_hash_fields {
    my ($buffer, $vars2replace, $prontus_style) = @_;

    $prontus_style = 1 if ($prontus_style eq '');    # default

    foreach my $k (keys %$vars2replace) {
        my $marca = $k;
        $marca = '%%' . $k . '%%' if ($prontus_style);
        my $value = $$vars2replace{$k};
        $buffer =~ s/$marca/$value/sg;
    }

    return $buffer;
}

sub replace_mtime {
    my ($buf, $path_artic, $mtime) = @_;

    # Si viene la fecha de modificacion desde el xml del articulo, se usa. sino se sigue usando la fecha de
    # modificacion del archivo.
    if ($mtime) {
        my $ts = glib_hrfec_03::time2ts($mtime);
        my $fechamlong = glib_hrfec_03::expande_fecha($ts);
        my $fechamshrt = glib_hrfec_03::des_normaliza_fecha($ts);

        my $fecham = substr($ts, 0, 8);
        my $horam = substr($ts, 8, 2). ':' . substr($ts, 10, 2) . ':' . substr($ts, 12, 2);

        $buf =~ s/%%_fechamlong%%/$fechamlong/g;
        $buf =~ s/%%_fechamshrt%%/$fechamshrt/g;
        $buf =~ s/%%_fecham%%/$fecham/g;
        $buf =~ s/%%_horam%%/$horam/g;
    } else {
        my $mtime = (stat $path_artic)[9];
        my $ts_lastmodif = glib_hrfec_03::time2ts($mtime);
        # warn("mtime[$mtime]ts_lastmodif[$ts_lastmodif]\n");

        my $fecham = substr($ts_lastmodif, 0, 8);
        my $horam  = substr($ts_lastmodif, 8, 6);
        $buf =~ s/%%_fecham%%/$fecham/g;

        my $flong = glib_hrfec_03::expande_fecha($fecham);
        $buf =~ s/%%_fechamlong%%/$flong/g;

        my $fshrt = glib_hrfec_03::des_normaliza_fecha($fecham);
        $buf =~ s/%%_fechamshrt%%/$fshrt/g;

        $horam = substr($horam, 0, 2) . ':' . substr($horam, 2, 2) . ':' . substr($horam, 4, 2);

        $buf =~ s/%%_horam%%/$horam/g;
    }

    return $buf;
}

# Reemplaza TS, FECHAC, FECHACLONG, FECHACSHRT, a partir del TS
sub replace_tsdata {
    my $buf = $_[0];
    my $ts  = $_[1];
    my $prettyprint_ts = defined($_[2]) ? $_[2] : '';

    # Timestamp
    if ($ts ne '') {
        my $fechac = substr($ts, 0, 8);
        my $horac  = substr($ts, 8, 2) . ':' . substr($ts, 10, 2) . ':' . substr($ts, 12, 2);
        $buf =~ s/%%_horac%%/$horac/g;

        my $fechaclong = glib_hrfec_03::expande_fecha($fechac);
        $buf =~ s/%%_fechaclong%%/$fechaclong/g;

        my $fechacshrt = glib_hrfec_03::des_normaliza_fecha($fechac);
        $buf =~ s/%%_fechacshrt%%/$fechacshrt/g;

    } else {
        $buf =~ s/%%_new_art%%/$prettyprint_ts/g;
        $buf =~ s/%%_ts%%//g;
        $buf =~ s/%%_fechaclong%%//g;
        $buf =~ s/%%_fechacshrt%%//g;
        $buf =~ s/%%_fechac%%//g;
        $buf =~ s/%%_horac%%//g;
    }
    return $buf;
}

# Ajusta el valor del campo de texto al max. nro. de caracteres
# $val_campo: string a ser recortada
# $nchars: cantidad de caracteres a conservar
sub ajusta_nchars {
    my ($val_campo, $nchars, $include_points) = @_;
    if (! $nchars) { # si pasan 0 o nada, no acortamos.
        return $val_campo;
    }
    my $val_campo_con_tags = $val_campo;
    # elimino los tags para que no interfieran en el conteo de palabras.
    $val_campo =~ s/<[^>]+>//sg;
    # elimina los &nbsp; insertados al formatear el texto (estos marcaban los saltos de linea.)
    $val_campo =~ s/&nbsp\;/ /sg;
    # Corta al nro. de caracteres especificado

    my $val_campo_original = $val_campo;
    # si el campo sin tags de formato tiene menos caracteres que el maximo
    # retorna el campo con los tags si los tenia originalmente
    if (length($val_campo_original) <= $nchars) {
        if (index($val_campo_con_tags, '<p>') > -1) {
            $val_campo_original = '<p>' . $val_campo_original . '</p>';
        }
        return $val_campo_original;
    }

    # Recorrer por palabras.
    my $string_ajustado = '';
    $val_campo = '';
    # busca palabras y las va uniendo hasta superar el maximo de caracteres
    while ($val_campo_original =~ /([^\s]+)/g) {
        $val_campo .= $1;
        # lib_stdlog::print_err("val_campo[$val_campo] largo[" . length($val_campo) . "][$string_ajustado]\n");

        # revisa si la frase acumulada sobrepasa el largo maximo permitido
        # si al agregar esta palabra sobrepasa el limite, no se agrega y
        # termina la busqueda de palabras
        # se restan 3 para que quede espacio para los puntos suspensivos
        if (length($val_campo) > $nchars) {
            # si el largo de una palabra es mayor al maximo, se recorta dicha palabra.
            if (length($string_ajustado) == 0) {
                $string_ajustado = substr($val_campo, 0, $nchars);
            }
            last;
        }
        # la frase actual no sobrepasa el limite, se guarda la frase acumulada
        $string_ajustado = $val_campo;
        $val_campo .= ' ';
    }

    # remueve espacio sobrante.
    $string_ajustado .= '...' if ($include_points);
    if (index($val_campo_con_tags, '<p>') > -1) {
        $string_ajustado = '<p>' . $string_ajustado . '</p>';
    }
    return $string_ajustado;
}

# Parsea fechaplong, fechaelong y fechapshrt, fechaeshrt en base a fechap en el articulo.
# @param string $buffer el texto a procesar
# @param string $val_campo el campo con la fecha
# @param string $nom_campo el campo con el nombre de la marca.
# @return string el buffer procesado
sub parser_fecha_ext {
    my ($buffer, $val_campo, $nom_campo) = @_;
    # %%_fechaplong%% = Fecha de publicación, en formato largo
    # %%_fechapshrt%% = Fecha de publicación, en formato corto
    # %%_fechap%%     = Fecha de publicación, en formato ISO --> Calza con el 2o. tipo de sustitucion estandar por tanto no se hace nada especial.
    # funciona igual para _fechae
    my $marca_shrt = $nom_campo . 'shrt';
    my $marca_long = $nom_campo . 'long';

    if ($val_campo) {
        my $fechaplong = glib_hrfec_03::expande_fecha($val_campo);

        $buffer =~ s/%%$marca_long%%/$fechaplong/g;

        my $fechapshrt = glib_hrfec_03::des_normaliza_fecha($val_campo);
        $buffer =~ s/%%$marca_shrt%%/$fechapshrt/g;
    } else {
        $buffer =~ s/%%$marca_shrt%%//g;
        $buffer =~ s/%%$marca_long%%//g;
    }

    return $buffer;
}

# Obtiene status de publicacion del articulo.
sub get_status_pub {
    my ($art, $control_fecha, $base) = @_;

    # Obtener datos necesarios del articulo de la bd
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_alta, art_fechap, art_horap, art_fechae, art_horae from $art_table where art_id = ?";
    my ($alta, $fechap, $horap, $fechae, $horae);
    my @params = ($art);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($alta, $fechap, $horap, $fechae, $horae));
    $salida->fetch;
    $salida->finish;

    # Filtro por fechap/horap y fechae/horae del artic
    my $pub = 1;          # indicador de si el artic. quedo publicado en el html o no.
    if ($control_fecha eq 'SI' && !lib_utils::fechas_ok('', $fechap, $horap, $fechae, $horae)) {
        $pub = 0;
    }
    # Filtro por alta
    if (!$alta) {
        $pub = 0;
    }

    return $pub;
}

# Formatea el texto submitido, linea a linea.
sub parrafea_texto {
    my $buffer = $_[0];
    my ($salida, $i) = ('', 0);

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $buffer =~ s/$crlf/<p>/sg;

    # Extrae TODAS las lineas y las pone en el arreglo @LINEAS.
    my @LINEAS = split(/<P[^>]*?>/i, $buffer);

    # Formatea las lineas de acuerdo a su contexto.
    for ($i = 0 ; $i <= $#LINEAS ; $i++) {
        # lib_stdlog::print_err("LINEA: $LINEAS[$i]\n\n");
        if ($LINEAS[$i] =~ /^\s*$/) {
            # Inserta espacios para que aparezcan las lineas usadas como separador.
            $LINEAS[$i] = '&nbsp;';
        }
        # Agrega parrafo a la salida.
        $salida .= $LINEAS[$i] . "</p>\n<p>";
    }

    # Saca ult. ret.
    $salida =~ s/<\/p>\n<p>$/<\/p>/i;
    $salida = '<p>' . $salida if ($salida =~ /\w/);    # esto es para asegurarse que el texto quede rodeado de <p>...</p>

    return $salida;
}

# Cargar xml del articulo a partir del path a su html o bien del mismo xml.
sub get_xml_data {
    my $path_final_xml = $_[0];
    if ($path_final_xml !~ /\.xml$/) {
        $path_final_xml =~ s/\/pags\/(\w+)(\.\w+)?$/\/xml\/$1\.xml/;
    }

    my $xml = glib_fildir_03::read_file($path_final_xml);
    return '' if ($xml eq '');

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $xml =~ s/$crlf/\x0a/sg;
    $xml =~ s|</?artic_data>||g;
    $xml =~ s|</?_private>||g;
    $xml =~ s|</?_public>||g;

    return $xml;
}

# Escapea lo minimo para que el texto no interfiera con los tags HTML.
sub escape_html_vtxt {
    my $toencode = $_[0];

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_ENCODE_CHARS'} eq 'SI') {
        # Hace escapeo de entities no numéricas.
        $toencode =~ s/&([^#][^0-9]+)/&amp;$1/g;
    }

    $toencode = lib_encodes::escape_html($toencode);

    return $toencode;
}

# Transforma comillas tipográficas, como en “Quotes1” «Quotes2» ‘Quotes3’ a &quot; para evitar problemas en el encoding
# parametros:
# - el texto a corregir
# retorna:
# - el texto corregido
sub convert_typography_quotes {
    my $to_clean = $_[0];
    $to_clean =~ s/[\p{Pi}\p{Pf}]/&quot;/g;
    $to_clean =~ s/\x{2013}/-/g;
    return $to_clean;
}

# Realiza escapeos minimos para poder guardar el campo tipo txt dentro de un cdata
# al interior de un XML.
sub escape_cdata {
    my $valor = $_[0];
    # para q no interfiera con cdata
    $valor =~ s/]]>/]] >/sg;
    return $valor;
}

sub parse_tag_prontus {
    my ($valor_campo, $nom_campo, $buffer, $noescape) = @_;

    # elimina secuencias <?
    $valor_campo =~ s/< *\?/< &#63;/g unless ($noescape);

    # para poder poner nombres de marcas
    $valor_campo =~ s/%%/&#37;&#37;/sg;

    my @temp = split(/_/, $nom_campo);
    my $tagtype = $temp[0]?$temp[0]:$temp[1];
    # Cambia rets carro por <p>
    $valor_campo = &parrafea_texto($valor_campo) if ($tagtype eq 'txt');

    # sustitucion en la pagina
    $buffer =~ s/%%$nom_campo%%/$valor_campo/sg;

    # parsea marcas con ajuste de chars
    $buffer = &parse_maxchars($nom_campo, $valor_campo, $buffer);

    # parsea otras salidas con restriccion de largo
    # parse raw
    # parse xml
    # parse html
    # parse codetext
    # parse jsontext
    # parse strip
    # parse urlencode
    $buffer = &parse_maxchars_special($nom_campo, $valor_campo, $buffer);

    return $buffer;
}

sub strip_text {
    my $to_encode = $_[0];

    # elimina saltos de linea y retornos de carro
    $to_encode =~ s/\s+/ /sg;
    $to_encode =~ s/<[^>]+>/ /g;
    $to_encode =~ s/ {2,}/ /g;
    $to_encode =~ s/ $//g;
    $to_encode =~ s/^ //g;
    $to_encode = lib_encodes::no_accent_mark($to_encode);
    # Elimina entidades html.
    $to_encode =~ s/&[^;]+;//g;
    $to_encode = lc $to_encode;
    $to_encode =~ s/[^abcdefghijklmnopqrstuvwxyz0-9_ ]//g;
    $to_encode =~ s/ /_/g;

    return $to_encode;
}

sub get_minitext_value {
    my $to_encode = $_[0];

    $to_encode =~ s/\s+/ /g;
    $to_encode =~ s/<.*?>/ /g;
    $to_encode =~ s/ {2,}/ /g;
    $to_encode =~ s/ $//g;
    $to_encode =~ s/^ //g;
    return $to_encode;
}

sub get_codetext_value {
    my $to_encode = $_[0];

    # caracteres imprimibles, del 33(dec) en adealnte, exceptuando caracteres no imprimibles.
    $to_encode =~ s/[^\x20-\x7e\x80\x82-\x9c\x9e-\xff]//sg;
    $to_encode =~ s/\\/\\\\/sg;
    $to_encode =~ s/"/\\"/sg;
    $to_encode =~ s/&quot;/\\"/sg;
    $to_encode =~ s/'/\\'/sg;
    $to_encode =~ s/&#39;/\\'/sg;

    return $to_encode;
}

# recibe una referencia a hash o array, o un escalar perl, devuelve una string json encodeada, rodeada en comillas dobles.
sub get_jsontext_value {
    my $to_encode = $_[0];
    my $json = JSON->new->allow_nonref;
    $to_encode = $json->encode($to_encode);
    $to_encode =~ s/^"(.*)"$/$1/gs;
    return $to_encode;
}

sub parse_maxchars_special {
    my ($nom_campo, $valor_campo, $buffer) = @_;

    my $buffer_aux = $buffer;
    my %already_parsed = ();

    # se hace utf8::encode para aumentar la velocidad de procesamiento de las regexp
    # ya que no estamos buscando caracteres extendidos
    utf8::encode($buffer_aux);
    while ($buffer_aux =~ /%%$nom_campo((?:\((\d+)\))?\.(\w+)(?:\.(\w+))?)%%/sg) {
        my $marca_extension = $1;
        my $maxchars        = $2;
        my $type            = lc($3);
        my $type2            = lc($4);
        my $temp_match = lc("$nom_campo$marca_extension");
        # lib_stdlog::print_err("[$temp_match] - [$nom_campo][$marca_extension][$maxchars][$type][$type2]\n");

        # las marcas se parsean todas de una vez, si se vuelve a encontrar
        # la misma marca en el html, ya se proceso y es innecesario continuar
        if (defined($already_parsed{"$temp_match"})) {
            next;
        }

        my $valor_real = $valor_campo;
        if (($maxchars > 0) && ($valor_campo)) {
            $valor_real = &ajusta_nchars($valor_campo, $maxchars, 1);
        }
        if ($type eq "code") {
            $valor_real = &get_codetext_value($valor_real);
        } elsif ($type eq "json") {
            $valor_real = &get_jsontext_value($valor_real);
        } elsif ($type eq "xml" ) {
            $valor_real = lib_encodes::escape_xml($valor_real);
        } elsif ($type eq "html" ) {
            $valor_real = lib_encodes::escape_html($valor_real);
        } elsif ($type eq "strip" ) {
            $valor_real = &strip_text($valor_real);
        } elsif ($type eq "urlencode" ) {
            $valor_real = lib_encodes::urlencode_text($valor_real);
        } elsif ($type eq "raw") {
            my $minitext = &get_minitext_value($valor_real);
            if ($type2 eq "code") {
                $valor_real = &get_codetext_value($minitext);
            } elsif ($type2 eq "json") {
                $valor_real = &get_jsontext_value($minitext);
            } elsif ($type2 eq "xml" ) {
                $valor_real = lib_encodes::escape_xml($minitext);
            } elsif ($type2 eq "html" ) {
                $valor_real = lib_encodes::escape_html($minitext);
            } elsif ($type2 eq "strip" ) {
                $valor_real = &strip_text($minitext);
            } elsif ($type2 eq "urlencode" ) {
                $valor_real = lib_encodes::urlencode_text($minitext);
            } else {
                $valor_real = $minitext;
            }
        }

        $buffer =~ s/%%$nom_campo\Q$marca_extension\E%%/$valor_real/g;
        $already_parsed{"$temp_match"} = 1;
    }
    return $buffer;
}

sub parse_maxchars {
    my ($nom_campo, $valor_campo, $buffer) = @_;

    my $buffer_aux = $buffer;
    my %already_parsed = ();

    while ($buffer_aux =~ /%%$nom_campo(\(([0-9]*)\))%%/sg) {
        my $marca_maxchars = $1;
        my $maxchars       = $2;
        my $temp_match = lc("$nom_campo$marca_maxchars");

        # las marcas se parsean todas de una vez, si se vuelve a encontrar
        # la misma marca en el html, ya se proceso y es innecesario continuar
        if (defined($already_parsed{"$temp_match"})) {
            next;
        }

        if (($maxchars > 0) && ($valor_campo)) {
            $valor_campo = &ajusta_nchars($valor_campo, $maxchars, 1);
        }
        $buffer =~ s/%%$nom_campo\Q$marca_maxchars\E%%/$valor_campo/g;
        $already_parsed{"$temp_match"} = 1;
    }
    return $buffer;
}

sub replace_in_xml {
    # tal vez esto debiera estar en Artic
    my ($xml_base, $nom_campo, $valor, $parse_as_cdata) = @_;

    # esto es necesario en html, no en xml
    # $valor =~ s/< *\?/< &#63;/g; # elimina secuencias <?

    if ($parse_as_cdata) {
        $valor = &escape_cdata($valor);
    } else {
        $valor = lib_encodes::escape_xml($valor);
    }

    if ($nom_campo =~ /^fotofija_/) {
        # quita servers name de fotos para que sean relativas al sitio
        $valor =~ s/^https?\:\/\/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'}//;
        $valor =~ s/^https?\:\/\/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}//;
    }

    if ($parse_as_cdata) {
        if (!($xml_base =~ s/<$nom_campo>.*?<\/$nom_campo>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>/s)) {
            # campo empieza con _, es privado
            if (substr($nom_campo, 0, 1) eq '_') {
                $xml_base =~ s/<\/_private>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>\n<\/_private>/;
            } else {
                $xml_base =~ s/<\/_public>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>\n<\/_public>/;
            }
        }
    } else {
        if (!($xml_base =~ s/<$nom_campo>.*?<\/$nom_campo>/<$nom_campo>$valor<\/$nom_campo>/s)) {
            # campo empieza con _, es privado
            if (substr($nom_campo, 0, 1) eq '_') {
                $xml_base =~ s/<\/_private>/<$nom_campo>$valor<\/$nom_campo>\n<\/_private>/;
            } else {
                $xml_base =~ s/<\/_public>/<$nom_campo>$valor<\/$nom_campo>\n<\/_public>/;
            }
        }
    }

    return $xml_base;
}

# Elimina tags y ret. de carro.
sub saca_tags_rets {
    my $toencode = $_[0];
    $toencode =~ s/\s+/ /sg;
    $toencode =~ s/<[^>]+>/ /sg;
    $toencode =~ s/ {2,}/ /sg;
    $toencode =~ s/ $//sg;
    $toencode =~ s/^ //sg;
    return $toencode;
}

# Retorna el nombre del directorio de ediciones mas nuevo, el que corresp. al nombre de
# la edicion mas nueva.
# Param: 0) Path absoluto al dir de ediciones
sub get_last_edic {
    my $dir_hpage = $_[0];

    my @entries = glib_fildir_03::lee_dir($dir_hpage);
    #  Utiliza un hash auxiliar con las fecha de las ediciones en iso + el correlativo para ordenar.
    my %aux_sort;
    foreach my $entry (@entries) {
        $entry =~ /^(\d\d\d\d)\_(\d\d)\_(\d\d)\_(\d+)$/;
        my $aaaammddnro = $1 . $2 . $3 . $4;
        $aux_sort{$entry} = $aaaammddnro;
    }

    # Ordena numericamente de mayor a menor.
    @entries = sort { $aux_sort{$b} <=> $aux_sort{$a} } @entries;

    # Ordenar descendentemente.
    foreach my $entry (@entries) {
        if (substr($entry, 0, 1) eq '.') {
            # Si es un dir valido
            if (-d "$dir_hpage/$entry") {
                return $entry;
            }
        }
    }
}

# Chequea existencia de dirs. de trabajo de la edicion seleccionada.
sub check_dirs_edic {
    my $dir_dest_edic = $_[0];

    # Dir de la edicion.
    if (!(glib_fildir_03::check_dir($dir_dest_edic))) {
        return lib_lang::get_message('global_error_invalid_edic_dir');
    }

    # Dir de portadas de la edicion.
    my $dir = $dir_dest_edic . $prontus_varglb::DIR_PORT;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_message('global_error_invalid_port_edic_dir');
    }

    # destino del xml de las portadas de la edicion
    $dir =~ s/$prontus_varglb::DIR_PORT$/\/xml/;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_message('global_error_invalid_xml_edic_dir');
    }

    # Dir de hpage de la edicion
    $dir = $dir_dest_edic . $prontus_varglb::DIR_HPAGES;
    if (!(glib_fildir_03::check_dir($dir))) {
        return lib_lang::get_message('global_error_invalid_home_edic_dir');
    }

    return '';
}

sub dev_tam_img {
    my $file = $_[0];
    my ($ancho, $alto, $msg, $orientation) = (0, 0, '', 0);

    if (!-e $file) {
        return (lib_lang::get_message('global_file_not_exist'), 0, 0, 0);
    } else {
        if ($file =~ /\.(jpg|jpe|jpeg)$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_jpg($file);
            # lib_stdlog::print_err("FILE[$file] ancho[$ancho] alto[$alto] MSG[$msg]\n");
            my $img_info = Image::ExifTool::ImageInfo($file, 'EXIF:Orientation#');
            $orientation = $img_info->{'Orientation #'}?$img_info->{'Orientation #'}:0;
        } elsif ($file =~ /\.gif$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_gif($file);
        } elsif ($file =~ /\.png$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_png($file);
        } elsif ($file =~ /\.svg$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_svg($file);
        } elsif ($file =~ /\.webp$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_webp($file);
        } else {
            return ("[$file]". lib_lang::get_message('global_error_no_image_file'), 0, 0);
        }

        return ($msg, $ancho, $alto, $orientation);
    }

    return (lib_lang::get_message('global_file_not_exist'), 0, 0, $orientation);
}    # dev_tam_img.

sub ancho_alto_webp {
    my $file = $_[0];

    my $img_info = Image::ExifTool::ImageInfo($file, 'ImageWidth', 'ImageHeight');
    my ($width, $height, $msg) = ('', '', '');

    if ($img_info) {
        $width = $img_info->{'ImageWidth'};
        $height = $img_info->{'ImageHeight'};
    } else {
        $msg = lib_lang::get_message('global_error_info_image');
    }

    return ($msg, $width, $height);
}

# (\d+)[\d.]*, rescarta el primer(os) numero(s) y descarta . y decimales si existen
sub ancho_alto_svg {
    my $file = $_[0];
    my $svg = glib_fildir_03::read_file($file);
    my ($w, $h, $msg) = '';

    if ($svg =~ /<svg([^>]*)/) {
        my $svg_props = $1;

        if ($svg_props =~ /width=["|'](\d+)[\d.]*["|']/) {
            $w = $1;
        }

        if ($svg_props =~ /height=["|'](\d+)[\d.]*["|']/) {
            $h = $1;
        }

        if (!$w || !$h) {
            if ($svg_props =~ /viewBox="(\d+)[\d.]* (\d+)[\d.]* (\d+)[\d.]* (\d+)[\d.]*"/) {
                $w = $3;
                $h = $4;
            }
        }
    }

    $msg = lib_lang::get_message('global_error_svg_dim') if (!$w || !$h);

    return ($msg, $w, $h);
}

sub ancho_alto_png {
    # Adaptacion de http://www.la-grange.net/2000/05/04-png.html
    my $file = $_[0];
    my $head = "";

    (open my $png, '<', $file) || return ( lib_lang::get_message('global_error_open_file') . " PNG", 0, 0);
    binmode $png;

    my ($a, $b, $c, $d, $e, $f, $g, $h) = 0;

    if (   defined($png)
        && read($png, $head, 8) == 8
        && $head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a"
        && read($png, $head, 4) == 4
        && read($png, $head, 4) == 4
        && $head eq "IHDR"
        && read($png, $head, 8) == 8)
    {
        ($a, $b, $c, $d, $e, $f, $g, $h) = unpack("C" x 8, $head);
        close $png;
        return ('', $a << 24 | $b << 16 | $c << 8 | $d, $e << 24 | $f << 16 | $g << 8 | $h);
    }
    close $png;
    return ( lib_lang::get_message('global_error_read_file') . ' png', 0, 0);
}

sub ancho_alto_jpg {
    my $file = $_[0];
    my ($c1, $c2, $ch, $s, $junk, $a, $b, $c, $d, $length);
    (open my $jpeg, '<', $file) || return (lib_lang::get_message('global_error_open_file') ." JPEG", 0, 0);
    binmode $jpeg;
    read($jpeg, $c1, 1);
    read($jpeg, $c2, 1);
    ((ord($c1) == 0xFF) && (ord($c2) == 0xD8)) || return (lib_lang::get_message('global_error_invalid_file') . ' JPG', 0, 0);

    my $done = 0;
    my $dummy;

    while (ord($ch) != 0xDA) {
        # Find next marker (JPEG markers begin with 0xFF)
        while (ord($ch) != 0xFF) {
            return (lib_lang::get_message('global_error_read_file') . ' JPG', 0, 0) unless read($jpeg, $ch, 1);
        }
        # JPEG markers can be padded with unlimited 0xFF's
        while (ord($ch) == 0xFF) {
            return (lib_lang::get_message('global_error_read_file') . ' JPG', 0, 0) unless read($jpeg, $ch, 1);
        }
        # Now, $ch contains the value of the marker.
        if ((ord($ch) >= 0xC0) && (ord($ch) <= 0xC3)) {
            return (lib_lang::get_message('global_error_invalid_file') . ' JPG', 0, 0) unless read($jpeg, $dummy, 3);
            return (lib_lang::get_message('global_error_invalid_file') . ' JPG', 0, 0) unless read($jpeg, $s,     4);
            ($a, $b, $c, $d) = unpack("C" x 4, $s);
            return ('', $c << 8 | $d, $a << 8 | $b);

        } else {
            # We **MUST** skip variables, since FF's within variable
            # names are NOT valid JPEG markers
            return (lib_lang::get_message('global_error_read_file') . ' JPG', 0, 0) unless read($jpeg, $s, 2);
            ($c1, $c2) = unpack("C" x 2, $s);
            $length = $c1 << 8 | $c2;
            if (!defined($length) || $length < 2) {
                close($jpeg);
                # Longitud del marcador JPEG errónea
                return (lib_lang::get_message('global_error_read_file') . ' JPG', 0, 0);
            }

            read($jpeg, $dummy, $length - 2);
        }
    }
}    # ancho_alto_jpg.

sub ancho_alto_gif {
    my $file = $_[0];
    my ($a, $b, $c, $d, $s, $type);
    (open my $gif, '<', $file) || return (lib_lang::get_message('global_error_open_file') ." GIF", 0, 0);
    binmode $gif;
    read($gif, $type, 6);
    $type =~ /GIF8[7,9]a/ || return (lib_lang::get_message('global_error_invalid_file') . ' GIF', 0, 0);
    read($gif, $s, 4) == 4 || return (lib_lang::get_message('global_error_corrupt_file') . ' GIF', 0, 0);
    close($gif);
    ($a, $b, $c, $d) = unpack("C" x 4, $s);
    return ('', ($b << 8 | $a), ($d << 8 | $c));
}    # ancho_alto_gif.

sub ancho_alto_bmp {
    my $file = $_[0];
    my ($a, $b, $c, $d, $e, $f, $g, $h, $s, $type);
    my $bmp;
    (open $bmp, '<', $file) || return (lib_lang::get_message('global_error_open_file') ." BMP", 0, 0);
    binmode $bmp;
    read($bmp, $type, 2);
    $type =~ /BM/ || return (lib_lang::get_message('global_error_invalid_file') . " BMP", 0, 0);
    read($bmp, $type, 12);
    read($bmp, $s, 4) == 4 || return (lib_lang::get_message('global_error_corrupt_file') . " BMP", 0, 0);
    $a = unpack("C" x 1, $s);

    if ($a - 4 == 12) {
        read($bmp, $s, 4) == 4 || return (lib_lang::get_message('global_error_corrupt_file') . " BMP", 0, 0);
        ($a, $b, $c, $d) = unpack("C" x 4, $s);
        close($bmp);
        return ('', ($b << 8 | $a), ($d << 8 | $c));
    } elsif ($a - 4 > 12) {
        read($bmp, $s, 8) == 8 || return (lib_lang::get_message('global_error_corrupt_file') . " BMP", 0, 0);
        ($a, $b, $c, $d, $e, $f, $g, $h) = unpack("C" x 8, $s);
        close($bmp);
        return ('', ($b << 8 | $a | $d << 8 | $c), ($f << 8 | $e | $h << 8 | $g));
    } else {
        close($bmp);
        return (lib_lang::get_message('global_error_invalid_file') . " BMP", 0, 0);
    }
}    # ancho_alto_bmp.

# determina si el tipo de imagen se puede editar
sub can_edit_img {
    my $img_type = $_[0];
    # Si no existe en el hash, se puede manipular.
    if (defined $prontus_varglb::IMG_TYPE_NOEDIT{$img_type}) {
        if ($img_type eq 'webp' && $prontus_varglb::IMG_TYPE_NOEDIT{$img_type} == 1) {
            if (GD::Image->can('webp')) {
                delete ($prontus_varglb::IMG_TYPE_NOEDIT{$img_type});
                return 1;
            } else {
                $prontus_varglb::IMG_TYPE_NOEDIT{$img_type} = 2;
            }
        }
        return 0;
    }

    return 1;
}

# detecta si es imagen animada, gif o webp
sub is_animated_image {
    my ($path, $img_type) = @_;
    # solo imagenes gif o webp pueden ser animadas
    if ($img_type ne 'gif' && $img_type ne 'webp') {
        return 0;
    }
    my $info = Image::ExifTool::ImageInfo($path);
    if ($$info{'Duration'}) {
        return 1;
    }
    return 0;
}

# Devuelve la string $buffer con las marcas _file y _fileurl* reemplazadas por las URL friendly correctas.
# el ultimo parametro es un hash con los datos necesarios del articulo:
# my %artic_data = (
#     'fid' => $campos_xml{'_fid'},
#     'custom_slug' => $campos_xml{'_custom_slug'},
#     'slug' => $campos_xml{'_slug'},
#     'url' => $campos_xml{'_url'},
#     'nom_seccion' => $campos_xml{'_nom_seccion1'},
#     'nom_tema' => $campos_xml{'_nom_tema1'},
#     'nom_subtema' => $campos_xml{'_nom_subtema1'},
#     'titular' => $campos_xml{'_txt_titular'},
#     'ts' => $this->{ts},
#     'prontus_id'  => $this->{prontus_id},
#     'mv' => $mv,
#     'ext' => lib_utils::get_file_extension($campos_xml{'_plt'})
# );
# my $fileurl = lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);
sub parse_filef_n {
    my $buffer = $_[0];
    my %artic_data = ();
    if (defined($_[1]) && ref($_[1]) eq ref {}) {
        %artic_data  = %{$_[1]};
    }

    return $buffer if (!$artic_data{'ts'} || !$artic_data{'prontus_id'} || !$artic_data{'titular'});
    return $buffer unless($buffer =~ /%%_file/);

    # Ajusta titular para usar en url
    my $titular = &ajusta_titular_friendly($artic_data{'titular'});
    unless ($artic_data{'ext'}) {
        $artic_data{'ext'} = 'html';
    }
    my $fecha4friendly = '';
    my $fecha = '';
    my $hora = '';

    if ($artic_data{'ts'} =~ /^(\d{4})(\d{2})(\d{2})(\d{6})/) {
        $fecha4friendly = $1 . '-' . $2 . '-' . $3;
        $fecha          = "$1$2$3";
        $hora           = $4;
    } else {
        return $buffer;
    }

    my $file = "/$artic_data{'prontus_id'}/site/artic/$fecha/pags/$artic_data{'ts'}.$artic_data{'ext'}";
    $buffer =~ s/%%_file%%/$file/g;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq 'N') {
        # Links normal, no friendly
        $buffer =~ s/%%_fileurl%%/$file/g;
        $buffer =~ s/%%_fileurl\(\w+\)%%/$file/g;
        $buffer =~ s/%%_fileurl_canon%%/$file/g;
    } else {
        my $prontus_proto = '';
        my $fileurl = '';
        my $fileurl_proto = '';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '1') {
            $fileurl = "/$titular/$artic_data{'prontus_id'}/$fecha4friendly/$hora.$artic_data{'ext'}";
        } else {
            my $tax = '';
            if (defined($artic_data{'nom_seccion'}) && $artic_data{'nom_seccion'} ne '') {
                $artic_data{'nom_seccion'} = &despulgar_texto_friendly($artic_data{'nom_seccion'});
                $tax = "/$artic_data{'nom_seccion'}";
                # no hay tema sin seccion
                if (defined($artic_data{'nom_tema'}) && $artic_data{'nom_tema'} ne '') {
                    $artic_data{'nom_tema'} = &despulgar_texto_friendly($artic_data{'nom_tema'});
                    $tax .= "/$artic_data{'nom_tema'}";
                    # no hay subtema sin tema
                    if (defined($artic_data{'nom_subtema'}) && $artic_data{'nom_subtema'} ne '') {
                        $artic_data{'nom_subtema'} = &despulgar_texto_friendly($artic_data{'nom_subtema'});
                        $tax .= "/$artic_data{'nom_subtema'}";
                    }
                }
            }

            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '2') {
                # Formato: /prontus/seccion/tema/subtema/titular/aaaa-mm-dd/hhnnss.extension
                $fileurl = "/$artic_data{'prontus_id'}$tax/$titular/$fecha4friendly/$hora.$artic_data{'ext'}";

            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '3') {
                # Formato: www.<prontus_id>.cl/seccion/tema/subtema/titular/aaaa-mm-dd/hhnnss.extension
                $fileurl = "$tax/$titular/$fecha4friendly/$hora.$artic_data{'ext'}";

            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '4') {
                # Formato: /prontus/seccion/tema/subtema/titular.extension
                # si el fid esta excluido de friendly 4, se genera url directa
                if (defined($artic_data{'fid'}) && !exists $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$artic_data{'fid'}}) {
                    # si se debe usar url custom
                    if (defined($artic_data{'custom_slug'}) && $artic_data{'custom_slug'} eq 'SI') {
                        # rescatamos el slug, si no existe usamos el titular de forma normal
                        if (defined($artic_data{'slug'}) && $artic_data{'slug'} ne '') {
                            $titular = &ajusta_titular_friendly($artic_data{'slug'});
                        }
                    }

                    my $vista = '';
                    # obtenemos la vista para la url
                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $artic_data{'mv'}) {
                        $vista = "/$artic_data{'mv'}";
                    }

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} eq 'SI') {
                        $prontus_proto = "/$artic_data{'prontus_id'}";
                    }

                    # evita casos en que la ultima taxonomia y el titular son iguales
                    if ($tax ne '' && $tax =~ m|/$titular$|) {
                        $fileurl = "$prontus_proto$vista$tax";
                        $fileurl_proto = $tax;
                    } else {
                        $fileurl = "$prontus_proto$vista$tax/$titular";
                        $fileurl_proto = "$tax/$titular";
                    }

                } else {
                    $fileurl = $file;
                }
            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                my $vista = '';

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $artic_data{'mv'}) {
                    $vista = "/$artic_data{'mv'}";
                }

                if (defined($artic_data{'fid'})
                    && !exists $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$artic_data{'fid'}}
                    && defined($artic_data{'url'})) {
                    $fileurl = $vista . $artic_data{'url'};
                    $fileurl_proto = $artic_data{'url'};
                } else {
                    # Si no existe la url se muestra la ruta directa al archivo.
                    $fileurl = $file;
                    $fileurl_proto = $fileurl;
                }
            }
        }
        #lib_stdlog::print_err("fileurl[$fileurl] prontus_proto[$prontus_proto] fileurl_proto[$fileurl_proto]\n");

        $buffer =~ s/%%_fileurl%%/$fileurl/g;    # Links friendly
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '4' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
            $buffer =~ s/%%_fileurl\((\w+)\)%%/$prontus_proto\/$1$fileurl_proto/g; # Links friendly con vista sin prontus
            $buffer =~ s/%%_fileurl_canon%%/$prontus_proto$fileurl_proto/g;    # link de url canonica, vista principal
        } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
            $buffer =~ s/%%_fileurl\((\w+)\)%%/\/$1$fileurl_proto/g; # Links friendly con vista sin prontus
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        } else {
            $buffer =~ s/%%_fileurl\(\w+\)%%/$fileurl/g; # Links friendly
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        }
    }

    return $buffer;
}

# Devuelve la string $buffer con las marcas _file y _fileurl* reemplazadas por las URL friendly correctas.
# el ultimo parametro es un hash con los datos necesarios del articulo:
# my %artic_data = (
#     'fid' => $campos_xml{'_fid'},
#     'url' => $campos_xml{'_url'},
#     'ts' => $this->{ts},
#     'prontus_id'  => $this->{prontus_id},
#     'mv' => $mv,
#     'ext' => lib_utils::get_file_extension($campos_xml{'_plt'})
# );
# my $fileurl = lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);
sub parse_filef {
    my $buffer = $_[0];
    my %artic_data = ();
    if (defined($_[1]) && ref($_[1]) eq ref {}) {
        %artic_data  = %{$_[1]};
    }

    return $buffer if (!$artic_data{'ts'} || !$artic_data{'prontus_id'});
    return $buffer unless($buffer =~ /%%_file/);

    unless ($artic_data{'ext'}) {
        $artic_data{'ext'} = 'html';
    }
    my $fecha4friendly = '';
    my $fecha = '';
    my $hora = '';

    if ($artic_data{'ts'} =~ /^(\d{4})(\d{2})(\d{2})(\d{6})/) {
        $fecha4friendly = $1 . '-' . $2 . '-' . $3;
        $fecha          = "$1$2$3";
        $hora           = $4;
    } else {
        return $buffer;
    }

    my $file = "/$artic_data{'prontus_id'}/site/artic/$fecha/pags/$artic_data{'ts'}.$artic_data{'ext'}";
    $buffer =~ s/%%_file%%/$file/g;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} ne '5') {
        # Links normal, no friendly
        $buffer =~ s/%%_fileurl%%/$file/g;
        $buffer =~ s/%%_fileurl\(\w+\)%%/$file/g;
        $buffer =~ s/%%_fileurl_canon%%/$file/g;
    } else {
        my $fileurl = '';
        my $fileurl_proto = '';

        my $vista = '';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $artic_data{'mv'}) {
            $vista = "/$artic_data{'mv'}";
        }

        if (defined($artic_data{'fid'})
            && !exists $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$artic_data{'fid'}}
            && defined($artic_data{'url'})) {
            $fileurl = $vista . $artic_data{'url'};
            $fileurl_proto = $artic_data{'url'};
        } else {
            # Si no existe la url se muestra la ruta directa al archivo.
            $fileurl = $file;
            $fileurl_proto = $fileurl;
        }

        $buffer =~ s/%%_fileurl%%/$fileurl/g;    # Links friendly
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
            $buffer =~ s/%%_fileurl\((\w+)\)%%/\/$1$fileurl_proto/g; # Links friendly con vista sin prontus
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        } else {
            $buffer =~ s/%%_fileurl\(\w+\)%%/$fileurl/g; # Links friendly
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        }
    }

    return $buffer;
}

# Funcion para ajustar el titular para friendly url
sub ajusta_titular_friendly {
    my $titular = $_[0];
    # recorta largo del titular
    $titular = &ajusta_nchars($titular, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'});

    return &despulgar_texto_friendly($titular);
}

sub despulgar_texto_friendly {
    my $texto = $_[0];
    $texto = &saca_tags_rets($texto);
    $texto = lib_encodes::no_accent_mark($texto);
    # Elimina entidades html.
    $texto =~ s/&[^;]+;//g;
    $texto = lc $texto;
    $texto =~ s/[^a-z0-9]+/-/sg;
    $texto =~ s/-{2,}/-/g;
    # elimina todos los guiones al principio
    $texto =~ s/^-+//sg;
    # elimina todos los guiones al final
    $texto =~ s/-+$//sg;

    return $texto;
}

# formatea numero en unidades binarias
# $bytes: cantidad de bytes a formatear
# $dec: cantidad de decimales a mostrar
sub format_bin_bytes {
    my ($bytes, $dec) = @_;
    if ($bytes) {
        my @units = ('B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB');

        my $k = 0;
        while ($bytes >= 1024) {
            $bytes = $bytes/1024;
            $k++;
        }

        $bytes = glib_str_03::format_miles_dec($bytes, $dec);
        return "$bytes $units[$k]";
    } else {
        return '0 B';
    }
}

# aaaammddhhmmss  -> Mon, 24 Apr 2006 16:52:32 GMT
sub get_dtime_rfc822 {
    my $aaaammddhhmmss = $_[0];
    return '' if ($aaaammddhhmmss =~ /^9999/);
    if ($aaaammddhhmmss =~ /^(\d{8})(\d{2})(\d{2})(\d{2})/) {
        my ($aaaammdd, $hh, $mm, $ss) = ($1, $2, $3, $4);
        return &fecha2rfc822($aaaammdd, $hh, $mm, $ss);
    } else {
        return '';
    }
}

# Obtiene hhmmss a partir de hh:mm
sub get_hora_iso {
    my $horap = $_[0];
    my $horap_iso;
    if ($horap =~ /^(\d\d):(\d\d)$/) {
        $horap_iso = $1 . $2 . '00';
    } else {
        $horap_iso = '0' x 6;
    }
    return $horap_iso;
}

# Toma una fecha en iso y la pone como "Mon, 24 Apr 2006 16:52:32 GMT"
# Param : string con la fecha ISO.
sub fecha2rfc822 {
    my ($fecha, $hh, $mm, $ss) = @_;

    my @dias = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
    my @meses = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
    $fecha =~ /(\d\d\d\d)(\d\d)(\d\d)/;
    my ($dia, $mes, $ano) = ($3, $2, $1);

    my $tiempo = Time::Local::timelocal_nocheck($ss, $mm, $hh, $dia, ($mes - 1), ($ano - 1900)) || return '';
    my $tiempo2 = Time::Local::timegm_nocheck($ss, $mm, $hh, $dia, ($mes - 1), ($ano - 1900)) || return '';

    $tiempo = $tiempo + ($tiempo - $tiempo2);

    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($tiempo);
    $year = $year + 1900;

    $sec  = sprintf('%02d', $sec);
    $min  = sprintf('%02d', $min);
    $hour = sprintf('%02d', $hour);

    return $dias[$wday] . ", $mday " . $meses[$mon] . " $year $hour:$min:$sec GMT";
}

# parsea invocacion a funciones definidas en lib_custom.pm
sub parser_custom_function {
    my $buffer       = $_[0];

    if (!$PF_ENABLED) {
        # Avisa del error solo en caso de que las p functions se esten utilizando.
        if ($buffer =~ /%%_pf_\w+\(.*?\)%%/s) {
            lib_stdlog::print_err(lib_lang::get_message('global_error_load_lib_custom'));
            my $msg = lib_lang::get_message('global_error_load_pf');
            $buffer =~ s/%%_pf_\w+\(.*?\)%%/<h2>$msg<h2>/sg;
        }
    } else {
        while ($buffer =~ /%%(_pf_(\w+)\((.*?)\))%%/sg) {
            my $match = $1;
            my $function = $2;
            my $params   = $3;

            my @args = ();
            # Para obtener los parametros de entrada
            if ($params ne '') {
                $params =~ s/^\s*["']//;
                $params =~ s/["']\s*$//;

                my @arrparams = split(/"\s*,\s*"|'\s*,\s*'/, $params);
                foreach my $keys (@arrparams) {
                    # borra marcas prontus
                    $keys =~ s/%%.*?%%//g;
                    push(@args, $keys);
                }
            }

            # devuelve CODE, pero si no existe se cae
            # lib_stdlog::print_err(ref(\&{lib_custom->${\$function}}) ."\n");
            # devuelve CODE(0x337ca78), direccion de memoria, no se cae si no existe
            # y retorna undef sirve para probar con defined
            # no estamos usando objetos pero es alternativa para probar
            # lib_stdlog::print_err(lib_custom->can($function) ."\n");

            # esta forma permite invocar como funcion sin romper el funcionamiento anterior
            my $action = "lib_custom::$function";
            if (defined(&{$action} )) {
                # lib_stdlog::print_err("Ejecutar lib_custom::$function\n");
                my $result = &{\&{"lib_custom::$function"}}(@args);

                if (!($buffer =~ s/%%\Q$match\E%%/$result/s)) {
                    lib_stdlog::print_err("[FAIL] replace[%%$match%%] x result[$result]\n");
                }
            } else {
                lib_stdlog::print_err(lib_lang::get_interpolated_message('global_error_pf_not_found', $function));
            }
        }
    }
    return $buffer;
}

# rotulos tax
sub despulga_item_tax {
    my $str = $_[0];
    $str =~ s/["']//g;
    $str =~ s/\t|\r|\n/ /sig;
    $str =~ s/\\/ /sg;
    $str =~ s/ +/ /g;
    $str =~ s/^ +//g;
    $str =~ s/ +$//g;
    return $str;
}

# rotulos tag
sub despulga_item_tag {
    my $str = $_[0];
    $str =~ s/["']//g;
    # la [,|<>] se estripean por req. de la gestion de tags en el FID
    $str =~ s/[\t\r\n\,\|\>\<]/ /sg;
    $str =~ s/\\/ /sg;
    $str =~ s/ +/ /g;
    $str =~ s/^ +//g;
    $str =~ s/ +$//g;
    return $str;
}

sub get_nomtax_envista {
    my ($mv, $nom4vistas, $nom_default) = @_;

    $nom_default = '' if (!defined($nom_default));

    if ($nom4vistas =~ /(^|\n)$mv\t(.*?)($|\n)/s) {
        # necesita ser escapeada html en destino antes de usar.
        return $2;
    } else {
        # necesita ser escapeada html en destino antes de usar.
        return $nom_default;
    }
}

sub get_nom4vistas {
    my ($bd, $mv, $id_s, $id_t, $id_st) = @_;

    my ($nom_s, $nom_t, $nom_st) = ('', '', '');
    my ($secc_nom4vistas, $temas_nom4vistas, $subtemas_nom4vistas) = ('', '', '');

    $mv = 'default' if (!$mv);

    my $salida;
    my @result;

    # nombres en la vista dada
    if ($id_s) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv})) {
            my @params = ($id_s);
            my $secc_table = lib_utils::get_table('secc');
            my $sql = "select secc_nom, secc_nom4vistas from $secc_table where secc_id = ?";
            $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_s, $secc_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_s);
            glib_str_03::correct_encode_utf8_string($secc_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{'default'} = $nom_s;

            my @vistas = split(/\n/, $secc_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv});
    } else {
        push (@result, '');
    }

    if ($id_t) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv})) {
            my @params = ($id_t);
            my $temas_table = lib_utils::get_table('temas');
            my $sql = "select temas_nom, temas_nom4vistas from $temas_table where temas_id = ?";
            $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_t, $temas_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_t);
            glib_str_03::correct_encode_utf8_string($temas_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{'default'} = $nom_t;

            my @vistas = split(/\n/, $temas_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv});
    } else {
        push (@result, '');
    }

    if ($id_st) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv})) {
            my @params = ($id_st);
            my $subtemas_table = lib_utils::get_table('subtemas');
            my $sql = "select subtemas_nom, subtemas_nom4vistas from $subtemas_table where subtemas_id = ?";
            my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_st, $subtemas_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_st);
            glib_str_03::correct_encode_utf8_string($subtemas_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{'default'} = $nom_st;

            my @vistas = split(/\n/, $subtemas_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv});
    } else {
        push (@result, '');
    }

    return @result;
}

# Valida xml bien formado.
sub valid_xml {
    my $buffer = $_[0];
    my $p = new XML::Parser();
    eval { $p->parse($buffer) };
    if ($@) {
        $@ =~ s/ at \/.*$//;
        $@ =~ s/^(\n|\r)*//;
        return $@;
    }
    # xml esta ok
    return '';
}

# Valida xml bien formado para la importacion
sub valid_xml_import {
    my $buffer = $_[0];
    $buffer =~ s/&/&amp;/g;
    my $p = new XML::Parser();
    eval { $p->parse($buffer) };
    if ($@) {
        $@ =~ s/ at \/.*$//;
        $@ =~ s/^(\n|\r)*//;
        return $@;
    }
    # xml esta ok
    return '';
}

# Asigna valores por defecto a la fechap y fechae cuando estos vienen vacios desde el fid.
# Para fechap vacio:
#    Si hay control fecha => poner 99999999
#    Si no hay control fecha => poner fecha de hoy en iso
# Para fechae vacio:
#    Siempre poner 99999999 (independiente del ctrl fecha)
sub asigna_fecha_default {
    my $nom_campo = $_[0];

    my $fecha_default;
    if ($nom_campo eq '_fechap') {
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $fecha_default = '9' x 8;
        } else {
            $fecha_default = glib_hrfec_03::get_date_pack4();
        }
    } elsif ($nom_campo eq '_fechae') {
        $fecha_default = '9' x 8;
    }
    return $fecha_default;
}

# Revisa si el reg. ya existe en la BD.
sub existe_registro {
    my ($sql, $basedat, @data_array) = @_;

    my ($salida, $campo);
    if (defined($_[2])) {
        $salida = glib_dbi_03::ejecutar_sql_bind_param($basedat, $sql, \@data_array, \($campo));
    } else {
        $salida = glib_dbi_03::ejecutar_sql_bind($basedat, $sql, \($campo));
    }
    $salida->fetch;
    $salida->finish;
    if ($campo ne '') {
        glib_str_03::correct_encode_utf8_string($campo);
        return $campo;
    } else {
        return 0;
    }
}

# Descripcion:
# Retorna un hash con pares (<nombre_del_campo>, <valor_del_campo>)
# en base a un buffer xml dado.
#
# Parametros de entrada:
# $xmlData : buffer xml, obligatorio
# $campos  : lista de campos a cargar, separados por comas, optativo
#            Si no viene, se cargan todos los campos en el hash.
#
# Retorna:
# %camposHash : hash con los campos cargados.
sub getCamposXml {
    my ($xmlData, $campos) = @_;
    my %camposHash;
    if ($campos) {
        $campos =~ s/\s//g;
        my @camposLista = split(/,/, $campos);
        foreach my $nombreCampo (@camposLista) {
            if ($xmlData =~ /<$nombreCampo>(.+?)<\/$nombreCampo>/s) {
                &get_tag_value($nombreCampo, $1, \%camposHash);
            }
        }
    } else {
        while ($xmlData =~ /<([\w\.]+?)>(.*?)<\/\1>/sg) {
            my $nombreCampo = lc($1);
            &get_tag_value($nombreCampo, $2, \%camposHash);
        }
    }
    return %camposHash;
}

# procesa el valor del tag xml segun el tipo de marca
sub get_tag_value {
    my ($nombreCampo, $campoValor, $camposHash) = @_;
    my @temp = split(/_/, $nombreCampo);
    my $tagtype = $temp[0]?$temp[0]: '_'. $temp[1];

    if ($tagtype eq 'foto' && $campoValor) {
        $camposHash->{$nombreCampo} = 1;
        #<foto_00000249>
        #    <_nomfoto_00000249>foto_0000024920200807223546_IMG_20160708_145344637.jpg</_nomfoto_00000249>
        #    <_wfoto_00000249>4608</_wfoto_00000249>
        #    <_hfoto_00000249>2592</_hfoto_00000249>
        #    <_bfoto_00000249>1684270</_bfoto_00000249>
        #    <_mfoto_00000249>1602887775</_mfoto_00000249>
        #    <_ofoto_00000249>1</_ofoto_00000249>
        #</foto_00000249>

        if ($campoValor =~ /<_nom$nombreCampo>(.+?)<\/_nom$nombreCampo>/i) {
            $camposHash->{"_nom$nombreCampo"} = $1;
        }
        if ($campoValor =~ /<_w$nombreCampo>(.+?)<\/_w$nombreCampo>/i) {
            $camposHash->{"_w$nombreCampo"} = $1;
        }
        if ($campoValor =~ /<_h$nombreCampo>(.+?)<\/_h$nombreCampo>/i) {
            $camposHash->{"_h$nombreCampo"} = $1;
        }
        if ($campoValor =~ /<_b$nombreCampo>(.+?)<\/_b$nombreCampo>/i) {
            $camposHash->{"_b$nombreCampo"} = $1;
        }
        if ($campoValor =~ /<_m$nombreCampo>(.+?)<\/_m$nombreCampo>/i) {
            $camposHash->{"_m$nombreCampo"} = $1;
        }
        if ($campoValor =~ /<_o$nombreCampo>(.+?)<\/_o$nombreCampo>/i) {
            $camposHash->{"_o$nombreCampo"} = $1;
        }
    }
    elsif ($tagtype eq 'multimedia') {
        if (index($temp[1], '.') < 0) {
            $campoValor =~ s/\s//g;
        }
        $camposHash->{$nombreCampo} = $campoValor;
    }
    # campos txt tienen CDATA en el valor y pueden ser vacios
    elsif ($tagtype eq '_txt') {
        if ($campoValor =~ /<!\[CDATA\[(.+?)\]\]>/s) {
            $camposHash->{$nombreCampo} = $1;
        } else {
            $camposHash->{$nombreCampo} = '';
        }
    }
    # procesa campos reservados excepto _txt
    elsif (substr($nombreCampo, 0, 1) eq '_') {
        # campos reservados pueden tener valor vacío
        $camposHash->{$nombreCampo} = $campoValor;
    }
    elsif ($campoValor && $campoValor =~ /<!\[CDATA\[(.*?)\]\]>/s) {
        $camposHash->{$nombreCampo} = $1;
    } else {
        $camposHash->{$nombreCampo} = $campoValor;
    }
}

# Lee un archivo por completo para efectos de parsearlo como ssi simulada en los FIDs
# Param. de entrada :
# 0) Path real del archivo.
# Retorna : El texto leido | '' en caso que el archivo no exista.
sub read_file4include {
    my $archivo = $_[0];
    # elimina cualquier ../ dentro del path
    $archivo =~ s/\.+[\/\\]+//g;

    my $full_path = $prontus_varglb::DIR_SERVER . $archivo;
    my $buffer = '';
    if (-f $full_path) {
        my $size = -s $full_path;
        my $ARCHIVO;
        unless (open($ARCHIVO, '<', $full_path)) {
            lib_stdlog::print_err("Fail Open file $full_path \n $!\n");
            return '';
        }
        binmode $ARCHIVO;
        read $ARCHIVO, $buffer, $size;
        close $ARCHIVO;
    }

    return $buffer;
}

sub parse_includes {
    my $buffer = $_[0];

    $buffer =~ s/%%include\((.+?)\)%%/&read_file4include($1)/ge;
    return $buffer;
}

sub parse_userinfo {
    my ($buffer, $users_id) = @_;

    $buffer =~ s/%%_users_name%%/$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$users_id}{'USERS_NOM'}/g;
    $buffer =~ s/%%_users_usr%%/$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$users_id}{'USERS_USR'}/g;
    $buffer =~ s/%%_users_lang%%/$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$users_id}{'USERS_LANG'}/g;

    return $buffer;
}

# Imprime error al STDERR y retorna un msg amistoso para el usuario, o bien, hace un exit sin mas tramite
sub handle_internal_error {
    my ($msg_internal, $msg_external, $options) = @_;

    lib_stdlog::print_err($msg_external . " - " . $msg_internal);
    return $msg_external;
}

# lee el archivo "plugin.json" para el path especificado, dentro del directorio de plugins de Prontus
# devuelve una referencia a un hash con todos los datos del plugin.
sub get_plugin_info {
    my $plugin_name = $_[0];

    # evitamos que intenten cargar archivos con . o ..
    $plugin_name =~ s/\.\.+/./sg;
    my $path_plugin = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/prontus_plugins/$plugin_name/plugin.json";
    return {} if (! -f $path_plugin);

    my $json_text = glib_fildir_03::read_file_bin($path_plugin);
    my $data = eval {JSON::decode_json($json_text)};
    if ($@) {
        lib_stdlog::print_err(lib_lang::get_interpolated_message('global_json_decode_error', $@, $json_text)."\n");
        return {};
    }

    # evitamos que intenten cargar archivos con . o ..
    $data->{'plugin'} =~ s/\.\.+/./sg;
    next unless $data->{'nombre'};

    return $data;
}

sub get_url {
    my ($url, $espera_segs, $type) = @_;

    $espera_segs = 30    if ($espera_segs eq '');
    $type        = 'GET' if ($type        eq '');
    return '' if (($url eq '') or ($url !~ /^https?/i));
    # https://stackoverflow.com/a/9401362 intenta hacer que realmente respete el timeout.
    local $SIG{ALRM} = sub { die "SERVER timeout\n" };
    my $ua = new LWP::UserAgent;
    # segs # default de este módulo es 180, 3 minutos.
    $ua->timeout($espera_segs);
    # default es "libwww-perl/#.##"
    $ua->agent('Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36');
    my $request = new HTTP::Request($type, $url) || return ('', $!);
    my $response;
    eval {
        alarm($espera_segs);
        $response = $ua->request($request) || return ('', $!);
    }; # este ; debe ir ya que termina la linea de eval, sino la linea queda abierta
    # detiene la alarma anterior si no se gatilló
    alarm(0);
    if ($@) {
        if ($@ =~ /SERVER timeout/) {
            return('', 'Request Timeout.');
        } else {
            return ('', "Request Error: $@");
        }
    } else {
        if ($response->is_success) {
            return ($response->content, '');
        } else {
            return ('', $response->status_line);
        }
    }
}    # get_url

# Obtiene el tamano del archivo segun el valor de la cabecera HTTP Content-length.
sub get_http_content_length {
    my $url = $_[0];

    my $ua = new LWP::UserAgent;
    $ua->agent('Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36');
    my $req = new HTTP::Request 'HEAD' => $url;
    $req->header('Accept' => 'text/html');
    my $res = $ua->request($req);
    if ($res->is_success) {
        my $headers = $res->headers;
        return $headers->content_length;
    }
    return 0;
}

sub get_formatos_multimedia {
    my $marca = $_[0];
    my %formatos;
    my $file_formatos = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::XCODING_DATA_PATH$prontus_varglb::XCODE_CFG";

    if ($marca ne '') {
        if (-f $file_formatos) {
            my $buffer_formatos = glib_fildir_03::read_file($file_formatos);
            while ($buffer_formatos =~ /\s*($marca\.\w+)\.(\w+)\s*=\s*'(.*?)'/g) {
                $formatos{$1}{$2} = $3;
            }
        }
    }

    return %formatos;
}

sub dropbox_backup {
    my $recurso       = $_[0];
    my $dir_dropbox   = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA/dropbox";
    my $pid           = $$;
    my $dir_semaf     = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA/semaforos";
    my $semaf_dropbox = "$dir_semaf/dropbox_backup.lck";
    my $file          = "$dir_dropbox/$^T_$pid.txt";

    glib_fildir_03::check_dir($dir_dropbox);
    glib_fildir_03::check_dir($dir_semaf);

    open(my $FILE, '>>', $file);
    print $FILE $recurso . "\n";
    close $FILE;

    if (-f $semaf_dropbox) {
        my $mtime = (stat($semaf_dropbox))[9];
        my $now   = time;
        my $diff  = $now - $mtime;
        if ($diff > 7200) {    # 2 hrs.
            # muy antiguo, eliminar.
            unlink $semaf_dropbox;
        }
    }

    if (!-f $semaf_dropbox && -f $file) {
        my $cmd = "/usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_dropbox_backup.pl $prontus_varglb::PRONTUS_ID $file >/dev/null 2>&1 &";
        glib_fildir_03::write_file($semaf_dropbox, "0");
        lib_stdlog::print_err("dropbox[$cmd]\n");
        system $cmd;
    }

}

# Escribe al archivo PURGEFILE definido en prontus_purge_cache.pl las líneas a purgar.
sub purge_cache {
    my $path_file = $_[0];

    return if ($DISABLE_PURGE_CACHE);

    my $servers = (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}});

    return if (!$servers && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} ne 'SI');

    my $relpath = lib_utils::remove_front_string($path_file, $prontus_varglb::DIR_SERVER);

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
        if ($relpath =~ m|/site/tax/port/|i && $relpath !~ /_1\.\w+$/) {
            return;
        }
    }

    my $dir_pend = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/purgepend";
    glib_fildir_03::check_dir($dir_pend);
    my $pid = $$;
    open(my $PURGEFILE, '>>', "$dir_pend/$^T_$pid.txt");
    print $PURGEFILE $relpath . "\n";
    close $PURGEFILE;
    $DO_PURGE = 1;
}

sub set_exclude_port_table {
    my ($path_xml, $exclude_areas, $base) = @_;

    my $nom_table = glib_str_03::random_string(8);
    $nom_table = "exclude_port_$nom_table";

    $exclude_areas =~ s/\s*//sg;
    $exclude_areas =~ s/\t//sg;
    $exclude_areas =~ s/\n//sg;

    my @numAreas = split(/,/, $exclude_areas);
    my %areas;
    @areas{@numAreas} = undef;

    if ($exclude_areas && -f $path_xml) {
        my $buffer = glib_fildir_03::read_file($path_xml);

        # Crear tabla temporal.
        my $sql_create = "create temporary table $nom_table (exclude_art_id bigint unsigned not null default 0);";

        glib_dbi_03::ejecutar_sql($base, $sql_create);
        if ($base->errstr) {
            my $error = lib_lang::get_message('artic_error_create_temp_table');
            lib_stdlog::print_err("$error [$sql_create][$!][".$base->errstr."]\n");
            return '';
        }

        my %rowartics = lib_ports::get_rowartics($buffer);

        # Rescatar la info de c/artic de la port actual
        foreach my $art (keys %rowartics) {
            #lib_stdlog::print_err("art[$art] area[$area]\n");

            my $sql_insert = "insert into $nom_table set exclude_art_id = ?";
            my @params = ($art);
            glib_dbi_03::ejecutar_sql_param($base, $sql_insert, \@params);
            #lib_stdlog::print_err("exclude[$art]\n");
            if ($base->errstr) {
                my $error = lib_lang::get_message('artic_error_insert_temp_table'). "[$art]";
                lib_stdlog::print_err("$error [$sql_insert][$!][".$base->errstr."]\n");
                return '';
            }
        }

        return $nom_table;
    }
    return '';
}

sub call_purge_proc {
    return if ($DISABLE_PURGE_CACHE);
    return if (!$DO_PURGE);

    my $cmd = "/usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_purge_cache.pl $prontus_varglb::PRONTUS_ID >/dev/null 2>&1 &";
    lib_stdlog::print_err("purge[$cmd]\n");
    system $cmd;
    $DO_PURGE = 0;
}

sub get_html_ediciones {
    my ($dir_edics, $form_edic) = @_;

    # Generar cod. html correspondiente a la combo de ediciones
    my $html_ediciones = &get_edic_list($dir_edics, $form_edic);
    my $qty_base_ports = @{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}};
    # warn "qty_base_ports[$qty_base_ports]";
    if (($qty_base_ports <= 0) && ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI')) {
        $html_ediciones =~ s/<option value="base".*?<\/option>//s;    # eliminar de la combo la edic. base.
    }
    return $html_ediciones;
}

sub get_edic_list {
    my ($path_dir, $edic_selected) = @_;

    # Abre directorio.
    my $DIR;
    unless (opendir($DIR, $path_dir) ) {
        lib_stdlog::print_err("Can't opendir" . $path_dir . $!);
        return ();
    }
    my @entries = readdir($DIR);
    closedir $DIR;

    @entries = sort { $b cmp $a } (@entries);
    my ($lista, $nro_elem) = ('', 0);
    foreach my $entry (@entries) {
        if (substr($entry, 0, 1) ne '.') {
            if ((-d ("$path_dir/$entry"))) {
                my $nom_arch = $entry;

                my $seleccionado = '';
                if ($nom_arch eq $edic_selected) {
                    $seleccionado = 'selected="selected"';
                }
                $lista    = $lista . '<option value="' . $nom_arch . '" ' . $seleccionado . '>';
                $lista    = $lista . $nom_arch . '</option>';                                      # 01.i
                $nro_elem = $nro_elem + 1;
            }
        }
    }

    return $lista;
}

# recibe el nombre de una portada. Deduce el path al archivo XML correspondiente, y calcula el md5sum del archivo.
sub get_md5sum_xml_port {
    my $nombre_port = $_[0];

    # Deduce del nombre de la portada, el del xml.
    my $path_xml = $nombre_port;
    $path_xml =~ s|(\w+?)\.\w*$|/xml/$1\.xml|;
    $path_xml = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic/base$path_xml";

    # lib_stdlog::print_err("nombre_port: $nombre_port, path_xml: $path_xml.\n");
    if (-f $path_xml) {
        my $xml_port = glib_fildir_03::read_file($path_xml);
        return Digest::MD5::md5_hex($xml_port);
    }

    return 0;
}

# recibe una string con un tamaño de archivo posiblemente terminado en letras, devuelve el valor en bytes.
# convierte hasta terabytes.
sub convert_dec_to_bytes {
    my $max_upload_size = $_[0];

    my $kilo = 1024;
    my $mega = $kilo * $kilo;
    my $giga = $mega * $kilo;
    my $tera = $giga * $kilo;

    if ($max_upload_size =~ /^\d+$/) {
        # es un valor en bytes, lo devuelvo tal cual
        return $max_upload_size;
    } else {
        # convertimos a bytes.
        my %factores = (
            'B'     => 1,
            'K'     => $kilo,
            'M'     => $mega,
            'G'     => $giga,
            'T'     => $tera,
        );
        # un número, seguido de posibles espacios, luego las letras B, K, M, G, o T
        # seguidas opcionalmente por la letra B. E.g. 200 M, 300GB.
        my ($amount, $mag) = ($max_upload_size =~ /^(\d+)\s*([BKMGT])B?$/i);
        if ($factores{$mag}) {
            return $amount * $factores{$mag};
        } else {
            lib_stdlog::print_err("ERROR: lib_prontus::convert_dec_to_bytes, unknown format: '$max_upload_size'\n");
            return $amount;
        }
    }

}

sub carga_multitag_json {
    my $num_elem = keys(%{$MULTITAG{$prontus_varglb::PRONTUS_ID}});
    if ($num_elem) {
        return;
    }
    my $src_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/multitag.json";
    my $str_json = glib_fildir_03::read_file_bin($src_file, 1);

    if ($str_json =~ /^\{.*\}$/) {
        %{$MULTITAG{$prontus_varglb::PRONTUS_ID}} = %{JSON::decode_json($str_json)};
    }
}

sub updateSeoPort {
    my ($file, $url_seo) = @_;

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my ($nom, $ext) = lib_utils::split_nom_y_extension($file);
    my $port_seo_file = "$port_seo_dir/$nom.json";

    my %jsonSeo;
    if (-f $port_seo_file) {
        my $jsonSeoBuffer = glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %jsonSeo = %{$jsonSeoRef};

    }
    $jsonSeo{'_seo_url'} = $url_seo;

    glib_fildir_03::check_dir($port_seo_dir);
    glib_fildir_03::write_file($port_seo_file, JSON::to_json(\%jsonSeo));
}

# redirecciona a la "home" del cpan, a la que todos los usuarios tienen acceso
sub redirect_home {
    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_dashboard.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
    exit;
}

sub cache_active {
    if (defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'}) {
        return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'};
    } else {
        my $servers = (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}});
        if ($servers || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} eq 'SI') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'} = 'SI'
        } else {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'} = 'NO'
        }
    }
    return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'};
}

# crea token para velidar peticion posteriormente
sub gen_token {
    my $dir_data_token = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/users/token";
    my $token;
    eval { $token = UUID4::Tiny::create_uuid_string() };
    if ($@) {
        $token = glib_str_03::random_string(32);
    }

    while (-f "$dir_data_token/$token") {
        eval { $token = UUID4::Tiny::create_uuid_string()};
        if ($@) {
            $token = glib_str_03::random_string(32);
        }
    }
    glib_fildir_03::write_file("$dir_data_token/$token", $token);
    return $token;
}

sub call_dam2save {
    my $fullpath_artic = $_[0];
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);

    my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/dam/prontus_dam_ppart_save.pl $fullpath_artic >/dev/null 2>&1 &";
    lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
}

sub call_dropbox_backup {
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} eq 'SI') {
        my $params = $_[0];
        &dropbox_backup($params);
    }
}

sub call_list_regen {
    my $param_especif_list = $_[0];
    return if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PROCESO_INTERNO'} ne 'SI');
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);
    my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_cron_list.pl $prontus_varglb::PRONTUS_ID $param_especif_list >/dev/null 2>&1 &";

    lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
}

sub call_s3_delete {
    my $ts = $_[0];
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);
    my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_s3_erase.pl $prontus_varglb::PRONTUS_ID $ts >/dev/null 2>&1 &";
    lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
}

sub call_sitemap {
    return if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS'} ne 'SI');
    my $anio_mes = $_[0];
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    my $cmd = "perl $rutaScript/sitemaps/sitemap_pags.pl $prontus_varglb::DIR_SERVER $prontus_varglb::PRONTUS_ID $anio_mes >/dev/null 2>&1 &";
    lib_stdlog::print_err("[" . &glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
}

sub call_tagonomicas_regen {
    my $param_especif_tagonomicas = $_[0];
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);
    my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_tags_ports.pl $prontus_varglb::PRONTUS_ID $param_especif_tagonomicas >/dev/null 2>&1 &";

    lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
}

sub call_taxports_regen {
    my ($param_especif_taxport, $param_ts) = @_;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} eq 'NO') {
        $param_ts = $param_ts || '';
        my $pathnice = lib_utils::get_path_nice();
        $pathnice = "$pathnice -n19 " if ($pathnice);
        my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_cron_taxport.pl $prontus_varglb::PRONTUS_ID $param_especif_taxport $param_ts >/dev/null 2>&1 &";

        lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
        system $cmd;
    } else {
        lib_tax::borrar_taxport($param_especif_taxport);
    }
}

# Generar combo de tipos de articulos, obteniendo la informacion desde
# el hash global definido en el arch. de configuracion.
# Retorna : Lista de seleccion con datos, lista para imprimirla.
sub generar_select_fids {
    my %art_tipos    = %{$_[0]};
    my $plt          = $_[1];
    my $selected_fid = $_[2];

    my $lista = '';
    if ($plt =~ /<!--FID-OPTION-->(.*?)<!--\/FID-OPTION-->/s) {
        my $option = $1;
        foreach my $tipo (sort keys %art_tipos) {
            $lista .= $option;
            $lista =~ s/%%_fid_option_value%%/$tipo/g;
            $lista =~ s/%%_fid_label%%/$art_tipos{$tipo}/g;
            if ($selected_fid && $selected_fid eq $tipo) {
                $lista =~ s/%%_fid_selected%%/selected=\"selected\"/g;
            } else {
                $lista =~ s/%%_fid_selected%%//g;
            }
        }
        $plt =~ s/<!--FID-OPTION-->.*?<!--\/FID-OPTION-->/$lista/s;
        return $plt;
    } else {
        return '';
    }
}

sub parse_multivistas {
    my ($fila, $nom4vistas, $nom_loop) = @_;

    my @loops_mv;
    while ($fila =~ /<!--$nom_loop-->(.*?)<!--\/$nom_loop-->/sg) {
        push(@loops_mv, $1);
    }
    my $hide = 'hide-element';
    my @vistas = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};

    foreach my $loop_mv(@loops_mv) {
        my $count = 0;
        my $loops = '';
        foreach my $mv (@vistas) {
            # parsea nombre de la vista
            my $loop = $loop_mv;
            $loop =~ s/%%_nom_vista%%/$mv/g;

            # Parsea nombre de la categ, para esta vista
            my $nom = lib_prontus::get_nomtax_envista($mv, $nom4vistas);
            $nom = lib_encodes::escape_html($nom);

            $loop =~ s/%%_nom_item_vista%%/$nom/g;
            if ($count > 0) {
                $loop =~ s/%%_vista_style%%/$hide/g;
            } else {
                $loop =~ s/%%_vista_style%%//g;
            }
            $count++;
            $loops .= $loop;
        }

        $fila =~ s/\Q$loop_mv\E/$loops/sg;
    }
    if (scalar @vistas == 1) {
        $fila =~ s/<!--VISTA_FLECHAS-->.*?<!--\/VISTA_FLECHAS-->//sg;
    } else {
        $fila =~ s/<!--\/?VISTA_FLECHAS-->//g;
    }
    $fila =~ s|<!--/?\Q$nom_loop\E-->||g;

    return $fila;
}

# hace una request HTTP POST a la URL pasada.
# Parámetros:
# 0: str, la URL para la request
# 1: datos del form.
# retorna una respuesta o redirige a una página de error.
sub post_http {
    my $url = $_[0];
    my $form = $_[1];
    my $ua = LWP::UserAgent->new(keep_alive=>1);
    $ua->default_header('Content-Type' => 'application/x-www-form-urlencoded');
    $ua->timeout(20);

    my $response = $ua->post($url, $form);
    if ($response->is_success) {
        return $response->decoded_content;
    } else {
        lib_stdlog::print_err($response->as_string);
        return 'error';
    }
}

# valores de signal validos
# stop|reload_conf|clear_vars|reset_url_cache
sub send_signal {
    if ($prontus_varglb::FCGI_MODE) {
        my $signal = $_[0];
        # path de cgis-cpn para ejecutar scripts
        my $path_cgicpn = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";
        my $res = `perl $path_cgicpn/fcgi_fpm/prontus_signal_fcgi.pl $signal`;
    }
}

sub end_tasks {
    %ARTICS = ();
    %MULTITAG = ();

    &call_purge_proc();
}
END {
    &end_tasks();
}

$SIG{INT} = \&end_tasks;

1;
