#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#---------------------------------------------------------------
# subrutinas usadas para analizar y validar archivo robots.txt
#---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
#---------------------------
package lib_robots;

use strict;
use utf8;
use prontus_varglb;

sub existe_robots_txt {
    return -f "$prontus_varglb::DIR_SERVER/robots.txt";
}

# revisa un archivo robots.txt y verifica que no tenga reglas inválidas.
sub es_valido_robots_txt {
    my $url = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'}://$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}/robots.txt";
    my $robots = shift;
    my $return_raw = shift; # devolver array perl o html formateado

    my %rules;
    %rules = &parse_robots($url, $robots) if defined ($robots);
    if (!%rules) {
        return lib_lang::get_message('robots_no_rules');
    }
    my @results = (lib_lang::get_message('robots_valid_rules'));
    foreach my $type (keys %rules) {
        if ($type eq 'Sitemap') {
            push @results, "Sitemap: $rules{$type}[0]";
        } else {
            foreach my $ua (keys %{$rules{$type}}) {
                foreach my $rule (@{$rules{$type}{$ua}}) {
                    push @results, "$type $rule para $ua";
                }
            }
        }
    }
    if ($return_raw) {
        return @results;
    } else {
        my $formatted = &format_results(@results);
        return $formatted;
    }
}

# recibe un array de resultados y lo formatea a titular + listado.
sub format_results {
    my @results = @_;
    my $results = '<p>'.$results[0].'</p><ul class="list-group">';
    shift @results; # descarto el elemento de título
    foreach my $result (@results) {
        $results .= "<li class='list-group-item'>$result</li>";
    }
    $results .= "<ul>";
    return $results;
}

# parsea un archivo robots.txt
sub parse_robots {
    my $robot_txt_uri = shift;
    my $text = shift;

    $robot_txt_uri = URI->new($robot_txt_uri) if ($robot_txt_uri);

    my %result;
    # blank lines are significant, so turn CRLF into LF to avoid generating
    # false ones
    $text =~ s/\015\012/\012/g;

    my $ua_pattern;
    # split at \012 or \015
    for (split(/[\012\015]/, $text)) {
        # Lines containing only a comment are discarded completely, and
        # therefore do not indicate a record boundary
        next if /^\s*\#/;

        s/\s*\#.*//; # Remove comments at end-of-line

        if (/^\s*User-Agent\s*:\s*(.*)/i) {
            $ua_pattern = $1;
            $ua_pattern =~ s/\s+$//;
        } elsif (/^\s*Disallow\s*:\s*(.*)/i) {
            if (! defined $ua_pattern) {
                $ua_pattern = '*';
            }

            my $disallow = $1;
            $disallow =~ s/\s+$//;
            if (length $disallow) {
                if ($robot_txt_uri) {
                    my $ignore;
                    eval {
                        my $u = URI->new_abs($disallow, $robot_txt_uri);
                        $ignore++ if $u->scheme ne $robot_txt_uri->scheme;
                        $ignore++ if lc($u->host) ne lc($robot_txt_uri->host);
                        $ignore++ if $u->port ne $robot_txt_uri->port;
                        $disallow = $u->path_query;
                        $disallow = "/" unless length $disallow;
                    };
                    next if $@;
                    next if $ignore;
                }
                push @{$result{'Disallow'}{$ua_pattern}}, $disallow;
            } else {
                push @{$result{'Disallow'}{$ua_pattern}}, lib_lang::get_message('robots_complete_access') ;
            }
        } elsif (/^\s*Allow\s*:\s*(.*)/i) {
            if (! defined $ua_pattern) {
                $ua_pattern = '*';
            }

            my $allow = $1;
            $allow =~ s/\s+$//;
            if (length $allow) {
                if ($robot_txt_uri) {
                    my $ignore;
                    eval {
                        my $u = URI->new_abs($allow, $robot_txt_uri);
                        $ignore++ if $u->scheme ne $robot_txt_uri->scheme;
                        $ignore++ if lc($u->host) ne lc($robot_txt_uri->host);
                        $ignore++ if $u->port ne $robot_txt_uri->port;
                        $allow = $u->path_query;
                        $allow = "/" unless length $allow;
                    };
                    next if $@;
                    next if $ignore;
                }
                push @{$result{'Allow'}{$ua_pattern}}, $allow;
            }
        } elsif (/^\s*Sitemap\s*:\s*(.*)/i) {
            push @{$result{'Sitemap'}}, $1;
        }
    }
    return wantarray ? %result : \%result
}

1;
