#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO .
# -----------
# Biblioteca de rutinas genericas para Prontus Search.
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------
package lib_search;

use strict;
use utf8;

# #############################################
#  Busqueda binaria

# ---------------------------------------------------------------
# Busqueda binaria en archivos de texto ordenados. Basado en Search::Dict
#
#  uso: binsearch (path_al_archivo,texto_buscado,tipo_de_busqueda)
#
# Asume que existe una linea por registro, donde el primer item es el indice,
# y la linea es de la forma:
#
# id=data|data|data|...
#
# o de la forma:
#
# id|data|data|data|...
#
# La comparacion es numerica o por string dependiendo de la instruccion de comparacion
# usada en las lineas marcadas 'CMP':
#   ($linea > $key) es para numeros (tipo_de_busqueda = 'num')
#   ($linea gt $key) es para strings (tipo_de_busqueda = 'str')
#
# Retorna '' si la busqueda falla.
sub binsearch {
    my($archivo,$key,$type) = @_;
    if (! -f $archivo) { return ''; }
    open my $DATAFILE, '<', $archivo;
    my $fh = *$DATAFILE;
    my(@stats,$idx);
    if (!( @stats = stat($fh) )) {
        close $fh;
        return '';
    }
    my($size, $blksize) = @stats[7,11];
    $blksize ||= 8192;   # Largo del bloque a leer
    $key =~ s/[=|]//g; # Excluye el separador, si estuviera.
    $key = lc $key;
    # Escoge el bloque correcto.
    my($min, $max) = (0, int($size / $blksize)); # Se ubica en los extremos del archivo.
    my ($mid,$linea);
    while ($max - $min > 1) {
        # print "debug: min=$min max=$max linea=$linea key=$key<br>\n";
        $mid = int(($max + $min) / 2);
        if (! seek($fh, $mid * $blksize, 0) ) { # Se ubica en el punto medio.
            close $fh;
            return '';
        }
        if ($mid > 0) { <$fh>; } # Descarta linea por si quedo en la mitad de ella.
        $linea = <$fh>;      # Lee la linea siguiente.
        # print "debug: testing_line=$linea<br>\n";
        chomp $linea;
        last if ($linea eq '');
        $linea =~ s/[=|].*$//s; # Se queda con el primer elemento.
        # print "debug: primer elemento=[$linea] key=$key type=$type<br>\n";
        # CMP. Compara y ajusta uno de los limites.
        if ($type eq 'str') {
            # print "debug: COMPARA STRING<br>\n";
            if ($linea lt $key) {
                $min = $mid;
            }else{
                $max = $mid;
            }
        }else{
            # print "debug: COMPARA NUMERO<br>\n";
            if ($linea < $key) {
                $min = $mid;
            }else{
                $max = $mid;
            }
        }
    } # while
    # Se posiciona y retorna el bloque correcto. Retrocede uno por si las moscas.
    $min--;
    if ($min < 0) { $min = 0; }
    # print "debug: min=[$min]<br>\n";
    if (! seek($fh, $min * $blksize, 0) ) {
        close $fh;
        return '';
    }
    if ($min > 0) { <$fh>; } # Descarta linea por si quedo en la mitad de ella.
    while ($linea = <$fh>) {
        chomp $linea;
        $idx = $linea;
        $idx =~ s/[=|].*$//s; # Se queda con el primer elemento.
        # print "debug: linea=[$linea] idx=[$idx] key=[$key]<br>\n";
        if ($type eq 'str') {
            last if  ($idx ge $key);
        }else{
            last if  ($idx >= $key);
        }
    }
    close $fh;
    if ($idx eq $key) {
        # print "debug resultado: linea=[$linea]<br>\n";
        return $linea;
    }else{
        # print "debug resultado: null<br>\n";
        return '';
    }
} # binsearch

# #############################################
#  Especiales del buscador

# --------------------------------------------------------------- #
# Retorna un hash con el indice de palabras.
sub get_palabras {
    my ($palabras_idx,$p_ref) = @_;
    my ($id,$palabra);
    if (!-f $palabras_idx) { return; }
    open my $IN, '<', $palabras_idx;
    while (<$IN>) {
        chomp;
        ($palabra,$id) = split(/=/,$_);
        $$p_ref{$palabra} = $id;
    }
    close $IN;
} # get_palabras

# Retorna un hash con las variables relevantes del cfg.
sub get_config {
    my $configfile = $_[0];
    my %pairs = ();

    my $buffer = glib_fildir_03::read_file($configfile);
    my @lines = split(/[\r\n]/,$buffer);
    foreach my $line (@lines) {
        if ( ($line ne '') && ($line !~ /^\s*#/) && ($line =~ /=/)) {
            $line =~ s/#.*$//; # Elimina comentarios.
            my ($name,$value) = split(/=/,$line);
            $name = glib_str_03::trim($name);
            $value = glib_str_03::trim($value);
            if ($name ne '') { $pairs{$name} = $value; }
        }
    }

    $pairs{'SEARCH_TIPO_PAGINACION'} = 0 unless($pairs{'SEARCH_TIPO_PAGINACION'});
    $pairs{'SEARCH_PAGCORTA_MAXPAGS'} = 5 unless($pairs{'SEARCH_PAGCORTA_MAXPAGS'});
    $pairs{'SEARCHTIPS_MAXRESULT'} = 5 unless($pairs{'SEARCHTIPS_MAXRESULT'});
    $pairs{'SEARCHTIPS_MINLEN'} = 1 unless($pairs{'SEARCHTIPS_MINLEN'});
    $pairs{'SEARCHTIPS_DURACION_CACHE'} = 300 unless($pairs{'SEARCHTIPS_DURACION_CACHE'});
    $pairs{'SEARCHTIPS_MAXREQUESTXIP'} = 500 unless($pairs{'SEARCHTIPS_MAXREQUESTXIP'});

    return %pairs;
} # get_config

# #############################################
#  Utilitarios para el manejo de strings

# Extrae todo caracter de control de un string.
sub no_controls {
    my($txt) = @_;
    $txt =~ s/[\x00-\x1f]//sg;
    return $txt;
} # no_controls

# Extrae todo caracter no alfanumerico de un string.
sub solo_alfa {
    my($txt) = @_;
    $txt =~ s/[^0-9a-z]//sig;
    return $txt;
} # solo_alfa

# Extrae todo caracter no alfanumerico o guiones o punto de un string.
sub solo_alfa2 {
    my($txt) = @_;
    $txt =~ s/[^0-9a-z\-\_\.]//sig;
    return $txt;
} # solo_alfa2

# Extrae todo caracter no hexadecimal de un string.
sub solo_hex {
    my($txt) = @_;
    $txt =~ s/[^0-9a-f]//sig;
    return $txt;
} # solo_hex

# Reemplaza tildes UTF-8 por letras normales.
sub notildesUtf8 {
    my($toencode) = $_[0];
    # se convierte a octetos ya que se buscan los valores hex y no los codigos unicode
    # también es mas rápido
    utf8::encode($toencode);
    $toencode =~ s/\xC3\xA1/a/sg;
    $toencode =~ s/\xC3\x81/A/sg;
    $toencode =~ s/\xC3\xA0/a/sg;
    $toencode =~ s/\xC3\x80/A/sg;
    $toencode =~ s/\xC3\xA2/a/sg;
    $toencode =~ s/\xC3\x82/A/sg;
    $toencode =~ s/\xC3\xA4/a/sg;
    $toencode =~ s/\xC3\x84/A/sg;

    $toencode =~ s/\xC3\xA9/e/sg;
    $toencode =~ s/\xC3\x89/E/sg;
    $toencode =~ s/\xC3\xA8/e/sg;
    $toencode =~ s/\xC3\x88/E/sg;
    $toencode =~ s/\xC3\xAA/e/sg;
    $toencode =~ s/\xC3\x8A/E/sg;
    $toencode =~ s/\xC3\xAB/e/sg;
    $toencode =~ s/\xC3\x8B/E/sg;

    $toencode =~ s/\xC3\xAD/i/sg;
    $toencode =~ s/\xC3\x8D/I/sg;
    $toencode =~ s/\xC3\xAC/I/sg;
    $toencode =~ s/\xC3\x8C/i/sg;
    $toencode =~ s/\xC3\xAE/I/sg;
    $toencode =~ s/\xC3\x8E/i/sg;
    $toencode =~ s/\xC3\xAF/I/sg;
    $toencode =~ s/\xC3\x8F/i/sg;

    $toencode =~ s/\xC3\xB3/o/sg;
    $toencode =~ s/\xC3\x93/O/sg;
    $toencode =~ s/\xC3\xB2/o/sg;
    $toencode =~ s/\xC3\x92/O/sg;
    $toencode =~ s/\xC3\xB4/o/sg;
    $toencode =~ s/\xC3\x94/O/sg;
    $toencode =~ s/\xC3\xB6/o/sg;
    $toencode =~ s/\xC3\x96/O/sg;

    $toencode =~ s/\xC3\xBA/u/sg;
    $toencode =~ s/\xC3\x9A/U/sg;
    $toencode =~ s/\xC3\xB9/u/sg;
    $toencode =~ s/\xC3\x99/U/sg;
    $toencode =~ s/\xC3\xBB/u/sg;
    $toencode =~ s/\xC3\x9B/U/sg;
    $toencode =~ s/\xC3\xBC/u/sg;
    $toencode =~ s/\xC3\x9C/U/sg;

    $toencode =~ s/\xC3\xB1/n/sg;
    $toencode =~ s/\xC3\x91/N/sg;
    $toencode =~ s/\xC3\xA7/c/sg;
    $toencode =~ s/\xC3\x87/C/sg;

    $toencode =~ s/&(.)acute;/$1/g;
    $toencode =~ s/&(.)tilde;/$1/g;
    $toencode =~ s/&(.)uml;/$1/g;
    $toencode =~ s/&(.)circ;/$1/g;
    $toencode =~ s/&(.)grave;/$1/g;

    $toencode =~ s/&[^;]+;//g; # Elimina toda otra entidad.

    # se convierte a representacion interna el resultado, para seguir trabajando con utf8
    utf8::decode($toencode);

    return $toencode;
} # notildesUtf8
# -----------------------------------------------------------------------#
# Escapea caracteres para formar queries html.
sub escapehtml {
    my($toencode) = $_[0];
    $toencode =~ s/([^a-zA-Z0-9!, ])/sprintf('%%%02x',ord($1))/ge;
    $toencode =~ tr/ /+/;       # Transforma los espacios en mas.
    return $toencode;
} # escapehtml

# Formatea un valor entero.
sub formatInteger {
    my($real) = $_[0];
    my($i,$d,$out);
    $real =~ s/\..*$//;
    # Agrega los puntos en los miles.
    $out = '';
    my @real = split(//,$real);
    $d = 0;
    for ($i=$#real; $i >= 0; $i--) {
        $out = $real[$i] . $out;
        $d++;
        if (($d % 3 == 0) && ($i > 0) && ($real[ $i - 1 ] =~ /[0-9]/)) { $out = '.' . $out; }
    }
    return $out;
} # formatInteger

# #############################################
# Fechas

# Retorna la fecha de hoy en formato ISO aaaammdd.
sub fecha_iso {
    # (0: $sec, 1: $min, 2: $hour, 3: $mday, 4: $mon, 5: $year, 6: $wday, 7: $yday, 8: $isdst)
    my(@fecha) = localtime(time);
    my($ano,$mes,$dia) = ($fecha[5],$fecha[4],$fecha[3]);
    if ($ano < 2000) { $ano += 1900; }
    $mes = sprintf('%02d',($mes + 1));
    $dia = sprintf('%02d',$dia);
    return "$ano$mes$dia";
} # fecha_iso

# Retorna la fecha y hora de hoy en formato ISO aaaammdd.
sub timestamp_iso {
    # (0: $sec, 1: $min, 2: $hour, 3: $mday, 4: $mon, 5: $year, 6: $wday, 7: $yday, 8: $isdst)
    my(@fecha) = localtime(time);
    my($ano,$mes,$dia,$hra,$min,$seg) = ($fecha[5],$fecha[4],$fecha[3],$fecha[2],$fecha[1],$fecha[0]);
    if ($ano < 2000) { $ano += 1900; }
    $mes = sprintf('%02d',($mes + 1));
    $dia = sprintf('%02d',$dia);
    $hra = sprintf('%02d',$hra);
    $min = sprintf('%02d',$min);
    $seg = sprintf('%02d',$seg);
    return "$ano$mes$dia$hra$min$seg";
} # timestamp_iso

# Toma una fecha d/m/a o aaaa-mm-dd y la retorna en formato ISO aaaammdd.
sub fecha2iso {
    my $fecha = $_[0];
    my($dia,$mes,$ano);
    if ($fecha =~ /\//) { # Separador = /
        $fecha =~ s/[^0-9\/]//g;
        ($dia,$mes,$ano) = split(/\//,$_[0]);
        if ($dia =~ /\d{4}/) { # era fecha en formato aaaa/mm/dd, damos vuelta.
            ($dia, $ano) = ($ano, $dia);
        }
    } elsif ($fecha =~ /\-/) { # Separador = -
        $fecha =~ s/[^0-9\-]//g;
        ($dia,$mes,$ano) = split(/\-/,$_[0]);
        if ($dia =~ /\d{4}/) { # era fecha en formato aaaa-mm-dd, damos vuelta.
            ($dia, $ano) = ($ano, $dia);
        }
    } else {
        return '';
    }
    if (($ano < 2000) && ($ano < 100)) { $ano += 2000; } # Asume solo anos despues del ano 2000.
    $mes = sprintf('%02d',$mes);
    $dia = sprintf('%02d',$dia);
    $ano = sprintf('%04d',$ano);
    return "$ano$mes$dia";
}

# Toma una fecha ISO y la retorna en formato dd/mm/aaaa.
sub iso2fechacorta {
    my $fecha = $_[0];
    return substr($fecha,6,2) . '/' . substr($fecha,4,2) . '/' . substr($fecha,0,4);
} # iso2fechacorta

##########################################################################
# Rutinas de Prontus copiadas aqui para mantener bajo acoplamiento.

sub get_listado_mis_busquedas {
    my $user = $_[0];
    my $dir = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_MY_SEARCH/user$user";
    my @busquedas;

    if(-d $dir) {
        my @entries = glib_fildir_03::lee_dir($dir);
        foreach my $file (@entries) {
            next unless($file =~ /(\d{14})\.txt/);
            my $ts = $1;

            my $strjson = glib_fildir_03::read_file_bin("$dir/$file");
            my $hashtemp = JSON::decode_json($strjson);

            my %hash;

            $hash{'info'} = {'ts' => $ts, 'nombre' => $hashtemp->{'name_search'}};
            delete $hashtemp->{'name_search'};
            $hash{'params'} = $hashtemp;

            push @busquedas, \%hash
        }
    }
    return \@busquedas;
}

sub get_file_mis_busquedas {
    my $user = shift;
    my $myts = shift;
    my $ts_busqueda;
    if($myts) {
        $ts_busqueda = $myts;
    } else {
        $ts_busqueda = glib_hrfec_03::get_dtime_pack4();
    }
    my $file = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_MY_SEARCH/user$user";
    glib_fildir_03::check_dir($file);
    $file = $file.'/'.$ts_busqueda.'.txt';
    return $file;
}

sub get_total_mis_busquedas {
    my $user = shift;
    my $dir = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_MY_SEARCH/user$user";
    if(-d $dir) {
        my @entries = glib_fildir_03::lee_dir($dir);
        my $total = 0;
        foreach my $file (@entries) {
            $total++ if($file =~ /\d{14}\.txt/);
        }
        return $total;
    }
    return 0;
}

sub garbage_collector {
    my ($dir_search_tips_cache, $duration) = $_[0];
    my @listado = glob("$dir_search_tips_cache/*");
    foreach my $dir (@listado) {
        my @archivos = glob("$dir/*.txt");
        foreach my $archivo (@archivos) {
            my $mtime = (stat($archivo))[9];
            if ((time - $mtime) > $duration) {
                unlink $archivo;
            }
        }
    }
}

sub search_cache {
    my ($dir_search_tips_cache, $key, $duration) = @_;
    my $first_letter = substr($key, 0, 1);

    glib_fildir_03::check_dir("$dir_search_tips_cache/$first_letter");

    my $file = "$dir_search_tips_cache/$first_letter/$key.txt";
    if (-f $file) {
        my $mtime = (stat($file))[9];
        if ((time - $mtime) > $duration) {
            unlink $file;
        } else {
            my $buffer = glib_fildir_03::read_file($file);
            return $buffer;
        }
    }

    return;
}

sub guardar_cache {
    my $key = $_[0];
    my $buffer = $_[1];
    my $dir_search_tips_cache = $_[2];
    my $first_letter = substr($key, 0, 1);
    my $file = "$dir_search_tips_cache/$first_letter/$key\.txt";

    glib_fildir_03::check_dir("$dir_search_tips_cache/$first_letter");
    glib_fildir_03::write_file($file, $buffer);
}

1;
