#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Funciones centralizadas para el sistema de tags
#---------------------------------------------------------------
# Requiere
# use lib_prontus;
# use glib_dbi_03;
# use glib_fildir_03;

package lib_tags;
use strict;

our $MAX_ARTICS_PER_SEARCH  = 20;
our $MAX_TAGS_SEARCH_RESULT = 20;
our $MAX_LIMIT = 50; # Limite maximo permitido para la generación de relacionados.
our %CFG_FIL_TAGPORT = ();

sub get_max_tags_search_result {
    return $MAX_TAGS_SEARCH_RESULT;
}

# Verifica que el nombre no exista en la tabla.
sub check_nom {
    my ($db, $id, $nombre) = @_;
    my @params;
    my $sql;

    my $tags_table = lib_utils::get_table('tags');
    if ($id) {
        # revisa si hay otros tags excluyendo el actual con el mismo nombre
        $sql = "select count(tags_tag) from $tags_table where lower(tags_tag) = lower(?) and tags_id <> ?";
        push @params, $nombre;
        push @params, $id;
    } else {
        # tag nuevo revisa ya existe el tag
        $sql = "select count(tags_tag) from $tags_table where lower(tags_tag) = lower(?)";
        push @params, $nombre;
    }

    my $cant;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($db, $sql, \@params, \$cant);

    $salida->fetch;
    $salida->finish;

    if ($cant > 0) {
        return lib_lang::get_message('global_item_name_exists');
    }

    return '';
}

sub get_tags_for_fid {
    my ($base, $ts, $tags_order) = @_;
    my @params;

    # tags asignados
    my $tagsart_table = lib_utils::get_table('tagsart');
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tagsart_idtags, tags_tag from $tagsart_table join $tags_table on tags_id=tagsart_idtags where tagsart_idart = ?";
    push @params, $ts;

    if ($tags_order) {
        $sql .= " order by field(tags_id, ?);";
        push @params, $tags_order;
    } else {
        $sql .= " order by tags_tag asc;";
    }

    my ($idtag_asignado, $tag_name);
    my $tags_query = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($idtag_asignado, $tag_name));

    # Se leen los tags de la BD
    my $tags_asignados = '';

    while ($tags_query->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_name);
        $tag_name = lib_encodes::escape_xml($tag_name);
        $tags_asignados = $tags_asignados . "$idtag_asignado|$tag_name,";
    }

    $tags_asignados =~ s/,$//;
    $tags_query->finish;

    return $tags_asignados;
}

sub get_all_tags {
    my $base = $_[0];
    my %hash_tags;
    my ($idtag, $tag_name);
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tags_id, tags_tag from $tags_table where tags_mostrar = 1 order by tags_tag asc";
    my $tab_output = glib_dbi_03::ejecutar_sql_bind($base, $sql, \($idtag, $tag_name));

    while ($tab_output->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_name);
        $hash_tags{$idtag} = $tag_name;
    }
    return %hash_tags;
}

sub get_tag_noms {
    my ($base, $tag_id) = @_;

    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tags_tag, tags_nom4vistas from $tags_table where tags_id = ?";
    my @params = ($tag_id);

    my %tag_noms;
    my $tag;
    my $nom4vistas;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($tag, $nom4vistas));
    $salida->fetch;
    $salida->finish;

    if ($tag) {
        glib_str_03::correct_encode_utf8_string($tag);
        glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tag_noms{''} = lib_encodes::escape_html($tag);    # nombre en la vista default o ppal

        my @nom_vistas = split(/\n/, $nom4vistas);
        foreach my $item (@nom_vistas) {
            my ($vista, $nom) = split(/\t/, $item);

            $tag_noms{$vista} = lib_encodes::escape_html($nom);    # nombre en cada vista
        }
    }

    return %tag_noms;
}

sub get_tags_noms_hash {
    my ($base, $tags_id) = @_;
    my @tags_id_arr = split("," , $tags_id);
    my %tags_noms;

    if (!$tags_id) {
        return %tags_noms;
    }

    my $placeholders = ', ?' x scalar @tags_id_arr;
    $placeholders = substr $placeholders, 2;
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tags_id, tags_tag, tags_nom4vistas from $tags_table where tags_id in ($placeholders)";
    my $tag;
    my $tag_id;
    my $nom4vistas;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@tags_id_arr, \($tag_id, $tag, $nom4vistas));

    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag);
        &glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tags_noms{$tag_id}{''} = lib_encodes::escape_html($tag);

        my @nom_vistas = split(/\n/, $nom4vistas);
        foreach my $item (@nom_vistas) {
            my ($vista, $nom) = split(/\t/, $item);
            $tags_noms{$tag_id}{$vista} = lib_encodes::escape_html($nom);
        }
    }

    $salida->finish;

    return %tags_noms;
}

# genera un listado de tags en formato json.
sub generar_json {
    my $bd = $_[0];
    my ($sql, $salida, $tag_id, $tag_nom);

    my %tags_url = ();
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
        my ($salida2, $urlf5_subtypeid, $urlf5_file, $urlf5_url);
        my $urlf5_table = lib_utils::get_table('urlf5');
        my $sql2 = "select urlf5_subtypeid, urlf5_file, urlf5_url from $urlf5_table where urlf5_type = ? and urlf5_prontus = ?";
        my @params = ('tag', $prontus_varglb::PRONTUS_ID);
        $salida2 = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql2, \@params, \($urlf5_subtypeid, $urlf5_file, $urlf5_url));

        while ($salida2->fetch) {
            $urlf5_file =~ s|/$prontus_varglb::PRONTUS_ID||;
            $tags_url{$urlf5_subtypeid}{$urlf5_file} = $urlf5_url;
        }
        $salida2->finish;
    }

    my $tags_table = lib_utils::get_table('tags');
    $sql = "select tags_id, tags_tag from $tags_table where tags_mostrar = 1";
    $salida = glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($tag_id, $tag_nom));
    my @tags;
    while ($salida->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_nom);
        my %urls = ();
        if (defined($tags_url{$tag_id})) {
            %urls = %{$tags_url{$tag_id}};
        }
        push @tags, {'id' => $tag_id, 'nom' => $tag_nom, 'urls' => \%urls};
    }

    $salida->finish;

    my $str_json = JSON::encode_json(\@tags);
    utf8::decode($str_json);

    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    glib_fildir_03::check_dir($dst_path);
    glib_fildir_03::write_file("$dst_path/tags.json", $str_json);
}

# Genera un archivo con los relacionados por tags.
sub generar_relacionados_tags {
    my ($tag_list_str, $dst_dir, $ts, $base, $mv) = @_;
    # borra extension, en caso de que venga con ella
    $ts =~ s/\..+$//;

    my $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAGS;
    # si path nuevo no existe, usamos path antiguo
    if (! -d $ruta_dir) {
        $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAGS_OLD;
    }

    if ($mv) {
        $ruta_dir = "$ruta_dir-$mv";
        $dst_dir =~ s/(\d{8})\/pags/$1\/pags-$mv/;
    }

    my @lisdir;
    @lisdir = glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);

    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..
    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            my $pagina = glib_fildir_03::read_file("$ruta_dir/$k");

            my $loop = '';
            if ($pagina =~ /%%loop%%(.*?)%%\/loop%%/s) {
                $loop = $1;
            }

            # Generar listas con los articulos especificados manualmente
            my ($lista) = &make_lista_tag($ts, $tag_list_str, $loop, $base, $mv);
            # lib_stdlog::print_err("lista[$lista]\n");

            $pagina =~ s/%%_num_relac%%//g;
            # Parsear y escribir paginas para incluir en articulos.
            &parse_and_write_tag($lista, $pagina, $dst_dir, $ts, $k);
        }
    }
}

sub parse_and_write_tag {
    my ($lista, $pagina, $dst_dir, $ts, $nomfile) = @_;
    $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;

    $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;

    $pagina = lib_parser::parse_globals($pagina, '');

    my ($crlf) = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

    $pagina =~ s/$crlf/\x0a/sg;

    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html y marcas prontus sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    glib_fildir_03::check_dir($dst_dir);
    glib_fildir_03::write_file("$dst_dir/" . $ts . '_tags_' . $nomfile , $pagina);
}

# genera la query que buscará los tags para relacionados y devuelve los resultados.
sub make_lista_tag {
    my ($art_ts, $tag_ids_str, $loop, $base, $mv) = @_;

    my $dthr_system = glib_hrfec_03::get_dtime_pack4();
    my $hhmm_system = substr($dthr_system, 0, 8);
    my $dt_system = substr($dthr_system, 8, 4);

    my $art_table = lib_utils::get_table('art');
    my $tagsart_table = lib_utils::get_table('tagsart');
    my ($filtros, $params_ref) = &genera_filtros_tag($art_ts, $tag_ids_str, $dt_system, $hhmm_system);
    my $sql = "select $tagsart_table.tagsart_idart, $tagsart_table.tagsart_idtags, $art_table.art_fechaphorap from $tagsart_table inner join $art_table on $tagsart_table.tagsart_idart=$art_table.art_id where $filtros group by $tagsart_table.tagsart_idart order by art_fechaphorap desc limit ?";
    my @params = @{$params_ref};

    my $tot_artics = &get_tot_artics($filtros, $params_ref, $base, $MAX_LIMIT);

    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $MAX_LIMIT;

    my ($art_id, $id_tag, $art_fechap);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_id, $id_tag, $art_fechap));
    my $filas = '';
    my $loopcounter = 0;

    while ($salida->fetch) {
        $loopcounter++;
        my $fila = lib_parser::generar_fila($art_id, $loop, $loopcounter, $tot_artics, $mv, 1);
        $filas .= $fila;
    }
    $salida->finish;

    return $filas;
}

# Genera segmento variable del sql para encontrar los articulos.
sub genera_filtros_tag {
    my ($art_ts, $tag_ids, $dt_system, $hhmm_system) = @_;
    my @params;
    my $filtros = ' 1 = 1 ';
    my $tagsart_table = lib_utils::get_table('tagsart');
    if ($tag_ids) {
        $filtros .= " and (";
        my $i = 0;
        foreach my $id_tag (split(/,\s*/ , $tag_ids)){
            if ($i == 0) {
                $filtros = $filtros." $tagsart_table.tagsart_idtags = ?";
            } else {
                $filtros = $filtros." or $tagsart_table.tagsart_idtags = ?";
            }
            push @params, $id_tag;
            $i++;
        }
        $filtros .= ")";

        $filtros = $filtros." and $tagsart_table.tagsart_idart <> ?";
        push @params, $art_ts;

        $filtros .= " and (art_fechaphorap <= ?) ";
        push @params, "$dt_system$hhmm_system";
        $filtros .= " and (art_alta = '1') ";

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $filtros .= " and ( (art_fechaehorae >= ?) or ( (art_fechaehorae < ?) and (art_soloportadas = '1') ) )";

            push @params, "$dt_system$hhmm_system";
            push @params, "$dt_system$hhmm_system";
        }

        # procesamos solamente los artículos que generan vistas.
        my %fids_hash = lib_helper::get_fids_with_view();
        my @fids_con_vista;
        foreach my $fid (keys %fids_hash) {
            push @fids_con_vista, "'$fid'";
        }
        my $fids_string = join(',', @fids_con_vista);
        $filtros .= " and art_tipoficha in ($fids_string) " if $fids_string;
    }
    return ($filtros, \@params);

}

sub get_tot_artics {
    my ($filtros, $params_ref, $base, $maxartics) = @_;
    my @params = @{$params_ref};

    my $art_table = lib_utils::get_table('art');
    my $tagsart_table = lib_utils::get_table('tagsart');
    my $sql = "select count(art_id) from $art_table inner join $tagsart_table on $art_table.art_id = $tagsart_table.tagsart_idart ";

    if ($filtros ne '') {
        $sql .= "where $filtros";
    }

    my ($count_art);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_art));
    $salida->fetch;
    $salida->finish;

    $count_art = '0' if $count_art eq '';
    $count_art = $maxartics if ($count_art > $maxartics);

    return $count_art;
}

sub make_mapa_tags {
    my ($mv, $bd) = @_;
    my $dir_plt = 'pags';
    my @postprocesos;

    $dir_plt .= "-$mv" if ($mv);

    my $ruta_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}/extra/tags/$dir_plt";
    my @lisdir;
    if (-d $ruta_dir) {
        @lisdir = glib_fildir_03::lee_dir($ruta_dir);
        # Elimina directorios . y ..
        @lisdir = grep !/^\./, @lisdir;
    }

    foreach my $plt_file (@lisdir) {
        next if (! -f "$ruta_dir/$plt_file");

        my $mapa_plt = glib_fildir_03::read_file("$ruta_dir/$plt_file");
        $mapa_plt = lib_parser::ajusta_crlf($mapa_plt);

        my $loop_tags = '';
        if ($mapa_plt =~ /%%loop_tags%%(.*?)%%\/loop_tags%%/s) {
            $loop_tags = $1;
        }

        my %config = &get_config_mapa_tags($mapa_plt);
        my $output = &get_arbol_mapa_tags($loop_tags, $bd, $mv, \%config);

        $mapa_plt =~ s/%%loop_tags%%.*?%%\/loop_tags%%/$output/s;

        $mapa_plt = lib_prontus::parser_custom_function($mapa_plt);

        $mapa_plt =~ s/%%_server_name%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}/g;
        $mapa_plt =~ s/%%_prontus_id%%/$prontus_varglb::PRONTUS_ID/g;

        my ($_plt_nom, $_plt_ext) = lib_utils::split_nom_y_extension($plt_file);
        $mapa_plt =~ s/%%_plt_nom%%/$_plt_nom/g;
        $mapa_plt =~ s/%%_plt_ext%%/$_plt_ext/g;
        $mapa_plt =~ s/%%_vista%%/$mv/g;
        $mapa_plt =~ s/%%_mv%%/$mv/g;

        # rescata el post_proceso
        if ($mv eq '') {
            while ($mapa_plt =~ /<!--POST_PROCESO=(.+?)-->/g) {
                push(@postprocesos, $1);
            }
        }
        $mapa_plt =~ s/<!--POST_PROCESO=.+?-->//g;

        # Borra marcas no sustituidas
        $mapa_plt =~ s/%%.+?%%//g;

        glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/tags/$dir_plt");
        my $dst_mapa = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/tags/$dir_plt/$plt_file";
        glib_fildir_03::write_file($dst_mapa, $mapa_plt);
        lib_prontus::purge_cache($dst_mapa) if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_MAPA'} eq 'SI');
    }

    # ejecutamos los postprocesos asociados
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    foreach my $pproc (@postprocesos) {
        $pproc =~ s/[#;&<>|~]+//g;
        my $cmd = "$rutaScript/$pproc $prontus_varglb::PRONTUS_ID $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} >/dev/null 2>&1 &";
        lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "][mapa_tags][PPROC][$cmd]\n");
        system $cmd;
    }
}

sub get_config_mapa_tags {
    my $plt = $_[0];
    my %config = (
        'order' => 'tags_id',
        'solo_con_artics' => 0
    );

    while ($plt =~ /<!--\s*CONFIG\s*(\w+)\s*=\s*(.*?)\s*-->/sg) {
        my $name = $1;
        my $value = $2;

        if ($name eq 'ORDERBY') {
            if ($value eq 'name') {
                $config{'order'} = 'tags_tag';
            }

            if ($value eq 'id') {
                $config{'order'} = 'tags_id';
            }
        }

        if ($name eq 'SOLO_CON_ARTICS') {
            if ($value =~ /^\d+$/) {
                $config{'solo_con_artics'} = $value;
            }
        }
    }

    return %config;
}

sub get_arbol_mapa_tags {
    my $loop =$_[0];
    my $bd = $_[1];
    my $mv = $_[2];
    my %config = %{$_[3]};
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tags_id, tags_tag, tags_nom4vistas, tags_mostrar, tags_port from $tags_table";

    $sql .= " order by ?";
    my @params = ($config{'order'});

    my ($tags_id, $tags_tag, $tags_nom4vistas, $tags_mostrar, $tags_port);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($tags_id, $tags_tag, $tags_nom4vistas, $tags_mostrar, $tags_port));
    my $mapa_tags;
    my $output;

    while ($salida->fetch) {
        $mapa_tags = $loop;

        $tags_tag = lib_prontus::get_nomtax_envista($mv, $tags_nom4vistas, $tags_tag) if ($mv);
        $tags_tag = lib_encodes::escape_html($tags_tag);
        $tags_port = lib_encodes::escape_html($tags_port);

        $mapa_tags =~ s/%%_id%%/$tags_id/g;
        $mapa_tags =~ s/%%_nom%%/$tags_tag/g;
        $mapa_tags =~ s/%%_mostrar%%/$tags_mostrar/g;
        my $tags_fixedurl = &get_tag_link($tags_port, $mv);
        $mapa_tags =~ s/%%_fixed_url%%/$tags_fixedurl/g;

        my %claves = ('_mostrar' => $tags_mostrar);
        my %nodata;

        $mapa_tags = lib_parser::procesa_condicional($mapa_tags, \%claves, \%nodata);

        $output .= $mapa_tags;
    }

    $salida->finish;

    return $output;
}

# entrega link a portada prontus o url para un tag.
sub get_tag_link {
    # mv: nombre de la vista
    my ($nom, $mv)  = @_;

    if ($nom) {
        # url
        if ($nom =~ /^(https?:\/\/|\/)/i) {
            return $nom;
        } else {
            # nombre de una port => linkear a port de la edic. vigente o bien de la base.
            my $edic;
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
                $edic = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}/ed_vigente.txt");
            } else {
                $edic = 'base';
            }
            my $dir_port = $prontus_varglb::DIR_PORT;
            # nombre de la vista.
            if ($mv) {
                $dir_port = $dir_port . '-' . $mv;
            }
            return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . "/$edic" . $dir_port . "/$nom";
        }
    } else {
        return '';
    }
}

# inserta tag en bd, si no viene id se crea uno nuevo
sub insert_tag {
    my ($bd, $id, $nom, $mostrar, $nom4vistas, $port) = @_;

    my $tags_table = lib_utils::get_table('tags');
    my $sql = "insert into $tags_table (tags_id, tags_tag, tags_mostrar, tags_nom4vistas, tags_port) values(?, ?, ?, ?, ?)";
    my @params = ($id, $nom, $mostrar, $nom4vistas, $port);
    my $id_value = glib_dbi_03::insert_dev_id_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return 0;
    } else {
        return $id_value;
    }
}

# actualiza valores asociados al tag id
sub update_tag {
    my ($bd, $id, $nom, $nom4vistas, $port) = @_;

    my $tags_table = lib_utils::get_table('tags');
    my $sql = "update $tags_table set tags_tag = ?, tags_nom4vistas = ?, tags_port = ? where tags_id = ?";
    my @params = ($nom, $nom4vistas, $port, $id);
    glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return 'tags_guardar_update_error';
    } else {
        return '';
    }
}

sub change_state_tag {
    my ($bd, $new_st, $id_tag) = @_;

    # Cambia el estado de un Tag.
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "update $tags_table set tags_mostrar = ? where tags_id = ? ";
    my @params = ($new_st, $id_tag);

    my $result = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return 'tags_cambia_st_error_updating';
    }
    return '';
}

# carga el id del tag
sub get_tag_urls {
    my ($bd, $tag_id, $prontus_id) = @_;

    $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id} = ();

    my @params = ($tag_id);
    my $tags_table = lib_utils::get_table('tags');
    my $sql = "select tags_port, tags_tag from $tags_table where tags_id = ?";
    my $salida = &glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'port'} = '';
    if (my $row = $salida->fetchrow_hashref) {
        $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'nom'} = $row->{'tags_tag'};
        if ($row->{'tags_port'}) {
            $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'port'} = $row->{'tags_port'};
        }
    }

    if ($prontus_varglb::CONFIG{$prontus_id}{'FRIENDLY_URLS_ENABLE'} eq '5') {
        my $urlf5_table = lib_utils::get_table('urlf5');
        $sql = "select urlf5_file, urlf5_url from $urlf5_table where urlf5_type = ? and urlf5_subtypeid = ? and urlf5_prontus = ?";
        @params = ('tag', $tag_id, $prontus_id);
        my ($urlf5_file, $urlf5_url);
        $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($urlf5_file, $urlf5_url));

        while ($salida->fetch) {
            $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'urls'}{$urlf5_file} = $urlf5_url;
        }
        $salida->finish;
        $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'urlf5'} = 1;
    } else {
        $lib_prontus::TAGS_HASH{$prontus_id}{$tag_id}{'urlf5'} = 0;
    }
}

sub cargar_fil_cfg {
    my ($file, $fil) = @_;

    return if (!-f $file); # no se hace nada si no existe el directorio y archivo del filtro.
    return if (exists $CFG_FIL_TAGPORT{$fil}); # para no cargarlo dos veces.

    my $cfg = glib_fildir_03::read_file($file);
    if ($cfg =~ m/TAGPORT_FIDS\s*=\s*['"](.+?)['"]/s) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        # dejar solo caracteres permitidos
        $value =~ s/[^a-zA-Z0-9_,]//sg;

        my @valores = split(',', $value);

        $CFG_FIL_TAGPORT{$fil}{'FIDS'} = \@valores;
    }

    if ($cfg =~ m/TAGPORT_PLANTILLAS\s*=\s*['"](.+?)['"]/s) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        # dejar solo caracteres permitidos
        $value =~ s/[^a-zA-Z0-9_\-,\.]//sg;

        my @valores = split(',', $value);

        foreach my $tpl (@valores) {
            $CFG_FIL_TAGPORT{$fil}{'PLANTILLAS'}{$tpl} = 1;
        }
    }

    if ($cfg =~ m/TAGPORT_FECHA_DESDE\s*=\s*['"](.+?)['"]/s) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        # dejar solo caracteres permitidos, numeros.
        $value =~ s/[^0-9]//sg;

        $value = '' if ($value !~ /^(\d{8})$/); # formato debe ser YYYYMMDD

        $CFG_FIL_TAGPORT{$fil}{'FECHA_DESDE'} = $value;
    }
}

# Genera segmento variable de la query SQL para encontrar los articulos.
# Aplica a portadas tag normales y a portadillas de calendarios tagonomicos
sub genera_filtros_tagports {
    my @params;
    my ($tag_id, $fid, $curr_dtime) = @_;
    my $fid_fil = $fid;

    if (substr($fid, 0, 4) eq 'fil_') {
        $fid = '';
    }

    my $fechahora_12c = substr($curr_dtime, 0 , 12);

    my $filtros = '';
    if ($tag_id) {
        push @params, $tag_id;
        $filtros = "(art_id = tagsart_idart and tagsart_idtags = ? )";
    }

    if ($fid && lib_helper::can_have_view($fid)){
        $filtros .= " and " if ($filtros);
        push @params, $fid;
        $filtros .= " (art_tipoficha = ? ) ";
    }

    if ($fid_fil && defined $lib_tags::CFG_FIL_TAGPORT{$fid_fil}{'FIDS'}) {
        my @fidlist = @{$lib_tags::CFG_FIL_TAGPORT{$fid_fil}{'FIDS'}};
        my $filtro_fids;

        if (scalar @fidlist) {
            foreach my $filfid (@fidlist) {
                next if(!lib_helper::can_have_view($filfid));
                push @params, $filfid;
                $filtro_fids .= "art_tipoficha = ? or ";
            }

            $filtro_fids = substr($filtro_fids, 0, (length($filtro_fids)-3));

            $filtros .= " and " if ($filtros);
            $filtros .= "($filtro_fids)";
        }
    }

    $filtros .= " and " if ($filtros);

    if ($fid_fil && defined $lib_tags::CFG_FIL_TAGPORT{$fid_fil}{'FECHA_DESDE'} && $lib_tags::CFG_FIL_TAGPORT{$fid_fil}{'FECHA_DESDE'} ne '') {
        push @params, $lib_tags::CFG_FIL_TAGPORT{$fid_fil}{'FECHA_DESDE'};
        $filtros .= " (art_fechap >= ? ) ";
    } else {
       push @params, "$fechahora_12c";
        $filtros .= " (art_fechaphorap <= ? ) ";
    }

    $filtros .= " and (art_alta = '1') ";

    # Aplicar exclusion de FIDs sin vista
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTIVAR_FIDS_SIN_VISTA'} eq 'SI') {
        foreach my $fid_list (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}}) {
            next if (exists($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}{$fid_list}));
            $filtros .= "and (art_tipoficha <> ?) ";
            push(@params, $fid_list);
        }
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
        push @params, "$fechahora_12c";
        push @params, "$fechahora_12c";
        $filtros .= " and ( (art_fechaehorae >= ? ) or ( (art_fechaehorae < ? ) and (art_soloportadas = '1') ) )";
    }

    return ($filtros, @params);
}

sub clear_tags_hash {
    %lib_prontus::TAGS_HASH = ();
}

1;
