#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
package lib_tax;
use strict;
use utf8;

# Limite maximo permitido.
our $MAX_LIMIT = 50;

our %CFG_FIL_TAXPORT = ();

our %SEO = ();
our %TAX = ();
our $MODTIME = 0;

# Obtiene seccion, tema, stema del articulo desde la base de datos.
sub get_taxonomia {
    my ($id, $base, $nivel) = @_;

    if ($nivel !~ /^\d+$/) {
        $nivel = 1;
    } elsif (!$nivel) {
        $nivel = 1;
    } elsif ($nivel > 3) {
        $nivel = 3;
    }

    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_idsecc$nivel, art_idtemas$nivel, art_idsubtemas$nivel from $art_table where art_id = ?";
    my @params = ($id);
    my ($secc, $tem, $stem);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($secc, $tem, $stem));

    $salida->fetch;
    $salida->finish;

    if ($secc) {
        $tem = '0' if ($tem eq '');
        $stem = '0' if ($stem eq '');

        return ($secc, $tem, $stem);
    } else {
        return ('-1','0','0');
    }
}

# Obtiene fixed_urls para seccion, tema, stema del articulo desde la base de datos.
sub get_fixed_urls {
    my ($seccion_id, $tema_id, $subtema_id) = @_;

    &carga_tax_json();

    if ($subtema_id) {
        return ($TAX{$seccion_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{subtemas}{$subtema_id}{port});
    }

    if ($tema_id) {
        return ($TAX{$seccion_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{port}, '');
    }

    if ($seccion_id) {
        return ($TAX{$seccion_id}{port}, '', '');
    }

    return ('', '','');
}

# ---------------------------------------------------------------
sub get_tot_artics {
    my ($filtros, $params_ref, $base, $maxartics) = @_;

    my $art_table = lib_utils::get_table('art');
    my $sql = "select count(art_id) from $art_table ";
    if ($filtros ne '') {
        $sql .= "where $filtros";
    }

    my $count_art;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, $params_ref, \($count_art));
    $salida->fetch;
    $salida->finish;

    $count_art = '0' if $count_art eq '';
    $count_art = $maxartics if ($count_art > $maxartics);

    return $count_art;
}

sub parse_tax_url {
    my $buffer = $_[0];
    my %campos = %{$_[1]};
    my $mv = $_[2];
    my $inc_pagnum = $_[3];

    my $aux = $buffer;
    my $first_page = 1;
    my $reldir_port_dst = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";

    $first_page = 0 if ($inc_pagnum eq 0);

    while ($aux =~ /(%%_seccion([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/g) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;
        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '___1.html';

        if ($campos{"_seccion$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page, $mv);

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    # buscamos marcas de url taxonomicas, tema
    while ($aux =~ /(%%_tema([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/g) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;

        if ($campos{"_tema$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '_'.$campos{"_tema$id"}.'__1.html';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page, $mv);

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    # buscamos marcas de url taxonomicas, subtema
    while ($aux =~ /(%%_subtema([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/g) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;

        if ($campos{"_subtema$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '_'.$campos{"_tema$id"}.'_'.$campos{"_subtema$id"}.'_1.html';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page, $mv);

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    return $buffer;
}

sub get_tax_if_last_art {
    my ($base, $secc_id, $tema_id, $subtema_id) = @_;
    my $sql;
    my $salida;
    my $count_secc;
    my $count_tems;
    my $count_subtems;
    my @params;

    my $art_table = lib_utils::get_table('art');
    $sql = "select count(*) from $art_table where art_idsecc1 = ? or art_idsecc2 = ? or art_idsecc3 = ?";

    @params = ($secc_id, $secc_id, $secc_id);

    $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_secc));
    $salida->fetch;
    $salida->finish;

    if ($tema_id) {
        $sql = "select count(*) from $art_table where art_idtemas1 = ? or art_idtemas2 = ? or art_idtemas2 = ?";

        @params = ($tema_id, $tema_id, $tema_id);

        $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_tems));
        $salida->fetch;
        $salida->finish;
    }

    if ($subtema_id) {
        $sql = "select count(*) from $art_table where art_idsubtemas1 = ? or art_idsubtemas2 = ? or art_idsubtemas3 = ?";

        @params = ($subtema_id, $subtema_id, $subtema_id);

        $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_subtems));
        $salida->fetch;
        $salida->finish;
    }

    if ($secc_id && !$tema_id && !$subtema_id) {
        return 1 if ($count_secc == 1);
    } elsif ($secc_id && $tema_id && !$subtema_id) {
        return 1 if ($tema_id == 1);
    } elsif ($secc_id && $tema_id && $subtema_id) {
        return 1 if ($subtema_id == 1);
    } else {
        return 0;
    }

    return 0;
}

sub generar_relacionados {
    my ($id_secc1, $id_tema1, $id_subtema1, $base, $mv) = @_;

    # corregimos para que no genere archivos "[secc]_0_0_art_relac.html", sino "[secc]___art_relac.html".
    if ($id_secc1 eq '0') {
        $id_secc1 = '';
    }

    if ($id_tema1 eq '0') {
        $id_tema1 = '';
    }

    if ($id_subtema1 eq '0') {
        $id_subtema1 = '';
    }

    # Procesa Plantillas que hayan en el dir.
    my ($plantilla, $pagina, $loop, $lista);
    my $use_old_path = 0;
    my $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXONOMIA;
    # si path nuevo no existe, usamos path antiguo
    if (! -d $ruta_dir) {
        $use_old_path = 1;
        $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXONOMIA_OLD;
    }

    $ruta_dir = "$ruta_dir-$mv" if ($mv);

    my @lisdir;
    @lisdir = glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);
    my $limit = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NUM_RELAC_DEFAULT'};

    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            # lib_stdlog::print_err("DIR[$ruta_dir/$k]\n");
            $plantilla = "$ruta_dir/$k";
            $pagina = glib_fildir_03::read_file($plantilla);

            if ($pagina =~ /%%_num_relac\s*=\s*(\d+)%%/) {
                $limit = $1;
            }

            # %%_TAX_LEVEL=seccion-tema-subtema%%
            my $taxlevel;
            if ($pagina =~ /%%_tax_level=(seccion|seccion-tema|seccion-tema-subtema)%%/) {
                $taxlevel = $1;
            } else {
                $taxlevel = 'seccion-tema-subtema'; # lo por defecto
            }

            $pagina =~ s/%%_tax_level\s*=[^%]*%%//g; # borra marca
            $pagina =~ s/%%_num_relac\s*=[^%]*%%//g; # borra marca

            my ($exclude_port, $exclude_port_area, $fids);

            if ($pagina =~ /%%_exclude_port\s*=([^%]+)%%/) {
                $exclude_port = $1;

                $exclude_port = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_EDIC'} . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_BASEEDIC'} . "/xml/$exclude_port";

                if (!-f $exclude_port && $exclude_port =~ /\.xml$/) {
                    $exclude_port = '';
                }
            }

            $pagina =~ s/%%_exclude_port\s*=[^%]*%%//g; # borra marca

            if ($pagina =~ /%%_exclude_port_area=([\d,]+)%%/s) {
                $exclude_port_area = $1;
                $exclude_port_area =~ s/,{2,}/,/g;
                $exclude_port_area =~ s/^,+//g;
                $exclude_port_area =~ s/,+$//g;

                #lib_stdlog::print_err("exclude_port_area[$exclude_port_area]\n");
            }

            $pagina =~ s/%%_exclude_port_area=.*?%%//g; # borra marca

            if ($pagina =~ /%%_fids=([^%]+)%%/s) {
                $fids = $1;
                $fids =~ s/,{2,}/,/g;
                $fids =~ s/^,+//g;
                $fids =~ s/,+$//g;

                #lib_stdlog::print_err("fids[$fids]\n");
            }

            $pagina =~ s/%%_fids=.*?%%//g; # borra marca

            if ($pagina =~ /%%loop%%(.*?)%%\/loop%%/s) {
                $loop = $1;
            }

            $limit++;

            if ($limit > $MAX_LIMIT) {
                $limit = $MAX_LIMIT;
            }

            my $nom_tabla_exclude;

            if ($exclude_port) {
                $nom_tabla_exclude = lib_prontus::set_exclude_port_table($exclude_port, $exclude_port_area, $base);
            }

            # Generar lista.
            my $hay_mas = 0;

            ($lista, $hay_mas) = &make_lista($id_secc1, $id_tema1, $id_subtema1, $loop, $limit, $base, $mv, $taxlevel, $fids, $nom_tabla_exclude);
            $limit--;
            $pagina =~ s/%%_num_relac%%/$limit/g;

            # lib_stdlog::print_err("lista[$lista]\n");
            &parse_and_write($id_secc1, $id_tema1, $id_subtema1, $lista, $pagina, $loop, $k, $hay_mas, $mv, $use_old_path);
        }
    }
}

sub make_lista {
    # Construye listas de articulos relacionados, para ser considerados, deben tener seccion a lo menos.
    my ($id_secc1, $id_tema1, $id_subtema1, $loop, $limit, $base, $mv, $taxlevel, $fids, $nom_tabla_exclude) = @_;

    my $dthr_system = glib_hrfec_03::get_dtime_pack4();
    my $hhmm_system = substr($dthr_system, 8, 4);
    my $dt_system = substr($dthr_system, 0, 8);

    my ($filtros, $params_ref) = &genera_filtros($id_secc1, $id_tema1, $id_subtema1, $dt_system, $hhmm_system, $taxlevel);
    my @params = @{$params_ref};

    # lib_stdlog::print_err("filtros[$filtros]\n");
    if ($nom_tabla_exclude) {
        if ($filtros ne '') {
            $filtros .= " and";
        }
        $filtros .= " art_id not in (select exclude_art_id from $nom_tabla_exclude)";
    }

    if ($fids) {
        my @arrfids = split /,/, $fids;

        my @tipo;
        foreach my $f (@arrfids) {
            push(@tipo, "art_tipoficha = ?");
            push(@params, $f);
        }

        if ($filtros ne '') {
            $filtros .= " and";
        }
        $filtros .= " (".join(' or ', @tipo).") ";
    }

    my $filtro_total = '';
    if ($filtros ne '') {
        $filtro_total = "where $filtros";
    } else {
        return '';
    }
    my $art_table = lib_utils::get_table('art');
    my $art_id;
    my $sql = "select art_id from $art_table $filtro_total order by art_fechaphorap desc limit ?";

    my $tot_artics = &get_tot_artics($filtros, \@params, $base, ($limit-1));
    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $limit;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_id));
    my $nro_filas = 1;
    my $filas = '';
    my $hay_mas = 0;
    my $loopcounter = 0;

    while ($salida->fetch) {
        if ($nro_filas >= $limit) {
            $hay_mas = 1;
            last;
        }

        $loopcounter++;
        my $fila = lib_parser::generar_fila($art_id, $loop, $loopcounter, $tot_artics, $mv, 1);
        $filas .= $fila;
        $nro_filas++;
    }

    $salida->finish;

    return ($filas, $hay_mas);
}

sub parse_and_write {
    my ($id_secc1, $id_tema1, $id_subtema1, $lista, $pagina, $loop, $nomfile, $hay_mas, $mv, $use_old_path) = @_;

    if ($lista eq '')  {
        $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;
        $lista =  lib_lang::get_message('artic_no_relac_art');
    }

    if (!$hay_mas) {
        $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;
    }

    $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;
    $pagina =~ s/%%_seccion1%%/$id_secc1/g;
    $pagina =~ s/%%_tema1%%/$id_tema1/g;
    $pagina =~ s/%%_subtema1%%/$id_subtema1/g;

    $pagina = lib_parser::parse_globals($pagina, '');

    if ($id_secc1 eq '-1') {
        $id_secc1 = '';
        $lista = '';
    }

    $pagina = lib_parser::ajusta_crlf($pagina);
    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    my $reldir_dst_artic_relac;
    if ($use_old_path) {
        $reldir_dst_artic_relac = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXONOMIA_OLD;
    } else {
        $reldir_dst_artic_relac = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXONOMIA;
    }

    if ($mv) {
        $reldir_dst_artic_relac = "$reldir_dst_artic_relac-$mv";
        glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$reldir_dst_artic_relac");
    }

    my $dst2write = "$prontus_varglb::DIR_SERVER$reldir_dst_artic_relac/" . $id_secc1 . '_' . $id_tema1 . '_' . $id_subtema1 . '_' . $nomfile;

    glib_fildir_03::write_file($dst2write, $pagina);
    lib_prontus::purge_cache($dst2write) if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_ART_RELAC'} eq 'SI');
}

sub genera_filtros {
    # Genera segmento variable del sql para encontrar los articulos relacionados.
    my ($id_secc1, $id_tema1, $id_subtema1, $dt_system, $hhmm_system, $taxlevel) = @_;
    # taxlevel: seccion | seccion-tema | seccion-tema-subtema, siempre al menos es "seccion"
    my @params;
    my $filtros = '1 = 1 and ';
    if ($id_secc1) {
        $filtros .= "(art_idsecc1 = ? or art_idsecc2 = ? or art_idsecc3 = ?)";

        push @params, $id_secc1;
        push @params, $id_secc1;
        push @params, $id_secc1;

        if (($id_tema1) && (lc $taxlevel ne 'seccion')) {
            $filtros .= " and (art_idtemas1 = ? or art_idtemas2 = ? or art_idtemas3 = ?)";

            push @params, $id_tema1;
            push @params, $id_tema1;
            push @params, $id_tema1;

            if (($id_subtema1) && (lc $taxlevel ne 'seccion') && (lc $taxlevel ne 'seccion-tema')) {
                $filtros .= " and (art_idsubtemas1 = ? or art_idsubtemas2 = ? or art_idsubtemas3 = ?)" if $filtros ne '';

                push @params, $id_subtema1;
                push @params, $id_subtema1;
                push @params, $id_subtema1;
            }
        }

        push @params, "$dt_system$hhmm_system";

        $filtros .= " and (art_fechaphorap <= ?) ";
        $filtros .= " and (art_alta = '1') ";

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $filtros .= " and ( (art_fechaehorae >= ?) or ( (art_fechaehorae < ?) and (art_soloportadas = '1') ) )";

            push @params, "$dt_system$hhmm_system";
            push @params, "$dt_system$hhmm_system";
        }

        # procesamos solamente los artículos que generan vistas.
        my %fids_hash = lib_helper::get_fids_with_view();
        my @fids_con_vista;
        foreach my $fid (keys %fids_hash) {
            push @fids_con_vista, "'$fid'";
        }
        my $fids_string = join(',', @fids_con_vista);
        $filtros .= " and art_tipoficha in ($fids_string) " if $fids_string;

        return ($filtros, \@params);
    } else {
        return ('', \@params);
    }
}

sub generar_relacionados_manualtax {
    my ($tax, $dst_dir, $ts, $base, $mv) = @_;

    # borra extension, en caso de que venga con ella
    $ts =~ s/\..+$//;

    my $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXONOMIA;
    # si path nuevo no existe, usamos path antiguo
    if (! -d $ruta_dir) {
        $ruta_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXONOMIA_OLD;
    }

    if ($mv) {
        $ruta_dir = "$ruta_dir-$mv";
        $dst_dir =~ s/(\d{8})\/pags/$1\/pags-$mv/;
    }

    my @lisdir;
    @lisdir = glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);

    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..
    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            my $pagina = glib_fildir_03::read_file("$ruta_dir/$k");
            my $loop = '';
            if ($pagina =~ /%%loop%%(.*?)%%\/loop%%/s) {
                $loop = $1;
            }

            # Geenerar listas con los articulos especificados manualmente
            my ($lista) = &make_lista_manualtax($tax, $loop, $base, $mv);
            # lib_stdlog::print_err("lista[$lista]\n");

            $pagina =~ s/%%_num_relac%%//g; # no aplica cuando hay manual tax.
            # Parsear y escribir paginas para incluir en articulos.
            &parse_and_write_manualtax($lista, $pagina, $dst_dir, $ts, $k);
        }
    }
}

sub parse_and_write_manualtax {
    my ($lista, $pagina, $dst_dir, $ts, $nomfile) = @_;
    # lib_stdlog::print_err("[$ts, $nomfile]\n");
    $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;

    $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;

    $pagina = lib_parser::parse_globals($pagina, '');

    $pagina = lib_parser::ajusta_crlf($pagina);

    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    glib_fildir_03::check_dir($dst_dir);
    glib_fildir_03::write_file("$dst_dir/" . $ts . '_' . $nomfile , $pagina);
}

sub genera_filtros_manualtax {
    # Genera segmento variable del sql para encontrar los articulos.
    my ($tax, $dt_system, $hhmm_system) = @_;
    $tax =~ s/[^0-9\,]//sg;

    my @ids = split(/\,/, $tax); # 10,20,35,112 o yyyymmddhhiiss
    my @params;

    my $filtros = '(art_id = 0';
    foreach my $id (@ids) {
        if ($id !~ /^\d+$/) {
            next;
        }

        if ($id =~ /^\d{14}$/) { # Es un TS.
            $filtros .= " or art_id = ? ";
        }

        push @params, $id;
    }
    $filtros .= ')';

    return ($filtros, \@params);
}

sub make_lista_manualtax {
    my ($tax, $loop, $base, $mv) = @_;

    my $dthr_system = glib_hrfec_03::get_dtime_pack4();
    $dthr_system =~ /^(\d{8})(\d\d\d\d)/;
    my $hhmm_system = $2;
    my $dt_system = $1;

    my ($filtros, $params_ref) = &genera_filtros_manualtax($tax, $dt_system, $hhmm_system);
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_id from $art_table where $filtros order by art_fechaphorap desc limit ?";
    my @params = @{$params_ref};

    my $tot_artics = &get_tot_artics($filtros, $params_ref, $base, $MAX_LIMIT);

    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $MAX_LIMIT;
    my $art_ts;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_ts));
    my $filas = '';
    my $loopcounter = 0;

    while ($salida->fetch) {
        $loopcounter++;
        my $fila = lib_parser::generar_fila($art_ts, $loop, $loopcounter, $tot_artics, $mv, 1);

        $filas .= $fila;
    }

    $salida->finish;

    return $filas;
}

sub carga_hash_seccion {
    my ($base, $only_enabled, $secc_id) = @_;

    my $secc_table = lib_utils::get_table('secc');
    my $sql = "select secc_id, secc_nom, secc_port, secc_nom4vistas, secc_mostrar from $secc_table ";

    my (@params, @filtros);

    if ($secc_id) {
        push @filtros, 'secc_id = ?';
        push @params, $secc_id;
    }
    if ($only_enabled) {
        push @filtros, 'secc_mostrar = ?';
        push @params, 1;
    }

    if (scalar(@filtros) > 0) {
        $sql .= 'where ' . join(' and ', @filtros);
    }

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    my %hash_datos;
    while (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);
        my @vistas = split(/\n/, $row->{'secc_nom4vistas'});
        my %hash_temp = ();
        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);
            $hash_temp{$temp[0]} = $temp[1];
        }
        $hash_datos{$row->{'secc_id'}}{'nombre'} = $row->{'secc_nom'};
        $hash_datos{$row->{'secc_id'}}{'port'} = $row->{'secc_port'};
        $hash_datos{$row->{'secc_id'}}{'mostrar'} = $row->{'secc_mostrar'};
        $hash_datos{$row->{'secc_id'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

sub carga_hash_temas {
    my ($base, $only_enabled, $id_padre, $temas_id) = @_;
    my ($sql, $salida, %hash_datos, @filtros);

    my $temas_table = lib_utils::get_table('temas');
    $sql = "select temas_id, temas_nom, temas_port, temas_idsecc, temas_nom4vistas, temas_mostrar from $temas_table ";

    my @params;
    if ($temas_id) {
        push @filtros, 'temas_id = ?';
        push @params, $temas_id;
    }
    if ($id_padre) {
        push @filtros, "temas_idsecc = ?";
        push @params, $id_padre;
    }

    if ($only_enabled) {
        push @filtros, "temas_mostrar = ?";
        push @params, $only_enabled;
    }

    if (scalar(@filtros) > 0) {
        $sql .= 'where ' . join(' and ', @filtros);
    }

    $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    while (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);

        my @vistas = split(/\n/, $row->{'temas_nom4vistas'});
        my %hash_temp = ();

        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);

            $hash_temp{$temp[0]} = $temp[1];
        }

        $hash_datos{$row->{'temas_id'}}{'id_padre'} = $row->{'temas_idsecc'} if (!$id_padre);
        $hash_datos{$row->{'temas_id'}}{'nombre'} = $row->{'temas_nom'};
        $hash_datos{$row->{'temas_id'}}{'port'} = $row->{'temas_port'};
        $hash_datos{$row->{'temas_id'}}{'mostrar'} = $row->{'temas_mostrar'};
        $hash_datos{$row->{'temas_id'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

sub carga_hash_subtemas {
    my ($base, $only_enabled, $id_padre, $subtemas_id) = @_;
    my ($sql, $salida, %hash_datos, @filtros, @params);

    my $subtemas_table = lib_utils::get_table('subtemas');
    $sql = "select subtemas_id, subtemas_nom, subtemas_port, subtemas_idtemas, subtemas_nom4vistas, subtemas_mostrar from $subtemas_table ";

    if ($subtemas_id) {
        push @filtros, 'subtemas_id = ?';
        push @params, $subtemas_id;
    }
    if ($id_padre) {
        push @params, $id_padre;
        push @filtros, "subtemas_idtemas = ?";
    }
    if ($only_enabled) {
        push @params, $only_enabled;
        push @filtros, "subtemas_mostrar = ?";
    }

    if (scalar(@filtros) > 0) {
        $sql .= 'where ' . join(' and ', @filtros);
    }

    $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);

        my @vistas = split(/\n/, $row->{'subtemas_nom4vistas'});
        my %hash_temp = ();

        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);

            $hash_temp{$temp[0]} = $temp[1];
        }

        $hash_datos{$row->{'subtemas_id'}}{'id_padre'} = $row->{'subtemas_idtemas'} if (!$id_padre);
        $hash_datos{$row->{'subtemas_id'}}{'nombre'} = $row->{'subtemas_nom'};
        $hash_datos{$row->{'subtemas_id'}}{'port'} = $row->{'subtemas_port'};
        $hash_datos{$row->{'subtemas_id'}}{'mostrar'} = $row->{'subtemas_mostrar'};
        $hash_datos{$row->{'subtemas_id'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

# carga configuracion propia del servidor si esta disponible
# sobreescribe los valores definidos en el cpan de prontus
sub carga_configuracion_local {
    my $buffer = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_taxport_conf.cfg");
    if ($buffer ne '') {
        if ($buffer =~ m/TAXPORT_MAX_WORKERS\s*=\s*["']([0-9]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} = $1;
        }
        if ($buffer =~ m/TAXPORT_MAX_MASTERS\s*=\s*["']([0-9]+)["']/) {
            $prontus_varglb::TAXPORT_MAX_MASTERS = $1;
        }
    }
}
# genera los filtros para los listados de taxport
sub genera_filtros_taxports {
    my ($id_secc1, $id_tema1, $id_subtema1, $fid, $curr_dtime) = @_;
    my $fid_fil = $fid;

    if (substr($fid, 0, 4) eq 'fil_') {
        $fid = '';
    }

    $id_secc1 =~ s/"/""/g;
    $id_tema1 =~ s/"/""/g;
    $id_subtema1 =~ s/"/""/g;

    my $dthhmm_system = substr($curr_dtime, 0, 12);
    my ($filtros, @params);

    if ($id_secc1) {
        $filtros = "(";
        $filtros .= "art_idsecc1 = ?";

        push @params, $id_secc1;

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} >= 2) {
            $filtros .= " or art_idsecc2 = ?";
            push @params, $id_secc1;
        }

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} == 3) {
            $filtros .= " or art_idsecc3 = ?";
            push @params, $id_secc1;
        }

        $filtros .= ") ";

        if ($id_tema1) { # Distinto de todos.
            if ($filtros ne '') {
                $filtros .= "and (";
                $filtros .= "art_idtemas1 = ?";
                push @params, $id_tema1;

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} >= 2) {
                    $filtros .= " or art_idtemas2 = ?";
                    push @params, $id_tema1;
                }

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} == 3) {
                    $filtros .= " or art_idtemas3 = ?";
                    push @params, $id_tema1;
                }

                $filtros .= ") ";
            }

            if ($id_subtema1) { # Distinto de todos.
                if ($filtros ne '') {
                    $filtros .= "and (";
                    $filtros .= "art_idsubtemas1 = ?";

                    push @params, $id_subtema1;

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} >= 2) {
                        $filtros .= " or art_idsubtemas2 = ?";
                        push @params, $id_subtema1;

                    }

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} == 3) {
                        $filtros .= " or art_idsubtemas3 = ?";
                        push @params, $id_subtema1;
                    }

                    $filtros .= ") ";
                }
            }
        }
    } else {
        $filtros = "(";
        $filtros .= "art_idsecc1 <> \"\"";
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} >= 2) {
            $filtros .= " or art_idsecc2 <> \"\"";
        }
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} == 3) {
            $filtros .= " or art_idsecc3 <> \"\"";
        }
        $filtros .= ")";
    }

    if ($fid && lib_helper::can_have_view($fid)){
        $filtros .= " and " if ($filtros);
        $filtros .= " (art_tipoficha = ?) ";
        push @params, $fid;
    }

    if ($fid_fil && defined $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FIDS'}) {
        my @fidlist = @{$CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FIDS'}};
        my $filtro_fids;

        if (scalar @fidlist) {
            foreach my $filfid (@fidlist) {
                next if(!lib_helper::can_have_view($filfid));
                $filtro_fids .= "art_tipoficha = ? or ";
                push @params, $filfid;
            }

            $filtro_fids = substr($filtro_fids, 0, (length($filtro_fids)-3));

            $filtros .= " and " if ($filtros);
            $filtros .= "($filtro_fids)";
        }
    }

    $filtros .= " and" if ($filtros);

    if ($fid_fil && defined $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'} && $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'} ne '') {
        $filtros .= " (art_fechap >= ?)";
        push @params, $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'};
    } else {
        $filtros .= " (art_fechaphorap <= ?)";
        push @params, $dthhmm_system;
    }

    $filtros .= ' and (art_alta=1) ';

    # Aplicar exclusion de FIDs sin vista
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTIVAR_FIDS_SIN_VISTA'} eq 'SI') {
        foreach my $fid_list (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}}) {
            next if (exists($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}{$fid_list}));
            $filtros .= "and (art_tipoficha <> ?) ";
            push(@params, $fid_list);
        }
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
        $filtros .= ' and ( (art_fechaehorae >= ?) or ( (art_fechaehorae < ?) and (art_soloportadas=1) ) )';
        push @params, $dthhmm_system;
        push @params, $dthhmm_system;
    }


    return ($filtros, \@params);
}

# cargo los filtros para los taxport filtradas
sub cargar_fil_cfg {
    my ($file, $fil) = @_;

    return if (!-f $file); # no se hace nada si no existe el directorio y archivo del filtro.
    return if (exists $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}); # para no cargarlo dos veces.

    my $cfg = glib_fildir_03::read_file($file);

    if ($cfg =~ m/TAXPORT_FIDS\s*=\s*["'](.+?)["']/) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        $value =~ s/[^a-zA-Z0-9_,]//sg; # dejar solo caracteres permitidos

        my @valores = split(',', $value);

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'FIDS'} = \@valores;
    }

    if ($cfg =~ m/TAXPORT_PLANTILLAS\s*=\s*["'](.+?)["']/) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        $value =~ s/[^a-zA-Z0-9_\-,\.]//sg; # dejar solo caracteres permitidos

        my @valores = split(',', $value);

        foreach my $tpl (@valores) {
            $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'PLANTILLAS'}{$tpl} = 1;
        }
    }

    if ($cfg =~ m/TAXPORT_FECHAP?_DESDE\s*=\s*["'](.+?)["']/s) { # fecha de publicacion, art_fechap
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;

        if ($value eq 'now') {
            $value = POSIX::strftime("%Y%m%d", localtime);
        } else {
            $value =~ s/[^0-9]//sg; # dejar solo caracteres permitidos, numeros.
            $value = '' if ($value !~ /^(\d{8})$/); # formato debe ser YYYYMMDD
        }

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'FECHA_DESDE'} = $value;
    }

    if ($cfg =~ m/\s*TAXPORT_ORDEN\s*=\s*("|')(.*?)("|')/s) { # fecha de publicacion, art_fechap
        my $value = $2;
        my $taxport_orden = 'art_fechap desc, art_horap desc'; # valor por defecto.

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;

        if ($value =~ /^(PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)$/) {
            if ($1 eq 'PUBLICACION') {
                $taxport_orden = "art_fechap $2, art_horap $2";
            } elsif ($1 eq 'TITULAR') {
                $taxport_orden = "art_titu $2";
            } elsif ($1 eq 'CREACION') {
                $taxport_orden = "art_id $2";
            }
        }

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'TAXPORT_ORDEN'} = $taxport_orden;
    }
}

# Se buscan los directorios que comiencen con fil_ en las plantillas de taxport.
sub load_taxport_fil {
    my $ruta_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_ID'}}{'DIR_TEMPLATESLATES'}$prontus_varglb::DIR_TAXPORT";
    my @listado = glob("$ruta_dir/fil_*");
    my @filtros;

    foreach my $dir (@listado) {
        if ($dir =~ /(fil_\w+)$/) {
            push @filtros, "fil_" . $1;
            &cargar_fil_cfg("$dir/filtros.cfg", $1);
        }
    }
    return @filtros;
}

# obtiene datos de articulos relacionados
sub get_relac_for_fid {
    my ($base, $ts_relacionados) = @_;
    # relacionados asignados
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_id, art_titu from $art_table where";
    my @ts_relacionados_array = split ',', $ts_relacionados;

    if ($ts_relacionados =~ /\d{14}/) {
        # son ts de articulos
        $sql .= " art_id";
    } else {
        # son ids de articulos
        $sql .= " ART_AUTOINC ";
    }

    my @placeholders = ('?') x scalar @ts_relacionados_array;
    my $placeholders_str = join ',', @placeholders;

    $sql .= " in ($placeholders_str) order by art_id asc";

    my ($idrel_asignado, $titular);
    my $relac_query = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@ts_relacionados_array, \($idrel_asignado, $titular));

    # Se leen los relacionados de la BD
    my %relac_asignados;

    while ($relac_query->fetch) {
        $titular = lib_encodes::escape_xml($titular);
        $relac_asignados{$idrel_asignado} = $titular;
    }

    $relac_query->finish;

    return \%relac_asignados;
}

sub carga_tax_json {
    my $num_elem = keys(%TAX);
    my $src_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/taxs.json";

    if ($num_elem) {
        # archivo fue modificado desde la ultima vez
        my $modtime = (stat($src_file))[9];
        return if ($MODTIME >= $modtime);
    }

    my $str_json = glib_fildir_03::read_file_bin($src_file);

    if ($str_json =~ /^\{.*\}$/) {
        %TAX = %{JSON::decode_json($str_json)};
        $MODTIME = (stat($src_file))[9];
    }
}

sub generar_json {
    my $bd = $_[0];
    my %taxonomia = &carga_hash_seccion($bd, 1);

    foreach my $secc_id (keys %taxonomia) {
        $taxonomia{$secc_id}{'temas'} = {&carga_hash_temas($bd, 1, $secc_id)};

        if (scalar(keys %{$taxonomia{$secc_id}{'temas'}}) > 0) {
            foreach my $tema_id (keys %{$taxonomia{$secc_id}{'temas'}}) {
                $taxonomia{$secc_id}{'temas'}{$tema_id}{'subtemas'} = {&carga_hash_subtemas($bd, 1, $tema_id)};
            }
        }
    }

    my $str_json = JSON::encode_json(\%taxonomia);
    utf8::decode($str_json);
    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    glib_fildir_03::check_dir($dst_path);
    glib_fildir_03::write_file("$dst_path/taxs.json", $str_json);
}

sub check_taxport_running {
    my ($res) = qx/ps axww | grep 'prontus_cron_taxport.pl' | grep -v grep | grep -v 'sh -c' | grep -v tail | wc -l/;

    $res =~ s/\D//gs;
    $res += 0; # para forzar el casteo a numero

    return $res;
}

sub check_worker_running {
    my ($res) = qx/ps axww | grep 'prontus_cron_taxport_worker.pl' | grep -v grep | grep -v 'sh -c' | grep -v tail | wc -l/;

    $res =~ s/\D//gs;
    $res += 0; # para forzar el casteo a numero

    return $res;
}

sub borrar_taxport {
    my $taxport = $_[0];
    my ($fid, $seccion, $tema, $subtema, $pagina) = split /\//, $taxport;
    my $reldir_port_dst = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";

    $seccion = '' if ($seccion eq '0');

    lib_stdlog::print_err("borrar_taxport [$taxport] fid[$fid] seccion[$seccion] tema[$tema] subtema[$subtema] pagina[$pagina]\n");

    # /prontus_develop/site/tax/port/all/taxport_1___1.html

    my $tax_dir = "$prontus_varglb::DIR_SERVER$reldir_port_dst";
    my @lisdir = glib_fildir_03::lee_dir($tax_dir);
    foreach my $subdir (@lisdir) {
        next if (substr($subdir, 0, 1) eq '.');
        next if (substr($subdir, 0, 1) eq '_');
        next if (! -d "$tax_dir/$subdir");

        my @lisdir_files = glib_fildir_03::lee_dir("$tax_dir/$subdir");
        foreach my $file (@lisdir_files) {
            next if (substr($file, 0, 1) eq '.');
            next if (substr($file, 0, 1) eq '_');

            if ($file =~ /^[a-zA-z0-9]+_$seccion\_/ || $file =~ /^[a-zA-z0-9]+____/) {
                my $path = "$tax_dir/$subdir/$file";
                unlink($path);
                lib_stdlog::print_err("delete [$path]\n");
                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAXPORT'} eq 'SI') {
                    lib_prontus::purge_cache($path);
                }
            }
        }
    }
}

# genera mapa del sitio en base a taxonomia
sub make_mapa {
    # $mv: nombre de la vista
    my ($mv, $bd) = @_;
    my $dir_plt = 'pags';    # rotulos tax
    $dir_plt .= "-$mv" if ($mv);    # rotulos tax
    my @postprocesos;

    my $ruta_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}/extra/mapa/$dir_plt";
    my @lisdir;
    if (-d $ruta_dir) {
        @lisdir = glib_fildir_03::lee_dir($ruta_dir);
        # Elimina directorios . y ..
        @lisdir = grep !/^\./, @lisdir;
    }
    foreach my $plt_file (@lisdir) {
        next if (!-f "$ruta_dir/$plt_file");

        my $mapa_plt = glib_fildir_03::read_file("$ruta_dir/$plt_file");
        $mapa_plt = lib_parser::ajusta_crlf($mapa_plt);
        my ($loop_mapa_s, $loop_mapa_t, $loop_mapa_st) = ('', '', '');
        if ($mapa_plt =~ /%%loop_seccion%%(.*?)%%\/loop_seccion%%/s) {
            $loop_mapa_s = $1;
        }
        if ($mapa_plt =~ /%%loop_tema%%(.*?)%%\/loop_tema%%/s) {
            $loop_mapa_t = $1;
        }
        if ($mapa_plt =~ /%%loop_subtema%%(.*?)%%\/loop_subtema%%/s) {
            $loop_mapa_st = $1;
        }
        my $base_indent = '15';
        my $mapa_total = &get_arbol_mapa($loop_mapa_s, $loop_mapa_t, $loop_mapa_st, $mv, $base_indent, $bd);
        $mapa_plt =~ s/%%loop_subtema%%.*?%%\/loop_subtema%%//s;
        $mapa_plt =~ s/%%loop_tema%%.*?%%\/loop_tema%%//s;
        $mapa_plt =~ s/%%loop_seccion%%.*?%%\/loop_seccion%%/$mapa_total/s;
        $mapa_plt = lib_prontus::parser_custom_function($mapa_plt);

        # parsear globales de prontus
        $mapa_plt = lib_parser::parse_globals($mapa_plt, '');

        my ($_plt_nom, $_plt_ext) = lib_utils::split_nom_y_extension($plt_file);
        $mapa_plt =~ s/%%_plt_nom%%/$_plt_nom/g;
        $mapa_plt =~ s/%%_plt_ext%%/$_plt_ext/g;
        $mapa_plt =~ s/%%_vista%%/$mv/g;

        # rescata el post_proceso
        if ($mv eq '') {
            while ($mapa_plt =~ /<!--POST_PROCESO=(.+?)-->/g) {
                push(@postprocesos, $1);
            }
        }
        $mapa_plt =~ s/<!--POST_PROCESO=.+?-->//g;

        # Borra marcas no sustituidas
        $mapa_plt =~ s/%%.+?%%//g;

        glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/mapa/$dir_plt");
        my $dst_mapa = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/mapa/$dir_plt/$plt_file";
        glib_fildir_03::write_file($dst_mapa, $mapa_plt);
        lib_prontus::purge_cache($dst_mapa) if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_MAPA'} eq 'SI');
    }

    # ejecutamos los postprocesos asociados
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";
    foreach my $pproc(@postprocesos) {
        $pproc =~ s/[#;&<>|~]+//g;
        my $cmd = "$rutaScript/$pproc $prontus_varglb::PRONTUS_ID $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} >/dev/null 2>&1 &";
        lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "][PPROC]$cmd\n");
        system $cmd;
    }
}

sub get_arbol_mapa {
    my ($loop_mapa_s, $loop_mapa_t, $loop_mapa_st, $mv, $indent, $bd) = @_;

    my ($local_indent, $mapa, $mapa_total, $tot_s, $tot_t, $tot_st);
    # lib_stdlog::print_err("loop_mapa[$loop_mapa]\n");

    # -----------------------------------------------------
    # secc
    my $secc_table = lib_utils::get_table('secc');
    my ($mapa_s, $nested_s, $secc_id, $secc_nom, $secc_port, $secc_nom4vistas, $secc_mostrar);
    my $sql = "select secc_id, secc_nom, secc_port, secc_nom4vistas, secc_mostrar from $secc_table where secc_orden > 0 order by secc_orden";
    my $salida = glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($secc_id, $secc_nom, $secc_port, $secc_nom4vistas, $secc_mostrar));

    $tot_s = $salida->rows;
    while ($salida->fetch) {
        # corrige codificacion
        glib_str_03::correct_encode_utf8_string($secc_nom);
        glib_str_03::correct_encode_utf8_string($secc_nom4vistas);
        # Cambia nom para la vista.
        # rotulos tax
        $secc_nom = lib_prontus::get_nomtax_envista($mv, $secc_nom4vistas, $secc_nom) if ($mv);

        $secc_nom  = lib_encodes::escape_html($secc_nom);
        $secc_port = lib_encodes::escape_html($secc_port);

        $local_indent = 0;

        $mapa_s = $loop_mapa_s;
        $mapa_s =~ s/%%_id%%/$secc_id/g;
        $mapa_s =~ s/%%_nom%%/$secc_nom/g;
        $mapa_s =~ s/%%_indent%%/$local_indent/g;
        $mapa_s =~ s/%%_mostrar%%/$secc_mostrar/g;

        $mapa_s =~ s/%%_fixed_url%%/$secc_port/g;
        $mapa_s =~ s/%%_url_fixed%%/$secc_port/g;
        $mapa_s =~ s/%%_seccion[1-3]%%/$secc_id/g;
        $mapa_s =~ s/%%_tema[1-3]%%//g;
        $mapa_s =~ s/%%_subtema[1-3]%%//g;

        my %campos = (
            '_seccion1' => $secc_id,
            '_seccion2' => $secc_id,
            '_seccion3' => $secc_id,
        );
        $mapa_s = &lib_tax::parse_tax_url($mapa_s, \%campos, $mv, 0);

        if ($mapa_s =~ /%%loop_tema%%.*?%%\/loop_tema%%/s) {
            $nested_s = 1;
        } else {
            $nested_s = 0;
        }

        # -----------------------------------------------------
        # temas
        my $temas_table = lib_utils::get_table('temas');
        $sql = "select temas_id, temas_nom, temas_port, temas_nom4vistas, temas_mostrar from $temas_table where temas_orden > 0 and temas_idsecc = ? order by temas_orden";

        my ($mapa_t, $mapa_total_t, $temas_id, $temas_nom, $temas_port, $temas_nom4vistas, $temas_mostrar);
        my @params = ($secc_id);
        my $salida_t = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($temas_id, $temas_nom, $temas_port, $temas_nom4vistas, $temas_mostrar));

        $tot_t = $salida_t->rows;

        while ($salida_t->fetch) {
            # corrige codificacion
            glib_str_03::correct_encode_utf8_string($temas_nom);
            glib_str_03::correct_encode_utf8_string($temas_nom4vistas);

            $temas_nom = lib_prontus::get_nomtax_envista($mv, $temas_nom4vistas, $temas_nom) if ($mv);    # rotulos tax
            $temas_nom = lib_encodes::escape_html($temas_nom);

            $local_indent = $indent;

            $mapa_t = $loop_mapa_t;
            $mapa_t =~ s/%%_id%%/$temas_id/g;
            $mapa_t =~ s/%%_nom%%/$temas_nom/g;
            $mapa_t =~ s/%%_indent%%/$local_indent/g;
            $mapa_t =~ s/%%_mostrar%%/$temas_mostrar/g;

            $mapa_t =~ s/%%_fixed_url%%/$temas_port/g;
            $mapa_t =~ s/%%_seccion[1-3]%%/$secc_id/g;
            $mapa_t =~ s/%%_tema[1-3]%%/$temas_id/g;
            $mapa_t =~ s/%%_subtema[1-3]%%//g;

            my %campos = (
                '_seccion1' => $secc_id,
                '_seccion2' => $secc_id,
                '_seccion3' => $secc_id,
                '_tema1'    => $temas_id,
                '_tema2'    => $temas_id,
                '_tema3'    => $temas_id,
            );
            $mapa_t = &lib_tax::parse_tax_url($mapa_t, \%campos, $mv, 0);

            # -----------------------------------------------------
            # subtemas
            my $subtemas_table = lib_utils::get_table('subtemas');
            $sql = "select subtemas_id, subtemas_nom, subtemas_port, subtemas_nom4vistas, subtemas_mostrar from $subtemas_table where subtemas_orden > 0 and subtemas_idtemas = ? order by subtemas_orden";

            my ($mapa_st, $mapa_st_total, $subtemas_id, $subtemas_nom, $subtemas_port, $subtemas_nom4vistas, $subtemas_mostrar);
            my @params = ($temas_id);
            my ($salida_st) = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($subtemas_id, $subtemas_nom, $subtemas_port, $subtemas_nom4vistas, $subtemas_mostrar));

            $tot_st = $salida_st->rows;

            while ($salida_st->fetch) {
                # corrige codificacion
                glib_str_03::correct_encode_utf8_string($subtemas_nom);
                glib_str_03::correct_encode_utf8_string($subtemas_nom4vistas);

                $subtemas_nom = lib_prontus::get_nomtax_envista($mv, $subtemas_nom4vistas, $subtemas_nom) if ($mv);    # rotulos tax
                $subtemas_nom = lib_encodes::escape_html($subtemas_nom);
                $local_indent = $indent * 2;

                $mapa_st = $loop_mapa_st;
                $mapa_st =~ s/%%_id%%/$subtemas_id/g;
                $mapa_st =~ s/%%_nom%%/$subtemas_nom/g;
                $mapa_st =~ s/%%_indent%%/$local_indent/g;
                $mapa_st =~ s/%%_mostrar%%/$subtemas_mostrar/g;

                $mapa_st =~ s/%%_fixed_url%%/$subtemas_port/g;
                $mapa_st =~ s/%%_seccion[1-3]%%/$secc_id/g;
                $mapa_st =~ s/%%_tema[1-3]%%/$temas_id/g;
                $mapa_st =~ s/%%_subtema[1-3]%%/$subtemas_id/g;

                my %campos = (
                    '_seccion1' => $secc_id,
                    '_seccion2' => $secc_id,
                    '_seccion3' => $secc_id,
                    '_tema1'    => $temas_id,
                    '_tema2'    => $temas_id,
                    '_tema3'    => $temas_id,
                    '_subtema1' => $subtemas_id,
                    '_subtema2' => $subtemas_id,
                    '_subtema3' => $subtemas_id,
                );
                $mapa_st = &lib_tax::parse_tax_url($mapa_st, \%campos, $mv, 0);

                my %claves;
                $claves{'_mostrar'} = $subtemas_mostrar;
                $mapa_st = lib_parser::procesa_ifs('', $mapa_st, \%claves);

                $mapa_st_total = $mapa_st_total . $mapa_st;
            }
            if ($mapa_t =~ /%%loop_subtema%%.*?%%\/loop_subtema%%/s) {
                $mapa_t =~ s/%%loop_subtema%%.*?%%\/loop_subtema%%/$mapa_st_total/sg;
            } else {
                $mapa_t = $mapa_t . $mapa_st_total;
            }

            my %claves;
            $claves{'_subtema'} = $tot_st if ($tot_st);
            $claves{'_mostrar'} = $temas_mostrar;

            $mapa_t = lib_parser::procesa_ifs('', $mapa_t, \%claves);

            $mapa_total_t = $mapa_total_t . $mapa_t;
            $salida_st->finish;
        }
        if ($nested_s) {
            $mapa_s =~ s/%%loop_tema%%.*?%%\/loop_tema%%/$mapa_total_t/s;
        } else {
            $mapa_s = $mapa_s . $mapa_total_t;
        }

        my %claves;
        $claves{'_tema'} = $tot_t if ($tot_t);
        $claves{'_mostrar'} = $secc_mostrar;
        $mapa_s = lib_parser::procesa_ifs('', $mapa_s, \%claves);

        $salida_t->finish;
        $mapa_total = $mapa_total . $mapa_s;
    }
    $salida->finish;
    return ($mapa_total);
}

sub get_nom_seccion {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    my $secc_table = lib_utils::get_table('secc');
    $sql = "select secc_nom from $secc_table where secc_id = ?";
    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

sub get_nom_tema {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    my $temas_table = lib_utils::get_table('temas');
    $sql = "select temas_nom from $temas_table where temas_id = ?";

    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

sub get_nom_subtema {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    my $subtemas_table = lib_utils::get_table('subtemas');
    $sql = "select subtemas_nom from $subtemas_table where subtemas_id = ?";

    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

sub genera_orden_taxports {
    my $fid = $_[0];

    if (substr($fid, 0, 4) eq 'fil_') {
        if (defined $lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid}{'TAXPORT_ORDEN'} && $lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid}{'TAXPORT_ORDEN'} ne '' ) {
            return $lib_tax::CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid}{'TAXPORT_ORDEN'};
        } else {
            return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'};
        }
    } else {
        return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'};
    }
}

sub get_sql_and_total {
    my ($secc_id, $temas_id, $subtemas_id, $fid, $CURR_DTIME, $bd) = @_;
    my ($filtros, $params_ref) = &genera_filtros_taxports($secc_id, $temas_id, $subtemas_id, $fid, $CURR_DTIME);
    my $taxport_order = &genera_orden_taxports($fid);
    my $tot_artics = &get_tot_artics($filtros, $params_ref, $bd, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'});

    if ($filtros ne '') {
        $filtros = "where $filtros";
    }
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_id, art_tipoficha from $art_table $filtros order by $taxport_order limit ?, ?";

    return ($params_ref, $sql, $tot_artics);
}

sub get_name_ports {
    my ($secc_id, $temas_id, $subtemas_id) = @_;
    my $secc_nom = $lib_tax::TABLA_SECC{$secc_id}{'nombre'};
    my $secc_port = $lib_tax::TABLA_SECC{$secc_id}{'port'} || '';
    my $temas_nom = $lib_tax::TABLA_TEM{$temas_id}{'nombre'} || '';
    my $temas_port = $lib_tax::TABLA_TEM{$temas_id}{'port'} || '';
    my $subtemas_nom = $lib_tax::TABLA_STEM{$subtemas_id}{'nombre'} || '';
    my $subtemas_port = $lib_tax::TABLA_STEM{$subtemas_id}{'port'} || '';

    my $tax_fixedurl = '';
    if ($secc_id && !$temas_id && !$subtemas_id) {
        $tax_fixedurl = $secc_port;
    } elsif ($secc_id && $temas_id && !$subtemas_id) {
        $tax_fixedurl = $temas_port;
    } elsif ($secc_id && $temas_id && $subtemas_id) {
        $tax_fixedurl = $subtemas_port;
    }

    return ($secc_nom, $secc_port, $temas_nom, $temas_port, $subtemas_nom, $subtemas_port, $tax_fixedurl)
}


sub incluir_navbar {
    my ($bd, $pagina, $secc_id, $temas_id, $subtemas_id, $mv, $fid, $extension, $nom_plt) = @_;

    if ($secc_id eq '' && $temas_id eq '' && $subtemas_id eq '') {
        $pagina =~ s/%%_secc_tema_stema_nom.*?%%//g;
        return $pagina;
    }

    # Se lee el separador
    my $separador = '/';
    if ($pagina =~ /%%_secc_tema_stema_nom\((.*?)\)%%/) {
        $separador = $1;
    }

    my $secc_tema_stema_nom = '';

    my $base_path = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
    # Seccion.
    if ($secc_id) {
        my $lnk_secc;
        if ($lib_tax::TABLA_SECC{$secc_id}{'port'}) {
            $lnk_secc = $lib_tax::TABLA_SECC{$secc_id}{'port'};
        } else {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                $lnk_secc = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '___1.' . $extension, 1, $mv);
            } else {
                $lnk_secc = "$base_path/$fid";
                $lnk_secc .= "-$mv" if ($mv);
                $lnk_secc .= "/$nom_plt\_$secc_id\___1.$extension";
            }
        }
        my $secc_nom;
        if ($mv && $lib_tax::TABLA_SECC{$secc_id}{'vistas'}{$mv}) {
            $secc_nom = $lib_tax::TABLA_SECC{$secc_id}{'vistas'}{$mv};
        } else {
            $secc_nom = $lib_tax::TABLA_SECC{$secc_id}{'nombre'};
        }
        $secc_nom = lib_encodes::escape_html($secc_nom);

        $secc_tema_stema_nom = "<a href=\"$lnk_secc\">$secc_nom</a>";
    }

    # Tema.
    if ($temas_id) {
        my $lnk_tem;
        if ($lib_tax::TABLA_TEM{$temas_id}{'port'}) {
            $lnk_tem = $lib_tax::TABLA_TEM{$temas_id}{'port'};
        } else {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                $lnk_tem = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '__1.' . $extension, 1, $mv);
            } else{
                $lnk_tem = "$base_path/$fid";
                $lnk_tem .= "-$mv" if ($mv);
                $lnk_tem .= "/$nom_plt\_$secc_id\_$temas_id\__1.$extension";
            }
        }
        my $tem_nom;
        if ($mv && $lib_tax::TABLA_TEM{$temas_id}{'vistas'}{$mv}) {
            $tem_nom = $lib_tax::TABLA_TEM{$temas_id}{'vistas'}{$mv};
        } else {
            $tem_nom = $lib_tax::TABLA_TEM{$temas_id}{'nombre'};
        }
        $tem_nom = lib_encodes::escape_html($tem_nom);

        $secc_tema_stema_nom .= " $separador <a href=\"$lnk_tem\">$tem_nom</a>";
    }

    # Subtema.
    if ($subtemas_id) {
        my $lnk_stem;
        if ($lib_tax::TABLA_STEM{$subtemas_id}{'port'}) {
            $lnk_stem = $lib_tax::TABLA_STEM{$subtemas_id}{'port'};
        } else {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                $lnk_stem = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '_' . $subtemas_id. '_1.' . $extension, 1, $mv);
            } else{
                $lnk_stem = "$base_path/$fid";
                $lnk_stem .= "-$mv" if ($mv);
                $lnk_stem .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_1.$extension";
            }
        }

        my $stem_nom = $lib_tax::TABLA_STEM{$subtemas_id}{'nombre'};
        if ($mv && $lib_tax::TABLA_STEM{$subtemas_id}{'vistas'}{$mv}) {
            $stem_nom = $lib_tax::TABLA_STEM{$subtemas_id}{'vistas'}{$mv};
        }
        $stem_nom = lib_encodes::escape_html($stem_nom);

        $secc_tema_stema_nom .=  " $separador <a href=\"$lnk_stem\">$stem_nom</a>";
    }

    $pagina =~ s/%%_secc_tema_stema_nom.*?%%/$secc_tema_stema_nom/g;
    return ($pagina, $secc_tema_stema_nom);
}

sub incluir_next_y_prev {
    my ($bd ,$tot_artics, $pagina, $nro_pag, $secc_id, $temas_id, $subtemas_id, $mv, $fid, $extension, $nom_plt) = @_;
    my $nro_paginas_totales = POSIX::ceil($tot_artics / $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ARTXPAG'});
    my ($prevpag, $nextpag, $prevlink, $nextlink);

    $nextpag = ($nro_pag + 1) if (($nro_pag + 1) <= $nro_paginas_totales);
    $prevpag = ($nro_pag - 1) if (($nro_pag - 1) > 0);

    my $base_path = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
    if ($prevpag) {
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            $prevlink = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '_' . $subtemas_id. "_$prevpag." . $extension, $prevpag, $mv);
        } else {
            $prevlink = "$base_path/$fid";
            $prevlink .= "-$mv" if ($mv);
            $prevlink .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_$prevpag.$extension";
        }
    }

    if ($nextpag) {
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            $nextlink = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '_' . $subtemas_id. "_$nextpag." . $extension, $nextpag, $mv);
        } else {
            $nextlink = "$base_path/$fid";
            $nextlink .= "-$mv" if ($mv);
            $nextlink .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_$nextpag.$extension";
        }
    }

    $pagina =~ s/%%_prevpag%%/$prevpag/g;
    $pagina =~ s/%%_nextpag%%/$nextpag/g;
    $pagina =~ s/%%_prevlink%%/$prevlink/g;
    $pagina =~ s/%%_nextlink%%/$nextlink/g;

    return ($pagina, $prevpag, $nextpag, $prevlink, $nextlink);
}

sub incluir_nrosdepag {
    my ($bd, $tot_artics, $pagina, $nro_pag, $secc_id, $temas_id, $subtemas_id, $mv, $fid, $extension, $nom_plt, $msg_noresults) = @_;

    my $tpl_nropag = '<a href="%%lnk%%">%%cnro_pag%%</a>';
    my $tpl_current_pag = '<span class="actual">%%cnro_pag%%</span>';
    my $tpl_separador = '...';
    my $html_nros_pag = '';

    # Carga configuración.
    my %cfg_paginacion;
    while ($pagina =~ /<!--\s*CONFIG\s*(\w+)\s*=\s*(.+?)\s*-->/g) {
        my $name = uc $1;
        my $value = $2;

        if ($name eq 'HTML_NRO_PAG') {
            $tpl_nropag = $value;
        } elsif ($name eq 'HTML_PAG_ACTUAL') {
            $tpl_current_pag = $value;
        } elsif ($name eq 'HTML_SEPARADOR') {
            $tpl_separador = $value;
        }
    }

    # Quitar comentarios de configuración.
    $pagina =~ s/<!--\s*CONFIG\s*(\w+)\s*=\s*(.+?)\s*-->//g;

    my $nro_paginas_totales = POSIX::ceil($tot_artics / $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ARTXPAG'});
    my $base_path = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
    my ($ini, $fin, $nextlink, $prevlink);

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_TIPO_PAGINACION'} eq '1') {
        # Se procesan las paginas hacia abajo
        $ini = ($nro_pag - $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_PAGCORTA_MAXPAGS'});

        if ($ini < 1) {
            $ini = 1;
        } else {
            $ini = $ini + 1;

            my $lnk;
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                $lnk = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '_' . $subtemas_id. '_1.' . $extension, 1, $mv);
            } else {
                $lnk = "$base_path/$fid";
                $lnk .= "-$mv" if ($mv);
                $lnk .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_1.$extension";
            }

            my $tpl_nropag_aux = $tpl_nropag;
            $tpl_nropag_aux =~ s/%%lnk%%/$lnk/g;
            $tpl_nropag_aux =~ s/%%cnro_pag%%/1/g;
            $prevlink =  $tpl_nropag_aux . ' ' . $tpl_separador . ' ';
        }

        # Se procesan las páginas hacia arriba
        $fin = ($nro_pag + $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_PAGCORTA_MAXPAGS'});

        if ($fin >= $nro_paginas_totales) {
            $fin = $nro_paginas_totales;
        } else {
            $fin = $fin - 1;

            my $lnk;
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                $lnk = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_' . $secc_id . '_' . $temas_id. '_' . $subtemas_id . "_.$nro_paginas_totales" . $extension, $nro_paginas_totales, $mv);
            } else {
                $lnk = "$base_path/$fid";
                $lnk .= "-$mv" if ($mv);
                $lnk .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_$nro_paginas_totales.$extension";
            }

            my $tpl_nropag_aux = $tpl_nropag;
            $tpl_nropag_aux =~ s/%%lnk%%/$lnk/g;
            $tpl_nropag_aux =~ s/%%cnro_pag%%/$nro_paginas_totales/g;
            $nextlink =  ' ' . $tpl_separador . ' ' . $tpl_nropag_aux;
        }
    } else {
        $ini = 1;
        $fin = $nro_paginas_totales;
    }

    my $current_link = '';
    for (my $pag = $ini; $pag <= $fin; $pag++) {
        my $tpl_nropag_aux;

        my $lnk;
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
            $lnk = lib_urlf5::get_link_tax($bd, "$base_path/$fid/$nom_plt" . '_'. $secc_id. '_' . $temas_id. '_' . $subtemas_id . "_$pag." . $extension, $pag, $mv);
        } else {
            $lnk = "$base_path/$fid";
            $lnk .= "-$mv" if ($mv);
            $lnk .= "/$nom_plt\_$secc_id\_$temas_id\_$subtemas_id\_$pag.$extension";
        }

        if ($pag eq $nro_pag) {
            $current_link = $lnk;
            $tpl_nropag_aux = $tpl_current_pag;
        } else {
            $tpl_nropag_aux = $tpl_nropag;
        }

        $tpl_nropag_aux =~ s/%%lnk%%/$lnk/g;
        $tpl_nropag_aux =~ s/%%cnro_pag%%/$pag/g;
        $html_nros_pag .= "$tpl_nropag_aux\n";
    }

    $html_nros_pag = $prevlink . $html_nros_pag . $nextlink;

    if ($html_nros_pag ne '') {
        $pagina =~ s/%%_html_nros_pag%%/ $html_nros_pag /g;
    } else {
        $pagina =~ s/%%_html_nros_pag%%//g;
        $pagina =~ s/%%_msg%%.*?%%\/_msg%%/$msg_noresults/sg;
    }

    $pagina =~ s/%%_current_link%%/$current_link/sg;

    return $pagina;
}

sub load_seo_data {
    my ($secc_id,$temas_id, $subtemas_id) = @_;
    my $entidad = 'seccion';
    my $id = $secc_id;
    if ($subtemas_id) {
        $entidad = 'subtema';
        $id = $subtemas_id;
    } elsif ($temas_id) {
        $entidad = 'tema';
        $id = $temas_id;
    }

    if ($id > 0 && !defined($SEO{$entidad}{$id})) {
        # Obtener informaciรณn de SEO desde JSON.
        my $tax_seo_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT/seo/$entidad";
        my $file = "$tax_seo_dir/". $id .".json";
        if (-f $file) {
            my $jsonSeoBuffer = glib_fildir_03::read_file($file);
            $SEO{$entidad}{$id} = JSON::from_json($jsonSeoBuffer);
        }
    }
    return ($entidad, $id);
}

sub parse_port {
    my ($bd, $pagina, $mv, $tot_artics, $current_art, $nro_pag, $nombase, $extension, $fid, $secc_nom, $temas_nom, $subtemas_nom, $secc_port, $temas_port, $subtemas_port, $secc_id, $temas_id, $subtemas_id, $tax_fixedurl, $msg_noresults, $entidad, $id) = @_;

    $pagina =~ s/%%_current_art_pag%%/$current_art/g;
    $pagina =~ s/%%_totartics%%/$tot_artics/g;
    $pagina =~ s/%%_plt_nom%%/$nombase/g;
    $pagina =~ s/%%_plt_ext%%/$extension/g;
    $pagina =~ s/%%nomsecc%%/$secc_nom/g;
    my $fid_path;
    if ($fid) {
        $pagina =~ s/%%_tax_fid%%/$fid/g;
        $fid_path = $fid;
    } else {
        $pagina =~ s/%%_tax_fid%%/all/g;
        $fid_path = 'all';
    }

    my $secc_tema_stema_nom = '';
    if (index($pagina, '%%_no_paginar%%') < 0) {
        ($pagina, $secc_tema_stema_nom) = lib_tax::incluir_navbar($bd, $pagina, $secc_id, $temas_id, $subtemas_id, $mv, $fid_path, $extension, $nombase);
        $pagina = &incluir_nrosdepag($bd, $tot_artics, $pagina, $nro_pag, $secc_id, $temas_id, $subtemas_id, $mv, $fid_path, $extension, $nombase, $msg_noresults);
    }

    if (defined($SEO{$entidad}{$id})) {
        foreach my $seoKey (keys %{$SEO{$entidad}{$id}}) {
            $pagina =~ s/%%$seoKey%%/$SEO{$entidad}{$id}{$seoKey}/sg;
            $pagina = lib_prontus::parse_tag_prontus($SEO{$entidad}{$id}{$seoKey}, $seoKey, $pagina);
        }
    }

    # reemplazar variables globales
    $pagina = lib_parser::parse_globals($pagina, '');

    $pagina =~ s/%%_nropagina%%/$nro_pag/g;
    $pagina =~ s/%%_vista%%/$mv/g;
    $pagina =~ s/%%_mv%%/$mv/g;
    $pagina =~ s/%%_fixed_url%%/$tax_fixedurl/g;

    # Parsea marca %%_seccionX.url()%%
    my %campos = (
        '_seccion1' => $secc_id,
        '_seccion2' => $secc_id,
        '_seccion3' => $secc_id,
        '_tema1'    => $temas_id,
        '_tema2'    => $temas_id,
        '_tema3'    => $temas_id,
        '_subtema1' => $subtemas_id,
        '_subtema2' => $subtemas_id,
        '_subtema3' => $subtemas_id
    );
    $pagina = lib_tax::parse_tax_url($pagina, \%campos, $mv, 0);

    # Parsea marcas
    # %%_prevpag%% (numero pagina anterior)
    # %%_nextpag%% (numero pagina siguiente)
    # %%_prevlink%% (link a pagina anterior)
    # %%_nextlink%% (link a pagina siguiente)
    #
    # Quedan vacios si no hay mas paginas.
    #
    my ($prevpag, $nextpag, $prevlink, $nextlink);
    ($pagina, $prevpag, $nextpag, $prevlink, $nextlink) = lib_tax::incluir_next_y_prev($bd, $tot_artics, $pagina, $nro_pag, $secc_id, $temas_id, $subtemas_id, $mv, $fid_path, $extension, $nombase);

    # Para poder hacer IF de seccion, tema y subtema
    my %claves = (
        '_current_art_pag' => $current_art,
        '_secc_tema_stema_nom' => $secc_tema_stema_nom,
        '_tax_seccion' => $secc_id,
        '_tax_tema' => $temas_id,
        '_tax_subtema' => $subtemas_id,
        '_tax_nom_seccion' => $secc_nom,
        '_tax_nom_tema' => $temas_nom,
        '_tax_nom_subtema' => $subtemas_nom,
        '_seccion1' => $secc_id,
        '_tema1' => $temas_id,
        '_subtema1' => $subtemas_id,
        '_nom_seccion1' => $secc_nom,
        '_nom_tema1' => $temas_nom,
        '_nom_subtema1' => $subtemas_nom,
        '_vista' => $mv,
        '_mv' => $mv,
        '_nropagina' => $nro_pag,
        '_tax_fid' => $fid,
        '_nextpag' => $nextpag,
        '_prevpag' => $prevpag,
        '_nextlink' => $nextlink,
        '_prevlink' => $prevlink,
        '_totartics' => $tot_artics,
        '_url_fixed_seccion' => $secc_port,
        '_url_fixed_tema' => $temas_port,
        '_url_fixed_subtema' => $subtemas_port,
        '_url_fixed_seccion1' => $secc_port,
        '_url_fixed_tema1' => $temas_port,
        '_url_fixed_subtema1' => $subtemas_port
    );

    # agrega valores de seo para los if
    if (defined($SEO{$entidad}{$id})) {
        foreach my $seoKey (keys %{$SEO{$entidad}{$id}}) {
            $claves{$seoKey} = $SEO{$entidad}{$id}{$seoKey};
        }
    }

    $pagina = lib_parser::procesa_condicional($pagina, \%claves);

    # Se parsean todas las marcas de seccion, tema y subtema.
    # Esto se mantiene por compatibilidad
    $pagina =~ s/%%_seccion[1-3]?%%/$secc_id/g;
    $pagina =~ s/%%_tema[1-3]?%%/$temas_id/g;
    $pagina =~ s/%%_subtema[1-3]?%%/$subtemas_id/g;
    $pagina =~ s/%%_nom_seccion[1-3]?%%/$secc_nom/g;
    $pagina =~ s/%%_nom_tema[1-3]?%%/$temas_nom/g;
    $pagina =~ s/%%_nom_subtema[1-3]?%%/$subtemas_nom/g;

    # Se parsean las nuevas marcas de taxport
    $pagina =~ s/%%_tax_seccion%%/$secc_id/g;
    $pagina =~ s/%%_tax_tema%%/$temas_id/g;
    $pagina =~ s/%%_tax_subtema%%/$subtemas_id/g;
    $pagina =~ s/%%_tax_nom_seccion%%/$secc_nom/g;
    $pagina =~ s/%%_tax_nom_tema%%/$temas_nom/g;
    $pagina =~ s/%%_tax_nom_subtema%%/$subtemas_nom/g;
    $pagina =~ s/%%_url_fixed_seccion[1-3]?%%/$secc_port/g;
    $pagina =~ s/%%_url_fixed_tema[1-3]?%%/$temas_port/g;
    $pagina =~ s/%%_url_fixed_subtema[1-3]?%%/$subtemas_port/g;

    # Parsear el nombre de la s, t, st segun la vista.
    if ($mv) {
        my ($mv_nom_seccion1, $mv_nom_tema1, $mv_nom_subtema1) = lib_prontus::get_nom4vistas($bd, $mv, $secc_id, $temas_id, $subtemas_id);
        $pagina =~ s/%%_tax_nom_seccion_$mv%%/$mv_nom_seccion1/g;
        $pagina =~ s/%%_tax_nom_tema_$mv%%/$mv_nom_tema1/g;
        $pagina =~ s/%%_tax_nom_subtema_$mv%%/$mv_nom_subtema1/g;
        $pagina =~ s/%%_nom_seccion[1-3]_$mv%%/$mv_nom_seccion1/g;
        $pagina =~ s/%%_nom_tema[1-3]_$mv%%/$mv_nom_tema1/g;
        $pagina =~ s/%%_nom_subtema[1-3]_$mv%%/$mv_nom_subtema1/g;
    }

    $pagina = lib_prontus::parser_custom_function($pagina);
    $pagina = lib_prontus::parse_includes($pagina);

    $pagina =~ s/%%.*?%%//sg;
    return lib_parser::clean_spaces($pagina);
}

sub get_path_port_template {
    my ($s, $t, $st, $ruta_dir, $fid, $mv, $ext_port_tmp, $nombase) = @_;
    my $tpl;

    my @paths;
    if ($fid) {
        @paths = ("$ruta_dir/$fid$mv", "$ruta_dir/all$mv", "$ruta_dir/all");
    } else {
        @paths = ("$ruta_dir/all$mv", "$ruta_dir/all");
    }

    for (my $i = 0; $i < 2; $i++) {
        # Busca de la plantilla mas especifica a la mas general
        $tpl = "$paths[$i]/$nombase\_$s" . "_$t" . "_$st$ext_port_tmp";
        return $tpl if (-f $tpl);

        $tpl = "$paths[$i]/$nombase\_$s" . "_$t$ext_port_tmp";
        return $tpl if (-f $tpl);

        $tpl = "$paths[$i]/$nombase\_$s$ext_port_tmp";
        return $tpl if (-f $tpl);

        $tpl = "$paths[$i]/$nombase$ext_port_tmp";
        return $tpl if (-f $tpl);

    }
    return '';
}

1;
