# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# Funciones para manejar urls f5
# requiere
# use glib_dbi_03;
# use lib_prontus;
# use glib_str_03; para verifica_url
# use lib_friendly;

package lib_urlf5;
use strict;
use utf8;

sub insert_url {
    my $base = $_[0];
    my %options = %{$_[1]};
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "insert into $urlf5_table set urlf5_type = ?, urlf5_subtype = ?,";
    $sql .= " urlf5_subtypeid = ?, urlf5_file = ?, urlf5_url = ?, urlf5_prontus = ?";

    $options{'url'} = lc($options{'url'});

    my @params = ($options{'type'}, $options{'subtype'}, $options{'subtype_id'}, $options{'file'}, $options{'url'}, $options{'prontus'});
    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    if ($base->errstr) {
        lib_stdlog::print_err(lib_lang::get_message('urlf5_error_insert'), " [".$base->errstr."]\n");
    }
}

sub borrar_url {
    my $base = $_[0];
    my %options = %{$_[1]};
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "delete from $urlf5_table where 1=1 ";
    my @params;

    if (defined($options{'type'}) && $options{'type'} ne '') {
        $sql .= " and urlf5_type = ?";
        push @params, $options{'type'};
    }
    if (defined($options{'url'}) && $options{'url'} ne '') {
        $sql .= " and urlf5_url = ?";
        push @params, $options{'url'};
    }
    if (defined($options{'file'}) && $options{'file'} ne '') {
        $sql .= " and urlf5_file = ?";
        push @params, $options{'file'};
    }
    if (defined($options{'subtype'}) && $options{'subtype'} ne '') {
        $sql .= " and urlf5_subtype = ?";
        push @params, $options{'subtype'};
    }
    if (defined($options{'subtype_id'}) && $options{'subtype_id'} ne '') {
        $sql .= " and urlf5_subtypeid = ?";
        push @params, $options{'subtype_id'};
    }
    $sql .= " and urlf5_prontus = ?";
    push @params, $options{'prontus'};

    if (scalar(@params) < 1) {
        return lib_lang::get_message('urlf5_error_delete_params');
    }

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        lib_stdlog::print_err(lib_lang::get_message('urlf5_error_delete') , " [".$base->errstr."]\n");
        return lib_lang::get_message('urlf5_error_delete') . " [".$base->errstr."]";
    } else {
        return '';
    }
}

sub actualizar_url {
    my $base = $_[0];
    my %options = %{$_[1]};
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "update $urlf5_table set urlf5_url = ? where 1=1 ";
    my @params = (lc($options{'url'}));

    if (defined($options{'type'}) && $options{'type'} ne '') {
        $sql .= " and urlf5_type = ?";
        push @params, $options{'type'};
    }
    if (defined($options{'file'}) && $options{'file'} ne '') {
        $sql .= " and urlf5_file = ?";
        push @params, $options{'file'};
    }
    if (defined($options{'subtype'}) && $options{'subtype'} ne '') {
        $sql .= " and urlf5_subtype = ?";
        push @params, $options{'subtype'};
    }
    if (defined($options{'subtype_id'}) && $options{'subtype_id'} ne '') {
        $sql .= " and urlf5_subtypeid = ?";
        push @params, $options{'subtype_id'};
    }
    $sql .= " and urlf5_prontus = ?";
    push @params, $options{'prontus'};

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        lib_stdlog::print_err(lib_lang::get_message('urlf5_error_update'), " [".$base->errstr."]\n");
        return  lib_lang::get_message('urlf5_error_update') . " [".$base->errstr."]";
    }
}

# Si la url ya existe le agrega un numero al final.
# Si supera los 50 intentos, le agrega un random.
sub verifica_url {
    my ($base, $url) = @_;
    my $nuevo_url = $url;
    my $do_random = 1;

    # Si no existe devuelve altiro la misma.
    if (!&existe_url($base, $url)) {
        return $url;
    }

    # Si la url es mas larga de lo permitido se achica dos caracteres para poder
    # agregar el numerito.
    if (length($url) >= $prontus_varglb::URL_MAX_LENGTH) {
        $nuevo_url = substr($nuevo_url, 0, length($nuevo_url) - 3);
    }

    my $index_from = 0;

    if ($nuevo_url =~ /^(.*?)\-(\d+)$/) {
        $nuevo_url = $1;
        $index_from = $2; # para partir de este numero.
    }

    for (my $x = $index_from; $x <= 50; $x++) {
        my $temp_url = "$nuevo_url-$x"; # /el-url-friendly-1

        if (!&existe_url($base, $temp_url)) { # se termina si no existe este nuevo url.
            $url = $temp_url;
            $do_random = 0;
            last;
        }
    }

    # Esto quiere decir que todas las urls consultadas existieron.
    if ($do_random) {
        if (length($url) >= $prontus_varglb::URL_MAX_LENGTH) {
            $url = substr($url, 0, length($url) - 5);
        }

        $url = "$url-" . glib_str_03::random_string(5);
    }

    return $url;
}

sub existe_url {
    my ($base, $url) = @_;

    my $count;
    my @params = ($url);
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select count(*) as count from $urlf5_table where urlf5_url = ?";

    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count));
    $salida->fetch;
    $salida->finish;

    return $count;
}

sub get_url_data {
    my ($base, $url) = @_;

    my @params = ($url);
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select * from $urlf5_table where urlf5_url = ?";

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    my $row = $salida->fetchrow_hashref;
    $salida->finish;

    return $row;
}

sub get_url_artic {
    my ($base, $ts) = @_;
    my ($sql, $salida, $url);
    my @params = ($ts);
    my $urlf5_table = lib_utils::get_table('urlf5');
    $sql = "select urlf5_url from $urlf5_table where urlf5_type = 'artic' and urlf5_subtype = 'artic' and urlf5_subtypeid = ?";

    $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($url));
    $salida->fetch;
    $salida->finish;

    return $url;
}

sub get_all_url_list {
    my $base = $_[0];
    my %options = %{$_[1]};
    my @urllist;
    my @params;

    my $filtro = '';
    if ($options{'type'} ne 'all') {
        $filtro .= "where urlf5_type = ?";
        push @params, $options{'type'};
    }

    if ($options{'search'} ne '') {
        if ($filtro eq '') {
            $filtro .= "where";
        } else {
            $filtro .= " and";
        }
        $filtro .= " (urlf5_url like ? or urlf5_file like ?)";
        push @params, '%'.$options{'search'}.'%', '%'.$options{'search'}.'%';
    }

    if ($options{'prontus'} ne '') {
        if ($filtro eq '') {
            $filtro .= "where";
        } else {
            $filtro .= " and";
        }
        $filtro .= " urlf5_prontus = ?";
        push @params, $options{'prontus'};
    }

    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select count(urlf5_id) from $urlf5_table $filtro";
    my $total;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($total));
    $salida->fetch;
    $salida->finish;

    $sql = "select urlf5_id, urlf5_type, urlf5_subtype, urlf5_subtypeid, urlf5_file, urlf5_url, urlf5_prontus from $urlf5_table $filtro";
    if ($options{'order'} eq 'u' || $options{'order'} eq '') {
        $sql .= " order by urlf5_url asc, urlf5_file asc";
    } elsif ($options{'order'} eq 'd') {
        $sql .= " order by urlf5_file asc, urlf5_url asc";
    } elsif ($options{'order'} eq 't') {
        $sql .= " order by urlf5_type asc, urlf5_url asc";
    }

    # calculo de paginación
    my $item_x_page = 20;
    my $start = ( ($options{'page'} - 1 ) * $item_x_page);
    if ($options{'page'} > 0) {
        $sql .= " limit $start,$item_x_page";
    }

    # variables de salida
    my ($urlf5_id, $urlf5_type, $urlf5_subtype, $urlf5_subtypeid, $urlf5_file, $urlf5_url, $urlf5_prontus);
    $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($urlf5_id, $urlf5_type, $urlf5_subtype, $urlf5_subtypeid, $urlf5_file, $urlf5_url, $urlf5_prontus));
    while ($salida->fetch) {
        push (@urllist, {'id' => $urlf5_id, 'url' => $urlf5_url, 'type' => $urlf5_type, 'subtype' => $urlf5_subtype, 'subtypeid' => $urlf5_subtypeid, 'file' => $urlf5_file, 'prontus' => $urlf5_prontus});
    }
    $salida->finish;

    my %result = (
        'list' => \@urllist,
        'pagination' => {
            'total' => $total,
            'start' => $start + 1,
            'page' => $options{'page'},
            'current' => scalar @urllist
        }
    );

    return \%result;
}

sub get_url_list {
    my $base = $_[0];
    my %options = %{$_[1]};
    my %urllist;
    my @params = ($options{'type'});
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select urlf5_file, urlf5_url from $urlf5_table where urlf5_type = ?";

    if ($options{'subtype'} ne '') {
        $sql .= " and urlf5_subtype = ?";
        push @params, $options{'subtype'};

        if ($options{'subtype_id'} ne '') {
            $sql .= " and urlf5_subtypeid = ?";
            push @params, $options{'subtype_id'};
        } else {
            return \%urllist;
        }
    }

    my ($urlf5_file, $urlf5_url);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($urlf5_file, $urlf5_url));
    while ($salida->fetch) {
        $urllist{$urlf5_file} = $urlf5_url;
    }
    $salida->finish;

    return \%urllist;
}

sub get_file_data {
    my ($base, $url) = @_;
    my @params = ($url);
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select * from $urlf5_table where urlf5_file = ?";

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    my $row = $salida->fetchrow_hashref;
    $salida->finish;

    return $row;
}

sub get_link_tax {
    my ($base, $file, $nro_pagina, $mv) = @_;

    my $file_orig = $file;
    # En la tabla se guarda la pagina 1. Hay que cambiar la pagina por 1 y mantener extension.
    $file =~ s/_\d+(\.\w+)$/_1$1/;
    # lib_stdlog::print_err("file[$file]\n");

    my $urlf5_url;
    my @params = ('tax', $file);
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select urlf5_url from $urlf5_table where urlf5_type = ? and urlf5_file = ?";
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($urlf5_url));

    $salida->fetch;
    $salida->finish;
    # lib_stdlog::print_err("get_link_tax pagina[$nro_pagina] friendly[$urlf5_url] -> file[$file_orig]\n");

    my $view = '';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $mv ne '') {
        $view = "/$mv";
    }

    if (!$urlf5_url) {
        if ($mv) {
            $file_orig =~ s|(.+/tax/port/[^/]+)(/.+)|$1-$mv$2|;
        }
        return $file_orig;
    } elsif ($nro_pagina) {
        return "$view$urlf5_url/p/$nro_pagina";
    } else {
        return "$view$urlf5_url/p/1";
    }
}

sub get_link_tag {
    my ($base, $file, $nro_pagina) = @_;

    my $file_orig = $file;
    # En la tabla se guarda la pagina 1. Hay que cambiar la pagina por 1 y mantener extension.
    $file =~ s/_\d+(\.\w+)$/_1$1/;

    my $urlf5_url;
    my @params = ('tag', $file);
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select urlf5_url from $urlf5_table where urlf5_type = ? and urlf5_file = ?";
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($urlf5_url));

    $salida->fetch;
    $salida->finish;
    # lib_stdlog::print_err("get_link_tag pagina[$nro_pagina] friendly[$urlf5_url] -> file[$file_orig]\n");

    if (!$urlf5_url) {
        return $file_orig;
    } else {
        if ($nro_pagina) {
            return "$urlf5_url/p/$nro_pagina";
        } else {
            return "$urlf5_url/p/1";
        }
    }
}

sub make_url {
    my ($titular, $seccion, $tema, $subtema) = @_;
    my $url = '';

    $titular = lib_prontus::ajusta_titular_friendly($titular);
    if ($seccion eq '') {
        $url = "/$titular";
    } else {
        $seccion = lib_prontus::ajusta_titular_friendly($seccion);

        if ($tema ne '') {
            $tema = "/" . lib_prontus::ajusta_titular_friendly($tema);
        }

        if ($subtema ne '') {
            $subtema = "/" . lib_prontus::ajusta_titular_friendly($subtema);
        }

        $url = "/$seccion$tema$subtema/$titular";
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URL_PREFIX'} ne '') {
        # Hay un prefijo, agregarlo antes de la url para verificar.
        $url = "/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URL_PREFIX'}$url";
    }

    # reemplaza //+ por /
    $url =~ s|//+|/|g;

    # largo maximo de url
    if (length($url) > $prontus_varglb::URL_MAX_LENGTH) {
        $url = substr($url, 0, $prontus_varglb::URL_MAX_LENGTH);
    }

    return lc($url);
}

# $path debe ser el path relativo desde el document root
sub guardar_url {
    my ($bd, $url, $path, $new_type, $new_subtype, $new_subtype_id, $prontus_id) = @_;

    # buscamos si este path ya tiene asignada una url
    my ($old_url, $type, $subtype, $prontus) = ('', '', '');
    my $urlf5_table = lib_utils::get_table('urlf5');
    if ($new_type eq 'redirect') {
        my $sql = "select urlf5_url, urlf5_type, urlf5_subtype, urlf5_prontus from $urlf5_table where urlf5_file = ? and urlf5_url = ?";
        my @params = ($path, $url);
        my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($old_url, $type, $subtype, $prontus));
        $salida->fetch;
        $salida->finish;
        # archivo existe, entonces debe ser un rewrite
        # el sistema usado no permite diferenciar si se quiere un redirect para un archivo existente
        # solo permite redirects para urls inexistentes
        if (-f "$prontus_varglb::DIR_SERVER$path") {
            # subtipo de redirect es 200, un rewrite, se cambia aqui para simplificar GUI
            $new_subtype = 200;
        } elsif ($new_subtype == 200 && !-f "$prontus_varglb::DIR_SERVER$path" ){
            # si se intenta un rewrite 200, pero no existe el archivo, debe ser 301 un redirect
            # es posible que esta url no sea valida tampoco
            $new_subtype = 301;
        }
    } else {
        my $sql = "select urlf5_url, urlf5_type, urlf5_subtype, urlf5_prontus from $urlf5_table where urlf5_file = ?";
        my @params = ($path);
        my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($old_url, $type, $subtype, $prontus));
        $salida->fetch;
        $salida->finish;
    }

    my $symlink;
    my $fullPath;
    lib_stdlog::print_err("URL [$new_subtype] eq [$subtype] && [$new_type] eq [$type] && [$old_url] eq [$url]\n");
    # el path tiene url, la url es la misma y ya existe en bd, solo trata de recrear symlinks
    if ($new_type eq $type && $old_url eq $url) {
        # se intenta recrear el link, en caso de cambio de tipo, se borra y se crea con el nuevo tipo
        my ($res, $data, $msg);
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
            if ($url ne '/') {
                ($res, $data, $msg) = lib_friendly::insertFriendly($url, $path, $new_subtype);
            } else {
                ($res, $data, $msg) = (0, '/', '');
            }
            lib_stdlog::print_err("Params ($url, $path, $new_subtype) insertFriendly($res, $data, $msg)\n");
        } else {
            # siempre se debe actalizar bd si no se usan symlinks
            $res = 1;
        }

        if ($res == 1) {
            # la url y el path de destino son el mismo pero cambia el tipo de redirect
            # hay que actualizar la bd con el nuevo tipo
            if ($new_subtype ne $subtype) {
                my $stamp = glib_hrfec_03::get_dtime_pack4();
                my @params = ($new_subtype, $stamp, 'redirect', $path, $url, $prontus_id);
                my $sql = "update urlf5 set urlf5_subtype = ?, urlf5_subtypeid = ? where urlf5_type = ? and urlf5_file = ? and urlf5_url = ? and urlf5_prontus = ?";
                my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
                $salida->finish;

                if ($bd->errstr) {
                    lib_stdlog::print_err(lib_lang::get_message('urlf5_error_sql'),  " [$sql] [".$bd->errstr."]\n");
                    return (0, lib_lang::get_message('urlf5_error_save_update'));
                }
            }

            return (1, $data);
        } elsif ($res == 0) {
            $symlink = "$prontus_varglb::DIR_SERVER$url/index.html";
            $symlink =~ s|//+|/|g;
            $fullPath = "$prontus_varglb::DIR_SERVER$path";
            $fullPath =~ s|//+|/|g;
            # borrar link simbolico
            unlink $symlink;

            # recrear link
            my $cmd = "ln -sr $fullPath $symlink 2>&1";
            my $result = `$cmd`;
            return (1, $path);
        } else {
            lib_stdlog::print_err("Error ($url, $path, $new_subtype) insertFriendly($res, $data, ". lib_friendly::get_message($msg). ")\n");
            return (0, lib_lang::get_message('urlf5_error_create_url') . " 1:[". lib_friendly::get_message($msg). "]");
        }
    }
    # el path tiene asignada una url pero no es la misma que la que se guarda, hay que actualizar y borrar el link anterior
    elsif (!defined($old_url) || $old_url ne $url) {
        # nueva url, revisamos si hay conflicto
        my @params = ($url);
        my $sql = "select count(*) from $urlf5_table where urlf5_url = ?";
        my $count = 0;
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params, \($count));
        $salida->fetch;
        $salida->finish;

        if ($bd->errstr) {
            lib_stdlog::print_err(lib_lang::get_message('urlf5_error_sql'), " [$sql] [".$bd->errstr."]\n");
            return (0, lib_lang::get_message('urlf5_error_sql_msg'));
        }

        my $newtypeDir = 0;
        my ($res, $data, $msg);
        # no existe conflicto en BD revisamos en links friendly
        if ($count == 0 && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
            # Chequea URL.
            if ($url ne '/') {
                ($res, $data, $msg) = &lib_friendly::readFriendly($url);
            } else {
                # si la url se esta asignando a la raiz del sitio
                ($res, $data, $msg) = (5, '/', '');
            }
            if ($res == 5) {
                $newtypeDir = 1;
                $symlink = "$prontus_varglb::DIR_SERVER$url/index.html";
                $symlink =~ s|//+|/|g;
                $fullPath = "$prontus_varglb::DIR_SERVER$path";
                $fullPath =~ s|//+|/|g;

                if (-l $symlink) {
                    my $linkPath = readlink $symlink;
                    # Elimina referencias a directorios hacia atras.
                    $linkPath =~ s/^(\.\.\/)+//;
                    $linkPath = "$prontus_varglb::DIR_SERVER$url/$linkPath";
                    $linkPath =~ s|//+|/|g;

                    if ($fullPath ne $linkPath ) {
                        lib_stdlog::print_err("[$url] ", lib_lang::get_message('urlf5_error_link_conflict')," [$symlink, $linkPath]\n");
                        return (0, lib_friendly::get_message('urlf5_error_url_exists'));
                    }
                } elsif (-f $symlink) {
                    lib_stdlog::print_err("[$url] ", lib_lang::get_message('urlf5_error_file_conflict') , " [$symlink]\n");
                    return (0, lib_friendly::get_message('urlf5_error_url_exists'));
                }
            }
            # hay un link en conflicto
            elsif ($res != 0) {
                lib_stdlog::print_err("[$url] ", lib_lang::get_message('urlf5_error_link_conflict')," [$res, $data, ", lib_friendly::get_message($msg), "]\n");
                return (0, lib_friendly::get_message('urlf5_error_url_exists'));
            }
        } elsif ($count != 0) {
            # existe un conflicto en BD
            lib_stdlog::print_err("[$url] ",lib_friendly::get_message('urlf5_error_db_conflict'),"\n");
            return (0, lib_friendly::get_message('urlf5_error_url_exists'));
        }

        my $typeDir = 0;
        # Chequea URL Antigua
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
            if ($old_url ne '/') {
                ($res, $data, $msg) = lib_friendly::readFriendly($old_url);
            } else {
                # si la url se esta asignando a la raiz del sitio
                ($res, $data, $msg) = (5, '/', '');
            }

            if ($res == 5) {
                $typeDir = 1;
            }

            # no existe ningun conflicto, se intenta guardar el nuevo link
            # borra antigua url
            if ($old_url ne '') {
                if (!$typeDir) {
                    ($res, $msg) = &lib_friendly::deleteFriendly($old_url);
                    lib_stdlog::print_err("[Delete old url][$old_url][$res]\n");
                    if ($res == 0) {
                        return (0, lib_lang::get_message('urlf5_error_create_url') . " 2:[". lib_friendly::get_message($msg). "]");
                    }
                    lib_prontus::send_signal('reset_url_cache');
                } else {
                    my $oldLink = "$prontus_varglb::DIR_SERVER$old_url/index.html";
                    $oldLink =~ s|//+|/|g;
                    # borrar link simbolico
                    unless(unlink $oldLink) {
                        return (0, "Error al crear la URL 2-1:[$msg]");
                    };
                    lib_stdlog::print_err("[Delete old url][$oldLink][index link]\n");
                }
            }

            # crea nuevo
            if (!$newtypeDir) {
                ($res, $data, $msg) = lib_friendly::insertFriendly($url, $path, $new_subtype);
            } else {
                my $cmd = "ln -sr $fullPath $symlink 2>&1";
                my $result = `$cmd`;
                $data = $path;
                # corresponde a creacion ok
                $res = 1;
            }
        } else {
            # no usa symlinks asi que se marca como correcta la operacion para actualizar bd
            $res = 1;
        }

        lib_stdlog::print_err("[New URL][$url][$data][$res][$msg]\n");
        if ($res == 1) {
            my %options = (
                'type'          => $new_type,
                'subtype'       => $new_subtype,
                'subtype_id'    => $new_subtype_id,
                'file'          => $path,
                'url'           => $url,
                'prontus'       => $prontus_id
            );

            if ($old_url ne '') {
                &actualizar_url($bd, \%options);
            } else {
                &insert_url($bd, \%options);
            }
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_USE_SYMLINK'} ne 'SI') {
                lib_prontus::send_signal('reset_url_cache');
            }

            # Si es tax, regenerar json de tax para actualizar las friendly v5.
            if ($new_type eq 'tax') {
                lib_tax::generar_json($bd);
            } elsif ($new_type eq 'tag') {
                # Si es un tag, regenerar json de tags para actualizar las friendly v5.
                lib_tags::generar_json($bd);
            }

            return (1, $data);
        } else {
            return (0, lib_lang::get_message('urlf5_error_create_url') . " 3:[". lib_friendly::get_message($msg). "]");
        }
    } else {
        return (0, lib_lang::get_message('urlf5_error_create_url') . " 4");
    }
}

sub valida_type {
    my $type = $_[0];
    return $type =~ /^tax|tag|list|port|artic|redirect$/;
}

sub get_url_art {
    my ($base, $ts, $prontus_id) = @_;
    my $urlf5_table = lib_utils::get_table('urlf5');
    my $sql = "select urlf5_url, urlf5_subtypeid from $urlf5_table where urlf5_subtypeid = ? and urlf5_prontus = ?";
    my @params = ($ts, $prontus_id);
    my ($friendlyAntigua, $artTs);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($friendlyAntigua, $artTs));
    $salida->fetch;
    $salida->finish;
    return ($friendlyAntigua, $artTs);
}

1;
