#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# funciones para manejo de usuarios
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use prontus_varglb;
# use glib_dbi_03;
# use lib_bd;
# use glib_str_03; # para load_all_user_data

use strict;
use utf8;

package lib_users;

# Chequea que el username pasado por parametro exista en el hash de users y retorna el users_id y el users_perfil.
# Si no se logra autenticar, retorna el id vacio y en el perfil retorna el mensaje de error.
sub check_user_session {
    my $redir = $_[0];    # redireccion en caso de que no se detecte una sesion activa.

    my $sess_obj = Session->new('prontus_id'    => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'},
                                'document_root' => $prontus_varglb::DIR_SERVER
                               ) || return ('', lib_lang::get_interpolated_message('global_error_session_obj', $Session::ERR));

    if ($sess_obj->{id_session} eq '' || $sess_obj->{username} eq '' || $sess_obj->{psw_hash} eq '') {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\r\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\r\n\r\n";
            exit;
        } else {
            # lib_stdlog::print_err("No se detectó una sesión activa\n");
            return ('',  lib_lang::get_message('global_error_no_session'), $sess_obj);
        }
    }

    # solo aca se inicializa esta variable
    $prontus_varglb::USERS_USR = $sess_obj->{username};

    my ($res, $err_msg) = &load_user_data($sess_obj->{username});
    if (!$res) {
        return ('', lib_lang::get_interpolated_message('global_error_user_data', $err_msg), $sess_obj);
    }

    my $login_result = prontus_auth::check_valid_hash_psw($sess_obj->{username}, $sess_obj->{psw_hash});

    # caso mas comun, usuario logeado correctamente
    if ($login_result == 1) {
        # Regenera la sesion para evitar que sea borrada por el gc.
        $sess_obj->rejuvenece_sesion();

        # valida que el usuario tenga acceso al prontus que intenta acceder
        if ($prontus_auth::USERS_PERFIL != 1 &&
            !defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$prontus_auth::USERS_USR_ID}{$prontus_varglb::PRONTUS_ID})) {
            return ('', lib_lang::get_message('global_access_restricted_message'), $sess_obj);
        }

        # carga los permisos de portadas y fids del usuario
        ($res, $err_msg) = &load_user_template_access($sess_obj->{username});
        if (!$res) {
            return ('', lib_lang::get_interpolated_message('global_error_user_permissions', $err_msg), $sess_obj);
        }

        lib_profile::load_profiles($prontus_auth::USERS_PERFIL); # cargamos la info de perfiles en varglb{PROFILES}

        my $language = lib_lang::get_user_language(); # validamos que el lenguaje efectivamente exista.
        lib_lang::load_translation_table($language);

        return ($prontus_auth::USERS_USR_ID, $prontus_auth::USERS_PERFIL, $sess_obj);
    }
    # contraseña expirada
    if ($login_result == 3) {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
            exit;
        } else {
            my $message = lib_lang::get_interpolated_message('global_error_password_expired', $sess_obj->{username});
            lib_stdlog::print_err("$message\n");
            return ('', lib_lang::get_message('global_error_password_expired_public'), $sess_obj);
        }
    }
    # contraseña invalida
    if ($login_result == 0) {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
            exit;
        } else {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('global_error_user_password', $sess_obj->{username})."\n");
            return ('', lib_lang::get_message('global_error_user_password_public'));
        }
    }
}

# cierra session del usuario en prontus
sub cerrar_sesion {
    my $sess_obj = Session->new('prontus_id'    => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'},
                                'document_root' => $prontus_varglb::DIR_SERVER
                        ) || return ( '', lib_lang::get_interpolated_message('global_error_session_obj', $Session::ERR));

    $sess_obj->end_session();

    return ($sess_obj->{username}, $sess_obj->{id_session});
}

# Verifica si un usuario existe en Prontus según un parámetro pasado.
# Implementada para sso/prontus_oidc_callback.cgi
# Parámetros:
#   field: el campo por el cual buscar. Por defecto es el nombre de usuario, puede ser
#   user, email, id.
#   all: puede ser vacío u "all". All implica considerar también a los usuarios desactivados.
# Retorna:
#   array (users_id, users_usr, users_perfil), o (0, error_msg, '').
sub check_user_simple {
    my ($field, $value, $all) = @_;

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    my $sub_query = " users_usr = ? ";
    if ($field eq 'email') {
        $sub_query = " users_email = ? ";
    } elsif ($field eq 'id') {
        $sub_query = " users_id = ? ";
    }

    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $exclude_deleted = ' and users_deleted = 0 ';
    $exclude_deleted = '' if $all eq 'all';
    my $sql = "select * from $users_table$users_table where $sub_query $exclude_deleted limit 1";
    my @params = $value;
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);

    if (my $row = $salida->fetchrow_hashref) {
        if ($row->{'users_id'}) {
            lib_profile::load_profiles($prontus_auth::USERS_PERFIL); # cargamos la info de perfiles en varglb{PROFILES}
            return ($row->{'users_id'}, $row->{'users_usr'}, $row->{'users_perfil'});
        }
    }
    return (0, lib_lang::get_message('global_error_user_not_found_public'), '');
}

# lista las passwords antiguas para un usuario determinado
sub list_old_passwords {
    my $user_id  =  $_[0];

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    my $old_psw_table = lib_utils::get_table('old_psw');
    my $sql = "select users_psw from $old_psw_table where users_id = ?";
    my $psw;
    my @params = ($user_id);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($psw));

    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    my @pws_list;
    while ($salida->fetch) {
        push(@pws_list, $psw);
    }
    $salida->finish;

    return @pws_list;
}

# guarda las passwords antiguas
# se asume que esta password no existe, por lo tanto es distinta a todas las anteriores
sub store_old_passwords {
    if ($prontus_auth::PSW_TO_REMEMBER == 0) {
        return;
    }
    my $user_id  =  $_[0];
    my $old_pass =  $_[1];

    if ($old_pass eq '') {
        return (0, "Password vacía");
    }

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    my $old_psw_table = lib_utils::get_table('old_psw');
    my $sql = "insert into $old_psw_table values(?,?,?,?)";
    my @params;
    push(@params, undef);
    push(@params, $user_id);
    push(@params, $old_pass);
    push(@params, time);

    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    $sql = "select count(*) from $old_psw_table where users_id = ?";
    my $total_count;
    @params = ($user_id);
    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($total_count));
    $salida->fetch;
    $salida->finish;
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    $prontus_auth::PSW_TO_REMEMBER--;
    # debemos borrar las contraseñas mas antiguas
    while ($total_count > $prontus_auth::PSW_TO_REMEMBER) {
        # buscamos la mas antigua
        $sql = "select old_psw_id from $old_psw_table where users_id = ? order by psw_epoch limit 1";
        my $old_id;
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($old_id));
        $salida->fetch;
        $salida->finish;
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        @params = ($old_id);
        $sql = "delete from $old_psw_table where old_psw_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # revisamos si hay que eliminar mas
        $sql = "select count(*) from $old_psw_table where users_id = ?";
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($total_count));
        $salida->fetch;
        $salida->finish;
    }

    return (1, "");
}

sub update_last_login {
    my $user_id = shift;
    return (0, "No se recibió un ID de usuario válido") if (!$user_id);

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, $msg_err_bd);
    }
    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $sql = "update $users_table set users_fec_last_login = current_timestamp where users_id = ?";
    my @params;
    push(@params, $user_id);

    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    $salida->finish;
    return (1, "");
}

# guarda los datos de un usuario en bd
sub save_user_data {
    my $user_id = $_[0];

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    my $sql;
    my $idvalue = 0;
    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    if ($user_id eq 'new_user') {
        $sql = "insert into $users_table (users_id, users_nom, users_usr, users_psw, users_profile_id, users_email, users_lang, users_exp_days, users_fec_exp, users_renew_psw, users_2fa, users_oidc) values(?,?,?,?,?,?,?,?,?,?,?,?)";
        my @params;
        push(@params, undef);
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_NOM'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EMAIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_LANG'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EXP_DAYS'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_FEC_EXP'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_RENEW_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_2FA'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_OIDC'});

        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $sql = "select last_insert_id()";
        $salida = glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($idvalue));
        $salida->fetch;
        $salida->finish;

        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # pasamos de vuelta el id del nuevo usuario.
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'}} = $idvalue;
    } else {
        $sql = "update $users_table set users_nom = ?, users_usr = ?, users_psw = ?, users_profile_id = ?, users_email = ?, users_lang = ?, users_exp_days = ?, users_fec_exp = ?, users_renew_psw = ?, users_2fa = ?, users_oidc = ? where users_id = ?";
        my @params;
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_NOM'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EMAIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_LANG'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EXP_DAYS'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_FEC_EXP'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_RENEW_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_2FA'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_OIDC'});
        push(@params, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $idvalue = $user_id;
    }
    return ($idvalue, '');
}

# guarda los permisos de acceso a portadas y articulos de los usuarios
sub save_user_permissions {
    my ($user_id, $idvalue) = @_;

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, $msg_err_bd);
    }

    if (!$idvalue) {
        $idvalue = $user_id;
    }

    # idvalue es el valor del usuario que se insertara en bd
    # los datos a insertar estan asociados a "user_id" que puede ser "new_user"
    # usuario administrador o perfil administrador no guarda asociaciones de articulos y portadas
    if ($idvalue && $idvalue > 1) {
        my $art_users_table = lib_utils::get_table('art_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        my @params = ($idvalue);
        my $sql = "delete from $art_users_table where art_user_id = ?";

        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        my $port_users_table = lib_utils::get_table('port_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        $sql = "delete from $port_users_table where port_user_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        my $prontus_users_table = lib_utils::get_table('prontus_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        $sql = "delete from $prontus_users_table where prontus_user_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        if ($prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'} > 1) {
            foreach my $prontus_id (keys %prontus_varglb::CONFIG) {
                foreach my $art (keys %{$prontus_varglb::CONFIG{$prontus_id}{'ARTUSERS'}{$user_id}}) {
                    $sql = "insert into $art_users_table values(?, ?, ?)";
                    my @insert_data = ($idvalue, $art, $prontus_id);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }

                foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_id}{'PORTUSERS'}{$user_id}}) {
                    $sql = "insert into $port_users_table values(?, ?, ?)";
                    my @insert_data = ($idvalue, $port, $prontus_id);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }

                foreach my $prontus (keys %{$prontus_varglb::CONFIG{$prontus_id}{'PRONTUS_USERS'}{$user_id}}) {
                    $sql = "insert into $prontus_users_table values(?, ?)";
                    my @insert_data = ($idvalue, $prontus);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }
            }
        }
    }
    return (1, "");
}

# actualiza el idioma de un usuario en bd
sub update_user_language {
    my $user_id = $_[0];
    my $language = $_[1];

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }
    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $sql = "update $users_table set users_lang = ? where users_id = ?";
    my @params;

    push(@params, $language);
    push(@params, $user_id);
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    return (1, "");
}

# borra un usuario en bd
sub delete_user {
    my $user_id = int($_[0]);

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    # usuario admin #1 no se puede borrar
    if ($user_id > 1 ) {
        my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        my $sql = "update $users_table set users_psw = ?, users_email = ?, users_exp_days = ?, users_fec_exp = ?, users_renew_psw = ?, users_2fa = ?, users_oidc = ?, users_deleted = ? where users_id = ?";
        my @params = ('', '', 0, 0, 0, 0, 0, 1, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # borra asociaciones de articulos y portadas
        my $art_users_table = lib_utils::get_table('art_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        @params = ($user_id);
        $sql = "delete from $art_users_table where art_user_id = ?";

        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
        my $port_users_table = lib_utils::get_table('port_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        $sql = "delete from $port_users_table where port_user_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        my $prontus_users_table = lib_utils::get_table('prontus_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        $sql = "delete from $prontus_users_table where prontus_user_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # borramos claves antiguas almacenadas
        @params = ($user_id);
        my $old_psw_table = lib_utils::get_table('old_psw');
        $sql = "delete from $old_psw_table where users_id = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
    } else {
        return (0, lib_lang::get_interpolated_message('global_error_username_deletion', $user_id));
    }

    return (1, "");
}

# recupera un usuario en bd
sub undelete_user {
    my $user_id = int($_[0]);

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    # usuario admin #1 no se puede borrar
    if ($user_id > 1 ) {
        my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        my $sql = "update $users_table set users_psw = ?, users_email = ?, users_exp_days = ?, users_fec_exp = ?, users_renew_psw = ?, users_2fa = ?, users_oidc = ?, users_deleted = ? where users_id = ?";
        my @params;
        push(@params, '');
        push(@params, '');
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
    } else {
        return (0, lib_lang::get_interpolated_message('global_error_username_recovery', $user_id));

    }

    return (1, "");
}

# carga los datos de todos los usaarios desde bd
# requiere glib_str_03
sub load_all_user_data {
    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $sql = "select * from $users_table";
    my $salida = glib_dbi_03::ejecutar_sql($bd, $sql);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    # nuevo hash para prontus
    $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID} = ();
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}= ();
    while (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);

        # nuevo hash para prontus
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_NOM'} = $row->{'users_nom'};
        # eliminar caracteres invalidos
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_NOM'} =~ s/[<>\/"'&;%]+//g;

        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_USR'} = $row->{'users_usr'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_PSW'} = $row->{'users_psw'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_PERFIL'} = $row->{'users_profile_id'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_EMAIL'} = $row->{'users_email'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_LANG'} = $row->{'users_lang'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_EXP_DAYS'} = $row->{'users_exp_days'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_EXP'} = $row->{'users_fec_exp'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_CREACION'} = $row->{'users_fec_creacion'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'users_fec_last_login'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_RENEW_PSW'} = $row->{'users_renew_psw'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_2FA'} = $row->{'users_2fa'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_OIDC'} = $row->{'users_oidc'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_DELETED'} = $row->{'users_deleted'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$row->{'users_usr'}} = $row->{'users_id'};
    }

    return (1, "");
}

sub load_user_template_access {
    my $user = $_[0];

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$user} eq '1') {
        return (1, "");
    }

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_user_db_connection', $msg_err_bd));
    }

    my $user_id = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$user};
    foreach my $prontus_id (keys %prontus_varglb::CONFIG) {
        $prontus_varglb::CONFIG{$prontus_id}{'ARTUSERS'}{$user_id} = undef;
        $prontus_varglb::CONFIG{$prontus_id}{'PORTUSERS'}{$user_id} = undef;
    }

    my $art_users_table = lib_utils::get_table('art_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $sql = "select * from $art_users_table where art_user_id = ?";
    my @params = ($user_id);
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    while (my $row = $salida->fetchrow_hashref) {
        $prontus_varglb::CONFIG{$row->{'art_prontus_id'}}{'ARTUSERS'}{$row->{'art_user_id'}}{$row->{'art_template'}} = 1;
    }

    my $port_users_table = lib_utils::get_table('port_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    $sql = "select * from $port_users_table where port_user_id = ?";
    $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    while (my $row = $salida->fetchrow_hashref) {
        $prontus_varglb::CONFIG{$row->{'port_prontus_id'}}{'PORTUSERS'}{$row->{'port_user_id'}}{$row->{'port_template'}} = 1;
    }

    return (1, "");
}

# carga los datos de un usuario desde bd
sub load_user_data {
    my ($user, $is_id) = @_;

    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return (0, lib_lang::get_interpolated_message('global_error_db_connection', $msg_err_bd));
    }

    $user =~ s/[^a-z\_\-\.0-9@]//;
    my $users_table = lib_utils::get_table('users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
    my $sql = "select * from $users_table where users_usr = ? limit 1";
    if (defined($is_id)) {
        $sql = "select * from $users_table where users_id = ? limit 1";
    }
    my @params = ($user);
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);

    if (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);

        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_NOM'} = $row->{'users_nom'};
        # eliminar caracteres invalidos
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_NOM'} =~ s/[<>\/"'&;%]+//g;

        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_USR'} = $row->{'users_usr'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_PSW'} = $row->{'users_psw'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_PERFIL'} = $row->{'users_profile_id'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_EMAIL'} = $row->{'users_email'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_LANG'} = $row->{'users_lang'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_EXP_DAYS'} = $row->{'users_exp_days'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_EXP'} = $row->{'users_fec_exp'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_CREACION'} = $row->{'users_fec_creacion'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'users_fec_last_login'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_RENEW_PSW'} = $row->{'users_renew_psw'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_2FA'} = $row->{'users_2fa'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_OIDC'} = $row->{'users_oidc'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'users_id'}}{'USERS_DELETED'} = $row->{'users_deleted'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$row->{'users_usr'}} = $row->{'users_id'};

        my $user_id = $row->{'users_id'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$user_id} = undef;
        my $prontus_users_table = lib_utils::get_table('prontus_users', $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        $sql = "select prontus_id from $prontus_users_table where prontus_user_id = ?";
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        while ($row = $salida->fetchrow_hashref) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$user_id}{$row->{'prontus_id'}} = 1;
        }
        $salida->finish;

    } else {
        return (0, lib_lang::get_interpolated_message('global_error_user_not_found', $user, $is_id));
    }
    return (1, "");
}

1;
