#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Utilidades generales para prontus
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use glib_fildir_03;
# use prontus_varglb;
# use glib_hrfec_03; para write_log y call_system_and_location
# use lib_waitlock; para write_log

use strict;
use utf8;

package lib_utils;

# Setea utf8 para stdout y stderr
sub setUp {
    binmode STDERR, ':encoding(UTF-8)';
    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    my $language = lib_lang::get_language();
    lib_lang::load_translation_table($language);
}

sub valida_prontus {
    my $prontus_id = $_[0];
    my $alerts_off = $_[1] || 0;

    if (defined($prontus_varglb::CONFIG{$prontus_id})) {
        return 1;
    }

    if ($prontus_id !~ /^[\w\_\-]+$/) {
        lib_stdlog::print_err(lib_lang::get_message('utils_invalid_prontus_id') , "[$prontus_id]\n") if (!$alerts_off);
        return 0;
    }

    # chequeamos que exista el prontus
    if (!-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id.cfg") {
        lib_stdlog::print_err(lib_lang::get_message('utils_invalid_prontus_conf'), "\n") if (!$alerts_off);
        return 0;
    }
    return 1;
}

sub update_reload_time {
    # touch a prontus_varglb para que apache recargue la configuración y libs
    my $current_epoch = time;
    if (defined($Prontus::modperl)) {
        utime($current_epoch, $current_epoch, "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_varglb.pm");
    } elsif ($prontus_varglb::FCGI_MODE) {
        my $cfg_file = $prontus_varglb::DIR_SERVER. "/$prontus_varglb::PRONTUS_ID/cpan/$prontus_varglb::PRONTUS_ID.cfg";
        utime($current_epoch, $current_epoch, $cfg_file);
    }
}

# Carga la configuracion de otro prontus usando el id proporcionado
# devuelve $prontus_varglb::PRONTUS_ID al valor original al terminar
sub load_another_prontus_config {
    my $prontus_id = $_[0];
    my $current_prontus_id = $prontus_varglb::PRONTUS_ID;

    my $msg = &load_config_by_id($prontus_id);

    $prontus_varglb::PRONTUS_ID = $current_prontus_id;

    return $msg;
}

# Carga la configuracion de prontus usando el id proporcionado
# Retorna los valores de c/u.
# ADVERTENCIA: NO usar greedy matching /g, en los if individuales, produce errores de matcheo
# solo usar en while
sub load_config_by_id {
    if (defined($_[0]) && $_[0] ne '' && $_[0] =~ /^[\w\-]+$/){
        $prontus_varglb::PRONTUS_ID = $_[0];
    } else {
        $prontus_varglb::PRONTUS_ID = 'prontus';
        lib_stdlog::print_err(lib_lang::get_message('utils_invalid_prontus_conf') , "\n");
        return lib_lang::get_message('utils_invalid_prontus_id');
    }

    my $cfg_file = $prontus_varglb::DIR_SERVER. "/$prontus_varglb::PRONTUS_ID/cpan/$prontus_varglb::PRONTUS_ID.cfg";
    if ($prontus_varglb::CONFIG_LOADED{$prontus_varglb::PRONTUS_ID} > 0) {
        my $modtime = (stat $cfg_file)[9];
        if (defined($Prontus::modperl) ||
           ($prontus_varglb::FCGI_MODE && ($modtime == $prontus_varglb::CONFIG_LOADED{$prontus_varglb::PRONTUS_ID})) ) {
            return '';
        }
    }
    # verifica existencia de conf
    elsif (!-f $cfg_file) {
        lib_stdlog::print_err(lib_lang::get_message('utils_invalid_prontus_conf') , " [$cfg_file]\n");
        return lib_lang::get_message('utils_invalid_prontus_conf');
    }

    # Lee archivos de configuración.
    my $buffer = glib_fildir_03::read_file($cfg_file);
    if ($buffer !~ /\w/) {
        lib_stdlog::print_err(lib_lang::get_message('utils_empty_prontus_conf'), "\n");
        return lib_lang::get_message('utils_empty_prontus_conf');
    }
    # inicializa valores por defecto de variables
    foreach my $key (keys %prontus_varglb::CFG_SINGLE_VAL) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$key} = $prontus_varglb::CFG_SINGLE_VAL{$key};
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CUSTOM_DIR'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_EXCLUDE_FID'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_SUBDOMAINS'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PLTS'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'POST_PROCESO'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}} = ();
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE_LIST'}} = ();

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'} = [];

    # procesa archivo de configuracion para cargar los valores
    while($buffer =~ /(\w+)\s*=\s*["']([^"']+)["']/g) {
        if ($1 eq 'FRIENDLY_EXCLUDE_FID') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'DROPBOX_CUSTOM_DIR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'CACHE_PURGE_EXCLUDE_FID') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'MULTIVISTA') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'CACHE_SERVER_NAME') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'CACHE_PURGE_SUBDOMAINS') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'LIST_PLTS') {
            next if (length($2) > 64);
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1}{$2} = 1;
        } elsif ($1 eq 'POST_PROCESO') {
            my $value = $2;
            if ($value =~ m/([^'"\(\)]+)\(([^'"\(\)]+)\)/) {
                # ART-BORRAR | ART-GUARDAR | ART-EDITAR | PORT_GUARDAR | etc
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'POST_PROCESO'}{$1} = $2;
            }
        } elsif ($1 eq 'FIDS_CONTENIDO') {
            %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}} = map { $_ => 1 } split(',', $2);
        } elsif ($1 eq 'FIDS_CALENDAR') {
            %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}} = map { $_ => 1 } split(',', $2);
        } elsif ($1 eq 'FORM_PLTS') {
            my $value = $2;
            if ($value =~ m/(.+?):(.+?)\((.*?)\)\((.*?)?\)/) {
                next if (length($1) >64);
                my @templates = split(/;/, $3);

                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'label'} = $2;
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'plt'} = \@templates;
                my @templates2 = split(/;/, $4);
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'plt_paralela'} = \@templates2;
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'} = $1 if (!$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'});
            }
        } elsif ($1 eq 'PORT_PLTS') {
            # PORT_PLTS = 'inicio_altotrafico.html(Inicio Alto Tráfico)(inicio3.html)'
            my $value = $2;
            if ($value =~ m/(.+?)\((.*?)\)\((.*?)\)(?:\(.*?\))?/) {
                next if (length($1) >64);
                my $plantilla    = $1;    # plantilla principal de la portada
                my $nombre       = $2;    # contenido del primer parentesis, nombre de la portada
                my $extra_ports  = $3;    # segundo (): portadas paralelas, ej: inicio2.php;inicio3.php
                if ($plantilla !~ /^[a-zA-Z0-9_]+\.[a-zA-Z0-9_]+$/ || !defined($nombre) ) {
                    lib_stdlog::print_err(lib_lang::get_interpolated_message('utils_invalid_port_template', $plantilla));
                    next;
                }
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'label'} = $nombre;
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'plt'} = $plantilla;
                my @templates = split(/;/, $extra_ports);
                $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'plt_paralela'} = \@templates;

            }
        } elsif ($1 eq 'BASE_PORTS') {
            push(@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}, $2);
        } else {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{$1} = $2;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} = int($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'});
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} > 3) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} = 3;
    } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} < 0) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} = 0;
    }

    # verifica configuracion de db
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'} eq '' ||
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} eq '' ||
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'} eq '' ||
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'} eq '' ) {
        my $db_conn_error = lib_lang::get_message('global_error_db_connection_parameters');
        lib_stdlog::print_err("$db_conn_error\n");
        return "Error: $db_conn_error";
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE'} ne '') {
        my @filext_exclude_list = split(/,/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE'});
        foreach my $ext (@filext_exclude_list) {
            $ext =~ s/\s//sg;
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE_LIST'}{$ext} = 1;
        }
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'} ne '' &&
        substr($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'}, -1) ne '/') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'} .= '/';
    }
    # extensiones permitidas para uploads
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPLOADS_PERMITIDOS'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPLOADS_PERMITIDOS'} = $prontus_varglb::UPLOADS_PERMITIDOS_DEFAULT;
    }

    if (keys(%{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}}) < 1) {
        lib_stdlog::print_err(lib_lang::get_message('utils_no_fids_prontus_conf'));
    }

    if (keys(%{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) < 1) {
        lib_stdlog::print_err(lib_lang::get_message('utils_no_ports_prontus_conf'));
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    }

    lib_lang::load_translation_table($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DEFAULT_LANGUAGE'});

    # Caja suscripcion snp
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_SUBSCRIBE'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_SUBSCRIBE'} = lib_lang::get_message('conf_snp_app_boxmsg_button_subscribe');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_UNSUBSCRIBE'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_UNSUBSCRIBE'} = lib_lang::get_message('conf_snp_app_boxmsg_button_unsubscribe');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBE_INVITATION'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBE_INVITATION'} = lib_lang::get_message('conf_snp_app_boxmsg_subscribe_invitation');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBED'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBED'} = lib_lang::get_message('conf_snp_app_boxmsg_subscribed');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOTIFICATION_DENIED'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOTIFICATION_DENIED'} = lib_lang::get_message('conf_snp_app_boxmsg_notification_denied');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOT_SUPPORTED'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOT_SUPPORTED'} = lib_lang::get_message('conf_snp_app_boxmsg_not_supported');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BODY_TEXT'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BODY_TEXT'} = lib_lang::get_message('conf_snp_app_modal_body_text');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_ACCEPT'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_ACCEPT'} = lib_lang::get_message('conf_snp_app_modal_button_accept');
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_CANCEL'} eq '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_CANCEL'} = lib_lang::get_message('conf_snp_app_modal_button_cancel');
    }

    # MAXARTICS
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} > $prontus_varglb::TAXPORT_MAXARTICS_SECURITY) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} = $prontus_varglb::TAXPORT_MAXARTICS_SECURITY;
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} > $prontus_varglb::TAXPORT_MAXARTICS_SECURITY) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} = $prontus_varglb::TAXPORT_MAXARTICS_SECURITY;
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} > 1000) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} = 1000;
    }

    # tagonomicas.TAGPORT_ORDEN
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} =~ m/(PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "art_fechaphorap $2, art_id $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "art_titu $2, art_id $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "art_id $2";
        } else {
            lib_stdlog::print_err(lib_lang::get_message('global_error_invalid_tagport_order'), " [$1][$2]\n");
        }
    }

    # list.LIST_ORDEN
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} =~ m/(PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "art_fechaphorap $2, art_id $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "art_titu $2, art_id $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "art_id $2";
        } else {
            lib_stdlog::print_err(lib_lang::get_message('global_error_invalid_list_order'), " [$1][$2]\n");
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_EN_PRONTUS'} = 'NO';
    if ($buffer =~ m/PRONTUS_EN_PRONTUS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_EN_PRONTUS'} = $1;
    }

    # TAXPORT_ORDEN
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} =~ m/(PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "art_fechaphorap $2, art_id $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "art_titu $2, art_id $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "art_id $2";
        } else {
            lib_stdlog::print_err(lib_lang::get_message('cron_list_taxport_cfg_error'), " [$1][$2]\n");
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'} = 'altavoz.net';
    if ($buffer =~ m/PRONTUS_MAIL_DOMAIN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FFMPEG_AVAILABLE'} = 0;
    # si el dir se define por cfg, usa ese en lugar del por defecto
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'} ne '') {
        if (!-s "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg") {
            my $dir_ffmpeg_error = lib_lang::get_message('cfg_ffmpeg_invalid_path') . " [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg]";
            lib_stdlog::print_err("$dir_ffmpeg_error\n");
        } else {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FFMPEG_AVAILABLE'} = 1;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_SITE";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_TEMPLATE";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA";

    if (  $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} eq 'SI'
       && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER'} eq 'NO'
       && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'} ne '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'} = "/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'}$prontus_varglb::DIR_CPAN_DATA";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'};
    } else {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN/data";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} = $prontus_varglb::PRONTUS_ID;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}  = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'}   = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log";
    # logs de queries de elasticsearch
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}   = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log_es";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_NRO_ARTIC'} = $prontus_varglb::MAX_NRO_ARTIC; # este valor se "copia"
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'} = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_LOCK";

    &init_db_table_names($prontus_varglb::PRONTUS_ID);

    # configuracion cargada para este prontus
    my $modtime = (stat $cfg_file)[9];
    $prontus_varglb::CONFIG_LOADED{$prontus_varglb::PRONTUS_ID} = $modtime;
    return '';
}

sub init_db_table_names {
    my $prontus_id = $_[0];
    my $clean_id = $prontus_id;
    # reemplaza posibles guiones del prontus id
    $clean_id =~ s/\-/_/g;
    %{$prontus_varglb::TABLES{$prontus_id}} = (
        'art' => $clean_id . '_art',
        'art_despublicar' => $clean_id . '_art_despublicar',
        'art_publicar' => $clean_id . '_art_publicar',
        'art_users' => $clean_id . '_art_users',
        'asset' => $clean_id . '_asset',
        'coment' => $clean_id . '_coment',
        'multitag_art_s' => $clean_id . '_multitag_art_s',
        'multitag_art_st' => $clean_id . '_multitag_art_st',
        'multitag_art_t' => $clean_id . '_multitag_art_t',
        'multitag_s' => $clean_id . '_multitag_s',
        'multitag_st' => $clean_id . '_multitag_st',
        'multitag_t' => $clean_id . '_multitag_t',
        'oidc_users' => $clean_id . '_oidc_users',
        'old_psw' => $clean_id . '_old_psw',
        'port_users' => $clean_id . '_port_users',
        'profiles' => $clean_id . '_profiles',
        'secc' => $clean_id . '_secc',
        'subtemas' => $clean_id . '_subtemas',
        'tags' => $clean_id . '_tags',
        'tagsart' => $clean_id . '_tagsart',
        'temas' => $clean_id . '_temas',
        'url' => $clean_id . '_url',
        'urlf5' => 'prontus_urlf5',
        'users' => $clean_id . '_users',
        'prontus_users' => $clean_id . '_prontus_users',
    );
}

sub get_table {
    my $prontus_id = $_[1] || $prontus_varglb::PRONTUS_ID;
    if (!$prontus_varglb::TABLES{$prontus_id}) {
        &init_db_table_names($prontus_id);
    }
    return $prontus_varglb::TABLES{$prontus_id}{$_[0]} || '';
}

sub get_path_nice {
    if ($prontus_varglb::PATHNICE ne '') {
        return $prontus_varglb::PATHNICE;
    }
    $prontus_varglb::PATHNICE = '/usr/bin/nice';
    if (!-f $prontus_varglb::PATHNICE) {
        $prontus_varglb::PATHNICE = '/usr/local/bin/nice';
        if (!-f $prontus_varglb::PATHNICE) {
            $prontus_varglb::PATHNICE = '/usr/nice';
            if (!-f $prontus_varglb::PATHNICE) {
                $prontus_varglb::PATHNICE = '';
            }
        }
    }
    return $prontus_varglb::PATHNICE;
}

sub garbage_preview {
    my $ruta_script = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/";

    my $pathnice = get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);

    my $script = $ruta_script . "prontus_preview_garbage_collector.pl";
    my $cmd    = "$pathnice $script $prontus_varglb::PRONTUS_ID 1>/dev/null 2>/dev/null &";

    lib_stdlog::print_err("\ngarbage_preview cmd[$cmd]\n");

    system($cmd);
}

#------------------------------------------------------------------------#
sub call_system_and_location {
    my ($nom_script, $params, $escape, $location) = @_;
    $escape = 1 if (!defined($escape));
    # ruta_script:
    #   En unix queda algo asi como /sites/misitio.cl/web/cgi-cpn/
    my $ruta_script = $ENV{'SCRIPT_FILENAME'};
    $ruta_script =~ s/\w+\.\w+$//;
    if (!$ruta_script) {
        $ruta_script = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/";
    }

    my $pathnice = get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);
    my $script = $ruta_script . "$nom_script";

    # limpiar parametros
    $params =~ s/[#;&<>|~]+//g if ($escape);

    my $cmd    = "$pathnice $script $params 1>/dev/null 2>/dev/null &";
    lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
    system $cmd;
    if ($location) {
        print "Status: 303 See Other\n";
        print "Location: $location\n\n";
    }
}

# Obtiene extension sin punto
sub get_file_extension {
    return $1 if ($_[0] =~ /\.(\w+)$/);
    return '';
}

#-----------------------------------------------------------------------
# Determina si el archivo es mas antiguo que N segundos, de acuerdo a fecha/hora de modificacion.
sub arch_antiguo {
    my ($pathArch, $n_segs) = @_;

    # Obtener estadisticas del arch.
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $pathArch;

    # Si los seg. de antiguedad de la pagina son mayores que $taxport_refresh_segs
    if ((time - $mtime) > $n_segs) {
        return 1;    # es mas antiguo que N segundos
    }
    return 0;
}


# escribe al log de operación de prontus
# requiere glib_hrfec_03
sub write_log {
    my ($accion, $objeto, $path, $contexto) = @_;
    my ($linea, $fecha, $hora, $buf, $nom_file, $usr);

    if (-d $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'}) {
        # <nombre del publicador>_aaaammdd.log
        $fecha    = glib_hrfec_03::get_date_pack4();
        $nom_file = $prontus_varglb::PRONTUS_ID . '_' . $fecha . '.txt';

        # dd/mm/aaaa hh:mm:ss - <ip> - <accion> - <user> - <ente afectado> - <path>
        $fecha = glib_hrfec_03::des_normaliza_fecha($fecha);
        $hora = glib_hrfec_03::get_date_time('', '', '', '', 1, '', time);
        if ($prontus_varglb::USERS_USR) {
            $usr = $prontus_varglb::USERS_USR;
        } else {
            if ($contexto) {
                $usr = $contexto;
            } else {
                $usr = 'System Script';
            }
        }

        my $user_ip = $ENV{'REMOTE_ADDR'};
        if (lib_prontus::cache_active() && defined($ENV{'HTTP_X_FORWARDED_FOR'})) {
            $user_ip = $ENV{'HTTP_X_FORWARDED_FOR'};
        }

        if ($accion !~ /login/i) {
            $path =~ s/^$prontus_varglb::DIR_SERVER//;
            $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $usr . "\t" . $accion . "\t" . $objeto . "\t" . $path . "\n";
        } else {
            $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $objeto . "\t" . $accion . "\t\t\t" . $path . "\n";
        }

        my $log_file = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'} . '/' . $nom_file;
        my ($status, $fh, $data) = lib_waitlock::lock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log.mutex");
        if (!$status) {
            lib_stdlog::print_err(lib_lang::get_message($fh). $data);
        }

        glib_fildir_03::append_file($log_file, $linea);

        my $result;
        ($status, $result, $data) = lib_waitlock::unlock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log.mutex", $fh);
        if (!$status) {
            lib_stdlog::print_err(lib_lang::get_message($result). $data, "\n");
        }
    }
}

# Ve si la portada esta asignada al usuario.
# Se requiere que ya se haya pasado por la funcion &check_user_session()
sub port_asoc {
    my $port = $_[0];
    my $user_id = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'};
    if (defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$user_id}{$port}) {
        return 1;
    }
    return 0;
}

# Elimina un substring al comienzo de un string
sub remove_front_string {
    my ($string, $substring) = @_;
    $string =~ s/^$substring//is;
    return $string;
}

# retorna nombre y extension (el ultimo punto en adelante)
sub split_nom_y_extension {
    my $nomfile = shift;
    return ($1, $2) if ($nomfile =~ /([\w\-\d]+)\.?(\w+)?$/);
    return '';
}

sub write_log_es {
    my $query = shift;
    my ($linea, $fecha, $hora, $nom_file);

    glib_fildir_03::check_dir($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'});
    $fecha    = glib_hrfec_03::get_date_pack4();
    $nom_file = $prontus_varglb::PRONTUS_ID . '_' . $fecha . '.txt';

    # dd/mm/aaaa hh:mm:ss - <ip> - <query>
    $fecha = glib_hrfec_03::des_normaliza_fecha($fecha);
    $hora = glib_hrfec_03::get_date_time('', '', '', '', 1, '', time);

    my $user_ip = $ENV{'REMOTE_ADDR'};
    if (lib_prontus::cache_active() && defined($ENV{'HTTP_X_FORWARDED_FOR'})) {
        $user_ip = $ENV{'HTTP_X_FORWARDED_FOR'};
    }
    $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $query . "\n";

    my $log_file = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'} . '/' . $nom_file;
    my ($status, $fh, $data) = lib_waitlock::lock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log_es.mutex");
    if (!$status) {
        lib_stdlog::print_err(lib_lang::get_message($fh). $data);
    }
    glib_fildir_03::append_file($log_file, $linea);
    my $result;
    ($status, $result, $data) = lib_waitlock::unlock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log_es.mutex", $fh);
    if (!$status) {
        lib_stdlog::print_err(lib_lang::get_message($result). $data, "\n");
    }
}

# Chequea que el rango definido por la fechap/horap y la fechae/horae del artic.
# incluyan la fecha actual, para saber si hay q publicarlo o no.
sub fechas_ok {
    my ($ts_preview, $fechap, $horap, $fechae, $horae) = @_;

    $fechap = '9' x 8 if ($fechap eq '');
    $fechae = '9' x 8 if ($fechae eq '');

    if ($horap eq '') {
        $horap  = '0' x 6;
    } elsif (length($horap) < 6) {
        if (index($horap, ':') > -1 ) {
            $horap = substr($horap, 0, 2) . substr($horap, 3, 2) . '00';
        } else {
            $horap .= '00';
        }
    }

    if ($horae eq '') {
        $horae  = '0' x 6;
    } elsif (length($horae) < 6) {
        if (index($horae, ':') > -1 ) {
            $horae = substr($horae, 0, 2) . substr($horae, 3, 2) . '00';
        } else {
            $horae .= '00';
        }
    }

    # ts_now
    my $ts_now;
    if ($ts_preview) {
        $ts_now = $ts_preview;
    } else {
        # TS de AHORA
        $ts_now = glib_hrfec_03::get_dtime_pack4();
        # $ts_now = '20110112000000'; # TS 0 de la mañana
    }

    # ts_publicacion
    my $ts_p = "$fechap$horap";

    # ts_expiracion
    my $ts_e = "$fechae$horae";

    if (($ts_p > $ts_now) || ($ts_e < $ts_now)) {
        # no publicar articulo
        return 0;
    }
    return 1;
}

# Imprime pagina con reporte de resultado.
# Param :
# 0) Titulo del mensaje
# 1) Mensaje a imprimir
# 2) 1 | 0: si no viene, se asume mostrar link para 'volver'
# 3) 1 | 0: salir, por defecto terminar ejecucion
# 4) 1 | 0: imprimir content type html, por defecto imprimir
# 5) '' | nolink, no muestra link de volver o cerrar
sub print_pag_result {
    my $tit    = $_[0];
    my $result = $_[1];
    my $cerrar = $_[2];
    my $exit   = defined($_[3]) ? $_[3] : 1;
    my $ctype  = defined($_[4]) ? $_[4] : 1;
    my $link   = defined($_[5]) ? $_[5] : '';

    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    # reemplaza caracteres especiales por espacios
    $result =~ s/[<>\/"'&;%]+/ /g;

    # Abrir y cargar archivo corresp. a la plantilla
    my $pagina = &load_cpan_core_plt("core/prontus_msg.html");
    if ($pagina ne '') {
        $pagina = &set_coreplt_ppal_simple($pagina);
        $pagina =~ s/%%title%%/$tit/g;
        $pagina =~ s/%%msg%%/$result/g;

        if ($link eq 'nolink') {
            $pagina =~ s/<!--VOLVER-->.*?<!--\/VOLVER-->//sg;
            $pagina =~ s/<!--CERRAR-->.*?<!--\/CERRAR-->//sg;
        } else {
            if ($cerrar) {
                $pagina =~ s/<!--VOLVER-->.*?<!--\/VOLVER-->//sg;
            } else {
                $pagina =~ s/<!--CERRAR-->.*?<!--\/CERRAR-->//sg;
            }
        }
    } else {
        $pagina = $result;
    }

    print "Content-Type: text/html\r\n\r\n" if ($ctype);
    utf8::encode($pagina) if ($prontus_varglb::FCGI_MODE);

    print $pagina;

    if ($exit) {
        if ($prontus_varglb::FCGI_MODE) {
            goto EXIT_OVERRIDE;
        } else {
            exit;
        }
    }
}

# Imprime pagina html con headers necesarios, termina la ejecucion del script
# Param :
# 0) html completo
# 1) 1 | 0: permite decidir si enviar headers de no cache
# 2) headers adicionales que se deban enviar, cookies, status 404,
# se imprimen primero
sub print_html {
    my ($html, $nocache, $headers) = @_;
    $nocache = 1 if (!defined($nocache));

    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    print $headers if ($headers);
    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n" if ($nocache);
    print "Content-type: text/html;charset=utf-8\r\n\r\n";

    utf8::encode($html) if ($prontus_varglb::FCGI_MODE);
    print $html;

    if ($prontus_varglb::FCGI_MODE) {
        goto EXIT_OVERRIDE;
    } else {
        exit;
    }
}

# Imprime xml con headers necesarios, termina la ejecucion del script
# Param :
# 0) xml completo
# 1) 1 | 0: permite decidir si enviar headers de no cache
# se imprimen primero
sub print_xml {
    my ($xml, $nocache) = @_;
    $nocache = 1 if (!defined($nocache));

    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n" if ($nocache);
    print "Content-type: text/xml;charset=utf-8\r\n\r\n";

    utf8::encode($xml) if ($prontus_varglb::FCGI_MODE);
    print $xml;

    if ($prontus_varglb::FCGI_MODE) {
        goto EXIT_OVERRIDE;
    } else {
        exit;
    }
}

# Imprime csv con headers necesarios, termina la ejecucion del script
# Param :
# 0) csv completo
# 1) nombre de archivo a descargar
# se imprimen primero
sub print_attach_csv {
    my ($csv,$filename) = @_;

    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n";
    print 'Content-Disposition: attachment; filename="'.$filename.'"\r\n';
    print "Content-type: text/csv\r\n\r\n";

    utf8::encode($csv) if ($prontus_varglb::FCGI_MODE);
    print $csv;

    if ($prontus_varglb::FCGI_MODE) {
        goto EXIT_OVERRIDE;
    } else {
        exit;
    }
}

#---------------------------------------------------------------
# Imprime mensaje json
sub print_json_string {
    my $string = $_[0];

    binmode STDOUT, ':encoding(UTF-8)' if (!$prontus_varglb::FCGI_MODE);

    print "Cache-Control: no-cache, must-revalidate\r\n";
    print "Content-type: application/json; charset=utf-8\r\n\r\n";

    utf8::decode($string) if (!$prontus_varglb::FCGI_MODE);
    print $string;

    if ($prontus_varglb::FCGI_MODE) {
        goto EXIT_OVERRIDE;
    } else {
        exit;
    }
}

sub load_cpan_core_plt {
    my $file           = $_[0];
    my $buffer         = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/$file");

    if ($buffer eq '') {
        return lib_lang::get_interpolated_message('global_error_template_not_found', "$prontus_varglb::DIR_PLT_CORE/$file");
    }

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $buffer =~ s/$crlf/\x0a/sg;
    return $buffer;
}

# Parsea plantilla de nivel principal en el core con reemplazos comunes.
# Asume que ya se paso por el load_config de prontus y se autentico al user.
# el 2do parametro es optativo, en caso de necesitar usar otro dir de macros
sub set_coreplt_ppal {
    my ($buffer, $no_lang) = @_;

    # En primer lugar, agrega macros
    # ubicacion macross
    my $dir_macros = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/macros";

    # agrega macros indicadas en la plantilla
    # pasamos profundidad = 1 para realizar reemplazo de globales al final
    $buffer = lib_parser::add_macros($buffer, $dir_macros, 1);

    # convierte codificacion url de %
    $buffer =~ s/%25%25/%%/g;

    # Perfilación de contenido
    $buffer = lib_profile::html_profiler($buffer);

    utf8::encode($buffer);
    # oculta html correspondiente solo a admin
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} ne '1') {
        $buffer =~ s|<!--ADMIN_ONLY-->.*?<!--/ADMIN_ONLY-->||sg;
        $buffer =~ s|%%admin_only%%.*?%%/admin_only%%||sg;
    } else {
        $buffer =~ s|<!--/?ADMIN_ONLY-->||g;
        $buffer =~ s|%%admin_only%%||g;
        $buffer =~ s|%%/?admin_only%%||g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} eq 'NO') {
        $buffer =~ s|<!--ELASTICSEARCH_HABILITADO-->.*?<!--/ELASTICSEARCH_HABILITADO-->||sg;
    } else {
        $buffer =~ s|<!--/?ELASTICSEARCH_HABILITADO-->||g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} ne 'SI') {
        $buffer =~ s|<!--PUSH_NOTIF-->.*?<!--/PUSH_NOTIF-->||sg;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'NO') {
        $buffer =~ s|<!--ADMIN_PLUGINS-->.*?<!--/ADMIN_PLUGINS-->||sg;
    } else {
        # despliega sistema de plugins si corresponde
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'SI') {
            my $menu_plugins = &parsea_menu_plugins();
            # el html esta procesandose como latin1, el nuevo texto a reemplazar viene como utf8
            # se pasa a latin 1 para mantener consistencias de encoding
            utf8::encode($menu_plugins);
            if ($menu_plugins eq '') {
                $buffer =~ s|<!--ADMIN_PLUGINS-->.*?<!--/ADMIN_PLUGINS-->||sg;
            } else {
                $buffer =~ s/%%plugins_lista%%/$menu_plugins/g;
            }
        } else {
            $buffer =~ s|%%plugins_lista%%||g;
        }
        $buffer =~ s|<!--/?ADMIN_PLUGINS-->||g;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} eq 'NO') {
         $buffer =~ s|<!--SEO_ACTIVO-->.*?<!--/SEO_ACTIVO-->||sg;
    } else {
        $buffer =~ s|<!--/?SEO_ACTIVO-->||g;
    }

    # oculta html correspondiente a opciones de multivistas si no está habilitado.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MV'} eq 'NO') {
        $buffer =~ s|<!--SEO_MV-->.*?<!--/SEO_MV-->||sg;
    } else {
        $buffer =~ s|<!--/?SEO_MV-->||g;
    }

    # oculta html prohibido a usuarios sin portadas asignadas
    if ( &tiene_portadas_asignadas() == 0) {
        $buffer =~ s|<!--USERS_CON_PORTS-->.*?<!--/USERS_CON_PORTS-->||sg;
    } else {
        $buffer =~ s|<!--/?USERS_CON_PORTS-->||g;
    }

    # oculta a html al admin
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq '1') {
        $buffer =~ s|<!--HIDE_TO_ADMIN-->.*?<!--/HIDE_TO_ADMIN-->||sg;
    } else {
        $buffer =~ s|<!--/?HIDE_TO_ADMIN-->||g;
    }

    # Para ocultar o mostrar opciones si la multiedicion esta activa/inactiva
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        $buffer =~ s|<!--MULTI_EDICION-->.*?<!--/MULTI_EDICION-->||sg;
        $buffer =~ s|%%multi_edicion%%.*?%%/multi_edicion%%||sg;
        $buffer =~ s|%%/?no_multi_edicion%%||g;
    } else {
        $buffer =~ s|%%/?multi_edicion%%||g;
        $buffer =~ s|<!--/?MULTI_EDICION-->||g;
        $buffer =~ s|%%no_multi_edicion%%.*?%%/no_multi_edicion%%||sg;
    }

    # friendly v5 no activa
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} ne '5') {
        $buffer =~ s|<!--FRIENDLY_V5-->.*?<!--/FRIENDLY_V5-->||sg;
        $buffer =~ s|<!--/?NO_FRIENDLY_V5-->||g;
    } else {
        $buffer =~ s|<!--/?FRIENDLY_V5-->||g;
        $buffer =~ s|<!--NO_FRIENDLY_V5-->.*?<!--/NO_FRIENDLY_V5-->||sg;
    }

    # muestra/oculta opciones de S3
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} eq 'SI') {
        $buffer =~ s|<!--/?S3-BLOCK-->||g;
        $buffer =~ s|%%/?s3%%||g;
    } else {
        $buffer =~ s|<!--S3-BLOCK-->.+?<!--/S3-BLOCK-->||sg;
        $buffer =~ s|%%s3%%.+?%%/s3%%||sg;
    }

    # para usar con archivos que deben recargarse siempre
    my $random = glib_str_03::random_string(10);
    $buffer =~ s/%%_random%%/$random/g;
    utf8::decode($buffer);
    # datos del usuario
    $buffer =~ s/%%_prontus_user_name%%/$prontus_auth::USERS_NOM/g;
    my $glosa_perfil = lib_utils::get_glosa_perfil($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'});
    $buffer =~ s/%%_prontus_user_perfil%%/$glosa_perfil/g;

    # parsea globales de prontus
    $buffer = lib_parser::parse_globals($buffer, '');

    # identificamos el idioma seleccionado
    my $user_lang = lib_lang::get_user_language();
    $buffer =~ s/%%_user_language%%/$user_lang/g;

    if ($buffer =~ m|<!--LANG_OPTION-->(.+)<!--/LANG_OPTION-->|) {
        my $tpl = $1;
        my $options = '';
        my %languages = %{lib_lang::get_available_languages()};
        # procesamos la lista de idiomas
        foreach my $lang_key (keys %languages) {
            $options .= $tpl;
            $options =~ s/%%lang_value%%/$lang_key/;
            $options =~ s/%%lang_label%%/$languages{$lang_key}/;
            if ($lang_key eq $user_lang) {
                $options =~ s/%%lang_selected%%/selected="selected"/;
            } else {
                $options =~ s/%%lang_selected%%//;
            }
        }
        $buffer =~ s|<!--LANG_OPTION-->.+<!--/LANG_OPTION-->|$options|;
    }

    # localización de textos de la plantilla
    unless ($no_lang) {
        $buffer = &parser_language_function($buffer);
    }

    return $buffer;
}
# Parsea plantilla de nivel principal en el core con reemplazos comunes.
# No realiza perfilacion de usuario, solo procesa configuraciones de prontus
# el 2do parametro es optativo, en caso de necesitar usar otro dir de macros
# las secciones que usan esta funcion no tiene menu lateral
sub set_coreplt_ppal_simple {
    my ($buffer, $pathdir_macros) = @_;

    # En primer lugar, agrega macros
    # ubicacion default
    my $dir_macros = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/macros";
    # para sobreescribir ubic default
    $dir_macros = $pathdir_macros if (($pathdir_macros) && (-d $pathdir_macros));

    # agrega macros indicadas en la plantilla
    # pasamos profundidad = 1 para realizar reemplazo de globales al final
    $buffer = lib_parser::add_macros($buffer, $dir_macros, 1);
    # convierte codificacion url de %
    $buffer =~ s/%25%25/%%/g;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} eq 'NO') {
        $buffer =~ s/<!--ELASTICSEARCH_HABILITADO-->.*?<!--\/ELASTICSEARCH_HABILITADO-->//sg;
    } else {
        $buffer =~ s/<!--\/?ELASTICSEARCH_HABILITADO-->//g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} ne 'SI') {
        $buffer =~ s/<!--PUSH_NOTIF-->.*?<!--\/PUSH_NOTIF-->//sg;
    }

    # oculta html correspondiente a prontus plugins
    $buffer =~ s|<!--ADMIN_PLUGINS-->.*?<!--/ADMIN_PLUGINS-->||sg;

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} eq 'NO') {
         $buffer =~ s/<!--SEO_ACTIVO-->.*?<!--\/SEO_ACTIVO-->//sg;
    } else {
        $buffer =~ s/<!--\/?SEO_ACTIVO-->//g;
    }

    # Para ocultar o mostrar opciones si la multiedicion esta activa/inactiva
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        $buffer =~ s/<!--MULTI_EDICION-->.*?<!--\/MULTI_EDICION-->//sg;
        $buffer =~ s/%%multi_edicion%%.*?%%\/multi_edicion%%//sg;
        $buffer =~ s/%%\/?no_multi_edicion%%//g;
    } else {
        $buffer =~ s/%%\/?multi_edicion%%//g;
        $buffer =~ s/<!--\/?MULTI_EDICION-->//g;
        $buffer =~ s/%%no_multi_edicion%%.*?%%\/no_multi_edicion%%//sg;
    }

    # friendly v5 no activa
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} ne '5') {
        $buffer =~ s/<!--FRIENDLY_V5-->.*?<!--\/FRIENDLY_V5-->//sg;
        $buffer =~ s/<!--\/?NO_FRIENDLY_V5-->//g;
    } else {
        $buffer =~ s/<!--\/?FRIENDLY_V5-->//g;
        $buffer =~ s/<!--NO_FRIENDLY_V5-->.*?<!--\/NO_FRIENDLY_V5-->//sg;
    }

    # muestra/oculta opciones de S3
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} eq 'SI') {
        $buffer =~ s|<!--/?S3-BLOCK-->||sg;
        $buffer =~ s|%%/?s3%%||sg;
    } else {
        $buffer =~ s|<!--S3-BLOCK-->.+?<!--/S3-BLOCK-->||sg;
        $buffer =~ s|%%s3%%.+?%%/s3%%||sg;
    }

    # para usar con archivos que deben recargarse siempre
    my $random = glib_str_03::random_string(10);
    $buffer =~ s/%%_random%%/$random/g;

    # parsea globales de prontus
    $buffer = lib_parser::parse_globals($buffer, '');

    my $language = lib_lang::get_language();
    $buffer =~ s/%%_user_language%%/$language/g;

    # localización de textos de la plantilla
    $buffer = &parser_language_function($buffer);

    return $buffer;
}

# lee los plugins instalados en el directorio [prontus_dir]/cpan/prontus_plugins/
# y los carga en el sidebar
# soporta un solo nivel de profundidad.
sub parsea_menu_plugins {
    my %plugins = &get_plugins_paths('habilitados');

    return '' if ! %plugins; # si no hay plugins termino ahora.

    my ($output, $plugin);
    # cargamos la plantilla para la sidebar
    my $tpl_plugins_menu = &load_cpan_core_plt("prontus_plugins/plugins_menu.html");
    foreach my $nombre (sort keys %plugins) {
        my $tmp = $tpl_plugins_menu;

        $plugins{$nombre} =~ s/\s+/$plugins{$nombre}/sg;
        $plugin = URI::Escape::uri_escape_utf8($plugins{$nombre});

        $tmp =~ s/%%plugin%%/$plugin/sg;
        $tmp =~ s/%%nombre%%/$nombre/sg;
        $output .= $tmp;
    }
    return $output;
}

# lee los archivos "plugin.json" en los subdir del directorio de plugins de Prontus
# devuelve un hash (nombre_plugin => path_plugin) con los plugins instalados y habilitados.
# parametro $estado: 'habilitados', 'todos'
sub get_plugins_paths {
    my $estado = $_[0];

    my @paths_plugins = glob "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/prontus_plugins/*/plugin.json";
    my %plugins;

    foreach my $json_file (@paths_plugins) {
        my $json_text = glib_fildir_03::read_file_bin($json_file);

        my $data = eval {JSON::decode_json($json_text)};
        if ($@) {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('global_json_decode_error', $@, $json_text)."\n");
            next;
        }

        next if ((!$data->{'habilitado'}) && (lc $estado eq 'habilitados'));
        next unless $data->{'nombre'};

        # perfil 1 (admin) puede ver todo
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} ne '1') {
            my $access_granted = 0;
            if (defined($data->{profiles})) {
                if ($data->{profiles}[0] == 0) {
                    $access_granted = 1;
                } else {
                    foreach my $profile (@{$data->{profiles}}) {
                        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq $profile) {
                            $access_granted = 1;
                            last;
                        }
                    }
                }
            }
            next if (!$access_granted);
        }

        $data->{'path'} = '#';
        if ($json_file =~ m(cpan/prontus_plugins/(.*?)/plugin.json)) {
            $data->{'path'} = $1;
        }
        # evitamos que intenten cargar archivos con . o .. ???
        $data->{'path'} =~ s/\.\.+/./sg;
        $plugins{$data->{'nombre'}} = $data->{'path'};
    }
    return %plugins;
}

# chequea si el usuario tiene portadas asignadas.
# @param void
# return bool
sub tiene_portadas_asignadas {
    # es perfil admin o esta activo el SSO
    return 1 if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq '1');

    # verifica si el usuario tiene portadas asignadas en los hashes correspondientes, hash definido y con elementos
    if (defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}})
        && keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}}}) {
        return 1;
    }
    return 0;
}

sub get_glosa_perfil {
    lib_profile::load_profiles();
    my $prontus_id = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'};
    if (defined($prontus_varglb::PROFILES{$prontus_id}{$_[0]}{'name'})) {
        return $prontus_varglb::PROFILES{$prontus_id}{$_[0]}{'name'};
    } else {
        return '';
    }
}

# parsea invocacion a funciones definidas en lib_lang.pm
sub parser_language_function {
    my $buffer = $_[0];
    utf8::encode($buffer);
    my $aux = $buffer;
    my %ready;
    # siempre tiene parametros
    while ($buffer =~ /(_lang_(\w+)\(["'](.+?)["']\)_)/g) {
        my $match    = $1;
        my $function = $2;
        my $params   = $3;
        # salta marcas ya procesadas
        next if ($ready{$match});

        my @args = ();
        # Para obtener los parametros de entrada
        if ($params ne '') {
            my @arrparams = split(/["']\s*,\s*["']/, $params);
            foreach my $keys (@arrparams) {
                push(@args, $keys);
            }
        }

        my $msg;
        if ($function eq 'get_message') {
            $msg = lib_lang::get_message(@args);
        } else {
            $msg = lib_lang::get_interpolated_message(@args);
        }
        utf8::encode($msg);
        $aux =~ s/\Q$match\E/$msg/g;
        $ready{$match} = 1;
    }
    utf8::decode($aux);
    return $aux;
}

sub getHash {
    my $txt = glib_fildir_03::read_file($_[0]);
    my %result;
    while($txt =~ /^([^=]+)=(.+)$/gm ){
        $result{$1} = $2;
    }
    return \%result;
}

# vers_cmp is adapted from Sort::Versions 1.3 1996/07/11 13:37:00 kjahds,
# which is not included with Perl by default, hence the need to copy it here.
# Seems silly to require it when this is the only place we need it...
sub vers_cmp {
    return if (@_ < 2 or @_ > 2);
    my ($a, $b) = @_;
    my (@A) = ($a =~ /(\.|\d+|[^\.\d]+)/g);
    my (@B) = ($b =~ /(\.|\d+|[^\.\d]+)/g);
    my ($A,$B);
    while (@A and @B) {
        $A = shift @A;
        $B = shift @B;
        if ($A eq "." and $B eq ".") {
            next;
        } elsif ($A eq ".") {
            return -1;
        } elsif ($B eq ".") {
            return 1;
        } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
            return $A <=> $B if $A <=> $B;
        } else {
            $A = uc $A;
            $B = uc $B;
            return $A cmp $B if $A cmp $B;
        }
    }
    @A <=> @B;
}

1;
