#!/usr/bin/perl

# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
# script para incializar el arreglo de includes para la instancia perl de apache
# es necesario que se defina el path de cgi-cpn en la configuracion de apache:
# PerlOptions +Parent
# PerlSwitches -I/var/www/html/cgi-cpn
# PerlRequire /var/www/html/cgi-cpn/prontus_apache_startup.pl

use strict;
use warnings 'all';
no warnings 'once';

my $cgi_path;
my $documentRoot;

BEGIN {
    # para usar libreria de json compatible con threads de apache
    $ENV{'PERL_JSON_BACKEND'} = 'Cpanel::JSON::XS,JSON::PP';

    $cgi_path = '';
    foreach my $path (@INC) {
        if (-f "$path/prontus_varglb.pm" ) {
            $cgi_path = $path;
            last;
        }
    }

    if ($cgi_path ne '' && -d $cgi_path) {
        # asigna document root
        $documentRoot = $cgi_path;
        print STDERR "Prontus Document Root [$documentRoot]\n";
        $documentRoot =~ s/\/[^\/]+$//;

        opendir(my $DIR, $cgi_path);
        my @entries = readdir($DIR);
        closedir $DIR;

        foreach my $entry (@entries) {
            next if ($entry =~ /^[\._]/);
            next if ($entry eq 'prontus_error_log' or $entry eq 'prontus_temp');
            if (-d "$cgi_path/$entry") {
                unshift(@INC, "$cgi_path/$entry");
            }
        }

        # ALD
        if (-d "$cgi_path/prontus_plugins") {
            opendir(my $DIR, "$cgi_path/prontus_plugins");
            @entries = readdir($DIR);
            closedir $DIR;

            foreach my $entry (@entries) {
                next if ($entry =~ /^[\._]/);
                next if ($entry eq 'prontus_error_log' or $entry eq 'prontus_temp');
                if (-d "$cgi_path/prontus_plugins/$entry") {
                    unshift(@INC, "$cgi_path/prontus_plugins/$entry");
                }
            }
        }

        $cgi_path =~ s/\/[^\/]+$//;
        $cgi_path .= '/cgi-bin';

        if (-d $cgi_path) {
            unshift(@INC, $cgi_path);
            opendir(my $dh, $cgi_path);
            @entries = readdir($dh);
            closedir $dh;
            foreach my $entry (@entries) {
                next if ($entry =~ /^[\._]/);
                next if ($entry eq 'prontus_error_log' or $entry eq 'prontus_temp');
                if (-d "$cgi_path/$entry") {
                    unshift(@INC, "$cgi_path/$entry");
                }
            }
        }
    }
}

# Busca prontuses y utliza el primero que encuentra (busca en orden alfabetico).
opendir(my $dh, $documentRoot);
my @entries = readdir($dh);
closedir $dh;

my $prontusId = '';
foreach my $entry (sort @entries) {
    next if ($entry =~ /^[\._]/);
    if (-f "$documentRoot/$entry/cpan/$entry.cfg") {
        $prontusId = $entry;
        last;
    }
}

$My::prontusId    = $prontusId;
$My::f5dir        = '/prontus_links';
$My::documentRoot = $documentRoot;

$Prontus::modperl = 1;

use Apache2::Const;
use Apache2::RequestIO ();
use Apache2::RequestRec ();
use Apache2::SubRequest ();
use APR::Table ();

use Artic;
use Crypt::Eksblowfish::Bcrypt;
use Crypt::JWT;
use Data::Dumper;
use DateTime; #lib_stats
use DBI;
use Digest::MD5;
use Digest::SHA;
use Email::MIME;
use Email::Sender::Simple;
use Email::Sender::Transport::SMTP;
use Email::Simple;
use Encode;
use Fcntl qw(:flock SEEK_END);
use File::Compare;
use File::Copy;
use File::LibMagic;
use GD;
use GD::Image;
use Getopt::Long;
use glib_cgi_05;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_html_03;
use glib_str_03;
use HTML::Entities;
use HTTP::Request;
use HTTP::Request::Common;
use HTTP::Response;
use Image::ExifTool; # soporte webp y rotacion
use IO::All;
use IO::Compress::Gzip qw(gzip $GzipError); # para sitemaps
use JSON;
use lib_api_seo;
use lib_api_snp;
use lib_artic;
use lib_bd;
use lib_captcha2;
use lib_cookies;
use lib_dam;
use lib_dropbox;
use lib_edit;
use lib_fid;
use lib_form;
use lib_friendly;
use lib_helper;
use lib_ipcheck;
use lib_lang;
use lib_loading;
use lib_lock;
use lib_logproc;
use lib_mail;
use lib_mam;
use lib_maxrunning;
use lib_multiediting;
use lib_multitag;
use lib_oidc;
use lib_parser;
use lib_platform;
use lib_profile;
use lib_ports;
use lib_prontus;
use lib_quota;
use lib_robots;
use lib_search;
use lib_setbd;
use lib_sitemap;
use lib_stats;
use lib_stdlog;
use lib_tags;
use lib_tax;
use lib_thumb;
use lib_upload;
use lib_urlf5;
use lib_users;
use lib_utils;
use lib_validator;
use lib_waitlock;
use lib_xcoding;
use LWP::ConnCache;
use LWP::UserAgent;
use MIME::Base64;
use MIME::Parser;
use mime_types;
use Net::Amazon::S3; # requerido por lib S3
use Net::Amazon::S3::Client; # requerido por lib S3
use POSIX;
use prontus_auth;
use prontus_varglb;
use Session;
use Time::HiRes;
use Time::Local;
use Update;
use URI::Escape;
use UUID4::Tiny;
use wizard_lib;
use XML::Parser;

eval {require Archive::Zip;};
if ($@) {
    $lib_prontus::ZIP_AVAILABLE = 0;
} else {
    Archive::Zip->import();
    $lib_prontus::ZIP_AVAILABLE = 1;
}

1;
