#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# prontus_cron_taxcalend.pl

# ---------------------------------------------------------------
# UBICACION.
# -----------
# /cgi-cpn/.

# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Generar portadas calendario con cada dia apuntando a los articulos con fechap de ese dia
# Se generan los calendarios y ademas las portadillas correspondientes a cada dia
# Se generan calendarios para cada seccion y tb. calendarios comunes (sin filtrar x seccion)
# Se parsean multivistas.
# Genera hojas de calendario para mes anterior, actual, siguiente y subsiguiente. Siempre
# genera estas hojas, aunque no hayan dias linkeados.
# Las marcas parseadas en las portadillas son las mismas usadas en las taxports tradicionales.
# Se consideran articulos cuya fechap calce con el dia correspondiente y que tengan Alta.
# TO-DO: - generalizar extensiones de plantillas.
#        - usar una plantilla distinta para la portadilla comun
# ---------------------------------------------------------------
# LLAMADAS A SCRIPTS.
# ------------------------
# No registra

# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# como cron, con lo sgtes. params:
# $ARGV[0] : Nombre del Prontus (ej. prontus_noticias)
# $ARGV[1] : aaaamm : mes a procesar, si no se indica, se asume el actual.
# lee aparte opciones de la línea de comandos:
# -i fid1,fid2 o --includefids fid1,fid2 : fids a procesar específicamente
# -x fid1,fid2 o --excludefids fid1,fid2 : fids a excluir del procesamiento.
# NOTA: la opción -i implica incluir solamente los fids especificados en el proceso,
#   por lo que usar la opción x en ese caso es redundante.
# Si no se incluyen ni excluyen fids, se incluyen todos.
# Ejemplos:
#       perl prontus_cron_taxcalend.pl -i fid_noticia,fid_video prontus_minrel
#       perl prontus_cron_taxcalend.pl -i fid_noticia,fid_video prontus_minrel 201403
#       perl prontus_cron_taxcalend.pl -x fid_link,fid_rotulo prontus_minrel
#       perl prontus_cron_taxcalend.pl -x fid_link prontus_minrel 201202
# ---------------------------------------------------------------
# ARCHIVOS DE ENTRADA.
# ------------------------
# Plantilla para mes de calendario:
# /<prontus_id>/plantillas/extra/calendar/mes/port(-vvv)/mescalendar.html

# Plantilla para portadillas:
# /<prontus_id>/plantillas/extra/calendar/fechap/pags(-vvv)/portcalendar.html <--- por seccion
# /<prontus_id>/plantillas/extra/calendar/fechap/pags(-vvv)/portallcalendar.html <--- sin discriminar seccion

# ---------------------------------------------------------------
# ARCHIVOS DE SALIDA.
# ------------------------
# Hojas de calendarios:
# /<prontus_id>/site/extra/calendar/<aaaamm>/port(-vvv)/<idseccion>.html <--- por cada mes y seccion
# /<prontus_id>/site/extra/calendar/<aaaamm>/port(-vvv)/all.html <--- por cada mes, sin discriminar seccion

# Portadillas:
# /<prontus_id>/plantillas/extra/calendar/<aaaammdd>/pags(-vvv)/<idseccion>.html <--- por cada dia y seccion
# /<prontus_id>/plantillas/extra/calendar/<aaaammdd>/pags(-vvv)/all.html <--- por cada dia, sin discriminar seccion
# ---------------------------------------------------------------
# Tablas.
# ------------------------
# BD: la configurada en Prontus. Tablas: 'ART', 'SECC', 'TEMAS', 'SUBTEMAS'
#
#
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;
use warnings;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Artic;
use DBI;
use File::Copy;
use HTML::Entities;
use JSON;
use POSIX;
use Time::Local;
use URI::Escape;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_helper;
use lib_lang;
use lib_parser;
use lib_prontus;
use lib_tags;
use lib_tax;
use lib_urlf5;
use lib_utils;
##/libs##

use Getopt::Long;

# ---------------------------------------------------------------
# MAIN.
# ---------------------------------------------------------------

my %TABLA_SECC;
my @EXCLUDEFIDS;
my @INCLUDEFIDS;

# CALENDARIOS
my $RELDIR_PORT_TMP_CALEND;
my (%BUF_PLT_CALEND, %BUF_PLT_LOOP_CALEND);
my ($BUF_PLT_CALEND_JSON, $BUF_PLT_LOOP_CALEND_JSON);
my @SEMANAS;
my %MONTH_LABEL;
my $PERP_EOM;

# PORTADILLAS
my $RELDIR_PORT_TMP;
my (%EXT_PORT_TMP, %BUF_PLT, %BUF_PLT_LOOP);
my (%EXT_PORT_TMP_PORTALL, %BUF_PLT_PORTALL, %BUF_PLT_LOOP_PORTALL);

my $CURR_DTIME = glib_hrfec_03::get_dtime_pack4();

# ---------------------------------------------------------------
# MAIN.
# -------------
main : {
    lib_utils::setUp();

    # Capturar options para exclusion o inclusion de fids
    GetOptions("excludefids|x=s" => \@EXCLUDEFIDS, "includefids|i=s" => \@INCLUDEFIDS); # .cgi -x fa,fb -i f1,f2 prontus_test
    @EXCLUDEFIDS = split(/,/,join(',', @EXCLUDEFIDS)); #por ej si se pasa -x fid_link -x fid_noticia,fid_general
    @INCLUDEFIDS = split(/,/,join(',', @INCLUDEFIDS));

    my $prontus_id = $ARGV[0];
    my $aaaamm = $ARGV[1] || '';

    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_insufficient_arguments');
        exit;
    }

    &valida_param($prontus_id, $aaaamm);

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($prontus_id);
    if ($msg_load ne '') {
        print STDERR "$msg_load\n";
        print "Error: $msg_load\n";
        exit;
    }

    if (scalar @EXCLUDEFIDS < 1 &&
        scalar @INCLUDEFIDS < 1 &&
        scalar (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}}) > 0) {
        @INCLUDEFIDS = (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}});
    }

    # CALENDARIOS
    $RELDIR_PORT_TMP_CALEND = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}/extra/calendar/mes/port";

    # PORTADILLAS
    $RELDIR_PORT_TMP = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}/extra/calendar/fechap/pags";
    $CURR_DTIME = glib_hrfec_03::get_dtime_pack4();

    my ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (! ref($base)) {
        print STDERR $msg_err_bd, "\n";
        exit;
    }

    # precarga tabla de temas en hash global para uso posterior reiterado.
    %TABLA_SECC = lib_tax::carga_hash_seccion($base);

    @{$MONTH_LABEL{'<vista_normal>'}} = ('Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre');

    # precarga buffers de plantillas de calend para cada vista.
    &precarga_plantillas_calend();

    # precarga buffers de plantillas de portadillas para cada vista.
    &precarga_plantillas_portadillas();

    # Obtiene meses a procesar
    my @meses_a_generar = &get_meses_generar($aaaamm);

    # secc
    my $secc_table = lib_utils::get_table('secc');
    my $sql = "select secc_id from $secc_table ";
    my ($secc_id);
    my $salida = glib_dbi_03::ejecutar_sql_bind($base, $sql, \($secc_id));

    while ($salida->fetch) {
        # Para cada mes
        foreach my $aaaamm (@meses_a_generar) {
            my ($aaaa, $mm) = split(/\t/, $aaaamm);
            $mm = '0' . $mm if (length($mm) == 1);
            my ($fechas_linkeadas, $duracion_eventos) = &get_fechas_linkeadas($secc_id, $aaaa, $mm, $base);

            # Para cada fecha linkeada, genera portadilla, en todas las vistas.
            my %artics;
            my $totartics = 0;
            foreach my $fecha (keys %{$fechas_linkeadas}) {
                # acumula articulos por dia
                $artics{$fecha} = &generar_portadilla($base, $fecha, $secc_id);
                $totartics += scalar @{$artics{$fecha}};
            }

            # Genera la hoja del calendario para cada una de las secciones, en todas las vistas.
            &generar_hoja_calend($secc_id, $aaaa, $mm, $fechas_linkeadas, $duracion_eventos, \%artics, $totartics);

            # Borrar portadillas de dias que NO hayan tenido articulos en este mes $aaaamm
            # se borra del tipo /prontus_mim/site/extra/calendar/$aaaamm/pags/$secc_id.ext
            my $dir2clean = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar";
            my @lisdir = glib_fildir_03::lee_dir($dir2clean);
            @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..

            foreach my $subdir (@lisdir) {
                if ( (-d "$dir2clean/$subdir/pags") && ($subdir =~ /^$aaaa$mm[0-9]{2}$/) && (!exists $fechas_linkeadas->{$subdir}) ) {
                    my %vistas = %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};
                    $vistas{''} = 1;
                    foreach my $mv (keys %vistas) {
                        $mv = '-' . $mv if ($mv);
                        my $dirmv = "$dir2clean/$subdir/pags$mv";
                        my @lisdirmv = glib_fildir_03::lee_dir($dirmv);
                        @lisdirmv = grep !/^\./, @lisdirmv; # Elimina directorios . y ..
                        foreach my $file2del (@lisdirmv) {
                            next if (!-f "$dirmv/$file2del");
                            next if ($file2del !~ /^$secc_id\.\w+$/);
                            # print STDERR "borrar[$dirmv/$file2del]\n";
                            unlink "$dirmv/$file2del";
                        }
                    }
                }
            }
        }
    }

    $salida->finish;

    # generar hoja de calend sin considerar secc.
    foreach my $aaaamm (@meses_a_generar) {
        my ($aaaa, $mm) = split(/\t/, $aaaamm);
        $mm = '0' . $mm if (length($mm) == 1);
        my ($fechas_linkeadas, $duracion_eventos) = &get_fechas_linkeadas(0, $aaaa, $mm, $base);

        # Generar portadillas correspondientes.
        my %artics;
        my $totartics = 0;
        foreach my $fecha (keys %{$fechas_linkeadas}) {
            $artics{$fecha} = &generar_portadilla($base, $fecha, 0);
            $totartics += scalar @{$artics{$fecha}};
        }

        &generar_hoja_calend(0, $aaaa, $mm, $fechas_linkeadas, $duracion_eventos, \%artics);
    }

    $base->disconnect;
}

# ---------------------------------------------------------------
# Obtiene meses a procesar, los que corresponden al actual, anterior, siguiente y subsiguiente.
# Si viene el parametro del mes $AAAAMM, entonces se toma ese como actual.
sub get_meses_generar {
    my $aaaamm = $_[0];
    # aaaamm actual
    my $aaaamm_procesar = $CURR_DTIME;

    $aaaamm_procesar = $aaaamm if ($aaaamm);

    # print "procesando[$aaaamm_procesar]\n";
    $aaaamm_procesar =~ /^(\d{4})(\d{2})/;
    my $aaaa_actual = $1;
    my $mm_actual = $2 * 1;

   return ("$aaaa_actual\t$mm_actual") if ($aaaamm);

    # aaaamm anterior
    my $mm_menos1 = $mm_actual - 1;
    my $aaaa_menos1 = $aaaa_actual;

    if ($mm_menos1 <= 0 ) {
        $aaaa_menos1 = $aaaa_actual - 1;
        $mm_menos1 = 12;
    }

    # aaaamm siguiente
    my $mm_mas1 = $mm_actual + 1;
    my $aaaa_mas1 = $aaaa_actual;

    if ($mm_mas1 > 12) {
        $aaaa_mas1 = $aaaa_actual + 1;
        $mm_mas1 = '01';
    }
    # aaaamm subsiguiente
    my $mm_mas2 = $mm_mas1 + 1;
    my $aaaa_mas2 = $aaaa_mas1;

    if ($mm_mas2 > 12) {
        $aaaa_mas2 = $aaaa_mas1 + 1;
        $mm_mas2 = '01';
    }

    return ("$aaaa_menos1\t$mm_menos1", "$aaaa_actual\t$mm_actual", "$aaaa_mas1\t$mm_mas1", "$aaaa_mas2\t$mm_mas2");
}

sub generar_portadilla {
    my ($base, $fecha, $secc_id) = @_;

    my $where = "(? between art_fecha_inicio and art_fecha_fin) and (art_alta = \"1\") ";
    my @params = ($fecha);

    if ($secc_id) {
        $where .= " and (art_idsecc1 = ? or art_idsecc2 = ? or art_idsecc3 = ?)";
        push(@params, $secc_id);
        push(@params, $secc_id);
        push(@params, $secc_id);
    }

    # Aplicar exclusion de FIDs
    foreach my $fid_exc (@EXCLUDEFIDS) {
        if ($fid_exc =~ /^[\w\-]+$/) {
            $where .= " and (art_tipoficha <> ?) ";
            push(@params, $fid_exc);
        }
    }

    # Aplicar exclusion de FIDs sin vista
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTIVAR_FIDS_SIN_VISTA'} eq 'SI') {
        foreach my $fid_list (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}}) {
            next if (exists($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}{$fid_list}));
            $where .= "and (art_tipoficha <> ?) ";
            push(@params, $fid_list);
        }
    }

    # Aplicar exclusion de FIDs
    # lo hacemos así y no con and art_tipoficha in (...) para parametrizar más fácilmente.
    if (@INCLUDEFIDS) {
        $where .= "and ( ";
        foreach my $fid_inc (@INCLUDEFIDS) {
            if ($fid_inc =~ /^[\w\-]+$/) {
                next if(!lib_helper::can_have_view($fid_inc));
                $where .= " art_tipoficha = ? or ";
                push(@params, $fid_inc);
            }
        }
        $where = substr $where, 0, -3; # eliminamos el último or
        $where .= " )";
    }

    # SQL para lista de articulos
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_id, art_titu"
            . " from $art_table where $where order by art_fechap asc, art_horap asc";

    # print STDERR "sql: [$sql]\n";
    my $tot_artics;
    my $sql_count = "select count(art_id) from $art_table where " . $where;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql_count, \@params, \($tot_artics));
    if ($base->errstr) {
        print STDERR lib_lang::get_interpolated_message('cron_taxcalend_error_count_arts', $sql, $!, $base->errstr)."\n";
        return 0;
    }
    $salida->fetch;
    $salida->finish;

    my ($art_id, $art_titu);

    $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_id, $art_titu));

    my $nro_filas = 0;
    my %filas;
    my @artics_dia;

    my @vistas = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};
    push(@vistas, '<vista_normal>');

    while ($salida->fetch) {
        $nro_filas++;
        glib_str_03::correct_encode_utf8_string($art_titu);
        my %art = ('ts' => $art_id, 'titular' => $art_titu);
        push(@artics_dia, \%art);

        # parsea esta fila en todas las multivistas
        foreach my $mv (@vistas) {
            my %aux_plt;
            if (!$secc_id) {
                %aux_plt = %{$BUF_PLT_LOOP_PORTALL{$mv}};
            } else {
                %aux_plt = %{$BUF_PLT_LOOP{$mv}};
            }

            foreach my $ext (keys %aux_plt) {
                my $plt_loop = $aux_plt{$ext};
                my $fila = lib_parser::generar_fila($art_id, $plt_loop, $nro_filas, $tot_artics, $mv, 1);

                $filas{$mv}{$ext} .= $fila;
            }
        }
    }

    # escribir
    if ($nro_filas) {
        &write_portadilla($fecha, $secc_id, \%filas);
    }

    $salida->finish;

    return \@artics_dia;
}

# ---------------------------------------------------------------
sub get_fechas_linkeadas {
    my ($secc_id, $aaaa, $mm, $base) = @_;
    # cargar todas las fechap de articulos q cumplan con los criterios.
    # esto es para saber de antemano si se debe linkear o no cada dia del mes.
    my $fecha_desde = $aaaa . $mm . '01';
    my $fecha_hasta = $aaaa . $mm . '31';

    my @params;
    # Obtener los meses que deben ser linkeados.
    # Obtener los articulos
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_fecha_inicio, art_fecha_fin from $art_table";
    # eventos dentro del mes calendario:
    # finalizan dentro del mes o
    # inician dentro del mes
    $sql .= " where ((art_fecha_fin between ? and  ?)";
    push @params, $fecha_desde;
    push @params, $fecha_hasta;

    $sql .= " or (art_fecha_inicio between ? and  ?)) and art_alta = 1";
    push @params, $fecha_desde;
    push @params, $fecha_hasta;

    # print "[$sql][$fecha_desde] [$fecha_hasta]\n";

    if ($secc_id) {
        $sql .= " and (art_idsecc1 = ? or art_idsecc2 = ? or art_idsecc3 = ?)";

        push @params, $secc_id;
        push @params, $secc_id;
        push @params, $secc_id;
    }

    # Aplicar exclusion de FIDs
    foreach my $fid (@EXCLUDEFIDS) {
        if ($fid =~ /^[\w\-]+$/) {
            $sql .= " and (art_tipoficha <> ?) ";
            push @params, $fid;
        }
    }

    # Aplicar exclusion de FIDs
    # lo hacemos así y no con and art_tipoficha in (...) para parametrizar más fácilmente.
    if (@INCLUDEFIDS) {
        $sql .= " and ( ";
        foreach my $fid_inc (@INCLUDEFIDS) {
            if ($fid_inc =~ /^[\w\-]+$/) {
                $sql .= " art_tipoficha = ? or ";
                push(@params, $fid_inc);
            }
        }
        $sql = substr $sql, 0, -3; # eliminamos el último or
        $sql .= " )";
    }

    my ($fec_ini, $fec_fin);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql,  \@params, \($fec_ini, $fec_fin));
    my %fechas_linkeadas;

    my %duracion_eventos;
    my @rangos;
    my %rangostmp;
    while ($salida->fetch) {
        if ($fec_fin eq '99999999' or $fec_fin eq '') {
            $fec_fin = $fec_ini;
        }
        my @fechas = &get_fechas($fec_ini, $fec_fin);

        if (exists $rangostmp{$fec_ini}) {
            # print "ya existe $fec_fin > $rangostmp{$fec_ini}\n";
            if ($fec_fin > $rangostmp{$fec_ini}) {
                $rangostmp{$fec_ini} = $fec_fin;
            }
        } else {
            $rangostmp{$fec_ini} = $fec_fin;
        }

        foreach my $fec (@fechas) {
            # print "\tfec[$fec]\n";
            $fechas_linkeadas{$fec} = 1;
        }
    }

    $salida->finish();

    my @listarangos;
    foreach my $key (sort  keys %rangostmp) {
        push @rangos, [$key, $rangostmp{$key}];
    }

    # Agrupar los rangos que se tocan en algun punto.
    foreach my $rangoref (@rangos) {
        my @rango = @{$rangoref};
        my $a = $rango[0];
        my $b = $rango[1];
        my $in_range = 0;

        foreach my $rango (@listarangos) {
            if (($a >= $rango->{a} && $a <= $rango->{b}) || ($b >= $rango->{a} && $b <= $rango->{b})) {
                if ($b > $rango->{b}) {
                    $rango->{b} = $b;
                }

                if ($a < $rango->{a}) {
                    $rango->{a} = $a;
                }

                $in_range++;

            }
        }

        if (not $in_range) {
            push @listarangos, {a => $a, b => $b};
        }

    }

    foreach my $rango (@listarangos) {
        # print "secc_id[$secc_id] $rango->{a}, $rango->{b}\n";
        $duracion_eventos{$rango->{a}} = $rango->{b};
    }

    return (\%fechas_linkeadas, \%duracion_eventos);
}

# ---------------------------------------------------------------
sub get_fechas {
    my $ini = $_[0];
    my $fin = $_[1];
    my @fechas;
    my $current = $ini;

    push @fechas, $ini;

    while ($current < $fin) {
        # print "current[$current] < fin[$fin]\n";
        $current =~ /(\d{4})(\d{2})(\d{2})/;
        my ($a, $m, $d) = ($1, $2, $3);
        $d = $d + 1;
        $m = $m * 1;

        # Si el nuevo dia es mayor a 31, se pasa al siguiente mes.
        if ($d > 31) {
            $d = 1; # se deja el dia primero
            $m++; # pasa el mes siguiente.
        }

        # Si el mes es mayor a 12, se pasa al siguiente año.
        if ($m > 12) {
            $m = 1; # asigna mes 1.
            $a++; # pasa al siguiente año.
        }

        $m = "0$m" if ($m < 10);
        $d = "0$d" if ($d < 10);
        $current = $a . $m . $d;

        push @fechas, $current;
    }

    return @fechas;
}

# ---------------------------------------------------------------
sub generar_hoja_calend {
    my ($secc_id, $aaaa, $mm, $ref_fechas_linkeadas, $ref_duracion_eventos, $ref_artics, $total_artics) = @_;
    my %fechas_linkeadas = %{$ref_fechas_linkeadas};
    my %duracion_eventos = %{$ref_duracion_eventos};
    my %artics = %$ref_artics; undef $ref_artics;

    my @sdays = ('lun', 'mar', 'mie', 'jue', 'vie', 'sab', 'dom');
    my $days_list = join('|',@sdays);

    @SEMANAS = ();

    &get_semanas($aaaa, int($mm));
    my %rows_out;

    # incluye las mv y la normal
    my @vistas = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};
    push(@vistas, '<vista_normal>');

    foreach my $mv (@vistas) {
        foreach my $ext (keys %{$BUF_PLT_LOOP_CALEND{$mv}}) {
            $rows_out{$mv}{$ext} = '';
            if ($BUF_PLT_LOOP_CALEND{$mv}{$ext} =~ /($days_list)%%/) {
                foreach my $semana (@SEMANAS) {
                    my $loop_out = $BUF_PLT_LOOP_CALEND{$mv}{$ext};

                    for (my $i = 1;$i <= 7;$i++) {
                        my $nomdia = $sdays[$i-1];

                        if (defined($semana->{$i})) {
                            my $dd = $semana->{$i};

                            # usa numero en caso de que no haya link
                            my $nrodia = $dd;
                            $dd = '0' . $dd if (length($dd) == 1);
                            my $currentday = "$aaaa$mm$dd";

                            # print STDERR "FECHA [$currentday] ...\n";
                            if (exists $fechas_linkeadas{$currentday}) {
                                my $clase_link = "active_s"; # evento de un solo dia.

                                # Ver si el evento comienza aca
                                if (exists $duracion_eventos{$currentday} && $duracion_eventos{$currentday} > $currentday) {
                                    $clase_link = "active_ml";
                                } elsif (exists $duracion_eventos{$currentday} && $duracion_eventos{$currentday} eq $currentday) {
                                    $clase_link = "active_s";
                                } else {
                                    # Ver si el evento comenzo pero no ha terminado.
                                    foreach my $ev_ini (keys %duracion_eventos) {
                                        my $ev_fin = $duracion_eventos{$ev_ini};

                                        if ($currentday > $ev_ini && $currentday < $ev_fin) {
                                            $clase_link = "active_m";
                                        }
                                        if ($currentday eq $ev_fin) {
                                            $clase_link = "active_m_r";
                                        }
                                    }
                                }

                                $nrodia = &linkear_dia($aaaa, $mm, $dd, $secc_id, $mv, $clase_link, $ext);

                                if ($BUF_PLT_LOOP_CALEND{$mv}{$ext} =~ /%%loop_$nomdia(\((\d+)?\))?%%(.+?)%%\/loop_$nomdia%%/s) {
                                    # rescata plt de loop para articulos de ese dia
                                    my $maxartics = defined($2)?$2:0;
                                    my $artics_loop = $3;

                                    # articulos de este dia
                                    my $artics_loop_out = '';
                                    my $count_artics = 0;
                                    # usamos el total de articulos para el día
                                    my $tot_artics = @{$artics{"$currentday"}};
                                    # si el total de articulos es mayor que el limite del loop != 0
                                    # usamos el limite de loop como total
                                    if ($maxartics > 0 && $tot_artics > $maxartics) {
                                        $tot_artics = $maxartics;
                                    }
                                    foreach my $art (@{$artics{"$currentday"}}) {
                                        $count_artics++;
                                        last if ($maxartics > 0 && $count_artics > $maxartics);
                                        my $fila = lib_parser::generar_fila($art->{'ts'}, $artics_loop, $count_artics, $tot_artics, $mv, 1);
                                        $artics_loop_out .= $fila;
                                    }
                                    $loop_out =~ s/%%loop_$nomdia(\(.*?\))?%%.*?%%\/loop_$nomdia%%/$artics_loop_out/sg;
                                }
                            } else {
                                $loop_out =~ s/%%loop_$nomdia(\(.*?\))?%%.*?%%\/loop_$nomdia%%//sg;
                            }
                            $loop_out =~ s/%%$nomdia%%/$nrodia/g;
                        } else {
                            $loop_out =~ s/%%$nomdia%%/&nbsp;/g;
                            $loop_out =~ s/%%loop_$nomdia(\(.*?\))?%%.*?%%\/loop_$nomdia%%//sg;
                        }
                    }
                    $loop_out =~ s/%%.*?%%//sg;
                    $rows_out{$mv}{$ext} .= $loop_out;
                }
            }
            # si no tiene las marcas LUN%% MAR%%.. etc significa que no parsea dias, por lo que se trata como una lista normal
            # similar a una taxonomica
            else {
                my @days = sort keys %artics;
                my %art_list;
                foreach my $currentday (@days) {
                    foreach my $art (@{$artics{"$currentday"}}) {
                        $art_list{$art->{'ts'}} = $art;
                    }
                }
                my $count_artics = 0;
                foreach my $ts (sort keys %art_list) {
                    $count_artics++;
                    my $fila = lib_parser::generar_fila($ts, $BUF_PLT_LOOP_CALEND{$mv}{$ext}, $count_artics, $total_artics, $mv, 1);
                    $rows_out{$mv}{$ext} .= $fila;
                }
            }
        }
    }

    # parseos generales y escritura del archivo "hoja de calendario" para cada vista.
    foreach my $mv (@vistas) {
        # Nombre de la secc en la vista correspondiente
        my $port_dir;
        my $mv_aux = $mv;
        if ($mv eq '<vista_normal>') {
            $mv_aux = '';
            $port_dir =  "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar/$aaaa$mm/port";
        } else {
            $port_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar/$aaaa$mm/port-$mv";
        }
        foreach my $ext (keys %{$BUF_PLT_CALEND{$mv}}) {
            my $nom_mes = uc &get_mes(int($mm), $mv);
            # parsea pagina
            my $pagina = $BUF_PLT_CALEND{$mv}{$ext};
            $pagina =~ s/%%loop%%.*?%%\/loop%%/$rows_out{$mv}{$ext}/sg;
            $pagina =~ s/%%nom_mes%%/$nom_mes/g;
            $pagina =~ s/%%aaaa%%/$aaaa/g;
            $pagina =~ s/%%mm%%/$mm/g;

            my $secc_nom = $TABLA_SECC{$secc_id}{'nombre'};
            if ($mv_aux && $TABLA_SECC{$secc_id}{'vistas'}{$mv_aux}) {
                $secc_nom = $TABLA_SECC{$secc_id}{'vistas'}{$mv_aux};
            }
            $secc_nom = lib_encodes::escape_html($secc_nom);

            $pagina =~ s/%%nomsecc%%/$secc_nom/g;
            $pagina =~ s/%%secc%%/$secc_id/g;

            # reemplazar variables globales
            $pagina = lib_parser::parse_globals($pagina, '');
            # procesa prontus function
            $pagina = lib_prontus::parser_custom_function($pagina);
            $pagina =~ s/%%.*?%%//sg;

            # Escribe pagina en dir correspondiente
            glib_fildir_03::check_dir($port_dir);
            $secc_id = 'all' if (!$secc_id);

            my $dst_hojacal = "$port_dir/$secc_id$ext";
            # print STDERR "writing[$dst_hojacal]\n";
            glib_fildir_03::write_file($dst_hojacal, $pagina);
            lib_prontus::purge_cache($dst_hojacal);
        }
    }
}

# ---------------------------------------------------------------
sub linkear_dia {
    my ($aaaa, $mm, $dd, $secc_id, $mv, $clase, $ext) = @_;

    $mv = '' if ($mv eq '<vista_normal>');
    $mv = '-' . $mv if ($mv);

    $secc_id = 'all' if (!$secc_id);
    my $dst_dia = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar/$aaaa$mm$dd/pags$mv/$secc_id$ext";

    $dd =~ s/^0//;
    return "<a href=\"$dst_dia\" class=\"$clase\">$dd</a>";
}

# ---------------------------------------------------------------
# Genera el arreglo de semanas @SEMANAS (cada componente corresponde a una semana).
sub get_semanas {
    my ($aaaa, $mm) = @_;
    my ($sem, $dia, $fec_secs, $sday);

    $sem = 0;
    &perpetual_calendar($mm,1,$aaaa);

    for ($dia = 1; $dia <= $PERP_EOM; $dia++) {
        # Fecha actual en formato segundos.
        $fec_secs = &POSIX::mktime(0,0,12,$dia,($mm - 1),($aaaa - 1900));
        # Dia de la semana.
        $sday = (localtime($fec_secs))[6];
        # domingo es 0, pero debe ser 7, ultimo dia de la semana
        $sday = 7 if ($sday == 0);

        # crea hash 'wday' => 'date' para buscar despues mas facilmente
        $SEMANAS[$sem]{$sday} = $dia;

        if ($sday == 7) {
            $sem++;
        }
    }
}

# ---------------------------------------------------------------
# This perpetual calendar routine provides accurate day/date
# correspondences for dates from 1601 to 2899 A.D.  It is based on
# the Gregorian calendar, so be aware that early correspondences
# may not always be historically accurate.  The Gregorian calendar
# was adopted by the Italian states, Portugal and Spain in 1582,
# and by the Catholic German states in 1583.  However, it was not
# adopted by the Protestant German states until 1699, by England
# and its colonies until 1752, by Sweden until 1753, by Japan
# until 1873, by China until 1912, by the Soviet Union until 1918,
# and by Greece until 1923.
sub perpetual_calendar {
    my ($perp_mon,$perp_day,$perp_year) = @_;
    my %day_counts =
      (1,0,2,31,3,59,4,90,5,120,6,151,7,181,
      8,212,9,243,10,273,11,304,12,334);
    my $perp_days = (($perp_year-1601)*365)+(int(($perp_year-1601)/4));
    $perp_days += $day_counts{$perp_mon};
    $perp_days += $perp_day;
    my $perp_sofar = $day_counts{$perp_mon};
    $perp_sofar += $perp_day;
    my $perp_togo = 365-$perp_sofar;
    if (int(($perp_year-1600)/4) eq (($perp_year-1600)/4)) {
        $perp_togo++;
        if ($perp_mon > 2) {
            $perp_days++;
            $perp_sofar++;
            $perp_togo -= 1;
        }
    }

    foreach my $key (1700,1800,1900,2100,2200,2300,2500,2600,2700) {
        if ((($perp_year == $key) && ($perp_mon > 2))
          || ($perp_year > $key)) {
            $perp_days -= 1;
        }
    }

    my $perp_dow = $perp_days - (int($perp_days/7)*7);
    if ($perp_dow == 7) { $perp_dow = 0; }

    $PERP_EOM = 31;
    if (($perp_mon == 4) || ($perp_mon == 6)
      || ($perp_mon == 9) || ($perp_mon == 11)) {
        $PERP_EOM = 30;
    }

    if (($perp_mon == 2)) {
        $PERP_EOM = 28;
    }

    if ((int(($perp_year-1600)/4) eq (($perp_year-1600)/4))
      && ($perp_mon == 2)) {
        $PERP_EOM = 29;
    }

    foreach my $key (1700,1800,1900,2100,2200,2300,2500,2600,2700) {
        if ($perp_year == $key) {
            if ($perp_mon == 1) {
                $perp_togo -= 1;
            } elsif ($perp_mon == 2) {
                $perp_togo -= 1;
                $PERP_EOM = 28;
            } else {
                $perp_sofar -= 1;
            }
        }
    }
}

# ---------------------------------------------------------------
sub precarga_plantillas_calend {
    # Plantilla completa --> %BUF_PLT
    # Loop de cada fila de hoja de calend --> %BUF_PLT_LOOP

    # Para vista normal.
    my $plt_dir = "$prontus_varglb::DIR_SERVER$RELDIR_PORT_TMP_CALEND";
    if (! -d "$plt_dir") {
        print lib_lang::get_interpolated_message('cron_taxcalend_invalid_dir', 'calend', $plt_dir)."\n";
        exit;
    }

    my @plts = glib_fildir_03::lee_dir("$plt_dir");

    my $counter = 0;
    foreach my $plt (@plts) {
        my $ext;
        if ($plt !~ /\w+(\.\w+)/) {
            next;
        } else {
            $ext = $1;
        }
        my $pagina = glib_fildir_03::read_file("$plt_dir/$plt");

        if ($pagina eq '') {
            next;
        }
        $counter++;

        $BUF_PLT_CALEND{'<vista_normal>'}{$ext} = $pagina;

        $BUF_PLT_LOOP_CALEND{'<vista_normal>'}{$ext} = '';
        if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
            $BUF_PLT_LOOP_CALEND{'<vista_normal>'}{$ext} = $1;
        }

        # <!-- MONTH_LIST=January,February,March,April,May,June,July,August,September,October,November,December -->
        if ($BUF_PLT_CALEND{'<vista_normal>'}{$ext} =~ s/<!--\s*MONTH_LIST\s*=\s*([\w,]+)\s*-->//g) {
            print STDERR 'normal  ', $1, "\n";
            @{$MONTH_LABEL{'<vista_normal>'}} = split(',', $1);
        }

    }

    if ($counter == 0) {
        print lib_lang::get_interpolated_message('cron_taxcalend_invalid_dir_view', 'normal')."\n";
        exit;
    }
    # fin vista normal

    # Multivistas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
        $plt_dir = "$prontus_varglb::DIR_SERVER$RELDIR_PORT_TMP_CALEND-$mv";
        if (! -d "$plt_dir") {
            print lib_lang::get_interpolated_message('cron_taxcalend_invalid_dir', $plt_dir)."\n";
            exit;
        }

        @plts = glib_fildir_03::lee_dir("$plt_dir");

        $counter = 0;
        foreach my $plt (@plts) {
            my $ext;
            if ($plt !~ /\w+(\.\w+)/) {
                next;
            } else {
                $ext = $1;
            }
            my $pagina = glib_fildir_03::read_file("$plt_dir/$plt");

            if ($pagina eq '') {
                next;
            }
            $counter++;

            $pagina =~ s/%%_mv%%/$mv/g;

            $BUF_PLT_CALEND{$mv}{$ext} = $pagina;

            $BUF_PLT_LOOP_CALEND{$mv}{$ext} = '';
            if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
                $BUF_PLT_LOOP_CALEND{$mv}{$ext} = $1;
            }

            if ($BUF_PLT_CALEND{$mv}{$ext} =~ s/<!--\s*MONTH_LIST\s*=\s*([\w,]+)\s*-->//g) {
                print STDERR "$mv  ", $1, "\n";
                @{$MONTH_LABEL{$mv}} = split(',', $1);
            }
        }

        if ($counter == 0) {
            # asignar plantillas de vista normal?
            %{$BUF_PLT_CALEND{$mv}} = %{$BUF_PLT_CALEND{'<vista_normal>'}};
            %{$BUF_PLT_LOOP_CALEND{$mv}} = %{$BUF_PLT_LOOP_CALEND{'<vista_normal>'}};
            print lib_lang::get_interpolated_message('cron_taxcalend_invalid_dir_view', $mv)."\n";
            @{$MONTH_LABEL{$mv}} = @{$MONTH_LABEL{'<vista_normal>'}};
        }
    }

        my $relpathfile_port_tmp_json = $RELDIR_PORT_TMP_CALEND . '/' . 'mes.json';

    if (-f "$prontus_varglb::DIR_SERVER$relpathfile_port_tmp_json") {
        my $pagina = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$relpathfile_port_tmp_json");
        $BUF_PLT_CALEND_JSON = $pagina;

        $BUF_PLT_LOOP_CALEND_JSON = '';
        if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/sg) {
            $BUF_PLT_LOOP_CALEND_JSON = $1;
        }
    }
}

# ---------------------------------------------------------------
sub valida_param {
    my ($prontus_id, $aaaamm) = @_;
    $prontus_id =~ s/\\/\//g;
    $prontus_id =~ s/\/$//g;

    if ((!-d "$prontus_varglb::DIR_SERVER/$prontus_id") || ($prontus_id eq '') || ($prontus_id eq '/') )  {
        print "\n".lib_lang::get_message('global_error_invalid_dir');
        print "\n".lib_lang::get_message('cron_taxcalend_error_params')."\n";
        exit;
    }

    $aaaamm =~ s/[^0-9]//g;

    if (($aaaamm !~ /^\d{6}$/) && ($aaaamm) )  {
        print "\n". lib_lang::get_message('cron_taxcalend_error_month');
        print "\n".lib_lang::get_message('cron_taxcalend_error_month_use')."\n";
        exit;
    }
}

# ---------------------------------------------------------------
sub get_mes {
    my ($mm, $mv) = @_;
    return $MONTH_LABEL{$mv}[$mm - 1];
}

# ---------------------------------------------------------------
sub precarga_plantillas_portadillas {
    # Plantilla completa --> %BUF_PLT
    # Loop --> %BUF_PLT_LOOP

    # Para vista normal.
    my $plt_dir = "$prontus_varglb::DIR_SERVER$RELDIR_PORT_TMP";
    if (! -d "$plt_dir") {
        print lib_lang::get_interpolated_message('cron_taxcalend_error_taxport_dir', $plt_dir)."\n";
        exit;
    }

    my @plts = glib_fildir_03::lee_dir("$plt_dir");

    my ($counter, $counter_all) = (0, 0);
    foreach my $plt (@plts) {
        my $ext;
        my $name;
        if ($plt !~ /(\w+)(\.\w+)/) {
            next;
        } else {
            $name = $1;
            $ext = $2;
        }

        my $pagina = glib_fildir_03::read_file("$plt_dir/$plt");
        if ($pagina eq '') {
            next;
        }

        if ($name eq 'portcalendar') {
            $BUF_PLT{'<vista_normal>'}{$ext} = $pagina;

            $BUF_PLT_LOOP{'<vista_normal>'}{$ext} = '';
            if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
                $BUF_PLT_LOOP{'<vista_normal>'}{$ext} = $1;
            }
            $counter++;
            next;
        }
        if ($name eq 'portallcalendar') {
            $BUF_PLT_PORTALL{'<vista_normal>'}{$ext} = $pagina;

            $BUF_PLT_LOOP_PORTALL{'<vista_normal>'}{$ext} = '';
            if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
                $BUF_PLT_LOOP_PORTALL{'<vista_normal>'}{$ext} = $1;
            }
            $counter_all++;
            next;
        }
    }

    if ($counter == 0) {
        print lib_lang::get_interpolated_message('cron_taxcalend_invalid_templates', 'portcalendar', 'normal')."\n";
        exit;
    }

    if ($counter_all == 0) {
        print lib_lang::get_interpolated_message('cron_taxcalend_invalid_templates', 'portallcalendar', 'normal')."\n";
        exit;
    }
    # fin vista normal

    # Multivistas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
        $plt_dir = "$prontus_varglb::DIR_SERVER$RELDIR_PORT_TMP-$mv";
        if (! -d "$plt_dir") {
            print lib_lang::get_interpolated_message('cron_taxcalend_invalid_dir', 'taxport', $plt_dir)."\n";
            exit;
        }

        my @plts = glib_fildir_03::lee_dir("$plt_dir");

        my ($counter, $counter_all) = (0, 0);
        foreach my $plt (@plts) {
            my $ext;
            my $name;
            if ($plt !~ /(\w+)(\.\w+)/) {
                next;
            } else {
                $name = $1;
                $ext = $2;
            }

            my $pagina = glib_fildir_03::read_file("$plt_dir/$plt");
            if ($pagina eq '') {
                next;
            }

            $pagina =~ s/%%_mv%%/$mv/g;

            if ($name eq 'portcalendar') {
                $BUF_PLT{$mv}{$ext} = $pagina;

                $BUF_PLT_LOOP{$mv}{$ext} = '';
                if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
                    $BUF_PLT_LOOP{$mv}{$ext} = $1;
                }
                $counter++;
                next;
            }
            if ($name eq 'portallcalendar') {
                $BUF_PLT_PORTALL{$mv}{$ext} = $pagina;

                $BUF_PLT_LOOP_PORTALL{$mv}{$ext} = '';
                if ($pagina =~ /%%loop%%(.+?)%%\/loop%%/s) {
                    $BUF_PLT_LOOP_PORTALL{$mv}{$ext} = $1;
                }
                $counter_all++;
                next;
            }
        }

        if ($counter == 0) {
            %{$BUF_PLT{$mv}} = %{$BUF_PLT{'<vista_normal>'}};
            %{$BUF_PLT_LOOP{$mv}} = %{$BUF_PLT_LOOP{'<vista_normal>'}};
            print lib_lang::get_interpolated_message('cron_taxcalend_invalid_subtemplates', 'portcalendar', $mv)."\n";
        }

        if ($counter_all == 0) {
            %{$BUF_PLT_PORTALL{$mv}} = %{$BUF_PLT_PORTALL{'<vista_normal>'}};
            %{$BUF_PLT_LOOP_PORTALL{$mv}} = %{$BUF_PLT_LOOP_PORTALL{'<vista_normal>'}};
            print lib_lang::get_interpolated_message('cron_taxcalend_invalid_subtemplates', 'portallcalendar', $mv)."\n";
        }
    }
}

# ---------------------------------------------------------------
sub write_portadilla {
    my ($fecha, $secc_id, $filas_hashref) = @_;
    my %filas = %$filas_hashref;

    # escribe pagina en todas las vistas incluida la normal
    # incluye las mv y la normal
    my @vistas = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}};
    push(@vistas, '<vista_normal>');

    foreach my $mv (@vistas) {
        my $pags_dir;
        my $mv_aux = $mv;
        if ($mv eq '<vista_normal>') {
            $mv_aux = '';
            $pags_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar/$fecha/pags";
        } else {
            $pags_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/calendar/$fecha/pags-$mv";
        }

        my %aux_plt;
        if (!$secc_id) {
            %aux_plt = %{$BUF_PLT_PORTALL{$mv}};
        } else {
            %aux_plt = %{$BUF_PLT{$mv}};
        }

        foreach my $ext (keys %aux_plt) {
            # print "mv[$mv] seccion[$secc_id]\n";
            # parsea pagina
            my $pagina = $aux_plt{$ext};
            my $lista = $filas{$mv}{$ext};
            $filas{$mv}{$ext} = ''; # reset

            $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;

            # Nombre de la secc en la vista correspondiente
            my $secc_nom = $TABLA_SECC{$secc_id}{'nombre'};
            if ($mv_aux && $TABLA_SECC{$secc_id}{'vistas'}{$mv_aux}) {
                $secc_nom = $TABLA_SECC{$secc_id}{'vistas'}{$mv_aux};
            }
            $secc_nom = lib_encodes::escape_html($secc_nom);

            $pagina =~ s/%%nomsecc%%/$secc_nom/g;
            $pagina =~ s/%%secc%%/$secc_id/g;

            # reemplazar variables globales
            $pagina = lib_parser::parse_globals($pagina, '');
            $pagina = lib_prontus::parser_fecha_ext($pagina, $fecha, '_fechap');
            $pagina = lib_prontus::parser_custom_function($pagina);
            $pagina =~ s/%%.*?%%//sg;

            # Prepara y chequea dir para escribir
            glib_fildir_03::check_dir($pags_dir);

            my $secc_id4filename = $secc_id;
            $secc_id4filename = 'all' if (!$secc_id);
            my $dst_portadilla = "$pags_dir/$secc_id4filename$ext";

            # Escribe pagina
            glib_fildir_03::write_file($dst_portadilla, $pagina);
            lib_prontus::purge_cache($dst_portadilla);
        }
    }
}
