#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

#
# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Generar un indice para el buscador de Prontus (Prontus Search).
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# Directorios Arbitrarios.
# Este indexador recorre todo el directorio de inicio, determinando
# si el indice debe ser reconstruido.
#
# El indice de los directorios arbitrarios es almacenado en:
# cpan/data/search/raw/
#
# Directorios Prontus.
# Este indexador recorre todo el directorio site/artic de un Prontus,
# identificando anos y determinando si los indices correspondientes
# deben ser reconstruidos.
#
# El indice de cada ano es almacenado en directorios de la forma:
# cpan/data/search/<nombre prontus>/aaaa/
#
# Los archivos de indice generados son:
#
# words.idx      - Indice de palabras.
#   Formato: Palabra=ID.
#               Las palabras se guardan ordenadas, pero los IDs se
#               generan on the fly, asi que no tienen un orden.
#
# cars.idx       - Indice alfabetico de primeros caracteres de palabras.
#   Formato: Letra=Offset.
#               Offset en el archivo de palabras donde parte ese caracter.
#               (Este archivo ya no se usa, pero se genera igual).
#
# files.idx      - Indice de articulos.
#   Formato:
#   IDarticulo=fechap|TS.extension|titular|meta1|meta2|meta3|resumen|seccion|tema|subtema|metadata1|..|metadata10
#   seccion, tema y subtema van en texto; no en IDs.
#   Si el archivo no es Prontus, fechap, seccion, tema y subtema van en
#   blanco, y TS.extension corresponde a la ubicacion del archivo.
#
# filesindex.idx - Indice de articulos y sus palabras.
#   Formato:
#   IDarticulo=|IDpalabra count|IDpalabra count|IDpalabra count|IDpalabra count|...
#   Este es un archivo temporal que es eliminado despues de la indexacion.
#
# filesindexf.idx - Indice de articulos y sus palabras, disenado para la busqueda por frases.
#   Formato:
#   IDarticulo=|IDpalabra ubicacion1 ubicacion2 ...|IDpalabra ubicacion1 ubicacion2 ...|...
#   Este es un archivo temporal que es eliminado despues de la indexacion.
#
# wordsindex.idx - Indice de palabras y los articulos que las contienen.
#   Formato:
#   IDpalabra=|IDarticulo count|IDarticulo count|IDarticulo count|IDarticulo count|...
#
# El archivo utilizado como semaforo es: semaforo.txt
#
# Dentro del archivo de configuracion Prontus Search, se tiene:
# # Directorios de otros Prontus a Indexar (i = 1, 2, 3,...) aparte del directorio de este Prontus.
# PRONTUS_DIR_i = <prontus dir>
# # Directorios Arbitrarios a Indexar (i = 1, 2, 3,...).
# RAW_DIR_i = <dir desde la raiz del sitio>
# # Tipos de Archivos Arbitrarios a Indexar.
# RAW_FILETYPES = <extensiones separadas por espacios>
# # Tipos de URLs a Indexar en modalidad spider.
# # Ademas de estos, se indexan los directorios (/) y urls que terminan en un string sin puntos.
# URL_FILETYPES = html htm shtml php asp jsp
# # Maximo de paginas a indexar.
# URL_MAXPAGS = 100
# # URLs a indexar en modalidad spider.
# URL_DIR_1 = http://www.nic.cl/
# # Límite asociado al punto de partida "n".
# # Esto permite acotar la búsqueda a subdirectorios dentro del URL inicial,
# # para indexar sólo una parte del sitio web.
# URL_SCOPE_1 = http://www.nic.cl/
# # Tipos de FID que seran consideradas validos para ser indexados.
# FIDS = fid_diario.html fid_general.html
# # Numero de caracteres a almacenar como resumen.
# RESUMEN = 100
# # Tamano maximo de la data a indexar.
# MAXCARS = 100000
# # Limite para palabras irrelevantes (%).
# RATIO = 50
# # Limite de caracteres para considerar que el texto es significativo.
# MINTEXT = 20
# # Variable Prontus que sera considerada el titular.
# TITLEVAR = TITULAR
# # Variables Prontus sobre las que se buscara.
# TEXTVARS = TEXTbajada TEXTarticulo
# # Resultados por pagina.
# RESPERPAG = 50
# # Maximo de paginas a mostrar.
# MAXPAGS = 20
# # Maximo de instancias permitidas (Unix).
# SEARCH_MAXEXEC = 5
# # Usa friendly urls (1) o no (0).
# USEFRIENDLYURLS = 1
# # Variables "META", usadas para perfilacion de contenidos u otra clasificacion.
# # Variables validas: META1 META2 y META3
# META1 = RDO_ROL
# META2 = RDO_AREA
# # Variables "META" adicionales, usadas para perfilacion de contenidos u otra clasificacion.
# # Variables validas: METADATA1 .. METADATA10
# METADATA1 = alerta
# METADATA2 = imagen
# # Filtro para archivos adjuntos.
# # FILEFILTER <extension> <ejecutable>
# # Si se especifica, indexara los archivos adjuntos de esa extension.
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Como cron, linea de comandos o post-proceso:
# prontus_indexer.pl <prontus id> [<forzar>]
#   donde <prontus id> es el nombre del Prontus.
#           Ejemplo: para /sites/mercuriovalpo.cl/web/prontus_noticias, el prontus id es "prontus_noticias".
#         <forzar> es el Prontus (o raw), o el año para el que se desea forzar un reindexado.
#
# Desde la web, como cgi:
# prontus_indexer.pl?_prontus_id=<prontus id>&forzar=[<forzar>]
#   Ejemplo:
#     www.prontus.cl/cgi-cpn/prontus_indexer.pl?_prontus_id=prontus
#     o
#     www.prontus.cl/cgi-cpn/prontus_indexer.pl?_prontus_id=prontus&forzar=7777
#     (En modalidad cgi, si no se incluye el document_root, Prontus lo inserta)
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
# No registra.
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

END {
    &tareas_salida();
}

use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use HTTP::Request;
use JSON;
use LWP::UserAgent;
use glib_fildir_03;
use glib_str_03;
use lib_encodes;
use lib_lang;
use lib_logproc;
use lib_search;
use lib_utils;
##/libs##

# Raiz del sitio web.
my $DOCUMENT_ROOT = $prontus_varglb::DIR_SERVER;

# Variables de la invocacion.
my $INVOCACION; # el origen de la invcacion: web o cron
my $PRONTUS_ID; # el ID del prontus que invoca a esta CGI.
my $FORZAR; # forzar la actualización de datos. Puede ser un año (formato YYYY, para indexar los últimos datos usar el año 7777), el nombre de un prontus, o un raw_dir.

my $FORZAR_ULTIMOS_DIAS = '7777';
my $FORZAR_TODO = '8888';

my $PRONTUS_DIR; # el path al webroot, esto se rellena a partir de la config obtenida con $PRONTUS_ID

# parametros de invocacion
my $prontus_id = $ARGV[0];
$FORZAR = $ARGV[1];

$INVOCACION = 'cron';
if ($ARGV[2] eq 'web') {
    $INVOCACION = 'web'; # se escriben archivos para presentar el avance via GUI
}

lib_utils::setUp();

if ($ARGV[3]) {
    lib_lang::load_translation_table($ARGV[3]);
}

# chequeamos existencia de parámetros básicos
my $uso_cgi = lib_lang::get_interpolated_message("indexer_use", $FORZAR_ULTIMOS_DIAS, $FORZAR_TODO);

if ($prontus_id eq '' || !lib_utils::valida_prontus($prontus_id)) {
    if ($INVOCACION eq 'web') {
        &printJSON(lib_lang::get_message('global_invalid_prontus_id'), 1);
    } else {
        print lib_lang::get_message("global_invalid_prontus_id") . "\n";
        print "$uso_cgi\n";
        print STDERR lib_lang::get_interpolated_message("global_invalid_prontus_id_int", $PRONTUS_ID)."\n";
    }
    exit;
}

my $msg_load = lib_utils::load_config_by_id($prontus_id);
if ($msg_load ne '') {
    if ($INVOCACION eq 'web') {
        &printJSON($msg_load, 1);
    } else {
        print "Error: $msg_load\n$uso_cgi\n";
        exit;
    }
}

if ($ARGV[3]) {
    lib_lang::load_translation_table($ARGV[3]);
}

# establecemos el webroot/prontus_id
$PRONTUS_DIR = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID";

# Establece log file
$lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_reindex_log.html";

# handlres de archivos
my $FILES;
my $FILESINDEX;
my $FILESINDEXF;

# Variables globales.
my $ROOTDIR; # Directorio raiz del sitio web.
my $SEARCH_DIR = "$PRONTUS_DIR/cpan/data/search"; # Directorio de trabajo.
my $CFG_FILENAME = 'buscador_prontus.cfg';     # Nombre del archivo de configuracion.
my $WORDS_FILENAME = 'words.idx';   # Nombre de los archivos de palabras.
my $CARS_FILENAME = 'cars.idx';     # Nombre de los archivos de caracteres.
my $FILES_FILENAME = 'files.idx';   # Nombre de los archivos indice de articulos.
my $FILESINDEX_FILENAME = 'filesindex.idx'; # Nombre de los archivos indice de archivos.
my $WORDSINDEX_FILENAME = 'wordsindex.idx'; # Nombre de los archivos de indice de palabras.
my $FILESINDEXF_FILENAME = 'filesindexf.idx'; # Nombre de los archivos indice de archivos para busqueda por frases.
my $SEMAFORO_FILENAME = 'semaforo.txt';     # Nombre del archivo semaforo.
my $RAW_DIRNAME = 'raw';     # Nombre del directorio de indices de directorios brutos (raw).
my %CFG = ();   # Variables del archivo de configuracion.
my %WORDS = (); # Palabras.
my $WIDX;       # Indice de palabras.
my $FIDX;       # Indice de archivos.
my @PRONTUS_DIRS; # Prontus que hay que indexar.
my @RAW_DIR;     # Directorios brutos que hay que indexar.
my @RAW_FILETYPES; # Extensiones validas para los archivos raw.
my $PRONTUS;    # Usado para almacenar cada uno de los publicadores Prontus a indexar. A diferencia de $PRONTUS_ID, esta variable puede cambiar.
my $DIR;        # Nombre del directorio bruto (raw).
my %FILEFILTER; # Filtros por extension, para indexar archivos de datos.

my $URL_DIRNAME = 'url'; # Nombre del directorio de indices de directorios URL (spider).
my @URL_DIR;             # URLs que hay que indexar.
my %LINKS;               # URL visitados por el spider.
my $URL_FILETYPES_REGEXP; # Expresion regular para validar los URL a indexar.
my $PROFUNDIDAD = 0;     # Profundidad de la navegacion del spider (para limitarla).

# Deduce el nombre del publicador Prontus.
$PRONTUS_DIR =~ s/\\/\//g; # Convierte backslashes en slashes.
if ($PRONTUS_DIR =~ /^(.+)(\/[^\/]+)$/) {
    $ROOTDIR = $1;
    $PRONTUS_DIRS[0] = $2; # (comentar para que no indexe de nuevo las noticias)
} else {
    my $error_dir_prontus = lib_lang::get_message('search_prontus_dir_invalid');
    if ($INVOCACION eq 'web') {
        lib_logproc::add_to_log($error_dir_prontus);
        &printJSON($error_dir_prontus, 1);
    } else {
        print STDERR "$error_dir_prontus [$PRONTUS_DIR].\n";
    }
    exit;
}

my $FECHAACTUAL = lib_search::fecha_iso(); # Fecha de hoy en formato ISO.
my $FECHAHORAACTUAL = lib_search::timestamp_iso(); # Fecha y hora de hoy en formato ISO.
my %ANOS;  # Anos a indexar. $ANOS{ano} = time del archivo mas recientemente modificado.
my $ANO;   # Ano actualmente indexandose.
my $MTIME; # Tiempo de modificacion de un indice.

# ###################################################
# Main
# Verifica semaforo.
if (!$FORZAR && -f "$SEARCH_DIR/$SEMAFORO_FILENAME") {

    # Si el semaforo es menos antiguo que 3 horas, aborta la ejecucion.
    # (0: $dev, 1: $ino, 2: $mode, 3: $nlink, 4: $uid, 5: $gid, 6: $rdev, 7: $size, 8: $atime, 9: $mtime, 10: $ctime, 11: $blksize, 12: $blocks).
    if ((stat("$SEARCH_DIR/$SEMAFORO_FILENAME"))[9] > (time - 10800)) {

        my $error_semaphore = lib_lang::get_message("indexer_active_semaphore");
        if ($INVOCACION eq 'web') {

            lib_logproc::add_to_log($error_semaphore);
            &printJSON($error_semaphore, 1);
        } else {
            print lib_lang::get_message("indexer_active_semaphore_retry");
        }
        exit;
    }
}
unlink "$SEARCH_DIR/$SEMAFORO_FILENAME";
open my $SEMAFORO, '>', "$SEARCH_DIR/$SEMAFORO_FILENAME";
print $SEMAFORO time;
close $SEMAFORO;

# Lee archivo de configuracion del buscador.
%CFG = lib_search::get_config("$prontus_varglb::DIR_SERVER/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/$CFG_FILENAME");

# Valida las variables del archivo de configuracion.
&validacfg();

# Detecta los Prontus y directorios brutos que hay que indexar.
&detectadirs();

# Parte indexando los Prontus.
my ($error_dir_prontus_noexiste, $buscando_articulos, $escribiendo_indices, $escribiendo_indices_cruzados, $sustituyendo_indices, $fin_indexacion);
foreach my $prontus (@PRONTUS_DIRS) {
    $PRONTUS = $prontus; # Esto es para visibilidad de esta variable dentro de las rutinas.
    %ANOS = ();

    # print "Indexando $PRONTUS "; # debug
    if ($INVOCACION eq 'web') {
        &printJSON(lib_lang::get_interpolated_message('indexer_indexing', $PRONTUS), 0);
        lib_logproc::add_to_log(lib_lang::get_interpolated_message('indexer_indexing', $PRONTUS));
    }

    if (!-d "$ROOTDIR$PRONTUS") {
        if ($INVOCACION eq 'web') {
            $error_dir_prontus_noexiste = lib_lang::get_interpolated_message("indexer_prontus_dir_missing", $PRONTUS);
            &printJSON($error_dir_prontus_noexiste, 0);
            lib_logproc::add_to_log($error_dir_prontus_noexiste);
        }
        next;
    }

    # print "$PRONTUS\n"; # debug
    if ($FORZAR =~ /^\d{4}$/ && $FORZAR ne $FORZAR_TODO) {
        # Si hay $FORZAR entonces se indexa solo el ano indicado.
        $ANOS{$FORZAR} = 1;
    } else {
        # Si no, determina los anos existentes y determina la fecha de modificacion mas reciente para cada uno.
        &busca_anos("$ROOTDIR$PRONTUS/site/artic");
    }

    # Recorre los anos reindexando en caso necesario.
    foreach my $ano (sort {$b cmp $a} keys %ANOS) {
        $ANO = $ano; # Esto es para visibilidad de esta variable dentro de las rutinas.
        # print "$ANO "; # debug
        if ($INVOCACION eq 'web') {
            &printJSON($ANO, 0);
            lib_logproc::add_to_log($ANO);
        }

        # Si no es necesario reindexar, continua con el siguiente ano.
        if ($FORZAR ne $FORZAR_TODO && -f "$SEARCH_DIR$PRONTUS/$ANO/$WORDSINDEX_FILENAME") {
            $MTIME = (stat("$SEARCH_DIR$PRONTUS/$ANO/$WORDSINDEX_FILENAME"))[9];

            # print "MTIME=$MTIME\n"; # debug
            # print $MTIME . '<=>' . $ANOS{$ANO} . "\n" ; # debug
            if ($FORZAR ne $ANO) {
                next if ($MTIME > $ANOS{$ANO});
            }
        }
        %WORDS = ();
        $WIDX = 0;   # Indice de palabras.
        $FIDX = 0;   # Indice de archivos.
        # Crea el directorio para este indice.
        glib_fildir_03::check_dir("$SEARCH_DIR$PRONTUS/$ANO");

        # Inicia el proceso de indexado global.
        if ($INVOCACION eq 'web') {
            $buscando_articulos = lib_lang::get_interpolated_message("indexer_searching_articles_year", $ANO);
            &printJSON($buscando_articulos, 0);
            lib_logproc::add_to_log($buscando_articulos);
        }
        print STDERR lib_lang::get_interpolated_message('indexer_searching_articles', $PRONTUS);
        &busca_articulos("$ROOTDIR$PRONTUS/site/artic");

        # Escribe indices de palabras.
        if ($INVOCACION eq 'web') {
            $escribiendo_indices = lib_lang::get_interpolated_message("indexer_writing_index_year", $ANO);
            &printJSON($escribiendo_indices, 0);
            lib_logproc::add_to_log($escribiendo_indices);
        }

        &escribe_palabras("$SEARCH_DIR$PRONTUS/$ANO");

        # Genera cross-index de palabras.
        if ($INVOCACION eq 'web') {
            $escribiendo_indices_cruzados = lib_lang::get_interpolated_message("indexer_writing_cross_index_year", $ANO);
            &printJSON($escribiendo_indices_cruzados, 0);
            lib_logproc::add_to_log($escribiendo_indices_cruzados);
        }

        &genera_wordsindex("$SEARCH_DIR$PRONTUS/$ANO");

        # Escribe los archivos definitivos.
        if ($INVOCACION eq 'web') {
            $sustituyendo_indices = lib_lang::get_interpolated_message("indexer_substituting_index_files_year", $ANO);
            &printJSON($sustituyendo_indices, 0);
            lib_logproc::add_to_log($sustituyendo_indices);
        }

        move("$SEARCH_DIR$PRONTUS/$ANO/$WORDS_FILENAME.tmp","$SEARCH_DIR$PRONTUS/$ANO/$WORDS_FILENAME");
        move("$SEARCH_DIR$PRONTUS/$ANO/$CARS_FILENAME.tmp","$SEARCH_DIR$PRONTUS/$ANO/$CARS_FILENAME");
        move("$SEARCH_DIR$PRONTUS/$ANO/$FILES_FILENAME.tmp","$SEARCH_DIR$PRONTUS/$ANO/$FILES_FILENAME");
        move("$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEXF_FILENAME.tmp","$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEXF_FILENAME"); #
        move("$SEARCH_DIR$PRONTUS/$ANO/$WORDSINDEX_FILENAME.tmp","$SEARCH_DIR$PRONTUS/$ANO/$WORDSINDEX_FILENAME");
        unlink "$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEX_FILENAME.tmp";
        unlink "$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEXF_FILENAME.tmp";
        if ($INVOCACION eq 'web') {
            $fin_indexacion = lib_lang::get_interpolated_message("indexer_end_int", $PRONTUS);
            &printJSON($fin_indexacion, 0);
            lib_logproc::add_to_log($fin_indexacion);
        }
    }
    # print "\n"; # debug
}

# Indexa los directorios brutos.
# print "RAWS DIR:@RAW_DIR\n"; # debug
if ($#RAW_DIR >= 0) {

    # Determina si es necesario reindexar o no.
    # print "$FORZAR ne $RAW_DIRNAME\n"; # debug
    if ($FORZAR ne $RAW_DIRNAME) {
        foreach my $dir (@RAW_DIR) {
            $DIR = $dir; # Esto es para visibilidad de esta variable dentro de las rutinas.
            if (! -f "$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDSINDEX_FILENAME") {
                $FORZAR = $RAW_DIRNAME; # Fuerza reindexado si es que no existe el directorio.
                last;
            }
        }
    }

    if ($FORZAR eq $RAW_DIRNAME) {
        %WORDS = ();
        $WIDX = 0;   # Indice de palabras.
        $FIDX = 0;   # Indice de archivos.
        # Crea el directorio para este indice.
        glib_fildir_03::check_dir("$SEARCH_DIR/$RAW_DIRNAME/0000");
        my $raws_indexados;
        unlink "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILES_FILENAME.tmp";
        unlink "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILESINDEX_FILENAME.tmp";
        my ($indexacion_dir, $indexacion_dir_noexiste, $buscando_articulos_dir, $escribiendo_indices_dir, $escribiendo_indices_cruzados_dir, $sustituyendo_indices_dir, $fin_indexacion_dir);

        foreach my $dir (@RAW_DIR) {
            $DIR = $dir; # Esto es para visibilidad de esta variable dentro de las rutinas.
            $MTIME = (stat("$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDSINDEX_FILENAME"))[9];

            # Verifica si es necesario indexar.
            if ($MTIME < &maxtime("$ROOTDIR$DIR")) {

                # print "Indexando $DIR\n"; # debug
                if ($INVOCACION eq 'web') {
                    $indexacion_dir = lib_lang::get_interpolated_message('indexer_indexing', $DIR);
                    &printJSON($indexacion_dir, 0);
                    lib_logproc::add_to_log($indexacion_dir);
                }
                if (!-d "$ROOTDIR$DIR") {
                    if ($INVOCACION eq 'web') {
                        $indexacion_dir_noexiste = lib_lang::get_interpolated_message("indexer_prontus_dir_missing", $DIR);
                        &printJSON($indexacion_dir_noexiste, 0);
                        lib_logproc::add_to_log($indexacion_dir_noexiste);
                    }
                    next;
                }
                open $FILES, '>>', "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILES_FILENAME.tmp";   # Archivo de articulos.
                binmode $FILES, ':encoding(UTF-8)';
                open $FILESINDEX, '>>', "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILESINDEX_FILENAME.tmp"; # Archivo indice de articulos.
                binmode $FILESINDEX, ':encoding(UTF-8)';
                open $FILESINDEXF, '>>', "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILESINDEXF_FILENAME.tmp"; # Archivo indice de articulos para busqueda por frases.
                binmode $FILESINDEXF, ':encoding(UTF-8)';
                if ($INVOCACION eq 'web') {
                    $buscando_articulos_dir = lib_lang::get_interpolated_message("indexer_searching_articles_dir", $DIR);
                    &printJSON($buscando_articulos_dir, 0);
                    lib_logproc::add_to_log($buscando_articulos_dir);
                }
                &busca_articulos_raw("$ROOTDIR$DIR");
                close $FILES;
                close $FILESINDEX;
                close $FILESINDEXF;
                $raws_indexados++;
            }
        }
        if ($raws_indexados) {

            # Escribe indices de palabras.
            if ($INVOCACION eq 'web') {
                $escribiendo_indices_dir = lib_lang::get_interpolated_message("indexer_writing_index_dir", $DIR);
                &printJSON($escribiendo_indices_dir, 0);
                lib_logproc::add_to_log($escribiendo_indices_dir);
            }
            &escribe_palabras("$SEARCH_DIR/$RAW_DIRNAME/0000");

            # Genera cross-index de palabras.
            if ($INVOCACION eq 'web') {
                $escribiendo_indices_cruzados_dir = lib_lang::get_interpolated_message("indexer_writing_cross_index_dir", $DIR);
                &printJSON($escribiendo_indices_cruzados_dir, 0);
                lib_logproc::add_to_log($escribiendo_indices_cruzados_dir);
            }
            &genera_wordsindex("$SEARCH_DIR/$RAW_DIRNAME/0000");

            # Escribe los archivos definitivos.
            if ($INVOCACION eq 'web') {
                $sustituyendo_indices_dir = lib_lang::get_interpolated_message("indexer_writing_cross_index_dir", $DIR);
                &printJSON($sustituyendo_indices_dir, 0);
                lib_logproc::add_to_log($sustituyendo_indices_dir);
            }
            move("$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDS_FILENAME.tmp","$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDS_FILENAME");
            move("$SEARCH_DIR/$RAW_DIRNAME/0000/$CARS_FILENAME.tmp","$SEARCH_DIR/$RAW_DIRNAME/0000/$CARS_FILENAME");
            move("$SEARCH_DIR/$RAW_DIRNAME/0000/$FILES_FILENAME.tmp","$SEARCH_DIR/$RAW_DIRNAME/0000/$FILES_FILENAME");
            move("$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDSINDEX_FILENAME.tmp","$SEARCH_DIR/$RAW_DIRNAME/0000/$WORDSINDEX_FILENAME");
            unlink "$SEARCH_DIR/$RAW_DIRNAME/0000/$FILESINDEX_FILENAME.tmp";
            if ($INVOCACION eq 'web') {
                $fin_indexacion_dir = lib_lang::get_interpolated_message('indexer_end_int', 'raw_dirs');
                &printJSON($fin_indexacion_dir, 0);
                lib_logproc::add_to_log($fin_indexacion_dir);
            }
        }
    }

    # print "\n"; # debug
}

# Indexa los URLs en modalidad spider.
# print "URLs:@URL_DIR\n"; # debug

my $dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/url_list_index";
glib_fildir_03::check_dir($dir);
my @index_list = glib_fildir_03::lee_dir($dir);
# con $#index_list > 1, hay un archivo ademas de . y ..
if ($#URL_DIR >= 0 || $#index_list > 1) {

    # Los URLs son siempre reindexados, ya que no hay forma de saber a priori si cambiaron o no.
    %WORDS = ();
    $WIDX = 0;   # Indice de palabras.
    $FIDX = 0;   # Indice de archivos.
    # Crea el directorio para este indice.
    glib_fildir_03::check_dir("$SEARCH_DIR/$URL_DIRNAME/0000");
    unlink "$SEARCH_DIR/$URL_DIRNAME/0000/$FILES_FILENAME.tmp";
    unlink "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME.tmp";
    my ($url_dir_msg, $leyendo_urls, $escribiendo_indices_urls, $escribiendo_indices_cruzados_urls, $sustituyendo_indices_urls, $fin_indexacion_urls);

    foreach my $file (@index_list) {
        if ($INVOCACION eq 'web') {
            $url_dir_msg = lib_lang::get_interpolated_message('indexer_indexing', $file);
            &printJSON($url_dir_msg, 0);
            lib_logproc::add_to_log($url_dir_msg);
        }
        open $FILES, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILES_FILENAME.tmp";   # Archivo de articulos.
        binmode $FILES, ':encoding(UTF-8)';
        open $FILESINDEX, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME.tmp"; # Archivo indice de articulos.
        binmode $FILESINDEX, ':encoding(UTF-8)';
        open $FILESINDEXF, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEXF_FILENAME.tmp"; # 1.19 Archivo indice de articulos para busqueda por frases.
        binmode $FILESINDEXF, ':encoding(UTF-8)';
        if ($INVOCACION eq 'web') {
            &printJSON($leyendo_urls, 0);
            lib_logproc::add_to_log($leyendo_urls);
        }

        open(my $LISTFILE, '<', "$dir/$file")
          || die "Fail Open file $file \n $!\n";
        binmode $LISTFILE, ':encoding(UTF-8)';
        while(<$LISTFILE>) {
            &parseURL($_);
        }
        close $LISTFILE;

        close $FILES;
        close $FILESINDEX;
        close $FILESINDEXF; # 1.19
    }

    # el limite de urls indexadas se mueve para evitar llegar al limite por las urls definidas manualmente
    $CFG{'URL_MAXPAGS'} += $FIDX;

    foreach my $i (@URL_DIR) {
        if ($INVOCACION eq 'web') {
            $url_dir_msg = lib_lang::get_interpolated_message('indexer_indexing', $i)." ".$CFG{"URL_DIR_$i"};
            &printJSON($url_dir_msg, 0);
            lib_logproc::add_to_log($url_dir_msg);
        }
        open $FILES, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILES_FILENAME.tmp";   # Archivo de articulos.
        binmode $FILES, ':encoding(UTF-8)';
        open $FILESINDEX, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME.tmp"; # Archivo indice de articulos.
        binmode $FILESINDEX, ':encoding(UTF-8)';
        open $FILESINDEXF, '>>', "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEXF_FILENAME.tmp"; # Archivo indice de articulos para busqueda por frases.
        binmode $FILESINDEXF, ':encoding(UTF-8)';
        if ($INVOCACION eq 'web') {
            &printJSON($leyendo_urls, 0);
            lib_logproc::add_to_log($leyendo_urls);
        }
        $PROFUNDIDAD = 0;
        &recursiveSpider($CFG{"URL_DIR_$i"},$CFG{"URL_SCOPE_$i"});
        close $FILES;
        close $FILESINDEX;
        close $FILESINDEXF;
    }

    # Escribe indices de palabras.
    if ($INVOCACION eq 'web') {
        $escribiendo_indices_cruzados_urls = lib_lang::get_interpolated_message('indexer_writing_index_dir', "urls");
        &printJSON($escribiendo_indices_urls, 0);
        lib_logproc::add_to_log($escribiendo_indices_urls);
    }
    &escribe_palabras("$SEARCH_DIR/$URL_DIRNAME/0000");

    # Genera cross-index de palabras.
    if ($INVOCACION eq 'web') {
        $escribiendo_indices_cruzados_urls = lib_lang::get_interpolated_message('indexer_writing_cross_index_dir', "urls");
        &printJSON($escribiendo_indices_cruzados_urls, 0);
        lib_logproc::add_to_log($escribiendo_indices_cruzados_urls);
    }
    &genera_wordsindex("$SEARCH_DIR/$URL_DIRNAME/0000");

    # Escribe los archivos definitivos.
    if ($INVOCACION eq 'web') {
        $sustituyendo_indices_urls = lib_lang::get_interpolated_message('indexer_substituting_index_files_dir', "urls");
        &printJSON($sustituyendo_indices_urls, 0);
        lib_logproc::add_to_log($sustituyendo_indices_urls);
    }
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$WORDS_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$WORDS_FILENAME");
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$CARS_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$CARS_FILENAME");
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$FILES_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$FILES_FILENAME");

    # ??? Investigar por que no se estaba conservando el archivo $FILESINDEX_FILENAME
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME");
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEXF_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEXF_FILENAME");
    move("$SEARCH_DIR/$URL_DIRNAME/0000/$WORDSINDEX_FILENAME.tmp","$SEARCH_DIR/$URL_DIRNAME/0000/$WORDSINDEX_FILENAME");
    unlink "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEX_FILENAME.tmp";
    unlink "$SEARCH_DIR/$URL_DIRNAME/0000/$FILESINDEXF_FILENAME.tmp";
    if ($INVOCACION eq 'web') {
        $fin_indexacion_urls = lib_lang::get_interpolated_message('indexer_end_int', "urls");
        &printJSON($fin_indexacion_urls, 0);
        lib_logproc::add_to_log($fin_indexacion_urls);
    }

    # print "\n"; # debug
}

my $fin_indexacion_total = lib_lang::get_message("indexer_end");
lib_logproc::add_to_log($fin_indexacion_total);
if ($INVOCACION eq 'web') {
    &printJSON($fin_indexacion_total, 1);
}

# Elimina el semaforo.
unlink "$SEARCH_DIR/$SEMAFORO_FILENAME";

# ###################################################
# Funciones

# ------------------------------------------------------------------------#
# Recorre el directorio de archivos en bruto buscando archivos que indexar.
sub busca_articulos_rawejemplo {
    my ($eldir) = $_[0];
    my (@days,$day,$fullpath,$filepath,@webpages,$webpage);

    my @files = glib_fildir_03::lee_dir($eldir); # Lee el directorio.

    foreach my $file (sort {$b cmp $a} @files) {
        next if ( $file =~ /^\./); # Descarta archivos invisibles.
        # print "file = $eldir/$file\n"; # debug
        $fullpath = "$eldir/$file";
        if (-d $fullpath) {

            # Si es un directorio, sige la busqueda de archivos en forma recursiva.
            &busca_articulos_raw($fullpath);
        } elsif (-f $fullpath) {

            # Si es un archivo y la extension coincide con las buscadas, lo indexa.
            next if (!(grep {$file =~ /\.$_$/} @RAW_FILETYPES) );

            # print "fullpath = $fullpath\n"; # debug
            if ($fullpath =~ /^$ROOTDIR(\/.+)$/) { # Extrae la parte bajo el rootdir.
                $filepath = $1;

                # Indexa el archivo.
                # print "indexando $filepath...\n"; # debug
                &indexa_raw($fullpath,$filepath);
            }
        }
    }
}

# ------------------------------------------------------------------------
# Rutina que recorre recursivamente un URL de partida, hasta
# detectar todos los URLs dentro del mismo 'scope'.
# El 'scope' es el limite superior de los links para ser recorridos.
# No seran recorridos links que apunten mas arriba, en el sentido de
# organizacion de directorios, que el scope pasado como parametro.
sub recursiveSpider {
    my ($url,$scope) = @_;
    my (@lnk) = &simpleSpider($url,$scope);
    my (@lnk2);

    # Primero recorre todos los links encontrados en esta pagina.
    foreach my $lnk (@lnk) {
        # print "lnk=$lnk\n"; # debug
        push @lnk2,&simpleSpider($lnk,$scope);
    }

    # Luego recorre los links encontrados en las paginas recorridas.
    foreach my $lnk (@lnk2) {
        # print "lnk2=$lnk\n"; # debug
        $PROFUNDIDAD++;
        if ($PROFUNDIDAD <= 20) { # Limite de recursividad.
            &recursiveSpider($lnk,$scope);
        }
        $PROFUNDIDAD--;
    }
}; # recursiveSpider

# ------------------------------------------------------------------------#
sub parseURL {
    my $url = $_[0];
    my ($protocolo,$host,$path,$file,$params,$lnk,$i,$l,$p);
    my @lnk = ();

    # print "parseURL: $url\n"; # debug
    return if ($LINKS{$url} ne '');
    $LINKS{$url} = 1;

    # Aborta si se ha llegado el limite de archivos.
    return if ($FIDX > $CFG{'URL_MAXPAGS'});

    # Lee y retorna la pagina web pasada como parametro.
    my $buffer = '';
    my $ua = new LWP::UserAgent;
    $ua->agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36");
    # no verifica certificados para poder acceder a sitios por ip que pueden entregar un certificado invalido
    $ua->ssl_opts(verify_hostname => 0,
              SSL_verify_mode => 0x00);
    $ua->max_size(100000); # Pone este limite para no eternizar la cosa.
    my ($request) = new HTTP::Request('GET', $url);
    my ($response) = $ua->request($request);
    if ($response->is_success) {
        $buffer = $response->content;
    } else {
        print STDERR $response->content;
        return;
    }

    my ($titular,$descripcion,$fechap,$texto,$robots) = ('','','','','');
    # Detecta metas "robots".
    # <meta name="robots" content="nofollow">
    while ($buffer =~ /<meta\s+name=\"robots\"\s+content=\"([^\"]+)\">/isg) {
        $robots .= " $1";
    }

    my ($ano,$mes,$dia); # Variables de respuesta.
    if (!utf8::is_utf8($buffer)) {
        utf8::decode($buffer);
    }
    # Si no hay que indexar, no hace nada y retorna
    if ($buffer =~ /<META\s+NAME=\"ROBOTS\"\s+CONTENT=\"NOINDEX\">/is) {
        return;
    }

    my $no_title=lib_lang::get_message('indexer_no_title');
    # Obtiene el titular.
    if ($buffer =~ /<title>(.+?)<\/title>/is) {
        $titular = $1;
    } else {
        $titular = $no_title;
    }
    # Elimina tags, \n y | dentro del titular.
    $titular =~ s/<[^>]+>/ /isg;
    $titular =~ s/[\|\s]+/ /sg;

    if ($titular =~ /^\s*$/) { $titular = $no_title; }
    if ($titular eq $no_title) {
        while($buffer =~ /(<meta[^>]+?>)/sig) {
            my $meta = $1;
            if ($meta =~ /property=["']og:title["']/ && $meta =~ /content="([^"]+)"/) {
                $titular = $1;
                last;
            }
            if ($meta =~ /name=["']twitter:title["']/ && $meta =~ /content="([^"]+)"/) {
                $titular = $1;
                last;
            }
        }
    }
    $titular = lib_encodes::no_accent_mark($titular);
    # Elimina entidades html.
    $titular =~ s/&[^;]+;//g;

    while($buffer =~ /(<meta[^>]+?>)/sig) {
        my $meta = $1;
        if ($meta =~ /property=["']og:description["']/ && $meta =~ /content="([^"]+)"/) {
            $texto = $1;
            last;
        }
        if ($meta =~ /name=["']twitter:description["']/ && $meta =~ /content="([^"]+)"/) {
            $texto = $1;
            last;
        }
    }

    # Reemplaza entidades de tildes por letras normales.
    $texto = lib_encodes::no_accent_mark($texto);
    # Elimina entidades html.
    $texto =~ s/&[^;]+;//g;

    # Toma primeras letras del texto para formar la descripcion.
    $descripcion = substr($texto,0,$CFG{'RESUMEN'});

    # Trunca a una palabra entera antes.
    $descripcion =~ s/[\&\w;#\x{c3a1}\x{c3a9}\x{c3ad}\x{c3b3}\x{c3ba}\x{c381}\x{c389}\x{c38d}\x{c393}\x{c39a}\x{c3b1}\x{c391}\x{c3bc}\x{c39c}]+$//s;
    $descripcion =~ s/[\|\n]/ /sg; # Elimina | y \n.
    # Concatena el titular al texto para construir el indice.
    $texto = "$titular $texto";

    # Pasa todo el texto a minusculas
    $texto = lc $texto;

    # Reemplaza cualquier cosa no caracter por un espacio.
    $texto =~ s/[^0-9a-z]+/ /sg;

    # Limita el texto a los primeros n caracteres.
    $texto = substr($texto,0,$CFG{'MAXCARS'});

    # print "[$titular]\n[$descripcion]\n[$fechap]\n[$texto]\n[$robots]\n";
    if ($titular eq '') {  # Si fracaso, no hace nada mas.
        print "*** FAIL *** $url\n"; # debug
        return;
    }

    # print "$FIDX $url\n"; # debug

    if ($INVOCACION eq 'web') {
        my $depth_msg= lib_lang::get_interpolated_message('indexer_spider_depth_url', $FIDX, $PROFUNDIDAD, $url);
        &printJSON($depth_msg, 0);
        lib_logproc::add_to_log($depth_msg);
    }

    # Indexa el archivo si no hay instrucciones en contrario.
    &indexa_url($titular,$descripcion,$fechap,$texto,$robots,$url);

    return @lnk;
}; # parseURL
# ------------------------------------------------------------------------#
# Rutina que indexa un URL y reporta todos los links que encuentra en el.
# El 'scope' es el limite superior de los links para ser recorridos.
# No seran recorridos links que apunten mas arriba, en el sentido de
# organizacion de directorios, que el scope pasado como parametro.
# Nota: no resulta llamar recursivamente a esta rutina, pq el spider se va solo
# por una rama del sitio, cuando lo que conviene es que vaya por capas.
sub simpleSpider {
    my ($url,$scope) = @_;
    my ($protocolo,$host,$path,$file,$params,$lnk,$i,$l,$p);
    my (@lnk) = ();

    # print "Spider: $url\n"; # debug
    return if ($LINKS{$url} ne '');
    $LINKS{$url} = 1;

    # Aborta si se ha llegado el limite de archivos.
    # 2019/11/19 No se limita la cantidad de urls a indexar
    # return if ($FIDX > $CFG{'URL_MAXPAGS'});
    my ($titular,$descripcion,$fechap,$texto,$robots,$buffer) = &getHTML($url);
    if ($titular eq '') {  # Si fracaso, no hace nada mas.
        # print "*** FAIL *** $url\n"; # debug
        return;
    }

    # print "$FIDX $url\n"; # debug
    if ($url =~ /(https?):\/\/([a-z0-9\.\-]+)\/*(\?|$)/i) {
        ($host,$path,$file,$protocolo) = ($2,'','',$1);
    } elsif ($url =~ /(https?):\/\/([a-z0-9\.\-]+)(\/[a-z0-9\.\-\_\%\@\~\,]+)$/i) {
        ($host,$path,$file,$protocolo) = ($2,'',$3,$1);
    } elsif ($url =~ /(https?):\/\/([a-z0-9\.\-]+)(\/[a-z0-9\.\-\_\%\@\~\,\/]+)(\/[a-z0-9\.\-\_\%\@\~\,]*)($|\?)/i) {
        ($host,$path,$file,$protocolo) = ($2,$3,$4,$1);
    }

    # print "$FIDX url=[$url] host=[$host] path=[$path] file=[$file]\n"; # debug
    if ($INVOCACION eq 'web') {
        &printJSON("$FIDX prof=[$PROFUNDIDAD] url=[$url]", 0);
        lib_logproc::add_to_log("$FIDX prof=[$PROFUNDIDAD] url=[$url]");
    }

    # Indexa el archivo si no hay instrucciones en contrario.
    &indexa_url($titular,$descripcion,$fechap,$texto,$robots,$url);

    # Extrae y sigue links si es que no hay instrucciones en contrario.
    return if ($robots =~ /nofollow/i);

    # Extrae y recorre los links.
    # Se define como link cualquier cosa que esta entre comillas
    # y coincide con $URL_FILETYPES_REGEXP
    while ($buffer =~ /([\"\'])([^\"\'\s]+)\1/ig) {
        $lnk = $2;
        if (   ($lnk =~ /($URL_FILETYPES_REGEXP)($|\?)/i)
            && ($lnk =~ /[a-z0-9\/]$/i)
            && ($lnk =~ /[a-z0-9]/i) ) {

            # print "found link: $lnk"; # debug
            # Arregla links derivados de un refresh.
            $lnk =~ s/\d+;url=(.+)/$1/i;

            # Elimina referencias al directorio actual.
            $lnk =~ s/^\.\/(.+)/$1/i;

            # Si el link "califica", entonces lo recorre tambien.
            if ($lnk =~ /^\//i) {  # Link absoluto.
                $lnk = "$protocolo://$host$lnk";

                # print "absoluto: $lnk\n"; # debug
            } elsif ($lnk =~ s/^((\.\.\/)+)//) { # Link relativo.
                $l = length($1) / 3;
                $p = $path;
                for ($i=0;$i<$l;$i++) {
                    $p =~ s/\/[^\/]+$//i; # Elimina tantos directorios como ../ hay en el link.
                }
                $lnk = "$protocolo:\/\/$host$p/$lnk";

                # print "relativo: $lnk\n"; # debug
            } elsif ($lnk !~ /^http/) { # Link relativo a partir del mismo directorio.
                $lnk = "$protocolo://$host$path/$lnk";

                # print "simple: $lnk\n"; # debug
            }
            # Si no se cumplio nada de lo anterior, se asume que el link viene con host.
            # Califica si el link esta en un nivel inferior al original.
            if ($lnk =~ /^$scope/) {
                if ($LINKS{$lnk} eq '') {
                    $lnk =~ s/(\w)\/\//$1/g; # Elimina doble / para evitar doble indexacion.
                    # &simpleSpider($lnk,$scope);
                    if ($LINKS{$lnk} eq '') {

                        # Solo recuerda links no indexados.
                        push @lnk,$lnk;
                    }
                }
            }
        }
    }

    return @lnk;
}

# ------------------------------------------------------------------------#
# Genera el indice cruzado de palabras.
sub genera_wordsindex {
    my ($dir) = $_[0];
    my ($archivo,$palabras,$locations,$numlocations);
    my (%wordsidx) = ();
    my ($numarchivos) = 0;

    # Lee el indice de archivos -> palabras y llena el hash de palabras -> archivos.
    open(my $FILESINDEXH, '<', "$dir/$FILESINDEX_FILENAME.tmp") || die "No pude abrir el archivo $dir/$FILESINDEX_FILENAME.tmp $!";
    while (<$FILESINDEXH>) {

        # 12=969 1|3175 1|2595 1|9 1|5708 1|607 1|
        chomp;
        ($archivo,$palabras) = split(/=/,$_);
        while ($palabras =~ /\|(\d+) (\d+)/g) {

            # La palabra esta en este archivo, con frecuencia $2.
            $wordsidx{$1} .= "$archivo $2|";
        }
        $numarchivos++;

        # if ( ($numarchivos % 100) == 0 ) { print $numarchivos . ' '; }; # debug
    }
    close $FILESINDEXH;

    # Crea limite de palabras no significativas, las cuales seran descartadas.
    $numarchivos = $numarchivos * $CFG{'RATIO'} / 100;

    # Este sera el archivo de ubicacion de palabras (palabras=|archivo veces|...).
    open(my $WORDSINDEXH, '>', "$dir/$WORDSINDEX_FILENAME.tmp") || die "could not open file $dir/$WORDSINDEX_FILENAME.tmp $!";
    binmode $WORDSINDEXH, ':encoding(UTF-8)';
    foreach my $widx (sort {$a <=> $b} keys %wordsidx) {

        # Descarta palabras muy repetidas.
        $locations = $wordsidx{$widx};
        $numlocations = ($locations =~ s/\|//g);
        if ($numlocations > $numarchivos) { #
            $wordsidx{$widx} = '';

            # print "Descartada la palabra $widx\n"; # debug
        }

        # Escribe el crossindex palabras -> archivos.
        print $WORDSINDEXH $widx . '=|' . $wordsidx{$widx} . "\n";
    }
    close $WORDSINDEXH;
    %wordsidx = ();
}

# ------------------------------------------------------------------------
# Escribe los indices de palabras.
sub escribe_palabras {
    my ($dir) = $_[0];
    my $letraold;
    my (%letras) = ();

    my $i = 0;
    $letraold = '';

    # Escribe indice de palabras (ordenado alfabeticamente).
    open my $WORDS, '>', "$dir/$WORDS_FILENAME.tmp";
    binmode $WORDS, ':encoding(UTF-8)';
    foreach my $palabra (sort keys %WORDS) {
        $i++;
        print $WORDS $palabra . '=' . $WORDS{$palabra} . "\n";
        my $letra = substr($palabra,0,1);
        if ($letra ne $letraold) {
            $letras{$letra} = $i;
            $letraold = $letra;
        }
    }
    close $WORDS;

    # Escribe indice de letras.
    open my $CARS, '>', "$dir/$CARS_FILENAME.tmp";
    binmode $CARS, ':encoding(UTF-8)';
    foreach my $letra (sort keys %letras) {
        print $CARS $letra . '=' . $letras{$letra} . "\n";
    }
    close $CARS;
}

# ------------------------------------------------------------------------
# Recorre el directorio de articulos buscando archivos que indexar.
sub busca_articulos {
    my ($eldir) = $_[0];
    my ($fullpath,$ts,$extension);
    my ($ano2index,$currentyear,$antesdeayer,$cc);

    # Archivo de articulos.
    open $FILES, '>', "$SEARCH_DIR$PRONTUS/$ANO/$FILES_FILENAME.tmp";
    binmode $FILES, ':encoding(UTF-8)';
    # Archivo indice de articulos.
    open $FILESINDEX, '>', "$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEX_FILENAME.tmp";
    binmode $FILESINDEX, ':encoding(UTF-8)';
    # Archivo indice de articulos para busqueda por frases.
    open $FILESINDEXF, '>', "$SEARCH_DIR$PRONTUS/$ANO/$FILESINDEXF_FILENAME.tmp";
    binmode $FILESINDEXF, ':encoding(UTF-8)';

    # Ordena al reves para busqueda cronologica (los mas antiguos quedan al final del archivo de articulos).
    my @days = sort {$b cmp $a} (glib_fildir_03::lee_dir($eldir));

    # Obtiene el ano en curso.
    $currentyear = (localtime(time))[5] + 1900;

    # Obtiene el dia de antes de ayer.
    $antesdeayer = ((localtime(time))[5] + 1900). sprintf('%02d',((localtime(time - 172800))[4]+1)). sprintf('%02d',(localtime(time - 172800))[3]);

    # Si $ANO es 7777 significa que se deben indexar solo los ultimos 2 dias del ano en curso.
    if ($ANO eq $FORZAR_ULTIMOS_DIAS) {
        $ano2index = $currentyear;
    } else {
        $ano2index = $ANO;
    }

    my $prontusid = $PRONTUS;
    $prontusid=~ s/^\///;

    foreach my $day (@days) {
        next if ( $day !~ /^\d{8}$/);       # 20050612
        next if ( $day !~ /^$ano2index/ && $ANO ne $FORZAR_TODO); # - 8888 indica que se debe forzar el indexado de todo el contenido.

        if ($ANO eq $FORZAR_ULTIMOS_DIAS) {

            # Si $ANO es 7777 se salta dias anteriores a antes de ayer.
            next if ($day <= $antesdeayer);
        } else {

            # Si $ANO no es 7777 se salta dias posteriores a antes de ayer.
            # next if ($day > $antesdeayer);

            # - Si $ANO no es 7777 se salta dias posteriores a antes de ayer. 8888 indica que se debe forzar el reindexado de todo el contenido.
            next if ($day > $antesdeayer && $ANO ne $FORZAR_TODO);
        }

        # print "day = $day ano = $ANO\n"; # debug
        $fullpath = "$eldir/$day/xml"; # Elimina compatibilidad < 10.
        if (-d "$fullpath") {

            # print "fullpath = $fullpath\n"; # debug
            my @files = glib_fildir_03::lee_dir($fullpath);
            foreach my $file (@files) {
                if ((-f "$fullpath/$file") && ($file =~ /^(\d{14})\.(\w+)$/)) { # Archivos con TS.
                    $ts = $1;

                    # Jamas se indexaran xml por error
                    $extension = '';

                    # Determina extension del archivo visible por web.
                    my @webpages = glib_fildir_03::lee_dir("$eldir/$day/pags");
                    foreach my $webpage (@webpages) {
                        if ($webpage =~ /^$ts\.(\w+)$/) {
                            $extension = $1;
                            last;
                        }
                    }

                    # Indexa el archivo. Solo indexa si el archivo existe
                    if ($extension) {
                        &indexa("$fullpath/$file",$ts,$extension,"$eldir/$day", $prontusid);
                    }
                }
            }
        }

        # Cada 10 dias descansa 1 segundo.
        $cc++;
        if ($cc >= 10) {
            sleep(1);
            $cc = 0;
        }
    }
    close $FILES;
    close $FILESINDEX;
    close $FILESINDEXF;
}

# ------------------------------------------------------------------------
# Recorre el directorio de archivos en bruto buscando archivos que indexar.
sub busca_articulos_raw {
    my ($eldir) = $_[0];
    my (@days,$day,$fullpath,$filepath,@webpages,$webpage);

    my @files = glib_fildir_03::lee_dir($eldir); # Lee el directorio.

    foreach my $file (sort {$b cmp $a} @files) {
        next if ( $file =~ /^\./); # Descarta archivos invisibles.
        # print "file = $eldir/$file\n"; # debug
        $fullpath = "$eldir/$file";
        if (-d $fullpath) {

            # Si es un directorio, sige la busqueda de archivos en forma recursiva.
            &busca_articulos_raw($fullpath);
        } elsif (-f $fullpath) {

            # Si es un archivo y la extension coincide con las buscadas, lo indexa.
            next if (!(grep {$file =~ /\.$_$/} @RAW_FILETYPES) );

            # print "fullpath = $fullpath\n"; # debug
            if ($fullpath =~ /^$ROOTDIR(\/.+)$/) { # Extrae la parte bajo el rootdir.
                $filepath = $1;

                # Indexa el archivo.
                # print "indexando $filepath...\n"; # debug
                &indexa_raw($fullpath,$filepath);
            }
        }
    }
}

# ------------------------------------------------------------------------
# Busca palabras de data proveniente de un URL y las mete en los indices correspondientes.
sub indexa_url {
    my ($titular,$descripcion,$fechap,$texto,$robots,$url) = @_;
    my (@palabras,%ocurrencias,$indice);

    # print "$titular,$descripcion,$texto\n"; # debug
    # No hace nada si el texto no es significativo.
    return if (length($texto) < $CFG{'MINTEXT'});

    # No hace nada si no debe hacer nada.
    return if ($robots =~ /noindex/i);

    # Registra archivo.
    $FIDX++;

    # IDarticulo=fechap|path al archivo|titular|primeras palabras|seccion|tema|subtema|slug_custom|slug|urlf5
    print $FILES "$FIDX=$fechap|$url|$titular||||$descripcion||||NO||\n";

    # Elimina todo lo que no sean caracteres validos.
    $texto =~ s/[^0-9a-z]/ /isg;

    # Separa en palabras.
    @palabras = split(/\s+/,$texto);

    # Registra palabras.
    foreach my $palabra (@palabras) {

        # $palabra =~ s/s$//; # Elimina 's' final a palabras para asimilar plurales.
        next if (length($palabra) < 2); # Elimina letras sueltas.
        if ( !exists $WORDS{$palabra}) {
            $WIDX++;
            $WORDS{$palabra} = $WIDX;
        }
        $ocurrencias{$palabra}++;
    }

    # Construye indice de palabras dentro de archivos.
    $indice = '';
    foreach my $palabra (keys %ocurrencias) {

        # Registra el indice de la palabra y el numero de ocurrencias dentro del archivo.
        $indice .= $WORDS{$palabra} . ' ' . $ocurrencias{$palabra} . '|';
    }
    print $FILESINDEX "$FIDX=|$indice\n";
}

# ------------------------------------------------------------------------
# Busca palabras dentro del archivo y las mete en los indices correspondientes.
sub indexa_raw {
    my ($fullpath,$filepath) = @_;
    my (@palabras,%ocurrencias,%ubicaciones,$indice,$indicef,$ipalabra);
    my ($titular,$descripcion,$fechap,$texto) = &get_contents_raw(glib_fildir_03::read_file($fullpath),$filepath);

    # print "$titular,$descripcion,$texto\n"; # debug
    # No hace nada si el texto no es significativo.
    return if (length($texto) < $CFG{'MINTEXT'});

    # Registra archivo.
    $FIDX++;

    # IDarticulo=fechap|path al archivo|titular|primeras palabras|seccion|tema|subtema|slug_custom|slug|urlf5
    print $FILES "$FIDX=$fechap|$filepath|$titular||||$descripcion||||NO||\n";
    # Elimina todo lo que no sean caracteres validos.
    $texto =~ s/[^0-9a-z]/ /isg;

    # Separa en palabras.
    @palabras = split(/\s+/,$texto);
    # Registra palabras.
    $ipalabra = 0; # Indice de la palabra. Usado para la busqueda por frases.
    foreach my $palabra (@palabras) {

        # $palabra =~ s/s$//; # Elimina 's' final a palabras para asimilar plurales.
        next if (length($palabra) < 2); # Elimina letras sueltas.
        if ( !exists $WORDS{$palabra}) {
            $WIDX++;
            $WORDS{$palabra} = $WIDX;
        }
        $ocurrencias{$palabra}++;
        $ubicaciones{$palabra} += " $ipalabra";
        $ipalabra++;
    }

    # Construye indice de palabras dentro de archivos.
    $indice = '';
    $indicef = '';
    foreach my $palabra (keys %ocurrencias) {

        # Registra el indice de la palabra y el numero de ocurrencias dentro del archivo.
        $indice .= $WORDS{$palabra} . ' ' . $ocurrencias{$palabra} . '|';
        $indicef .= $WORDS{$palabra} . $ubicaciones{$palabra} . '|';
    }
    print $FILESINDEX "$FIDX=|$indice\n";
    print $FILESINDEXF "$FIDX=|$indicef\n";
}

# ------------------------------------------------------------------------
# Busca palabras dentro del archivo y las mete en los indices correspondientes.
sub indexa {
    my ($archivo,$ts,$extension,$daypath, $prontus) = @_; # $daypath = "$eldir/$day" (site/artic/20070402)
    my (@palabras,%ocurrencias,%ubicaciones,$indice,$indicef,$filepath,$dir);
    my (@data2index,$ipalabra);

    # Busca archivos de data de este articulo que haya que indexar.
    # site/artic/20070402/asocfile/20070402165132
    # $archivo,$ts,$extension,$daypath
    my @files = glib_fildir_03::lee_dir("$daypath/asocfile/$ts");
    foreach my $file (@files) {
        foreach my $ext (keys %FILEFILTER) {
            if ($file =~ /\.$ext$/i) {
                push @data2index,"$daypath/asocfile/$ts/$file";
            }
        }
    }

    my $buffer = glib_fildir_03::read_file($archivo);
    my ($titular,$meta1,$meta2,$meta3,$descripcion,$fechap,$texto,$seccion,$tema,$subtema,$custom_slug,$slug,$urlf5,@data) = &get_contents($buffer, $prontus, $ts);

    # print "$archivo = $titular,$descripcion,$fechap,$texto,$seccion,$tema,$subtema\n"; # debug
    while ($titular ne '') {
        # No hace nada si el texto no es significativo.
        if (length($texto) >= $CFG{'MINTEXT'}) {

            # Registra archivo.
            $FIDX++;

            # print DEB "archivo=[$archivo][$FIDX]["; # debug
            # Determina el path relativo al articulo.
            if ( $archivo =~ /\.xml$/ ) {
                $dir = substr($ts,0,8);
                $filepath = "$PRONTUS/site/artic/$dir/pags/$ts\.$extension";
            } else {
                # Extrae la parte bajo el rootdir.
                if ($archivo =~ /^$ROOTDIR(.+)$/) {
                    $filepath = $1;
                }
            }

            my $strtofile = "$FIDX=$fechap|$filepath|$titular|$meta1|$meta2|$meta3|$descripcion|$seccion|$tema|$subtema|$custom_slug|$slug|$urlf5";
            for (my $k=0; $k<20;$k++) {
                $strtofile = $strtofile."|$data[$k]";
            }
            $strtofile = $strtofile."\n";
            print $FILES $strtofile;

            # Elimina todo lo que no sean caracteres validos.
            $texto =~ s/[^0-9a-z]/ /isg;

            # Separa en palabras.
            @palabras = split(/\s+/,$texto);
            # Registra palabras.
            $ipalabra = 0; # Indice de la palabra. Usado para la busqueda por frases.
            foreach my $palabra (@palabras) {

                next if (length($palabra) < 2); # Elimina letras sueltas.
                if ( !exists $WORDS{$palabra}) {
                    $WIDX++;
                    $WORDS{$palabra} = $WIDX;
                }
                $ocurrencias{$palabra}++;
                $ubicaciones{$palabra} .= " $ipalabra";
                $ipalabra++;
            }

            # print DEB "]\n"; # debug
            # Construye indice de palabras dentro de archivos.
            $indice = '';
            $indicef = '';
            foreach my $palabra (keys %ocurrencias) {

                # Registra el indice de la palabra y el numero de ocurrencias dentro del archivo.
                $indice .= $WORDS{$palabra} . ' ' . $ocurrencias{$palabra} . '|';
                $indicef .= $WORDS{$palabra} . $ubicaciones{$palabra} . '|';
            }
            print $FILESINDEX "$FIDX=|$indice\n";
            print $FILESINDEXF "$FIDX=|$indicef\n";
        }

        $titular = '';
        $archivo = pop @data2index;
        if ($archivo =~ /\.([^\.]+)$/) {
            my $ext = $1;
            if ($archivo ne '') {
                $buffer = qx/$FILEFILTER{$ext} $archivo/;
                ($titular,$descripcion,$texto) = &get_contents_data($buffer);

                # Sobreescribe el titular como el nombre del archivo.
                if ($archivo =~ /\/([^\/]+)$/) {
                    $titular = $1;
                    $titular =~ s/_/ /g;
                    $titular = ucfirst $titular;
                }
            } else {
                last;
            }
        }
    }
}

# ---------------------------------------------------------------
# Obtiene el contenido de interes de un archivo de datos.
sub get_contents_data {
    my ($buffer) = $_[0];
    my ($titular,$descripcion,$texto); # Variables de respuesta.
    if (!utf8::is_utf8($buffer)) {
        utf8::decode($buffer);
    }
    # Considera como titular las palabras dentro de los primeros 40 caracteres del texto.
    $titular = substr($buffer,0,40);
    $titular =~ s/[^\s]+$//s;
    $titular .= '...';

    # Elimina tags, \n y | dentro del titular.
    $titular =~ s/[\|\n\r\<\>]+/ /sg;

    # Obtiene texto significativo.
    $texto = $buffer;

    # Elimina tags que no contienen links ni informacion relevante.
    $texto =~ s/<style.+?<\/style>//sig;   # (la 's')
    $texto =~ s/<script.+?<\/script>//sig; # (la 's')
    $texto =~ s/<select.+?<\/select>//sig; # (la 's')
    # Elimina los links, ya que apuntan a otra parte.
    $texto =~ s/<a .+?<\/a>//ig;

    # Elimina los espacios excesivos, retornos de carro tabuladores y "|"
    $texto =~ s/[\|\s]+/ /g;

    # Toma primeras letras del texto para formar la descripcion.
    $descripcion = substr($texto,0,$CFG{'RESUMEN'});

    # Trunca a una palabra entera antes.
    $descripcion =~ s/[^\s]+$//s;
    $descripcion =~ s/[\|\n\r\<\>]+/ /sg; # Elimina | y \n.
    # Reemplaza tildes por letras normales.
    $texto = lib_encodes::no_accent_mark($texto);
    # Elimina entidades html.
    $texto =~ s/&[^;]+;//g;
    # Pasa todo el texto a minusculas. # Esto antes estaba antes de notildesUtf8.
    $texto = lc $texto;

    # Reemplaza cualquier cosa no caracter por un espacio.
    $texto =~ s/[^0-9a-z]+/ /sg;

    # Limita el texto a los primeros n caracteres.
    $texto = substr($texto,0,$CFG{'MAXCARS'});
    return ($titular,$descripcion,$texto);
}

# ---------------------------------------------------------------
# Obtiene el contenido de interes de un archivo bruto (raw).
sub get_contents_raw {
    my ($buffer,$filepath) = @_;
    my ($titular,$descripcion,$fechap,$texto,$ano,$mes,$dia); # Variables de respuesta.
    if (!utf8::is_utf8($buffer)) {
        utf8::decode($buffer);
    }
    # Si no hay que indexar, no hace nada y retorna todo vacio.
    if ($buffer =~ /<META\s+NAME=\"ROBOTS\"\s+CONTENT=\"NOINDEX\">/is) { return ('','',''); }

    # Obtiene el titular.
    my $no_title = lib_lang::get_message('indexer_no_title');
    if ($buffer =~ /<!--TITULAR-->(.+?)<!--\/TITULAR-->/is) {
        $titular = $1;
    } elsif ($buffer =~ /<h1>(.+?)<\/h1>/is) {
        $titular = $1;
    } elsif ($buffer =~ /<h2>(.+?)<\/h2>/is) {
        $titular = $1;
    } elsif ($buffer =~ /<h3>(.+?)<\/h3>/is) {
        $titular = $1;
    } elsif ($buffer =~ /<title>(.+?)<\/title>/is) {
        $titular = $1;
    } else {
        $titular = $no_title;
    }
    if ($titular =~ /^\s*$/) { $titular = $no_title; }
    # Intenta descubrir una fechap.
    # Formato PX <_FECHAP>20050109</_FECHAP>
    if ($buffer =~ /<_FECHAP>(\d+)<\/_FECHAP>/s) {
        $fechap = $1;
    } elsif ($buffer =~ /<!--FECHAP=(\d+)-->/s) { # <!--FECHAP=20050622-->
        $fechap = $1;
    } elsif ($filepath =~ /(\d{8})/s) {
        $fechap = $1;
    } elsif ($filepath =~ /(\d{8})\d{6}/s) {
        $fechap = $1;
    }

    # Validacion de fechap
    $ano = substr($fechap,0,4);
    $mes = substr($fechap,4,2);
    $dia = substr($fechap,6,2);
    if ( ($ano < 1900) || ($ano > 2010) || ($mes < 1) || ($mes > 12) || ($dia <1) || ($dia > 31) ) {
        $fechap = '';
    }

    # print "fechap=$fechap\n"; # debug
    # Elimina tags, \n y | dentro del titular.
    $titular =~ s/<[^>]+>/ /sg;
    $titular =~ s/[\|\s]+/ /sg;

    # Obtiene texto significativo.
    if ($buffer =~ /<body.+?<\/body>/is) { # Bodys
        $texto = $&;
    } elsif ($buffer =~ /<body.+/is) { # Bodys largos o mal formados.
        $texto = $&;
    }

    # Elimina tags que no contienen links ni informacion relevante.
    $texto =~ s/<style.+?<\/style>//sig;   # (la 's')
    $texto =~ s/<script.+?<\/script>//sig; # (la 's')
    $texto =~ s/<select.+?<\/select>//sig; # (la 's')
    # Elimina los links, ya que apuntan a otra parte.
    $texto =~ s/<a .+?<\/a>//ig;

    # elimina comentarios html
    $texto =~ s/<!--.+?-->//sg;

    # Elimina tags del texto;
    $texto =~ s/<[^>]+>/ /sg;

    # Elimina los espacios excesivos, retornos de carro, tabuladores y "|"
    $texto =~ s/&nbsp;/ /g;
    $texto =~ s/[\|\s]+/ /g;

    # Toma primeras letras del texto para formar la descripcion.
    $descripcion = substr($texto,0,$CFG{'RESUMEN'});

    # Trunca a una palabra entera antes.
    $descripcion =~ s/[\&\w;#\x{c3a1}\x{c3a9}\x{c3ad}\x{c3b3}\x{c3ba}\x{c381}\x{c389}\x{c38d}\x{c393}\x{c39a}\x{c3b1}\x{c391}\x{c3bc}\x{c39c}]+$//s;
    $descripcion =~ s/[\|\n]/ /sg; # Elimina | y \n.
    # Concatena el titular al texto para construir el indice.
    $texto = "$titular $texto";

    # Reemplaza tildes por letras normales.
    $texto = lib_encodes::no_accent_mark($texto);
    # Elimina entidades html.
    $texto =~ s/&[^;]+;//g;
    # Pasa todo el texto a minusculas. # Esto antes estaba antes de notildesUtf8.
    $texto = lc $texto;

    # Reemplaza cualquier cosa no caracter por un espacio.
    $texto =~ s/[^0-9a-z]+/ /sg;

    # Limita el texto a los primeros n caracteres.
    $texto = substr($texto,0,$CFG{'MAXCARS'});
    return ($titular,$descripcion,$fechap,$texto);
}

# ---------------------------------------------------------------
# Obtiene el contenido de interes de la pagina recien leida.
sub get_contents {
    my ($buffer) = shift;
    my ($prontus) = shift;
    my ($ts) = shift;
    # Variables de respuesta.
    my ($titular,$meta1,$meta2,$meta3,$descripcion,$fechap,$texto,$seccion,$tema,$subtema,$custom_slug,$slug,$urlf5);
    my (@metadata, @metadata_var);
    my ($tipoficha,$k);
    my $horap = '0000';
    my $fechae = '99999999';
    my $horae = '0000';
    my $alta = 1; # por defecto asume que el articulo esta de alta.
    my $title_var = $CFG{'TITLEVAR'};
    my $meta1_var = $CFG{'META1'};
    my $meta2_var = $CFG{'META2'};
    my $meta3_var = $CFG{'META3'};

    for($k=0;$k<20;$k++) {
        $metadata_var[$k]=$CFG{'METADATA'.($k+1)};
    }
    my (@textvars) = split(/\s+/,$CFG{'TEXTVARS'});
    my (@tiposficha) = split(/\s+/,$CFG{'FIDS'});
    # Obtiene el titular, fechap y texto significativo.
    # <_alta>1</_alta>
    if ($buffer =~ /<_alta>(.*?)<\/_alta>/s) {
        $alta = $1;
    }

    # print "alta=$alta "; # debug
    return if ($alta != 1); # Si no esta en alta, retorna todo vacio.
    # <_tipo_ficha>fid_rotulo.html</_tipo_ficha>
    if ($buffer =~ /<_fid>(.+?)<\/_fid>/s) {
        $tipoficha = $1;
    }

    # print "tipoficha=$tipoficha "; # debug
    return if ( !(grep {$_ eq $tipoficha} @tiposficha)); # Si el tipo de ficha no corresponde, retorna.
    # <_fechap>20050109</_fechap>
    if ($buffer =~ /<_fechap>(\d+)<\/_fechap>/s) {
        $fechap = $1;
    }

    # si se encuentra indexar 0, no se indexa el articulo
    return if ($buffer =~ /<_indexar>0<\/_indexar>/s);

    # print "fechap=$fechap $FECHAACTUAL "; # debug
    # Obtiene la horap, fechae y horae y compara.
    if ($buffer =~ /<_horap>(.*?)<\/_horap>/s) {
        $horap = $1;
        $horap =~ s/[^0-9]//g; # Elimina separador de elementos.
    }

    # Solo considera fechae y horae si NO esta <_soloportadas>1</_soloportadas>.
    if ($buffer !~ /<_soloportadas>1<\/_soloportadas>/s) {
        if ($buffer =~ /<_fechae>(\d+)<\/_fechae>/s) {
            $fechae = $1;
        }
        if ($buffer =~ /<_horae>(.*?)<\/_horae>/s) {
            $horae = $1;
            $horae =~ s/[^0-9]//g; # Elimina separador de elementos.
        }
    }

    # Retorna vacio si la fecha del articulo es posterior a hoy.
    # No es necesario recortar $FECHAHORAACTUAL a los minutos, pq la comparacion gt es caracter por caracter.
    return if ( ($fechap.$horap) gt $FECHAHORAACTUAL );
    return if ( ($fechae.$horae) lt $FECHAHORAACTUAL );

    # <_TXT_TITULAR>
    # <![CDATA[xxx]]>
    # </_TXT_TITULAR>
    my $no_title = lib_lang::get_message('indexer_no_title');
    if ($buffer =~ /<$title_var>\s*<!\[CDATA\[(.+?)\]\]>\s*<\/$title_var>/s) {
        $titular = $1;
    } elsif ($buffer =~ /<$title_var>\s*(.+?)\s*<\/$title_var>/s) { # Busca titular sin CDATA.
        $titular = $1;
    } else {
        $titular = $no_title;
    }

    if ($titular ne $no_title) {
        # Elimina tags, \n y | del titular.
        $titular =~ s/<[^>]+>/ /sg;
        $titular =~ s/[\|\s]+/ /sg;
        $titular =~ s/&nbsp;/ /g;
    }

    # META1
    if ($buffer =~ /<$meta1_var>\s*<!\[CDATA\[(.+?)\]\]>\s*<\/$meta1_var>/is) {
        $meta1 = $1;
    } elsif ($buffer =~ /<$meta1_var>(.+?)<\/$meta1_var>/is) {
        $meta1 = $1;
    }

    # META2
    if ($buffer =~ /<$meta2_var>\s*<!\[CDATA\[(.+?)\]\]>\s*<\/$meta2_var>/is) {
        $meta2 = $1;
    } elsif ($buffer =~ /<$meta2_var>(.+?)<\/$meta2_var>/is) {
        $meta2 = $1;
    }

    # META3
    if ($buffer =~ /<$meta3_var>\s*<!\[CDATA\[(.+?)\]\]>\s*<\/$meta3_var>/is) {
        $meta3 = $1;
    } elsif ($buffer =~ /<$meta3_var>(.+?)<\/$meta3_var>/is) {
        $meta3 = $1;
    }

    # Quita retornos de carro y "|"
    $meta1 =~ s/[\|\n\r]+/ /sg;
    $meta2 =~ s/[\|\n\r]+/ /sg;
    $meta3 =~ s/[\|\n\r]+/ /sg;

    # METADATAk
    for($k=0;$k<20;$k++) {
        my $mdata = $metadata_var[$k];
        next unless($mdata);
        if ($mdata eq '_prontus_id') {
            $metadata[$k] = $prontus;
            next;
        } elsif ($mdata eq '_ts') {
            $metadata[$k] = $ts;
            next;
        }
        if ($buffer =~ /<$mdata>\s*<!\[CDATA\[(.*?)\]\]>\s*<\/$mdata>/s) {
            $metadata[$k] = $1;
        } elsif ($buffer =~ /<$mdata>.*?<!\[CDATA\[(.*?)\]\]>(.*?)<\/$mdata>/s) {
            $metadata[$k] = $1;
        } elsif ($buffer =~ /<$mdata>(.+?)<\/$mdata>/s) {
            $metadata[$k] = $1;
        } else {
            $metadata[$k] = '';
        }

        # Quita retornos de carro y "|"
        $metadata[$k] =~ s/[\|\n\r]+/ /sg;
    }

    foreach my $var (@textvars) {
        if ($buffer =~ /<$var>\s*<!\[CDATA\[(.+?)\]\]>\s*<\/$var>/s) {
            $texto .= " $1";
        } elsif ($buffer =~ /<$var>\s*(.+?)\s*<\/$var>/s) {
            $texto .= " $1";
        }
    }

    # <_nom_seccion1>...</_nom_seccion1>
    if ($buffer =~ /<_nom_seccion1>([^<]+)<\/_nom_seccion1>/s) {
        $seccion = $1;
    }

    # <_nom_tema1>...</_nom_tema1>
    if ($buffer =~ /<_nom_tema1>([^<]+)<\/_nom_tema1>/s) {
        $tema = $1;
    }

    # <_nom_subtema1>...</_nom_subtema1>
    if ($buffer =~ /<_nom_subtema1>([^<]+)<\/_nom_subtema1>/s) {
        $subtema = $1;
    }

    $custom_slug = 'NO';
    $slug = '';

    $urlf5 = '';
    if ($buffer =~ m|<_url>([a-z0-9_\-/]+)</_url>|) {
        $urlf5 = $1;
    }

    # Elimina tags del texto;
    $texto =~ s/<[^>]+>/ /sg;

    # Elimina los espacios excesivos, los retornos de carro, los tabuladores y "|"
    $texto =~ s/&nbsp;/ /g;
    $texto =~ s/[\|\s]+/ /g;

    # Toma primeras letras del texto para formar la descripcion.
    $descripcion = substr($texto,0,$CFG{'RESUMEN'});
    if (length($texto) > $CFG{'RESUMEN'}) { # Solo trunca el resumen si es necesario.
        # Trunca a una palabra entera antes.
        $descripcion =~ s/[\&\w;#\xe1\xe9\xed\xf3\xfa\xc1\xc9\xcd\xd3\xda\xf1\xd1\xfc\xdc]+$//s;
        # $descripcion =~ s/[\&\w;#\x{c3a1}\x{c3a9}\x{c3ad}\x{c3b3}\x{c3ba}\x{c381}\x{c389}\x{c38d}\x{c393}\x{c39a}\x{c3b1}\x{c391}\x{c3bc}\x{c39c}]+$//s;
        $descripcion =~ s/[\|\n]/ /sg; # Elimina | y \n.
    }

    # Concatena el titular al texto para construir el indice.
    $texto = "$titular $texto";

    # Reemplaza tildes por letras normales.
    $texto = lib_encodes::no_accent_mark($texto);
    # Pasa todo el texto a minusculas. # Esto antes estaba antes de notildesUtf8.
    $texto = lc $texto;

    # Reemplaza cualquier cosa no caracter por un espacio.
    $texto =~ s/[^0-9a-z]+/ /sg;

    # Limita el texto a los primeros n caracteres.
    $texto = substr($texto,0,$CFG{'MAXCARS'});
    return ($titular,$meta1,$meta2,$meta3,$descripcion,$fechap,$texto,$seccion,$tema,$subtema,$custom_slug,$slug,$urlf5,@metadata);
}

# ------------------------------------------------------------------------
# Recorre el directorio de articulos determinando los anos que existen.
# Tambien determina la fecha de modificacion mas reciente de los archivos
# contenidos en el.
# Si se esta forzando un reindexado, no hace nada mas que simular que el
# indice de ese ano expiro.
# En el anio actual se omiten los ultimos dos dias.
# Se inserta un delay de 1 segundo cada 10 directorios.
sub busca_anos {
    my ($dir) = $_[0];
    my ($ano,$maxtime,$antesdeayer,$cc);

    # Obtiene el dia de antes de ayer.
    $antesdeayer = ((localtime(time))[5] + 1900). sprintf('%02d',((localtime(time - 172800))[4]+1)). sprintf('%02d',(localtime(time - 172800))[3]);

    my @days = glib_fildir_03::lee_dir($dir);

    $ANOS{$FORZAR_ULTIMOS_DIAS} = 0; # Fuerza la existencia del ano ficticio 7777 e inicializa su antiguedad en 0 (super antiguo).

    foreach my $day (sort @days) {
        next if ($day !~ /^\d{8}$/); # 20050612
        if ($day <= $antesdeayer) { # En la busqueda de anos no considera los dos ultimos dias.
            if ( $day =~ /^(\d{4})/) {
                $ano = $1;
                $cc++; # Cada 50 directorios descansa 1 segundo.
                if ($cc >= 50) {
                    sleep(1);
                    $cc = 0;
                }
                if ($FORZAR ne $PRONTUS && $FORZAR ne $FORZAR_TODO) {
                    $maxtime = &maxtime("$dir/$day/pags");
                } else {
                    # $maxtime = 0; # Con esto fuerza el reindexado de estos anos.
                    $maxtime = 1; # Con esto fuerza el reindexado de todos los anos de este Prontus.
                }
                if ($ANOS{$ano} < $maxtime) {
                    # my $mtime = (stat("$SEARCH_DIR$PRONTUS/$ano/$WORDSINDEX_FILENAME"))[9]; # debug
                    # print "day=$day maxtime=$maxtime\n"; # debug
                    $ANOS{$ano} = $maxtime;
                }
            }
        } else {
            # Fechas de modificacion de los ultimos 2 dias.
            $maxtime = &maxtime("$dir/$day/pags");
            if ($ANOS{$FORZAR_ULTIMOS_DIAS} < $maxtime) {

                # my $mtime = (stat("$SEARCH_DIR$PRONTUS/$ano/$WORDSINDEX_FILENAME"))[9]; # debug
                # print "day=$day maxtime=$maxtime\n"; # debug
                $ANOS{$FORZAR_ULTIMOS_DIAS} = $maxtime;
            }
        }
    }
}

# ------------------------------------------------------------------------
# Detecta el archivo mas reciente de la coleccion de directorios dado
# y retorna su antiguedad en segundos.
# Cambia el nombre de esta funcion a maxtime.
sub maxtime {
    my $dir = $_[0];
    my @files = glib_fildir_03::lee_dir($dir);
    my ($maxtime,$mtime) = (0,0);
    foreach my $file (@files) {
        next if ($file =~ /^\./);
        if (-f "$dir/$file") {

            #  (0: $dev, 1: $ino, 2: $mode, 3: $nlink, 4: $uid, 5: $gid, 6: $rdev, 7: $size, 8: $atime, 9: $mtime, 10: $ctime, 11: $blksize, 12: $blocks)
            $mtime = (stat "$dir/$file")[9];
            if ( $maxtime < $mtime  ) { $maxtime = $mtime; }
        } elsif (-d "$dir/$file") {
            $mtime = &maxtime("$dir/$file");
            if ( $maxtime < $mtime  ) { $maxtime = $mtime; }
        }
    }
    return $maxtime;
}

# ------------------------------------------------------------------------
# Valida las variables del archivo de configuracion.
sub validacfg {
    if ($CFG{'URL_MAXPAGS'} eq '') { $CFG{'URL_MAXPAGS'} = 2000; }
    if ($CFG{'FIDS'} eq '') { $CFG{'FIDS'} = 'fid_general'; }
    $CFG{'FIDS'} =~ s/\.\w+( |$)/ /g; # Elimina extensiones si las trae.
    if ($CFG{'RESUMEN'} < 0) { $CFG{'RESUMEN'} = 100; }
    if ($CFG{'RESUMEN'} > 400) { $CFG{'RESUMEN'} = 400; }
    if ($CFG{'MAXCARS'} < 10000) { $CFG{'MAXCARS'} = 10000; }
    if ($CFG{'RATIO'} < 10) { $CFG{'RATIO'} = 10; }
    if ($CFG{'RATIO'} > 100) { $CFG{'RATIO'} = 100; }
    if ($CFG{'MINTEXT'} < 0) { $CFG{'MINTEXT'} = 20; }
    if ($CFG{'TITLEVAR'} eq '') { $CFG{'TITLEVAR'} = '_TXT_TITULAR'; }
    if ($CFG{'TEXTVARS'} eq '') { $CFG{'TEXTVARS'} = 'TXT_bajada'; }
    my @ext = split(/\s+/,$CFG{'RAW_FILETYPES'});

    foreach my $ext (@ext) {
        $ext =~ s/[^a-zA-Z0-9]//g;
        if ($ext ne '') { push @RAW_FILETYPES, $ext; }
    }
    @ext = split(/\s+/,$CFG{'URL_FILETYPES'});
    foreach my $ext (@ext) {
        $ext =~ s/[^a-zA-Z0-9]//g;
        $URL_FILETYPES_REGEXP .= "\.$ext|";
    }

    $URL_FILETYPES_REGEXP .= '\/|\/[\w\-]+';

    # Obtiene definicion de filtros para archivos de datos.
    foreach my $key (keys %CFG) {
        if ($key =~ /^FILEFILTER/) {

            # FILEFILTER1 pdf /cgi-cpn/pdftotext.sh
            my ($ext,$filtro) = split(/\s+/,$CFG{$key});
            $filtro =~ s/\.\.\///g;
            if ( ($ext =~ /^\w+$/) && (-f "$ROOTDIR$filtro") ) {

                # Inscribe el filtro en el hash de filtros.
                $FILEFILTER{$ext} = "$ROOTDIR$filtro";
            }
        }
    }
}

# ------------------------------------------------------------------------
# Detecta los Prontus que hay que indexar.
sub detectadirs {
    foreach my $key (keys %CFG) {
        if ($key =~ /PRONTUS_DIR_\d+/) {
            push @PRONTUS_DIRS,$CFG{$key};

            # print 'Hay que indexar:' . $CFG{$key} . "\n"; # debug
        }
        if ($key =~ /RAW_DIR_\d+/) {
            push @RAW_DIR,$CFG{$key};

            # print 'Hay que indexar:' . $CFG{$key} . "\n"; # debug
        }
        if ($key =~ /URL_DIR_(\d+)/) {
            push @URL_DIR,$1;

            # print 'Hay que indexar:' . $CFG{$key} . " $1\n"; # debug
        }

        # print "KEY[$key] - DATA[$CFG{$key}]" .  "<BR>"; # debug
    }
}

# ---------------------------------------------------------------
# Lee y retorna la pagina web pasada como parametro.
sub getHTML {
    my $url = $_[0];
    my $buffer = '';
    my ($titular,$descripcion,$fechap,$texto,$robots) = ('','','','','');
    my $ua = new LWP::UserAgent;
    $ua->agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36");
    $ua->max_size(100000); # Pone este limite para no eternizar la cosa.
    # no verifica certificados para poder acceder a sitios por ip que pueden entregar un certificado invalido
    $ua->ssl_opts(verify_hostname => 0,
              SSL_verify_mode => 0x00);
    my ($request) = new HTTP::Request('GET', $url);
    my ($response) = $ua->request($request);

    if ($response->is_success) {
        $buffer = $response->content;

        # print "$buffer\n"; # debug
        # Detecta metas "robots".
        # <meta name="robots" content="nofollow">
        while ($buffer =~ /<meta\s+name=\"robots\"\s+content=\"([^\"]+)\">/isg) {
            $robots .= " $1";
        }
        ($titular,$descripcion,$fechap,$texto) = &get_contents_raw($buffer,$url);
    }
    return ($titular,$descripcion,$fechap,$texto,$robots,$buffer);
}

# -------------------------------------------------------------
# Va modificando un archivo .json con el resultado de la indexación
sub printJSON {
    my $msg = $_[0];
    my $status = defined $_[1] ? $_[1] : 1; # 0 para continuar, 1 para terminar. Default 1.
    my $inprogress = '';
    if ($status == 0) {
        $inprogress = ", \"inprogress\":\"$msg\"";
    }
    my $result_file = "/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/reindex.json";
    $msg = '{"status":'.$status.', "msg":"'.$msg.'"'.$inprogress.'}';
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$result_file", $msg);
}
# -------------------------------------------------------------
# tareas a ejecutar siempre, ya sea que la indexación termine exitosamente o fracase. Llamada en el bloque END {}
# por ahora, borra el semáforo si hubo un error.
sub tareas_salida {
    if ($SEARCH_DIR ne '' && $SEMAFORO_FILENAME ne '') {
        unlink "$SEARCH_DIR/$SEMAFORO_FILENAME";
    }
}
# -------------------------------------------------------------
# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    if ($SEARCH_DIR ne '' && $SEMAFORO_FILENAME ne '') {
        unlink "$SEARCH_DIR/$SEMAFORO_FILENAME";
    }
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}
