#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Ejecuta tareas de ajustes necesarias despúes de actualizar prontus
# requeridas por nuevas funciones implementadas
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Por linea de comando
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use utf8;
use strict;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus = "$Bin/multitag";
    unshift(@INC,$pathLibsProntus);
}

use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_lang;
use lib_maxrunning;
use lib_multitag;
use lib_prontus;
use lib_setbd;
use lib_tags;
use lib_tax;
use lib_utils;
use prontus_auth;
##/libs##

my $ERROR = '';

main: {
    lib_utils::setUp();

    if (lib_maxrunning::maxExcedido(1)) {
        exit;
    }

    print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "] ", lib_lang::get_message("post_update_tasks_starting");
    my $status_file = "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.json";
    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER/prontus/cpan/data/");
    glib_fildir_03::write_file($status_file, '{"status": 1, "progress":0, "msg":"'.lib_lang::get_message("index_processing").'"}');

    my $worker = "$prontus_varglb::DIR_SERVER/prontusSW.min.js";
    if (!-f $worker && !-l $worker) {
        my $cmd = "ln -sr $prontus_varglb::DIR_SERVER/prontus/front/prontusPush/prontusSW.min.js $worker 2>&1";
        my $res = `$cmd`;
        print STDERR $res, "\n";
    }

    # path de cgis-cpn para ejecutar scripts
    my $path_cgicpn = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    # reinicio de procesos prontus
    my $res = `perl $path_cgicpn/fcgi_fpm/prontus_signal_fcgi.pl stop`;
    # espera que reinicio se complete
    sleep 5;

    # definicion de columnas de bd que hay que cambiar de tipo char a numerico respecto a versiones anteriores de prontus
    # ej:
    # {'table' => 'tagsart', 'column' => 'tagsart_idart', 'action' => 'def', 'coldef' => 'bigint unsigned not null default 0'},
    # {'table' => 'tags', 'column' => 'tags_count', 'action' => 'delete'},
    my $largo = $prontus_varglb::URL_MAX_LENGTH;
    my @columnas_bd = (
        {'table' => 'tags', 'column' => 'tags_count', 'action' => 'delete'},
        {'table' => 'secc', 'column' => 'secc_mostrar', 'action' => 'def', 'coldef' => 'tinyint unsigned not null default 1'},
        {'table' => 'temas', 'column' => 'temas_mostrar', 'action' => 'def', 'coldef' => 'tinyint unsigned not null default 1'},
        {'table' => 'subtemas', 'column' => 'subtemas_mostrar', 'action' => 'def', 'coldef' => 'tinyint unsigned not null default 1'},
        {'table' => 'tags', 'column' => 'tags_mostrar', 'action' => 'def', 'coldef' => 'tinyint unsigned not null default 1'},
        {'table' => 'art', 'column' => 'art_autor', 'action' => 'def', 'coldef' => "char(64) not null default ''"},
        {'table' => 'art', 'column' => 'art_titu', 'action' => 'def', 'coldef' => "char(255) not null default ''"},
        {'table' => 'art', 'column' => 'art_baja', 'action' => 'def', 'coldef' => "char(200) not null default ''"},
        {'table' => 'art', 'column' => 'art_extension', 'action' => 'def', 'coldef' => "char(4) not null default ''"},
        {'table' => 'art', 'column' => 'art_tipoficha', 'action' => 'def', 'coldef' => "char(64) not null default ''"},
        {'table' => 'urlf5', 'column' => 'urlf5_file', 'action' => 'def', 'coldef' => "char($largo) not null default ''"},
        {'table' => 'urlf5', 'column' => 'urlf5_url', 'action' => 'def', 'coldef' => "char($largo) not null default ''"},
        {'table' => 'urlf5', 'column' => 'urlf5_prontus', 'action' => 'def', 'coldef' => "char(32) not null default ''"},
        {'table' => 'users', 'column' => 'users_usr', 'action' => 'def', 'coldef' => "char(32) not null default ''"},
        {'table' => 'users', 'column' => 'users_email', 'action' => 'def', 'coldef' => "char(255) not null default ''"},
        {'table' => 'secc', 'column' => 'secc_nom', 'action' => 'def', 'coldef' => "char(128) not null default ''"},
        {'table' => 'temas', 'column' => 'temas_nom', 'action' => 'def', 'coldef' => "char(128) not null default ''"},
        {'table' => 'subtemas', 'column' => 'subtemas_nom', 'action' => 'def', 'coldef' => "char(128) not null default ''"},
        {'table' => 'tags', 'column' => 'tags_tag', 'action' => 'def', 'coldef' => "char(255) not null default ''"},
        {'table' => 'asset', 'column' => 'asset_file', 'action' => 'def', 'coldef' => "char(150) not null default ''"},
        {'table' => 'asset', 'column' => 'asset_placement', 'action' => 'def', 'coldef' => "char(40) not null default ''"},
        {'table' => 'multitag_s', 'column' => 'multitag_s_friendly', 'action' => 'def', 'coldef' => "char(64) not null default ''"},
        {'table' => 'multitag_t', 'column' => 'multitag_t_friendly', 'action' => 'def', 'coldef' => "char(64) not null default ''"},
        {'table' => 'multitag_st', 'column' => 'multitag_st_friendly', 'action' => 'def', 'coldef' => "char(64) not null default ''"},
    );

    my @index_tables = (
        {'table' => 'urlf5', 'column' => 'urlf5_subtypeid'},
        {'table' => 'asset', 'column' => 'asset_file'},
        {'table' => 'asset', 'column' => 'asset_art_id'},
        {'table' => 'asset', 'column' => 'asset_pix'},
        {'table' => 'asset', 'column' => 'asset_original'},
        {'table' => 'secc', 'column' => 'secc_mostrar'},
        {'table' => 'secc', 'column' => 'secc_orden'},
        {'table' => 'temas', 'column' => 'temas_mostrar'},
        {'table' => 'temas', 'column' => 'temas_orden'},
        {'table' => 'subtemas', 'column' => 'subtemas_mostrar'},
        {'table' => 'subtemas', 'column' => 'subtemas_orden'},
        {'table' => 'tags', 'column' => 'tags_mostrar'},
        {'table' => 'art', 'column' => 'art_idtemas1'},
        {'table' => 'art', 'column' => 'art_idsubtemas1'},
        {'table' => 'art', 'column' => 'art_idtemas2'},
        {'table' => 'art', 'column' => 'art_idsubtemas2'},
        {'table' => 'art', 'column' => 'art_idtemas3'},
        {'table' => 'art', 'column' => 'art_idsubtemas3'},
        {'table' => 'art', 'column' => 'art_autor'},
        {'table' => 'art', 'column' => 'art_fechae'},
        {'table' => 'art', 'column' => 'art_titu'},
        {'table' => 'art', 'column' => 'art_baja'},
        {'table' => 'art', 'column' => 'art_idusr'},
        {'table' => 'art', 'column' => 'art_soloportadas'},
        {'table' => 'art', 'column' => 'art_fecha_inicio'},
        {'table' => 'art', 'column' => 'art_fecha_fin'},
        {'table' => 'art', 'column' => 'art_modtime'},
        {'table' => 'art', 'column' => 'art_s3_size'},
        {'table' => 'art', 'column' => 'art_s3'},
        {'table' => 'art', 'column' => 'art_indexar'},
        {'table' => 'urlf5', 'column' => 'urlf5_type'},
        {'table' => 'urlf5', 'column' => 'urlf5_file'},
        {'table' => 'multitag_s', 'column' => 'multitag_s_estado'},
        {'table' => 'multitag_t', 'column' => 'multitag_t_estado'},
        {'table' => 'multitag_st', 'column' => 'multitag_st_estado'},
        {'table' => 'users', 'column' => 'users_deleted'},
        {'table' => 'users', 'column' => 'users_email'},
        {'table' => 'oidc_users', 'column' => 'oidc_idusers'},
        {'table' => 'old_psw', 'column' => 'users_id'},
        {'table' => 'port_users', 'column' => 'port_user_id'},
        {'table' => 'art_users', 'column' => 'art_user_id'},
        {'table' => 'prontus_users', 'column' => 'prontus_user_id'},
        {'table' => 'prontus_users', 'column' => 'prontus_id'},
    );

    my @prontus_dirs = lib_prontus::get_prontus_dirs();
    my $prontus_count = scalar @prontus_dirs;
    my $counter = 0;
    my $prontus_percent = 1/$prontus_count;

    # borramos archivos que ya no se usan y fueron removidos de la release
    my @delete_files = (
        '/cgi-bin/prontus_captcha.cgi',
        '/cgi-bin/prontus_make_taxport.cgi',
        '/cgi-cpn/lib_captcha.pm',
        '/cgi-cpn/mam/lib_fechas.pm',
        '/cgi-cpn/mam/lib_mam.pm',
        '/cgi-cpn/prontus_art_check_url.cgi',
        '/cgi-cpn/prontus_art_repair.cgi',
        '/cgi-cpn/utils/prontus_corrige_friendly4.pl',
        '/cgi-cpn/wizard/wizard_lib.pm',
        '/prontus/cpan/css/bootstrap-datetimepicker.css',
        '/prontus/cpan/css/bootstrap.css',
        '/prontus/cpan/css/coment.css',
        '/prontus/cpan/css/editor_fid.css',
        '/prontus/cpan/css/editor.css',
        '/prontus/cpan/css/fontawesome/webfonts/fa-brands-400.eot',
        '/prontus/cpan/css/fontawesome/webfonts/fa-brands-400.svg',
        '/prontus/cpan/css/fontawesome/webfonts/fa-brands-400.woff',
        '/prontus/cpan/css/fontawesome/webfonts/fa-regular-400.eot',
        '/prontus/cpan/css/fontawesome/webfonts/fa-regular-400.svg',
        '/prontus/cpan/css/fontawesome/webfonts/fa-regular-400.woff',
        '/prontus/cpan/css/fontawesome/webfonts/fa-solid-900.eot',
        '/prontus/cpan/css/fontawesome/webfonts/fa-solid-900.svg',
        '/prontus/cpan/css/fontawesome/webfonts/fa-solid-900.woff',
        '/prontus/cpan/css/next.css',
        '/prontus/cpan/css/prontus_bootstrap.css',
        '/prontus/cpan/css/tagsfid.css',
        '/prontus/cpan/css/validator.css',
        '/prontus/cpan/imag/boto/arrow_down.png',
        '/prontus/cpan/imag/boto/arrow_left.png',
        '/prontus/cpan/imag/boto/arrow_right.png',
        '/prontus/cpan/imag/boto/arrow_up.png',
        '/prontus/cpan/imag/boto/mover2.png',
        '/prontus/cpan/imag/loading_bar.gif',
        '/prontus/cpan/imag/loading_button.gif',
        '/prontus/cpan/imag/loading.gif',
        '/prontus/cpan/js-local/Acciones.class.js',
        '/prontus/cpan/js-local/bootstrap-filestyle/bootstrap-filestyle.js',
        '/prontus/cpan/js-local/bootstrap/bootstrap.js',
        '/prontus/cpan/js-local/Botones.class.js',
        '/prontus/cpan/js-local/clipboard/clipboard.js',
        '/prontus/cpan/js-local/Editor.class.js',
        '/prontus/cpan/js-local/prontus.js',
        '/prontus/cpan/plantillas/core/prontus_check_install_form.html',
        '/prontus/cpan/plantillas/core/prontus_include_common.html',
        '/prontus/cpan/twitter_post/tmhOAuth-0.8.4/cacert.pem',
        '/prontus/cpan/twitter_post/tmhOAuth-0.8.4/composer.json',
        '/prontus/cpan/twitter_post/tmhOAuth-0.8.4/LICENSE',
        '/prontus/cpan/twitter_post/tmhOAuth-0.8.4/README.md',
        '/prontus/cpan/twitter_post/tmhOAuth-0.8.4/tmhOAuth.php',
        '/prontus/front/js-local/embedAudio/playerConfig.js',
        '/prontus/front/js-local/embedVideo/playerConfig.js',
        '/prontus/front/js-local/jquery/jquery-3.3.1.min.js',
        '/prontus/front/js-local/jquery/jquery-3.3.1.min.map',
        '/prontus/front/prontusAudioPlayer/css/prontusAudioPlayer.min.css',
        '/prontus/front/prontusAudioPlayer/imag/player/ffw_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/main_play.png',
        '/prontus/front/prontusAudioPlayer/imag/player/main_replay.png',
        '/prontus/front/prontusAudioPlayer/imag/player/mute_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/pause_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/play_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/pploading.gif',
        '/prontus/front/prontusAudioPlayer/imag/player/prontusLogo.png',
        '/prontus/front/prontusAudioPlayer/imag/player/rew_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/unmute_ico.png',
        '/prontus/front/prontusAudioPlayer/imag/player/vol_ico.png',
        '/prontus/front/prontusAudioPlayer/js-local/defaultConfig.js',
        '/prontus/front/prontusAudioPlayer/js-local/prontusAudioPlayer.min.js',
        '/prontus/front/prontusPlayer/css/prontusPlayer.min.css',
        '/prontus/front/prontusPlayer/externalApi.min.js',
        '/prontus/front/prontusPlayer/flash/prontusPlayer.swf',
        '/prontus/front/prontusPlayer/img/arrow.png',
        '/prontus/front/prontusPlayer/img/close_ads.png',
        '/prontus/front/prontusPlayer/img/close_ico.png',
        '/prontus/front/prontusPlayer/img/conf_ico.png',
        '/prontus/front/prontusPlayer/img/fb_ico.png',
        '/prontus/front/prontusPlayer/img/ffw_ico.png',
        '/prontus/front/prontusPlayer/img/full_ico.png',
        '/prontus/front/prontusPlayer/img/gp_ico.png',
        '/prontus/front/prontusPlayer/img/main_play.png',
        '/prontus/front/prontusPlayer/img/main_replay.png',
        '/prontus/front/prontusPlayer/img/mute_ico.png',
        '/prontus/front/prontusPlayer/img/pause_ico.png',
        '/prontus/front/prontusPlayer/img/play_ico.png',
        '/prontus/front/prontusPlayer/img/pploading.gif',
        '/prontus/front/prontusPlayer/img/prontus-player-bg640x360.jpg',
        '/prontus/front/prontusPlayer/img/prontusLogo.png',
        '/prontus/front/prontusPlayer/img/rew_ico.png',
        '/prontus/front/prontusPlayer/img/rolling.png',
        '/prontus/front/prontusPlayer/img/share_ico.png',
        '/prontus/front/prontusPlayer/img/tw_ico.png',
        '/prontus/front/prontusPlayer/img/vol_ico.png',
        '/prontus/front/prontusPlayer/img/wp_ico.png',
        '/prontus/front/prontusPlayer/prontusPlayer.min.js',
    );

    foreach my $files (@delete_files) {
        unlink "$prontus_varglb::DIR_SERVER$files";
    }

    print STDERR "Prontus count [$prontus_count] ". join(';', @prontus_dirs). "\n";
    # Iterar por cada prontus
    foreach my $prontus_dir (@prontus_dirs) {
        next if ($prontus_dir eq 'prontus');
        print STDERR lib_lang::get_interpolated_message("post_update_tasks_processing_prontus_dir", $prontus_dir);
        my $ready = 100*$counter/$prontus_count;
        $counter++;

        # se reinician los valores de prontus_varglb
        prontus_varglb::init();
        # Carga variables de configuracion.
        my $msg_load = lib_utils::load_config_by_id($prontus_dir);
        if ($msg_load ne '') {
            print STDERR lib_lang::get_interpolated_message("global_error_loading_config", $msg_load)."\n";
            next;
        }

        # crea directorios necesarios para el funcionamiento de Prontus, por si se agregaron nuevos directorios
        lib_prontus::check_dirs();
        &update_progress($status_file, int($ready+$prontus_percent*2));

        # empiezan modificaciones a BD
        my $msg_err_bd = '';
        ($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd) = lib_bd::conectar_prontus_bd();
        if (! ref($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID})) {
            print STDERR $msg_err_bd, "\n";
            print STDERR lib_lang::get_message("post_update_tasks_skipping_prontus");
            next;
        }

        my ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_prontus_users($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID});
        if (!$hay_err || $msg_ret eq 'db_exists_prontus_users') {
            my $prontus_users_table = lib_utils::get_table('prontus_users');
            my $sql = "select count(*) from $prontus_users_table";
            my $count_prontususers;
            my $salida = glib_dbi_03::ejecutar_sql_bind($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \($count_prontususers));
            $salida->fetch;
            $salida->finish;
            # tabla vacia, asigna permiso de acceso a prontus a los usuarios
            if ($count_prontususers < 1) {
                # si hay sso le asigna permisos de acceso a todos los pronts que maneja este prontus
                my @prontus_list;
                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} eq 'SI'
                    && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} eq $prontus_varglb::PRONTUS_ID) {
                    @prontus_list = lib_prontus::get_prontus_sso_dirs();
                } else {
                    @prontus_list = ($prontus_varglb::PRONTUS_ID);
                }
                my $users_table = lib_utils::get_table('users');
                $sql = "select users_id from $users_table where users_iD > 1";
                my $user_id;
                $salida = glib_dbi_03::ejecutar_sql_bind($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \($user_id));
                while ($salida->fetch) {
                    foreach my $prontus (@prontus_list) {
                        $sql = "insert into $prontus_users_table values(?, ?)";
                        my @insert_data = ($user_id, $prontus);
                        glib_dbi_03::ejecutar_sql_param($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \@insert_data);
                        if ($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}->errstr) {
                            print STDERR lib_lang::get_interpolated_message("post_update_tasks_error_updating_table", 'prontus_users', $sql, $prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}->errstr);
                        }
                    }
                }
                $salida->finish;
            }
        } else {
            $ERROR = "($msg_ret, $hay_err)";
            exit;
        }
        &update_progress($status_file, int($ready+$prontus_percent*48));

        # modificaciones a columnas de BD
        # dejamos con 0 posibles valores vacios de columnas mostrar
        my $table_name = lib_utils::get_table('secc');
        my $sql_update = "update $table_name set secc_mostrar='0' where secc_mostrar=''";
        my $salida_update = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql_update);
        $table_name = lib_utils::get_table('temas');
        $sql_update = "update $table_name set temas_mostrar='0' where temas_mostrar=''";
        $salida_update = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql_update);
        $table_name = lib_utils::get_table('subtemas');
        $sql_update = "update $table_name set subtemas_mostrar='0' where subtemas_mostrar=''";
        $salida_update = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql_update);
        $table_name = lib_utils::get_table('tags');
        $sql_update = "update $table_name set tags_mostrar='0' where tags_mostrar=''";
        $salida_update = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql_update);

        foreach my $column (@columnas_bd) {
            my $mod_table = lib_utils::get_table($$column{'table'});
            my ($res, $nom, $type, $null, $key, $default, $extra) = glib_dbi_03::get_table_column_def($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $mod_table, $$column{'column'});
            # columna existe y es char
            if ($res) {
                # cambia tipo de la columna
                if ($$column{'action'} eq 'def') {
                    my $sql = "alter table $mod_table modify column $$column{'column'} $$column{'coldef'}";
                    my $salida = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql);
                    $salida->finish;
                }
                # borra columna
                elsif ($$column{'action'} eq 'delete') {
                    my $sql = "alter table $mod_table drop column $$column{'column'}";
                    my $salida = glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql);
                    $salida->finish;
                }
            }
        }

        # agregamos indices a tablas de prontus
        foreach my $column (@index_tables) {
            my $mod_table = lib_utils::get_table($$column{'table'});
            my $exist = glib_dbi_03::check_index_table($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $mod_table, $$column{'column'});

            if (!$exist) {
                # indice no existe, lo agregamos
                my $sql_index = "alter table $mod_table add index ($$column{'column'})";
                my $salida = &glib_dbi_03::ejecutar_sql($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql_index);
                $salida->finish;
            }
        }
        &update_progress($status_file, int($ready+$prontus_percent*80));
        # terminan modificaciones a BD

        # este metodo recarga la configuracion del prontus_id indicado
        # Se recarga la configuracion.
        $msg_load = lib_utils::load_config_by_id($prontus_varglb::PRONTUS_ID);
        if ($msg_load ne '') {
            print STDERR lib_lang::get_interpolated_message('post_update_tasks_error_loading_config' , $msg_load);
            exit;
        }

        lib_prontus::generaJsonProntus();

        &update_progress($status_file, int($ready+$prontus_percent*88));

        # Actualiza el embed.html de prontus/front, reemplaza ##timestamp##
        my $path_embed_tpl = "$prontus_varglb::DIR_SERVER/prontus/cpan/plantillas/front/embed_tpl.html";
        my $path_embed = "$prontus_varglb::DIR_SERVER/prontus/front/embed.html";
        my $buffer_embed = glib_fildir_03::read_file($path_embed_tpl);
        my $timestamp = time();
        $buffer_embed =~ s/##timestamp##/$timestamp/g;
        glib_fildir_03::write_file($path_embed, $buffer_embed);

        &update_progress($status_file, int($ready+$prontus_percent*90));

        lib_tags::generar_json($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID});
        &update_progress($status_file, int($ready+$prontus_percent*94));

        lib_tax::generar_json($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID});
        &update_progress($status_file, int($ready+$prontus_percent*98));

        lib_multitag::generar_json($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID});

        lib_multitag::generar_json_fid($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID});

        # desconectamos la bd de prontus
        $prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}->disconnect;
        $prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID} = undef;

        &update_progress($status_file, int($ready+$prontus_percent*100));

        print STDERR "\n\n";
    }
    glib_fildir_03::write_file($status_file, '{"status": 1, "progress":100, "msg":"'.lib_lang::get_message('post_update_tasks_msg_ended').'"}');

    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER/prontus/data") if (-d "$prontus_varglb::DIR_SERVER/prontus/data");
    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER/prontus/ejemplos") if (-d "$prontus_varglb::DIR_SERVER/prontus/ejemplos");

    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.txt", $prontus_varglb::VERSION_PRONTUS);
}

sub update_progress {
    my ($status_file, $percent) = @_;
    glib_fildir_03::write_file($status_file, '{"status": 1, "progress":'. $percent .', "msg":"'.lib_lang::get_message("index_processing").'"}');
}

# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}

END {
    if ($ERROR ne '') {
        my $status_file = "$prontus_varglb::DIR_SERVER/prontus/cpan/data/prontus_updated.json";
        glib_fildir_03::write_file($status_file, '{"status": 0, "progress":0, "msg":"'.$ERROR.'"}');
        print STDERR "$ERROR\n";
    }
    unlink "$prontus_varglb::DIR_SERVER/prontus/cpan/data/_login.block";
}
