#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

package prontus_varglb;

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Definir e inicializar variables globales
# ---------------------------------------------------------------

use strict;
use utf8;

# variables publicas con valores fijos
our $VERSION_PRONTUS = '12.3.1.2 - 2024/02/20';
# string por defecto para subtitulos de articulos
our $DEFAULT_TITHTML = '<a name=%%_subtit_aname%%>%%_subtit%%</a><a href="#top" class="linkSubtit">&nbsp;|&nbsp;subir</a>';
our $DIR_CGI_CPAN = 'cgi-cpn';
our $DIR_CGI_PUBLIC = 'cgi-bin';
our $DIR_IP_CONTROL = 'ip_control';
our $UPLOADS_PERMITIDOS_DEFAULT = 'jpg,jpeg,jpe,gif,png,svg,webp,bmp,tif,tiff,ico,asf,asx,wax,wmv,wmx,avi,divx,flv,mov'
                             . ',qt,mpeg,mpg,mpe,3gp,txt,rtx,css,htm,html,mp3,m4a,mp4,m4v,ra,ram,wav,ogg,oga,ogv,mid'
                             . ',midi,wma,rtf,pdf,doc,docx,pot,pps,ppt,pptx,wri,xla,xls,xlsx,xlt,xlw,mdb'
                             . ',mpp,swf,tar,zip,gz,gzip,odt,odp,ods,odg,odc,odb,odf,csv,mkv';
# VALIDACION DE SEGURIDAD PARA TAXPORT_MAXARTICS
our $TAXPORT_MAXARTICS_SECURITY = 100000;
our $XCODE_CFG = "formatos_adv.cfg";
our $XCODING_DATA_PATH = "/cpan/data/xcoding/";

# La API URL de Cloudflare
our $CLOUDFLARE_API_URL = 'https://api.cloudflare.com/client/v4';
# url manuales prontus
our $URL_MANUAL = 'https://wiki.prontus.cl';

# Directorios de Prontus
# Directorios correspondientes al contenido de las paginas generadas
# relativos al publicador

our $DIR_SITE = '/site';
our $DIR_LIST = '/list/port';
our $DIR_LIST_MACROS = '/list/macros';
our $DIR_ARTIC = '/artic';
our $DIR_FECHA = '/fecha';
our $DIR_PAG = '/pags'; # Dir. donde se almacenaran las paginas generadas
our $DIR_PORT = '/port';   # Dir. de portadas
our $DIR_MACROS = '/macros';   # Dir. de portadas
our $DIR_IMAG = '/imag';   # Dir. donde se almacenaran las imagenes incluidas en las paginas generadas
our $DIR_MMEDIA = '/mmedia';   # Dir. donde se almacenaran los archivos de video cargados en los articulos
our $DIR_ASOCFILE = '/docs';   # Dir. donde se almacenaran los archivos genericos asociados en las paginas publicadas.
our $DIR_EXMEDIA = '/mm';  # Ubicacion de los archivos multimedia, al mismo nivel de artic
our $DIR_THUMBS = '/thumbs'; # ubicacion de las thumbnails de los videos dentro de exmedia
our $DIR_BKP = '/_prontus_bkp'; # ubicacion de los respaldos de artículos
# mam
our $DIR_MAM = '/mam';
our $INDEX_MAM = 'index.json';

# Directorios correspondientes al contenido del sitio web, relativos a $DIR_CONTENIDO
our $DIR_HPAGES = '/home';
our $DIR_EDIC = '/edic';
our $DIR_NROEDIC = '/nroedic';
our $DIR_BASEEDIC = '/base';

# variables para friendly url v5
our $DIR_FRIENDLY5 = '/prontus_links'; # Directorio donde se almacenaran los includes de friendly urls

# directorio cache multitag
our $DIR_MULTITAG = '/cache/multitag';
# Directorios correspondientes a los templates, relativos al publicador
our $DIR_TEMPLATE = '/plantillas';

# Directorios del panel de control, relativos al server, donde estan los formularios usados por el publicador.
our $DIR_PATH_CPAN = '/cpan';
our $DIR_CPAN_DATA = '/cpan/data';
our $DIR_LOCK = '/lock';
our $DIR_DATA4FIDS = '/data4fids';
our $DIR_CORE = '/prontus/cpan';
our $DIR_CORE_FRONT = '/prontus/front';
our $DIR_PLT_CORE = '/prontus/cpan/plantillas';
our $DIR_MY_SEARCH = '/users/misbusquedas';
our $MAX_MY_SEARCH = 10;

# variables para friendly url
# directorio donde se almacenaran los includes de friendly urls
our $DIR_FRIENDLY = '/friendly/links';
# numero base de epoch para restar de forma de no generar ids adicionales muy grandes pero siempre crecientes para diferenciar las urls
our $URL_NUMBER = 1690000000;
our $URL_MAX_LENGTH = 500;

# Taxonomia
our $DIR_TAXONOMIA_OLD = '/cache/taxonomia/pags';
our $DIR_TAXONOMIA = '/relac/tax/pags';
our $DIR_TAGS_OLD = '/cache/tags/pags';
our $DIR_TAGS = '/relac/tags/pags';
our $DIR_TAXPORT = '/tax/port';
our $DIR_TAXPORT_MACROS = '/tax/macros';
our $DIR_TAGPORT = '/tag/port';
our $DIR_TAGPORT_MACRO = '/tag/macros';

# Tipos de imagenes que no se pueden manipular.
our %IMG_TYPE_NOEDIT = ('svg' => 1, 'webp' => 1);
# fotos a mostrar por defecto en el banco de imagenes del fid
our $BANCO_IMG_MAX = 5;

# Nombre del template de homepage de cada edicion, ubicado dentro del dir $DIR_HPAGES del dir. de los templates ($DIR_TEMP)
# Se toma el primer archicov que se encuentre en el directorio y se usa ese nombre, salvo para el wizard.
our $INDEX_EDIC = 'home.html';

# En la raiz del sitio, este es el index que es manipulado por
# por el Adm. de Ediciones para setear la ed. vigente.
# El template es index_tmp.shtml SOLO MULTI-ED
our $INDEX_SITIO = 'index.html';
# SOLO MULTI-ED
our $TPL_INDEX_SITIO = 'index_tmp.html';

# variables publicas que cambian de valor al cargar configuracion
our $PRONTUS_ID = 'prontus';
our $RAMA_INSTALADA = '';
our $NRO_REVISION_INSTALADA = '';
our $BETA_REVISION_INSTALADA = '';

our %CONFIG = ();
our %CONFIG_LOADED = ();
our $FCGI_MODE = 0;
our %FCGI_CONFIG = (
    'single_log' => 0,
    'num_procs_prontus' => 2,
    'num_procs_friendly' => 4,
    'f5_proc_name' => 'prontus-f5-pm',
    'f5_worker_name' => 'prontus-f5-worker',
    'fcgi_proc_name' => 'prontus-fcgi-pm',
    'fcgi_worker_name' => 'prontus-fcgi-worker'
);

our %CFG_SINGLE_VAL = (
    'FRIENDLY_URLS_USE_SYMLINK' => 'SI',
    'FRIENDLY_URLS_ENABLE' => '1',
    'FRIENDLY_URLS_LARGO_TITULAR' => 75,
    'FRIENDLY_URL_PREFIX' => '',
    'FRIENDLY_INCLUDE_VIEW_NAME' => 'NO',
    'FRIENDLY_V4_INCLUDE_PRONTUS_ID' => 'SI',
    'CAPTCHA_OBLIGATORIO' => 'SI',
    'RECAPTCHA_API_URL' => 'https://www.google.com/recaptcha/api/siteverify',
    'RECAPTCHA_SECRET_CODE' => '',
    'RECAPTCHA_SITEKEY' => '',
    'RECAPTCHA_V3_SCORE' => '0.5',
    'DROPBOX' => 'NO',
    'DROPBOX_FILEXT_EXCLUDE' => '',
    'DROPBOX_APPNAME' => '',
    'DROPBOX_CLIENT_ID' => '',
    'DROPBOX_CLIENT_SECRET' => '',
    'DROPBOX_AUTHORIZATION_CODE' => '',
    'DROPBOX_REFRESH_TOKEN' => '',
    'DROPBOX_OAUTH_TOKEN' => '',
    'CLOUDFLARE' => 'NO',
    'CLOUDFLARE_API_TOKEN' => '',
    'CLOUDFLARE_ZONE' => '',
    'CLOUDFLARE_GLOBAL_PURGE' => '',
    'CACHE_PURGE_TAXPORT' => 'SI',
    'CACHE_PURGE_TAGPORT' => 'SI',
    'CACHE_PURGE_TAXPORT_MV' => 'SI',
    'CACHE_PURGE_TAGPORT_MV' => 'SI',
    'CACHE_PURGE_MAPA' => 'SI',
    'CACHE_PURGE_ART_RELAC' => 'SI',
    'ACTIVAR_FIDS_SIN_VISTA' => 'NO',
    'N_THREADS' => 0,
    'XCODING_PPROC' => '',
    'GEN_HLS' => 'NO',
    'PRECISION_HLS' => 'NO',
    'LIMIT_BITRATE' => 'NO',
    'MODO_PARALELO' => 'NO',
    'MAX_VIDEO_BITRATE' => 2400,
    'MAX_AUDIO_BITRATE' => 128,
    'RUTA_TEMPORAL_XCODING' => '',
    'XCODE_MAX_PIXEL' => 1280,
    'XCODE_MAX_PARALELO' => 3,
    'USAR_XCODER_EXTERNO' => 'NO',
    'XCODER_HOST' => '',
    'XCODER_PORT' => '',
    'UPLOADS_PERMITIDOS' => $prontus_varglb::UPLOADS_PERMITIDOS_DEFAULT,
    'PUBLIC_SERVER_NAME' => '',
    'CONTROL_FECHA' => 'NO',
    'MULTI_EDICION' => 'NO',
    'ADMIN_PORT' => 'NO',
    'PUBLIC_PORT' => 'SI',
    'PRONTUS_SSO' => 'NO',
    'PRONTUS_SSO_MASTER' => 'NO',
    'PRONTUS_SSO_MASTER_ID' => '',
    'PORT_INI_SELECTED' => '',
    'PORT_HOME' => '',
    'SNP_APP' => 'NO',
    'SNP_APP_SERVER_KEY' => '',
    'SNP_API_SERVER' => '',
    'SNP_API_KEY' => '',
    'SNP_API_APP_ID' => '',
    'SNP_API_CLIENT_ID' => '',
    'SNP_APP_CUSTOM_CSS' => '',
    'SNP_APP_ANALYTICS_UTM' => '',
    'SNP_APP_FOTOFIJA' => '',
    'ELASTICSEARCH_HABILITADO' => 'NO',
    'ELASTICSEARCH_HOST' => 'localhost',
    'ELASTICSEARCH_HOST_PROTOCOL' => 'http',
    'ELASTICSEARCH_PORT' => '9200',
    'ELASTICSEARCH_BATCH_CHUNK' => '50',
    'ELASTICSEARCH_INDEX' => 'documentos_prontus',
    'ELASTICSEARCH_FIDS' => '',
    'ELASTICSEARCH_MAXCARS' => '100000',
    'ELASTICSEARCH_MINTEXT' => '3',
    'ELASTICSEARCH_RATIO' => '80',
    'ELASTICSEARCH_TEXTVARS' => '_TXT_bajada VTXT_CUERPO',
    'ELASTICSEARCH_RESPERPAG' => '50',
    'ELASTICSEARCH_MAXPAGS' => '20',
    'ELASTICSEARCH_MAXEXEC' => '5',
    'ELASTICSEARCH_METADATA_IMG' => '',
    'ELASTICSEARCH_SEARCH_META1' => '',
    'ELASTICSEARCH_SEARCH_META2' => '',
    'ELASTICSEARCH_SEARCH_META3' => '',
    'ELASTICSEARCH_SEARCH_META4' => '',
    'ELASTICSEARCH_SEARCH_META5' => '',
    'ELASTICSEARCH_SEARCH_META1_TIPO' => '',
    'ELASTICSEARCH_SEARCH_META2_TIPO' => '',
    'ELASTICSEARCH_SEARCH_META3_TIPO' => '',
    'ELASTICSEARCH_SEARCH_META4_TIPO' => '',
    'ELASTICSEARCH_SEARCH_META5_TIPO' => '',
    'ELASTICSEARCH_METADATA1' => '',
    'ELASTICSEARCH_METADATA2' => '',
    'ELASTICSEARCH_METADATA3' => '',
    'ELASTICSEARCH_METADATA4' => '',
    'ELASTICSEARCH_METADATA5' => '',
    'ELASTICSEARCH_METADATA6' => '',
    'ELASTICSEARCH_METADATA7' => '',
    'ELASTICSEARCH_METADATA8' => '',
    'ELASTICSEARCH_METADATA9' => '',
    'ELASTICSEARCH_METADATA10' => '',
    'ELASTICSEARCH_PROMOTED_TEXT1' => '',
    'ELASTICSEARCH_PROMOTED_TEXT2' => '',
    'ELASTICSEARCH_PROMOTED_TEXT3' => '',
    'ELASTICSEARCH_PROMOTED_TEXT4' => '',
    'ELASTICSEARCH_PROMOTED_TEXT5' => '',
    'ELASTICSEARCH_PROMOTED_TS1' => '',
    'ELASTICSEARCH_PROMOTED_TS2' => '',
    'ELASTICSEARCH_PROMOTED_TS3' => '',
    'ELASTICSEARCH_PROMOTED_TS4' => '',
    'ELASTICSEARCH_PROMOTED_TS5' => '',
    'ELASTICSEARCH_TIPO_PAGINACION' => '0',
    'ELASTICSEARCH_PAGCORTA_MAXPAGS' => '5',
    'ELASTICSEARCH_SEARCHTIPS_MINLEN' => '2',
    'SEO_API_HABILITADO' => 'NO',
    'SEO_TWITTER_HANDLE' => '',
    'SEO_FB_ID' => '',
    'SEO_GLOBAL_ID' => '',
    'SEO_API_KEY' => '',
    'SEO_API_SERVER' => '',
    'SEO_API_PORT' => '',
    'SEO_API_VERSION' => 'v1',
    'SEO_NAME_TITULAR' => 'seo_titulo',
    'SEO_NAME_SLUG' => '_slug',
    'SEO_NAME_DESCRIPCION' => 'seo_descripcion',
    'SEO_NAME_CUERPO' => 'vtxt_cuerpo',
    'SEO_MIN_CARACTERES_TITULAR' => '25',
    'SEO_MAX_CARACTERES_TITULAR' => '60',
    'SEO_MIN_PALABRAS_TITULAR' => '3',
    'SEO_MAX_PALABRAS_TITULAR' => '10',
    'SEO_MIN_CARACTERES_TITULAR' => '',
    'SEO_MAX_CARACTERES_TITULAR' => '',
    'SEO_MIN_PALABRAS_TITULAR' => '',
    'SEO_MAX_PALABRAS_TITULAR' => '',
    'SEO_MIN_CARACTERES_DESCRIPCION' => '120',
    'SEO_MAX_CARACTERES_DESCRIPCION' => '156',
    'SEO_MIN_PALABRAS_DESCRIPCION' => '20',
    'SEO_MAX_PALABRAS_DESCRIPCION' => '26',
    'SEO_MIN_CARACTERES_CUERPO' => '1500',
    'SEO_MAX_CARACTERES_CUERPO' => '30000',
    'SEO_MIN_PALABRAS_CUERPO' => '250',
    'SEO_MAX_PALABRAS_CUERPO' => '5000',
    'SEO_MIN_CARACTERES_SLUG' => '4',
    'SEO_MAX_CARACTERES_SLUG' => '90',
    'SNP_APP_MODAL_SHOW' => 'NO',
    'SNP_APP_MODAL_DELAY' => '1',
    'SNP_APP_MODAL_ICON_IMG' => '/prontus/cpan/imag/logo/prontus-logo-40x40w.png',
    'SERVER_BD' => '',
    'NOM_BD' => '',
    'USER_BD' => '',
    'PWD_BD' => '',
    'TAXPORT_MAXARTICS' => 500,
    'TAXPORT_MAX_WORKERS' => 4,
    'TAXPORT_TIPO_PAGINACION' => '0',
    'TAXPORT_PAGCORTA_MAXPAGS' => '5',
    'TAXONOMIA_NIVELES' => 1,
    'TAXPORT_DINAMICAS' => 'NO',
    'ARTIC_ACTUALIZA_PORTS' => 'NO',
    'TAGPORT_MAXARTICS' => 500,
    'TAGPORT_MAXARTICS' => 1000,
    'TAGPORT_ARTXPAG' => 20,
    'INCLUIR_PRONTUS_ID_URL' => 'NO',
    'LIST_PROCESO_INTERNO' => 'SI',
    'LIST_PORT_PPROC' => 'NO',
    'LIST_MAXARTICS' => 20,
    'LIST_MAXARTICS' => 1000,
    'TAGPORT_ORDEN' => 'art_fechaphorap desc, art_id desc',
    'LIST_ORDEN' => 'art_fechaphorap desc, art_id desc',
    'TAXPORT_ORDEN' => 'art_fechaphorap desc, art_id desc',
    'FFMPEG_AVAILABLE' => 0,
    'DIR_FFMPEG' => '',
    'FFMPEG_AVAILABLE' => 1,
    'FORMATO_HTML_VTXT' => 'HTML',
    'VTXT_ENCODE_CHARS' => 'SI',
    'VTXT_PRONTUS_CARDS' => 'NO',
    'VTXT_IMAGE_SIZE_LIMIT' => '0',
    'ENABLE_SPELL_CHECKER' => 'NO',
    'REGENERACION_TAX_TAG' => 'SI',
    'OIDC_HABILITADO' => 'NO',
    'OIDC_TITULAR' => '',
    'OIDC_ENDPOINT_URI' => '',
    'OIDC_CLIENT_ID' => '',
    'OIDC_APP_SECRET' => '',
    'ACTUALIZACIONES' => 'SI',
    'VTXT_RELPATH_LINK' => 'SI',
    'ABRIR_FIDS_NEW_TAB' => 'SI',
    'USAR_PUBLIC_SERVER_NAME_VER_ARTIC' => 'NO',
    'NUM_RELAC_DEFAULT' => 5,
    'TAXPORT_ARTXPAG' => 20,
    'PORT_AREA_MAX_ARTICS' => '',
    'EDICBASE_INI_SELECTED' => 'NO',
    'SERVER_SMTP' => '',
    'SERVER_PROTOCOLO_FRONTEND' => 'http',
    'SERVER_PROTOCOLO_CPAN' => 'http',
    'DEFAULT_LANGUAGE' => 'es',
    'FORM_CSV_CHARSET' => 'utf-8',
    'SEO_ACTIVO' => 'NO',
    'SEO_MV' => 'NO',
    'HABILITAR_PLUGINS' => 'NO',
    'FORM_INCLUIR_ADJUNTO' => 'SI',
    'SCRIPT_QUOTA' => '',
    'PLAYER_EMBED_VIDEO' => '/prontus/front/player/video.html',
    'PLAYER_EMBED_AUDIO' => '/prontus/front/player/audio.html',
    'FOTO_MAX_PIXEL' => '2560x1440',
    'REDUCIR_CALIDAD_JPEGS' => 'NO',
    'NIVEL_OPTIMIZACION_JPG' => '85',
    'MAX_XCODING' =>  512,
    'BLOQUEO_EDICION' => 0,
    'USAR_LIB_FDK' => 'NO',
    'UPDATE_SERVER' => 'https://www.prontus.cl',
    'FACEBOOK_POST' => 'NO',
    'FB_PAGE_ID' => '',
    'FB_APP_ID' => '',
    'FB_APP_SECRET' => '',
    'FB_ACCESS_TOKEN' => '',
    'FB_PICTURE_WALL' => '',
    'TWITTER_POST' => 'NO',
    'TT_ID' => '',
    'TT_CONSUMER_KEY' => '',
    'TT_CONSUMER_SECRET' => '',
    'TT_AUTH_TOKEN' => '',
    'TT_AUTH_TOKEN_SECRET' => '',
    'TT_BITLY_USER' => '',
    'TT_BITLY_KEY' => '',
    'TT_GOOGL_USER' => '',
    'TT_GOOGL_KEY' => '',
    'TT_PICTURE_WALL' => '',
    'YOUTUBE_POST' => 'NO',
    'YOUTUBE_APPNAME' => '',
    'YOUTUBE_CLIENT_ID' => '',
    'YOUTUBE_CLIENT_SECRET' => '',
    'YOUTUBE_AUTHORIZATION_CODE' => '',
    'YOUTUBE_REFRESH_TOKEN' => '',
    'YOUTUBE_OAUTH_TOKEN' => '',
    'S3_BUCKET_SIZE' => 0,
    'S3_HABILITAR' => 'NO',
    'S3_ANTIGUEDAD_MINIMA' => '1',
    'S3_ENDPOINT' => '',
    'S3_ACCESS_KEY' => '',
    'S3_SECRET_KEY' => '',
    'S3_BUCKET_NAME' => '',
    'SITEMAP_NRO_MAX_PAGS' => 1000,
    'SITEMAP_NRO_MAX_NEWS' => 200,
    'SITEMAP_ARTICULOS' => "",
    'SITEMAP_ARTICULOS_NEWS' => "",
    'SITEMAP_PORTADAS' => "",
    'SITEMAP_NEWS_IMG_PORT' => "",
    'SITEMAP_NEWS_IMG_ART' => "",
    'SITEMAP_NEWS_CAPTION_PORT' => "",
    'SITEMAP_NEWS_CAPTION_ART' => "",
    'SITEMAP_SITENAME' => "",
    'SITEMAP_PAGS' => "NO",
    'SITEMAP_PAGS_SEPARAR' => "NO",
    'SITEMAP_NEWS' => "NO",
    'AD_ENABLE' => "NO",
    'AD_INFO_MSG' => " La publicidad termina en __TIME__ segundos.",
    'AD_TAG' => "");

our $DEFAULT_LANGUAGE = 'es';
our %PROFILES_LOADED = ();

# variables internas
our $loaded = 0;

sub init {
    # variable ya se inicializó una vez
    if ($loaded) {
        return;
    }
    $loaded = 1;

    &set_info_version_prontus();

    # Maximo de articulos no publicados a desplegar en la lista de articulos
    # valor por defecto.
    $prontus_varglb::MAX_NRO_ARTIC = 50;

    %prontus_varglb::BD_CONN = ();

    # Determinar Dir del server.
    $prontus_varglb::DIR_SERVER = &get_dir_server();

    # MAXIMO DE PROCESOS DE CRON TAXPORT SIMULTANEOS AL REGENERAR
    $prontus_varglb::TAXPORT_MAX_MASTERS = 3;

    %prontus_varglb::USERS = ();
    %prontus_varglb::USER_DATA = ();
    %prontus_varglb::PROFILES = ();


    # id del usuario logeado
    $prontus_varglb::USERS_ID = '';
    # nombre de usuario logeado
    $prontus_varglb::USERS_USR = '';
    # Mostrar en la adm. de articulos y ediciones las xx ultimas ediciones, ordenadas por timestamp
    $prontus_varglb::NRO_EDICS_WORK = 15;

    #   <NOTAS>
    # <![CDATA[
    #
    # There are 5 predefined entity references in XML:
    # Regla uno (R1):
    # &lt; < less than
    # &gt; > greater than
    # &amp; & ampersand
    # &apos; ' apostrophe
    # &quot; " quotation mark
    # ]]>
    #
    #   </NOTAS>
    #

    # "<?xml version='1.0' encoding='utf-8'?>
    # <port_data>
    #      <rowartic>
    #        <file></file>
    #        <area></area>
    #        <ord></ord>
    #        <vb></vb>
    #        <pub></pub>
    #      </rowartic>
    # </port_data>";
    return '';
}

sub get_dir_server {
    # DIR_SERVER:
    #   En unix queda algo asi como /sites/misitio.cl/web
    my $dir_server = '';
    $dir_server = $ENV{'DOCUMENT_ROOT'} if (defined($ENV{'DOCUMENT_ROOT'}));
    if ($dir_server eq '') {
        # si hay modperl el directorio ya se determino antes
        if (defined($Prontus::modperl)) {
            return $My::documentRoot;
        }
        # para linea de comandos
        use FindBin qw($Bin);
        $dir_server = $Bin; # por ej /sites/misitio.cl/web/cgi-cpn
        # /sites/misitio.cl/web es document root, se asume que el script que use este varglb siempre
        # estara en la carpeta cgi-xxx o similar, o en una subcarpeta desde la raiz del servidor
        # eliminamos desde cgi-xxx en adelante, cgi-bin o cgi-cpn
        $dir_server =~ s/\/cgi\-\w+.*//;
        # para pruebas unitarias, elimina directorio unit-tests
        $dir_server =~ s/\/unit\-tests.*//;
    }

    # para eliminar los slash del final
    $dir_server =~ s/\/+$//;

    if ((! -d $dir_server) || (!$dir_server)) {
        if ($ENV{'SERVER_NAME'} ne '') { # ambiente web
            print "Content-Type: text/html\n\n";
        }
        print '[Error] DOCUMENT ROOT not detected or invalid.';
        exit;
    }

    return $dir_server;
}

sub set_info_version_prontus {
    my $version = substr($prontus_varglb::VERSION_PRONTUS, 0, index($prontus_varglb::VERSION_PRONTUS, ' '));

    my @aux = split(/\./, $version);
    $RAMA_INSTALADA = "$aux[0].$aux[1]";
    $NRO_REVISION_INSTALADA = $aux[2];
    if ($aux[4]) {
        $BETA_REVISION_INSTALADA = ".$aux[3].$aux[4]";
    } else {
        $BETA_REVISION_INSTALADA = ".$aux[3]";
    }
    return 1;
}

1;
