#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Ejecuta tareas de ajustes necesarias despúes de actualizar prontus
# requeridas por nuevas funciones implementadas
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Por linea de comando
# ---------------------------------------------------------------

# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use utf8;
use strict;

BEGIN {
    use FindBin qw($Bin);
    unshift(@INC,$Bin); # Para dejar disponibles las librerias
    my $pathLibsProntus = $Bin;
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_prontus;
use lib_setbd;
use lib_tags;
use lib_tax;
use lib_utils;
use lib_lang;

main: {
    print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "] \n";

    my $base_dir = $prontus_varglb::DIR_SERVER;
    my @prontus_dirs = glib_fildir_03::lee_dir($base_dir);

    my $prontus_count = scalar @prontus_dirs;
    my @conf_type = ('art.cfg',
        'port.cfg',
        'bd.cfg',
        'tax.cfg',
        'tag.cfg',
        'list.cfg',
        'usr.cfg',
        'var.cfg',
        'xcoding.cfg',
        'dropbox.cfg',
        'cloudflare.cfg',
        'cache.cfg',
        'sitemap.cfg',
        'pplayer.cfg',
        'redes.cfg',
        'oidc.cfg',
        'snp.cfg',
        'elasticsearch.cfg',
        'seo.cfg');

    print STDERR "Prontus count [$prontus_count] ". join(';', @prontus_dirs). "\n";
    # Iterar por cada prontus
    foreach my $prontus_dir (@prontus_dirs) {
        next if (substr($prontus_dir, 0, 1) eq '.');
        next if (!-d "$base_dir/$prontus_dir");
        print STDERR "$prontus_dir\n";
        my $path_cfg = $base_dir. "/$prontus_dir/cpan/$prontus_dir";
        if (-f "$path_cfg.cfg") {
            print STDERR "Found [$path_cfg.cfg]\n";
            next;
        }

        my %config;
        foreach my $type (@conf_type) {
            my $buffer = glib_fildir_03::read_file("$path_cfg-$type");
            if ($buffer ne '') {
                $buffer = "[$type]\n$buffer\[/$type]";
                $config{$type} = $buffer;
            }
        }

        my $final_conf = '';
        foreach my $key (sort keys %config) {
            $final_conf .= $config{$key}. "\n";
        }

        if ($final_conf) {
            print STDERR "$final_conf\n";
            glib_fildir_03::write_file("$path_cfg.cfg", $final_conf);
        }
    }
}
