#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use glib_cgi_05;
use glib_fildir_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_lang;
use lib_parser;
use lib_setbd;
use lib_utils;
use wizard_lib;
##/libs##

our $CRLF = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

main:{
    lib_utils::setUp();
    glib_cgi_05::new();
    my ($msg_err,
        $prontus_id,
        $extension,
        $smtp,
        $model_nom,
        $new_title_site_name,
        $server_bd,
        $nom_bd,
        $user_bd,
        $pwd_bd,
        $superuser_bd,
        $superpwd_bd) = &check_paso_anterior_deploy();

    my $lnk_paso1 = '<br><br><a href="/wizard_prontus/core/paso1.html">'.lib_lang::get_message('wizard_deploy_back_step_1').'</a><br><br>';

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err . $lnk_paso1, 0, 1, 1, "nolink");
    }

    my ($err_basedatos, $err_postprocesos) = &deploy($prontus_id, $extension, $smtp, $model_nom, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);

    if ($err_basedatos ne '') {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $err_basedatos . "<br><br>".lib_lang::get_message('wizard_deploy_error_db')."<br><br>$lnk_paso1", 0, 1, 1, "nolink");
    }

    if ($err_postprocesos ne '') {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('wizard_deploy_warning_label'), $err_postprocesos . "<br><br>".lib_lang::get_message('wizard_deploy_error_postproc')."<br><br>".lib_lang::get_interpolated_message('wizard_deploy_link_postproc', $prontus_id)."<br><br>", 0, 1, 1, "nolink");
    }

    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/wizard/wizard_reporte.cgi?DIR=/$prontus_id&TOPE=/\n\n";
}

sub deploy {
    my ($prontus_id, $extension, $smtp, $model_nom, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd) = @_;

    # Crea la BD si es que se indicaron los datos y si es que no existe.
    my $err_createdb;
    if (($superuser_bd ne '') && ($superpwd_bd ne '')) {

        # ver si existe
        my @databases = DBI->data_sources("mysql", {"host" => $server_bd, "user" => $superuser_bd, "password" => $superpwd_bd});
        my $existe_base = 0;
        foreach my $k (@databases) {
            if ($k eq $nom_bd) {
                $existe_base = 1;
                last;
            }
        }

        # si no existe, crearla
        if (!$existe_base) {
            # crear BD
            my $drh = DBI->install_driver('mysql') || warn lib_lang::get_interpolated_message('wizard_deploy_error_db_handler', $DBI::err, $DBI::errstr);
            my $error_creating_db = lib_lang::get_interpolated_message("wizard_deploy_error_creating_db", $nom_bd, $DBI::err, $DBI::errstr);
            if ($DBI::err) {
                $err_createdb = $error_creating_db;
            } else {
                my $rc = $drh->func("createdb", $nom_bd, $server_bd, $superuser_bd, $superpwd_bd, 'admin') || warn lib_lang::get_interpolated_message("wizard_deploy_error_createdb", $DBI::err, $DBI::errstr);
                $err_createdb = $error_creating_db if ($DBI::err);
            }

            return ($err_createdb, '') if ($err_createdb ne '');
            # Si llega hasta aca, es porque se ha creado ok.
        }

        # Si se especifico superusuario, haya existido o no la bd, igual regulariza el user y el charset

        # Conectarse a la bd recien creada, para asignar user y setear charset
        my ($base_admin, $conn_result_admin) = lib_bd::conectar_prontus_bd($nom_bd, $server_bd, $superuser_bd, $superpwd_bd,);
        return ($conn_result_admin, '') if ($conn_result_admin ne '');

        # asignar el usuario, previa conn normal a la BD
        my $sql_grant = "grant all privileges on $nom_bd.* to '$user_bd'\@'$server_bd' identified by '$pwd_bd'";
        $base_admin->do($sql_grant)
                     || warn lib_lang::get_interpolated_message("wizard_deploy_error_grant", $DBI::err, $DBI::errstr, $sql_grant);
        return (lib_lang::get_interpolated_message("wizard_deploy_error_privileges", $DBI::err, $DBI::errstr), '') if ($DBI::err);

        # asignar el charset, previa conn normal a la BD
        $base_admin->do("alter database $nom_bd CHARACTER SET=utf8mb4 COLLATE=utf8mb4_unicode_ci")
                     || warn lib_lang::get_interpolated_message("wizard_deploy_error_charset", $DBI::err, $DBI::errstr);
        return (lib_lang::get_interpolated_message("wizard_dbi_error_charset_public", $DBI::err, $DBI::errstr), '') if ($DBI::err);

        $base_admin->disconnect;
    }

    # Intenta conectarse a la BD existente, si no resulta, aborta todo.
    my ($base, $conn_result) = lib_bd::conectar_prontus_bd($nom_bd, $server_bd, $user_bd, $pwd_bd);
    return ($conn_result, '') if ($conn_result ne '');

    # En este punto, se supone que ya existe la BD y esta creada la conn,
    # Por tanto ya se pueden crear las tablas.
    # Si ya existen, no crea nada y arroja error, puede significar que se esta usando la misma BD para 2 prontuses!

    $prontus_varglb::PRONTUS_ID = $prontus_id;
    lib_utils::init_db_table_names($prontus_varglb::PRONTUS_ID);
    my ($msg_ret, $hay_err);
    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_secc($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_temas($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_subtemas($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_tags($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_tagsart($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_asset($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_art($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_art_publicar($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_art_despublicar($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_urlf5($base, $prontus_varglb::URL_MAX_LENGTH);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err && $msg_ret ne 'db_exists_urlf5');

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_s($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_t($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_st($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_art_s($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_art_t($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_multitag_art_st($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_users($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    # insertar usuario admin
    my $users_table = lib_utils::get_table('users');
    my $sql = "insert into $users_table (users_id, users_nom, users_usr, users_psw, users_profile_id, users_email, users_exp_days, users_fec_exp, users_renew_psw, users_2fa, users_oidc, users_deleted) values(1, 'Administrador', 'admin', '', 1, '', 0, 0, 0, 0, 0, 0)";
    my $res = $base->do($sql);
    if (!$res) {
        return (lib_lang::get_interpolated_message("wizard_deploy_error_updating_table", 'USERS', $sql, $!), '');
    }

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_perfiles($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_old_psw($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_port_users($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_art_users($base);
    return (lib_lang::get_message($msg_ret), '') if ($hay_err);

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_prontus_users($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = lib_setbd::crear_tabla_oidc($base);
    return ($msg_ret, '') if ($hay_err ne '');

    $base->disconnect;

    my $dir_models = "$prontus_varglb::DIR_SERVER/wizard_prontus/models";
    my $path_model = "$dir_models/$model_nom";

    # copiar /wizard_prontus/models/<modelo> --> /<prontus_id>
    glib_fildir_03::copy_tree($dir_models, $model_nom, $prontus_varglb::DIR_SERVER, $prontus_id);

    my $path_to_cfg = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$model_nom/$model_nom.cfg";

    # Existencia de cfg del modelo
    if (!-f $path_to_cfg) {
        return ('', lib_lang::get_interpolated_message('wizard_deploy_error_model_config', $model_nom));
    }

    # Extension de paginas del modelo
    my $buffer_cfg = glib_fildir_03::read_file($path_to_cfg);

    # Rescata nom del prontus_base
    my $prontus_base = $model_nom;
    if ($buffer_cfg =~ /PRONTUS_BASE\s*=\s*["']([^'"]+)["']/) {
        $prontus_base = $1;
    } else {
        return ('', lib_lang::get_message('wizard_deploy_error_config_missing'));
    }

    # No copiar el id, lo crea la aplicacion
    # al copiar la carpeta completa del modelo ya se copiaron los cfg, ahora solo hay que renombrarlos.
    File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base.cfg", "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id.cfg");

    # Agrega nuevas configuraciones
    my %conf = ('SERVER_SMTP' => $smtp,
        'USER_BD' => $user_bd,
        'NOM_BD' => $nom_bd,
        'SERVER_BD' => $server_bd,
        'PWD_BD' => $pwd_bd,
        '_new_prontus_id' => $prontus_id);

    &replace_prontus_cfg($prontus_id, \%conf);

    # Cambia referencias
    &cambia_referencias("$prontus_varglb::DIR_SERVER/$prontus_id", $prontus_base, $prontus_id, $new_title_site_name);

    # Elimina archivos de usuario
    my $dir2del = "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/users";
    if (glib_fildir_03::check_dir($dir2del)) {
        my @lisdir = glib_fildir_03::lee_dir($dir2del);
        foreach my $k (@lisdir) {
            next if ($k eq '.' or $k eq '..');
            unlink "$dir2del/$k" if (-f "$dir2del/$k");
        }
    }

    # Elimina archivos del modelo, utiles solo para el wizard
    my $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.cfg";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base-big.png";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base-thumb.png";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.tgz";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.tgz.md5";
    unlink $file2del if (-f $file2del);

    # Renombra carpeta de indices del buscador
    if (-d "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_base") {
        File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_base",
        "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_id");
    }

    # Borra cache de no publicados
    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/cache");

    # Regenerar tabla de articulos, para que se pueble en base a los XML
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_regenerabd_real.pl $prontus_id";
    return ('', lib_lang::get_interpolated_message('wizard_deploy_error_populating_table', 'de artículos')) if ($? != 0);

    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/dam/prontus_dam_regen_real.pl $prontus_id";
    return ('', lib_lang::get_interpolated_message('wizard_deploy_error_populating_table', 'multimedia')) if ($? != 0);

    # Importa XML de tax que venia en el modelo en la carpeta $path_model/cpan/procs/tax_import.xml y que se copio al prontus re100 instalado
    if (!-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/procs/tax_import.xml") {
        return ('', lib_lang::get_message("wizard_deploy_error_importing_categories").lib_lang::get_interpolated_message("wizard_deploy_error_file_not_found" , "/$prontus_id/cpan/procs/tax_import.xml"))
    }

    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_tax_import_real.pl $prontus_varglb::DIR_SERVER $prontus_id";
    return ('', lib_lang::get_message("wizard_deploy_error_importing_categories")) if ($? != 0);

    # importar tags del modelo
    if (-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/procs/tags_import.xml") {
        system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_tags_import_real.pl $prontus_varglb::DIR_SERVER $prontus_id";
    }

    # importar multitags del modelo
    if (-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/procs/multitags_import.xml") {
        system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/multitag/prontus_multitag_import_real.pl $prontus_id";
    }

    # importar urlsf5 del modelo
    if (-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/procs/urlf5_db.csv") {
        system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/urlf5/prontus_urlf5_db_export_import.pl $prontus_id import";
        # regenera links
        system "perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/utils/prontus_urlf5_regen.pl $prontus_id all";
    }

    # Regenerar articulos.
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_art_regen_real.pl $prontus_id";
    return ('', lib_lang::get_message('wizard_deploy_error_regen_art')) if ($? != 0);

    # Regenerar portadas.
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_port_regen_real.pl $prontus_id";
    return ('', lib_lang::get_message('wizard_deploy_error_regen_port')) if ($? != 0);

    return ('', '');
}

# Cambia sintaxis de includes a plantillas y site en prontus destino
sub cambia_referencias {
    my ($dir, $prontus_base, $prontus_id, $new_title_site_name) = @_;

    if (-d $dir) {
        # Abre directorio.
        opendir(DIR, $dir) || die "Can't opendir" . $dir . $!;
        my @entries = readdir(DIR);
        closedir DIR;

        my $new_tsn = lib_encodes::escape_xml($new_title_site_name);
        foreach my $entry (@entries) {
            if (($entry ne '.') and ($entry ne '..')) {
                if (-d "$dir/$entry") {
                    &cambia_referencias("$dir/$entry", $prontus_base, $prontus_id, $new_title_site_name);
                } else {
                    $entry =~ /\.(\w+)$/;
                    my $ext = $1;
                    if ($ext eq 'html' || $ext eq 'shtml' || $ext eq 'php' || $ext eq 'asp' || $ext eq 'xml' || $ext eq 'js' || $ext eq 'css' || $ext eq 'idx') {
                        my $buffer = glib_fildir_03::read_file("$dir/$entry");
                        $buffer =~ s/$CRLF/\x0a/sg;

                        # Cambia nombre del publicador de origen por el que escogio el usuario
                        $buffer =~ s/%%new_prontus_id%%/$prontus_id/ig;

                        # CAMBIA SITE NAME
                        $buffer =~ s/%%new_site_name%%/$new_tsn/ig;

                        glib_fildir_03::write_file("$dir/$entry", $buffer);
                    }
                }
            }
        }
    }
}

# Agrega texto al cfg del publicador destino.
sub replace_prontus_cfg {
    my $prontus_id = $_[0];
    my %options = %{$_[1]};

    my $cfg_file = "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id.cfg";
    my $buffer_cfg = glib_fildir_03::read_file($cfg_file);

    foreach my $key (keys %options) {
        $buffer_cfg =~ s/##$key##/$options{$key}/g;
    }

    glib_fildir_03::write_file($cfg_file, $buffer_cfg);
}

# Chequea que se haya pasado por el paso 1
sub check_paso_anterior_deploy {
    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    if (!-f $INF_FILE) {
        return lib_lang::get_message("wizard_error_invalid_request");
    }

    # Leer y cargar y validar contenido del INF.
    my $buffer = glib_fildir_03::read_file($INF_FILE);
    my ($prontus_id, $extension, $smtp);
    my $new_title_site_name; # del segmento PRONTUS (ingresado por el usuario)
    my ($server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);

    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
            return lib_lang::get_interpolated_message('wizard_deploy_error_corrupt_data_1', 0);
        }

        # smtp
        if ($buffer_prontus =~ /SERVER_SMTP=([\w\.\-]+)\n/) {
            $smtp = $1;
        }

        if ($buffer_prontus !~ /NEW_TITLE_SITE_NAME=(.+?)\n/) {
            return lib_lang::get_interpolated_message('wizard_deploy_error_corrupt_data_1', 1);
        } else {
            $new_title_site_name = $1;
        }

        $server_bd = $1 if ($buffer_prontus =~ /server_bd=(.*?)\n/i);
        $nom_bd = $1 if ($buffer_prontus =~ /nom_bd=(.*?)\n/i);
        $user_bd = $1 if ($buffer_prontus =~ /user_bd=(.*?)\n/i);
        $pwd_bd = $1 if ($buffer_prontus =~ /pwd_bd=(.*?)\n/i);
        $superuser_bd = $1 if ($buffer_prontus =~ /superuser_bd=(.*?)\n/i);
        $superpwd_bd = $1 if ($buffer_prontus =~ /superpwd_bd=(.*?)\n/i);
    } else {
        return lib_lang::get_interpolated_message('wizard_deploy_error_corrupt_data_1', 2);
    }

    lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_deploy_install", $prontus_id));

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return lib_lang::get_interpolated_message("wizard_error_dir_exists", "[/$prontus_id]");
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (glib_fildir_03::check_dir($dir_prontus)) {
          glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return lib_lang::get_interpolated_message("wizard_dir_not_created_int", $prontus_id);
        }
    }

    # valida q se haya pasado por paso2
    my $model_nom;
    if ($buffer =~ /(\[MODEL\].*\[\/MODEL\])/s) {
        my $buffer_model = $1;
        # Validar id
        if ($buffer_model !~ /MODEL_NOM=(\w+)\n/) {
            return lib_lang::get_interpolated_message('wizard_deploy_error_corrupt_data_2', 0);
        } else {
            $model_nom = $1;
        }

        if ($buffer_model !~ /MODEL_EXT=(\w+)\n/) {
            return lib_lang::get_message('wizard_deploy_error_ext');
        } else {
            $extension = $1;
        }
    } else {
        return lib_lang::get_interpolated_message('wizard_deploy_error_corrupt_data_2', 1);
    }

    return ('', $prontus_id,$extension, $smtp, $model_nom, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
}
