#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::MD5;
use File::Copy;
use HTTP::Request;
use JSON;
use LWP::UserAgent;
use POSIX;
use glib_fildir_03;
use lib_lang;
use lib_prontus;
use lib_utils;
use wizard_lib;
##/libs##

our $STATUS_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/progress.txt";
our ($TOTAL_DOWNLOADED, $PROGRESS, $REMOTE_FILE_SIZE) = (0, 0, 0);

# Este script corre en CLI, e intenta descargar el modelo cuyo nombre se le pasa como primer argumento.
# Escribe el resultado en el archivo "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/progress.txt".
main:{
    lib_utils::setUp();
    my $modelid = $ARGV[0];

    # En el caso que no exista el directorio de los modelos
    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$wizard_lib::MODELS_DIR");

    # Se descarga el modelo
    my $mensaje = &descarga_modelo($modelid);

    if ($mensaje) {
        glib_fildir_03::write_file($STATUS_FILE, $mensaje);
    }

    print STDERR lib_lang::get_interpolated_message("wizard_models_download_real_msg", $mensaje);
}

sub descarga_modelo {
    my $modelid = $_[0];
    my $dirmodel = "$prontus_varglb::DIR_SERVER$wizard_lib::MODELS_DIR/$modelid";

    # Descarga tgz
    my $url = $wizard_lib::URL_MODELS . '/' . $modelid . '/' . $modelid . '.tgz';

    $REMOTE_FILE_SIZE = &get_remote_file_size($url);

    print STDERR lib_lang::get_interpolated_message("wizard_models_download_rfs", $REMOTE_FILE_SIZE);

    if (!$REMOTE_FILE_SIZE) {
        return lib_lang::get_interpolated_message("wizard_models_download_real_size_0", $url);
    }

    my ($tgz_content, $msg_err) = &get_url($url, 30);

    if ($msg_err) {
        if ($msg_err =~ /^404 /) {
            return lib_lang::get_interpolated_message("wizard_models_download_real_404", $url);
        } else {
            return lib_lang::get_interpolated_message("wizard_models_download_real_error", $url, $msg_err);
        }
    } else {
        # Si ok el tgz, descarga md5
        my $url_md5 = $url . ".md5";
        my ($md5_remoto, $msg_err_md5) = &get_url($url_md5, 10);
        if ($msg_err_md5) {
            if ($msg_err_md5 =~ /^404 /) {
                return lib_lang::get_interpolated_message("wizard_models_download_real_md5_404", $url_md5);
            } else {
                return lib_lang::get_interpolated_message("wizard_models_download_real_md5_error", $url_md5, $msg_err_md5);
            }
        }
        if ($md5_remoto =~ /([a-z0-9]{32})/) {
            $md5_remoto = $1;
        } else {
            return lib_lang::get_interpolated_message("wizard_models_download_real_md5_invalid", $url_md5, $md5_remoto);
        }

        # Si descargo ok el md5, verificar el tgz con este
        my $md5_local = Digest::MD5::md5_hex($tgz_content);

        if ($md5_local ne $md5_remoto) {
            return lib_lang::get_interpolated_message("wizard_models_download_real_md5_failed_match", $url, $md5_local, $md5_remoto);

        }

        # Si verificacion ok, guardar tgz y descomprimir en ubicacion de destino
        my $dirdownload = "$prontus_varglb::DIR_SERVER$wizard_lib::DOWNLOAD_DIR";
        return lib_lang::get_message("wizard_models_download_real_dl_dir") if (! glib_fildir_03::check_dir($dirdownload));
        my $path_local_tgz = "$dirdownload/download" . $modelid . '.tgz';
        glib_fildir_03::write_file_bin($path_local_tgz, $tgz_content);

        if (! (-s $path_local_tgz) || ! (-f $path_local_tgz)) {
            return lib_lang::get_interpolated_message("wizard_models_download_real_dl_release", $url, $msg_err);
        }

        # guardar el .md5 tb, por siaca
        glib_fildir_03::write_file("$path_local_tgz.md5", $md5_remoto);

        # Creamos un directorio al azar
        my $randomdir = int(rand(1000000));
        $randomdir = sprintf('%06d', $randomdir);

        return lib_lang::get_message("wizard_models_download_real_dl_dir") if (! glib_fildir_03::check_dir("$dirdownload/$randomdir"));

        # descomprimir en el mismo dir, el .tgz
        system("tar xzf $path_local_tgz -C $dirdownload/$randomdir");

        if ($? != 0) {
            glib_fildir_03::borra_dir("$dirdownload/$randomdir");
            return lib_lang::get_interpolated_message("wizard_models_download_real_decompress_error", $path_local_tgz, "$dirdownload/$randomdir", $!);
        }

        print STDERR lib_lang::get_interpolated_message("wizard_models_download_real_decompress", "$dirdownload/$randomdir");

        # Se copia y se borra lo descargado
        my $dir_destino = "$prontus_varglb::DIR_SERVER$wizard_lib::MODELS_DIR";
        glib_fildir_03::copy_tree($dirdownload, $randomdir, $dir_destino, $modelid);

        # Descarga el resto de los componentes
        my $resp = &wizard_lib::descarga_componente($modelid, "$modelid.cfg");
        return $resp if($resp);
        $resp = &wizard_lib::descarga_componente($modelid, "$modelid-thumb.png", 'bin');
        return $resp if($resp);
        &wizard_lib::descarga_componente($modelid, "$modelid-big.png", 'bin');
        return $resp if($resp);
        &wizard_lib::descarga_componente($modelid, "release_notes.txt");
        return $resp if($resp);

        glib_fildir_03::borra_dir("$dirdownload/$randomdir");

        # En este punto el modelo ya fue descargado y descomprimido
        unlink("$path_local_tgz");
        unlink("$path_local_tgz.md5");
    }
    return '';
}

sub get_url {
    my ($url, $espera_segs, $type) = @_;

    $espera_segs = 30 if ($espera_segs eq '');
    $type = 'GET' if ($type eq '');

    return '' if ($url eq '');
    my $ua = new LWP::UserAgent;

    $ua->timeout($espera_segs); # segs # default es 180
    $ua->agent('Prontus CMS'); # default es "libwww-perl/#.##"
    $ua->add_handler('response_data' => \&callback);

    my $request = new HTTP::Request($type, $url) || return ('', $!);
    my $response = $ua->request($request) || return ('', $!);

    if ($response->is_success) {
        return ($response->content, '');
    } else {
        return ('', $response->status_line);
    }
}

sub callback {
    my ($response, $ua, $h, $data) = @_;
    $TOTAL_DOWNLOADED += length $data;
    #print STDERR "TOTAL_DOWNLOADED[$TOTAL_DOWNLOADED]\n";

    $PROGRESS = ($TOTAL_DOWNLOADED * 100) / $REMOTE_FILE_SIZE;
    $PROGRESS = POSIX::ceil($PROGRESS);
    $PROGRESS = 100 if ($PROGRESS > 100);

    #print STDERR "PROGRESS[$PROGRESS]\n";

    glib_fildir_03::write_file($STATUS_FILE, $PROGRESS);

    return 1;
}

sub get_remote_file_size {
    my $url = shift;
    my $ua = new LWP::UserAgent;
    $ua->agent("Prontus CMS");
    my $req = new HTTP::Request 'HEAD' => $url;
    $req->header('Accept' => 'text/html');
    my $res = $ua->request($req);

    if ($res->is_success) {
        my $headers = $res->headers;
        my $length = $headers->content_length;
        return $length;
    } else {
        return 0;
    }
}
