#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use glib_cgi_05;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_utils;
use wizard_lib;
##/libs##

# ---------------------------------------------------------------
# MAIN.
# ------
our %FORM = ();

main:{
    lib_utils::setUp();
    glib_cgi_05::new();

    $FORM{'PRONTUS_ID'} = glib_cgi_05::param('PRONTUS_ID');
    $FORM{'SERVER_BD'} = glib_cgi_05::param('SERVER_BD');
    $FORM{'NOM_BD'} = glib_cgi_05::param('NOM_BD');
    $FORM{'USER_BD'} = glib_cgi_05::param('USER_BD');
    $FORM{'PWD_BD'} = glib_cgi_05::param('PWD_BD');
    $FORM{'SUPERUSER_BD'} = glib_cgi_05::param('SUPERUSER_BD');
    $FORM{'SUPERPWD_BD'} = glib_cgi_05::param('SUPERPWD_BD');
    $FORM{'PRONTUS_SMTP'} = glib_cgi_05::param('PRONTUS_SMTP');
    $FORM{'NEW_TITLE_SITE_NAME'} = glib_cgi_05::param('NEW_TITLE_SITE_NAME');

    $FORM{'action'} = glib_cgi_05::param('action');

    my $msg_err = &validate_wizard_1();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    &grabar_datos_paso1();

    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/wizard/wizard_show_paso2.cgi\n\n";
}

# Graba la info del paso 1 en el arch. de texto /wizard_prontus/data/inf.txt, seccion [PRONTUS]...[/PRONTUS]
# creando el dir. inf si no existe.
sub grabar_datos_paso1 {
    my $buffer;

    $buffer = "[PRONTUS]\n"
            . "PRONTUS_ID=$FORM{'PRONTUS_ID'}\n"
            . "SERVER_BD=$FORM{'SERVER_BD'}\n"
            . "NOM_BD=$FORM{'NOM_BD'}\n"
            . "USER_BD=$FORM{'USER_BD'}\n"
            . "PWD_BD=$FORM{'PWD_BD'}\n"
            . "PRONTUS_ID=$FORM{'PRONTUS_ID'}\n"
            . "NEW_TITLE_SITE_NAME=$FORM{'NEW_TITLE_SITE_NAME'}\n";

    $buffer .= "SERVER_SMTP=$FORM{'PRONTUS_SMTP'}\n" if ($FORM{'PRONTUS_SMTP'});

    if (($FORM{'SUPERUSER_BD'} ne '') && ($FORM{'SUPERPWD_BD'} ne '')) {
        $buffer .= "SUPERUSER_BD=$FORM{'SUPERUSER_BD'}\n";
        $buffer .= "SUPERPWD_BD=$FORM{'SUPERPWD_BD'}\n";
    }

    $buffer .= "[/PRONTUS]\n\n";

    my $INF_DIR = "$prontus_varglb::DIR_SERVER/" . wizard_lib::get_inf_dir();
    if (!glib_fildir_03::check_dir($INF_DIR)) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), lib_lang::get_message('wizard_paso1_missing_inf_dir'), 0);
    }

    my $INF_FILE = "$prontus_varglb::DIR_SERVER/" . wizard_lib::get_inf_dir() . '/' . wizard_lib::get_inf_file();
    glib_fildir_03::write_file($INF_FILE, $buffer);
}

# Valida datos provenientes del formulario paso1.
# Retorna msg de error si lo hay o nada si todo ok.
sub validate_wizard_1 {
    if ($FORM{'PRONTUS_ID'} !~ /^[\w\_\-]+$/) {
        return lib_lang::get_message('global_error_invalid_dir');
    }

    if ($FORM{'PRONTUS_ID'} !~ /^[\w\_\-]{1,32}$/) {
        return lib_lang::get_message('wizard_paso1_id_max_length');
    }

    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$FORM{'PRONTUS_ID'}";

    if (-d $dir_prontus) {
        return lib_lang::get_interpolated_message("wizard_error_dir_exists", "Prontus");
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (glib_fildir_03::check_dir($dir_prontus)) {
            glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return lib_lang::get_message("wizard_dir_not_created");
        }
    }

    if ($FORM{'SERVER_BD'} !~ /^[\w\-\.]{1,128}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_server');
    }

    if ($FORM{'NOM_BD'} !~ /^[\w\-]{1,64}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_name');
    }

    if ($FORM{'USER_BD'} !~ /^[\w\-]{1,16}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_user');
    }

    if ($FORM{'PWD_BD'} !~ /^[\w\-\.\@\:\$%!#]{1,64}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_pass');
    }

    if ($FORM{'SUPERUSER_BD'} !~ /^[\w\-]{0,16}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_root_user');
    }

    if ($FORM{'SUPERPWD_BD'} !~ /^[\w\-\.\@\:\$%!#]{0,64}$/) {
        return lib_lang::get_message('wizard_paso1_invalid_db_root_pass');
    }

    if ($FORM{'PRONTUS_SMTP'} ne '') {
        if ($FORM{'PRONTUS_SMTP'} !~ /^[A-Z\.a-z\_\-0-9]+$/) {
            return lib_lang::get_message('wizard_paso1_invalid_smtp');
        }
    }

    if ($FORM{'NEW_TITLE_SITE_NAME'} eq '') {
        return lib_lang::get_message("wizard_paso1_missing_site_name");
    }

    if (lc($FORM{'action'}) !~ /siguiente/) {
        return lib_lang::get_message("wizard_invalid_exec_request");
    }

    return '';
}
