#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use glib_cgi_05;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_utils;
use wizard_lib;
##/libs##

our %FORM = ();
our $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";

main:{
    lib_utils::setUp();
    glib_cgi_05::new();
    $FORM{'PRONTUS_MODEL'} = glib_cgi_05::param('PRONTUS_MODEL');
    $FORM{'action'} = glib_cgi_05::param('action');

    my ($msg_err, $model_ext, $title_site_name) = &validate_wizard_2();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    &grabar_datos_paso_2($model_ext, $title_site_name);

    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/wizard/wizard_show_confirm.cgi\n\n";
};

sub grabar_datos_paso_2 {
    # Graba la info del paso actual en el arch. de texto /wizard_prontus/data/inf.txt, seccion [MODEL]...[/MODEL]
    # creando el dir. inf si no existe.
    my $model_ext = $_[0];
    my $title_site_name = $_[1];
    my $buffer = glib_fildir_03::read_file($INF_FILE);
    # Borra seccion en caso de que exista.
    $buffer =~ s/\[MODEL\].*\[\/MODEL\]//sg;

    # Agrega seccion
    $buffer .= "[MODEL]\nMODEL_NOM=$FORM{'PRONTUS_MODEL'}\nMODEL_EXT=$model_ext\nTITLE_SITE_NAME=$title_site_name\n[/MODEL]\n\n";

    glib_fildir_03::write_file($INF_FILE, $buffer);
};

sub validate_wizard_2 {
    # Valida datos provenientes del formulario PASO 2.
    # Retorna msg de error si lo hay o nada si todo ok.
    if (!$FORM{'PRONTUS_MODEL'}) {
        return lib_lang::get_message('wizard_paso2_invalid_model');
    }

    # Algunas validaciones basicas para el modelo escogido
    my $path_to_cfg = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$FORM{'PRONTUS_MODEL'}/$FORM{'PRONTUS_MODEL'}.cfg";

    # Existencia de cfg del modelo
    if (!-f $path_to_cfg) {
        return lib_lang::get_interpolated_message("wizard_paso2_model_config_not_found", $FORM{'PRONTUS_MODEL'});
    }

    # Extension de paginas del modelo
    my $buffer_cfg = glib_fildir_03::read_file($path_to_cfg);

    return lib_lang::get_interpolated_message("wizard_paso2_error_cfg_missing_ext", $FORM{'PRONTUS_MODEL'}) if ($buffer_cfg !~ /MODEL_EXT\s*=\s*["'](\w+)["']\n/);
    my $model_ext = $1;

    return lib_lang::get_interpolated_message("wizard_paso2_error_cfg_missing_title", $FORM{'PRONTUS_MODEL'}) if ($buffer_cfg !~ /TITLE_SITE_NAME\s*=\s*["'](.*?)["']\n/);
    my $title_site_name = $1;

    my $invalid_req= lib_lang::get_message("wizard_invalid_exec_request");
    if (lc($FORM{'action'}) !~ /siguiente/) {
        return $invalid_req;
    }

    # Leer y cargar y validar contenido del paso1.
    if (!-f $INF_FILE) {
        return "[errInfFile] ".$invalid_req;
    }

    my $buffer = glib_fildir_03::read_file($INF_FILE);
    my $prontus_id;

    my $prev_step_error_msg = lib_lang::get_message('wizard_paso2_prev_info_corrupt');
    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
            return $prev_step_error_msg;
        }
    } else {
        return $prev_step_error_msg;
    }

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return lib_lang::get_interpolated_message("wizard_error_dir_exists", "prontus");
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (glib_fildir_03::check_dir($dir_prontus)) {
            glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return lib_lang::get_message("wizard_paso2_cannot_create_dir");
        }
    }

    return ('', $model_ext, $title_site_name);
}
