#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use glib_cgi_05;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_utils;
use wizard_lib;
##/libs##

our $CRLF = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

# Esta CGI muestra un reporte de la instalación de un Prontus
# recibe dos parámetros por GET o POST:
# @param DIR el nombre del prontus
# @param TOPE el directorio webroot
main: {
    lib_utils::setUp();

    my ($plantilla, $pagina, $lista, $aux, $lnk_cpan);

    glib_cgi_05::new();
    my %FORM;
    $FORM{'DIR'} = glib_cgi_05::param('DIR');
    $FORM{'TOPE'} = glib_cgi_05::param('TOPE');
    $FORM{'TOPE'} = '' if ($FORM{'TOPE'} eq '/');
    $FORM{'DIR'} = '' if ($FORM{'DIR'} eq '/');

    if ($FORM{'DIR'} !~ /^$FORM{'TOPE'}/) {
        $FORM{'DIR'} = $FORM{'TOPE'};
    }

    # borra ultimo slash si es que viene.
    $FORM{'DIR'} =~ s/\/$//;
    # borra ../, no se aceptan directorios relativos
    $FORM{'DIR'} =~ s/\.\.\///g;

    if (!(-d "$prontus_varglb::DIR_SERVER$FORM{'DIR'}")) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), lib_lang::get_message('global_invalid_dir'));
    }

    my ($msgerr, $prontus_id, $extension) = &get_model_data();

    if ($msgerr) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msgerr);
    }

    if ($FORM{'DIR'} !~ /^\/$prontus_id/) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), '[2]' .lib_lang::get_message('global_invalid_dir'));
    }

    $lnk_cpan = "/$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_id";
    # Generar pagina final
    $plantilla = "$prontus_varglb::DIR_SERVER/wizard_prontus/core/reporte.html";
    $pagina = glib_fildir_03::read_file($plantilla);

    $pagina =~ /<!--LOOP-->(.*?)<!--\/LOOP-->/s;
    my $loop = $1;
    $lista = &make_lista_dir("$prontus_varglb::DIR_SERVER$FORM{'DIR'}", $loop, $FORM{'TOPE'});
    $pagina =~ s/<!--LOOP-->(.*?)<!--\/LOOP-->/$lista/sg;

    $pagina =~ s/%%dir_actual%%/\//g if ($FORM{'DIR'} eq '');
    $pagina =~ s/%%dir_actual%%/$FORM{'DIR'}/g if ($FORM{'DIR'} ne '');
    $pagina =~ s/%%dir_sup%%// if ($FORM{'DIR'} eq '');
    $aux = $FORM{'DIR'};
    $aux =~ s/(.*)\/.+$/$1/;
    $pagina =~ s/%%dir_sup%%/\// if ($FORM{'DIR'} eq '');
    $pagina =~ s/%%dir_sup%%/$aux/ if ($FORM{'DIR'} ne '');
    $pagina =~ s/%%tope%%/$FORM{'TOPE'}/;

    if ($FORM{'TOPE'} eq $FORM{'DIR'} || $FORM{'DIR'} eq "/$prontus_id") {
        $pagina =~ s/<!--ATRAS-->.*<!--\/ATRAS-->//s;
    }

    $pagina =~ s/%%lnk_cpan%%/$lnk_cpan/;
    my $lnk_sitio = "/$prontus_id/site/edic/base/port/inicio.$extension";
    $pagina =~ s/%%lnk_sitio%%/$lnk_sitio/g;

    lib_utils::print_html($pagina);
}

# Obtiene extension y otros datos del modelo Prontus instalado.
# lee un archivo de información generado por el wizard.
# @param void
# @return mixed strings (errores) o array ('', $prontus_id, $extension)
sub get_model_data {
    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    if (! -f $INF_FILE) {
        return "[errInfFile] " .lib_lang::get_message('wizard_invalid_exec_request');
    }

    # Leer y cargar y validar contenido del INF.
    my $buffer = glib_fildir_03::read_file($INF_FILE);

    # borra archivo pues ya termino de instalar la instancia
    unlink $INF_FILE;

    # prontus_id en paso1
    my $prontus_id;

    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
          return lib_lang::get_interpolated_message('wizard_corrupt_info', '1', '0');
        }
    } else {
        return lib_lang::get_interpolated_message('wizard_corrupt_info', '1', '1');
    }

    # valida q se haya pasado por paso2
    my $extension;

    if ($buffer =~ /(\[MODEL\].*\[\/MODEL\])/s) {
        my $buffer_model = $1;

        if ($buffer_model =~ /MODEL_EXT=(\w+)\n/) {
            $extension = $1;
        }
    }

    return lib_lang::get_interpolated_message('wizard_reporte_corrupt_info', '2', '0') if (!$extension);
    return ('', $prontus_id, $extension);
};

# Crea la lista de directorios instalados.
# @param $dir_desde el nombre: "$prontus_varglb::DIR_SERVER$FORM{'DIR'}"
# @param $theloop todo el código entre tags <!--LOOP-->(.*?)<!--\/LOOP--> en la plantilla
# @param $TOPE el directorio webroot.
# @return string la lista de directorios instalados.
sub make_lista_dir {
    my $dir_desde = $_[0];
    my $theloop = $_[1];
    my $TOPE = $_[2];

    my @entries = glib_fildir_03::lee_dir($dir_desde);

    # Un truco para dejar los archivos primero y los dirs. despues.
    my @aux;
    my @aux2;

    foreach my $entry (@entries) {
        if (-f "$dir_desde/$entry") {
            push @aux2, $entry;
        } else {
            push @aux, $entry;
        }
    }

    @entries = ();
    @entries = (@aux2, @aux);

    my $filas = '';
    foreach my $entry (@entries) {
        # Solo dirs. validos.
        if (($entry !~ /^\./g) and ((-d "$dir_desde/$entry") or (-f "$dir_desde/$entry")) and ($entry !~ /^cgi/) and ($entry !~ /^wizard/)) {
            my $local_loop = $theloop;

            if (-d "$dir_desde/$entry") {
                $local_loop =~ s/%%icono%%/folder/;
            } else {
                $local_loop =~ s/%%icono%%/file/;
                my $reldir_desde = $dir_desde;
                $reldir_desde =~ s/^$prontus_varglb::DIR_SERVER//;
                $local_loop =~ s/href=".*?"/href="$reldir_desde\/$entry"/;
            }

            my $reldir_desde = $dir_desde;

            $reldir_desde =~ s/^$prontus_varglb::DIR_SERVER//;
            $local_loop =~ s/%%nomdir%%/$entry/;
            $local_loop =~ s/%%dir%%/$reldir_desde\/$entry/;

            $local_loop =~ s/%%dir_actual%%/\// if ($reldir_desde eq '');
            $local_loop =~ s/%%dir_actual%%/$reldir_desde/ if ($reldir_desde ne '');
            $local_loop =~ s/%%tope%%/$TOPE/;

            $filas .= $local_loop;

            if (-d "$dir_desde/$entry") {
                my $lp = $theloop;
                $lp =~ s/<td>/<td><img src="\/wizard_prontus\/core\/imag\/blank.gif" width="20" height="22" border="0" alt="blank">/;
            }
        }
    }

    return $filas;
}
