#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_utils;
use wizard_lib;
##/libs##

main:{
    lib_utils::setUp();

    # Valida informacion de paso anter.
    my ($msg_err,
                $prontus_id,
                $extension,
                $smtp,
                $model_nom,
                $new_title_site_name,
                $server_bd,
                $nom_bd,
                $user_bd,
                $pwd_bd,
                $superuser_bd,
                $superpwd_bd) = &check_paso_anterior_confirm();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    my $buffer = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/wizard_prontus/core/confirm.html");

    my $unspecified_msg = lib_lang::get_message('wizard_show_confirm_unspecified');
    $buffer =~ s/%%prontus_id%%/$prontus_id/g;
    $smtp = $unspecified_msg if (!$smtp);
    $buffer =~ s/%%server_smtp%%/$smtp/g;
    $buffer =~ s/%%prontus_extension%%/$extension/g;
    $buffer =~ s/%%model_nom%%/$model_nom/g;
    $buffer =~ s/%%new_title_site_name%%/$new_title_site_name/g;

    $buffer =~ s/%%server_bd%%/$server_bd/g;
    $buffer =~ s/%%nom_bd%%/$nom_bd/g;
    $buffer =~ s/%%user_bd%%/$user_bd/g;
    $buffer =~ s/%%pwd_bd%%/$pwd_bd/g;
    $superuser_bd = $unspecified_msg if (!$superuser_bd);
    $buffer =~ s/%%superuser_bd%%/$superuser_bd/g;
    $superpwd_bd = $unspecified_msg if (!$superpwd_bd);
    $buffer =~ s/%%superpwd_bd%%/$superpwd_bd/g;

    lib_utils::print_html($buffer);
}

sub check_paso_anterior_confirm {
    # Chequea que se haya pasado por el paso 1
    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    if (! -f $INF_FILE) {
        return "[errInfFile] " .lib_lang::get_message("wizard_invalid_exec_request");
    }

    # Leer y cargar y validar contenido del INF.
    my $buffer = glib_fildir_03::read_file($INF_FILE);
    my ($prontus_id, $extension, $smtp);
    my ($server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
    my $new_title_site_name;

    my $corrupt_info_msg = lib_lang::get_message('wizard_show_confirm_corrupt_info');
    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
            return $corrupt_info_msg;
        }

        # smtp
        if ($buffer_prontus =~ /SERVER_SMTP=([\w\.\-]+)\n/) {
            $smtp = $1;
        }

        if ($buffer_prontus !~ /NEW_TITLE_SITE_NAME=(.+?)\n/) {
            return $corrupt_info_msg;
        } else {
            $new_title_site_name = $1;
        }

        $server_bd = $1 if ($buffer_prontus =~ /server_bd=(.*?)\n/i);
        $nom_bd = $1 if ($buffer_prontus =~ /nom_bd=(.*?)\n/i);
        $user_bd = $1 if ($buffer_prontus =~ /user_bd=(.*?)\n/i);
        $pwd_bd = $1 if ($buffer_prontus =~ /pwd_bd=(.*?)\n/i);
        $superuser_bd = $1 if ($buffer_prontus =~ /superuser_bd=(.*?)\n/i);
        $superpwd_bd = $1 if ($buffer_prontus =~ /superpwd_bd=(.*?)\n/i);
    } else {
        return $corrupt_info_msg;
    }

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return lib_lang::get_interpolated_message("wizard_error_dir_exists", 'prontus');
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (glib_fildir_03::check_dir($dir_prontus)) {
            glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return lib_lang::get_message("wizard_dir_not_created");
        }
    }

    # valida q se haya pasado por paso2
    my $model_nom;

    if ($buffer =~ /(\[MODEL\].*\[\/MODEL\])/s) {
        my $buffer_model = $1;

        if ($buffer_model !~ /MODEL_EXT=(\w+)\n/) {
            return lib_lang::get_interpolated_message('wizard_corrupt_info', '1', '0');
        } else {
            $extension = $1;
        }

        # Validar id
        if ($buffer_model !~ /MODEL_NOM=(\w+)\n/) {
            return lib_lang::get_interpolated_message('wizard_corrupt_info', '2', '1');
        } else {
            $model_nom = $1;
        }
    } else {
        return lib_lang::get_interpolated_message('wizard_corrupt_info', '2', '2');
    }

    return ('', $prontus_id, $extension, $smtp, $model_nom, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
}
