#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use HTTP::Request;
use JSON;
use LWP::UserAgent;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_prontus;
use lib_utils;
use wizard_lib;
##/libs##

main:{
    lib_utils::setUp();
    # Valida informacion de paso 1.
    my $msg_err = &wizard_lib::check_paso1();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = $wizard_lib::CORE_DIR; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    # lee la plantilla
    my $plantilla = "$prontus_varglb::DIR_SERVER$wizard_lib::CORE_DIR/models.html";
    my $buffer = glib_fildir_03::read_file($plantilla);

    # Se leen los modelos instalados y disponibles
    my $refmodels = &wizard_lib::get_models();
    my %models = %$refmodels;

    # Parsea los modelos e imprime la plantilla
    $buffer = &parsea_modelos($buffer, $refmodels);

    lib_utils::print_html($buffer);
}

# Parsea los modelos prontus disponibles.
sub parsea_modelos {
    my ($buffer, $refmodels) = @_;
    my %models = %$refmodels;
    my $nro_models;
    my ($loop_tpl, $loop_out, $loop_item);

    if ($buffer =~ /<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/s) {
        $loop_tpl = $1;
    } else {
        my $msg_err = lib_lang::get_message('wizard_show_models_error_template');
        $prontus_varglb::DIR_CORE = $wizard_lib::CORE_DIR; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    my ($loop1, $loop2);

    foreach my $k (keys %models) {
        # localiza cfg descriptor del modelo,
        # si lo encuentra asume q se trata de un modelo.
        $loop_item = $loop_tpl;

        foreach my $dato (keys %{$models{$k}}) {
            # lib_stdlog::print_err("$dato = $models{$k}{$dato}\n");
            $loop_item =~ s/%%model_$dato%%/$models{$k}{$dato}/g;
        }

        $loop_item =~ s/%%model_id%%/$k/g;

        if ($models{$k}{'instalado'} eq 'instalado') {
            $loop1 = $loop1 . $loop_item;
        } else {
            $loop2 = $loop2 . $loop_item;
        }
    }

    $loop_out = $loop1 . $loop2;
    # Debe haber por lo menos 1 modelo.
    $buffer =~ s/<!--LOOP_MODELS-->.*?<!--\/LOOP_MODELS-->/$loop_out/is;

    return $buffer;
}
