#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use glib_fildir_03;
use glib_str_03;
use lib_lang;
use lib_parser;
use lib_utils;
use wizard_lib;
##/libs##

main:{
    lib_utils::setUp();
    # Valida informacion de paso 1.
    my $msg_err = &wizard_lib::check_paso1();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    my $buffer = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/wizard_prontus/core/paso2.html");
    $msg_err = '';
    ($msg_err, $buffer) = &carga_modelos($buffer);

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err, 0);
    }

    lib_utils::print_html($buffer);
}

sub carga_modelos {
    # Valida y carga modelos prontus disponibles.
    my $buffer = $_[0];
    my $models_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models";

    if (!-d $models_dir) {
        my $msg = "<div class=\"mensaje\">".lib_lang::get_message('wizard_no_models_available')."<br>";
        $msg = $msg . lib_lang::get_message("wizard_show_paso2_select_option")."</div>";
        $buffer =~ s/<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/$msg<div class="separador"><\/div>/sg;
        return ('', $buffer);
    }

    my @lisdir = glib_fildir_03::lee_dir($models_dir);
    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..

    my ($loop_tpl, $loop_out, $loop_item);

    if ($buffer =~ /<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/s) {
        $loop_tpl = $1;
    } else {
        return lib_lang::get_message('wizard_error_template');
    }

    my $nro_models = 0;
    foreach my $k (@lisdir) {
        # localiza cfg descriptor del modelo,
        # si lo encuentra asume q se trata de un modelo.
        if (-f "$models_dir/$k/$k.cfg") {
            $loop_item = $loop_tpl;

            # valida icono
            my $imagen = "/wizard_prontus/models/$k/$k-thumb.png";
            if (-f "$prontus_varglb::DIR_SERVER$imagen") {
                $loop_item =~ s/%%model_img%%/$imagen/g;
            } else {
                $imagen = "/wizard_prontus/models/$k/$k.gif";
                if (-f "$prontus_varglb::DIR_SERVER$imagen") {
                    $loop_item =~ s/%%model_img%%/$imagen/g;
                } else {
                    return "En Modelo [$k] falta icono";
                }
            }

            my $buffer = glib_fildir_03::read_file("$models_dir/$k/$k.cfg");
            my $refcfg = &wizard_lib::carga_model_cfg($buffer);
            my %cfg = %$refcfg;

            $nro_models++;
            $loop_item =~ s/%%model_id%%/$k/g;

            if ($cfg{'MODELO_NOMBRE_REAL'}) {
                $loop_item =~ s/%%model_nom%%/$cfg{'MODELO_NOMBRE_REAL'}/g;
            } else {
                $loop_item =~ s/%%model_nom%%/$k/g;
            }

            $loop_item =~ s/%%model_desc%%/$cfg{'MODELO_DESCRIPTION'}/g;

            if ($cfg{'MODELO_VERSION'}) {
                $loop_item =~ s/%%model_version%%/$cfg{'MODELO_VERSION'}/g;
            } else {
                $loop_item =~ s/%%model_version%%/Desconocida/g;
            }

            my $plt_port_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$k/plantillas/edic/nroedic/port/";
            my @listportdir = glib_fildir_03::lee_dir($plt_port_dir);
            @listportdir = grep !/^\./, @listportdir; # Elimina directorios . y ..
            my $num_port = scalar @listportdir;

            $loop_item =~ s/%%model_num_ports%%/$num_port/g;

            my $plt_art_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$k/plantillas/artic/fecha/pags/";
            my @listartdir = glib_fildir_03::lee_dir($plt_art_dir);
            @listartdir = grep !/^\./, @listartdir; # Elimina directorios . y ..
            my $num_art = scalar @listartdir;

            $loop_item =~ s/%%model_num_arts%%/$num_art/g;

            $loop_out .= $loop_item;
        }
    }

    # Debe haber por lo menos 1 modelo.
    if (!$nro_models) {
        my $msg = "<div class=\"mensaje\">".lib_lang::get_message('wizard_no_models_available')."<br>";
        $msg = $msg . lib_lang::get_message("wizard_show_paso2_select_option")."</div>";
        $buffer =~ s/<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/$msg<div class="separador"><\/div>/sg;
        return ('', $buffer);
    } else {
        $buffer =~ s/<!--LOOP_MODELS-->.*?<!--\/LOOP_MODELS-->/$loop_out/s;
        return ('', $buffer);
    }
}
