#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

package wizard_lib;
use strict;
use utf8;

our $INF_DIR = "/wizard_prontus/_data";
our $INF_FILE = "inf.txt";
our $CORE_DIR = '/wizard_prontus/core';
our $MODELS_DIR = '/wizard_prontus/models';
our $URL_MODELS = "https://www.prontus.cl/models/prontus.12.3";
our $FILE_MODELS = 'models.txt';
our $DOWNLOAD_DIR = '/_prontus_update/models/download';
our $BACKUP_DIR = '/_prontus_update/models/backup';

sub get_inf_dir {
    return $INF_DIR;
}

sub get_inf_file {
    return $INF_FILE;
}
# Chequea que se haya pasado por el paso 1
sub check_paso1 {
    my $file = "$prontus_varglb::DIR_SERVER$INF_DIR/$INF_FILE";

    if (!-f $file) {
        return lib_lang::get_message("wizard_error_invalid_request");
    }

    # Leer y cargar y validar contenido del paso1.
    my $buffer = glib_fildir_03::read_file($file);
    my $prontus_id;

    my $msg_error_info= lib_lang::get_message('wizard_info_corrupt');
    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &get_prontus_id($buffer_prontus);
        lib_stdlog::print_err("prontus_id[$prontus_id]\n");

        if ($prontus_id !~ /^[\w\_\-]+$/) {
            return $msg_error_info;
        }
    } else {
        return $msg_error_info;
    }

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return lib_lang::get_interpolated_message("wizard_error_dir_exists", "[/$prontus_id]");
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (glib_fildir_03::check_dir($dir_prontus)) {
            glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return lib_lang::get_interpolated_message("wizard_dir_not_created_int", $prontus_id);
        }
    }

    return '';
}

sub get_prontus_id {
    my $buffer_prontus = shift;

    if ($buffer_prontus =~ /PRONTUS_ID=(.+?)\n/) {
        my $prontus_id = $1;

        if ($prontus_id =~ /^[\w\_\-]+$/) {
            return $prontus_id;
        }
    }
    return '';
}

sub backup_model {
    my $nom_origen = shift;
    my $dir_origen = "$prontus_varglb::DIR_SERVER$MODELS_DIR";

    if (-d "$dir_origen/$nom_origen") {
        my $dir_destino = "$prontus_varglb::DIR_SERVER$BACKUP_DIR";
        glib_fildir_03::check_dir($dir_destino);

        # Se calcula el directorio para los backup
        my $ts = glib_hrfec_03::get_dtime_pack4();
        my $nom_destino = $ts.'_'.$nom_origen;

        while (-d "$dir_destino/$nom_destino") {
            sleep(1);

            $ts = glib_hrfec_03::get_dtime_pack4();
            $nom_destino = $ts.'_'.$nom_origen;
        }

        glib_fildir_03::copy_tree($dir_origen, $nom_origen, $dir_destino, $nom_destino);
        &garbage_dirs($dir_destino, 5);

        return 1;
    }

    return 0;
}

sub garbage_dirs {
    # Deja solo los ultimos 3 respaldos generados por el update
    my $dir = shift;
    my $max = shift;

    $max = 5 unless($max);
    return unless(-d $dir);

    my @entries = glib_fildir_03::lee_dir($dir);

    @entries = grep !/^\./, @entries; # Elimina directorios . y ..

    # Ordena por fecha dejando primero los mas nuevos
    @entries = sort { (stat("$dir/$b"))[9] <=> (stat("$dir/$a"))[9] } @entries;

    # Elimina del 3r dir en adelante
    my $numdir = 0;
    foreach my $entry (@entries) {
        next if (!-d "$dir/$entry");
        $numdir++;

        if ($numdir > $max) {
            # lib_stdlog::print_err("\tgarbage: borrando [$dir/$entry]\n");
            glib_fildir_03::borra_dir("$dir/$entry");
        }
    }
}

sub get_models {
    my %models;
    my $models_dir = "$prontus_varglb::DIR_SERVER$wizard_lib::MODELS_DIR";
    my ($rutamodelo, $rutamodelofull);

    if (!-d $models_dir) {
        glib_fildir_03::check_dir($models_dir);
        lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_missing_dir", $models_dir));

    } else {
        my @lisdir = glib_fildir_03::lee_dir($models_dir);
        my $nro_models = 0;

        foreach my $model (@lisdir) {
            next if($model =~ /^\./);

            my %array;

            $rutamodelo = "$wizard_lib::MODELS_DIR/$model";
            $rutamodelofull = "$prontus_varglb::DIR_SERVER$wizard_lib::MODELS_DIR/$model";

            my $buffer = glib_fildir_03::read_file("$rutamodelofull/$model.cfg");

            if ($buffer eq '') {
                lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_missing_cfg", "$rutamodelofull$model"));
                next;
            }

            my $refcfg = &carga_model_cfg($buffer);
            my %cfg = %$refcfg;

            # valida q este el arch de obs del modelo
            if (!-f "$rutamodelofull/descripcion/index.html") {
                lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_missing_description", $model));
                next;
            }

            # valida la imagen del modelo
            my ($thumb, $imagen);

            if (-f "$rutamodelofull/$model-thumb.png") {
                $thumb = "$rutamodelo/$model-thumb.png";
                $imagen = "$rutamodelo/$model-big.png";

            } elsif (-f "$rutamodelofull/$model.gif") {
                $thumb = "$rutamodelo/$model.gif";
                $imagen = "$rutamodelo/$model.gif";

            } else {
                lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_missing_image", $model, $rutamodelo));
                next;
            }

            $nro_models++;
            $models{$model}{'thumb'} = $thumb;
            $models{$model}{'imagen'} = $imagen;

            if ($cfg{'MODELO_VERSION'}) {
                $models{$model}{'version'} = $cfg{'MODELO_VERSION'};
            } else {
                $models{$model}{'version'} = '1.0.0.beta';
            }

            $models{$model}{'last_version'} = '<span>'.lib_lang::get_message('wizard_version_not_available').'</span>';

            if ($cfg{'MODELO_NOMBRE_REAL'}) {
                $models{$model}{'nombre'} =  $cfg{'MODELO_NOMBRE_REAL'};

            } else {
                $models{$model}{'nombre'} = $model;
            }

            $models{$model}{'desc'} = $cfg{'MODELO_DESCRIPTION'};
            $models{$model}{'instalado'} = lib_lang::get_message('wizard_installed');
            $models{$model}{'status'} = lib_lang::get_message('wizard_not_available');
        }
    }

    my ($text, $msg_err) = lib_prontus::get_url("$URL_MODELS/$FILE_MODELS", 30);

    if ($msg_err) {
        lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_models_not_accessible_int", "$URL_MODELS/$FILE_MODELS", $msg_err));
        $msg_err = lib_lang::get_message("wizard_models_not_accessible");
        $prontus_varglb::DIR_CORE = $CORE_DIR; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err);
    }

    my @modelos = split(/\s/, $text);
    my $nro_models = 0;

    foreach my $onlinemodel (@modelos) {
        #~ lib_stdlog::print_err("Leyendo modelo online: $onlinemodel\n");
        my ($buffercfg, $msg_err) = lib_prontus::get_url("$URL_MODELS/$onlinemodel/$onlinemodel.cfg", 30);

        if ($msg_err || $buffercfg eq '') {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('wizard_missing_model_cfg', "$URL_MODELS/$onlinemodel/$onlinemodel.cfg", $msg_err));
            next;
        }

        $rutamodelo = "$URL_MODELS/$onlinemodel";

        my $refcfg = &carga_model_cfg($buffercfg);
        my %cfg = %$refcfg;

        # En este caso, se confía que el modelo vendrá ok
        unless($models{$onlinemodel}{'thumb'}) {
            $models{$onlinemodel}{'thumb'} = "$rutamodelo/$onlinemodel-thumb.png";
        }

        unless($models{$onlinemodel}{'imagen'}) {
            $models{$onlinemodel}{'imagen'} = "$rutamodelo/$onlinemodel-big.png";
        }

        if ($cfg{'MODELO_NOMBRE_REAL'}) {
            $models{$onlinemodel}{'nombre'} = $cfg{'MODELO_NOMBRE_REAL'};
        } else {
            $models{$onlinemodel}{'nombre'} = $onlinemodel;
        }

        $models{$onlinemodel}{'desc'} = $cfg{'MODELO_DESCRIPTION'};

        if ($models{$onlinemodel}{'version'}) {
            # Este modelo esta instalado
            $models{$onlinemodel}{'last_version'} = $cfg{'MODELO_VERSION'};
            $models{$onlinemodel}{'online'} = 'online';
            if ($models{$onlinemodel}{'version'} eq $models{$onlinemodel}{'last_version'}) {
                $models{$onlinemodel}{'status'} = lib_lang::get_message('wizard_update');
            } elsif (&es_actualizable($models{$onlinemodel}{'version'}, $models{$onlinemodel}{'last_version'}, $onlinemodel)) {
                $models{$onlinemodel}{'status'} = lib_lang::get_message('wizard_updated');
            } else {
                $models{$onlinemodel}{'status'} = lib_lang::get_message('wizard_not_available');
            }
        } else {
            # El modelo no está instalado
            $models{$onlinemodel}{'version'} = lib_lang::get_message('wizard_not_installed_label');
            $models{$onlinemodel}{'last_version'} = $cfg{'MODELO_VERSION'};
            $models{$onlinemodel}{'instalado'} = lib_lang::get_message('wizard_not_installed');
            $models{$onlinemodel}{'status'} = lib_lang::get_message('wizard_download');
        }

        $nro_models++;
    }

    if ($nro_models == 0) {
        $msg_err = lib_lang::get_message("wizard_no_models_available");
        $prontus_varglb::DIR_CORE = $CORE_DIR; # solo para efectos de la plantilla de mensaje
        lib_utils::print_pag_result(lib_lang::get_message('global_error_label'), $msg_err);
    }

    return \%models;
}

sub descarga_componente {
    my $modelid = shift;
    my $nombre = shift;
    my $mode = shift; # 'bin' para archivos binarios.

    my $url = $wizard_lib::URL_MODELS . '/' . $modelid . '/' .$nombre;
    my ($content, $msg_err) = lib_prontus::get_url($url, 30);

    if ($msg_err) {
        if ($msg_err =~ /^404 /) {
            return lib_lang::get_interpolated_message("wizard_error_downloading_model", $nombre, $url);
        } else {
            return lib_lang::get_interpolated_message("wizard_error_downloading_url", $url, $msg_err);
        }
    }

    # Se escribe el archivo leído
    if ($mode eq 'bin') {
        glib_fildir_03::write_file_bin("$prontus_varglb::DIR_SERVER$MODELS_DIR/$modelid/$nombre", $content);
    } else {
        glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$MODELS_DIR/$modelid/$nombre", $content);
    }

    return '';
}

sub carga_model_cfg {
    my $buffer = $_[0];
    my %cfg;

    $buffer =~ s/\n+/\n/g;

    while ($buffer =~ /(\w+)\s*=\s*("|')(.*?)\2/g) {
        $cfg{$1} = $3;
    }

    return \%cfg;
}

sub es_actualizable {
    my ($actual, $last, $model) = @_;
    my ($a1, $a2, $a3, $a4, $l1, $l2, $l3);

    if ($actual =~ /(\d+)\.(\d+)\.(\d+)(\.beta)?/) {
        $a1 = $1;
        $a2 = $2;
        $a3 = $3;
        $a4 = 1 if($4);
    } else {
        $a1 = 1;
        $a2 = 0;
        $a3 = 0;
        $a4 = 1;
    }

    if ($last =~ /(\d+)\.(\d+)\.(\d+)/) {
        $l1 = $1;
        $l2 = $2;
        $l3 = $3;
        if($a1 != $l1) {
            return 0;
        } elsif ($a2 < $l2) {
            return 1;
        } elsif ($a3 < $l3) {
            return 1;
        } elsif ($a3 < $l3) {
            return 1;
        } else {
            if($a3 eq $l3 && $a4) {
                return 1;
            }
        }
    } else {
        lib_stdlog::print_err(lib_lang::get_interpolated_message("wizard_error_version_unreadable", $model, $last));
    }

    return 0;
}

1;
