#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
package lib_xcoding;
use strict;
use utf8;

our $HLS = 0;
our $PRECISION_HLS = 0;
our $MODO_PARALELO = 0;
our $MAX_PARALELO = 3;
our $FDK = 0;
our $PATHNICE = '';
our $MAX_PIXEL = '';
our $MAX_VRATE = '';
our $MAX_ARATE = '';
our $RUTA_TEMPORAL = '';
our $RUTA_PRONTUS = '';
our $ARTIC_filename = '';
our $ARTIC_ts_articulo = '';
our $N_THREADS = 0;
our $VBITRATE = 0;
our $ABITRATE = 0;
our $ANCHO = 0;
our $ALTO = 0;
our $VCODEC = '';
our $ACODEC = '';

# reinica/inicia los valores de las variables de la lib
sub init {
    $HLS = 0;
    $PRECISION_HLS = 0;
    $MODO_PARALELO = 0;
    $MAX_PARALELO = 3;
    $FDK = 0;
    $MAX_PIXEL = '';
    $MAX_VRATE = '';
    $MAX_ARATE = '';
    $RUTA_TEMPORAL = '';
    $RUTA_PRONTUS = '';
    $ARTIC_filename = '';
    $ARTIC_ts_articulo = '';
    $N_THREADS = 0;
    $VBITRATE = 0;
    $ABITRATE = 0;
    $ANCHO = 0;
    $ALTO = 0;
    $VCODEC = '';
    $ACODEC = '';
}
# Se leen todas las configuraciones disponibles
sub get_all_formatos {
    my $marca = $_[0];
    my $ts = $_[1];
    my $xcoderFormat = $_[2];
    my %formatos = ();
    my $file_formatos = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::XCODING_DATA_PATH$prontus_varglb::XCODE_CFG";

    my $buffer_formatos = glib_fildir_03::read_file($file_formatos);
    if ($buffer_formatos eq '') {
        lib_stdlog::print_err(lib_lang::get_interpolated_message('xcoding_no_formats_all', $file_formatos));
        return %formatos;
    }

    while ($buffer_formatos =~ /\s*(\S+\.?\w*\.)(\w+)\s*=\s*'(.*?)'/ig) {
        if ($3 ne '') {
            my $formatName = lc($1);
            my $paramName = lc($2);
            my $value = lc($3);

            next if ($marca ne '' && $formatName !~ /^$marca\./i );

            if ($xcoderFormat) {
                $formatName =~ s/\./$ts/;
            }
            $formatName =~ s/\.$//;

            if ($xcoderFormat) {
                $formatName .= '.mp4';
            }
            $formatos{$formatName}{$paramName} = $value;
        }
    }

    return %formatos;
}

# Se lee la configuracion por defecto, o lee configuracion para versiones
sub get_formatos {
    my $marca = $_[0];
    my $por_defecto = $_[1];
    my %formatos;
    my $file_formatos = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::XCODING_DATA_PATH$prontus_varglb::XCODE_CFG";

    if ($marca ne '') {
        my $buffer_formatos = glib_fildir_03::read_file($file_formatos);
        if ($buffer_formatos eq '') {
            lib_stdlog::print_err(lib_lang::get_interpolated_message('xcoding_no_formats', $file_formatos));
            return %formatos;
        }

        if ($por_defecto != 1) {
            while ($buffer_formatos =~ /($marca\.\w+)\.(\w+)\s*=\s*'(.+?)'/ig) {
                # lib_stdlog::print_err("{$1}{$2} = $3);\n";
                if ($3 ne '') {
                    $formatos{lc($1)}{lc($2)} = $3;
                }
            }
        } else {
            while ($buffer_formatos =~ /($marca)\.(\w+)\s*=\s*'(.+?)'/ig) {
                # lib_stdlog::print_err("{$1}{$2} = $3);\n";
                if ($3 ne '') {
                    $formatos{lc($1)}{lc($2)} = $3;
                }
            }
        }
    }

    return %formatos;
}

sub get_info_media {
    my $origen = $_[0];
    my @info;
    if (-s $origen) {
        @info =`$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg -i $origen 2>&1`;
    }

    my ($width, $height, $vcodec, $acodec, $ext, $vbitrate, $abitrate, $fps, $duration, $rate) = (0, 0, '', '', '', 0, 0, 0, 0, 0);
    if ($origen =~ /.+\/\d{8}\/mmedia\/multimedia_video\w+\d{14}\.(\w+)$/) {
        $ext = $1;
    }

    foreach (@info) {
        # Duration: 00:05:42.90, start: 0.000000, bitrate: 2136 kb/s
        if ($_ =~ /Duration: (\d+):(\d+):([\d\.]+).*/) {
            #~ lib_stdlog::print_err("Duration: [$1] [$2] [$3] \n");
            $duration = $1*3600 + $2*60 + $3;
            next;
        }
        # Video: h264 (Main) (avc1 / 0x31637661), yuv420p, 534x360, 306 kb/s, 29.97 fps, 29.97 tbr, 16k tbn, 59.94 tbc (default)
        # Video: h264 (Baseline), yuv420p, 1920x1080, 20745 kb/s, 29.97 fps, 29.97 tbr, 600 tbn, 1200 tbc
        # Video: h264 (Main), yuv420p, 854x356 [SAR 1:1 DAR 427:178], 24.42 fps, 23.98 tbr, 1k tbn, 47.95 tbc
        if ($_ =~ /Video: ([^,]+),/) {
            $vcodec = $1;
            # buscamos la resolucion
            if ($_ =~ /Video:.*?(\d\d+)x(\d\d+)/) {
                $width = $1;
                $height = $2;
            }
            # buscamos bitrate
            if ($_ =~ /Video:.*?([0-9\.]+) kb\/s/) {
                $vbitrate = $1;
            }

            if ($_ =~ /Video:.*?([0-9\.]+) fps/) {
                $fps = $1;
            }
            next;
        }

        # Audio: aac, 44100 Hz, mono, s16, 62 kb/s
        if ($_ =~ /Audio: ([^,]+),(.+?)([0-9\.]+) kb\/s/is) {
            $acodec = $1;
            my $other = $2;
            $abitrate = $3;

            if ($other =~ /(\d+)\s*Hz/) {
                $rate = $1;
            }
        }
    }

    if ($abitrate == 0) {
        $abitrate = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'}/2;
    }
    if ($vbitrate == 0) {
        $vbitrate = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_VIDEO_BITRATE'}/2;
    }

    return ($width, $height, $vcodec, $acodec,$vbitrate, $abitrate, $fps, $duration, $rate);
}

# arma el comando de ffmpeg a ejecutar para transcodificar
sub get_cmd_ffmpeg {
    my $origen = $_[0];
    my $destino = $_[1];
    my $pass = $_[2];
    my $formato = $_[3];
    my $video_string = 'copy'; #por defecto no se recodifica video
    my $audio_string = 'copy'; #por defecto no se recodifica audio
    my $resize_string = '';
    my $resize = 0;
    my $nd_pass = 0;
    my $ruta_trabajo = '';
    my $force_frames = '';
    my $threads_string = '';
    my $max_size = $MAX_PIXEL; # tamaño maximo del frame por defecto
    my $max_vrate = POSIX::ceil(0.9*$MAX_VRATE); # bitrate maximo por defecto para video, se saca el 10%
    my $max_arate = $MAX_ARATE; # bitrate maximo por defecto para audio
    my ($nuevo_alto, $nuevo_ancho) = (0 ,0);

    my $videoFlags = '';
    my $audioFlags = '';

    # si existen cargamos los valores del formato a generar, sino usamos los valores
    # por defecto seteados anteriormente
    if (defined($formato->{'videosize'}) && $formato->{'videosize'} ne '')  {
        $max_size = $formato->{'videosize'};
    }
    if (defined($formato->{'videobitrate'}) && $formato->{'videobitrate'} ne '')  {
        $max_vrate = $formato->{'videobitrate'};
    }

    # se carga el bitrate de audio, si se indica en el formato, sino usamos el maximo por defecto
    # ya seteado
    if (defined($formato->{'audiobitrate'}) && $formato->{'audiobitrate'} ne '')  {
        $max_arate = $formato->{'audiobitrate'};
    }

    if ($N_THREADS > 0) {
        $threads_string = "-threads $N_THREADS";
    }

    # ej: /var/www/sitio.cl/web/prontus/site/mm/20140319/mmedia/multimedia_video120140319122141.flv
    # si existe ruta temporal de trabajo se usa, si no usamos la misma ruta del archivo original en prontus
    if ($RUTA_TEMPORAL eq '') {
        $ruta_trabajo = $RUTA_PRONTUS;
    } else {
        $ruta_trabajo = $RUTA_TEMPORAL;
    }

    # la resolucion SIEMPRE debe ser par, si no es par se debe redimensionar
    $nuevo_ancho = $ANCHO;
    $nuevo_alto = $ALTO;
    if ($ANCHO %2 != 0) {
        $nuevo_ancho += 1;
        $resize = 1;
    }
    if ($ALTO%2 != 0) {
        $nuevo_alto += 1;
        $resize = 1;
    }

    # si el bitrate de video es mayor al maximo se debe cambiar la resolucion
    # o si el alto o ancho superan el tamaño definido
    if ($VBITRATE > $max_vrate || $ANCHO > $max_size || $ALTO > $max_size) {
        # si el tamaño se sobrepasa, ajustarlo.
        if ($ANCHO > $max_size || $ALTO > $max_size) {
            # si el ancho es > alto => ancho es mayor que $MAX_PIXEL
            if ($ANCHO > $ALTO) {
                #se ajusta el alto manteniendo la proporcion y el ancho se deja fijo al maximo
                $nuevo_alto = int ($max_size*$ALTO/$ANCHO);
                $nuevo_ancho = $max_size;
            } else {
                #se ajusta el ancho manteniendo la proporcion y el alto se deja fijo al maximo
                $nuevo_ancho = int ($max_size*$ALTO/$ANCHO);
                $nuevo_alto = $max_size;
            }
            $resize = 1;
        }
    }

    # la resolucion SIEMPRE debe ser par, si no es par se debe redimensionar
    if ($nuevo_ancho %2 != 0) {
        $nuevo_ancho += 1;
        $resize = 1;
    }
    if ($nuevo_alto%2 != 0) {
        $nuevo_alto += 1;
        $resize = 1;
    }

    # se arma string de cambio de tamaño de video:
    if ($resize) {
        $resize_string = "-s $nuevo_ancho" . 'x' . "$nuevo_alto";
    }

    #parametros de configuracion por defecto para video
    # si no es h264, o se hace resize, o el bitrate es muy alto, se necesita recodificar y hacer 2 pasos
    # tambien si hay que generar HLS se debe recodificar para generar keyframes alineados
    if ($VCODEC !~ /h264/i || $resize || $VBITRATE > $max_vrate || $HLS) {
        if (!defined($formato->{'x264'}) || (defined($formato->{'x264'}) && $formato->{'x264'} eq '')) {
            $videoFlags = 'b-adapt=2:trellis=1:cabac=1:bframes=3:keyint=90'; #configuracion de compresion para utilizar main por defecto y key int cada 2 segundos approx
        } else {
            $videoFlags = $formato->{'x264'}; #cargamos los parametros indicados
        }

        # si el video es h264 y el bitrate original es menor que el maximo indicado para el formato se usa el mismo
        if ($VBITRATE < $max_vrate && $VCODEC =~ /h264/i) {
            $videoFlags .= ':vbv-maxrate='.POSIX::ceil(1.05*$VBITRATE).':vbv-bufsize='.(3*$VBITRATE).'" -b:v '.$VBITRATE.'000';
        } else {
            # sino usamos el bitrate indicado en el formato
            $videoFlags .= ':vbv-maxrate='.POSIX::ceil(1.05*$max_vrate).':vbv-bufsize='.(3*$max_vrate).'" -b:v '.$max_vrate.'000';
        }

        # eliminamos las opciones que hacen demorar mas el primer paso
        if (!$pass) {
            $videoFlags =~ s/trellis=1://igs;
            $videoFlags =~ s/cabac=1://igs;
        }

        # eliminamos las opciones que no son necesarias en el paso 2
        if ($pass) {
            $videoFlags =~ s/b-adapt=2://igs; # no se necesita en paso 2
        }

        # se define el profile
        if (defined($formato->{'h264profile'}) && $formato->{'h264profile'} ne '')  {
            $videoFlags .= " -profile:v " . $formato->{'h264profile'};
        } else {
            $videoFlags .= " -profile:v main";
        }
        $videoFlags .= " -pix_fmt yuv420p";

        # si este es el primer paso, agregamos opciones de paso 1 mas rapido
        if (!$pass) {
            $videoFlags = 'pass=1:subq=1:frameref=1:' . $videoFlags;
        }
        # al usar x264 ffmpeg 1.x usa -passlog file, ffmpeg 2.x usa stats=, se indican los 2 para compatibilidad
        # ya que si no se indica passlogfile en ffmpeg 1, escribe un log con prefijo "ffmpeg2pass" en la misma carpeta de xcoding
        # y usa el mismo para cada video
        $video_string = "libx264 -x264opts \"stats=$ruta_trabajo$ARTIC_filename.log:$videoFlags";

        # se necesitan 2 pasos si se recodifica el video
        $nd_pass = 1;
    }

    # se transcodifica audio si no es aac o sobrepasa el limite de bitrate definido
    # 2014/10/24 por problemas al generar HLS se transcodifica siempre el audio si se crea HLS
    if ($ACODEC !~ /aac/i || $ABITRATE > $max_arate || $HLS) {
        # generamos el string de compresion de audio
        # se elige encoder segun disponibilidad
        $audio_string = "aac"; # por defecto se usa aac de ffmpeg
        if ($FDK) {
            $audio_string = "libfdk_aac";
        }
        # se elige frecuencia de sampleo, si no se especifica usa la original
        if (defined($formato->{'audiosampling'}) && $formato->{'audiosampling'} ne '') {
            $audio_string .= " -ar $formato->{'audiosampling'}";
        }

        # se elige bitrate de audio, si es mayor se usa el maximo, sino conservamos el que venia
        if ($ABITRATE > $max_arate) {
            # si el bitrate de audio es mayor que el maximo, usamos el maximo
            $audio_string .= " -ab ".$max_arate."k";
        } else {
            # si no usamos el mismo que tenia
            $audio_string .= " -ab ".$ABITRATE."k";
        }
        # se elige numero de canales, si no se especifica usa la original
        if (defined($formato->{'audiochannels'}) && $formato->{'audiochannels'} ne '') {
            $audio_string .= " -ac $formato->{'audiochannels'}";
        }
    }

    # para HLS se generan key frames alineados
    if ($HLS && $PRECISION_HLS) {
        $force_frames = '-force_key_frames expr:gte\(t,n_forced*10\)';
    }

    if ($nd_pass) {
        #si se hacen 2 pasos hay que entregar el string correspondiente a cada paso
        if (!$pass) {
            # primer paso los datos se envian a /dev/null ya que no se necesitan, no se procesa resize tampoco $ruta_trabajo$ARTIC_filename
            return ("$PATHNICE $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg $threads_string -i $origen -y $resize_string $force_frames -vcodec $video_string -pass 1 -an -passlogfile $ruta_trabajo$ARTIC_filename.log -f rawvideo /dev/null", 1);
        } else {
            # segundo paso
            return ("$PATHNICE $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg $threads_string -i $origen -y $resize_string $force_frames -vcodec $video_string -pass 2 -acodec $audio_string -passlogfile $ruta_trabajo$ARTIC_filename.log -movflags +faststart -f mp4 $ruta_trabajo$destino", 1);
        }
    } else {
        #no se necesita segundo paso si no se codifica video
        return ("$PATHNICE $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg $threads_string -i $origen -y -vcodec copy -acodec $audio_string -movflags +faststart -f mp4 $ruta_trabajo$destino", 0);
    }
}

# arma el comando de ffmpeg a ejecutar para transcodificar
sub get_cmd_ffmpeg_audio {
    my $origen = $_[0];
    my $destino = $_[1];
    my $ruta_trabajo = '';
    my $audioFlags = '';

    # ej: /var/www/sitio.cl/web/prontus/site/mm/20140319/mmedia/multimedia_video120140319122141.flv
    # si existe ruta temporal de trabajo se usa, si no usamos la misma ruta del archivo original en prontus
    if ($RUTA_TEMPORAL eq '')  {
        $ruta_trabajo = $RUTA_PRONTUS;
    } else {
        $ruta_trabajo = $RUTA_TEMPORAL;
    }

    my $audio_string = ''; #por defecto no se recodifica audio
    # se transcodifica audio si no es mp3 o sobrepasa el limite de bitrate definido
    if ($ACODEC !~ /mp3/i || $ABITRATE > $MAX_ARATE) {
        # generamos el string de compresion de audio
        # se elige encoder segun disponibilidad
        $audio_string = "-c:a libmp3lame -ar 44100";

        # se elige bitrate de audio, si es mayor se usa el maximo, sino conservamos el que venia
        if ($ABITRATE > $MAX_ARATE) {
            # si el bitrate de audio es mayor que el maximo, usamos el maximo
            $audio_string .= " -ab ".$MAX_ARATE."k";
        } else {
            # si no usamos el mismo que tenia
            $audio_string .= " -ab ".$ABITRATE."k";
        }
    }

    return "$PATHNICE $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg -i $origen -y -vn $audio_string -f mp3 $ruta_trabajo$destino";
}

# genera hls para el video indicado
sub generar_HLS {
    my $origen = $_[0];

    my $path_hls;
    if ($origen =~ /(\/[\w\.\-\/]*\/\d{8}\/mmedia\/multimedia_video\w+\d{14}\S*)\.mp4$/) {
        $path_hls = $1;
    } else {
        return;
    }

    # si existe borramos el directorio y su contenido
    if (-d $path_hls) {
        glib_fildir_03::borra_dir($path_hls);
    }
    glib_fildir_03::check_dir($path_hls);

    my $cmd = "$PATHNICE $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg -i $origen -y -codec copy -bsf:v h264_mp4toannexb -f segment -segment_list $path_hls/playlist.m3u8 -segment_list_type m3u8 -segment_time 10 -segment_time_delta 0.05 $path_hls/%03d.ts";

    #~ lib_stdlog::print_err("generar HLS origen [$origen][$cmd]\n");

    my $res = qx/$cmd 2>&1/;
    &die_stderr(lib_lang::get_interpolated_message('xcoding_error_make_hls', $cmd), "[$!][$res].") if ($? != 0);

    # leemos el directorio para hacer purge
    my @archivos = glib_fildir_03::lee_dir($path_hls);
    my $filepath = '';
    # hacemos purge de los archivos generados
    foreach my $archivo (@archivos) {
        next if($archivo eq '.' || $archivo eq '..');
        # armamos la ruta completa
        $filepath = "$path_hls/$archivo";
        # dejamos la ruta relativa
        if ($filepath =~ /.*(\/.*?\/site\/\w+\/\d{8}\/mmedia\/multimedia_video\w+\S?.*)/) {
            lib_prontus::purge_cache($1);
        }
    }
}
sub genera_info_media_audio {
    my $fullpath_audio = $_[0];
    return if (!-s $fullpath_audio);

    my ($width, $height, $vcodec, $acodec, $vbitrate, $abitrate, $fps, $duration, $rate) = &get_info_media($fullpath_audio);
    my %data = ();
    $data{'acodec'} = $acodec;
    $data{'abitrate'} = $abitrate;
    $data{'duration'} = $duration;
    $data{'duration_hms'} = &format_time($duration);
    $data{'rate'} = $rate;
    $data{'size'} = glib_str_03::bytes2kb(-s $fullpath_audio);

    my $path_json = $fullpath_audio;
    $path_json =~ s/\w+$/json/;

    my $buffer_json = JSON::encode_json(\%data);
    utf8::decode($buffer_json);

    lib_prontus::purge_cache($path_json);
    glib_fildir_03::write_file($path_json, $buffer_json);

}
sub genera_info_media {
    my $fullpath_video = $_[0];
    return if (!-f $fullpath_video);

    my ($width, $height, $vcodec, $acodec, $vbitrate, $abitrate, $fps, $duration) = &get_info_media($fullpath_video);
    my %data = ();
    $data{'width'} = $width;
    $data{'height'} = $height;
    $data{'vcodec'} = $vcodec;
    $data{'acodec'} = $acodec;
    $data{'vbitrate'} = $vbitrate;
    $data{'abitrate'} = $abitrate;
    $data{'fps'} = $fps;
    $data{'duration'} = $duration;
    $data{'duration_hms'} = &format_time($duration);

    $fullpath_video =~ m|(.*?)/[\w\_\-]+?/site/\w+/\d{8}/mmedia/(multimedia_video\w+)(\d{8})(\d{6})(\.\w+)$|;
    my $path_prontus = $1;
    my $marca_video = $2;
    my $fechac = $3;
    my $ts = "$3$4";

    my $path_media = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$fechac$prontus_varglb::DIR_MMEDIA/$marca_video$ts";
    my @files_multimedia = glob("$path_prontus$path_media*.mp4");
    $data{'qualities'} = ();
    my $counter = 1;
    foreach my $media (sort @files_multimedia) {
        if ($media =~ /$marca_video$ts(\w*)/) {
            $data{'qualities'}{$counter}{'file'} = "$path_media$1.mp4";
            $counter++;
        }
    }

    if (-s "$fullpath_video.m3u8" ) {
        $data{'hls'} = "$path_media.mp4.m3u8";
    }

    $data{'size'} = glib_str_03::bytes2kb(-s $fullpath_video);

    $data{'custom_imag'} = '';
    $data{'screenshot'} = '';
    # busca poster
    my $path_thumbs = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$fechac$prontus_varglb::DIR_THUMBS";
    if (-s "$path_prontus$path_thumbs/$marca_video$ts.jpg" ) {
        $data{'poster'} = "$path_thumbs/$marca_video$ts.jpg";
        $data{'screenshot'} = $data{'poster'};
    }
    my $xml_art = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/artic/$fechac/xml/$ts.xml");
    if ($xml_art =~ /<fotofija_$marca_video>\s*?<!\[CDATA\[(.+?)\]\]>\s*?<\/fotofija_$marca_video>/is ) {
        my $path_foto = $1;
        $path_foto =~ s/(foto_\d+)\_(.*)$/$1\/$2/si;
        $data{'poster'} = $path_foto;
        $data{'custom_imag'} = $path_foto;
    }

    if ($xml_art =~ /<chk_ad_disable_$marca_video>\s*?<!\[CDATA\[(.+?)\]\]>\s*?<\/chk_ad_disable_$marca_video>/is ) {
        $data{"chk_ad_disable"} = ($1 eq 'SI')?1:0;
    }

    if ($xml_art =~ /<ad_tag_$marca_video>\s*?<!\[CDATA\[(.+?)\]\]>\s*?<\/ad_tag_$marca_video>/is ) {
        $data{"ad_tag"} = $1;
    }

    $data{'thumbnails'} = '';
    # busca thumbnails
    if (-s "$path_prontus$path_thumbs/$marca_video$ts\_thumbs.vtt" ) {
        $data{'thumbnails'} = "$path_thumbs/$marca_video$ts\_thumbs.vtt";
    }

    # busca subtitle
    if (-s "$path_prontus$path_thumbs/$marca_video$ts\_subtitle.vtt" ) {
        $data{'subtitle'} = "$path_thumbs/$marca_video$ts\_subtitle.vtt";
    }

    my $buffer_json = JSON::encode_json(\%data);
    utf8::decode($buffer_json);

    lib_prontus::purge_cache("$path_prontus$path_media.json");

    glib_fildir_03::write_file("$path_prontus$path_media.json", $buffer_json);
}
sub format_time {
    my $segundos = $_[0];
    # todas las fracciones de horas son minutos o segundos:
    my $horas = POSIX::floor($segundos/3600);
    $segundos -= ($horas *3600);
    my $minutos = POSIX::floor($segundos/60);
    $segundos = $segundos - ($minutos *60);
    $segundos = sprintf("%.3f", $segundos);
    return sprintf('%02d', $horas).":".sprintf('%02d', $minutos).":". sprintf('%02d', $segundos);
}
sub die_stderr {
    my ($msg, $detalle) = @_;

    lib_stdlog::print_err("[ERROR][$ARTIC_ts_articulo][$msg][$detalle]\n");
    exit 1;
}

1;
