#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
#
# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Realiza efectivamente la transcodificacion de un video de avi, wmv, mp4, mpeg, rm y otros a mp4.
# Cuando termina, deduce donde esta el xml del articulo Prontus y modifica las variables.
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# Al terminar elimina el video original y sustituye el nombre en el xml del articulo Prontus.
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Por linea de comandos.
# Los parametros son:
# 0- Path absoluto al video a transcodificar.
# 1- prontus id
#
# /usr/bin/perl prontus_audio_do_xcode.pl /sites/prontus_lab/web/prontus_proto/site/artic/20100531/mmedia/multimedia_audio120100531182139.avi prontus_proto
#
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
#  No usa plantillas.
# ---------------------------------------------------------------
use utf8;
use strict;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

END {
    &tareas_salida();
}

use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Artic;
use DBI;
use Digest::MD5;
use File::Copy;
use HTML::Entities;
use JSON;
use POSIX;
use Time::Local;
use URI::Escape;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_friendly;
use lib_helper;
use lib_lang;
use lib_mam;
use lib_parser;
use lib_prontus;
use lib_tags;
use lib_tax;
use lib_urlf5;
use lib_utils;
use lib_waitlock;
use lib_xcoding;
##/libs##
# use Data::Dumper;

my $ORIGEN = $ARGV[0];
my $PRONTUS_ID = $ARGV[1];
my $ARTIC_dirfecha = '';
my $ARTIC_ts_articulo = '';
my $ARTIC_filename = '';
my $ARTIC_media_path = '';
my $RUTA_PRONTUS = '';  # ruta de la carpeta prontus donde esta el video en caso de no usar ruta temporal
my %FORMATOS_VERSIONES = (); # hash que guarda los formatos de las versiones del video
my $VERSIONS_FILE = ''; # archivo temporal para crear las versiones del video original
my $MARCA = ''; # nombre de la marca prontus del video
my $CLIENT = undef; # user agent para hacer consultas remotas
my $SLEEP = 5; # tiempo de espera entre peticiones remotas

my $LOCK = '';
my $LOCKFH;

main: {
    lib_utils::setUp();
    lib_xcoding::init();
    lib_xcoding::die_stderr(lib_lang::get_message('xcoding_audio_invalid_origin'), "") if ((!-f "$ORIGEN") || (!-s "$ORIGEN"));
    lib_xcoding::die_stderr(lib_lang::get_message("global_invalid_id"), "") if (! lib_utils::valida_prontus($PRONTUS_ID));

    my ($cmd, $res);
    my ($total, $segundos);
    my $start = time; # tiempo inicial para medir la duracion del proceso

    # Deduce ubicacion del xml del articulo.
    if ($ORIGEN =~ /(.+\/(.+?)\/(\d{8})\/mmedia\/)(multimedia_audio\w+(\d{6}))\.(\w+)$/i) {
        $ARTIC_media_path = $1;
        $ARTIC_dirfecha = $3;
        $ARTIC_ts_articulo = $3 . $5;
        $ARTIC_filename = $4;
    } else {
        lib_xcoding::die_stderr(lib_lang::get_interpolated_message("xcoding_audio_info_unobtainable", join(';', @ARGV)));
    }

    $ORIGEN =~ /\/mmedia\/(multimedia_audio\w+)\d{14}\.(\w+)$/i;
    $MARCA = $1;

    # Path conf y load config de prontus
    my $msg_load = lib_utils::load_config_by_id($PRONTUS_ID);
    if ($msg_load ne '') {
        print STDERR "$msg_load\n";
        exit;
    }

    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] [$ARTIC_filename] " . lib_lang::get_message('global_start_process') . "\n";

    # Forma el nombre de la pelicula destino sustituyendo la extension
    my $destino = $ORIGEN;
    $destino =~ s/\.\w+$/\.mp3/;

    # para obtener la carpeta prontus del video
    $destino =~ /(\/.*\/\d{8}\/mmedia\/)multimedia_audio.*\d{6}\S?\.\w+$/i;
    $RUTA_PRONTUS = $1;
    #~ print STDERR "RUTA_PRONTUS [$RUTA_PRONTUS]\n";

    # inicializamos los valores de la libreria
    lib_utils::get_path_nice();
    $lib_xcoding::PATHNICE = "$prontus_varglb::PATHNICE -n19 " if ($prontus_varglb::PATHNICE);
    $lib_xcoding::MAX_ARATE = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'};
    $lib_xcoding::RUTA_TEMPORAL = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'};
    $lib_xcoding::RUTA_PRONTUS = $RUTA_PRONTUS;
    $lib_xcoding::ARTIC_filename = $ARTIC_filename;
    $lib_xcoding::ARTIC_ts_articulo = $ARTIC_ts_articulo;

    #se obtienen los datos del video original
    ($lib_xcoding::ANCHO, $lib_xcoding::ALTO, $lib_xcoding::VCODEC, $lib_xcoding::ACODEC, $lib_xcoding::VBITRATE, $lib_xcoding::ABITRATE) = lib_xcoding::get_info_media($ORIGEN);

    # Se transcodifican mp4s si el bitrate de video es muy alto
    # Sin embargo, se aplica la correccion de los headers y offset, moviendolo al comienzo
    if ($ORIGEN =~ /\.mp3$/i) {
        if ($lib_xcoding::ABITRATE <= $lib_xcoding::MAX_ARATE) {
            lib_xcoding::die_stderr(lib_lang::get_message('xcoding_audio_conversion_unnecessary'), "");
        }
    }

    # realizamos la transcoficacion del original
    &do_xcode($ORIGEN, $destino); # $key es vacio

    # si existe el mp3
    if (-s $destino) {
        # Actualizamos el articulo  articulo. solo guardamos el xml en el articulo.
        &actualizar_articulo($MARCA, 0);
        # para obtener la carpeta prontus relativa del video
        $destino =~ /\/[\w\.\-\/]*(\/[\w\_\-]+?\/site\/\w+\/\d{8}\/mmedia\/multimedia_audio.*\d{6}\S?\.\w+)$/i;
        lib_prontus::purge_cache($1);
    } else {
        # sino terminamos
        lib_xcoding::die_stderr(lib_lang::get_message('xcoding_audio_error_generating'), $destino);
    }

    # Actualizar articulo.
    &actualizar_articulo($MARCA, 1);

    $total = time - $start;
    $segundos = $total % 60;
    $segundos = $segundos < 10? '0'.$segundos : $segundos;
    print STDERR lib_lang::get_interpolated_message('xcoding_total_time', $ARTIC_filename, int($total/60), $segundos);
}

sub do_xcode {
    my $origen = $_[0];
    my $destino = $_[1];

    $destino =~ /\/\d{8}\/mmedia\/(multimedia_audio.*\d{6}\S?\.mp3)$/i;
    my $archivo_destino = $1;
    $archivo_destino =~ s/\.mp3$/\.tmp/i;

    my $cmd = lib_xcoding::get_cmd_ffmpeg_audio($origen, $archivo_destino);

    print STDERR lib_lang::get_interpolated_message('xcoding_audio_cmd', $cmd);
    my $start = time;
    # Ejecuta la transcodificacion redirigiendo stderr to stdout.
    # Por ahora no se analiza la salida del ffmpeg. La redireccion del stderr al stdout es porque ffmpeg imprime su salida al stderr en vez de al stdout
    my $res = `$cmd 2>&1`;

    my $end = time;
    my $total = $end - $start;
    my $segundos = $total % 60;
    $segundos = $segundos < 10? '0'.$segundos : $segundos;
    print STDERR lib_lang::get_interpolated_message('xcoding_ffmpeg_time', $ARTIC_filename, int($total/60), $segundos);
    lib_xcoding::die_stderr(lib_lang::get_message("xcoding_failed"), "[$!][$res].") if ($? != 0);

    #si el archivo de destino existe, lo borramos
    if (-f $destino) {
        unlink $destino;
    }

    #si existe ruta temporal de trabajo se mueve el archivo a su destino final
    if ($lib_xcoding::RUTA_TEMPORAL ne '')  {
        # se mueve el mp3 a su destino final
        File::Copy::move("$lib_xcoding::RUTA_TEMPORAL$archivo_destino",$destino);
    } else {
        #sino se renombra de tmp a mp3
        File::Copy::move("$RUTA_PRONTUS$archivo_destino",$destino);
    }

    if (-f $destino) {
        #se borra el original si no es mp4, si es mp4 en esta etapa ya es el destino
        unlink $ORIGEN if ($ORIGEN !~ /\.mp3$/i );
    } else {
        lib_xcoding::die_stderr(lib_lang::get_message('xcoding_audio_error_destination_file'), "");
    }
}

# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}

# realiza tareas adicionales al terminar
sub tareas_salida {
    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] [$ARTIC_filename] " . lib_lang::get_message('global_end_process') . "\n";
    # si esta definido un post proceso lo ejecutamos en segundo plano
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} ne '') {
        if ($ARTIC_ts_articulo ne '' && $PRONTUS_ID ne '') {
            system("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} $PRONTUS_ID $ARTIC_ts_articulo >/dev/null 2>&1 &");
        }
    }
    if ($LOCK ne '') {
        my ($status, $result, $data) = lib_waitlock::unlock_file($LOCK, $LOCKFH);
        if (!$status) {
            print STDERR lib_lang::get_message($result). $data, "\n";
        }
    }
}

# actualiza el articulo
sub actualizar_articulo {
    my $marca = $_[0];

    my $artic_obj;
    unless ($artic_obj = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $ARTIC_ts_articulo,
                'no_check_dirs'      => 1)
        ) {
        return lib_lang::get_interpolated_message('global_error_artic_init', "$Artic::ERR TS[$ARTIC_ts_articulo]");
    }

    # se asegura el xml mientras se edita y guarda
    $LOCK = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/$ARTIC_ts_articulo.mutex";
    my ($status, $data);
    ($status, $LOCKFH, $data) = lib_waitlock::lock_file($LOCK);
    if (!$status) {
        # LOCKFH contiene string en caso de error
        return lib_lang::get_message($LOCKFH). $data;
    }

    $artic_obj->{xml_data} = glib_fildir_03::read_file($artic_obj->{fullpath_xml});
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, $marca, "$ARTIC_filename.mp3", 0);
    my $audio_size = -s "$artic_obj->{dst_multimedia}/$ARTIC_filename.mp3";

    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.s3", '', 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, '_s3', 0, 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "_modtime", time, 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.bytes", $audio_size, 0);

    $audio_size += -s "$artic_obj->{dst_multimedia}/$ARTIC_filename.json";
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.total", $audio_size, 0);

    # actualizamos tamaño del artículo
    # este dato solo se puede obtener desde el xml
    my $s3_size = 0;
    if ($artic_obj->{xml_data} =~ /<_s3_size>(\d+)<\/_s3_size>/) {
        $s3_size = $1;
    }

    my ($disk_size, $imag_size, $asset_size, $media_size) = $artic_obj->get_disk_size();
    my $total_size = $s3_size + $disk_size;
    $artic_obj->update_art_size($total_size, $disk_size, $imag_size, $asset_size, $media_size);

    $artic_obj->_flush_xml();
    # xml de articulo ya se actualizó, se libera el mutex
    my $result;
    ($status, $result, $data) = lib_waitlock::unlock_file($LOCK, $LOCKFH);
    if (!$status) {
        print STDERR lib_lang::get_message($result). $data, "\n";
    }

    # actualiza datos en bd
    my ($bd, $msg) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return $msg;
    }
    $artic_obj->art_update_bd($bd);

    $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo} = $artic_obj;

    # debemos generar la metadata
    my $fullpath_audio = $ORIGEN;
    $fullpath_audio =~ s/\.\w+$/\.mp3/is;
    lib_xcoding::genera_info_media_audio($fullpath_audio);

    # Generar vista (a partir del xml)
    $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art('') || return $Artic::ERR;

    my %datos = lib_prontus::getCamposXml($artic_obj->{xml_data}, '_fid,_plt');
    my $fid = $datos{'_fid'};
    my $plt = $datos{'_plt'};

    # Parsear plantillas paralelas.
    foreach my $plt_paralela (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$fid}{'plt_paralela'}}) {
        $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art('', $plt_paralela, 1) || return $Artic::ERR;
    }

    # Generar vistas secundarias
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
        $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art($mv) || return $Artic::ERR;
    }

    # Actualizar el DAM
    my $ext = lib_utils::get_file_extension($plt);
    my $fullpath_artic = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/artic/$ARTIC_dirfecha/pags/$ARTIC_ts_articulo.$ext";
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";
    my $cmd = "$rutaScript/dam/prontus_dam_ppart_save.pl $fullpath_artic >/dev/null 2>&1 &";
    print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
    system $cmd;

    # Dropbox.
    lib_prontus::call_dropbox_backup("art;$ARTIC_ts_articulo");

    return '';
}

sub die_stderr {
    my $msg = $_[0];
    my $detalle = $_[1];
    my $write = $_[2];
    &write_status($msg) if ($write);
    print STDERR "[ERROR] $msg - $detalle";
    exit 1;
}

sub write_status {
    if ($ARTIC_ts_articulo ne '' && $prontus_varglb::PRONTUS_ID ne 'prontus') {
        my $msg = $_[0];
        $msg =~ s/\n//sg;
        my $file = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/procs/xcoding_status_$ARTIC_ts_articulo.txt";

        glib_fildir_03::write_file($file, $msg);
    }
}
