#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# prontus_videos_metadata_upd.pl
# ---------------------------------------------------------------
# UBICACION.
# -----------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Actualiza el json de metadata asociado a los videos, sin transcodificar
# o ejecutar algun otro proceso
# ---------------------------------------------------------------
# LLAMADAS A SCRIPTS.
# ------------------------
# Usa las cgis de transcodificacion de prontus
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# A traves de linea de comando, ej:
# Parametros:
# 1: nombre del prontus a procesar
# 2: identificador del fid a procesar
# 3: marca o marcas del video, si es mas de una se separan con coma,
#  ej: multimedia_video1,multimedia_video2
#
# Ejemplo de uso:
# perl prontus_video_metadata_upd.pl mi_prontus_id fid_noticia multimedia_video1,multimedia_video2
# ---------------------------------------------------------------
# ARCHIVOS DE ENTRADA.
# ------------------------
# ---------------------------------------------------------------
# ARCHIVOS DE SALIDA.
# ------------------------
# ---------------------------------------------------------------
# Tablas.
# ------------------------
# No utiliza.
# ---------------------------------------------------------------

# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    unshift(@INC,$Bin); # Para dejar disponibles las librerias
    my $pathLibsProntus = $Bin;
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use File::Copy;
use JSON;
use POSIX;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_lang;
use lib_prontus;
use lib_utils;
use lib_xcoding;
##/libs##

# ---------------------------------------------------------------
# MAIN.
# ---------------------------------------------------------------
my $separator_orig = '   ';
my $separator = '';
my $COUNTER = 0;

my @BUSCAR;
my @CAMPOS;
my $BUSCAR_CAMPOS = 0;
my $FID = '';
my $PRONTUS_ID = '';
my $TS = '';
my $input;

main:{
    lib_utils::setUp();
    lib_xcoding::init();

    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_use_instructions');
        print lib_lang::get_interpolated_message("video_metadata_upd_use1", $0);
        print lib_lang::get_interpolated_message("video_metadata_upd_use2", $0);
        print lib_lang::get_interpolated_message("video_metadata_upd_use3", $0);
        exit;
    }
    $PRONTUS_ID = $ARGV[0];

    if (! lib_utils::valida_prontus($PRONTUS_ID)) {
        print lib_lang::get_message('global_invalid_prontus_id')."\n";
        exit;
    }

    if ($ARGV[1] ne '' && $ARGV[1] !~ /^fid\_/ && $ARGV[1] !~ /^\d{14}$/) {
        print lib_lang::get_message("video_metadata_upd_invalid_param2");
    }

    # cargamos configuracion de prontus
    lib_utils::load_config_by_id($PRONTUS_ID);

    if ($ARGV[1] =~ /^\d{14}$/) {
        $TS = $ARGV[1];
    } elsif ($ARGV[1] =~ /^fid\_/) {
        $FID = $ARGV[1];
    }

    if ($ARGV[2] eq '') {
        $BUSCAR_CAMPOS = 1;
    } else {
        @BUSCAR = split(',', $ARGV[2]);
    }

    $input = $Bin;
    $input =~ s/\/[^\/]+\/?$//;
    $input =~ s/\/[^\/]+\/?$//;
    $input .= "/$PRONTUS_ID/site/artic/";

    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] ". lib_lang::get_interpolated_message('video_metadata_upd_start', $PRONTUS_ID, $TS, $FID);

    if ($TS ne '') {
        $input .= substr($TS, 0, 8). "/xml/";
        &procesa_file("$TS.xml", $input);
    } else {
        &procesa_dir($input);
    }
    print STDERR lib_lang::get_interpolated_message('video_metadata_upd_processed_num', $COUNTER) if $COUNTER;
    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] ". lib_lang::get_interpolated_message('video_metadata_upd_end', $PRONTUS_ID, $TS, $FID);
}

#-----------------------------------------------------------------------#
sub procesa_dir {
    my($input) = $_[0];

    unless(-d $input) {
        print lib_lang::get_interpolated_message('global_missing_dir', $input);
        return;
    }
    $input = $input . '/' unless ($input =~ /\/$/);

    my @entries = glib_fildir_03::lee_dir($input);
    foreach my $k (@entries) {
        next if($k eq '.' || $k eq '..');
        if(-f $input . $k) {
            &procesa_file($k, $input);

        } elsif(-d $input . $k) {
            $separator = $separator . $separator_orig;
            &procesa_dir($input . $k);
        }
    }
    $separator =~ s/$separator_orig$//;
}

#-----------------------------------------------------------------------#
sub procesa_file {
    my($filename, $path) = ($_[0], $_[1]);
    my $filepath = $path . $filename;
    my $titular;
    my $ts;
    if ($filename !~ /^(\d{14})\.xml/i) {
        return;
    }
    $ts = $1;

    if (-f $filepath) {
        my $file = glib_fildir_03::read_file($filepath);
        if ($FID ne ''  && $file !~ /$FID/is) {
            return;
        }
        if ($BUSCAR_CAMPOS) {
            my %hash_campos;
            while($file =~ /<(multimedia_\w+)>/ig) {
                $hash_campos{$1} = 1;
            }
            @BUSCAR = keys %hash_campos;
        }
        my %data;
        my $cmd = '';
        my $result = '';
        my $result2 = '';
        my $media = '';
        foreach my $campo (@BUSCAR) {
            my $found = '';
            if ($file =~ /<$campo>([^<>]+)<\/$campo>/is) {
                $found = $1;
            } elsif ($file =~ /<$campo>.*?<\!\[CDATA\[(.*?)\]\]>.*?<\/$campo>/is) {
                $found = $1;
            }
            $found =~ s/[\n\r]//g;

            if ($file =~ /<_txt_titular>.*?<!\[CDATA\[(.*?)\]\]>.*?<\/_txt_titular>/is) {
                $titular = $1;
            }
            if ($found =~ /video/) {
                print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] ". lib_lang::get_interpolated_message('video_metadata_upd_updating', $ts, $titular, $campo, $found);
                $media =  "$prontus_varglb::DIR_SERVER/$PRONTUS_ID/site/mm/". substr($ts ,0, 8) ."/mmedia/$found";
                next if (!-f $media);

                lib_xcoding::genera_info_media($media);
            } elsif ($found =~ /audio/) {
                print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] ". lib_lang::get_interpolated_message('video_metadata_upd_updating', $ts, $titular, $campo, $found);;
                $media =  "$prontus_varglb::DIR_SERVER/$PRONTUS_ID/site/mm/". substr($ts ,0, 8) ."/mmedia/$found";
                next if (!-f $media);

                lib_xcoding::genera_info_media_audio($media);
            }
        }
    }
}
