#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
#
# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Realiza efectivamente la transcodificacion de un video de avi, wmv, mp4, mpeg, rm y otros a mp4.
# Cuando termina, deduce donde esta el xml del articulo Prontus y modifica las variables.
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# Al terminar elimina el video original y sustituye el nombre en el xml del articulo Prontus.
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Por linea de comandos.
# Los parametros son:
# 0- Path absoluto al video a transcodificar.
# 1- prontus id
#
# /usr/bin/perl prontus_videodoxcode.pl /sites/prontus_lab/web/prontus_proto/site/artic/20100531/mmedia/multimedia_video120100531182139.avi prontus_proto
#
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
#  No usa plantillas.
# ---------------------------------------------------------------
use utf8;
use strict;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

END {
    &garbageTempFiles();
    &tareas_salida();
}

use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Artic;
use DBI;
use Digest::MD5;
use File::Copy;
use HTML::Entities;
use JSON;
use LWP::ConnCache;
use LWP::UserAgent;
use POSIX;
use Time::Local;
use URI::Escape;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_friendly;
use lib_helper;
use lib_lang;
use lib_mam;
use lib_parser;
use lib_prontus;
use lib_tags;
use lib_tax;
use lib_urlf5;
use lib_utils;
use lib_waitlock;
use lib_xcoding;
##/libs##
# use Data::Dumper;

my $ORIGEN = $ARGV[0];
my $PRONTUS_ID = $ARGV[1];
my $GENERAR_VERSIONES = $ARGV[2];
my $ARTIC_dirfecha = '';
my $ARTIC_ts_articulo = '';
my $ARTIC_filename = '';
my $ARTIC_media_path = '';
my $RUTA_PRONTUS = '';  # ruta de la carpeta prontus donde esta el video en caso de no usar ruta temporal
my %FORMATOS_VERSIONES = (); # hash que guarda los formatos de las versiones del video
my $VERSIONS_FILE = ''; # archivo temporal para crear las versiones del video original
my $MARCA = ''; # nombre de la marca prontus del video
my $CLIENT = undef; # user agent para hacer consultas remotas
my $SLEEP = 5; # tiempo de espera entre peticiones remotas

my $LOCK = '';
my $LOCKFH;

main: {
    lib_utils::setUp();
    lib_xcoding::init();

    lib_xcoding::die_stderr(lib_lang::get_message("video_invalid_origin"), "") if ((!-f "$ORIGEN") || (!-s "$ORIGEN"));
    lib_xcoding::die_stderr(lib_lang::get_message("global_invalid_id"), "") if (! lib_utils::valida_prontus($PRONTUS_ID));

    $GENERAR_VERSIONES = 0 if (!$GENERAR_VERSIONES);
    my ($cmd, $res);
    my ($total, $segundos);
    my $start = time; # tiempo inicial para medir la duracion del proceso

    # Deduce ubicacion del xml del articulo.
    if ($ORIGEN =~ /(.+\/(.+?)\/(\d{8})\/mmedia\/)(multimedia_video\w+(\d{6}))\.(\w+)$/i) {
        $ARTIC_media_path = $1;
        $ARTIC_dirfecha = $3;
        $ARTIC_ts_articulo = $3 . $5;
        $ARTIC_filename = $4;
    } else {
        lib_xcoding::die_stderr(lib_lang::get_interpolated_message("videodoxcode_info_unobtainable", join(';', @ARGV)));
    }

    $ORIGEN =~ /\/mmedia\/(multimedia_video\w+)\d{14}\.(\w+)$/i;
    $MARCA = $1;

    # Path conf y load config de prontus
    my $msg_load = lib_utils::load_config_by_id($PRONTUS_ID);
    if ($msg_load ne '') {
        print STDERR "$msg_load\n";
        exit;
    }

    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] [$ARTIC_filename] " . lib_lang::get_message('global_start_process') . "\n";
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_XCODER_EXTERNO'} eq 'SI') {
        print STDERR lib_lang::get_message("videodoxcode_external_xcoder");
        my ($content, $archivo, $result, $respuesta);

        lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_invalid_xcoder_host"), "") if ( $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_HOST'} eq '');

        # inicializamos el user agent
        &initUA();
        # armamos los datos de la peticion
        my $host = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
        my $xcoder_host = 'http://' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_HOST'};
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_PORT'} ne '') {
            $xcoder_host .= ':'.$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_PORT'};
        }

        my $path_video = $ORIGEN;
        if ($path_video !~ s/[\w\.\-\/]*(\/[^\/]+\/site\/\w+\/\d+\/mmedia\/multimedia_video\w+\d{14}\.\w+)$/$1/i) {
            lib_xcoding::die_stderr(lib_lang::get_interpolated_message("videodoxcode_error_adapting_origin_path", $path_video), "");
        }

        my $hls = 0;
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'GEN_HLS'} eq 'SI') {
            $hls = 1;
        }
        # datos del trabajo
        # valores limite
        my %max = ( "max_video_bitrate" => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_VIDEO_BITRATE'},
                    "max_audio_bitrate" => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'},
                    "max_videosize"     => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PIXEL'} );

        my %versions = lib_xcoding::get_all_formatos($MARCA, $ARTIC_ts_articulo, 1);

        my %profiles = (
                    'max'=> \%max,
                    'versions' => \%versions);

        my %data = ('host' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'}."://$host",
                    'prontus_id' => $PRONTUS_ID,
                    'path' => $path_video,
                    'tag' => $MARCA,
                    'ts' => $ARTIC_ts_articulo,
                    'multi_version' => $GENERAR_VERSIONES,
                    'profiles' => \%profiles,
                    'hls' => $hls );

        my $json = JSON::encode_json(\%data);
        utf8::decode($json);
        my %post_data = ('data'   => $json);

        # registramos el trabajo con el servidor de xcoding
        $respuesta = &postUrl("$xcoder_host/cgi-cpn/xcoding/xcoder_add_job.cgi", \%post_data);

        if ($respuesta ne 'ERROR') {
            utf8::encode($respuesta);
            eval { $result = JSON::decode_json($respuesta); };
            if ($@) {
                print STDERR lib_lang::get_interpolated_message('global_json_decode_error', $@ . '[xcoder_add_job.cgi]', $respuesta), "\n";
                exit;
            }

            # si el estado es 1 el trabajo se agrego correctamente
            # se entra en modo de espera que termine
            if (defined($$result{'status'}) && $$result{'status'}) {
                print STDERR lib_lang::get_message("videodoxcode_job_added");
                my $pendiente = 1;
                my $id = $$result{'id'};
                # esperamos ante sde consultar por primera vez
                sleep($SLEEP);
                while ($pendiente) {
                    %post_data = ('id' => $id);

                    # consultamos el estado del trabajo
                    $respuesta = &postUrl("$xcoder_host/cgi-cpn/xcoding/xcoder_check_job.cgi", \%post_data);

                    #~ print STDERR $respuesta;
                    if ($respuesta ne 'ERROR') {
                        $result = ();
                        utf8::encode($respuesta);
                        eval { $result = JSON::decode_json($respuesta); };
                        if ($@) {
                            print STDERR lib_lang::get_interpolated_message('global_json_decode_error', $@ . '[xcoder_check_job.cgi]', $respuesta), "\n";
                            exit;
                        }

                        # si el trabajo esta listo, terminamos el bucle y descargamos los archivos
                        if ($$result{'trabajo'} == 2) {
                            $pendiente = 0;
                            foreach my $file (@{$$result{'files'}}) {
                                if ($file =~ /\/download\/\Q$prontus_varglb::PRONTUS_ID\E\/(.+\.\w+)$/) {
                                    my $destino = "$ARTIC_media_path$1";
                                    # si existe el archivo de destino se borra antes
                                    unlink $destino if (-f $destino);

                                    # revisa y crea subdirectorios faltantes
                                    if ($destino =~ /(.+)\/\w+\.\w+$/) {
                                        my $dir = $1;
                                        glib_fildir_03::check_dir($dir);
                                    }
                                    my ($success, $msg) = &getFile("$xcoder_host$file", $destino);
                                    if (!$success) {
                                        lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_error_downloading_file"), "[$xcoder_host$file][$msg]");
                                    }

                                } else {
                                    lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_invalid_path"), $file);
                                }
                            }

                            # borramos original si no es un mp4
                            unlink $ORIGEN if (substr($ORIGEN, -4) ne '.mp4');
                            # avisa a xcoder que ya se descargo todo para que borre los archivos generados
                            $respuesta = &postUrl("$xcoder_host/cgi-cpn/xcoding/xcoder_cleanup_job.cgi", \%post_data);

                            last;
                        } elsif ($$result{'trabajo'} == 3) {
                            lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_error_remote_processing"), $respuesta);
                        }
                    } else {
                        lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_error_remote_server_verify"), $respuesta);
                    }
                    sleep($SLEEP);
                }
                # Actualizar articulo.
                &actualizar_articulo($MARCA, 1);
                # se purgean los archivos generados
                &purgeExterno();
                print STDERR lib_lang::get_message("videodoxcode_job_ended");
            }
        } else {
            lib_xcoding::die_stderr((lib_lang::get_message("videodoxcode_error_remote_server_add")), $respuesta);
        }
        # terminamos la cgi
        exit;
    }

    # se inicializa el uso de libfdk_aac, es comun para ambos metodos
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_LIB_FDK'} eq 'SI') {
        $lib_xcoding::FDK = 1;
    }

    # Forma el nombre de la pelicula destino sustituyendo la extension
    my $destino = $ORIGEN;
    $destino =~ s/\.\w+$/\.mp4/;

    # para obtener la carpeta prontus del video
    $destino =~ /(\/.*\/\d{8}\/mmedia\/)multimedia_video.*\d{6}\S?\.\w+$/i;
    $RUTA_PRONTUS = $1;
    #~ print STDERR "RUTA_PRONTUS [$RUTA_PRONTUS]\n";

    # inicializamos los valores de la libreria
    lib_utils::get_path_nice();
    $lib_xcoding::PATHNICE = "$prontus_varglb::PATHNICE -n19 " if ($prontus_varglb::PATHNICE);
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'GEN_HLS'} eq 'SI') {
        $lib_xcoding::HLS = 1;
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRECISION_HLS'} eq 'SI') {
        $lib_xcoding::PRECISION_HLS = 1;
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MODO_PARALELO'} eq 'SI') {
        $lib_xcoding::MODO_PARALELO = 1;
    }
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'N_THREADS'} =~ /^\d+$/) {
        $lib_xcoding::N_THREADS = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'N_THREADS'};
    }
    $lib_xcoding::MAX_PIXEL = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PIXEL'};
    $lib_xcoding::MAX_VRATE = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_VIDEO_BITRATE'};
    $lib_xcoding::MAX_ARATE = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'};
    $lib_xcoding::RUTA_TEMPORAL = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'};
    $lib_xcoding::RUTA_PRONTUS = $RUTA_PRONTUS;
    $lib_xcoding::ARTIC_filename = $ARTIC_filename;
    $lib_xcoding::ARTIC_ts_articulo = $ARTIC_ts_articulo;
    # se resta 1 para descontar el proceso actual
    $lib_xcoding::MAX_PARALELO = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PARALELO'} - 1;

    #se obtienen los datos del video original
    ($lib_xcoding::ANCHO, $lib_xcoding::ALTO, $lib_xcoding::VCODEC, $lib_xcoding::ACODEC, $lib_xcoding::VBITRATE, $lib_xcoding::ABITRATE) = lib_xcoding::get_info_media($ORIGEN);

    # cargamos los formatos para la marca para saber si hay que generar versiones y
    # necesitamos conservar el archivo original o no
    %FORMATOS_VERSIONES = lib_xcoding::get_formatos($MARCA);

    if ($GENERAR_VERSIONES) {
        if (scalar(keys %FORMATOS_VERSIONES) > 0) {
            &crear_versiones_video($MARCA, $ORIGEN, $destino);
        }
        # si es modo paralelo debemos esperar que terminen los otros procesos
        if ($lib_xcoding::MODO_PARALELO) {
            &esperar_procesos();
        }
        # Actualizar articulo.
        &actualizar_articulo($MARCA, 1);
        if ($lib_xcoding::HLS) {
            &generar_lista_HLS();
        }
        $total = time - $start;
        $segundos = $total % 60;
        $segundos = $segundos < 10? '0'.$segundos : $segundos;
        print STDERR lib_lang::get_interpolated_message('xcoding_total_time', $ARTIC_filename, int($total/60), $segundos);
        exit;
    }

    # Se transcodifican mp4s si el bitrate de video es muy alto
    # Sin embargo, se aplica la correccion de los headers y offset, moviendolo al comienzo
    if ($ORIGEN =~ /\.mp4$/i) {
        if ($lib_xcoding::VBITRATE <= $lib_xcoding::MAX_VRATE && $lib_xcoding::ABITRATE <= $lib_xcoding::MAX_ARATE && !$lib_xcoding::HLS) {
            print STDERR lib_lang::get_message("videodoxcode_no_xcoding_mp4_low_bitrate");
            print STDERR lib_lang::get_message("videodoxcode_moving_headers");

            $cmd = "/usr/bin/perl $Bin/qtfaststart.pl $ORIGEN";
            $res = qx/$cmd 2>&1/;

            lib_xcoding::die_stderr(lib_lang::get_message('videodoxcode_adjust_fail'), "[$!][$res][$cmd]") if ($? != 0);

            # actualizamos metadata de video
            lib_xcoding::genera_info_media($ORIGEN);
            lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_xcoding_unnecessary"), "");
        }
    }

    # si hay formatos > 0 copiamos el archivo original para usarlo al generar las versiones
    # para no tener que realizar un nuevo analisis para transcodificar
    # si no hay directorio de trabajo se guarda en la misma ubicacion del original
    if ($lib_xcoding::RUTA_TEMPORAL eq '')  {
        $VERSIONS_FILE = "$RUTA_PRONTUS$ARTIC_filename.prontustmp";
    } else {
        # si hay directorio de trabajo se guarda ahi
        $VERSIONS_FILE = "$lib_xcoding::RUTA_TEMPORAL$ARTIC_filename.prontustmp";
    }
    if (scalar(keys %FORMATOS_VERSIONES) > 0) {
        # si existe el archivo temporal, se borra
        if (-e $VERSIONS_FILE) {
            unlink $VERSIONS_FILE;
        }
        File::Copy::copy($ORIGEN, $VERSIONS_FILE) or lib_xcoding::die_stderr(lib_lang::get_interpolated_message("videodoxcode_copy_failed", $VERSIONS_FILE), "[$!][$res].") ;
    }

    # cargamos los formatos por defecto si estan redefinidos
    my %formatos = lib_xcoding::get_formatos($MARCA, 1);
    # realizamos la transcoficacion del original
    &do_xcode($ORIGEN, $destino, 0, $formatos{uc $MARCA}, ''); # $key es vacio

    # si existe el mp4
    if (-s $destino) {
        # Actualizamos el articulo  articulo. solo guardamos el xml en el articulo.
        &actualizar_articulo($MARCA, 0);
        # para obtener la carpeta prontus relativa del video
        $destino =~ /\/.*(\/.*?\/site\/\w+\/\d{8}\/mmedia\/multimedia_video.*\d{6}\S?\.\w+)$/i;
        lib_prontus::purge_cache($1);
    } else {
        # sino terminamos
        lib_xcoding::die_stderr(lib_lang::get_message("videodoxcode_error_generating"), $destino);
    }

    # segmentamos el video para generar HLS
    if ($lib_xcoding::HLS) {
        lib_xcoding::generar_HLS($destino);
    }

    # si no es modo paralelo se procesan las versiones
    if (!$lib_xcoding::MODO_PARALELO) {
        # si hay formatos para versiones generamos las versiones indicadas
        if (scalar(keys %FORMATOS_VERSIONES) > 0) {
            # se obtienen los datos del video creado para generar las versiones de este
            ($lib_xcoding::ANCHO, $lib_xcoding::ALTO, $lib_xcoding::VCODEC, $lib_xcoding::ACODEC, $lib_xcoding::VBITRATE, $lib_xcoding::ABITRATE) = lib_xcoding::get_info_media($destino);

            # Crear versiones adicionales del video.
            &crear_versiones_video($MARCA, $VERSIONS_FILE, $destino);
        }
    } else {
        # si es modo paralelo debemos esperar que terminen los otros procesos
        &esperar_procesos();
    }

    if ($lib_xcoding::HLS) {
        &generar_lista_HLS();
    }

    # Actualizar articulo.
    &actualizar_articulo($MARCA, 1);

    $total = time - $start;
    $segundos = $total % 60;
    $segundos = $segundos < 10? '0'.$segundos : $segundos;
    print STDERR lib_lang::get_interpolated_message('xcoding_total_time', $ARTIC_filename, int($total/60), $segundos);
}
# ---------------------------------------------------------------
# determina cuantos procesos en paralelo estan corriendo para este video
sub cuenta_procesos {
    my $origen =  $_[0];
    my $res = `ps auxww |grep 'prontus_videoxcodeparallel.pl $origen' | grep -v grep | wc -l`;
    #~ print STDERR "cuenta_procesos [$res]\n";
    if ($res eq '') {
        return 0;
    } else {
        return $res + 0;
    }
}
# ---------------------------------------------------------------
# espera que terminen los procesos que estan corriendo en background
sub esperar_procesos {
    # si es modo paralelo debemos esperar que terminen los otros procesos
    while (1) {
        my $res = `ps auxww |grep 'prontus_videoxcodeparallel.pl' | grep $PRONTUS_ID | grep $ARTIC_filename |grep -v grep | grep -v tail`;
        #~ print STDERR "[$res]\n";
        # si ya no hay procesos corriendo terminamos
        if ($res eq '') {
            last;
        }
        # esperamos segundos antes de volver a chequear
        sleep(5);
    }
}
# ---------------------------------------------------------------
sub crear_versiones_video {
    my $marca = $_[0];
    my $origen = $_[1];
    my $destino = $_[2];

    print STDERR lib_lang::get_interpolated_message("videodoxcode_versioning", $marca);

    foreach my $key (sort keys(%FORMATOS_VERSIONES)) {
        # si estamos procesando en modo paralelo se lanzan las versiones en background
        if ($lib_xcoding::MODO_PARALELO) {
            while (&cuenta_procesos($origen) > $lib_xcoding::MAX_PARALELO) {
                # print STDERR "Muchos procesos, esperando...\n";
                # esperamos que termine algun proceso antes de lanzar mas
                sleep(10);
            }
            my $cmd = "$lib_xcoding::PATHNICE /usr/bin/perl $Bin/prontus_videoxcodeparallel.pl $origen $PRONTUS_ID $key $RUTA_PRONTUS";
            print STDERR lib_lang::get_interpolated_message("videodoxcode_parallel_xcoding", $origen, $key);
            system("$cmd >/dev/null 2>&1 &");
        } else {
            my $new_name = $key;
            $new_name =~ s/\./$ARTIC_ts_articulo/sg;
            $new_name = lc $new_name;
            my $new_destino = $destino;
            $new_destino =~ s/\/multimedia_video\w+\d{14}\.(\w+)$/\/$new_name\.mp4/is;
            print STDERR lib_lang::get_interpolated_message("videodoxcode_processing_format", $key, $new_destino);

            if (-f $new_destino) {
                unlink $new_destino;
            };
            &do_xcode($origen, $new_destino, 1, $FORMATOS_VERSIONES{$key}, $key);
            # para obtener la carpeta prontus relativa del video
            $new_destino =~ /\/.*(\/.*?\/site\/\w+\/\d{8}\/mmedia\/multimedia_video.*\d{6}\S?\.\w+)$/i;
            # purgeamos el archivo
            lib_prontus::purge_cache($1);
            # segmentamos el video para generar HLS
            if ($lib_xcoding::HLS) {
                lib_xcoding::generar_HLS($new_destino);
            }
        }
    }
}
# ---------------------------------------------------------------
sub do_xcode {
    my $origen = $_[0];
    my $destino = $_[1];
    my $no_borr_origen = $_[2];
    my $formato = $_[3];
    my $key = $_[4];

    # obtenemos y guardamos el nombre de la marca
    if ($key =~ /\.(\w+)$/) {
        $key = lc $1;
    }

    my ($cmd, $res, $nd_pass, $start, $end, $total, $segundos);

    $destino =~ /\/\d{8}\/mmedia\/(multimedia_video.*\d{6}\S?\.mp4)$/i;
    my $archivo_destino = $1;
    $archivo_destino =~ s/\.mp4$/\.tmp/i;

    ($cmd,$nd_pass) = lib_xcoding::get_cmd_ffmpeg($origen, $archivo_destino, 0, \%{$formato});

    # se determina la ubicacion del archivo de estadisticas
    my $stats_file = '';
    if ($lib_xcoding::RUTA_TEMPORAL eq '')  {
        $stats_file = "$RUTA_PRONTUS$ARTIC_filename.log";
    } else {
        $stats_file = "$lib_xcoding::RUTA_TEMPORAL$ARTIC_filename.log";
    }

    # cargamos el formato por defecto de la marca para verificar si se cambio el profile
    # si el profile no es main se debe hacer los 2 pasos al transcodificar
    my %formato_defecto = lib_xcoding::get_formatos($MARCA, 1);

    my $perfil_defecto = 'main'; # por defecto se usa main
    if (defined($formato_defecto{'h264profile'}) && $formato_defecto{'h264profile'} ne 'main')  {
        $perfil_defecto = $formato_defecto{'h264profile'};
    }

    # verificamos si es main o no
    my $perfil_formato = 'main'; # por defecto se usa main
    if (defined($formato->{'h264profile'}) && $formato->{'h264profile'} ne 'main')  {
        $perfil_formato = $formato->{'h264profile'};
    }

    my $paso_1 = 0;
    # si el perfil de la version principal es distinto al de la version general se debe hacer el paso 1
    if ($perfil_defecto ne $perfil_formato) {
        print STDERR lib_lang::get_interpolated_message("videodoxcode_error_different_version_profile", $MARCA, uc($key), $perfil_defecto, $perfil_formato);
        $paso_1 = 1;
    }

    my $nuevolog = 0;
    # si no hay que hacer 2 pasos se procesa siempre
    # o si hay que hacer 2 pasos y ya existe el log de estadisticas nos saltamos el paso 1
    # $stats_file-0.log es el log que genera ffmpeg 1
    #~ if (!$nd_pass || !($nd_pass && (-s $stats_file))) {
    if ((!$nd_pass || !($nd_pass && (-s $stats_file || -s "$stats_file-0.log"))) || $paso_1) {
        # si no existe el archivo de estadisticas debemos crear uno unico para este proceso
        if (!-s $stats_file || $paso_1) {
            # cambiamos el nombre del log en el comando
            $cmd =~ s/\.log/\.log$key/g;
            # hay que usar este log para el paso 2 tambien
            $nuevolog = 1;
        }
        print STDERR lib_lang::get_interpolated_message("videodoxcode_pass_1", $cmd);
        $start = time;
        # Ejecuta la transcodificacion redirigiendo stderr to stdout.
        # Por ahora no se analiza la salida del ffmpeg. La redireccion del stderr al stdout es porque ffmpeg imprime su salida al stderr en vez de al stdout
        $res = `$cmd 2>&1`;

        $end = time;
        $total = $end - $start;
        $segundos = $total % 60;
        $segundos = $segundos < 10? '0'.$segundos : $segundos;
        print STDERR lib_lang::get_interpolated_message('xcoding_ffmpeg_time', $ARTIC_filename, int($total/60), $segundos);
        lib_xcoding::die_stderr(lib_lang::get_message("xcoding_failed"), "[$!][$res].") if ($? != 0);
    }

    # si no hay que borrar el origen, se esta haciendo el video principal
    # despues del analisis podemos lanzar las versiones en paralelo
    if (!$no_borr_origen && $lib_xcoding::MODO_PARALELO) {
        &crear_versiones_video($MARCA, $VERSIONS_FILE, $destino);
    }

    # se hace el segundo paso si es necesario
    if ($nd_pass) {
        ($cmd,$nd_pass) = lib_xcoding::get_cmd_ffmpeg($origen, $archivo_destino, 1, \%{$formato});
        if ($nuevolog) {
            # cambiamos el nombre del log en el comando
            $cmd =~ s/\.log/\.log$key/g;
        }
        print STDERR lib_lang::get_interpolated_message("videodoxcode_pass_2", $cmd);
        $start = time;
        $res = `$cmd 2>&1`;

        $end = time;
        $total = $end - $start;
        $segundos = $total % 60;
        $segundos = $segundos < 10? '0'.$segundos : $segundos;
        print STDERR lib_lang::get_interpolated_message('xcoding_ffmpeg_time', $ARTIC_filename, int($total/60), $segundos);
        lib_xcoding::die_stderr(lib_lang::get_message("xcoding_failed"), "[$!][$res].") if ($? != 0);
    }

    #si el archivo de destino existe, lo borramos
    if (-f $destino) {
        unlink $destino;
    }

    #si existe ruta temporal de trabajo se mueve el archivo a su destino final
    my $adjust_failed = lib_lang::get_message('videodoxcode_adjust_fail');
    if ($lib_xcoding::RUTA_TEMPORAL ne '')  {
        #se ajusta el mp4
        $cmd = "/usr/bin/perl $Bin/qtfaststart.pl $lib_xcoding::RUTA_TEMPORAL$archivo_destino";
        $res = `$cmd 2>&1`;

        lib_xcoding::die_stderr("1 $adjust_failed", "[$!][$res][$cmd]") if ($? != 0);

        # se mueve el mp4 a su destino final
        File::Copy::move("$lib_xcoding::RUTA_TEMPORAL$archivo_destino",$destino);
    } else {
        #se ajusta el mp4
        $cmd = "/usr/bin/perl $Bin/qtfaststart.pl $RUTA_PRONTUS$archivo_destino";
        $res = `$cmd 2>&1`;

        lib_xcoding::die_stderr("2 $adjust_failed", "[$!][$res][$cmd]") if ($? != 0);

        #sino se renombra de tmp a mp4
        File::Copy::move("$RUTA_PRONTUS$archivo_destino",$destino);
    }

    if (-f $destino) {
        #se borra el original si no es mp4, si es mp4 en esta etapa ya es el destino
        unlink $ORIGEN if (!$no_borr_origen && $ORIGEN !~ /\.mp4$/i );
    } else {
        lib_xcoding::die_stderr(lib_lang::get_message("video_endfile_not_generated"), "");
    }
}
# ---------------------------------------------------------------
# genera la playlist que contiene las listas de las variantes del video
sub generar_lista_HLS {
    my ($ancho, $alto, $vcodec, $acodec, $vbitrate, $abitrate, $fps, $duration, $fullpath, $hlspath);
    my @archivos  = glob "$RUTA_PRONTUS$ARTIC_filename*";

    my $ult_resolucion = '';
    my %list_items = ();
    foreach my $archivo (@archivos) {
        # se busca la informacion del mp4 asociado
        # /var/www/prontus_development/prontus_develop/site/artic/20140814/mmedia/multimedia_video120140814172639.mp4
        if ($archivo !~ /(\/.*(\/[^\/]+\/[^\/]+\/[^\/]+\/\d{8}\/mmedia\/multimedia_video\w+\d{14}\S*))\.mp4$/i) {
            next;
        }
        $hlspath = $2;
        $fullpath = $1;

        if (-s "$fullpath/playlist.m3u8") {
            # se obtienen los datos del video para usar en la lista
            ($ancho, $alto, $vcodec, $acodec, $vbitrate, $abitrate, $fps, $duration) = lib_xcoding::get_info_media($archivo);
            if ($ult_resolucion ne "$ancho"."x$alto") {
                my $item .= "#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=". 1100*($vbitrate +$abitrate).",RESOLUTION=$ancho"."x$alto";
                # etiquetas para las resoluciones estandar
                if ($ancho == 1920) {
                    $item .= ',NAME="1080p"';
                } elsif ($ancho == 1280) {
                    $item .= ',NAME="720p"';
                } elsif ($ancho == 854) {
                    $item .= ',NAME="480p"';
                } elsif ($ancho == 640) {
                    $item .= ',NAME="360p"';
                } elsif ($ancho == 426) {
                    $item .= ',NAME="240p"';
                } else {
                    $item .= ",NAME=\"$alto"."p\"";
                }
                $item .= "\n$hlspath/playlist.m3u8\n";
                $ult_resolucion = "$ancho"."x$alto";
                $list_items{ $ancho * $alto } = $item;
            }
        }
    }
    my $list = "#EXTM3U\n#EXT-X-VERSION:3\n";
    foreach my $stream ( reverse sort keys %list_items) {
        $list .= $list_items{$stream};
    }

    my $filename = "$RUTA_PRONTUS$ARTIC_filename.mp4.m3u8";
    glib_fildir_03::write_file($filename, $list);
    if ($filename =~ /.*(\/.*?\/site\/\w+\/\d{8}\/mmedia\/multimedia_video\w+\S?.*)/i) {
        lib_prontus::purge_cache($1);
    }
}
# ---------------------------------------------------------------
# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}
# ---------------------------------------------------------------
# borra los archivos temporales asociados a la transcodificacion.
sub garbageTempFiles {
    #borramos los logs de multipasos y archivos temporales que pudieran quedar
    my @logs;
    if ($ARTIC_filename ne '')  {
        if ($lib_xcoding::RUTA_TEMPORAL ne '')  {
            @logs = glob "$lib_xcoding::RUTA_TEMPORAL$ARTIC_filename*";
        } elsif ($RUTA_PRONTUS ne '') {
            @logs = glob "$RUTA_PRONTUS$ARTIC_filename*";
        }
        # print STDERR Dumper(\@logs);
        foreach my $log (@logs) {
            # no se borran:
            # 1.- archivo mp4, es el resultado de la transcodificacion
            # 2.- directorio con el mismo nombre, es el directorio de HLS
            # 3.- si es igual al origen, indica problema de trancodificacion
            if ($log =~ /\.mp4$/ || $log =~ /\.m3u8$/ || $log =~ /\.json$/ || -d $log || $log eq $ORIGEN) {
                #~ print STDERR "no borrar $log\n";
                next;
            }

            print STDERR lib_lang::get_interpolated_message("videodoxcode_deleting_log", $log);
            unlink $log;
        }
    } else {
        print STDERR lib_lang::get_message("videodoxcode_empty_filename");
    }
}

# ---------------------------------------------------------------
# realiza tareas adicionales al terminar
sub tareas_salida {
    print STDERR "[".glib_hrfec_03::fecha_human()." ". glib_hrfec_03::hora_human()."] ". lib_lang::get_interpolated_message('videodoxcode_process_end', $ARTIC_filename);
    # si esta definido un post proceso lo ejecutamos en segundo plano
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} ne '') {
        if ($ARTIC_ts_articulo ne '' && $PRONTUS_ID ne '') {
            system("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} $PRONTUS_ID $ARTIC_ts_articulo >/dev/null 2>&1 &");
        }
    }
    if ($LOCK ne '') {
        my ($status, $result, $data) = lib_waitlock::unlock_file($LOCK, $LOCKFH);
        if (!$status) {
            print STDERR lib_lang::get_message($result). $data, "\n";
        }
    }
}

# ---------------------------------------------------------------
# actualiza el articulo
sub actualizar_articulo {
    my $marca = $_[0];
    my $generar_actualizar_dam = $_[1];

    my $artic_obj;
    unless ($artic_obj = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $ARTIC_ts_articulo)
        ) {
        return lib_lang::get_interpolated_message('global_error_artic_init', "$Artic::ERR TS[$ARTIC_ts_articulo]");
    }

    # debemos generar la metadata si no existe o si no existe el poster
    my $fullpath_video = "$artic_obj->{dst_multimedia}/$ARTIC_filename.mp4";
    my $path_poster = "$artic_obj->{dst_mm_thumbs}/$ARTIC_filename.jpg";
    if (! -f $path_poster) {
        print STDERR "Generar thumbs para [$fullpath_video]\n";
        my $pathnice = lib_utils::get_path_nice();
        system("$pathnice /usr/bin/perl $Bin/prontus_genera_thumbnailsvideo.pl $fullpath_video 1 &");
    } else {
        # actualizamos metadata de video
        lib_xcoding::genera_info_media($fullpath_video);
    }

    # se asegura el xml mientras se edita y guarda
    $LOCK = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/$ARTIC_ts_articulo.mutex";
    my ($status, $data);
    ($status, $LOCKFH, $data) = lib_waitlock::lock_file($LOCK);
    if (!$status) {
        return lib_lang::get_message($LOCKFH). $data;
    }

    $artic_obj->{xml_data} = glib_fildir_03::read_file($artic_obj->{fullpath_xml});
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, $marca, "$ARTIC_filename.mp4", 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.bytes", -s "$artic_obj->{dst_multimedia}/$ARTIC_filename.mp4" , 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.s3", '', 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, '_s3', 0, 0);
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "_modtime", time, 0);

    my $media_size = 0;
    # buscamos todos los archivos multimedia.
    # multimedia: datos del video y thumbs
    my @file_list = glob ("$artic_obj->{dst_mm_thumbs}/$marca$ARTIC_ts_articulo*");
    foreach my $file (@file_list) {
        $media_size += -s $file;
    }

    # multimedia: videos y calidades
    @file_list = glob ("$artic_obj->{dst_multimedia}/$marca$ARTIC_ts_articulo*");
    foreach my $file (@file_list) {
        if (-d $file) {
            my @subdir_file_list = glob ($file. '/*');
            foreach my $sub_file (@subdir_file_list) {
                $media_size += -s $sub_file;
            }
        } else {
            if ($file !~ /\.mp4$/ && $file !~ /\.m3u8$/ && $file !~ /\.json$/) {
                next;
            }
            $media_size += -s $file;
        }
    }
    $artic_obj->{xml_data} = lib_prontus::replace_in_xml($artic_obj->{xml_data}, "$marca.total", $media_size, 0);

    # actualizamos tamaño del artículo
    # este dato solo se puede obtener desde el xml
    my $s3_size = 0;
    if ($artic_obj->{xml_data} =~ /<_s3_size>(\d+)<\/_s3_size>/) {
        $s3_size = $1;
    }

    my ($disk_size, $imag_size, $asset_size);
    ($disk_size, $imag_size, $asset_size, $media_size) = $artic_obj->get_disk_size();
    my $total_size = $s3_size + $disk_size;
    $artic_obj->update_art_size($total_size, $disk_size, $imag_size, $asset_size, $media_size);

    $artic_obj->_flush_xml();
    my $result;
    ($status, $result, $data) = lib_waitlock::unlock_file($LOCK, $LOCKFH);
    if (!$status) {
        print STDERR lib_lang::get_message($result). $data, "\n";
    }

    # actualiza datos en bd
    my ($bd, $msg) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        return $msg;
    }
    $artic_obj->art_update_bd($bd);

    $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo} = $artic_obj;

    if ($generar_actualizar_dam) {
        # Generar vista (a partir del xml)
        $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art('') || return $Artic::ERR;

        my %datos = lib_prontus::getCamposXml($artic_obj->{xml_data}, '_fid,_plt');
        my $fid = $datos{'_fid'};
        my $plt = $datos{'_plt'};

        # Parsear plantillas paralelas.
        foreach my $plt_paralela (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$fid}{'plt_paralela'}}) {
            $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art('', $plt_paralela, 1) || return $Artic::ERR;
        }

        # Generar vistas secundarias
        foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
            $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$ARTIC_ts_articulo}->generar_vista_art($mv) || return $Artic::ERR;
        }

        # Actualizar el DAM
        my $ext = lib_utils::get_file_extension($plt);
        my $fullpath_artic = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/artic/$ARTIC_dirfecha/pags/$ARTIC_ts_articulo.$ext";
        my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";
        my $cmd = "$rutaScript/dam/prontus_dam_ppart_save.pl $fullpath_artic >/dev/null 2>&1 &";
        print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
        system $cmd;
    }

    # Dropbox.
    lib_prontus::call_dropbox_backup("art;$ARTIC_ts_articulo");

    return '';
}

sub die_stderr {
    my $msg = $_[0];
    my $detalle = $_[1];
    my $write = $_[2];
    &write_status($msg) if ($write);
    print STDERR "[ERROR] $msg - $detalle";
    exit 1;
}

sub write_status {
    if ($ARTIC_ts_articulo ne '' && $prontus_varglb::PRONTUS_ID ne 'prontus') {
        my $msg = $_[0];
        $msg =~ s/\n//sg;
        my $file = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/procs/xcoding_status_$ARTIC_ts_articulo.txt";

        glib_fildir_03::write_file($file, $msg);
    }
}

# ---------------------------------------------------------------
# Funciones para usar transcodificador externo
# ---------------------------------------------------------------

# descarga el archivo
sub getFile {
    my $url = $_[0];
    my $filename =  $_[1];
    my ($respuesta, $success) = ('', 0);
    print STDERR lib_lang::get_interpolated_message("videodoxcode_msg_download", $url, $filename);
    for (my $i = 0; $i < 4; $i++) {
        my $response = $CLIENT->get($url, ':content_file' => $filename);
        if ($response->is_success) {
            $respuesta = $response->status_line ."\n";
            return (1, $respuesta);
        } else {
            print STDERR " Error: ".$response->as_string."; $!\n";
            $respuesta = $response->as_string;
        }
    }
    return ($success, $respuesta);
}; # getFile.

# realiza una peticion a una url externa
sub postUrl {
    my ($url,$content) = @_;
    #~ print STDERR "llamando [$url]\n";
    my($respuesta,$request,$response,$ready);
    $ready = 0;
    while (!$ready) {
        $response = $CLIENT->post($url, [$content]);
        #~ print Dumper($response);
        #~ my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
        #~ $mon++;
        #~ $year += 1900;
        #~ $hour = $hour<10?'0'.$hour:$hour;
        #~ $min = $min<10?'0'.$min:$min;
        #~ $sec = $sec<10?'0'.$sec:$sec;
        #~ $mon = $mon<10?'0'.$mon:$mon;
        #~ $mday = $mday<10?'0'.$mday:$mday;
        #~ print STDERR "[$hour:$min:$sec $year/$mon/$mday] [" .$response->status_line ."]\n";
        if ($response->is_success) {
            return $response->content;
        } else {
            if ($response->status_line =~ /5\d{2}/) {
                print STDERR lib_lang::get_interpolated_message("videodoxcode_error_remote_server", $url, $response->as_string);
                return 'ERROR';
            }
            print STDERR lib_lang::get_interpolated_message("videodoxcode_execution_error", $url, $response->as_string, $!);
            return 'ERROR';
        }
    }
}

# Inicializa el user-agent.
sub initUA {
    # Inicializa el UA.
    $CLIENT = LWP::UserAgent->new();
    $CLIENT->agent('Prontus CMS');
    my $conn_cache = LWP::ConnCache->new();
    $CLIENT->conn_cache($conn_cache);
}

# purgea el cache de los archivos generados por el transcodificador externo
sub purgeExterno {
    my $path_files = '';
    my $filepath = '';

    # Deduce ubicacion de los archivos generados desde el archivo origen
    if ($ORIGEN =~ /(\/.*\/[^\/]+\/site\/\w+\/\d{8}\/mmedia\/multimedia_video\w+)\.\w+$/i) {
        $path_files = $1;
        my @entries = glob "$path_files*";
        foreach my $entry (@entries) {
            if (-d $entry) {
                my @archivos = glib_fildir_03::lee_dir($entry);
                # hacemos purge de los archivos generados
                foreach my $archivo (@archivos) {
                    next if($archivo eq '.' || $archivo eq '..');
                    # armamos la ruta completa
                    $filepath = "$entry/$archivo";
                    # obtenemos la ruta relativa
                    if($filepath =~ /.*(\/[^\/]+\/site\/\w+\/\d{8}\/mmedia\/multimedia_video\w+.*)/i) {
                        lib_prontus::purge_cache($1);
                    }
                }
            } elsif($entry =~ /.*(\/[^\/]+\/site\/\w+\/\d{8}\/mmedia\/multimedia_video\w+.*)/i) {
                lib_prontus::purge_cache($1);
            }
        }
    }
}
