// ---------------------------------------------------------------
// SCRIPT
// ---------------------------------------------------------------
// EditorImag.class.js
//
// ---------------------------------------------------------------
// PROPOSITO
// ---------------------------------------------------------------
// Manejo de eventos y funciones del editor de imagenes v2.0
// ---------------------------------------------------------------

(function (window) {
    var _self = null;
    var _EditorImag = {
        vars: {
            // Datos de la imagen que se está editando.
            img: {
                path: EditorImagConfig.foto,
                width: parseInt(EditorImagConfig.width),
                height: parseInt(EditorImagConfig.height),
                fotofija: '',
                fotofijaW: null,
                fotofijaH: null,
            },
            initFotofija: EditorImagConfig.initFotofija,
            prontus_id: EditorImagConfig.prontus_id,
            ts: EditorImagConfig.artic_ts,
            idImgElement: "#image",
            validEvents: ['click', 'hover'],
            hasChanges: false,
            resizeOrigWidth: null,
            resizeOrigHeight: null,
            resizeChain: true,
            resizeWidth: null,
            resizeHeight: null,
            zoomRatio: 1,
            currentTool: 'crop',
            galleryOpen: false,
            galleryScroll: true,
            cropFirstTime: false,
            cropper: null
        },
        init: function (force) {
            _self = this;

            if (typeof force != 'undefined') {
                _self.methods.bindEvents(force);
            } else {
                _self.methods.bindEvents();
            }

            _self.methods.getFotosFijas();
            _self.methods.adjustInitFotofija();


            $('.foto_width').text(_self.vars.img.width);
            $('.foto_height').text(_self.vars.img.height);

            _self.methods.initCrop();

            $('#resize_chain').off('click').on('click', function () {
                if ($(this).is(':checked')) {
                    _self.vars.resizeChain = true;
                    _self.methods.destroyResize();
                    _self.methods.initResize(true);
                } else {
                    _self.vars.resizeChain = false;
                    _self.methods.destroyResize();
                    _self.methods.initResize(false);
                }
            });

            _self.methods.initGallery();

            var lazyImages = [].slice.call(document.querySelectorAll("div.galeria_foto > img"));
            if ("IntersectionObserver" in window) {
                var lazyImageObserver = new IntersectionObserver(function(entries) {
                    entries.forEach(function(entry) {
                        if (entry.isIntersecting) {
                            var lazyImage = entry.target;
                            lazyImage.src = lazyImage.dataset.src;
                            lazyImageObserver.unobserve(lazyImage);
                        }
                    });
                });

                lazyImages.forEach(function(lazyImage) {
                    lazyImageObserver.observe(lazyImage);
                });
            }
        },
        methods: {
            // Bindea todos las acciones a sus respectivos metodos.
            bindEvents: function (force) {
                var selector;

                if (typeof force == 'undefined') force = 0;

                if (!force) {
                    selector = '[data-event]:not([data-bind])';
                } else {
                    selector = '[data-event]';
                }

                $(selector).each(function () {
                    var method = $(this).attr("data-method");
                    var event = $(this).attr("data-event");

                    // Validar si es un evento válido.
                    if ($.inArray(event, _self.vars.validEvents) >= 0) {
                        $(this).attr("data-bind", "true");
                        $(this).off(event).on(event, function (e) {
                            e.preventDefault();
                            _self.helper.execFunction(method, _EditorImag, {e: e, helper: $(this)});
                        });
                    }
                });
            },
            unBindEvents: function (methods) {
                for (var x = 0; x < methods.length; x++) {
                    $('[data-method="' + methods[x] + '"]').off(); // por ahora quita todos los eventos.
                    $('[data-method="' + methods[x] + '"]').removeAttr("data-bind");
                }
            },
            initGallery: function () {
                if (_self.vars.img.fotofija != '') {
                    $('.btn-galeria').trigger('click');
                }
            },
            adjustInitFotofija: function () {
                if (_self.vars.initFotofija) {
                    $('#_fotosfijas').val(_self.vars.initFotofija);

                    var indexOfPath = _self.vars.img.path.indexOf('/foto_') + 1;
                    _self.vars.img.fotofija = _self.vars.initFotofija;

                    if (indexOfPath) {
                        var fotoname = _self.vars.img.path.substring(indexOfPath, indexOfPath + 13);
                        var w = parent.$('input[name="_w' + fotoname + '"]').val();
                        var h = parent.$('input[name="_h' + fotoname + '"]').val();

                        _self.vars.img.width = w;
                        _self.vars.img.height = h;

                    } else {
                        console.log("ERROR: no se pudo determinar info. de la foto.");
                    }
                }
            },
            getFotosFijas: function () {
                parent.$('div[id^="recuadro_'+ Admin.const.fotoFijaTag +'"]').each(function() {
                    var id = $(this).attr("id").replace("recuadro_"+ Admin.const.fotoFijaTag , "");
                    var maxW = parent.$('input[name="' + Admin.const.fotoFijaMaxWTag + id + '"]').val();
                    var maxH = parent.$('input[name="' + Admin.const.fotoFijaMaxHTag + id + '"]').val();

                    if (id.indexOf('temporal') == -1) { // Se usan en la galeria. Se ignoran.
                        var aspectRatio = parseFloat(maxW / maxH).toFixed(2);
                        var name = parent.$('#fotofija-label-' + id).text();
                        var selected = '';
                        $('#_fotosfijas').append('<option value="{0}" data-aspectratio="{2}" data-width="{3}" data-height="{4}" {5}>{1}</option>'.format(id, name, aspectRatio, maxW, maxH, selected));
                    }

                });
            },
            initCrop: function (now) {
                _self.vars.currentTool = 'crop';

                if (typeof now == 'undefined') {
                    setTimeout(function () {
                        if (_self.vars.initFotofija) {
                            var aspectRatio = $('#_fotosfijas option:selected').attr("data-aspectratio");
                            var w = $('#_fotosfijas option:selected').attr("data-width");
                            var h = $('#_fotosfijas option:selected').attr("data-height");
                            var options = {
                                cropBoxResizable: false,
                                aspectRatio: aspectRatio,
                                notUpdateCrop: true
                            };

                            _self.vars.img.fotofija = _self.vars.initFotofija;
                            _self.vars.img.fotofijaW = parseInt(w);
                            _self.vars.img.fotofijaH = parseInt(h);

                            $('#crop_width').text(w);
                            $('#crop_height').text(h);

                            _self.plugins.cropper.init(options);
                        } else {
                            _self.plugins.cropper.defaultInit();
                        }
                    }, 500);
                } else {
                    _self.plugins.cropper.defaultInit();
                }

                $('#_fotosfijas').off('change').on('change', function () {
                    var name = $(this).val();
                    var aspectRatio = $('#_fotosfijas option:selected').attr("data-aspectratio");
                    var w = $('#_fotosfijas option:selected').attr("data-width");
                    var h = $('#_fotosfijas option:selected').attr("data-height");
                    var options = {
                        cropBoxResizable: false,
                        aspectRatio: aspectRatio,
                        notUpdateCrop: true
                    };

                    _self.vars.img.fotofija = name;
                    _self.vars.img.fotofijaW = parseInt(w);
                    _self.vars.img.fotofijaH = parseInt(h);

                    if (!w && !h) {
                        _self.vars.img.fotofijaW = null;
                        _self.vars.img.fotofijaH = null;
                    }

                    $('.zoom_warning').hide();

                    if (name != '') {
                        $('#crop_width').text(w);
                        $('#crop_height').text(h);

                        _self.plugins.cropper.init(options);
                    } else {
                        _self.plugins.cropper.defaultInit();
                    }
                });
            },
            destroyCrop: function () {
                _self.vars.cropper.destroy();

                _self.methods.hideSaveOptions();
            },
            initResize: function (aspectRatio) {
                _self.vars.currentTool = 'resize';

                $('#resize_width').val(_self.vars.img.width);
                $('#resize_height').val(_self.vars.img.height);
                $(_self.vars.idImgElement).addClass("resizing");

                var maxWidth = _self.vars.resizeOrigWidth = $(_self.vars.idImgElement).width();
                var maxHeight = _self.vars.resizeOrigHeight = $(_self.vars.idImgElement).height();

                $(_self.vars.idImgElement).resizable({
                    aspectRatio: aspectRatio,
                    minHeight: 80,
                    minWidth: 80,
                    maxWidth: maxWidth,
                    maxHeight: maxHeight,
                    handles: "all",
                    resize: function(event, ui) {
                        $(this).css({
                            "top": 0,
                            "left": 0,
                            "right": 0,
                            "bottom": 0,
                            "margin": "auto"
                        });

                        var p1 = (ui.size.width * 100) / maxWidth;
                        var p2 = (ui.size.height * 100) / maxHeight;
                        var w = Math.round($(_self.vars.idImgElement)[0].naturalWidth * p1/100);
                        var h = Math.round($(_self.vars.idImgElement)[0].naturalHeight * p2/100);

                        $('#resize_width').val(w);
                        $('#resize_height').val(h);

                        _self.vars.resizeWidth = w;
                        _self.vars.resizeHeight = h;
                        _self.vars.hasChanges = true;
                        _self.methods.showSaveOptions();
                    },
                    create: function () {
                        $(this).css({
                            top: 0,
                            left: 0,
                            right: 0,
                            bottom: 0,
                            margin: "auto"
                        });

                        $('#resize_width').val($(_self.vars.idImgElement)[0].naturalWidth);
                        $('#resize_height').val($(_self.vars.idImgElement)[0].naturalHeight);

                        $("#resize_width").off('keyup').on("keyup", function (e) {
                            if (e.keyCode == 13) { // enter
                                _self.methods.updateResizeWidth(e);
                                _self.vars.hasChanges = true;
                                _self.methods.showSaveOptions();
                            }
                        });

                        $("#resize_height").off('keyup').on("keyup", function (e) {
                            if (e.keyCode == 13) { // enter
                                _self.methods.updateResizeHeight(e);
                                _self.vars.hasChanges = true;
                                _self.methods.showSaveOptions();
                            }
                        });

                        $("#resize_height").off('blur').on('blur', function(e){
                            _self.methods.updateResizeHeight(e);
                            _self.vars.hasChanges = true;
                            _self.methods.showSaveOptions();
                        });

                        $("#resize_width").off('blur').on('blur', function(e){
                            _self.methods.updateResizeWidth(e);
                            _self.vars.hasChanges = true;
                            _self.methods.showSaveOptions();
                        });

                        _self.methods.hideLoading();
                    }
                });
            },
            getCropperInstance: function () {
                return _self.vars.cropper;
            },
            destroyResize: function () {
                if (_self.methods.getResizeInstance()) {
                    $(_self.vars.idImgElement).resizable("destroy");
                }

                $(_self.vars.idImgElement).removeClass("resizing");
                $(_self.vars.idImgElement).attr("style", "");

                _self.methods.hideSaveOptions();
            },
            getResizeInstance: function () {
                var resizeInstance = typeof $(_self.vars.idImgElement).resizable('instance');
                return (resizeInstance == 'object');
            },
            updateResizeWidth: function () {
                var val = $("#resize_width").val();

                if ($.isNumeric(val) === false) {
                    showModalAlert({
                        title: ProntusL10n.get('global_server_msg'),
                        msg: ProntusL10n.get('img_edit_numeric_width'),
                        type: "orange",
                        icon: "glyphicon glyphicon-warning-sign",
                        cbBtnOK: function () {}
                    });

                    return false;
                }

                var w = Math.round((val * _self.vars.resizeOrigWidth) / $(_self.vars.idImgElement)[0].naturalWidth);

                if (val > $(_self.vars.idImgElement)[0].naturalWidth) {
                    showModalAlert({
                        title: ProntusL10n.get('global_server_msg'),
                        msg: ProntusL10n.get('img_edit_resize_minimun_width', $(_self.vars.idImgElement)[0].naturalWidth),
                        type: "orange",
                        icon: "glyphicon glyphicon-warning-sign",
                        cbBtnOK: function () {}
                    });

                    return false;
                }

                if (_self.vars.resizeChain) {
                    var newSize = _self.helper.calculateSize($(_self.vars.idImgElement)[0].naturalWidth, $(_self.vars.idImgElement)[0].naturalHeight, $("#resize_width").val(), false);
                    w = (newSize[0] * _self.vars.resizeOrigWidth) / $(_self.vars.idImgElement)[0].naturalWidth;
                    var h = (newSize[1] * _self.vars.resizeOrigHeight) / $(_self.vars.idImgElement)[0].naturalHeight;

                    _self.vars.resizeWidth = newSize[0];
                    _self.vars.resizeHeight = newSize[1];

                    $('#resize_height').val(_self.vars.resizeHeight);
                    $(".editor_imag_workspace .ui-wrapper").width(w);
                    $(".editor_imag_workspace .ui-wrapper img").width(w);
                    $(".editor_imag_workspace .ui-wrapper").height(h);
                    $(".editor_imag_workspace .ui-wrapper img").height(h);

                } else {
                    $(".editor_imag_workspace .ui-wrapper").width(w);
                    $(".editor_imag_workspace .ui-wrapper img").width(w);

                    _self.vars.resizeWidth = parseInt($('#resize_width').val());
                    _self.vars.resizeHeight = parseInt($('#resize_height').val());
                }
            },
            updateResizeHeight: function () {
                var val = $('#resize_height').val();

                if ($.isNumeric(val) === false) {
                    showModalAlert({
                        title: ProntusL10n.get('global_server_msg'),
                        msg: ProntusL10n.get('img_edit_numeric_height'),
                        type: "orange",
                        icon: "glyphicon glyphicon-warning-signf",
                        cbBtnOK: function () {}
                    });

                    return false;
                }

                var h = Math.round((val * _self.vars.resizeOrigHeight) / $(_self.vars.idImgElement)[0].naturalHeight);

                if (val > $(_self.vars.idImgElement)[0].naturalHeight) {
                    showModalAlert({
                        title: ProntusL10n.get('global_server_msg'),
                        msg: ProntusL10n.get('img_edit_resize_minimun_height', $(_self.vars.idImgElement)[0].naturalHeight),
                        type: "orange",
                        icon: "glyphicon glyphicon-warning-sign",
                        cbBtnOK: function () {
                        }
                    });

                    return false;
                }

                if (_self.vars.resizeChain) {
                    var newSize = _self.helper.calculateSize($(_self.vars.idImgElement)[0].naturalWidth, $(_self.vars.idImgElement)[0].naturalHeight, false, $("#resize_height").val());
                    var w = (newSize[0] * _self.vars.resizeOrigWidth) / $(_self.vars.idImgElement)[0].naturalWidth;
                    h = (newSize[1] * _self.vars.resizeOrigHeight) / $(_self.vars.idImgElement)[0].naturalHeight;

                    _self.vars.resizeWidth = newSize[0];
                    _self.vars.resizeHeight = newSize[1];

                    $('#resize_width').val(_self.vars.resizeWidth);
                    $(".editor_imag_workspace .ui-wrapper").width(w);
                    $(".editor_imag_workspace .ui-wrapper img").width(w);
                    $(".editor_imag_workspace .ui-wrapper").height(h);
                    $(".editor_imag_workspace .ui-wrapper img").height(h);

                } else {
                    $(".editor_imag_workspace .ui-wrapper").height(h);
                    $(".editor_imag_workspace .ui-wrapper img").height(h);

                    _self.vars.resizeWidth = parseInt($('#resize_width').val());
                    _self.vars.resizeHeight = parseInt($('#resize_height').val());
                }
            },
            showSaveOptions: function () {
                $('.options_save').show();
            },
            hideSaveOptions: function () {
                $('.options_save').hide();
            },
            checkHasChanges: function (callback, customMsg) {
                if (typeof customMsg == 'undefined') {
                    customMsg = ProntusL10n.get('img_edit_tool_change_warning');
                }

                if (_self.vars.hasChanges) {
                    showModalConfirm({
                        msg: customMsg,
                        type: "blue",
                        icon: "glyphicon glyphicon-question-sign",
                        okTitle: ProntusL10n.get('img_edit_discard_changes'),
                        cancelTitle: ProntusL10n.get('img_edit_continue_edit'),
                        btnClassOk: 'hovwhite radius15 noborder txtnextwhite pad-top-bot7 pad-lef-rig20 int-nextred nextred',
                        cbBtnOK: function () {
                            _self.vars.hasChanges = false;
                            callback();
                        },
                        cbBtnCancel: function () {
                            _self.methods.hideLoading();
                        }
                    });
                } else {
                    callback();
                }
            },
            showLoading: function () {
                $('.editor_imag_workspace .loading').show();
            },
            hideLoading: function () {
                $('.editor_imag_workspace .loading').hide();
            }
        },
        actions: {
            rotateRight: function (ui) {
                ui.e.preventDefault();
                _self.vars.cropper.rotate(90);
                _self.vars.hasChanges = true;
                _self.methods.showSaveOptions();
            },
            rotateLeft: function (ui) {
                ui.e.preventDefault();
                _self.vars.cropper.rotate(-90);
                _self.vars.hasChanges = true;
                _self.methods.showSaveOptions();
            },
            toggleCrop: function (ui) {
                ui.e.preventDefault();

                _self.methods.showLoading();

                _self.methods.checkHasChanges(function () {
                    $('button[data-method="actions.toggleResize"]').removeClass('int-nextred nextred active txtnextwhite').addClass('int-nextgray4 nextgray4 txtnextgray0');
                    $('button[data-method="actions.toggleCrop"]').addClass('int-nextred nextred active txtnextwhite').removeClass('int-nextgray4 nextgray4 txtnextgray0');
                    $('button[data-method="actions.rotateLeft"]').removeAttr('disabled');
                    $('button[data-method="actions.rotateRight"]').removeAttr('disabled');

                    $('.options_resize').hide();
                    $('.options_zoom').show();
                    $('.options_rotate').show();
                    $('.options_crop').show();

                    _self.methods.destroyResize();
                    _self.methods.destroyCrop();
                    _self.methods.initCrop(true);
                });
            },
            resetResize: function () {
                _self.methods.destroyResize();
                _self.methods.initResize(_self.vars.resizeChain);
            },
            toggleResize: function (ui) {
                ui.e.preventDefault();

                _self.methods.showLoading();

                _self.methods.checkHasChanges(function () {
                    $('button[data-method="actions.toggleCrop"]').removeClass('int-nextred nextred active txtnextwhite').addClass('int-nextgray4 nextgray4 txtnextgray0');
                    $('button[data-method="actions.toggleResize"]').addClass('int-nextred nextred active txtnextwhite').removeClass('int-nextgray4 nextgray4 txtnextgray0');

                    $('button[data-method="actions.rotateLeft"]').attr('disabled', 'disabled');
                    $('button[data-method="actions.rotateRight"]').attr('disabled', 'disabled');

                    _self.vars.img.fotofija = '';
                    _self.vars.img.fotofijaW = null;
                    _self.vars.img.fotofijaH = null;

                    $('.options_crop').hide();
                    $('.options_zoom').hide();
                    $('.options_rotate').hide();
                    $('.options_resize').show();

                    _self.methods.destroyCrop();
                    _self.methods.destroyResize();
                    _self.methods.initResize(true);
                });
            },
            zoomIn: function (ui) {
                ui.e.preventDefault();
                _self.vars.cropper.zoom(0.1);
                _self.vars.hasChanges = true;
            },
            zoomOut: function (ui) {
                ui.e.preventDefault();
                _self.vars.cropper.zoom(-0.1);
                _self.vars.hasChanges = true;
            },
            resetCrop: function (ui) {
                ui.e.preventDefault();

                _self.methods.showLoading();
                $('.zoom_warning').hide();

                _self.methods.destroyCrop();
                _self.methods.initCrop(true);
                $('#_fotosfijas').change();
            },
            cancel: function (ui) {
                ui.e.preventDefault();

                if (!_self.vars.hasChanges) {
                    parent.MediaManager.bancoimg.methods.closeModalEditor();
                } else {
                    showModalConfirm({
                        msg: ProntusL10n.get('img_edit_discard_changes_confirm'),
                        type: "blue",
                        icon: "glyphicon glyphicon-question-sign",
                        okTitle: ProntusL10n.get('img_edit_discard_changes'),
                        cancelTitle: ProntusL10n.get('img_edit_continue_edit'),
                        btnClassOk: 'hovwhite radius15 noborder txtnextwhite pad-top-bot7 pad-lef-rig20 int-nextred nextred',
                        cbBtnOK: function () {
                            parent.MediaManager.bancoimg.methods.closeModalEditor();
                        },
                        cbBtnCancel: function () {}
                    });
                }
            },
            apply: function (ui) {
                _self.methods.showLoading();

                if (_self.vars.currentTool == 'crop') {
                    _self.actions.applyCrop(ui);
                } else {
                    _self.actions.applyResize(ui);
                }
            },
            applyCrop: function (ui) {
                ui.e.preventDefault();

                if (!_self.vars.hasChanges) {
                    showModalConfirm({
                        msg: ProntusL10n.get('img_edit_no_changes_continue'),
                        type: "blue",
                        icon: "glyphicon glyphicon-question-sign",
                        okTitle: ProntusL10n.get('img_edit_continue_confirm'),
                        cancelTitle: ProntusL10n.get('img_edit_close_editor'),
                        cbBtnOK: function () {
                            _self.methods.hideLoading();
                        },
                        cbBtnCancel: function () {
                            parent.MediaManager.bancoimg.methods.closeModalEditor();
                        }
                    });

                    return false;
                }

                showModalConfirm({
                    msg: ProntusL10n.get('img_edit_confirm_changes'),
                    type: "blue",
                    icon: "glyphicon glyphicon-question-sign",
                    okTitle: ProntusL10n.get('img_edit_confirm_changes_ok'),
                    cancelTitle: ProntusL10n.get('img_edit_confirm_changes_cancel'),
                    btnClassOk: 'hovwhite radius15 noborder txtnextwhite pad-top-bot7 pad-lef-rig20 int-nextgreen nextgreen',
                    cbBtnOK: function () {
                        var cropData = _self.vars.cropper.getData(true);
                        var imageData = _self.vars.cropper.getImageData();
                        var data = {
                            srcX: cropData.x,
                            srcY: cropData.y,
                            width: cropData.width,
                            height: cropData.height,
                            aspectRatio: imageData.aspectRatio,
                            rotate: cropData.rotate,
                            foto: _self.vars.img.path,
                            zoomRatio: _self.vars.zoomRatio,
                            _prontus_id: _self.vars.prontus_id,
                            ts: _self.vars.ts,
                            fotow: _self.vars.img.fotofijaW,
                            fotoh: _self.vars.img.fotofijaH
                        };

                        _self.vars.cropper.disable();

                        $.ajax({
                            url: "/" + Admin.dir_cgi_cpn + "/prontus_editor_imag_guardar.cgi",
                            type: "POST",
                            dataType: "json",
                            data: data,
                            success: function(response) {
                                if (response.status === 0) {
                                    showModalAlert({
                                        title: ProntusL10n.get('global_server_msg'),
                                        msg: response.msg,
                                        type: "red",
                                        icon: "glyphicon glyphicon-exclamation-sign",
                                        cbBtnOK: function () {
                                            _self.vars.cropper.enable();
                                        }
                                    });

                                } else {
                                    _self.actions.imagGuardarCallback(response, false);
                                }
                            },
                            error: function() {
                                showModalAlert({
                                    title: ProntusL10n.get('global_server_msg'),
                                    msg: ProntusL10n.get('img_edit_proccess_error'),
                                    type: "red",
                                    icon: "glyphicon glyphicon-exclamation-sign",
                                    cbBtnOK: function () {
                                        _self.vars.cropper.enable();
                                    }
                                });
                            }
                        });
                    },
                    cbBtnCancel: function () {
                        _self.methods.hideLoading();
                    }
                });
            },
            applyResize: function (ui) {
                ui.e.preventDefault();

                if (!_self.vars.hasChanges) {
                    showModalConfirm({
                        msg: ProntusL10n.get('img_edit_no_changes_continue'),
                        type: "blue",
                        icon: "glyphicon glyphicon-question-sign",
                        okTitle: ProntusL10n.get('img_edit_continue_confirm'),
                        cancelTitle: ProntusL10n.get('img_edit_close_editor'),
                        cbBtnOK: function () {
                            _self.methods.hideLoading();
                        },
                        cbBtnCancel: function () {
                            parent.MediaManager.bancoimg.methods.closeModalEditor();
                        }
                    });

                    return false;
                }

                var checkbox_continue = '<br><br><label class="checkbox-inline label-padding-confirm"><input type="checkbox" value="1" id="continue_edit">'+ ProntusL10n.get('img_edit_continue_edit') +'</label>';

                showModalConfirm({
                    msg: ProntusL10n.get('img_edit_confirm_changes') + " " + checkbox_continue,
                    type: "blue",
                    icon: "glyphicon glyphicon-question-sign",
                    okTitle: ProntusL10n.get('img_edit_confirm_changes_ok'),
                    cancelTitle: ProntusL10n.get('img_edit_confirm_changes_cancel'),
                    cbBtnOK: function () {
                        $(_self.vars.imgElementId).resizable("destroy");

                        var continue_edit = $('#continue_edit').is(':checked');
                        var data = {
                            foto: _self.vars.img.path,
                            _prontus_id: _self.vars.prontus_id,
                            ts: _self.vars.ts,
                            fotow: _self.vars.resizeWidth,
                            fotoh: _self.vars.resizeHeight,
                            only_resize: 1
                        };

                        $.ajax({
                            url: "/" + Admin.dir_cgi_cpn + "/prontus_editor_imag_guardar.cgi",
                            type: "POST",
                            dataType: "json",
                            data: data,
                            success: function (response) {
                                if (response.status == 1) {
                                    _self.actions.imagGuardarCallback(response, continue_edit);
                                } else {
                                    showModalAlert({
                                        title: ProntusL10n.get('global_server_msg'),
                                        msg: response.msg,
                                        type: "red",
                                        icon: "glyphicon glyphicon-exclamation-sign-sign",
                                        cbBtnOK: function () {
                                            $(_self.vars.imgElementId).css({
                                                width: "auto",
                                                height: "auto",
                                                margin: "auto"
                                            });
                                        }
                                    });
                                }
                            },
                            error: function () {
                                showModalAlert({
                                    title: ProntusL10n.get('global_server_msg'),
                                    msg: ProntusL10n.get('img_edit_proccess_error'),
                                    type: "red",
                                    icon: "glyphicon glyphicon-exclamation-sign-sign",
                                    cbBtnOK: function () {
                                    }
                                });
                            }
                        });
                    },
                    cbBtnCancel: function () {
                        _self.methods.hideLoading();
                    }
                });
            },
            imagGuardarCallback: function (response, continue_edit) {
                var arr = response.msg.split(';');
                var url = arr[0];
                var numfoto = arr[1];
                var wfoto = arr[2];
                var hfoto = arr[3];

                // Agregar wfoto y hfoto.  obtener esos datos del banco de imagenes.
                var html = [
                    '<input type="hidden" name="_w%%numfoto%%" value="%%wfoto%%">',
                    '<input type="hidden" name="_h%%numfoto%%" value="%%hfoto%%">'
                ].join("").replace(/%%numfoto%%/g, numfoto).replace(/%%wfoto%%/g, wfoto).replace(/%%hfoto%%/g, hfoto);

                parent.$("#_mainFidForm").prepend(html);

                // Asigna al campo de fotofija que fue seleccionado.
                if (_self.vars.img.fotofija) {
                    parent.MediaManager.fotofija.foto.set(_self.vars.img.fotofija, url, wfoto, hfoto);
                }

                // Se actualiza el banco de imagenes.
                parent.MediaManager.bancoimg.methods.reloadBancoImagenes(); // Guardar despues de recargar el banco de imagenes.

                if (continue_edit) {
                    _self.methods.showLoading();
                    _self.vars.img.path = url;
                    _self.vars.img.width = wfoto;
                    _self.vars.img.height = hfoto;

                    $('button[data-method="actions.toggleResize"]').removeClass('active');
                    $('button[data-method="actions.toggleCrop"]').addClass('active');
                    $('button[data-method="actions.rotateLeft"]').removeAttr('disabled');
                    $('button[data-method="actions.rotateRight"]').removeAttr('disabled');

                    $('.options_resize').hide();
                    $('.options_zoom').show();
                    $('.options_rotate').show();
                    $('.options_crop').show();

                    $('.foto_width').text(_self.vars.img.width);
                    $('.foto_height').text(_self.vars.img.height);

                    $('#image').attr("src", url);

                    _self.methods.destroyResize();
                    _self.methods.destroyCrop();
                    _self.methods.initCrop(true);
                } else {
                    // Cerrar modal.
                    parent.MediaManager.bancoimg.methods.closeModalEditor();
                }
            },
            toggleGallery: function (ui) {
                ui.e.preventDefault();

                if (ui.helper.hasClass('active')) {
                    $('.editor_imag_galeria').hide();
                    _self.vars.galleryOpen = false;
                    ui.helper.removeClass('active');

                    $('.editor_imag_galeria .galeria_layer').off('click');
                } else {
                    $('.editor_imag_galeria').show();
                    ui.helper.addClass('active');
                    _self.vars.galleryOpen = true;

                    if (_self.vars.galleryScroll) {
                        var foto = document.querySelector('.galeria_foto.active');
                        foto.parentNode.insertBefore(foto, foto.parentNode.firstElementChild );
                        _self.vars.galleryScroll = false;
                    }

                    $('.editor_imag_galeria .galeria_layer').on('click', function () {
                        $('button[data-method="actions.toggleGallery"]').trigger('click');
                    });
                }
            },
            useGalleryImage: function (ui) {
                _self.methods.checkHasChanges(function () {
                    $('.galeria_foto').removeClass('active');
                    ui.e.preventDefault();

                    ui.helper.addClass('active');

                    $('.btn-galeria').trigger('click');

                    _self.methods.showLoading();
                    _self.vars.img.path = ui.helper.attr("data-foto");
                    _self.vars.img.width = ui.helper.attr("data-fotow");
                    _self.vars.img.height = ui.helper.attr("data-fotoh");

                    $('.foto_width').text(_self.vars.img.width);
                    $('.foto_height').text(_self.vars.img.height);

                    $('#image').attr("src", _self.vars.img.path);

                    if (_self.vars.currentTool == 'crop') {
                        setTimeout(function () {
                            _self.actions.resetCrop(ui);
                        }, 500);
                    } else {
                        setTimeout(function () {
                            _self.actions.resetResize();
                        }, 500);
                    }
                }, ProntusL10n.get('img_edit_change_img_confirm'));
            }
        },
        helper: {
            // Ejecuta un metodo de manera dinamica.
            // https://stackoverflow.com/questions/359788/how-to-execute-a-javascript-function-when-i-have-its-name-as-a-string
            execFunction: function (functionName, context) {
                var args = Array.prototype.slice.call(arguments, 2);
                var namespaces = functionName.split(".");
                var func = namespaces.pop();

                for(var i = 0; i < namespaces.length; i++) {
                    context = context[namespaces[i]];
                }

                return context[func].apply(context, args);
            },
            calculateSize: function (origwidth, origheight, width, height) {
                var ratio = origwidth / origheight;

                if (width !== false) {
                    width = parseFloat(width);
                    var newheight = width / ratio;
                    newheight = Math.round(newheight);

                    return [width, newheight];
                } else if (height !== false) {
                    height = parseFloat(height);
                    var newwidth = ratio * height;
                    newwidth = Math.round(newwidth);

                    return [newwidth, height];
                }
            },
        },
        plugins: {
            cropper: {
                defaultInit: function () {
                    _self.plugins.cropper.init({});
                },
                init: function (extraOptions) {
                    var options = {
                        viewMode: 1,
                        dragMode: 'move',
                        autoCropArea: 0.65,
                        restore: false,
                        guides: false,
                        highlight: false,
                        rotable: false,
                        cropBoxMovable: false,
                        cropBoxResizable: true,
                        minCropBoxWidth: 80,
                        minCropBoxHeight: 80,
                        zoom: function (e) {
                            _self.vars.zoomRatio = e.detail.ratio;

                            var data = _self.vars.cropper.getData(true);
                            if (_self.vars.img.fotofijaW && _self.vars.img.fotofijaH) {
                                if (data.width+5 < _self.vars.img.fotofijaW || data.height+5 < _self.vars.img.fotofijaH) {
                                    $('.zoom_warning .msg').text("Con este zoom el recorte puede verse pixelado.");
                                    $('.zoom_warning').show();
                                } else {
                                    $('.zoom_warning').hide();
                                }
                            }

                            _self.vars.hasChanges = true;
                            _self.methods.showSaveOptions();
                        },
                        crop: function () {
                            var data = _self.vars.cropper.getData(true);

                            if (_self.vars.img.fotofijaW && _self.vars.img.fotofijaH) {
                                if (data.width+5 < _self.vars.img.fotofijaW || data.height+5 < _self.vars.img.fotofijaH) {
                                    $('.zoom_warning .msg').text("Con este zoom el recorte puede verse pixelado.");
                                    $('.zoom_warning').show();
                                } else {
                                    $('.zoom_warning').hide();
                                }
                            }

                            if (typeof extraOptions.notUpdateCrop == 'undefined') {
                                var cropData = _self.vars.cropper.getData(true);
                                $('#crop_width').text(cropData.width);
                                $('#crop_height').text(cropData.height);
                            }

                            _self.methods.hideLoading();
                        },
                        cropstart: function () {
                            _self.vars.hasChanges = true;
                            _self.methods.showSaveOptions();
                        }
                    };

                    if (_self.vars.cropper !== null) {
                        _self.vars.cropper.destroy();
                    }

                    $.extend(options, extraOptions);

                    _self.vars.cropper = new Cropper(document.querySelector(_self.vars.idImgElement), options);

                    $(_self.vars.idImgElement).off('zoom.cropper').on('zoom.cropper', function (e) {
                        _self.vars.zooming = e.timeStamp;
                    });
                },
                methods: {
                    checkCropperInstance: function () {
                        return $('.cropper-container').length;
                    }
                }
            },
        }
    };
    window.EditorImag = {
        hasChanges: _EditorImag.vars.hasChanges,
        init: function () {
            _EditorImag.init();
        }
    };
})(this);

$(document).ready(function () {
    EditorImag.init();
});
