/**
 * Objeto para localizar mensajes javascript.
 * Requiere la carga de un archivo de lenguaje desde prontus/js-local/lang/[lang].js.
 */
var ProntusL10n = {
    locale: 'es',
    /**
     * Intenta encontrar la string correspondiente a una key de traducción en el objeto global lang
     * @param {string} key la key para buscar el mensaje a traducir en el objeto lang.
     * @param {array} args eventuales parámetros a interpolar en la string traducida.
     * @returns {string} la traducción con variables interpoladas, o la key si no encuentra traducción.
     */
    get: function() {
        let key = arguments[0];
        let args;
        if (arguments.length > 1) {
            args = Array.from(arguments);
            args.shift();
        }

        if (typeof l10nProntus === 'undefined' || typeof l10nProntus[key] === 'undefined') {
            return key;
        }

        if (typeof args === 'undefined') {
            return l10nProntus[key];
        } else if (Array.isArray(args)) {
            return l10nProntus[key].replace(/{(\d+)}/g, function(match, number) {
                return typeof args[number] != 'undefined'
                ? args[number]
                : match
                ;
            });
        } else {
            return key;
        }
    },
    getLocale: function() {
        return l10nProntus.l10n_locale;
    },
    getLocaleDate: function() {
        return l10nProntus.date_locale;
    },
    getMonths: function() {
        return l10nProntus.months;
    },
    getMonth: function(month) {
        return l10nProntus.months[month];
    },
    getDays: function() {
        return l10nProntus.days;
    },
    getDay: function(day) {
        return l10nProntus.days[day];
    }
};
