<?php
defined('PRONTUS') OR exit('');
/**
 * PostingYoutube
 *
 * Especie de API para manejar las peaticiones a Youtube.
 * Tambien posee algunas funciones utilitarias
 *
 * @package     youtube_post
 * @author      SCT <desarrollo@altavoz.net>
 * @version
 * 1.0.0 - 30/05/2016 - SCT - Primera version.
 *
 */
require_once('Utiles.class.php');
class PostingYoutube {

    public $CONFIG_YOUTUBE;
    public $utiles;
    /**
     * La config espera todos los datos necesarios para conectar a youtube, como aparecen en config.php.
     */
    public function __construct($config = false) {
        $this->CONFIG_YOUTUBE = $config;
        $this->utiles = new Utiles($config);
    }

    /**
     * Funcion para publicar un video a Youtube
     * @param string $file ruta al archivo para subir
     * @param array $params Arreglo de parámetros de identificacion
     * @return string Error, or NULL if no error
     */
    public function publish($input) {
        $ts         = $input['ts'];
        $ruta       = $input['ruta_youtube'];
        $xml        = $input['ruta_xml'];
        $file       = $input['ruta_video'];
        $prontus_id = $input['prontus_id'];
        $video_final = "";

        //$ruta, $xml, $mmedia, $ts
        $buffer = $this->utiles->getXmlContent($xml);
        $check = $this->utiles->getElementFromXML($buffer, 'updatetoyoutube');

        if ($check != 'si') {
            // escribimos json de estado
            $this->writeJSONResp($ruta, $ts, 'null', 'No se efectúa la carga a YouTube.');
            $this->utiles->printErrorLog("No se publica en Youtube", 'debug');
            return '';
        }

        if ($file === '' || !is_file($file)) {
            $this->utiles->printErrorLog('No venía video para enviar a Youtube', 'debug');
            $this->writeJSONResp($ruta, $ts, 'null', 'Ha ocurrido un error en el proceso de upload.');
            return '';
        }

        // escribimos json de estado
        $this->writeJSONResp($ruta, $ts, 'null', 'Comienza la carga.');

        $application_name = $this->CONFIG_YOUTUBE['YOUTUBE_APPNAME'];
        $client_secret = $this->CONFIG_YOUTUBE['YOUTUBE_CLIENT_SECRET'];
        $client_id = $this->CONFIG_YOUTUBE['YOUTUBE_CLIENT_ID'];
        $filepath_access_token =  $this->CONFIG_YOUTUBE['DOCUMENT_ROOT'] . "/$prontus_id/cpan/data/youtube_post/_token/access_token.json";
        $app_access_token = $this->utiles->readSingleFile($filepath_access_token);
        $scope = array('https://www.googleapis.com/auth/youtube.upload', 'https://www.googleapis.com/auth/youtube', 'https://www.googleapis.com/auth/youtubepartner');

        $videoPath = $file;
        $videoTitle =$this->utiles->getElementFromXML($buffer, 'titular_youtube');
        $videoDescription =$this->utiles->getElementFromXML($buffer, 'youtube_bajada');
        $videoCategory =$this->utiles->getElementFromXML($buffer, 'cmb_youtube_categoria');
        $videoTags =$this->utiles->getElementFromXML($buffer, 'youtube_tags');
        $tags_default =$this->utiles->getElementFromXML($buffer, 'youtube_tags_default');
        if ($tags_default) {
            $videoTags = $tags_default . ',' . $videoTags;
        }

        try{
            // Client init
            $client = new Google_Client();
            $client->setApplicationName($application_name);
            $client->setClientId($client_id);
            $client->setAccessType('offline');
            $client->setAccessToken($app_access_token);
            $client->setScopes($scope);
            $client->setClientSecret($client_secret);

            if ($client->getAccessToken()) {

                /**
                 * Check to see if our access token has expired.
                 * If so, get a new one and save it to file for future use.
                 */
                if ($client->isAccessTokenExpired()) {
                    $newToken = $client->getAccessToken();
                    $client->refreshToken(json_encode($newToken['refresh_token']));
                    file_put_contents($filepath_access_token, json_encode($client->getAccessToken()));
                }

                $this->utiles->printErrorLog('Identificado correctamente');

                $youtube = new Google_Service_YouTube($client);

                // escribimos json de estado
                $this->writeJSONResp($ruta, $ts, '0% completado', 'uploading');

                // Create a snipet with title, description, tags and category id
                $snippet = new Google_Service_YouTube_VideoSnippet();
                $snippet->setTitle($videoTitle);
                $snippet->setDescription($videoDescription);
                if (!empty($videoCategory)) {
                    $snippet->setCategoryId($videoCategory);
                }
                $snippet->setTags($videoTags);

                // Create a video status with privacy status. Options are "public", "private" and "unlisted".
                $status = new Google_Service_YouTube_VideoStatus();
                $status->setPrivacyStatus($this->CONFIG_YOUTUBE['PRIVACY_STATUS']);

                // Create a YouTube video with snippet and status
                $video = new Google_Service_YouTube_Video();
                $video->setSnippet($snippet);
                $video->setStatus($status);

                // Size of each chunk of data in bytes. Setting it higher leads faster upload (less chunks,
                // for reliable connections). Setting it lower leads better recovery (fine-grained chunks)
                $chunkSizeBytes = 1 * 1024 * 1024;

                // Setting the defer flag to true tells the client to return a request which can be called
                // with ->execute(); instead of making the API call immediately.
                $client->setDefer(true);

                // Create a request for the API's videos.insert method to create and upload the video.
                $insertRequest = $youtube->videos->insert("status,snippet", $video);

                // Create a MediaFileUpload object for resumable uploads.
                $media = new Google_Http_MediaFileUpload(
                    $client,
                    $insertRequest,
                    'video/*',
                    null,
                    true,
                    $chunkSizeBytes
                );
                $filesize = exec("stat -c %s " .$videoPath);
                $media->setFileSize($filesize);
                $this->utiles->printErrorLog("FILESIZE: [$filesize]", 'upload');

                $chunk_count = 0;
                $total_chunks = round($filesize / $chunkSizeBytes);

                // Read the media file and upload it chunk by chunk.
                $status = false;
                $handle = fopen($videoPath, "rb");
                while (!$status && !feof($handle)) {
                    $chunk = fread($handle, $chunkSizeBytes);
                    $status = $media->nextChunk($chunk);
                    $chunk_count++;
                    $total = round(($chunk_count * 100) / $total_chunks);
                    $this->writeJSONResp($ruta, $ts, "$total% completado", 'uploading');
                }

                fclose($handle);

                /**
                 * Video has successfully been uploaded, now lets perform some cleanup functions for this video
                 */
                if ($status->status['uploadStatus'] == 'uploaded') {
                    $video_final = "https://www.youtube.com/watch?v=";
                    $video_final = $video_final.$status['id'];
                    $this->utiles->printErrorLog('ID: ' . $status['id']);
                    $this->utiles->printErrorLog('Estado: ' . $status->status['uploadStatus']);
                    $this->writeJSONResp($ruta, $ts, $video_final, "ready");
                    $this->utiles->printErrorLog('Se escribe archivo JSON de respuesta', 'info');
                    return $video_final;
                    // Actions to perform for a successful upload
                }

                // If you want to make other calls after the file upload, set setDefer back to false
                $client->setDefer(true);

            } else {
                // @TODO Log error
                $this->utiles->printErrorLog('Error en la autenticacion');
                return '';
            }

        } catch(Google_Service_Exception $e) {
            $this->utiles->printErrorLog("Caught Google service Exception 1 ".$e->getCode(). " message is ".$e->getMessage());
            $this->utiles->printErrorLog("Stack trace is ".$e->getTraceAsString());
            $this->writeJSONResp($ruta, $ts, '', 'Error al subir a YouTube');
        } catch (Exception $e) {
            $this->utiles->printErrorLog("Caught Google service Exception 2 ".$e->getCode(). " message is ".$e->getMessage());
            $this->utiles->printErrorLog("Stack trace is ".$e->getTraceAsString());
            $this->writeJSONResp($ruta, $ts, '', 'Error al subir a YouTube');
        }
    }

    /**
     * Funcion para chequear si el video ya ha sido procesado
     * @param array contiene los campos
     *  $ruta_youtube ruta al archivo de status.
     *  $ts ts del articulo que posee el video.
     */
    public function chequeaEstado($input) {
        $ruta   = $input['ruta_youtube'];
        $ts     = $input['ts'];

        $this->utiles->printErrorLog("Petición actualmente en proceso, chequeando estado", 'info');

        // Se comprueba que el ID de Youtube exista
        $jsonData = $this->readJSONResp($ruta, $ts);
        if ($jsonData['id'] == '') {
            $this->utiles->printErrorLog('el JSON no tiene ID YouTube del Video.');
            $this->writeJSONResp($ruta, $ts, '', 'pending');
            return '';

        }
        return $jsonData;
    }

    /**
     * Busca si existe el ID del video en youtube en el archivo de status
     * @param array $input la ruta al archivo de status y el ts Prontus
     * @return true or false dependiendo ese articulo ya habia subido o no un video a YouTube
     */
    public function wasUploaded($input) {
        $ruta   = $input['ruta_youtube'];
        $ts     = $input['ts'];

        $jsonData = $this->readJSONResp($ruta, $ts);
        return ($jsonData['id'] !== '' && $jsonData['id'] != 'null' );
    }

    /**
     * Escribe a JSON el estado del upload.
     * @param string $ruta ruta al archivo de status.
     * @param string $ts id del artículo Prontus.
     * @param string $videoYoutubeID contiene el id entregado por YouTube
     * @param string $videoYoutubeState contiene el etado en el que se encuentra el video.
     */
    private function writeJSONResp($ruta, $ts, $videoYoutubeID, $videoYoutubeState) {
        if (substr($ruta, 0, 13) == '/prontus_next') {
            $ruta = $this->CONFIG_YOUTUBE['DOCUMENT_ROOT'].$ruta;
        }
        $dir_json = $ruta . 'files/' . substr($ts, 0, 6);
        if (!is_dir($dir_json)) {
            $this->utiles->printErrorLog($dir_json);
            mkdir($dir_json, 0755, true);
        }
        $archivo = $dir_json . '/resp' . $ts . '.json';
        $cont = array(
            'video_id'=>$videoYoutubeID,
            'video_state'=>$videoYoutubeState
        );
        $cont = json_encode($cont);
        file_put_contents($archivo, $cont);
    }

    /**
     * Read JSON response
     * @param string $ruta ruta al archivo de status
     * @param string $ts id del artículo Prontus.
     */
    private function readJSONResp($ruta, $ts) {
        if (substr($ruta, 0, 13) == '/prontus_next') {
            $ruta = $this->CONFIG_YOUTUBE['DOCUMENT_ROOT'].$ruta;
        }
        $archivo = $ruta . 'files/'.substr($ts, 0, 6).'/resp' . $ts . '.json';
        $jsonData['id'] = 0;
        $jsonData['status'] = '';

        if(!is_file($archivo)) {
           $this->utiles->printErrorLog("El JSON de respuesta [$archivo] no existe [$ts]", 'debug');
            $jsonData['id'] = 0;
            return $jsonData;
        }
        $buffer = $this->utiles->getJSONContent($archivo);
        $buffer = json_decode($buffer, true);
        $jsonData['id'] = $buffer['video_id'];
        $jsonData['status'] = $buffer['video_state'];
        return $jsonData;
    }
}
