(function (window, document) {
    var Wizard = {
        downloadingModel: false,
        cgi_cpn: '/cgi-cpn/wizard/',

        /**
         * Funcion que inicia los pasos comunes.
         * Recibe como parámetro el paso que se va a ejecutar
         **/
        init: function(paso) {
            Wizard.bindEvents();
            if (paso == 'paso1') {
                $('#form1').attr('action', Wizard.cgi_cpn + 'wizard_paso1.cgi');

                $('#prontus_id').keypress(function (event) {
                    if (event.keyCode == 46 || event.keyCode == 8 || event.keyCode == 9 || (event.keyCode >= 35 && event.keyCode <= 39)){
                        return true;
                    } else {
                        var regex = new RegExp("^[a-z0-9_\-]+$");
                        var key = event.key || String.fromCharCode(event.which || event.charCode);
                        if (!regex.test(key)) {
                            event.preventDefault();
                            return false;
                        }
                    }
                });
            } else if (paso == 'paso2') {
                $('.modelo a').on("click", function() {
                    var url = $(this).attr('href');
                    $(this).colorbox({
                        open: true,
                        href: url,
                        width:'900px',
                        height:'650px',
                        iframe: true
                    });
                    return false;
                });

                $('.btnSeleccionarModelo').on("click", function (e) {
                    e.preventDefault();
                    var model_id = $(this).attr("data-model-id");

                    $('#PRONTUS_MODEL').val(model_id);
                    Wizard.enviarPaso2();
                });

                $('.modelo:first').addClass('chequeado').find('input').attr('checked', 'checked');

                $('.modelo input').change(function() {
                    $('.modelo').removeClass('chequeado');
                    $(this).parents('.modelo').addClass('chequeado');
                });
            }
        },

        bindEvents: function () {
            $('#btnCheckModules').on("click", function(e) {
                e.preventDefault();
                $('#modal-body-verify').load(Wizard.cgi_cpn + 'wizard_check_platform.cgi', function () {
                    $('#modal_check_modules').modal('show');
                });
            });

            $('#link-license').on("click", function(e) {
                e.preventDefault();
                $('#modal-body-license').load('/prontus/cpan/license/license.html', function () {
                    $('#modal_license').modal('show');
                });
            });

            $('#btnAyuda').on("click", function(e) {
                e.preventDefault();
                Utiles.subWin('/wizard_prontus/core/ayuda.html', 'Ayuda', 800, 600);
            });

            $('#btnPaso1').on("click", function (e) {
                e.preventDefault();
                Wizard.enviarPaso1();
            });

            $('#btnPaso2').on("click", function (e) {
                e.preventDefault();
                Wizard.enviarPaso2();
            });

            $('#btnAnteriorPaso1').on("click", function (e) {
                e.preventDefault();
                Wizard.showPaso1();
            });

            $('#btnAnteriorPaso2').on("click", function (e) {
                e.preventDefault();
                Wizard.showPaso2();
            });

            $('#btnGenerarSitio').on("click", function (e) {
                e.preventDefault();
                Wizard.enviarConfirmacion();
            });

            $('#btnImprimirInforme').on("click", function (e) {
                e.preventDefault();
                Wizard.imprimirResumen();
            });

            $('#btnDownloadModels').on("click", function (e) {
                e.preventDefault();
                Wizard.showDescargar();
            });

            $('#btnCheckInstall').on("click", function (e) {
                e.preventDefault();
                Wizard.checkInstall();
            });

            $('#btnCancelar').on("click", function (e) {
                e.preventDefault();
                Wizard.cancelarWizard();
            });

            $('.btnEliminarModelo').on("click", function (e) {
                e.preventDefault();
                var model_id = $(this).attr("data-id");

                Wizard.eliminarModelo(model_id);
            });

            $('.btnDescargarModelo').on("click", function (e) {
                e.preventDefault();
                var model_id = $(this).attr("data-id");

                Wizard.descargarModelo(model_id);
            });

            $('.btnActualizarModelo').on("click", function (e) {
                e.preventDefault();
                var model_id = $(this).attr("data-id");

                Wizard.actualizarModelo(model_id);
            });

            $('#prontus_id').change(function (e) {
                Wizard.checkInstancia($(this).val());
            });
        },

        /**
         * Inicializa el link para desplegar la ayuda en algunos de los pasos del Wizard
         * El link a la ayuda se define en el botón correspondiente. Lo que hace este función
         * es setear el handler para abrirla en colorbox
         **/
        initHelp: function() {
            $('h3').bind('click', function() {
                $('.info p').hide();
                $(this).next().find('p').show();
            });

            $('.info p').hide();

            var paso = window.location.hash;

            if (paso) {
                paso = paso.replace('#', '');
                $('#titu-' + paso).trigger('click');
            } else {
                $('h3:first').trigger('click');
            }
        },

        /**
         * Ejecuta la CGI para el chequeo de plataforma
         **/
        checkInstall: function() {
            var validator = Wizard.validarPaso1();

            if (!validator.validar()) {
                return false;
            }

            var urlcheck = Wizard.cgi_cpn + 'wizard_check_install.cgi?_prontus_id=' + $('#prontus_id').val();

            $('#modal-body-verify').load(urlcheck, function () {
                $('#modal_check_install').modal('show');
            });
        },

        // Chequea que la instancia no exista.
        checkInstancia: function (prontus_id) {
            var url = Wizard.cgi_cpn + 'wizard_check_instancia.cgi';
            $.ajax({
                url: url,
                dataType: 'json',
                type: 'post',
                data: {_prontus_id: prontus_id},
                success: function(response) {
                    if (response.status == 0) {
                        $('#prontus_id').attr("style", "background-color: rgb(253, 248, 193);");
                        $('#check_prontus_msg').show();
                    } else {
                        $('#prontus_id').removeAttr("style");
                        $('#check_prontus_msg').hide();
                    }
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    Wizard.handleError(urlCGI, jqXHR, textStatus, errorThrown);

                },
            });
        },

        /**
         * Validaciones para el áso numero 1
         * Luego de validar, envía el formulario para pasar al paso 2
         **/
        enviarPaso1: function() {
            var validator = Wizard.validarPaso1();

            if (!validator.validar()) {
                return false;
            }

            validator.send();
        },

        validarPaso1: function () {
            var validator = new Validador('form1','one','#FDF8C1');

            validator.addconstraint('PRONTUS_ID','obligatorio','','Debe indicar el nombre del publicador.');
            validator.addconstraint('PRONTUS_ID','regex',new RegExp(/^[a-z0-9\_\-]{1,32}$/),'Nombre de publicador no válido. Debe contener letras minúsculas sin tilde, números, guión o underscore, con un máximo de 32 caracteres.');
            validator.addconstraint('NEW_TITLE_SITE_NAME','obligatorio','','Debe indicar el Nombre del Sitio Prontus.');
            validator.addconstraint('SERVER_BD','obligatorio','','Debe indicar el Servidor de BD');
            validator.addconstraint('SERVER_BD','regex',new RegExp(/^[\w\-\.]{1,128}$/),"Servidor de BD no es válido.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, punto, guión y underscore, máximo 128.");
            validator.addconstraint('NOM_BD','obligatorio','','Debe indicar el Nombre de BD');
            validator.addconstraint('NOM_BD','regex',new RegExp(/^[\w\-]{1,64}$/),"Nombre de BD no es válido.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, guión y underscore, máximo 64.");
            validator.addconstraint('USER_BD','obligatorio','','Debe indicar el Usuario de BD');
            validator.addconstraint('USER_BD','regex',new RegExp(/^[\w\-]{1,16}$/),"Usuario de BD no es válido.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, guión y underscore, máximo 16.");
            validator.addconstraint('PWD_BD','obligatorio','','Debe indicar la Contraseña para usuario de BD');
            validator.addconstraint('PWD_BD','regex',new RegExp(/^[\w\-\.\@\:\$%!#]{0,64}$/),"Contraseña para usuario de BD no es válida.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, y los caracteres ., _, -, @, $, %, # y !, máximo 64.");

            /* para creacion BD */
            var superuser = $('input[name="SUPERUSER_BD"]').val();

            if (superuser !== "") {
                validator.addconstraint('SUPERUSER_BD','regex',new RegExp(/^[\w\-]{1,16}$/),"Usuario para creación de BD no es válido.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, guión y underscore, máximo 16.");
                validator.addconstraint('SUPERPWD_BD','obligatorio','','Debe indicar la Contraseña para creación de BD');
                validator.addconstraint('SUPERPWD_BD','regex',new RegExp(/^[\w\-\.\@\:\$%!#]{1,64}$/),"Contraseña para creación de BD no es válida.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, y los caracteres ., _, -, @, $, %, # y !, máximo 64.");
            }

            validator.addconstraint('PRONTUS_SMTP','obligatorio','','Debe indicar el servidor SMTP');
            validator.addconstraint('PRONTUS_SMTP','regex',new RegExp(/^[A-Z\.a-z\_\-0-9]+$/),"Servidor SMTP no válido.\nCaracteres permitidos:letras minúsculas o mayúsculas, dígitos, punto, guión y underscore.");

            return validator;
        },

        /**
         * Validaciones y envío del paso 2
         * Luego de validar, envía el formulario para pasar al resumen
         **/
        enviarPaso2: function() {
            var validator = new Validador('form1','one','#FDF8C1');
            validator.addconstraint('PRONTUS_MODEL','obligatorio','','Debe indicar el Modelo Prontus que va a utilizar.');

            if (!validator.validar()) {
                return false;
            }

            validator.send();
        },

        /**
         * Gatilla la CGI que genera el Prontus, al final del Wizard
         * Por ahor asólo se hace un submit, pero a futuro se podrían agregar
         * validaciones o confirmaciones, etc
         **/
        enviarConfirmacion: function() {
            $('[name="form1"]').trigger('submit');
        },

        /**
         * Funcion encargada de volver al comienzo del Wizard.
         * Como ese se abre generalmente en una ventana normal, no se pude hacer window.close()
         * Se usa esta función, para centralizar el comportamiento del botón.
         **/
        cancelarWizard: function() {
            window.location.href = '/wizard_prontus/';
        },

        /**
         * Centraliza la función de imprimir del Paso de Confirmación
         * Actualmente sólo realiza un window.print(), pero a futuro, se podría implementar
         * algo un poco más elaborado.
         **/
        imprimirResumen: function() {
            window.print();
        },

        /**
         * Muestra la ventana de administración de los modelos
         **/
        showDescargar: function() {
            window.location.href = 'wizard_show_models.cgi';
        },

        /**
         * Muestra el paso numero 2.
         * Se usa para volver, desde el resumen o el admin de modelos
         **/
        showPaso2: function() {
            window.location.href = 'wizard_show_paso2.cgi';
        },

        /**
         * Muestra el paso numero 2.
         * Se usa para volver, desde el paso 2
         **/
        showPaso1: function() {
            window.location.href = '/wizard_prontus/core/paso1.html';
        },

        /**
         * Handler utilizado en el Administrador de Modelos.
         * Recibe como parámetro el modelo que se desea eliminar
         * Invoca la CGI de borrado y luego borra el modelo del listado
         **/
        eliminarModelo: function(id) {
            if (confirm('¿Está seguro que desea eliminar este modelo?')) {
                if (Wizard.downloadingModel) {
                    alert('Por favor, espere a que termine la descarga anterior');
                    return;
                }

                Wizard.showLoading(id, true);
                var urlCGI = './wizard_models_delete.cgi';

                $.ajax({
                    url: urlCGI,
                    data: {modelid: id},
                    success: function(resp) {
                        if (typeof resp !== 'Object' && typeof resp !== 'object') {
                            alert('Respuesta no valida desde el servidor');
                            return;
                        }

                        if (resp.status == 0) {
                            alert(resp.msg);
                        } else {
                            if (resp.nodisponible) {
                                $('#idmodel-' + id).fadeOut(function() {
                                    $(this).remove();
                                });
                            } else {
                                $('#idmodel-'+id).fadeOut('slow', function() {
                                    $('#idmodel-' + id).insertAfter('#models .instalado:last').show();
                                    $('#idmodel-' + id + ' .version .actual').html('No instalado');
                                    $('#idmodel-' + id).removeClass('actualizar').removeClass('actualizado').removeClass('nodisponible');
                                    $('#idmodel-' + id).removeClass('instalado').addClass('noinstalado');
                                });
                            }
                        }
                    },
                    complete: function() {
                        Wizard.showLoading(id, false);
                    },
                    error: function(jqXHR, textStatus, errorThrown) {
                        Wizard.handleError(urlCGI, jqXHR, textStatus, errorThrown);
                    }
                });
            }
        },

        /**
         * Metodo encargado de invocar la CGI que hace la descarga del modelo
         **/
        descargarModelo: function(id) {
            if (Wizard.downloadingModel) {
                alert('Por favor, espere a que termine la descarga anterior');
                return;
            }

            if (!confirm('Se descargará este modelo. ¿Estás seguro?')) {
                return;
            }

            Wizard.showLoading(id, true);
            var urlCGI = './wizard_models_download.cgi';

            $.ajax({
                url: urlCGI,
                data: {modelid: id},
                success: function(resp) {
                    if (typeof resp !== 'Object' && typeof resp !== 'object') {
                        alert('Respuesta no valida desde el servidor');
                        return;
                    }

                    if (resp.status == 0) {
                        alert(resp.msg);
                    } else {
                        Wizard.checkStatus(id);
                    }
                },
                complete: function() {
                    //Wizard.showLoading(id, false);
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    Wizard.handleError(urlCGI, jqXHR, textStatus, errorThrown);

                }
            });

        },

        checkStatus: function (id, type) {
            if (typeof type === 'undefined') {
                type = 'download';
            }

            Wizard.showLoading(id, true);

            var urlCGI = './wizard_models_status.cgi';
            $.ajax({
                url: urlCGI,
                data: {modelid: id, type: type},
                success: function(resp) {
                    if (typeof resp !== 'Object' && typeof resp !== 'object') {
                        alert('Respuesta no valida desde el servidor');
                        return;
                    }

                    if (resp.status == 1) {
                        Wizard.showLoading(id, true, resp.msg);

                        setTimeout(function () {
                            Wizard.checkStatus(id, type);
                        }, 1500);
                    } else if (resp.status == 0) { // error.
                        alert(resp.msg);
                    } else {
                        if (type == 'update') {
                            $('#idmodel-' + id + ' .version-actual').text($('#idmodel-' + id + ' .version .last').html());
                            $('#idmodel-' + id).removeClass('actualizar').addClass('actualizado');
                        } else {
                            $('#idmodel-' + id).find('.btnDescargarModelo').hide();
                            $('#idmodel-' + id).find('.btnEliminarModelo').show();
                            $('#idmodel-' + id).find('.btnActualizarModelo').hide();
                            $('#idmodel-' + id).find('.version-instalada').text($('#idmodel-' + id).find('.ultima-version').text());
                            $('#idmodel-' + id).removeClass('noinstalado').addClass('instalado');
                            $('#idmodel-' + id).addClass('actualizado');
                        }

                        Wizard.showLoading(id, false);
                    }
                },
                complete: function() {

                },
                error: function(jqXHR, textStatus, errorThrown) {
                    Wizard.handleError(urlCGI, jqXHR, textStatus, errorThrown);
                }
            });
        },

        /**
         * Este metodo se usa para Actualizar un modelo
         * Pese a que usa la misma CGI de descarga, el comportamiento post descarga es distinto
         **/
        actualizarModelo: function(id) {
            if (Wizard.downloadingModel) {
                alert('Por favor, espere a que termine la descarga anterior');
                return;
            }

            Wizard.showLoading(id, true);

            var urlCGI = './wizard_models_download.cgi';

            $.ajax({
                url: urlCGI,
                data: {modelid: id},
                success: function(resp) {
                    if (typeof resp !== 'Object' && typeof resp !== 'object') {
                        alert('Respuesta no valida desde el servidor');
                        return;
                    }

                    if (resp.status == 0) {
                        alert(resp.msg);
                    } else {
                        Wizard.checkStatus(id, 'update');
                    }
                },
                complete: function() {
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    Wizard.handleError(urlCGI, jqXHR, textStatus, errorThrown);
                }
            });
        },

        /**
         * Para manejar el evento de descarga. Se centraliza en este método
         * la lógica de "bloquear" la interface mientras está descargando
         **/
        showLoading: function(id, flag, msg) {
            if (typeof msg == 'undefined') {
                msg = '';
            }

            if (flag) {
                Wizard.downloadingModel = true;
                $("#idmodel-" + id + ' .acciones').hide();
                $("#idmodel-" + id + ' .loading').show();

                if (msg) {
                    $("#idmodel-" + id + ' .loading .msg-loading').text(msg);
                    $("#idmodel-" + id + ' .loading .msg-loading').show();
                }
            } else {
                Wizard.downloadingModel = false;
                $("#idmodel-" + id + ' .acciones').show();
                $("#idmodel-" + id + ' .loading').hide();
            }
        },

        /**
         * Funcion encargada de manejar el error que viene de las respuestas Ajax
         * Se deja centralizado para cambiar el tipo de manejo en caso necesario
         **/
        handleError: function(url, XMLHttpRequest, textStatus, errorThrown) {
            alert("Server error procesando request ajax:\nURL invocada:"
                    + url + "\ntextStatus="
                    + textStatus + "\nXMLHttpRequest.status="
                    + XMLHttpRequest.status + '-' + XMLHttpRequest.statusText
                    + "\nXMLHttpRequest.responseText=[" + XMLHttpRequest.responseText
                    + "]\nResponseHeaders=" + XMLHttpRequest.getAllResponseHeaders() );
        }
    };

    window.Wizard = Wizard;
})(window, document);
