#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl/
# 2002 - 2012 Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

package Artic;
use strict;
use utf8;
use warnings;
no warnings 'uninitialized';

# ---------------------------------------------------------------
# errores orientados a que los pueda leer el usuario, sin info del ambiente.
# el texto completo del error se tira al STDERR
our $ERR = '';

# elementos privados de un articulo Prontus
our @PRIVATE_ELEMENTS = ('_txt_titular', '_url', '_slug', '_custom_slug', '_users_id', '_fid', '_plt', '_fechap', '_horap', '_fechae', '_horae', '_fecha_ini', '_fecha_fin', '_soloportadas', '_publicarvistas', '_seccion1', '_tema1', '_subtema1', '_seccion2', '_tema2', '_subtema2', '_seccion3', '_tema3', '_subtema3', '_nom_seccion1', '_nom_tema1', '_nom_subtema1', '_nom_seccion2', '_nom_tema2', '_nom_subtema2', '_nom_seccion3', '_nom_tema3', '_nom_subtema3', '_autor', '_txt_bajada', '_alta', '_tax', '_tags', '_tagnames', '_multitag_seccion', '_multitag_tema', '_multitag_subtema', '_galeria_prontus_conf', '_galeria_flag_imagen_temporal', '_galeria_prontus_str', '_txt_galeria_description_total', '_txt_galeria_credito_total', '_s3', '_mam', '_s3_size', '_s3_media_size', '_s3_asset_size', '_total_size', '_disk_size', '_media_size', '_asset_size', '_imag_size', '_modtime', '_indexar', '_priorizar');

# Crea el objeto y retorna una referencia a el.
# Este metodo es el primer paso obligado tanto para crear o leer un articulo.
sub new {
    my $clase = shift;
    my $artic = {@_};
    bless $artic, $clase;

    # Atributos recibidos.
    $artic->{document_root}      ||= $ENV{'DOCUMENT_ROOT'};    # valor por omision
    $artic->{prontus_id}         ||= '';
    $artic->{public_server_name} ||= '';
    $artic->{cpan_server_name}   ||= '';
    $artic->{no_check_dirs}      ||= '';
    if (   (!lib_utils::valida_prontus($artic->{prontus_id}))
        || ($artic->{public_server_name} eq '')
        || ($artic->{cpan_server_name}   eq '')) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_new_error_param', $artic->{prontus_id}, $artic->{public_server_name}, $artic->{cpan_server_name});
        return 0;
    }

    # Si no se indica el TS, asigna uno nuevo
    $artic->{ts} ||= '';
    if ($artic->{ts} ne '') {
        if ($artic->{ts} !~ /^\d{14}$/) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_new_error_ts', $artic->{ts});
            return 0;
        }
    } else {
        $artic->{ts} = $artic->_get_new_ts();
    }

    $artic->{fechac} = substr($artic->{ts}, 0, 8);

    # Establece y chequea dirs
    $artic->_set_dirs() || return 0;
    $artic->{fullpath_xml} = "$artic->{dst_xml}/$artic->{ts}" . '.xml';
    $artic->{fullpath_json} = "$artic->{dst_json}/$artic->{ts}" . '.json';

    # Si el hash no viene, se inicializa como vacio.
    # Los campos de este hash se accesan asi: my $val_campo = $artic->{campos}->{$nom_campo};
    if (!ref($artic->{campos})) {
        $artic->{campos} = {};    # es {} porque es una referencia, sino seria () # asume todos los nombres de campo en minusc.
    }

    # Inicializa hash de datos que cargaran desde el xml
    $artic->{xml_content} = {};
    # contendra el xml leido desde el disco
    $artic->{xml_data} = '';

    # Inicializa hash de datos de videos multimedia
    $artic->{media_metadata} = {};

    # Se llena en el metodo que genera la vista generar_vista_art()
    $artic->{post_proceso_lista} = ();

    # indica si el artículo esta publicado (tiene alta y cumple la fecha de publicacion)
    $artic->{published} = -1;

    # Indica si el articulo se está previsualizando o no.
    $artic->{is_preview} = 0;

    # Indica si el articulo se guardó, aunque no haya completado los demas procesos
    $artic->{saved} = 0;
    return $artic;
}

# Establece que el articulo es un preview.
sub set_preview_artic {
    my ($this, $ts) = @_;

    $this->{is_preview} = 1;
    # Fuerza el alta en el xml del preview.
    $this->{'campos'}->{'_alta'} = 1;

    # Setea los directorios para realizar el preview.
    $this->_set_dirs_preview();

    # Re-asigna el path xml
    $this->{fullpath_xml} = "$this->{dst_preview}/$this->{fechac}/xml/$this->{ts}" . '.xml';

    lib_stdlog::print_err("set_preview_artic: fullpath_xml[$this->{fullpath_xml}]\n");
}

# Retorna la ruta absoluta del artic si es que se cuenta con ella en el objeto.
# plt: puede venir de los campos recibidos del form, o bien, desde el xml
sub get_fullpath_artic {
    my ($this, $plt, $mv, $is_paralela) = @_;

    return $this->{document_root} . $this->get_rel_path_artic($plt, $mv, $is_paralela);
}

# Retorna la ruta del artic si es que se cuenta con ella en el objeto.
# plt: puede venir de los campos recibidos del form, o bien, desde el xml
sub get_rel_path_artic {
    my ($this, $plt, $mv, $is_paralela) = @_;

    if (($plt ne '') && ($this->{ts} ne '')) {
        my $fulldir_vista = $this->_get_path_vista($mv, $is_paralela);
        if ($is_paralela) {
            return "$fulldir_vista/$this->{ts}_$plt";
        } else {
            my $ext = lib_utils::get_file_extension($plt);
            return "$fulldir_vista/$this->{ts}" . '.' . $ext;
        }
    }
    return '';
}

# Retorna el nombre del archivo del articulo, por ejemplo 20090522104356.html
# plt: puede venir de los campos recibidos del form, o bien, desde el xml
sub get_nom_artic {
    my ($this, $plt)  = @_;
    return $this->{ts} . '.' . lib_utils::get_file_extension($plt);

}

# No implementada. Se llama automaticamente al destruirse el objeto
sub DESTROY {
}

# Una vez escrito el articulo, ubica el nombre del script de post-proceso (con extension y
# con path absoluto completo) y lo ejecuta.
# Script de postproceso es optativo y puede ir en cualquier parte del articulo.
# Pueden ser varios.
sub exec_post_proceso_art {
    my ($this, $pathArticulo, $isNew) = @_;

    return if (!defined($this->{post_proceso_lista}));

    # Ejecuta en bground el proceso pasandole x param. el path completo al articulo.
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    my $newOrEdit = 'E';
    $newOrEdit = 'N' if ($isNew);

    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if($pathnice);

    lib_stdlog::print_err("[PPROC]\n");
    # lista de postprocesos
    foreach my $pp (@{$this->{post_proceso_lista}}) {
        # Para que sea un script valido debe ubicarse en el mismo dir. de cgi del prontus o a
        # lo mas un nivel hacia arriba.
        if (($pp =~ /^\w/) || ($pp =~ /^\.\.(\/|\\)\w/)) {
            $pp =~ s/[#;&<>|~]+//g;
            my $cmd = "$pathnice $rutaScript/$pp $pathArticulo $this->{public_server_name} $newOrEdit >/dev/null 2>&1 &";
            lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
            system $cmd;
        }
    }
    lib_stdlog::print_err("[/PPROC]\n");
}

# Asigna timestamp unico para el articulo.
sub _get_new_ts {
    my $this = shift;
    my $ts     = glib_hrfec_03::get_dtime_pack4();
    my $fechac = substr($ts, 0, 8);

    while (-f "$this->{document_root}/$this->{prontus_id}/site/artic/$fechac/xml/$ts" . '.xml') {
        $ts = glib_hrfec_03::suma_segs($ts, 1);
        $fechac = substr($ts, 0, 8);
    }

    return $ts;
}

# Genera la estructura de datos esencial para guardar en XML, y escribe a disco.
# En este XML queda solo lo esencial. Los datos derivados que se calculan u obtienen en base a los esenciales no se almacenan aqui.
sub generar_xml_artic {
    my ($this, $keep_time) =  @_;

    # Construccion xml articulo
    $this->{xml_data} = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<artic_data>\n<_private>\n";

    # Adecuar algunos datos que lo requieran para luego reemplazar todo de una en el XML
    if ($this->{campos}->{'_fechap'} eq '') {
        $this->{campos}->{'_fechap'} = lib_prontus::asigna_fecha_default('_fechap');
    }

    # Cada vez que se guarda el articulo se actualiza la fecha de modificacion en formato epoch.
    $this->{campos}->{'_modtime'} = time if (!$keep_time);
    # si campo indexar no viene se asigna valor 1
    $this->{campos}->{'_indexar'} = 0 if (!defined($this->{campos}->{'_indexar'}));

    $this->{campos}->{'_s3'} = 0 if (!defined($this->{campos}->{'_s3'}));
    $this->{campos}->{'_s3_size'} = 0 if (!defined($this->{campos}->{'_s3_size'}));

    if ($this->{campos}->{'_fechae'} eq '') {
        $this->{campos}->{'_fechae'} = lib_prontus::asigna_fecha_default('_fechae');
    }

    # titular
    if ($this->{campos}->{'_txt_titular'} eq '') {
        $this->{campos}->{'_txt_titular'} = 'SIN TITULO ' . (time - $prontus_varglb::URL_NUMBER);
    } else {
        $this->{campos}->{'_txt_titular'} =~ s/^\s+//;
        $this->{campos}->{'_txt_titular'} =~ s/\s+$//;
    }

    $this->{campos}->{'_tags'} =~ s/[^\w\,\-áéíóúüñÁÉÍÓÚÜÑ \.]//g;
    $this->{campos}->{'_tags'} =~ tr/ÁÉÍÓÚÜÑ/áéíóúüñ/;
    $this->{campos}->{'_tags'} = lc($this->{campos}->{'_tags'});

    $this->_artic_ajusta_horas();

    # fechas de calendario
    if (!defined($this->{campos}->{'_fecha_ini'}) or $this->{campos}->{'_fecha_ini'} eq '') {
        $this->{campos}->{'_fecha_ini'} = $this->{campos}->{'_fechap'};
    }

    if ($this->{campos}->{'_fecha_ini'} eq '99999999' or $this->{campos}->{'_fecha_ini'} eq '') {
        $this->{campos}->{'_fecha_ini'} = substr($this->{ts}, 0, 8);
    }

    if (!defined($this->{campos}->{'_fecha_fin'}) or $this->{campos}->{'_fecha_fin'} eq '') {
        $this->{campos}->{'_fecha_fin'} = $this->{campos}->{'_fechap'};
    }

    if ($this->{campos}->{'_fecha_fin'} eq '99999999' or $this->{campos}->{'_fecha_fin'} eq '') {
        $this->{campos}->{'_fecha_fin'} = substr($this->{ts}, 0, 8);
    }

    # Genera y arma campos privados de xml
    foreach my $nom_campo (@PRIVATE_ELEMENTS) {
        my $val_campo = $this->{campos}->{$nom_campo};
        $val_campo = '' if (!defined($val_campo));
        $val_campo = '' if ($nom_campo =~ /^(_seccion|_tema|_subtema)/ && $val_campo eq '0');
        if (($nom_campo eq '_soloportadas' or $nom_campo eq '_alta') and $val_campo eq '' ) {
            $val_campo = 0;
        }

        # Codigo de reemplazo por ALD 08/02/2011
        # quita caracteres de control
        $val_campo =~ s/[\x01-\x09\x0B\x0C\x0E-\x1F]//g;
        if ($val_campo ne '' && substr($nom_campo, 0, 5) eq  '_txt_' || $val_campo =~ /[<>&]/) {
            $val_campo = "<![CDATA[$val_campo]]>";
        }
        $this->{xml_data} .= "<$nom_campo>$val_campo</$nom_campo>\n";
    }

    $this->{xml_data} .= "</_private>\n<_public>\n</_public>\n</artic_data>\n";

    # Guarda recursos del articulo
    $this->_guarda_recursos('asocfile');
    $this->_guarda_recursos('multimedia');
    $this->_guarda_recursos('foto');

    # Fotos fijas, a continuacion de las fotos
    $this->_guarda_fotosfijas();

    foreach my $nom_campo (keys %{$this->{campos}}) {
        # Solo permite campos con _ para los reservados que van en el XML
        next if (substr($nom_campo, 0, 1)  eq '_');

        # Salta los recursos, ya procesados previamente
        next if ($nom_campo =~ /^(asocfile_|multimedia_|foto_|fotofija_|_wfoto_|_hfoto_)/);
        my $val_campo = $this->{campos}->{$nom_campo};
        next if (!defined($val_campo));
        # Codigo de reemplazo por ALD 08/02/2011
        $val_campo =~ s/[\x01-\x09\x0B\x0C\x0E-\x1F]//g;
        next if ($val_campo eq '');

        if (substr($nom_campo, 0, 4)  eq 'vtxt') {
            # extrae fotos en base 64 publicadas en el cuerpo
            $val_campo = $this->_guarda_fotos_cuerpo($val_campo);
        }

        if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'VTXT_RELPATH_LINK'} eq 'SI' && $nom_campo =~ /^vtxt_/i) {
            $val_campo =~ s|https?://$this->{cpan_server_name}/|/|ig;
            $val_campo =~ s|https?://$this->{public_server_name}/|/|ig;
        }
        $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, $nom_campo, $val_campo, 1);
    }

    $this->{campos}->{'_s3_size'} = 0 if (!defined($this->{campos}->{'_s3_size'}) or $this->{campos}->{'_s3_size'} eq '');
    my ($disk_size, $imag_size, $asset_size, $media_size) = $this->get_disk_size();

    my $total_size = $this->{campos}->{'_s3_size'} + $disk_size;
    $this->update_art_size($total_size, $disk_size, $imag_size, $asset_size, $media_size);

    # borra el contenido anterior del hash de campos, porque si se escribe uno nuevo, pues ya no corre.
    # esta var lo usa el obj. para no leer mas de 1 vez el xml
    %{$this->{xml_content}} = ();

    # Se escribe el XML a disco
    return $this->_flush_xml();
}

# Retorna hash de datos leidos del xml con los nombres de campo en minusculas.
# Si los datos ya estan cargados en el objeto, no lo lee de nuevo.
sub get_xml_content {
    my $this = $_[0];
    my $num_elem = scalar keys(%{$this->{xml_content}});
    # lib_stdlog::print_err("ts [$this->{ts}] get content[$this->{fullpath_xml}] num_elem [$num_elem]\n");
    if ($num_elem < 1) {
        $this->{xml_data} = glib_fildir_03::read_file($this->{fullpath_xml});
        my $xml_data = $this->{xml_data};
        # borra tags innecesarios
        $xml_data =~ s|</?artic_data>||g;
        $xml_data =~ s|</?_private>||g;
        $xml_data =~ s|</?_public>||g;
        %{$this->{xml_content}} = lib_prontus::getCamposXml($xml_data);

        # Cargar datos de los archivos multimedia_video
        foreach my $media (keys %{$this->{xml_content}}) {
            next unless(substr($media, 0, 11) eq 'multimedia_' && index($media, '.') < 0);

            my $type = substr($media, 0, 16);
            my $fullpath_json = "$this->{dst_multimedia}/$media$this->{ts}.json";
            # lib_stdlog::print_err("path json [$fullpath_json]\n");
            if (-f $fullpath_json) {
                my $buffer_json = glib_fildir_03::read_file_bin($fullpath_json);
                # lib_stdlog::print_err("[$this->{ts}] buffer_json [$buffer_json]\n");

                # decodifica datos de json
                eval { $this->{media_metadata}->{$media} = JSON::decode_json($buffer_json); };
                if ($@) {
                    lib_stdlog::print_err(lib_lang::get_interpolated_message('global_json_file_decode_error', $@, $fullpath_json, $buffer_json) . "\n");
                    $this->{media_metadata}->{$media} = undef;
                    next if ($type eq 'multimedia_video');
                }

                $this->{xml_content}->{"$media.json"} = lib_utils::remove_front_string($fullpath_json, $this->{document_root});
            }

            if ($type eq 'multimedia_video') {
                # pasamos los datos de un hash a marcas separadas por ".", hasta 3 niveles de profundidad
                foreach my $key1 (keys %{$this->{media_metadata}->{$media}}) {
                    if ( ref $this->{media_metadata}->{$media}->{$key1} eq 'HASH') {
                        foreach my $key2 (keys %{$this->{media_metadata}->{$media}->{$key1}}) {
                            if ( ref $this->{media_metadata}->{$media}->{$key1}->{$key2} eq 'HASH') {
                                foreach my $key3 (keys %{$this->{media_metadata}->{$media}->{$key1}->{$key2}}) {
                                    $this->{xml_content}->{"$media.$key1.$key2.$key3"} = $this->{media_metadata}->{$media}->{$key1}->{$key2}->{$key3};
                                }
                            } else {
                                $this->{xml_content}->{"$media.$key1.$key2"} = $this->{media_metadata}->{$media}->{$key1}->{$key2};
                            }
                        }
                    } else {
                        $this->{xml_content}->{"$media.$key1"} = $this->{media_metadata}->{$media}->{$key1};
                    }
                }

                # sino existe 'size' en la metadata se debe determinar
                if (!defined($this->{media_metadata}->{$media}->{'size'})) {
                    # Obtenemos tamaño del archivo original
                    $this->{xml_content}->{"$media.size"} = glib_str_03::bytes2kb(-s "$this->{dst_multimedia}/$this->{xml_content}->{$media}");
                }

            } elsif ($type eq 'multimedia_audio') {
                # Si no existen los datos intentamos generarlos
                if (!defined($this->{media_metadata}->{$media}) or !defined($this->{media_metadata}->{$media}->{duration})) {
                    my $fullpath      = $this->{dst_multimedia} .'/'. $this->{xml_content}->{$media};
                    my $resp;
                    if (-s $fullpath) {
                        $resp = `$prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_FFMPEG'}/ffmpeg -i $fullpath 2>&1`;
                    }

                    my %data = ();
                    if ($resp =~ /Duration:\s*((\d+):(\d+):([\d\.]+))/is) {
                        $data{duration_hms} = $1;
                        $data{duration} = $2*3600 + $3*60 + $4;
                        if ($resp =~ /Audio: ([^,]+),(.+?)([0-9\.]+) kb\/s/is) {
                            $data{acodec} = $1;
                            my $other = $2;
                            $data{abitrate} = $3;

                            if ($other =~ /(\d+)\s*Hz/) {
                                $data{rate} = $1;
                            }
                            $data{size} = glib_str_03::bytes2kb(-s "$this->{dst_multimedia}/$this->{xml_content}->{$media}");

                            my $buffer_json = JSON::encode_json(\%data);
                            glib_fildir_03::write_file_bin($fullpath_json, $buffer_json);
                        } else {
                            lib_stdlog::print_err(lib_lang::get_interpolated_message('artic_media_get_data_error', $this->{ts}, $fullpath, $media) . "\n");
                        }
                    } else {
                        lib_stdlog::print_err(lib_lang::get_interpolated_message('artic_media_get_duration_error',$this->{ts}, $fullpath, $media). "\n");
                    }

                    $this->{media_metadata}->{$media} = \%data;
                }

                # pasamos los datos de un hash a marcas separadas por "."
                foreach my $key1 (keys %{$this->{media_metadata}->{$media}}) {
                    $this->{xml_content}->{"$media.$key1"} = $this->{media_metadata}->{$media}->{$key1};
                }
            }

            # extension de la media
            $this->{xml_content}->{"$media.ext"} = lib_utils::get_file_extension($this->{xml_content}->{$media});
        }

        # indexar solo puede valer 1 o 0, si no existía en xml, entonces debe ser 1
        if (!defined($this->{xml_content}->{'_indexar'})) {
            $this->{xml_content}->{'_indexar'} = 1;
        }

        # priorizar solo puede valer 1 o 0, si no existía en xml, entonces debe ser 0
        if (!defined($this->{xml_content}->{'_priorizar'})) {
            $this->{xml_content}->{'_priorizar'} = 0;
        }

        # publicar vistas no esta definido en artículos antiguos
        if (!defined($this->{xml_content}->{'_publicarvistas'})) {
            if (defined($this->{xml_content}->{'_soloportadas'}) && $this->{xml_content}->{'_soloportadas'}) {
                $this->{xml_content}->{'_publicarvistas'} = 1;
            }
        }

        # inicializa con valor vacío campos reservados del artículo que no se encontraron
        foreach my $nom_campo (@PRIVATE_ELEMENTS) {
            if (!defined($this->{xml_content}->{$nom_campo})) {
                $this->{xml_content}->{$nom_campo} = '';
            }
        }

        # las taxonomias no deben tener valor cero/'0'
        for (my $i = 1 ; $i <= 3 ; $i++) {
            $this->{xml_content}->{"_seccion$i"} = '' if ($this->{xml_content}->{"_seccion$i"} eq '0');
            $this->{xml_content}->{"_tema$i"}    = '' if ($this->{xml_content}->{"_tema$i"}    eq '0');
            $this->{xml_content}->{"_subtema$i"} = '' if ($this->{xml_content}->{"_subtema$i"} eq '0');
        }

        if ($this->{xml_content}->{'_fechap'} eq '') {
            $this->{xml_content}->{'_fechap'} = lib_prontus::asigna_fecha_default('_fechap');
        }
        if ($this->{xml_content}->{'_fechae'} eq '') {
            $this->{xml_content}->{'_fechae'} = lib_prontus::asigna_fecha_default('_fechae');
        }

        # fechas de calendario
        if ($this->{xml_content}->{'_fecha_ini'} eq '' and $this->{xml_content}->{'_fechap'} ne '99999999') {
            $this->{xml_content}->{'_fecha_ini'} = $this->{xml_content}->{'_fechap'};
        } else {
            $this->{xml_content}->{'_fecha_ini'} = substr($this->{ts}, 0, 8);
        }

        if ($this->{xml_content}->{'_fecha_fin'} eq '' and $this->{xml_content}->{'_fechae'} ne '99999999') {
            $this->{xml_content}->{'_fecha_fin'} = $this->{xml_content}->{'_fechae'};
        } else {
            $this->{xml_content}->{'_fecha_fin'} = substr($this->{ts}, 0, 8);
        }
    }

    $this->{xml_content}->{'_prontus_art'} = $this->{prontus_id};
    $this->{xml_content}->{'_fechac'} = $this->{fechac};
    $this->{xml_content}->{'_ts'} = $this->{'ts'};
    $this->{xml_content}->{'prontus_id'} = $this->{'prontus_id'};
    return %{$this->{xml_content}};
}

# Guarda tags en bd
# Se invoca al momento de guardar un articulo.
sub tags2bd {
    my ($this, $base) = @_;

    $this->get_xml_content();
    my @tags = split(/,/, $this->{xml_content}->{'_tags'});
    # para evitar posibles tags repetidos
    my %temp;
    foreach my $tag (@tags) {
        $temp{$tag} = 1;
    }
    @tags = keys %temp;

    # Primero elimina los tags del artic actual:
    my $tagsart_table = lib_utils::get_table('tagsart');
    my $sql = "delete from $tagsart_table where tagsart_idart = ?";
    my @params = ($this->{ts});
    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_tags_delete_error', $this->{ts}) ."\n";
        lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
        return 0;
    }

    # Asocia los TAGS al artic
    foreach my $idtag (@tags) {
        $sql = "insert into $tagsart_table set tagsart_idtags= ?, tagsart_idart = ?";
        @params = ($idtag, $this->{ts});
        $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_tags_update_error', $this->{ts}) ."\n";
            lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
            return 0;
        }
    }

    return 1;
}

# puebla tabla de multitag
# Se invoca al momento de guardar un articulo.
sub multitags2bd {
    my ($this, $base) = @_;
    my %tipos = ('s' => '_multitag_seccion', 't' => '_multitag_tema', 'st' => '_multitag_subtema');

    $this->get_xml_content();
    foreach my $type (keys %tipos) {
        # Primero elimina los tags del artic actual:
        my $multitag_art_table = lib_utils::get_table("multitag_art_$type");
        my $sql = "delete from $multitag_art_table where multitag_art_$type\_art_id = ?";
        my @params = ($this->{ts});
        my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_multitags_delete_error', $this->{ts}) ."\n";
            lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
            return 0;
        }

        # Asocia los TAGS al artic
        next if (!defined($this->{'xml_content'}->{$tipos{$type}}));
        my @multitags = split(/,/, $this->{'xml_content'}->{$tipos{$type}});
        foreach my $id (@multitags) {
            $sql = "insert into $multitag_art_table set multitag_art_$type\_art_id = ?, multitag_art_$type\_id = ?";
            @params = ($this->{ts}, $id);
            my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
            if ($base->errstr) {
                $Artic::ERR = lib_lang::get_interpolated_message('artic_multitags_update_error', $this->{ts}) ."\n";
                lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
                return 0;
            }
        }
    }
    return 1;
}

# puebla tabla de urls friendly v5
# Se invoca al regenerar la DB de prontus
sub friendly_v5_2bd {
    my ($this, $base) = @_;

    $this->get_xml_content();

    if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_EXCLUDE_FID'}{$this->{'xml_content'}->{'_fid'}}) {
        my %options = %{$_[1]};
        my $urlf5_table = lib_utils::get_table('urlf5');
        my $sql = "insert into $urlf5_table set urlf5_type = ?, urlf5_subtype = ?,";
        $sql .= " urlf5_subtypeid = ?, urlf5_file = ?, urlf5_url = ?, urlf5_prontus = ?";

        $this->{'xml_content'}{'_plt'} =~ /\.(\w+)$/;
        my $ext = $1;
        my $artic_path = "$prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/";
        $artic_path .= $this->{fechac} . $prontus_varglb::DIR_PAG . "/$this->{ts}.$ext";
        if ($this->{'xml_content'}{'_url'} eq '') {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_update_error', $this->{ts}) . "\n";
            lib_stdlog::print_err($Artic::ERR);
            return 0;
        }

        my @params = ('artic', 'artic', $this->{ts}, $artic_path, $this->{'xml_content'}{'_url'}, $this->{prontus_id});
        my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_insert_error', $this->{ts}) . "\n";
            lib_stdlog::print_err("$Artic::ERR [$sql][$!][".$base->errstr."]\n");
            return 0;
        }
    }
    return 1;
}

# genera todo lo que necesita la friendly v5 para funcionar
# guarda el titular formateado en la BD
# genera el symlink en el filesystem
# si no esta activada friendly 5 no se hace nada
sub genera_friendly_v5 {
    my ($this, $base) = @_;

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_ENABLE'} ne '5') {
        return 1;
    }

    if ($this->{is_preview} == 1) {
        return 1;
    }

    $this->get_xml_content();

    if (exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_EXCLUDE_FID'}{$this->{'xml_content'}->{'_fid'}}) {
        return 1;
    }

    $this->{'xml_content'}{'_plt'} =~ /\.(\w+)$/;
    my $ext = $1;
    my $file = $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_ARTIC . '/' . $this->{fechac} . $prontus_varglb::DIR_PAG . '/' . $this->{ts} . ".$ext";

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
        my @vistas = (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}});
        lib_friendly::init($prontus_varglb::DIR_SERVER, $prontus_varglb::DIR_FRIENDLY5, \@vistas);
    }

    my ($friendlyAntigua, $artTs) = ('', '');
    if ($base) {
        # se busca la url antigua de este articulo, si existe, para borrarla antes de generar una nuevo
        # nunca es vacio si esta asociado un ts a una url, ya que no se permiten titulares vacios
        ($friendlyAntigua, $artTs) = lib_urlf5::get_url_art($base, $this->{ts}, $this->{prontus_id});
        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_select_error', $this->{ts}) . "\n";
            lib_stdlog::print_err("$Artic::ERR [$!][".$base->errstr."]\n");
            return 0;
        }

        # La url fue borrada, se borra si existe
        if ($this->{'xml_content'}->{'_url'} eq '') {
            if ($friendlyAntigua ne '') {
                if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
                    lib_friendly::deleteFriendly($friendlyAntigua);
                }
                my %options = (
                    'type'          => 'artic',
                    'subtype_id'    => $this->{ts},
                    'file'          => $file,
                    'url'           => $friendlyAntigua,
                    'prontus'       => $this->{prontus_id}
                );

                lib_urlf5::borrar_url($base, \%options);
                lib_prontus::send_signal('reset_url_cache');
            }
            return 1;
        }

        my ($ts, $prontus) = $this->verifica_colision_url_v5($base);
        if ($ts) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_conflict_error', $this->{ts}, "[$ts en $prontus]") . "\n";
            return 0;
        }

        # si hay friendly asociada al ts, se debe eliminar
        if ($friendlyAntigua ne '' && $prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
            lib_friendly::deleteFriendly($friendlyAntigua);
        }

        # actualizamos la DB con este articulo
        # si el art_id es vacio, es articulo nuevo
        my %options = (
            'type'          => 'artic',
            'subtype'       => 'artic',
            'subtype_id'    => $this->{ts},
            'file'          => $file,
            'url'           => $this->{'xml_content'}->{'_url'},
            'prontus'       => $this->{prontus_id}
        );
        if ($artTs eq '') {
            lib_urlf5::insert_url($base, \%options);
        } else {
            # si hay TS se debe actualizar el registro
            lib_urlf5::actualizar_url($base, \%options);
            lib_prontus::send_signal('reset_url_cache');
        }
    }

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
        my $published = $this->check_publish_art();
        if ($published && $this->{'xml_content'}->{'_url'} ne '') {
            my ($res, $data, $msg) = lib_friendly::insertFriendly($this->{'xml_content'}->{'_url'}, $file, 200);
            if ($res == 0) {
                $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_create_error', $res, lib_friendly::get_message($msg), $this->{ts}) . "\n";
                lib_stdlog::print_err($Artic::ERR);
                return 0;
            }
        }
    }
    return 1;
}

# Revisa el titular a usar para url friendly v5 para evitar colisiones con otros existentes.
sub verifica_colision_url_v5 {
    my ($this, $base) = @_;
    # revisamos si existe otro articulo con el mismo titular (las urls son unicas)
    my $data = lib_urlf5::get_url_data($base, $this->{'xml_content'}->{'_url'});
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_urlf5_check_collision_error', $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR [$!][".$base->errstr."]");
        return (0, '');
    }

    my ($artTs, $prontus) = ($data->{'urlf5_subtypeid'}, $data->{'urlf5_prontus'});

    # si existe un articulo con la misma url que no es este
    # no se continua el proceso y se devuelve el ts encontrado
    if (defined($artTs) && $artTs ne '' && $artTs ne $this->{ts}) {
        return ($artTs, $prontus);
    }
    return (0, '');
}

# verifica si el articulo es visible:
# si es una preview, es visible. De otro modo,
# alta = 1
# fecha de publicacion es anterior a la fecha actual
# fecha de expiracion es posterior a la fecha actual
# si esta marcado "solo despublicar de portadas" el articulo es visible
sub check_publish_art {
    my $this = shift;

    if ($this->{is_preview} == 1)  {
        return 1;
    }

    $this->get_xml_content();
    if ($this->{'xml_content'}{'_alta'} ne '1') {
        return 0;
    }

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'CONTROL_FECHA'} eq 'SI') {
        my $ts_now = glib_hrfec_03::get_dtime_pack4();
        my $fechahorap = $this->{'xml_content'}{'_fechap'} . $this->{'xml_content'}{'_horap'} . '00';
        $fechahorap =~ s/://g;
        my $fechahorae = $this->{'xml_content'}{'_fechae'} . $this->{'xml_content'}{'_horae'} . '00';
        $fechahorae =~ s/://g;
        # fecha actual es mayor o = que fecha de publicación
        if ($ts_now >= $fechahorap) {
            if ($ts_now <= $fechahorae || $this->{'xml_content'}{'_soloportadas'}) {
                return 1;
            } else {
                # articulo expirado, no hay vistas
                return 0;
            }
        } elsif ($this->{'xml_content'}{'_publicarvistas'}) {
            # vistas se crean aunque no se cumpla la fecha de publicación
            return 1;
        } else {
            # articulo aun no se debe publicar
            return 0;
        }
    }
    return 1;
}

# Invocada al eliminar vistas de artículos para el cron fechas.
# Elimina todos los archivos de vista.
# Elimina archivos de friendly URL.
# No elimina XML ni archivos asociados.
sub borra_pags_artic {
    my ($this, $base, $no_run_regen, $list_procs) = @_;

    # si el articulo esta publicado, no se eliminan las paginas
    if ($this->check_publish_art()) {
        return '';
    }

    my $ts        = $this->{ts};
    my $dir_fecha = $this->{fechac};

    my $dirpag      = $this->{dst_pags};
    my $dirpagpar   = $this->{dst_pagspar};

    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);

    # Cargar datos del xml. Necesarios para poder armar friendly url.
    my %campos_xml = $this->get_xml_content();

    # Borra paginas generadas
    my @files2delete = glob("$dirpag/$ts" . '.*');
    foreach my $file2delete (@files2delete) {
        unlink $file2delete;

        if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
            lib_prontus::purge_cache($file2delete);
        }
    }

    if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
        # Arma friendly url para hacer el purge
        my %artic_data = (
            'fid' => $campos_xml{'_fid'},
            'url' => $campos_xml{'_url'},
            'ts' => $this->{ts},
            'prontus_id'  => $this->{prontus_id},
            'mv' => '',
            'ext' => lib_utils::get_file_extension($campos_xml{'_plt'})
        );

        my $fileurl = lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);

        lib_prontus::purge_cache($fileurl);
    }

    # Borrar paginas paralelas
    @files2delete = glob("$dirpagpar/$ts*" . '.*');
    foreach my $file2delete (@files2delete) {
        unlink $file2delete;
        if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
            lib_prontus::purge_cache($file2delete);
        }
    }

    # Borra paginas de multivistas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}}) {
        my $dir_art_mv = $dirpag;
        $dir_art_mv =~ s/(\d{8})\/pags/$1\/pags-$mv/;
        my @files2delete_mv = glob("$dir_art_mv/$ts" . '.*');
        foreach my $file2delete (@files2delete_mv) {
            unlink $file2delete;
            if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
                lib_prontus::purge_cache($file2delete);
            }
        }

        if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
            my %artic_data = (
                'fid' => $campos_xml{'_fid'},
                'url' => $campos_xml{'_url'},
                'ts' => $this->{ts},
                'prontus_id'  => $this->{prontus_id},
                'mv' => $mv,
                'ext' => lib_utils::get_file_extension($campos_xml{'_plt'})
            );

            my $fileurl = lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);
            lib_prontus::purge_cache($fileurl);
        }


        # Paginas paralelas
        $dir_art_mv = $dirpagpar;
        $dir_art_mv =~ s/(\d{8})\/pagspar/$1\/pagspar-$mv/;
        @files2delete_mv = glob("$dir_art_mv/$ts*" . '.*');
        foreach my $file2delete (@files2delete_mv) {
            unlink $file2delete;
            if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}) {
                lib_prontus::purge_cache($file2delete);
            }
        }
    }

    # Borra relacionados manual tax y tag
    lib_parser::borrar_relacionados($dirpag, $ts);

    # Obtiene taxonomia antes de eliminar
    my %hashtemp;
    my @taxtodelete;

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'} > 0) {
        for (my $i = 1; $i <= $prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'}; $i++) {
            my ($secc, $tem, $stem) = lib_tax::get_taxonomia($ts, $base, $i);
            my $if_last_in_tax = lib_tax::get_tax_if_last_art($base, $secc, $tem, $stem);

            lib_stdlog::print_err("[$secc/$tem/$stem] if_last_in_tax[$if_last_in_tax]\n");
            if ($secc && $if_last_in_tax) {
                my $t = $tem;
                my $st = $stem;
                $t = "" if (!$tem);
                $st = "" if (!$stem);

                push @taxtodelete, "_$secc\_$t\_$st\_";
            }

            $hashtemp{$i} = "$secc/$tem/$stem" if ($secc > 0);    # se evita para generar relacionados sin taxonomia.
        }
    }

    foreach my $todelete (@taxtodelete) {
        my $path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
        my @files = `find $path -name "*$todelete*"`;
        lib_stdlog::print_err("files todelete path[$path -name \"*$todelete*\"]\n");

        foreach my $filetodelete (@files) {
            $filetodelete =~ s/\n//sg;
            lib_stdlog::print_err("filetodelete: $filetodelete\n");
            unlink $filetodelete;
        }
    }

    # Regenera relacionados
    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'} > 0) {
        for (my $i = 1 ; $i <= $prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'} ; $i++) {
            if (defined $hashtemp{$i}) {
                my ($secc, $tem, $stem) = split /\//, $hashtemp{$i};
                lib_tax::generar_relacionados($secc, $tem, $stem, $base);
                # Ahora parsea articulos relacionados para MVs
                foreach my $mv (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}}) {
                    lib_stdlog::print_err("generar_relacionados [$secc, $tem, $stem]\n");
                    lib_tax::generar_relacionados($secc, $tem, $stem, $base, $mv);
                }
            }
        }
    }

    # regenera tagports y taxports
    my $fid = $campos_xml{'_fid'};

    # TODO: agrupar los datos de tags entre todos los artículos procesados para regenerar una sola vez.
    my $tags_data = $campos_xml{'_tags'};

    if ($tags_data) {
        # Regenerar portadas tagonomicas.
        my $param_especif_tagonomicas = $tags_data;
        $param_especif_tagonomicas =~ s/,/\//sg;
        my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_tags_ports.pl $this->{prontus_id} $param_especif_tagonomicas $fid >/dev/null 2>&1 &";
        lib_stdlog::print_err("[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
        system $cmd;
    }

    # Borra cache listas de articulo
    lib_ports::borrar_json_artic_pubs($this->{prontus_id});

    # regenera taxports
    my $cmd;
    my $notax = 1;
    for (my $i = 1; $i <= $prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'}; $i++) {
        last if (scalar @taxtodelete > 0);

        my $taxonomia = $hashtemp{$i};
        my ($secc, $tem, $stem) = split /\//, $taxonomia;

        # si el articulo no tiene taxonomia no hay que regenerar taxports
        next if ($secc < 1);

        $notax = 0;
        my $param_especif = $fid . '/' . $secc . '/' . $tem . '/' . $stem;

        if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXPORT_DINAMICAS'} eq 'NO') {
            $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_cron_taxport.pl $this->{prontus_id} $param_especif >/dev/null 2>&1 &";

            if ($no_run_regen) {
                $$list_procs{"prontus_cron_taxport.pl $this->{prontus_id} $param_especif"} = $cmd;
            } else {
                lib_stdlog::print_err("[" . &glib_hrfec_03::get_dtime_pack4() . "] $cmd\n");
                system $cmd;
            }
        } else {
            lib_tax::borrar_taxport($param_especif);
        }

        # Regenera las salidas List
        $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_cron_list.pl $this->{prontus_id} $param_especif >/dev/null 2>&1 &";

        if ($no_run_regen) {
            $$list_procs{"prontus_cron_list.pl $this->{prontus_id} $param_especif"} = $cmd;
        } else {
            lib_stdlog::print_err("[" . &glib_hrfec_03::get_dtime_pack4() . "] $cmd\n");
            system $cmd;
        }
    }

    # si el articulo no tiene taxonomia debemos regenerar las listas
    if ($notax) {
        # Regenera las salidas List
        $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_cron_list.pl $this->{prontus_id} $fid/// >/dev/null 2>&1 &";
        if ($no_run_regen) {
            $$list_procs{"prontus_cron_list.pl $this->{prontus_id} $fid///"} = $cmd;
        } else {
            lib_stdlog::print_err("[" . &glib_hrfec_03::get_dtime_pack4() . "]$cmd\n");
            system $cmd;
        }
    }

    return '';
}

# Elimina un artículo con todos sus datos asociados.
sub borra_artic {
    my ($this, $base, $no_run_regen, $list_procs) = @_;
    my $ts        = $this->{ts};

    # borramos multitags asociados al articulo
    # se borra el articulo de las tablas multitag
    my @tipos = ('s', 't', 'st');
    my @params = ($this->{ts});
    foreach my $type (@tipos) {
        # Primero elimina los tags del artic actual:
        my $multitag_art_table = lib_utils::get_table("multitag_art_$type");
        my $sql = "delete from $multitag_art_table where multitag_art_$type\_art_id = ?";
        my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_multitags_delete_error', $this->{ts}) . "\n";
            lib_stdlog::print_err("$Artic::ERR [$sql][$!][".$base->errstr."]\n");
            return $Artic::ERR;
        }
    }

    # Desasocia tags al artic
    my $tagsart_table = lib_utils::get_table('tagsart');
    my $sql_deltag = "delete from $tagsart_table where tagsart_idart = ?";
    glib_dbi_03::ejecutar_sql_param($base, $sql_deltag, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_tags_delete_error', $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR [$sql_deltag][$!][".$base->errstr."]\n");
        return $Artic::ERR;
    }

    # borra articulo de tablas art_publicar y art_despublicar
    my $art_pub_table = lib_utils::get_table('art_publicar');
    my $sql_delpub = "delete from $art_pub_table where art_id = ?";
    glib_dbi_03::ejecutar_sql_param($base, $sql_deltag, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_art_publish_update_error', $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR [$sql_deltag][$!][".$base->errstr."]\n");
        return $Artic::ERR;
    }
    my $art_delpub_table = lib_utils::get_table('art_despublicar');
    $sql_delpub = "delete from $art_delpub_table where art_id = ?";
    glib_dbi_03::ejecutar_sql_param($base, $sql_deltag, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_art_un_publish_update_error', $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR [$sql_deltag][$!][".$base->errstr."]\n");
        return $Artic::ERR;
    }

    # Cargar datos del xml para poder modificarlos en memoria antes de realizar el borrado de paginas
    $this->get_xml_content();
    # marca articulo sin alta para poder borrar las paginas asociadas
    $this->{'xml_content'}{'_alta'} = '0';

    $this->borra_pags_artic($base, $no_run_regen, $list_procs);

    # Elimina el archivo de la Base de Datos
    my $resp = $this->art_delete_bd($base);
    if (!$resp) {
        return $Artic::ERR;
    }

    # borra url f5 del articulo
    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_ENABLE'} eq '5') {
        if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_USE_SYMLINK'} eq 'SI') {
            my @vistas = (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}});
            lib_friendly::init($prontus_varglb::DIR_SERVER, $prontus_varglb::DIR_FRIENDLY5, \@vistas);

            lib_friendly::deleteFriendly($this->{'xml_content'}{'_url'});
        }

        # lib_stdlog::print_err("$this->{'xml_content'}{'_url'} -> res[$res] msg[$msg]\n");
        my %options = (
            'type'          => 'artic',
            'subtype'       => 'artic',
            'subtype_id'    => $this->{ts},
            'prontus'       => $this->{prontus_id}
        );

        lib_urlf5::borrar_url($base, \%options);
    }

    unlink ($this->{dst_xml} . "/$ts.xml");    # Borra xml
    unlink ($this->{dst_json} . "/$ts.json");    # Borra JSON

    # Borra imagenes
    $this->borrar_artic_files($this->{dst_foto});

    # Borra multimedia.
    $this->borrar_artic_files($this->{dst_multimedia});

    # borra thumbails de multimedia y asociados
    $this->borrar_artic_files($this->{dst_mm_thumbs});

    # Borra swf
    $this->borrar_artic_files($this->{dst_swf});

    # Borra archivos de MaM
    glib_fildir_03::borra_dir($this->{dst_mam});

    # Borra archivos asociados/documentos
    glib_fildir_03::borra_dir($this->{dst_asocfile});

    # Borra cache listas de articulo
    lib_ports::borrar_json_artic_pubs($this->{prontus_id});

    # Regenera la tabla del DAM
    my $pathnice = lib_utils::get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);

    my $cmd = "$pathnice $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/dam/prontus_dam_ppart_delete.pl $ts $this->{prontus_id} $prontus_varglb::CONFIG{$this->{prontus_id}}{'PUBLIC_SERVER_NAME'}  >/dev/null 2>&1 &";
    system $cmd;

    return '';
}


# Elimina los archivos y directorios asociados a un artículo dentro del directorio especificado.
sub borrar_artic_files {
    my ($this, $dir_aux) = @_;

    my $ts        = $this->{ts};
    my $dir_fecha = $this->{fechac};

    my @files2delete = glob($dir_aux . '/*' . $ts . '*');
    foreach my $entry (@files2delete) {
        if (-d $entry) {
            glib_fildir_03::borra_dir($entry);
        } else {
            unlink $entry;
        }
    }
}

# Borra la fila de un artículo en la tabla ART de la base de datos.
sub art_delete_bd {
    my ($this, $base) = @_;
    # Elimina
    my $art_table = lib_utils::get_table('art');
    my $sql = "delete from $art_table where art_id = ?";
    # lib_stdlog::print_err("$sql\n");
    my @params = ($this->{ts});
    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_delete_error', $this->{ts}, $DBI::err, $base->errstr) . "\n";
        lib_stdlog::print_err("$Artic::ERR sql[$sql][$!][".$DBI::errstr."]");
        return 0;
    }
    return 1;
}

# Actualiza articulo en base de datos prontus a partir de info del xml
sub art_update_bd {
    my ($this, $base) = @_;

    my %campos = $this->_get_campos_art4bd();
    my $art_table = lib_utils::get_table('art');
    my $sql = "select art_id from $art_table where art_id = ?";
    my $art_id = lib_prontus::existe_registro($sql, $base, $this->{ts});

    if ($art_id) {
        $sql = "update $art_table set art_autor = ?, "
            . " art_idsecc1 = ?, art_idtemas1 = ?, art_idsubtemas1 = ?,"
            . " art_idsecc2 = ?, art_idtemas2 = ?, art_idsubtemas2 = ?,"
            . " art_idsecc3 = ?, art_idtemas3 = ?, art_idsubtemas3 = ?,"
            . " art_titu = ?, art_baja = ?, art_horap = ?, art_tipoficha = ?, art_alta = ?, art_fechaphorap = ?,"
            . " art_idusr = ?, art_fechap = ?, art_fechae = ?, art_horae = ?, art_soloportadas = ?,art_fechaehorae = ?,"
            . " art_fecha_inicio = ?, art_fecha_fin = ?, art_modtime = ?, art_s3 = ?,"
            . " art_s3_size = ?, art_s3_media_size = ?, art_s3_asset_size = ?,"
            . " art_disk_size = ?, art_media_size = ?, art_asset_size = ?, art_imag_size = ?, art_total_size = ?, "
            . " art_indexar = ?, art_priorizar = ? "
            .  " where art_id = ? ";

        my @params = ($campos{'_autor'},
            $campos{'_seccion1'}, $campos{'_tema1'}, $campos{'_subtema1'},
            $campos{'_seccion2'}, $campos{'_tema2'}, $campos{'_subtema2'},
            $campos{'_seccion3'}, $campos{'_tema3'}, $campos{'_subtema3'},
            $campos{'_txt_titular'}, $campos{'_txt_bajada'}, $campos{'_horap'}, $campos{'_fid'}, $campos{'_alta'}, $campos{'_fechap_horap'},
            $campos{'_users_id'}, $campos{'_fechap'}, $campos{'_fechae'}, $campos{'_horae'},
            $campos{'_soloportadas'}, $campos{'_fechae_horae'}, $campos{'_fecha_ini'},
            $campos{'_fecha_fin'}, $campos{'_modtime'}, $campos{'_s3'},
            $campos{'_s3_size'}, $campos{'_s3_media_size'}, $campos{'_s3_asset_size'},
            $campos{'_disk_size'}, $campos{'_media_size'}, $campos{'_asset_size'}, $campos{'_imag_size'}, $campos{'_total_size'},
            $campos{'_indexar'}, $campos{'_priorizar'}, $this->{ts});
        my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
        # lib_stdlog::print_err("$sql\n");
        if ($base->errstr) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_art_update_error', $this->{ts}, $DBI::err) . "\n";
            lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
            return 0;
        }
    } else {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_art_id_error', $this->{ts}) . "\n";
        return 0;
    }

    return $art_id;
}

# Crea articulo en base de datos prontus a partir de info del xml
sub art_insert_bd {
    my ($this, $base) = @_;
    # basicamente apara regeneracion de tabla de arts.

    my %campos4bd = $this->_get_campos_art4bd();
    my @params = ($this->{ts}, $campos4bd{'_seccion1'}, $campos4bd{'_tema1'}, $campos4bd{'_subtema1'},
                  $campos4bd{'_seccion2'}, $campos4bd{'_tema2'}, $campos4bd{'_subtema2'},
                  $campos4bd{'_seccion3'}, $campos4bd{'_tema3'}, $campos4bd{'_subtema3'},
                  $campos4bd{'_autor'}, $campos4bd{'_txt_titular'}, $campos4bd{'_txt_bajada'},
                  $campos4bd{'_fechap'}, $campos4bd{'_horap'}, $campos4bd{'_fechap_horap'}, $this->{fechac}, $campos4bd{'_extension'},
                  $campos4bd{'_fid'}, $campos4bd{'_alta'}, $campos4bd{'_users_id'}, $campos4bd{'_fechae'},
                  $campos4bd{'_horae'}, $campos4bd{'_soloportadas'}, $campos4bd{'_fechae_horae'},
                  $campos4bd{'_fecha_ini'}, $campos4bd{'_fecha_fin'}, $campos4bd{'_modtime'},
                  $campos4bd{'_s3'}, $campos4bd{'_s3_size'}, $campos4bd{'_s3_media_size'}, $campos4bd{'_s3_asset_size'},
                  $campos4bd{'_disk_size'}, $campos4bd{'_media_size'}, $campos4bd{'_asset_size'}, $campos4bd{'_imag_size'},
                  $campos4bd{'_total_size'}, $campos4bd{'_indexar'}, $campos4bd{'_priorizar'});

    my $art_table = lib_utils::get_table('art');
    my $sql = "insert into $art_table (art_id,"
        . " art_idsecc1, art_idtemas1, art_idsubtemas1,"
        . " art_idsecc2, art_idtemas2, art_idsubtemas2,"
        . " art_idsecc3, art_idtemas3, art_idsubtemas3,"
        . " art_autor, art_titu, art_baja, art_fechap, art_horap,"
        . " art_fechaphorap, art_dirfecha, art_extension, art_tipoficha,"
        . " art_alta, art_idusr, art_fechae, art_horae, art_soloportadas,"
        . " art_fechaehorae, art_fecha_inicio, art_fecha_fin, art_modtime,"
        . " art_s3, art_s3_size, art_s3_media_size, art_s3_asset_size,"
        . " art_disk_size, art_media_size, art_asset_size, "
        . " art_imag_size, art_total_size, art_indexar, art_priorizar)"
        . " values(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

    # es articulo nuevo
    glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_art_insert_error', 2, $this->{ts}, $DBI::err) . "\n";
        lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]  $this->{fullpath_xml} [$DBI::errstr]");
        return 0;
    }

    return $this->{ts};
}

# Genera vista html (o lo que corresponda a la plantilla) del articulo y graba el archivo.
# Toma como entrada los datos del XML, no los que vengan en el hash $this->{campos}.
# @param self
# @param string $mv la multivista si corresponde
# @param string $plt la plantilla a usar
# @param bool $is_paralela si es vista paralela
# @param bool $only_pproc si solamente debe ejecutar postprocesos
#
sub generar_vista_art {
    my ($this, $mv, $plt, $is_paralela, $only_pproc) = @_;

    # Carga campos
    my %campos_xml = $this->get_xml_content();

    if ($mv eq '' && !defined($plt) && !defined($is_paralela) && !defined($only_pproc)
        && $prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_URLS_ENABLE'} eq '5') {
        my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
        if (!ref($bd)) {
            lib_stdlog::print_err($msg_err_bd, "\n");
        }
        # generamos el link friendly v5
        $this->genera_friendly_v5($bd) || return $Artic::ERR;
    }

    $plt = $campos_xml{'_plt'} if (!$plt);
    # Path completo al articulo a generar
    my $fullpath_vista = $this->get_fullpath_artic($plt, $mv, $is_paralela);

    # si el articulo no esta publicado, no se generan vistas, se borran si existen
    if (!defined($only_pproc) && !$this->check_publish_art()) {
        unlink($fullpath_vista) if (-f $fullpath_vista);

        # Solo se entra aqui si el articulo que se esta parseando no es paralelo, ya que se genería un loop infinito.
        if (!$is_paralela) {
            # Plantillas paralelas del fid.
            foreach my $plt_paralela (@{$prontus_varglb::CONFIG{$this->{prontus_id}}{'FORM_PLTS'}{$campos_xml{'_fid'}}{'plt_paralela'}}) {
                $this->generar_vista_art($mv, $plt_paralela, 1);
            }
        }

        # retorna 1 porque no es un error
        return 1;
    }

    # Escribe JSON con la información del artículo.
    if (!$mv && !$is_paralela) {
        $this->write_json();
    }

    # si el artículo no debe generar vista, terminamos acá.
    return 1 if (!lib_helper::can_have_view($campos_xml{'_fid'}));

    # Carga plantilla
    my $fullpath_plt       = "$this->{pathdir_plt_pags}/$plt";
    my $buffer = lib_prontus::carga_buffer_plt($fullpath_plt, $this->{pathdir_plt_macros}, $mv);

    if ($buffer) {
        # Se parsean los loops de Articulo
        $buffer = lib_parser::procesa_loop_artic($buffer,'##');

        # que no estan en el xml del artic
        my %claves_adicionales;

        # Parsea datos del artic
        $buffer = $this->parse_artic_data($buffer, \%claves_adicionales, $mv);

        # prontus functions
        $buffer = lib_prontus::parser_custom_function($buffer);

        # Borra marcas no sustituidas
        $buffer =~ s/%%.+?%%//g;
        # restituyo los %% escritos por el usuario al interior de los campos
        $buffer =~ s/&#37;&#37;/%%/sg;

        if (!$mv && !$is_paralela || defined($only_pproc)) {
            # Se listan los postprocesos
            $buffer = $this->_genera_lista_post_procesos($buffer);
        }

        if (defined($only_pproc)) {
            return 1;
        }

    } else {
        my $titular = lib_prontus::saca_tags_rets($campos_xml{'_txt_titular'});
        my $vista = '';
        $vista = "<br/>Vista asociada[$mv]\n" if ($mv);
        $Artic::ERR = lib_lang::get_interpolated_message('artic_no_template_error', $this->{ts}, $titular , $campos_xml{'_plt'} , $vista) . "\n";
        lib_stdlog::print_err("$Artic::ERR fullpath_plt[$fullpath_plt]");
        return 0;
    }

    # Check dir
    my $fulldir_vista = $this->_get_fulldir_vista($mv, $is_paralela);
    if (!glib_fildir_03::check_dir($fulldir_vista)) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_make_dir_error', $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR fulldir_vista[$fulldir_vista] $!");
        return 0;
    }

    # Escribe vista del articulo
    glib_fildir_03::write_file($fullpath_vista, $buffer);

    # chequea que el archivo no haya quedado vacio.
    if (!-s $fullpath_vista) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_write_main_view_error', $this->{ts}, $plt) . "\n";
        lib_stdlog::print_err("$Artic::ERR fullpath_vista[$fullpath_vista] $!");
        return 0;
    }

    # regenera vistas de mam
    if ($campos_xml{'_mam'} && !$mv && !$is_paralela) {
        lib_mam::regeneraVistas($this);
    }

    # Para la vista ppal,
    if (!$mv) {
        # hace purge de la vista, ya sea del path normal o friendly url, dependiendo de como este configurado.
        if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$campos_xml{'_fid'}}
            && glib_hrfec_03::get_antiguedad_ts($this->{ts}) > 60) {
            my $marca_file = $this->get_rel_path_artic($plt, $mv, $is_paralela);

            my %artic_data = (
                'fid' => $campos_xml{'_fid'},
                'url' => $campos_xml{'_url'},
                'ts' => $this->{ts},
                'prontus_id'  => $this->{prontus_id},
                'mv' => '',
                'ext' => lib_utils::get_file_extension($plt)
            );

            my $fileurl = lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);

            lib_prontus::purge_cache($marca_file);
            lib_prontus::purge_cache($fileurl);
        }
    }

    # Solo se entra aqui si el articulo que se esta parseando no es paralelo, ya que se genería
    # un loop infinito.
    if (!$is_paralela) {
        # Plantillas paralelas del fid.
        foreach my $plt_paralela (@{$prontus_varglb::CONFIG{$this->{prontus_id}}{'FORM_PLTS'}{$campos_xml{'_fid'}}{'plt_paralela'}}) {
            # lib_stdlog::print_err("plt_paralela[$plt_paralela]\n");
            $this->generar_vista_art($mv, $plt_paralela, 1);
        }
    }

    return 1;
}

# parsea todas las marcas del xml mas algunas derivadas de estas en el buffer dado.
# Este metodo NO elimina las marcas que queden sin sustituir.
sub parse_artic_data {
    my ($this, $buffer, $vars_adicionales, $mv) = @_;

    my %campos_xml = $this->get_xml_content();

    # Buscamos las eventuales fixed_urls en db.
    $this->get_tax_fixed_urls($vars_adicionales);

    # Agrega algunas vars adicionales que no vienen en el XML.
    # Es posible que estas sobreescriban las del XML
    if (ref $vars_adicionales) {
        foreach my $k (keys %{$vars_adicionales}) {
            $campos_xml{lc $k} = $$vars_adicionales{$k};
        }
    } else {
        $vars_adicionales = {};
    }

    # Agrega multivista para los condicionales
    $campos_xml{'_mv'} = $mv;

    # procesar loops e if de multitag
    $buffer = $this->parsea_loop_multitag($buffer, \%campos_xml, $vars_adicionales);

    # Procesar IFs y NIFs
    $buffer = lib_parser::procesa_condicional($buffer, \%campos_xml, $vars_adicionales);

    # parsea urlf5 de taxonomia
    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        lib_stdlog::print_err($msg_err_bd, "\n");
    }

    my $aux = $buffer;

    # No incluye el número de página en el link friendly de la taxport.
    $buffer = lib_tax::parse_tax_url($buffer, \%campos_xml, $mv, 0);

    # Obtiene nom de secc, tema y subtema en vista correspondiente
    for (my $i = 1; $i <= 3; $i++) {
        ($campos_xml{"_nom_seccion$i"}, $campos_xml{"_nom_tema$i"}, $campos_xml{"_nom_subtema$i"}) = lib_prontus::get_nom4vistas($bd, $mv, $campos_xml{"_seccion$i"}, $campos_xml{"_tema$i"}, $campos_xml{"_subtema$i"});
    }

    # buscamos marcas de multimedia_video adicionales que haya que parsear
    while ($buffer =~ /%%(multimedia_video\w+)\.([^\.%]+)%%/sg) {
        my $marca_video = lc $1;
        my $property = lc $2;

        # no hay video asociado a esta marca
        next if (!defined($campos_xml{$marca_video}));

        my $marca = "$marca_video.$property";
        # si esta definido el campo continuamos, se cargan en get_xml_content o en un bucle anterior
        next if (defined($campos_xml{$marca}));

        # no hay extension, no hay video original
        next if (!defined($this->{xml_content}->{"$marca_video.ext"}));

        # suponemos que es una version del video original, verifica si existe
        my $file_version = "$marca_video$this->{ts}$property.".$this->{xml_content}->{"$marca_video.ext"};
        if (!$this->{xml_content}->{'_s3' } && -f "$this->{dst_multimedia}/$file_version") {
            $campos_xml{$marca} = "/$this->{prontus_id}/site$prontus_varglb::DIR_EXMEDIA/$this->{fechac}$prontus_varglb::DIR_MMEDIA/$file_version";
        } else {
            my $found = 0;
            if (defined($this->{media_metadata}->{$marca_video}->{'qualities'})) {
                foreach my $key ( keys %{$this->{media_metadata}->{$marca_video}->{'qualities'}}) {
                    if ($this->{media_metadata}->{$marca_video}->{'qualities'}->{$key}->{'file'} =~ /$file_version$/i) {
                        $campos_xml{$marca} = $this->{media_metadata}->{$marca_video}->{'qualities'}->{$key}->{'file'};
                        $found = 1;
                    }
                }
            }

            next if ($found);
            # si no existe, poner el video original.
            $campos_xml{$marca} = "/$this->{prontus_id}/site$prontus_varglb::DIR_EXMEDIA/$this->{fechac}$prontus_varglb::DIR_MMEDIA/$campos_xml{$marca_video}";
        }
        # agrega la clave al objeto artic para no volver a revisar el disco
        $this->{xml_content}->{$marca} = $campos_xml{$marca};
    }

    # parsea tags de articulo
    my @id_tags = split("," , $campos_xml{'_tags'});
    my @name_tags = split("," , $campos_xml{'_tagnames'});
    my $tag_count = scalar @name_tags;
    my %tagsnomhash = lib_tags::get_tags_noms_hash($bd, $campos_xml{'_tags'});
    # procesa loop de tags
    while ($aux =~ /%%_loop_tags%%(.*?)%%\/_loop_tags%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        for (my $i = 0; $i < $tag_count; $i++) {
            my $auxloop = $loop;
            if (!defined($lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]})) {
                lib_tags::get_tag_urls($bd, $id_tags[$i], $this->{'prontus_id'});
            }
            my $tag_nom = $tagsnomhash{$id_tags[$i]}{''};
            my $tag_nom_vista = $tagsnomhash{$id_tags[$i]}{$mv};

            $tag_nom_vista = $tag_nom if ($tag_nom_vista eq '');

            $auxloop =~ s/%%_tag_id%%/$id_tags[$i]/g;
            $auxloop =~ s/%%_tag_nom%%/$tag_nom/g;
            $auxloop =~ s/%%_tag_nom_vista%%/$tag_nom_vista/g;

            # Filtro: all, fid_noticia, fid_general, fid_link
            # Plantilla: tagport u otro nombre
            # %%_tag_url('<filtro>', '<plantilla>')%%
            while ($auxloop =~ /(%%_tag_url\(['"](.*?)['"],\s*['"](.*?)['"]\)%%)/g) {
                my $tag_func = $1;
                my $tag_filtro = $2;
                my $tag_plt = $3;

                my $path_tag_url = "$prontus_varglb::CONFIG{$this->{'prontus_id'}}{'DIR_SITE'}/tag/port/$tag_filtro/$tag_plt\_$id_tags[$i]_1.html";
                # lib_stdlog::print_err("tag_id [$id_tags[$i]] tag_filtro[$tag_filtro] tag_plt[$tag_plt]\n");

                # hay url f5 para este tag, plantilla y filtro
                if ($lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]}{'urlf5'} &&
                    $lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]}{'urls'}{$path_tag_url}) {
                    my $tag_url = $lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]}{'urls'}{$path_tag_url};
                    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI'
                        && $mv ne '') {
                        $auxloop =~ s/\Q$tag_func\E/$mv$tag_url/g;
                    } else {
                        $auxloop =~ s/\Q$tag_func\E/$tag_url/g;
                    }
                } elsif ($lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]}{'port'}) {
                    my $tag_url = $lib_prontus::TAGS_HASH{$this->{'prontus_id'}}{$id_tags[$i]}{'port'};
                    $auxloop =~ s/\Q$tag_func\E/$tag_url/g;
                } else {
                    $auxloop =~ s/\Q$tag_func\E/$path_tag_url/g;
                }
            }

            $totloop .=  $auxloop;
        }

        $buffer =~ s/%%_loop_tags%%\Q$loop\E%%\/_loop_tags%%/$totloop/s;
    }

    # Parsea campos
    foreach my $nom_campo (keys %campos_xml) {
        my $val_campo = $campos_xml{$nom_campo};
        next if ($val_campo eq '');
        my @temp = split(/_/, $nom_campo);
        my $tagtype = $temp[0]?$temp[0]: '_'. $temp[1];

        next if ($tagtype eq 'chk' && $temp[1] eq 'cuadrar'  && $temp[2] eq 'fotofija' );
        next if ($tagtype eq 'foto' || $tagtype eq '_nomfoto' || $tagtype eq '_wfoto' ||
            $tagtype eq '_hfoto' || $tagtype eq '_mfoto' || $tagtype eq '_ofoto' || $tagtype eq '_bfoto');

        if ($tagtype eq 'vtxt') {
            # warn "[$nom_campo][$val_campo]";
            $buffer = $this->_parsing_vtxt($buffer, $nom_campo, $val_campo);
        }
        elsif ($tagtype eq 'multimedia') {
            my $subtype = substr($temp[1], 0, 5);
            if (index($temp[1], '.') > -1 || ($subtype ne 'audio' && $subtype ne 'video')) {
                $buffer =~ s/%%$nom_campo%%/$val_campo/g;
                next;
            } else {
                # tipo compuesto (multimedia_video|multimedia_audio)
                my $type = $tagtype .'_'.$subtype;
                $buffer = $this->_parsing_mmedia($nom_campo, $buffer);
            }
        }
        elsif ( $tagtype eq 'asocfile') {
            if (index($temp[1], '.') > -1) {
                $buffer =~ s/%%$nom_campo%%/$val_campo/g;
                next;
            } else {
                $buffer = $this->_parsing_asocfile($nom_campo, $buffer);
            }
        }
        elsif ($tagtype eq 'fotofija') {
            $buffer = $this->_parsing_fotos($nom_campo, $val_campo, $buffer);
        }
        else {
            # Replace en artic, incluye minitext
            $buffer = lib_prontus::parse_tag_prontus($val_campo, $nom_campo, $buffer);
        }
    }

    # Realizar sustituciones especiales.
    $buffer = lib_prontus::parser_fecha_ext($buffer, $campos_xml{'_fechap'}, '_fechap');
    $buffer = lib_prontus::parser_fecha_ext($buffer, $campos_xml{'_fechae'}, '_fechae');

    # Replace en artic el UTC de publicacion
    my $fechap_horap_iso = $campos_xml{'_fechap'} . lib_prontus::get_hora_iso($campos_xml{'_horap'});
    my $utc_pub          = lib_prontus::get_dtime_rfc822($fechap_horap_iso);
    $buffer = lib_prontus::parse_tag_prontus($utc_pub, '_utcp', $buffer);

    # Reemplaza _ts, _fechac, _fechaclong, _fechacshrt
    $buffer = lib_prontus::replace_tsdata($buffer, $this->{ts});

    # reemplaza %%_fechamlong%% - %%_horam%%
    $buffer = lib_prontus::replace_mtime($buffer, $this->{fullpath_xml},$campos_xml{'_modtime'});

    # Parsea nombre y usuario.
    $buffer = lib_prontus::parse_userinfo($buffer, $campos_xml{'_users_id'});

    # Generar urls del articulo
    my %artic_data = (
        'fid' => $campos_xml{'_fid'},
        'url' => $campos_xml{'_url'},
        'ts' => $this->{ts},
        'prontus_id'  => $this->{prontus_id},
        'mv' => $mv,
        'ext' => lib_utils::get_file_extension($campos_xml{'_plt'})
    );

    $buffer = lib_prontus::parse_filef($buffer,\%artic_data);

    # parsear globales de prontus
    $buffer = lib_parser::parse_globals($buffer, '');

    # realiza includes de prontus %%include(file.ext)%%
    $buffer = lib_prontus::parse_includes($buffer);

    # Elimina marcas no parseadas del VTXT.
    $buffer =~ s/%%_subtit_(.*?)%%(.*?)%%\/_subtit_\1%%//sg;

    return $buffer;
}

sub parsea_loop_multitag {
    my $this       = $_[0];
    my $buffer     = $_[1];
    my %campos_xml = %{$_[2]};
    my %vars_adicionales = %{$_[3]};

    lib_prontus::carga_multitag_json();
    my $aux = $buffer;
    my @multitag_subtema = split("," , $campos_xml{'_multitag_subtema'});
    # busca todos los loops, se usa aux para no interferir con el contador interno de la regexp
    while ($aux =~ /%%_loop_multitag_subtema%%(.*?)%%\/_loop_multitag_subtema%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_subtema) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_subtema_id%%/$id/g;
            $auxloop =~ s/%%_multitag_subtema_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_subtema_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_subtema_id'} = $id;
                $data_loop{'_multitag_subtema_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'name'};
                $data_loop{'_multitag_subtema_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'friendly'};
                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        # reemplaza la primera ocurrencia, se asume que los loops no se repiten exactamente iguales
        $buffer =~ s/%%_loop_multitag_subtema%%\Q$loop\E%%\/_loop_multitag_subtema%%/$totloop/s;
    }

    $aux = $buffer;
    my @multitag_tema = split("," , $campos_xml{'_multitag_tema'});
    while ($aux =~ /%%_loop_multitag_tema%%(.*?)%%\/_loop_multitag_tema%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_tema) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_tema_id%%/$id/g;
            $auxloop =~ s/%%_multitag_tema_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_tema_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_tema_id'} = $id;
                $data_loop{'_multitag_tema_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'name'};
                $data_loop{'_multitag_tema_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'friendly'};
                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        $buffer =~ s/%%_loop_multitag_tema%%\Q$loop\E%%\/_loop_multitag_tema%%/$totloop/s;
    }

    $aux = $buffer;
    my @multitag_seccion_ids = split("," , $campos_xml{'_multitag_seccion'});
    # Se parsean los loops de multitag
    while ($aux =~ /%%_loop_multitag_seccion%%(.*?)%%\/_loop_multitag_seccion%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_seccion_ids) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_seccion_id%%/$id/g;
            $auxloop =~ s/%%_multitag_seccion_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_seccion_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_seccion_id'} = $id;
                $data_loop{'_multitag_seccion_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'name'};
                $data_loop{'_multitag_seccion_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'friendly'};

                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        $buffer =~ s/%%_loop_multitag_seccion%%\Q$loop\E%%\/_loop_multitag_seccion%%/$totloop/s;
    }

    $aux = $buffer;
    # parsea todas las multitag
    @multitag_subtema = keys  %{$lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}};
    while ($aux =~ /%%_loop_all_multitag_subtema%%(.*?)%%\/_loop_all_multitag_subtema%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_subtema) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_subtema_id%%/$id/g;
            $auxloop =~ s/%%_multitag_subtema_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_subtema_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_subtema_id'} = $id;
                $data_loop{'_multitag_subtema_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'name'};
                $data_loop{'_multitag_subtema_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'subtema'}{$id}{'friendly'};
                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        $buffer =~ s/%%_loop_all_multitag_subtema%%\Q$loop\E%%\/_loop_all_multitag_subtema%%/$totloop/s;
    }

    $aux = $buffer;
    @multitag_tema = keys %{$lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}};
    while ($aux =~ /%%_loop_all_multitag_tema%%(.*?)%%\/_loop_all_multitag_tema%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_tema) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_tema_id%%/$id/g;
            $auxloop =~ s/%%_multitag_tema_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_tema_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_tema_id'} = $id;
                $data_loop{'_multitag_tema_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'name'};
                $data_loop{'_multitag_tema_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'tema'}{$id}{'friendly'};
                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        $buffer =~ s/%%_loop_all_multitag_tema%%\Q$loop\E%%\/_loop_all_multitag_tema%%/$totloop/s;
    }

    $aux = $buffer;
    @multitag_seccion_ids = keys %{$lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}};
    while ($aux =~ /%%_loop_all_multitag_seccion%%(.*?)%%\/_loop_all_multitag_seccion%%/sg) {
        my $loop   = $1;
        my $totloop = '';

        foreach my $id (@multitag_seccion_ids) {
            my $auxloop = $loop;

            $auxloop =~ s/%%_multitag_seccion_id%%/$id/g;
            $auxloop =~ s/%%_multitag_seccion_nom%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'name'}/g;
            $auxloop =~ s/%%_multitag_seccion_friendly%%/$lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'friendly'}/g;

            # hay un if dentro del loop
            if (index($auxloop, 'if') > -1) {
                my %data_loop = %vars_adicionales;
                $data_loop{'_multitag_seccion_id'} = $id;
                $data_loop{'_multitag_seccion_nom'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'name'};
                $data_loop{'_multitag_seccion_friendly'} = $lib_prontus::MULTITAG{$this->{prontus_id}}{'seccion'}{$id}{'friendly'};

                $auxloop = lib_parser::procesa_condicional($auxloop, \%campos_xml, \%data_loop);
            }

            $totloop .=  $auxloop;
        }
        $buffer =~ s/%%_loop_all_multitag_seccion%%\Q$loop\E%%\/_loop_all_multitag_seccion%%/$totloop/s;
    }

    return $buffer;
}

# Recibe una ref a la base de datos, y un comando (pub|exp).
# hace un insert o update de los datos en la tabla art_publicar o art_despublicar, según el comando pasado.
# Si la fecha pasada es anterior a la fecha actual, intenta hacer un delete de la misma tabla.
sub guardar_pub_exp_artic {
    my ($this, $base, $comando) = @_;
    my %campos = $this->get_xml_content();
    my ($tabla, $fechahora, $colfecha, $colhora, $colfechahora, $sql, $res);
    my $ts_now = glib_hrfec_03::get_dtime_pack4();

    if ($comando eq 'pub') {
        $tabla = lib_utils::get_table('art_publicar');
        $colfechahora = 'art_fechaphorap';
        if ($campos{'_fechap'} eq '99999999') {
            return 1;
        }
        $fechahora = $campos{'_fechap'} . $campos{'_horap'} . '00';
    } elsif ($comando eq 'exp') {
        $tabla = lib_utils::get_table('art_despublicar');
        $colfechahora = 'art_fechaehorae';
        if ($campos{'_fechae'} eq '99999999') {
            return 1;
        }
        $fechahora = $campos{'_fechae'} . $campos{'_horae'} . '00';
    }

    $fechahora =~ s/://g;

    my @params;
    if ($fechahora > $ts_now) {
        # Intenta insertar. Si los datos ya existían en la db, se actualizan.
        $sql = "insert into $tabla (art_id, $colfechahora) values (?, ?) on duplicate key update $colfechahora = ?";
        @params = ($this->{ts}, $fechahora, $fechahora);
    } else {
        # si la fecha pasada es igual o anterior a la fecha actual, borro los eventuales registros que hayan en db.
        $sql = "delete from $tabla where art_id = ?";
        @params = ($this->{ts});
    }

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_table_update_error', $tabla, $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
        return 0;
    }

    return 1;
}

# Recibe una ref a la base de datos y un comando (pub|exp).
# borra todos los registros en la tabla art_publicar o art_despublicar con fecha anterior a la actual, según el comando pasado.
sub borrar_pub_exp_artic {
    my ($this, $base, $comando) = @_;
    my ($tabla, $colfechahora);
    my $ts_now = glib_hrfec_03::get_dtime_pack4();

    if ($comando eq 'pub') {
        $tabla = lib_utils::get_table('art_publicar');
        $colfechahora = 'art_fechaphorap';
    } elsif ($comando eq 'exp') {
        $tabla = lib_utils::get_table('art_despublicar');
        $colfechahora = 'art_fechaehorae';
    }

    my $sql = "delete from $tabla where $colfechahora < ?";
    my @params =($ts_now);

    my $salida = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    if ($base->errstr) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_table_delete_error', $tabla, $this->{ts}) . "\n";
        lib_stdlog::print_err("$Artic::ERR sql[$sql][$!]");
        return 0;
    }

    return 1;
}

# Recibe referencias a los arrays claves_adicionales y campos_xml.
# Modifica el array claves_adicionales agregando las claves url_fixed_* para la tripleta 1.
sub get_tax_fixed_urls {
    my ($this, $ref_claves_adicionales) = @_;

    my $nivel = 1;
    $this->get_xml_content();

    while ($nivel <= $prontus_varglb::CONFIG{$this->{prontus_id}}{'TAXONOMIA_NIVELES'}) {
        my ($url_seccion, $url_tema, $url_subtema)= lib_tax::get_fixed_urls($this->{'xml_content'}->{"_seccion$nivel"},
                                $this->{'xml_content'}->{"_tema$nivel"}, $this->{'xml_content'}->{"_subtema$nivel"});
        $ref_claves_adicionales->{"_url_fixed_seccion$nivel"} = "$url_seccion";
        $ref_claves_adicionales->{"_url_fixed_tema$nivel"} = "$url_tema";
        $ref_claves_adicionales->{"_url_fixed_subtema$nivel"} = "$url_subtema";
        $nivel++;
    }
}

# METODOS PRIVADOS
# Lee el path en dst_pags y retorna el mismo path pero adaptado a la vista pasada.
sub _get_fulldir_vista {
    my ($this, $mv, $is_paralela) = @_;

    my $dir = $this->{dst_pags};
    if ($is_paralela) {
        $dir = $this->{dst_pagspar};
    }
    if ($mv) {
        return $dir . "-$mv";
    } else {
        return $dir;
    }
}

# obtiene el path del articulo desde el document root
sub _get_path_vista {
    my ($this, $mv, $is_paralela) = @_;

    my $dir = $this->{path_pags};
    if ($is_paralela) {
        $dir = $this->{path_pagspar};
    }
    if ($mv) {
        return $dir . "-$mv";
    } else {
        return $dir;
    }
}

# Configura directorios de trabajo del articulo.
sub _set_dirs {
    my $this = $_[0];

    $this->{dst_base}       = $this->{document_root} . $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_ARTIC . "/$this->{fechac}";
    $this->{dst_base_mm}    = $this->{document_root} . $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_EXMEDIA . "/$this->{fechac}";
    $this->{dst_base_asoc}  = $this->{document_root} . $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_ASOCFILE ."/$this->{fechac}";
    $this->{dst_bkp}        = $this->{document_root} . $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_BKP ."/$this->{fechac}";

    $this->{path_base}       = $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_ARTIC . "/$this->{fechac}";
    $this->{path_multimedia} = $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_EXMEDIA . "/$this->{fechac}$prontus_varglb::DIR_MMEDIA";
    $this->{path_mm_thumbs}  = $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_EXMEDIA . "/$this->{fechac}$prontus_varglb::DIR_THUMBS";
    $this->{path_pagspar}   = "$this->{path_base}/pagspar";
    $this->{path_pags}      = "$this->{path_base}/pags";
    $this->{path_mam}       = "$this->{path_base}$prontus_varglb::DIR_MAM/$this->{ts}";
    $this->{path_asocfile}  =  $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'} . $prontus_varglb::DIR_ASOCFILE ."/$this->{fechac}";

    $this->{dst_preview}    = $this->{document_root} . $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'}. "/preview";
    $this->{path_preview}   = $prontus_varglb::CONFIG{$this->{prontus_id}}{'DIR_SITE'}. "/preview";
    $this->{dst_xml}        = "$this->{dst_base}/xml";
    $this->{dst_json}       = "$this->{dst_base}/json";
    $this->{dst_pags}       = "$this->{dst_base}/pags";
    $this->{dst_pagspar}    = "$this->{dst_base}/pagspar";
    $this->{dst_asocfile}   = "$this->{dst_base_asoc}/$this->{ts}";
    $this->{dst_foto}       = "$this->{dst_base}/imag";
    $this->{dst_swf}        = "$this->{dst_base}/swf";
    $this->{dst_multimedia} = "$this->{dst_base_mm}$prontus_varglb::DIR_MMEDIA";
    $this->{dst_mm_thumbs}  = "$this->{dst_base_mm}$prontus_varglb::DIR_THUMBS";
    $this->{dst_mam}        = "$this->{dst_base}$prontus_varglb::DIR_MAM/$this->{ts}";

    $this->{pathdir_plt}        = $this->{document_root} . "/$this->{prontus_id}" . "/plantillas/artic/fecha";
    $this->{pathdir_plt_macros} = "$this->{pathdir_plt}/macros";
    $this->{pathdir_plt_pags}   = "$this->{pathdir_plt}/pags";

    if (!$this->{no_check_dirs}) {
        if (!$this->_check_artic_dirs()) {
            $Artic::ERR = lib_lang::get_interpolated_message('artic_make_dirs_error', $Artic::ERR) . "\n";
            lib_stdlog::print_err("$Artic::ERR [$!]\n");
            return 0;
        }
    }

    return 1;
}

# Configura los directorios de trabajo del articulo preview.
sub _set_dirs_preview {
    my $this = shift;

    # Directorio principal del preview.
    glib_fildir_03::check_dir($this->{dst_preview});

    # Misma estructura que el artículo normal.
    $this->{dst_xml}        = "$this->{dst_preview}/$this->{fechac}/xml";
    $this->{dst_pags}       = "$this->{dst_preview}/$this->{fechac}/pags";
    $this->{dst_pagspar}    = "$this->{dst_preview}/$this->{fechac}/pagspar";
    $this->{dst_asocfile}   = "$this->{dst_preview}/$this->{fechac}/docs/$this->{ts}";
    $this->{dst_foto}       = "$this->{dst_preview}/$this->{fechac}/imag";
    $this->{path_pags}      = "$this->{path_preview}/$this->{fechac}/pags";

    if (!$this->_check_artic_dirs()) {
        $Artic::ERR = lib_lang::get_interpolated_message('artic_make_dirs_error', $Artic::ERR) . "\n";
        lib_stdlog::print_err("$Artic::ERR [$!]\n");
        return 0;
    }

    return 1;
}

# Revisa la existencia de todos los directorios necesarios para la creación del artículo.
sub _check_artic_dirs {
    my $this = shift;
    unless(glib_fildir_03::check_dir($this->{dst_xml})) {
        $ERR = $this->{dst_xml};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_json})) {
        $ERR = $this->{dst_json};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_pags})) {
        $ERR = $this->{dst_pags};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_pagspar})) {
        $ERR = $this->{dst_pagspar};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_asocfile})) {
        $ERR = $this->{dst_asocfile};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_foto})) {
        $ERR = $this->{dst_foto};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_multimedia})) {
        $ERR = $this->{dst_multimedia};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_mm_thumbs})) {
        $ERR = $this->{dst_mm_thumbs};
        return 0;
    }
    unless(glib_fildir_03::check_dir($this->{dst_bkp})) {
        $ERR = $this->{dst_bkp};
        return 0;
    }

    return 1;
}

sub write_json {
    my $this = $_[0];

    my %campos_xml = $this->get_xml_content();

    # Agregar marca _fileurl
    $campos_xml{"_fileurl"} = $this->get_friendly_url();

    foreach my $nom_campo (keys %campos_xml) {
        my @temp = split(/_/, $nom_campo);
        my $tagtype = $temp[0]?$temp[0]: '_'. $temp[1];

        if ($tagtype eq 'vtxt') {
            my $vtxtBuffer = $campos_xml{$nom_campo};
            # Quitar de VTXT prontus_card.
            $vtxtBuffer =~ s/<prontus_card(.*?)>.*?<\/prontus_card>//sg;
            # Parsear vtxt.
            $campos_xml{$nom_campo} = $this->_parsing_vtxt("%%$nom_campo%%", $nom_campo, $vtxtBuffer);
        } elsif ($tagtype eq 'fotofija') {
            $campos_xml{$nom_campo} =~ /(foto_\d+)$this->{ts}/;
            my $fotoNom = $1;

            $campos_xml{$nom_campo.".width"} = int($campos_xml{"_w$fotoNom"});
            $campos_xml{$nom_campo.".height"} = int($campos_xml{"_h$fotoNom"});
        } elsif ($tagtype eq 'foto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_ofoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_nomfoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_bfoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_wfoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_hfoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq '_mfoto') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq  '_galeria') {
            delete $campos_xml{$nom_campo};
        } elsif ($tagtype eq 'chk' && $temp[1] eq 'cuadrar'  && $temp[2] eq 'fotofija' ) {
            delete $campos_xml{$nom_campo};
        }
    }

    delete $campos_xml{"_alta"};
    delete $campos_xml{"_asset_size"};
    delete $campos_xml{"_autor"};
    delete $campos_xml{"_disk_size"};
    delete $campos_xml{"_indexar"};
    delete $campos_xml{"_media_size"};
    delete $campos_xml{"_plt"};
    delete $campos_xml{"_priorizar"};
    delete $campos_xml{"_s3_asset_size"};
    delete $campos_xml{"_s3_media_size"};
    delete $campos_xml{"_s3_size"};
    delete $campos_xml{"_s3"};
    delete $campos_xml{"_soloportadas"};
    delete $campos_xml{"_txt_galeria_credito_total"};
    delete $campos_xml{"_txt_galeria_description_total"};
    delete $campos_xml{"_users_id"};

    glib_fildir_03::check_dir($this->{dst_json});
    glib_fildir_03::write_file_bin($this->{fullpath_json}, JSON::encode_json(\%campos_xml));
}

# Escribe a disco el XML del artículo
sub _flush_xml {
    my $this = $_[0];

    # comparamos el md5 del xml existente, vs lo que se guardara
    # para determinar si hay que hacer un respaldo
    my $old_xml = glib_fildir_03::read_file($this->{fullpath_xml});
    # xml existe y no es vacio
    if ($old_xml ne ''){
        # modtime se quita ya que cambia al guardar
        $old_xml =~ s/<_modtime>\d+<\/_modtime>//;
        utf8::encode($old_xml);
        my $old_md5 = Digest::MD5::md5_hex($old_xml);

        my $aux = $this->{xml_data};
        $aux =~ s/<_modtime>\d+<\/_modtime>//;
        utf8::encode($aux);
        my $new_md5 = Digest::MD5::md5_hex($aux);
        # si son diferentes se crea respaldo
        if ($new_md5 ne $old_md5) {
            #respaldamos el xml anterior antes de sobreescribirlo
            # se copia  y verifica que no ocurrio un error al copiar
            my $backup_path ="$this->{dst_bkp}/$this->{ts}.xml." . time;
            if (File::Copy::copy($this->{fullpath_xml}, $backup_path) == 0) {
                $Artic::ERR = lib_lang::get_interpolated_message('artic_backup_error', $this->{fullpath_xml}, $backup_path);
                lib_stdlog::print_err("$Artic::ERR [$!]");
                return 0;
            }

            # No hubo error al crear el respaldo, guarda XML nuevo
            glib_fildir_03::write_file($this->{fullpath_xml}, $this->{xml_data});
        }

        # si son iguales no se vuelve a guardar, ya que son los mismos datos
    } else {
        # xml antiguo es vacio o no existe, guarda xml nuevo
        glib_fildir_03::write_file($this->{fullpath_xml}, $this->{xml_data});
        # guardado ok

    }
    # proceso terminado ok
    return 1;
}

# Ajusta horas desde los valores de entrada antes de guardarlos en el xml
sub _artic_ajusta_horas {
    my ($this) = @_;
    my %horas;

    $horas{'_horap'} = $this->{campos}->{'_horap'};
    if (!$horas{'_horap'}) {
        $horas{'_horap'} = glib_hrfec_03::get_dtime_pack4();
        if ($horas{'_horap'} =~ /(\d{2})(\d{2})\d{2}$/) {
            $horas{'_horap'} = $1 . ':' . $2;
        }
    }
    $horas{'_horae'} = $this->{campos}->{'_horae'};

    my ($iso_horap, $iso_horae);

    foreach my $nom_campo (keys %horas) {
        my $val_campo = $horas{$nom_campo};
        $val_campo = '00:00' if ($val_campo !~ /^\d\d:\d\d$/);
        my ($hh, $mm);
        if ($val_campo =~ /^(\d\d):(\d\d)$/) {
            $hh        = $1;
            $mm        = $2;
            $hh        = '00' if ($hh eq '24');
            $hh        = '23' if ($hh > 23);
            $hh        = '59' if ($mm > 59);
            $val_campo = $hh . ':' . $mm;
        }
        $this->{campos}->{$nom_campo} = $val_campo;
    }    # foreach horas
}

# Guarda en xml y disco los recursos de un cierto tipo (FOTO, ASOCFILE, MULTIMEDIA) asociados al articulo
# type: FOTO, ASOCFILE, MULTIMEDIA
sub _guarda_recursos {
    my ($this, $type) = @_;
    my $dst_dir;
    return if ($type ne 'foto' && $type ne 'asocfile' && $type ne 'multimedia');
    $dst_dir = $this->{'dst_' . $type};
    # lib_stdlog::print_err("type[$type]\n");

    foreach my $nom_campo (keys %{$this->{campos}}) {
        my $val_campo = $this->{campos}->{$nom_campo};
        next if ($nom_campo !~ /^$type\_\w+/);

        # Ver si hay archivo existente para el recurso
        # ej: foto_1120090528115643.jpg
        my $arch_existente = $this->{campos}->{'_hidd_' . $nom_campo};
        $arch_existente = '' if (!defined($arch_existente));
        my $nom_arch = '';

        # Variables especiales solo para FOTOs
        my ($msg_size, $wfoto, $hfoto, $ofoto, $orientation, $size, $mtime);

        # Se subio un nuevo archivo
        if ($val_campo ne '') {
            $nom_arch = $this->_get_newnom_arch($type, $nom_campo, $val_campo, $arch_existente);
            next if ($nom_arch eq '');
            glib_fildir_03::check_dir($dst_dir);

            # Antes de mover lo nuevo, se elimina lo antiguo, tipo multimedia
            if ($arch_existente ne '' && $type eq 'multimedia') {
                # Obtener nombre de arch. sin extension
                my $nom_sin_ext = $nom_arch;
                $nom_sin_ext =~ s/\.\w*$//;

                my @archivos_rel = glob("$dst_dir/$nom_sin_ext*");
                foreach my $archivo_rel (@archivos_rel) {
                    if (-d $archivo_rel) {
                        glib_fildir_03::borra_dir($archivo_rel);
                    } else {
                        unlink $archivo_rel;
                    }
                }

                if ($nom_sin_ext =~ /multimedia_video/) {
                    @archivos_rel = glob("$this->{dst_mm_thumbs}/$nom_sin_ext*");
                    foreach my $archivo_rel (@archivos_rel) {
                        if (-d $archivo_rel) {
                            glib_fildir_03::borra_dir($archivo_rel);
                        } else {
                            unlink $archivo_rel;
                        }
                    }
                }
            }
            # lib_stdlog::print_err("val_campo[$val_campo] [$dst_dir/$nom_arch]\n");

            # Se mueve el nuevo archivo a la ruta final
            File::Copy::move($val_campo, "$dst_dir/$nom_arch");

            # tamaño del archivo
            $size = -s "$dst_dir/$nom_arch";

            # Si es foto, medirla
            if ($type eq 'foto') {
                my $file = "$dst_dir/$nom_arch";
                ($msg_size, $wfoto, $hfoto, $orientation) = lib_prontus::dev_tam_img($file);
                if ($msg_size) {
                    lib_stdlog::print_err("Error: $msg_size foto[$dst_dir/$nom_arch] - dim[$wfoto, $hfoto]\n");
                    next;
                }

                my $img_type = lib_thumb::get_imag_extension($file);

                if ($img_type eq 'svg') {
                    lib_thumb::clean_svg($file);
                }

                if (lib_prontus::can_edit_img($img_type)) {
                    # corrige la rotacion de la imagen de acuerdo al exif
                    if ($img_type eq 'jpg' && $orientation > 1) {
                        my $binfoto;
                        eval { $binfoto = &lib_thumb::apply_orientation_jpg($file, $orientation) };
                        if ($@) {
                            lib_stdlog::print_err("apply_orientation_jpg: ", $@ , "\n");
                        } else {
                            lib_thumb::write_image($file, $binfoto);
                        }
                    }
                    # Redimensiona la foto al limite establecido si es que lo supera.
                    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'FOTO_MAX_PIXEL'} ne '' &&
                        !lib_prontus::is_animated_image($file, $img_type)) {
                        # si no está vacio se reduce calidad de jpg, si corresponde.
                        my ($wmax, $hmax) = split("x", $prontus_varglb::CONFIG{$this->{prontus_id}}{'FOTO_MAX_PIXEL'});

                        if ($wfoto > $wmax || $hfoto > $hmax) {
                            my ($wnew, $hnew) = lib_thumb::calcular_proporcion_img($wfoto, $hfoto, $wmax, $hmax);
                            my ($binfoto, $wfoto, $hfoto);
                            eval { ($binfoto, $wfoto, $hfoto) = lib_thumb::make_resize($wnew, $hnew, $file) };
                            if ($@) {
                                lib_stdlog::print_err("make_resize: ", $@ , "\n");
                            } else {
                                # si es gif, se genera png
                                if ($file =~ /\.gif$/i) {
                                    unlink $file; # se borra la foto original.
                                    $file =~ s/\.gif$/\.png/i;
                                    $nom_arch =~ s/\.gif$/\.png/i;
                                }
                                lib_thumb::write_image($file, $binfoto) if ($binfoto);
                            }
                        }
                    } elsif ($img_type eq 'jpg' && $prontus_varglb::CONFIG{$this->{prontus_id}}{'REDUCIR_CALIDAD_JPEGS'} eq 'SI') {
                        # Si es un JPEG, y está habilitada la reducción de calidad de las imágenes, lo guardamos con la calidad especificada en prontus_varglb::NIVEL_OPTIMIZACION_JPG.
                        my ($ancho, $alto, $ratio);
                        eval { ($ancho, $alto, $ratio) = lib_thumb::get_propiedades($file) };
                        if ($@) {
                            lib_stdlog::print_err("get_propiedades: ", $@ , "\n");
                        } else {
                            my ($binfoto, $wfoto, $hfoto);
                            eval { ($binfoto, $wfoto, $hfoto) = lib_thumb::make_resize($ancho, $alto, $file) };
                            if ($@) {
                                lib_stdlog::print_err("make_resize: ", $@ , "\n");
                            } else {
                            # Se reemplaza el path por el de la foto redimensionada. Si el redimensionado fue exitoso
                                if ($binfoto) {
                                    # se borra la foto original.
                                    unlink $file;
                                    # se reemplaza con la foto "optimizada"
                                    lib_thumb::write_image($file, $binfoto);
                                }
                            }
                        }
                    }
                }
                # tamaño del archivo
                $size = -s $file;
            }

            # Si no se esta subiendo nada nuevo, ver si habia uno existente
        } else {
            if ($arch_existente ne '') {
                $nom_arch = $arch_existente;

                if ($type eq 'foto') {
                    # ancho
                    $wfoto = $this->{campos}->{'_w' . $nom_campo};
                    # alto
                    $hfoto = $this->{campos}->{'_h' . $nom_campo};
                    # si es original o no
                    $ofoto = 0;
                    if ($this->{campos}->{'_o' . $nom_campo}  =~ /^\d+$/) {
                        $ofoto = $this->{campos}->{'_o' . $nom_campo};
                    }

                    # tamaño del archivo
                    if (defined($this->{campos}->{'_b' . $nom_campo}) && $this->{campos}->{'_b' . $nom_campo} ne '') {
                        $size = $this->{campos}->{'_b' . $nom_campo};
                    } else {
                        $size = -s "$dst_dir/$nom_arch";
                    }
                    if (defined($this->{campos}->{'_m' . $nom_campo}) && $this->{campos}->{'_m' . $nom_campo} ne '') {
                        $mtime = $this->{campos}->{'_m' . $nom_campo};
                    } else {
                        $mtime = (stat("$dst_dir/$nom_arch"))[9];
                    }
                } else {
                    # tamaño del archivo
                    if (defined($this->{campos}->{"$nom_campo.bytes"}) && $this->{campos}->{"$nom_campo.bytes"} ne '') {
                        $size = $this->{campos}->{"$nom_campo.bytes"};
                    } else {
                        $size = -s "$dst_dir/$nom_arch";
                    }
                }

                if ($type eq 'multimedia' && substr($nom_campo, 0, 11) eq 'multimedia_') {
                    if (!defined($this->{campos}->{"$nom_campo.total"}) || $this->{campos}->{"$nom_campo.total"} eq '') {
                        $this->{campos}->{"$nom_campo.total"} = $size;
                    }
                }

                # se debe borrar un archivo cargado anteriormente
                if (defined($this->{campos}->{'_borr_' . $nom_campo}) && $this->{campos}->{'_borr_' . $nom_campo} eq 'S') {
                    # Obtener nombre de arch. sin extension
                    my $nom_sin_ext = $nom_arch;
                    $nom_sin_ext =~ s/\.\w*$//;

                    # Borra todos los relacionados
                    my @archivos_rel = glob("$dst_dir/$nom_sin_ext*");
                    foreach my $archivo_rel (@archivos_rel) {
                        if (-d $archivo_rel) {
                            glib_fildir_03::borra_dir($archivo_rel);
                        } else {
                            unlink $archivo_rel;
                        }
                    }

                    if ($type eq 'multimedia' &&  $nom_sin_ext =~ /multimedia_video/) {
                        @archivos_rel = glob("$this->{dst_mm_thumbs}/$nom_sin_ext*");
                        foreach my $archivo_rel (@archivos_rel) {
                            if (-d $archivo_rel) {
                                glib_fildir_03::borra_dir($archivo_rel);
                            } else {
                                unlink $archivo_rel;
                            }
                        }
                    }
                    # para no grabar la entrada en el xml
                    $nom_arch = '';
                }
            }
        }

        # Guardar nombre del archivo en el xml
        if ($nom_arch ne '') {
            if ($type eq 'foto') {
                $this->_add_foto_prontus_xml($wfoto, $hfoto, $nom_arch, $size, $ofoto, $mtime);
                if ($nom_campo =~ /^foto_posting_.*?$/) {
                    $this->_guarda_fotos_posting($nom_campo, $nom_arch);
                }
            } else {
                $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, $nom_campo, $nom_arch);
                $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "$nom_campo.bytes", $size) if ($size);
                $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "$nom_campo.total", $this->{campos}->{"$nom_campo.total"}) if (defined($this->{campos}->{"$nom_campo.total"}));
                $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "$nom_campo.s3", $this->{campos}->{"$nom_campo.s3"}) if (defined($this->{campos}->{"$nom_campo.s3"}));
            }
        }

        # Purge de archivos en cache
        # retorna si no corresponde el fid
        next if (exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$this->{campos}->{'_fid'}});

        # Para el caso de Video, revisamos el screenshot
        if ($nom_campo =~ /^multimedia_video\d+$/) {
            my @videos;
            push @videos, "$nom_campo$this->{ts}.mp4";

            # Se obtienen los formatos de video
            my %formatos = lib_prontus::get_formatos_multimedia($nom_campo);
            foreach my $formato (keys %formatos) {
                if ($formato =~ /\.(\w+)$/) {
                    my $newfile = "$nom_campo$this->{ts}" . lc($1).".mp4";
                    push @videos, $newfile;
                }
            }

            # Recorremos todas las versiones
            foreach my $file (@videos) {
                if (-f "$dst_dir/$file") {
                    # lib_stdlog::print_err("Agregando a purge [$dst_dir/$file]\n");
                    if (glib_hrfec_03::get_antiguedad_ts($this->{ts}) > 60 && glib_fildir_03::get_antiguedad_archivo("$dst_dir/$file") <= 60) {
                        lib_prontus::purge_cache("$dst_dir/$file");
                    }
                }
                my $img = $file;
                $img =~ s/\.mp4/.jpg/;
                if (-f "$dst_dir/$img") {
                    # lib_stdlog::print_err("Agregando a purge [$dst_dir/$img]\n");
                    if (glib_hrfec_03::get_antiguedad_ts($this->{ts}) > 60 && glib_fildir_03::get_antiguedad_archivo("$dst_dir/$img") <= 60) {
                        lib_prontus::purge_cache("$dst_dir/$img");
                    }
                }
            }
        } else {
            if (-f "$dst_dir/$nom_arch") {
                # lib_stdlog::print_err("Agregando a purge [$dst_dir/$nom_arch]\n");
                if ( glib_hrfec_03::get_antiguedad_ts($this->{ts}) > 60 && glib_fildir_03::get_antiguedad_archivo("$dst_dir/$nom_arch") <= 60) {
                    lib_prontus::purge_cache("$dst_dir/$nom_arch");
                }
            }
        }
    }
}

# Obtiene un nuevo nombre para el recurso
sub _get_newnom_arch {
    my ($this, $type, $nom_campo, $val_campo, $arch_existente, $nom_foto_orig) = @_;

    # Valida extensiones permitidas por prontus (seguridad)
    return '' if (!lib_prontus::check_extension($val_campo, $prontus_varglb::CONFIG{$this->{prontus_id}}{'UPLOADS_PERMITIDOS'}));

    my $nom_arch;

    if ($type eq 'multimedia') {
        my $extension_arch = lib_utils::get_file_extension($val_campo);
        $extension_arch = 'xxx' if ($extension_arch eq '');
        $extension_arch = lc($extension_arch);
        $nom_arch = "$nom_campo$this->{ts}.$extension_arch";
    }

    if ($type eq 'foto') {
        $this->_check_ext_foto($val_campo) || return '';
        $nom_arch = $this->_get_nom_foto($val_campo, $nom_foto_orig);
    }

    if ($type eq 'asocfile') {
        my $real_path = $this->{campos}->{'_real_path'}{$nom_campo};
        $nom_arch = $this->_get_nom_asocfile($real_path, $arch_existente);
    }

    return $nom_arch;
}

# Valida la extensión de un archivo de foto.
sub _check_ext_foto {
    my ($this, $path_foto) = @_;

    if ($path_foto =~ /\.(jpe?g|jpeg?|gif|png|svg|webp)$/i) {
        return 1;
    } else {
        return 0;
    }
}

# retorna nombre de archivo para asocfile_i, con extension y sin path
sub _get_nom_asocfile {
    my ($this, $texto, $af_existente) = @_;

    my $nomfile = '';
    my $ext     = '';
    # lib_stdlog::print_err("texto[$texto] - af_existente[$af_existente]\n");
    if ($texto =~ /([^\/\\]+?)(\.\w+|)$/) {
        $nomfile = lc $1;
        $ext     = lc $2;    # ext con punto
    }
    $ext = '.xxx' if ($ext eq '');

    # lib_stdlog::print_err("nomfile[$nomfile]\n");
    $nomfile =~ tr/\xe1\xe9\xed\xf3\xfa\xc1\xc9\xcd\xd3\xda\xd1\xf1\x20\xfc\xdc/aeiouaeiounn_uu/;
    $nomfile =~ s/\W/_/sg;
    # quita simbolos utf8 superiores a ascii para urls
    $nomfile =~ s/[\x{7b}-\x{ffffff}]//g;

    return '' if ($nomfile eq '');

    # elimina el existente porque ahora viene uno nuevo
    if (($af_existente ne '') && (-f "$this->{dst_asocfile}/$af_existente")) {
        unlink "$this->{dst_asocfile}/$af_existente";
    }

    if (-f "$this->{dst_asocfile}/$nomfile$ext") {
        # Si el arch. existe debo ponerle un nombre distinto,
        # salvo q ya haya un archivo para este control,
        # en cuyo caso lo reemplazo, usease, uso el mismo nombre de archivo.
        if ($af_existente eq "$nomfile$ext") {
            return "$nomfile$ext";
        }

        my $sufijo       = '1';
        my $nomfile_orig = $nomfile;
        while (-f "$this->{dst_asocfile}/$nomfile$ext") {
            $nomfile = $nomfile_orig . '_' . $sufijo;
            $sufijo++;
        }
        return "$nomfile$ext";
    }

    return "$nomfile$ext";
}

# Obtiene nombre de la siguiente foto que toca para este articulo.
# Dicho nombre comenzara con el correlativo sgte al ultimo ingresado
# Se procesa el nombre original de la foto para agregarlo al path final
sub _get_nom_foto {
    my $this = $_[0];
    my $path_foto = $_[1];
    # para mantener compatibilidad si vienen 4 parametros, en vez de 3
    my $nom_foto_orig = $_[(scalar @_) - 1];

    if ($nom_foto_orig) {
        my $ext = '';
        if ($nom_foto_orig =~ /(.*)(\.\w+)$/) {
            $nom_foto_orig = $1;
            $ext = lc $2; # ext con punto
        }
        # la extensión sigue vacía, se intenta obtener del path
        if ($ext eq '' && $path_foto =~ /(\.\w+)$/) {
            # ext con punto
            $ext = lc $1;
        }
        $nom_foto_orig = lib_encodes::no_accent_mark($nom_foto_orig);
        # Elimina entidades html.
        $nom_foto_orig =~ s/&[^;]+;//g;
        #convierte todos los caracteres que no son de palabras en espacios
        $nom_foto_orig =~ s/[^a-zA-Z0-9_\-\.]/ /g;
        # elimina espacios repetidos
        $nom_foto_orig =~ s/ {2,}/ /g;
        # elimina espacios iniciales
        $nom_foto_orig =~ s/^\s+//g;
        # elimina espacios finales
        $nom_foto_orig =~ s/\s+$//g;
        # elimina todos los guiones al principio
        $nom_foto_orig =~ s/^-+//g;
        # elimina todos los guiones al final
        $nom_foto_orig =~ s/-+$//g;

        $nom_foto_orig = lib_prontus::ajusta_nchars($nom_foto_orig, 50);
        # convierte espacios a _
        $nom_foto_orig =~ s/ /\_/g;
        if ($nom_foto_orig ne '') {
            $nom_foto_orig = "_$nom_foto_orig$ext";
        } else {
            $nom_foto_orig = $ext;
        }
    } else {
        # no viene nombre de archivo de la foto original
        # asi que hay que obtener la extensión desde el path
        # que debe venir completo incluyendo el nombre de archivo de la foto
        # con extensión
        $nom_foto_orig = '';
        if ($path_foto =~ /(\.\w+)$/) {
            # ext con punto
            $nom_foto_orig = lc $1;
        }
    }

    my @files = glib_fildir_03::lee_dir($this->{dst_foto});

    my $max = 0;
    foreach my $imag (@files) {
        if ($imag =~ /foto_(\d+)$this->{ts}/) {
            my $thisnum = $1;
            if ($thisnum > $max) {
                $max = $thisnum;
            }
        }
    }
    my $num = $max + 1;
    $num = sprintf('%08d', $num) if (length($num) < 8);

    my $nomfile  = 'foto_' . $num . $this->{ts} . $nom_foto_orig;
    return $nomfile;
}

# Guarda imágenes externas o locales al prontus. Delega el trabajo a subs como _add_foto_sitioexterno().
sub _guarda_fotosfijas {
    my $this             = shift;
    my $cpan_server_name = $this->{cpan_server_name};
    my $document_root    = $this->{document_root};

    foreach my $nom_campo (keys %{$this->{campos}}) {
        next if (substr($nom_campo, 0, 9) ne 'fotofija_');
        next if (!$this->{campos}->{$nom_campo});
        my $val_campo = $this->{campos}->{$nom_campo};

        my $cuadrar = $this->{campos}->{'chk_cuadrar_' . $nom_campo};
        #warn("cuadrar[$cuadrar], nom_campo[$nom_campo]");

        # foto externa al sitio
        if (($val_campo =~ /^https?:/i) && ($val_campo !~ /^https?\:\/\/$cpan_server_name/i)) {
            $this->_add_foto_sitioexterno($nom_campo, $val_campo);
            next;
        }

        # foto local al sitio, asi q saca el server name.
        $val_campo =~ s/^https?\:\/\/$cpan_server_name//i;

        # no se permiten rutas relativas
        # warn("1-nom_campo[$nom_campo] val_campo[$val_campo]");
        if ($val_campo =~ /^\.\.\//) {
            $val_campo =~ s/^(\.\.\/)+(\w)/\/$2/;
        }
        # warn("2-nom_campo[$nom_campo] val_campo[$val_campo]");

        # En adelante, puede tratarse de una foto local al articulo, o bien,
        # local al sitio (pero no al artic). -
        if (!-f "$document_root$val_campo") {
            my $parse_as_cdata = 1;
            $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, $nom_campo, '', $parse_as_cdata);
            next;
        }

        # Obtiene dimensiones actuales de la foto
        my ($msg, $foto_dimx, $foto_dimy) = lib_prontus::dev_tam_img("$document_root$val_campo");
        my $img_type = lib_thumb::get_imag_extension($val_campo);

        # Obtiene dimensiones maximas permitidas para la foto
        my ($maxw) = $this->{campos}->{'_maxw' . $nom_campo};
        my ($maxh) = $this->{campos}->{'_maxh' . $nom_campo};

        # Al hacer esto la imagen no se agrega como nueva al xml. Se reutiliza la misma.
        # Como algunas imágenes no se pueden manipular en prontus, se mantiene el archivo original,
        # solo se asigna al controlador de fotofija.
        if (!lib_prontus::can_edit_img($img_type)) {
            $foto_dimx = $maxw;
            $foto_dimy = $maxh;
        }

        # Solo procede si viene ancho o alto
        if (($maxw =~ /^\d+$/) || ($maxh =~ /^\d+$/)) {
            # si la FOTO es externa al articulo pero local al server:
            # 1.1 Se hace match del nombre de la foto
            # 1.2 Se agrega if para fotos que mantengan el nombre con el que se sube - SCT
            my $nom_foto_orig = "";
            if ($val_campo =~ /\/(foto_\w+)$this->{ts}_(.*?)\.\w+/) {
                # foto local al articulo
                my $nom_campo_orig = $1;
                my $nom_foto_orig  = $2;

                $this->_add_foto_sitiolocal_articulolocal($nom_campo, $val_campo, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig, $nom_foto_orig);
            } elsif ($val_campo =~ /\/(foto_\w+)$this->{ts}\.\w+/) {
                # foto local al articulo
                my $nom_campo_orig = $1;
                $this->_add_foto_sitiolocal_articulolocal($nom_campo, $val_campo, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig, $nom_foto_orig);
            } else {
                # lib_stdlog::print_err("add_foto_sitiolocal_articuloexterno\n");
                $this->_add_foto_sitiolocal_articuloexterno($nom_campo, $val_campo, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar);
            }
        }
    }
}

# Guarda imágenes enviadas via form posting.
sub _guarda_fotos_posting {
    my ($this, $nom_campo_foto, $nom_foto) = @_;
    my $ts               = $this->{ts};
    return unless ($ts =~ /(\d{8})\d{6}/);
    my $fechac = $1;

    my $full_path_foto = $this->{dst_foto} . "/$nom_foto";
    my $path_foto = lib_utils::remove_front_string($full_path_foto, $this->{document_root});

    # Se comprueba que la foto realmente exista
    if (!-f $full_path_foto) {
        lib_stdlog::print_err("\t$full_path_foto no found\n");
        return;
    }

    # Obtiene dimensiones actuales de la foto
    my ($msg, $foto_dimx, $foto_dimy) = lib_prontus::dev_tam_img($full_path_foto);
    unless ($foto_dimx && $foto_dimy) {
        lib_stdlog::print_err("\tError dev_tam_img [0] $full_path_foto: $msg\n");
        return;
    }

    foreach my $nom_campo (keys %{$this->{campos}}) {
        my $val_campo = $this->{campos}->{$nom_campo};
        next if ($nom_campo !~ /^fotofija_\w+/);
        next if (!$val_campo);
        next if ($val_campo !~ /$nom_campo_foto *\((\d+) *\, *(\d+)\)/);

        # Obtiene dimensiones maximas permitidas para la foto
        my ($maxw) = $1;
        my ($maxh) = $2;

        # Solo procede si viene ancho o alto
        if (($maxw =~ /^\d+$/) || ($maxh =~ /^\d+$/)) {
            # lib_stdlog::print_err("\tval_campo[$val_campo], maxw[$maxw], maxh[$maxh]\n");

            my $cuadrar = $this->{campos}->{'chk_cuadrar_' . $nom_campo};
            #lib_stdlog::print_err("\tcuadrar[$cuadrar], nom_campo[$nom_campo]");

            if ($path_foto =~ /\/(foto_\w+)$this->{ts}\.\w+/) {
                my $nom_campo_orig = $1;
                $this->{campos}->{'_hidd_' . $nom_campo_orig} = $nom_foto;
                $this->{campos}->{'_path_' . $nom_campo_orig} = $path_foto;
                $this->{campos}->{$nom_campo}                 = '';
                $this->_add_foto_sitiolocal_articulolocal($nom_campo, $path_foto, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig);
                lib_stdlog::print_err("\tdata: $nom_campo, $path_foto, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig\n");

            }
        }
    }
}

# Carga en el objeto xml_data los datos de una foto externa a Prontus.
sub _add_foto_sitioexterno {
    my ($this, $nom_campo, $val_campo) = @_;

    my $wfoto_externa = $this->{campos}->{'_w' . $nom_campo};
    my $hfoto_externa = $this->{campos}->{'_h' . $nom_campo};

    # warn "add foto sitio externo val_campo[$val_campo] nom_campo[$nom_campo] wfoto_externa[$wfoto_externa]\n";

    my $foto_externa_xml = "<$nom_campo>" . "\n<_w$nom_campo>$wfoto_externa</_w$nom_campo>" . "\n<_h$nom_campo>$hfoto_externa</_h$nom_campo>" . "\n<![CDATA[$val_campo]]>" . "\n</$nom_campo>\n";

    if (!($this->{xml_data} =~ s/<$nom_campo>.*?<\/$nom_campo>/$foto_externa_xml/s)) {
        $this->{xml_data} =~ s/<_public>/<_public>\n$foto_externa_xml/s;
    }
}

# Guarda una foto del artículo basada en otra foto del mismo artículo
sub _add_foto_sitiolocal_articulolocal {
    my ($this, $nom_campo, $val_campo, $maxw, $maxh, $dx, $dy, $cuadrar, $nom_campo_orig, $nom_foto_orig) = @_;
    my $document_root  = $this->{document_root};
    my ($binfoto, $nom_arch, $final_dimx, $final_dimy);

    my $editable = 1;
    my $img_type = lib_thumb::get_imag_extension($val_campo);
    if (lib_prontus::is_animated_image("$document_root$val_campo", $img_type)) {
        $editable = 0;
    }

    # si la foto es mas grande lo que debe ser, se redimensiona
    # y se crea una nueva fotoprontus y esa es asignada a fotofija
    if ($editable && ($dx > $maxw || $dy > $maxh)) {
        eval { ($binfoto, $final_dimx, $final_dimy) = lib_thumb::make_thumbnail($maxw, $maxh, "$document_root$val_campo", $cuadrar) };
        if ($@) {
            $ERR = "Error make_thumbnail: ". $@;
            lib_stdlog::print_err($ERR, "\n");
            return;
        }
        $nom_arch = $this->_guarda_binfoto_prontus($val_campo, $binfoto, $nom_foto_orig);
        return if ($nom_arch eq '');

        # Reasigna val_campo para parsear en el artic.
        $val_campo = "$this->{dst_foto}/$nom_arch";
        # tamaño del archivo
        my $size = -s $val_campo;
        my $mtime = (stat($val_campo))[9];
        $this->_add_foto_prontus_xml($final_dimx, $final_dimy, $nom_arch, $size, 0, $mtime);
        # quita doc root del path completo
        $val_campo =~ s/^$document_root//;
    }

    # foto local, asi q saca el server name.
    $val_campo =~ s/^https?:\/\/$this->{cpan_server_name}//i;
    $val_campo =~ s/^https?:\/\/$this->{public_server_name}//i;

    # Reemplaza fotofija en xml
    my $parse_as_cdata = 1;
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, $nom_campo, $val_campo, $parse_as_cdata);

    return ($final_dimx, $final_dimy, $nom_arch, $val_campo);
}

# Parsea los datos de una foto y los guarda al xml de artículo.
# nom_arch: FOTO_23<TS>.jpg
sub _add_foto_prontus_xml {
    my ($this, $wfoto, $hfoto, $nom_arch, $size, $original, $modified_time) = @_;
    $original = $original || 0;
    $modified_time = $modified_time || 0;
    my $num = '';
    my $ts = $this->{ts};
    if ($nom_arch =~ /^foto_(\w{5,8})$ts/) {
        $num = $1;

        # lib_stdlog::print_err("_add_foto_prontus_xml: [$ts] $num\n");
        # Guardar foto en el xml
        my $fotoxml = "<foto_$num>\n<_nomfoto_$num>$nom_arch</_nomfoto_$num>\n<_wfoto_$num>$wfoto</_wfoto_$num>";
        $fotoxml .= "\n<_hfoto_$num>$hfoto</_hfoto_$num>";
        $fotoxml .= "\n<_bfoto_$num>$size</_bfoto_$num>" if (defined($size));
        $fotoxml .= "\n<_mfoto_$num>$modified_time</_mfoto_$num>";
        $fotoxml .= "\n<_ofoto_$num>$original</_ofoto_$num>\n</foto_$num>";
        $this->{xml_data} =~ s/<_public>/<_public>\n$fotoxml/is;
    } else {
        lib_stdlog::print_err($Artic::ERR = lib_lang::get_interpolated_message('artic_img_id_error', $this->{ts}, $nom_arch), "\n");
    }
}

# Guarda una foto externa al articulo pero local al Prontus
# Si la foto externa es mas grande de lo que debe ser, se redimensiona.
# Requiera o no ser redimensionada, siempre se genera una nueva foto
# prontus para el articulo y luego esta nueva foto, ya internalizada, se asigna a la fotofija.
sub _add_foto_sitiolocal_articuloexterno {
    my ($this, $nom_campo, $val_campo, $maxw, $maxh, $dx, $dy, $cuadrar) = @_;
    my $document_root = $this->{document_root};

    my ($binfoto, $final_dimx, $final_dimy);
    if (($dx > $maxw) || ($dy > $maxh)) {
        eval { ($binfoto, $final_dimx, $final_dimy) = lib_thumb::make_thumbnail($maxw, $maxh, "$document_root$val_campo", $cuadrar) };
        if ($@) {
            $ERR = "Error make_thumbnail: ". $@;
            lib_stdlog::print_err($ERR , "\n");
        }
        # si no se puede redim, tonces la agarra asi no mas.
        if (!$binfoto || !$final_dimx || !$final_dimy) {
            ($final_dimx, $final_dimy) = ($dx, $dy);
            $binfoto = glib_fildir_03::read_file_bin("$document_root$val_campo");
        }
    } else {
        ($final_dimx, $final_dimy) = ($dx, $dy);
        $binfoto = glib_fildir_03::read_file_bin("$document_root$val_campo");
    }

    my $nom_arch = $this->_guarda_binfoto_prontus($val_campo, $binfoto);
    return if ($nom_arch eq '');

    # Reasigna val_campo al nuevo valor de la nueva foto generada para el articulo.
    $val_campo = "$this->{dst_foto}/$nom_arch";
    # tamaño del archivo
    my $size = -s $val_campo;
    my $mtime = (stat($val_campo))[9];
    $this->_add_foto_prontus_xml($final_dimx, $final_dimy, $nom_arch, $size, 0, $mtime);
    # quita doc root del path completo
    $val_campo =~ s/^$document_root//;

    # Guardar en xml y parsear en html la nueva fotofija
    my $parse_as_cdata = 1;
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, $nom_campo, $val_campo, $parse_as_cdata);
}

# Copia un archivo de foto, y agrega los datos de la foto (nombre, dimensiones) al xml del artículo.
# a pesar de llamarse con _, es usada solamente en cgis.
sub _add_foto_filesystem {
    my ($this, $path_foto, $nom_foto_orig, $foto_original) = @_;
    my $document_root    = $this->{document_root};
    my $ts               = $this->{ts};

    return unless ($ts =~ /(\d{8})\d{6}/);
    if ($path_foto eq '' || $nom_foto_orig eq '') {
        lib_stdlog::print_err("Error: path_foto[$path_foto] nom_foto_orig[$nom_foto_orig]\n");
        return;
    }

    my $fechac             = $1;
    my $new_full_path_foto = $this->{dst_foto};
    my $new_path_foto      = lib_utils::remove_front_string($new_full_path_foto, $this->{document_root});
    my $nom_foto           = $this->_get_nom_foto($new_path_foto, $nom_foto_orig);

    # lib_stdlog::print_err("copiando... $document_root$path_foto  -->  $document_root$new_path_foto$nom_foto\n");
    File::Copy::copy("$document_root$path_foto", "$document_root$new_path_foto/$nom_foto");

    # Obtiene dimensiones actuales de la foto
    my ($msg, $foto_dimx, $foto_dimy) = lib_prontus::dev_tam_img("$document_root$new_path_foto/$nom_foto");
    unless ($foto_dimx && $foto_dimy) {
        lib_stdlog::print_err("\tError dev_tam_img [1] $document_root$new_path_foto/$nom_foto: $msg\n");
        return;
    }

    # tamaño del archivo
    my $size = -s "$document_root$new_path_foto/$nom_foto";
    my $mtime = (stat("$document_root$new_path_foto/$nom_foto"))[9];
    # Se agrega la imagen al xml
    $this->_add_foto_prontus_xml($foto_dimx, $foto_dimy, $nom_foto, $size, $foto_original, $mtime);
    return $nom_foto;
}
# extrae fotos en base 64 publicadas en un cuerpo del articulo
# las guarda como archivo binario y las asocia al articulo
sub _guarda_fotos_cuerpo {
    my ($this, $cuerpo) = @_;
    # data:[<mediatype>][;base64],<data>
    my $aux = $cuerpo;
    while($aux =~ /src="(data:(image\/\w+);base64,([a-zA-Z0-9+\/=]+))"/g){
        my $match = $1;
        my $type = $2;
        my $binfoto = MIME::Base64::decode_base64($3);
        my $ext = '.'. mime_types::type_2_ext($type);
        # para convertir las extensiones a jpeg a jpg
        $ext = '.jpg' if ($ext eq '.jpeg');
        my $nom_arch = $this->_get_nom_foto($this->{dst_foto}, $ext);
        # lib_stdlog::print_err("dst_img[$this->{dst_foto}] - nom_arch[$nom_arch]\n");
        my $path_foto = "$this->{dst_foto}/$nom_arch";
        # guarda la foto en disco
        lib_thumb::write_image($path_foto, $binfoto);
        # tamaño del archivo
        my $size = -s $path_foto;
        my $mtime = (stat($path_foto))[9];
        my ($msg_size, $wfoto, $hfoto, $orientation) = lib_prontus::dev_tam_img($path_foto);
        # foto no es original ya que se genero con el editor del cuerpo
        $this->_add_foto_prontus_xml($wfoto, $hfoto, $nom_arch, $size, 0, $mtime);
        # quita doc root del path completo
        $path_foto =~ s/^$this->{document_root}//;
        $cuerpo =~ s/\Q$match\E/$path_foto/;
    }
    return $cuerpo;
}

# Genera un archivo binario a partir de una foto. En caso de que el original sea un GIF, genera un archivo PNG.
sub _guarda_binfoto_prontus {
    my ($this, $val_campo, $binfoto, $nom_foto_orig) = @_;
    return '' if (!$binfoto);
    $this->_check_ext_foto($val_campo) || return '';

    my $nom_arch = $this->_get_nom_foto($val_campo, $nom_foto_orig);

    # si es gif , se genera png
    if ($val_campo =~ /\.gif$/) {
        $nom_arch =~ s/\.gif$/\.png/;
    }

    # warn "dst_img[$this->{dst_foto}] - nom_arch[$nom_arch]";
    my $dst_binfoto = "$this->{dst_foto}/$nom_arch";
    lib_thumb::write_image($dst_binfoto, $binfoto);

    if (!exists $prontus_varglb::CONFIG{$this->{prontus_id}}{'CACHE_PURGE_EXCLUDE_FID'}{$this->{campos}->{'_fid'}}) {
        lib_prontus::purge_cache($dst_binfoto)
          if (   glib_hrfec_03::get_antiguedad_ts($this->{ts}) > 60
              && glib_fildir_03::get_antiguedad_archivo($dst_binfoto) <= 60);
    }

    return $nom_arch;
}

# Genera una fotofija con nuevas dimensiones a partir de los datos pasados.
# Valida datos y llama a $this->_add_foto_sitiolocal_articulo_local()
# Recibe como parametro:
# - El nombre de la foto_n. Ej: foto_134520110619122632.jpg
# - El nombre la foto fija (nombre marca marca)
# - El ancho maximo
# - El alto maximo
# - Si se debe cuadrar la imagen o no: si|<anything>
sub _genera_fotofija {
    my ($this, $nom_foto, $fotofija, $maxw, $maxh, $cuadrar) = @_;
    my $img_type = lib_thumb::get_imag_extension($nom_foto);

    $cuadrar = '' unless ($cuadrar eq 'si' || lib_prontus::can_edit_img($img_type));

    my $full_path_foto = $this->{dst_foto} . "/$nom_foto";
    if (lib_prontus::is_animated_image($full_path_foto, $img_type)) {
        $cuadrar = '';
    }

    my $path_foto = lib_utils::remove_front_string($full_path_foto, $this->{document_root});

    # Se revisa que la foto exista
    return (lib_lang::get_interpolated_message('artic_img_exist_error', $nom_foto, $full_path_foto), '', '', '', '') unless (-f "$full_path_foto");

    my ($msg, $foto_dimx, $foto_dimy);

    # Si el tipo de imagen se puede editar, continua normal.
    if (lib_prontus::can_edit_img($img_type)) {
        # Obtiene dimensiones actuales de la foto
        ($msg, $foto_dimx, $foto_dimy) = lib_prontus::dev_tam_img("$full_path_foto");
        return ($msg, '', '', '', '') if ($msg);
    } else {
        ($msg, $foto_dimx, $foto_dimy) = ('', $maxw, $maxh);
    }

    # Solo procede si viene ancho o alto
    if (($maxw =~ /^\d+$/) || ($maxh =~ /^\d+$/) || ($foto_dimx =~ /^\d+$/) || ($foto_dimy =~ /^\d+$/)) {
        if ($nom_foto =~ /(foto_\w+)$this->{ts}(_[\w\-\.]+)?\.\w+/) {
            # Foto local al articulo
            my $nom_campo_orig = $1;
            #lib_stdlog::print_err("_genera_fotofija: $fotofija, $path_foto, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig\n");
            my ($final_dimx, $final_dimy, $final_nomfoto, $final_relfoto) = $this->_add_foto_sitiolocal_articulolocal($fotofija, $path_foto, $maxw, $maxh, $foto_dimx, $foto_dimy, $cuadrar, $nom_campo_orig);

            return ('', $final_dimx, $final_dimy, $final_nomfoto, $final_relfoto);
        } else {
            return (lib_lang::get_interpolated_message('artic_img_regexp_error', $nom_foto), '', '', '', '');
        }
    } else {
        return (lib_lang::get_interpolated_message('artic_img_dim_error', $maxw, $maxh, $foto_dimx, $foto_dimy), '', '', '', '');
    }
}

# limpia y formatea los campos a ingresar en base de datos.
sub _get_campos_art4bd {
    my $this = $_[0];
    my %campos    = $this->get_xml_content();

    if (!defined($campos{'_autor'})) {
        $campos{'_autor'} = '';
    } else {
        $campos{'_autor'} = substr($campos{'_autor'}, 0, 60);
    }
    $campos{'_fid'}   = '' if (!defined($campos{'_fid'}));
    $campos{'_alta'}  = 0 if (!defined($campos{'_alta'}) || $campos{'_alta'} eq '');
    $campos{'_indexar'}  = 0 if (!defined($campos{'_indexar'}) || $campos{'_indexar'} eq '');
    $campos{'_priorizar'}  = 0 if (!defined($campos{'_priorizar'}) || $campos{'_priorizar'} eq '');

    $campos{'_txt_bajada'}   = '' if (!defined($campos{'_txt_bajada'}));
    $campos{'_users_id'}     = '' if (!defined($campos{'_users_id'}));
    $campos{'_soloportadas'} = 0 if (!$campos{'_soloportadas'});

    $campos{'_seccion1'} = 0 if ($campos{'_seccion1'} eq '');
    $campos{'_tema1'}    = 0 if ($campos{'_tema1'}    eq '');
    $campos{'_subtema1'} = 0 if ($campos{'_subtema1'} eq '');
    $campos{'_seccion2'} = 0 if ($campos{'_seccion2'} eq '');
    $campos{'_tema2'}    = 0 if ($campos{'_tema2'}    eq '');
    $campos{'_subtema2'} = 0 if ($campos{'_subtema2'} eq '');
    $campos{'_seccion3'} = 0 if ($campos{'_seccion3'} eq '');
    $campos{'_tema3'}    = 0 if ($campos{'_tema3'}    eq '');
    $campos{'_subtema3'} = 0 if ($campos{'_subtema3'} eq '');

    $campos{'_s3'} = 0 if ($campos{'_s3'} eq '');
    $campos{'_s3_size'} = 0 if ($campos{'_s3_size'} eq '');
    $campos{'_s3_media_size'}  = 0 if ($campos{'_s3_media_size'} eq '');
    $campos{'_s3_asset_size'}  = 0 if ($campos{'_s3_asset_size'} eq '');

    $campos{'_disk_size'}  = 0 if ($campos{'_disk_size'} eq '');
    $campos{'_media_size'} = 0 if ($campos{'_media_size'} eq '');
    $campos{'_asset_size'} = 0 if ($campos{'_asset_size'} eq '');
    $campos{'_imag_size'}  = 0 if ($campos{'_imag_size'} eq '');
    $campos{'_total_size'} = 0 if ($campos{'_total_size'} eq '');

    $campos{'_modtime'} = 0 if ($campos{'_modtime'} eq '');

    $campos{'_indexar'} = 0 if ($campos{'_indexar'} eq '');

    $campos{'_horap'} =~ s/://g;
    $campos{'_horap'} = 0 if ($campos{'_horap'} eq '');

    $campos{'_horae'} =~ s/://g;
    $campos{'_horae'}  = 0     if ($campos{'_horae'}  eq '');
    $campos{'_fechae'} = 99999999 if ($campos{'_fechae'} eq '');

    $campos{'_extension'} = lib_utils::get_file_extension($campos{'_plt'});
    $campos{'vtxt_cuerpo'} = '' if (!defined($campos{'vtxt_cuerpo'}));

    # Ajusta bajada
    my $bajada = $campos{'_txt_bajada'};
    if ($bajada eq '' && $campos{'vtxt_cuerpo'} ne '') {
        $bajada = $campos{'vtxt_cuerpo'};
        $bajada = lib_prontus::get_minitext_value($bajada);
    }
    if ($bajada ne '') {
        $bajada =~ s/ +/ /sg;
        $bajada =~ s/^\s+//s;
        $bajada =~ s/\s+$//s;

        # Ajusta en caso de bajada muy larga, de manera de evitar truncado en la BD
        if ((length $bajada) > 200) {
            $bajada = lib_prontus::ajusta_nchars($bajada, 200);
        }
        $campos{'_txt_bajada'} = $bajada;
    }

    # Ajusta titular
    my $titu = $campos{'_txt_titular'};
    # Ajusta en caso de titulares muy largos, de manera de evitar truncado en la BD
    # campo titular no tiene tags
    if ((length $titu) > 255) {
        $titu = substr($titu, 0, 252) . '...';
    }
    $campos{'_txt_titular'} = $titu;

    $campos{'_fechap_horap'} = "$campos{'_fechap'}$campos{'_horap'}";
    $campos{'_fechae_horae'} = "$campos{'_fechae'}$campos{'_horae'}";

    return %campos;
}

# Parsea en vista del artic los recursos de un cierto tipo asociados al mismo
# nom_campo: asocfile_xx, multimedia_xx
sub _parsing_mmedia {
    my ($this, $nom_campo, $buffer) = @_;
    # valor del campo no lleva ningun tipo de directorio, por ej multimedia_video120190926165638.mp4
    my $path = "$this->{'dst_multimedia'}/". $this->{xml_content}->{$nom_campo};

    my $parse_val = "$this->{path_multimedia}/" . $this->{xml_content}->{$nom_campo};
    if (-f $path || $this->{xml_content}->{"_s3"}) {
        # Parsea el recurso como tal
        $buffer =~ s/%%$nom_campo%%/$parse_val/g;
    } else {
        lib_stdlog::print_err("[$this->{ts}] File not found [$path]\n");
        $buffer =~ s/%%$nom_campo%%/$nom_campo\[$parse_val\]FILE_NOT_FOUND/g;
    }

    return $buffer;
}

# procesa marcas asocfile y sus campos especiales asociados
sub _parsing_asocfile {
    my ($this, $nom_campo, $buffer) = @_;

    my $path = "$this->{dst_asocfile}/$this->{ts}/" . $this->{xml_content}->{$nom_campo};

    # Parsea el size.
    my $size;
    if (!defined($this->{xml_content}->{"$nom_campo.bytes"})) {
        # para no repetir la busqueda
        $this->{xml_content}->{"$nom_campo.bytes"} = -s $path;
    }
    $size = glib_str_03::bytes2kb($this->{xml_content}->{"$nom_campo.bytes"});
    $buffer =~ s/%%$nom_campo.size%%/$size/g;

    # Parsea el recurso como tal
    $path = "$this->{path_asocfile}/$this->{ts}/" . $this->{xml_content}->{$nom_campo};
    $buffer =~ s/%%$nom_campo%%/$path/g;
    $buffer =~ s/%%$nom_campo.file%%/$this->{xml_content}->{$nom_campo}/g;

    # Parsear la extension del archivo
    my $ext;
    if (defined($this->{xml_content}->{"$nom_campo.ext"})) {
        $ext = $this->{xml_content}->{"$nom_campo.ext"};
    } else {
        $ext = lib_utils::get_file_extension($this->{xml_content}->{$nom_campo});

        # para no repetir la busqueda
        $this->{xml_content}->{"$nom_campo.ext"} = $ext;
    }
    $buffer =~ s/%%$nom_campo.ext%%/$ext/g;

    return $buffer;
}

# Parseo de fotos fijas
sub _parsing_fotos {
    my ($this, $nom_campo, $val_campo, $buffer) = @_;
    my ($msg, $foto_dimx, $foto_dimy);
    my %campos = $this->get_xml_content();

    $val_campo =~ s/(foto_\d+)\_(.*)$/$1\/$2/s;
    $buffer =~ s/%%$nom_campo%%/$val_campo/g;

    my $este_prontus = $this->{prontus_id};

    if ($val_campo =~ /$este_prontus/) {    # val_campo es del tipo: /prontus_dev/site/artic/20060410/imag/FOTO_0120060410165548.jpg
        my $ts = $this->{ts};
        # parseo ademas las dimensiones de la foto en el articulo
        if ($val_campo =~ /(foto_\d+)$ts(.+?)?\.(\w+)$/) {
            my $nom_foto_original = lc $1;
            my $ext = lc $3;

            $foto_dimx = $campos{"_w$nom_foto_original"};
            $foto_dimy = $campos{"_h$nom_foto_original"};
            # gif no se reemplaza por webp
            if ($ext ne 'gif') {
                $val_campo =~ s/\.[a-zA-Z]+$/\.webp/;
            }
            $buffer =~ s/%%$nom_campo\.webp%%/$val_campo/g;

            $buffer =~ s/%%$nom_campo\.width%%/$foto_dimx/g;
            $buffer =~ s/%%$nom_campo\.height%%/$foto_dimy/g;
            $buffer =~ s/%%$nom_campo\.ext%%/$ext/g;
            $buffer =~ s/%%$nom_campo\.size%%/$campos{"_b$nom_foto_original"}/g;
        }

    } elsif ($val_campo =~ /https?:\/\//) {    # url externa
        # parseo dimensiones
        $foto_dimx = $campos{"_w$nom_campo"};    # _wfotofija_art200
        $foto_dimy = $campos{"_h$nom_campo"};

        # warn "foto externa nom_campo[$nom_campo] nom_campo_w[_w$nom_campo] foto_dimx[$foto_dimx]";
        $buffer =~ s/%%$nom_campo\.width%%/$foto_dimx/g;
        $buffer =~ s/%%$nom_campo\.height%%/$foto_dimy/g;
    }
    return $buffer;
}

# Parsea en vista del artic los campos vtxt asociadas al mismo.
sub _parsing_vtxt {
    my ($this, $buffer, $nom_campo, $val_campo) = @_;

    $val_campo =  '' if (!defined($val_campo));

    my ($looptit, $tithtml) = $this->_get_data4subtit($buffer, $nom_campo);
    my $curr_nrotit = '0';
    my %hash_subtits;

    if ($prontus_varglb::CONFIG{$this->{prontus_id}}{'VTXT_RELPATH_LINK'} eq 'SI') {
        # Saca el server name.
        $val_campo =~ s|https?://$this->{cpan_server_name}/|/|ig;
        $val_campo =~ s|https?://$this->{public_server_name}/|/|ig;
    }

    $val_campo =~ s/^\s+//g;
    $val_campo =~ s/^(&nbsp;)+//ig;

    $val_campo =~ s/\s+$//g;
    $val_campo =~ s/(&nbsp;)+$//ig;

    # Corrige issue ffox 3.6.11, elimina las rutas relativas, ppalmente de las imagenes.
    # Da por sentado que si encuentra alguna ruta relativa, al sacarle los puntos quedara absoluta
    # Esto siempre sera true para imagenes prontus
    if ($val_campo =~ /src *= *("|')\.\.\//i) {
        $val_campo =~ s/src *= *("|')(\.\.\/)+(\w)/src=$1\/$3/ig;
    } elsif ($val_campo =~ /src *= *("|')\w+?\//i) {
        $val_campo =~ s/(src) *= *("|')(\w+?)\//$1=$2\/$3\//ig;
    }

    my $vtxt_aux = $val_campo;

    # Elimina funciones para double click del html del vtxt.
    $vtxt_aux =~ s/ondblclick *= *".+?"//isg;

    # Parsea los subtits en el vtxt
    my $vtxt_aux_consubtit = $vtxt_aux;
    while ($vtxt_aux =~ /(<(\w+)\s+[^<>]*class=["']subtit([a-zA-Z0-9\-]*)["'][^>]*>[^<]*?<\/\2>)/isg) {
        my $tag          = $1;
        my $level_subtit = $3;
        my $tit          = $tag;
        $tit =~ s/<.*?>//sg;
        next if ($tit !~ /\w/);
        if ($tithtml) {
            # Usa mini tpl si es q esta en la plantilla del articulo
            my $tithtml_aux = $tithtml;
            $tithtml_aux        =~ s/%%_subtit%%/$tit/;
            $tithtml_aux        =~ s/%%_subtit_aname%%/$nom_campo\_T$curr_nrotit/;
            $tithtml_aux        =~ s/%%_level_subtit%%/$level_subtit/g;
            $vtxt_aux_consubtit =~ s/\Q$tag\E/$tithtml_aux/s;
        } else {
            $vtxt_aux_consubtit =~ s/\Q$tag\E/<a name="$nom_campo\_T$curr_nrotit">$tag<\/a>/s;
        }

        # Guarda item a incluir en el menu
        my $looptit_aux = $looptit;
        $looptit_aux =~ s/%%_subtit_key%%/#$nom_campo\_T$curr_nrotit/g;
        $looptit_aux =~ s/%%_subtit%%/$tit/g;
        $looptit_aux =~ s/%%_level_subtit%%/$level_subtit/g;
        # acumula item para ponerlo despues en el menu
        $hash_subtits{$curr_nrotit} = '<!--STIT_' . $nom_campo . '-->' . $looptit_aux . '<!--/STIT_' . $nom_campo . '-->';
        $curr_nrotit++;
    }

    foreach my $st (sort { $a <=> $b } (keys %hash_subtits)) {
        # lib_stdlog::print_err("st[$st]\n");
        my $item_menu = $hash_subtits{$st};
        # Reemplazar en la pagina misma.
        $buffer =~ s/%%_subtit_loop_$nom_campo%%(.*?)%%\/_subtit_loop_$nom_campo%%/$item_menu%%_subtit_loop_$nom_campo%%$1%%\/_subtit_loop_$nom_campo%%/sg;
    }
    # Eliminar TITLOOP sobrante.
    $buffer =~ s/%%_subtit_loop_$nom_campo%%(.*?)%%\/_subtit_loop_$nom_campo%%//sg;
    # Eliminar BLOQUE %%TITHTML%% sobrante
    $buffer =~ s/%%_subtit_html_$nom_campo%%(.*?)%%\/_subtit_html_$nom_campo%%//sg;

    # Detectar si el VTXT esta insertando invocaciones a funciones js derivadas del uso del boton media.
    # Si las encuentra, incluye el JS necesario.

    # Para que no escapear código PHP
    # elimina secuencias <?
    $vtxt_aux_consubtit =~ s/< *\?/< &#63;/g;

    # Para el plugin prontus_insert
    my $safe_counter = 0;
    while ($vtxt_aux_consubtit =~ /(<prontus_insert(.*?)>.*?<\/prontus_insert>)/s) {
        my $code  = $1;
        my $attrs = $2;

        $safe_counter++;
        if ($safe_counter > 100) {
            lib_stdlog::print_err("[vtxt] safe exit <prontus_insert> $this->{ts}\n");
            $vtxt_aux_consubtit =~ s/<prontus_insert(.*?)>.*?<\/prontus_insert>//sg;
            last;
        }
        # <prontus_insert data-type="js" data-code="var%20myvar%20%3D%20'Hello'%3B%0D%0Aalert(myvar)%3B">Código Javascript</prontus_insert>

        my $newnode = '';
        if ($attrs =~ / data-type="(php|ssi|ssi2|ssi_virtual)"/) {
            my $tipo = $1;
            if ($attrs =~ / data-src="(.*?)"/) {
                my $file = $1;

                if ($file =~ /^\// && $file !~ /\/..\//) {
                    if (!-f "$this->{document_root}$file") {
                        $newnode = lib_lang::get_interpolated_message('artic_file_not_found_error', "$this->{document_root}$file");
                    } else {
                        if ($tipo eq 'ssi') {
                            $newnode = '<!--#include file="' . $file . '" -->';
                        } elsif ($tipo eq 'ssi_virtual' || $tipo eq 'ssi2') {
                            $newnode = '<!--#include virtual="' . $file . '" -->';
                        } else {
                            $newnode = '<?php include($_SERVER["DOCUMENT_ROOT"] . "' . $file . '"); ?>';
                        }
                    }
                } else {
                    $newnode = lib_lang::get_interpolated_message('artic_file_has_no_path_error', $file);
                }
            }
        } elsif ($attrs =~ / data-type="js"/) {
            if ($attrs =~ / data-src="(.*?)"/) {
                my $file = $1;
                if ($file =~ /^\// && $file !~ /\/..\//) {
                    if (-f "$this->{document_root}$file") {
                        $newnode = '<script src="' . $file . '"></script>';
                    } else {
                        $newnode = lib_lang::get_interpolated_message('artic_file_not_found_error', "$this->{document_root}$file");
                    }
                } elsif ($file =~ /^https?:\/\//) {
                    $newnode = '<script src="' . $file . '"></script>';
                }
            } elsif ($attrs =~ / data-code="(.*?)"/s) {
                my $jscode = $1;
                $jscode = URI::Escape::uri_unescape($jscode);
                $jscode = HTML::Entities::decode_entities(HTML::Entities::decode_entities($jscode));
                $newnode = '<script>'."\n".$jscode."\n".'</script>';
            }
        }
        $code = quotemeta $code;
        $vtxt_aux_consubtit =~ s/$code/$newnode/is;
    }

    # para el plugin html_insert
    $safe_counter = 0;
    while ($vtxt_aux_consubtit =~ /(<html_insert(.*?)>.*?<\/html_insert>)/s) {
        my $code  = $1;
        my $attrs = $2;

        $safe_counter++;
        if ($safe_counter > 100) {
            lib_stdlog::print_err("[vtxt] safe exit <html_insert> $this->{ts}\n");
            $vtxt_aux_consubtit =~ s/<html_insert(.*?)>.*?<\/html_insert>//sg;
            last;
        }
        # <html_insert data-code="&amp;lt;iframe width=&amp;quot;560&amp;quot; height=&amp;quot;315&amp;quot; src=&amp;quot;https://www.youtube.com/embed/FLcyRrbh1RI?rel=0&amp;quot; frameborder=&amp;quot;0&amp;quot; allow=&amp;quot;encrypted-media&amp;quot; allowfullscreen&amp;gt;&amp;lt;/iframe&amp;gt;">Código HTML</html_insert>

        my $newnode = '';
        if ($attrs =~ / data-code="(.*?)"/s) {
            my $htmlcode = $1;
            $htmlcode = URI::Escape::uri_unescape($htmlcode);
            $htmlcode = HTML::Entities::decode_entities(HTML::Entities::decode_entities($htmlcode));
            $newnode = "\n<div>".$htmlcode."</div>\n";
        }
        $code = quotemeta $code;
        $vtxt_aux_consubtit =~ s/$code/$newnode/;
    }

    # Prontus Card - <prontus_card />
    $safe_counter = 0;
    while ($vtxt_aux_consubtit =~ /(<prontus_card(.*?)>.*?<\/prontus_card>)/is) {
        my $code  = $1;
        my $attrs = $2;

        $safe_counter++;
        if ($safe_counter > 100) {
            lib_stdlog::print_err("[vtxt] Salida <prontus_card> $this->{ts}\n");
            $vtxt_aux_consubtit =~ s/<prontus_card(.*?)>.*?<\/prontus_card>//isg;
            last;
        }

        # <prontus_card data-ts="20210912213325" data-plt="card_large">Prontus Card 'card_large' - Artículo 20210912213325</prontus_card>

        my $newnode = '';
        my ($dataTs, $dataPlt, $dataUrl, $dataExtJson);

        if ($attrs =~ / data-ts="(.*?)"/s) {
            $dataTs = $1;
        }

        if ($attrs =~ / data-plt="(.*?)"/s) {
            $dataPlt = $1;
            $dataPlt =~ s/[^a-z0-9_]+//sig;
        }

        if ($attrs =~ / data-url="(.*?)"/s) {
            $dataUrl = $1;
        }

        if ($attrs =~ / data-data="(.*?)"/s) {
            $dataExtJson = $1;
        }

        if ($dataTs && $dataPlt) {
            # Encontrar html de plantilla
            my $dirPlt = "$prontus_varglb::DIR_SERVER/$this->{prontus_id}/plantillas/vtxt/cards";
            my $pathPlt = "$dirPlt/$dataPlt.html";

            lib_stdlog::print_err("pathPlt[$pathPlt]\n");

            if (-f $pathPlt) {
                my $bufferPlt = glib_fildir_03::read_file($pathPlt);
                # Convertir <script *> en type text/template.
                $bufferPlt =~ s/<script(.*?)>/<script type=\"text\/template\">/sg;

                $newnode = "<div class=\"prontus-card-container\"><div class=\"prontus-card-content\" data-ts=\"$dataTs\" data-prontus-id=\"$this->{prontus_id}\" style=\"display:none;\">\n$bufferPlt\n</div></div>";
            }
        }

        if ($dataUrl && $dataPlt && $dataExtJson) {
            # Encontrar html de plantilla
            my $dirPlt = "$prontus_varglb::DIR_SERVER/$this->{prontus_id}/plantillas/vtxt/cards";
            my $pathPlt = "$dirPlt/$dataPlt.html";

            if (-f $pathPlt) {
                my $bufferPlt = glib_fildir_03::read_file($pathPlt);

                # Convertir <script *> en type text/template.
                $bufferPlt =~ s/<script(.*?)>/<script type=\"text\/template\">/sg;

                $dataExtJson =~ s/&quot;/"/sg;

                $newnode = "<div class=\"prontus-card-container\"><div class=\"prontus-card-content\" data-url=\"$dataUrl\" data-data='$dataExtJson' style=\"display:none;\">\n$bufferPlt\n</div></div>";
            }
        }

        $code = quotemeta $code;
        $vtxt_aux_consubtit =~ s/$code/$newnode/is;
    }

    # Se le indica que no se deben escapear los PHP
    $buffer = lib_prontus::parse_tag_prontus($vtxt_aux_consubtit, $nom_campo, $buffer, 1);

    return $buffer;
}

# Parsea el buffer de artículo buscando campos de subtítulo.
# Obtiene un template de indice de titulos, y busca las marcas _SUBTIT_HTML_xx, devuelve el loop repetido para cada título y el valor de cada título.
sub _get_data4subtit {
    my ($this, $buffer, $nom_campo) = @_;

    # Obtencion de template de indice de titulos
    my ($looptit, $tithtml);

    # La cosa es sin framesets, o sea el indice va en la pagina misma.
    $buffer =~ /%%_subtit_loop_$nom_campo%%(.*?)%%\/_subtit_loop_$nom_campo%%/s;
    $looptit = $1;    # segmento html que se repite para cada titulo.
    $looptit =~ s/%%_prontus_id%%/$this->{prontus_id}/g;

    # Plantilla para los subtitulos
    if ($buffer =~ /%%_subtit_html_$nom_campo%%(.+?)%%\/_subtit_html_$nom_campo%%/s) {
        $tithtml = $1;
        $tithtml =~ s/%%_prontus_id%%/$this->{prontus_id}/g;
    }

    return ($looptit, $tithtml);
}
# Parsea el buffer de artículo buscando tags de postproceso.
# Agrega cada postproceso encontrado a la variable Artic->post_proceso_lista.
# Retorna el buffer limpio.
sub _genera_lista_post_procesos {
    my ($this, $buffer)= @_;

    while ($buffer =~ /<!--POST_PROCESO=(.+?)-->/g) {
        # $this->{post_proceso_lista} .= "$1\t";
        push(@{$this->{post_proceso_lista}}, $1);
    }
    $buffer =~ s/<!--POST_PROCESO=.+?-->//g;
    return $buffer;
}

# Devuelve la friendly URL correcta para el artículo en cuestión según la configuración de Prontus.
sub get_friendly_url {
    my ($this) = @_;
    # carga datos del xml si no estan
    $this->get_xml_content();

    my %artic_data = (
        'fid' => $this->{xml_content}{'_fid'},
        'url' => $this->{xml_content}{'_url'},
        'ts' => $this->{ts},
        'prontus_id'  => $this->{prontus_id},
        'mv' => '',
        'ext' => lib_utils::get_file_extension($this->{xml_content}{'_plt'})
    );

    return lib_prontus::parse_filef('%%_fileurl%%', \%artic_data);
}

# Calcula y retorna el tamaño que utiliza el artículo en el disco físico
# Indexar imagenes, multimedia y adjuntos por separado, html se obtiene del total - el contenido
sub get_disk_size {
    my ($this) = @_;
    my $ts = $this->{ts};

    my $disk_size = -s ($this->{dst_xml}."/$ts.xml");

    # paginas principales
    my @file_list = glob ($this->{dst_pags}."/$ts*");
    foreach my $file (@file_list) {
        $disk_size += -s $file;
    }

    # vistas alternativas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}}) {
        @file_list = glob ($this->{dst_pags}."-$mv/$ts*");
        foreach my $file (@file_list) {
            $disk_size += -s $file;
        }
    }

    # paginas paralelas
    @file_list = glob ($this->{dst_pagspar}."/$ts*");
    foreach my $file (@file_list) {
        $disk_size += -s $file;
    }

    my $imag_size = 0;
    # imagenes del articulo
    @file_list = glob ($this->{dst_foto}."/foto*$ts*");
    foreach my $file (@file_list) {
        $imag_size += -s $file;
    }

    my $asset_size = 0;
    # documentos/adjuntos del articulo
    @file_list = glob ($this->{dst_asocfile}. '/*');
    foreach my $file (@file_list) {
        $asset_size += -s $file;
    }

    my $media_size = 0;
    # multimedia: datos del video y thumbs
    @file_list = glob ($this->{dst_mm_thumbs}. "/*$ts*");
    foreach my $file (@file_list) {
        $media_size += -s $file;
    }

    # multimedia: videos y calidades
    @file_list = glob ($this->{dst_multimedia}. "/*$ts*");
    foreach my $file (@file_list) {
        if (-d $file) {
            my @subdir_file_list = glob ($file. '/*');
            foreach my $sub_file (@subdir_file_list) {
                $media_size += -s $sub_file;
            }
        } else {
            $media_size += -s $file;
        }
    }

    $disk_size = 0 if (!defined($disk_size));
    $imag_size = 0 if (!defined($imag_size));
    $asset_size = 0 if (!defined($asset_size));
    $media_size = 0 if (!defined($media_size));

    $disk_size = $disk_size + $imag_size + $asset_size + $media_size;

    return ($disk_size, $imag_size, $asset_size, $media_size);
}

sub update_art_size {
    my ($this, $total_size, $disk_size, $imag_size, $asset_size, $media_size) = @_;

    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_media_size", $media_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_asset_size", $asset_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_imag_size", $imag_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_disk_size", $disk_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_total_size", $total_size, 0);
}

sub update_s3_size {
    my ($this, $s3_size, $s3_media_size, $s3_asset_size) = @_;

    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_s3_size", $s3_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_s3_media_size", $s3_media_size, 0);
    $this->{xml_data} = lib_prontus::replace_in_xml($this->{xml_data}, "_s3_asset_size", $s3_asset_size, 0);
}

1;

# Para desplegar la doc. desde linea de comandos:
# perldoc Artic.pm
# Para generar la doc en html:
# perldoc -T -o html Artic.pm > Artic.pm.html

__END__

=encoding utf8

=head1 NAME

B<Artic.pm> - Clase para manipulacion de articulos Prontus, por medio de la creacion de un objeto tipo "Artic"

=for comment

=head1 DESCRIPTION

Esta clase contiene operaciones tipicas a realizar con los artículos, como por ejemplo leer su XML,
obtener el path de este, generar el html, parsear sus datos, etc., sin necesidad de conocer la logica
interna, solo basta instanciar correctamente el objeto de tipo 'Artic'. Esta clase se utiliza en todas
las partes de Prontus en donde se manipulan los articulos.
Ademas de ser util internamente para las CGIs de prontus, tambien puede servir para simplificar
la programacion de post-procesos, scripts de migracion de contenidos, etc., utilizando los metodos
publicos y atributos de la clase.

=for comment

=head1 SYNOPSIS

Instanciar y acceder a un metodo publico:

    use Artic;

    # Creacion del objeto Artic.

OPCION 1: utilizando variables globales prontus ($prontus_varglb::NOM_VARIABLE).
Esta es la opción más segura y recomendada, pero requiere cargar la conf de prontus primero,
de la sgte. forma:

    # Path conf y load config de prontus
    use prontus_varglb;
    prontus_varglb::init();
    use lib_prontus;
    use lib_users;
    use lib_utils;
    use lib_bd;
    use glib_dbi_03;
    my $prontus_id = 'prontus_noticias';
    lib_utils::load_config_by_id($prontus_id);

    # Creacion de obj Artic usando variables globales Prontus
    my $artic_obj = Artic->new(
                    'prontus_id'        => $this->{prontus_id},
                    'public_server_name'=> $prontus_varglb::CONFIG{$this->{prontus_id}}{'PUBLIC_SERVER_NAME'},
                    'cpan_server_name'  => $prontus_varglb::CPAN_SERVER_NAME,
                    'document_root'     => $prontus_varglb::DIR_SERVER,
                    'ts'                => $ts,
                    'no_check_dirs'      => 1
                    # ********** Ver Nota (1) ************
                    'campos'=>{})
                    || die(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");

Nota (1): la opción "no_check_dirs" se utiliza para no verificar ni crear los directorios asociados al artículo, se puede usar
cuando se necesita acceder a los datos de un artículo existente sin modificarlo


OPCION 2: Creacion de obj Artic sin usar var. globales Prontus

    # Creacion del obj Artic, sin var Prontus
    my $artic_obj = Artic->new(
                    'prontus_id'        => 'prontus_noticias',
                    'public_server_name'=> 'www.misitio.cl',
                    'cpan_server_name'  => 'prontus.misitio.cl',
                    'document_root'     => '/sites/misitio/web',
                    'ts'                => '20081023172849',
                    # ********** Ver Nota (1) ************
                    'campos'=>{})
                    # ********** Ver Nota (2) ************
                    || die(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");

Ejemplo de utilizacion de metodos del obj Artic

    # Generar vista o html del articulo.
    $artic_obj->generar_vista_art($mv)
                || die("Error: $Artic::ERR");

    # carga hash con campos del XML del articulo, con nombre_campo => valor_campo
    my %campos_xml = $artic_obj->get_xml_content();

Nota (1):
El atributo 'campos' en el constructor es para poder pasarle un hash con todos los datos
necesarios para poder crear un articulo. Para el caso de que solo se necesite LEER info del articulo,
basta con dejarlo de la forma indicada en el ejemplo.

Nota (2):
Si se trata del procesamiento de un solo articulo, generalmente el die() sera suficiente.
pero en caso de que estemos procesando varios articulos dentro de un bucle, en donde se desea que
aunque uno falle, se continue con el resto, en vez de die() se puede realizar algo como lo siguiente:

    foreach my $ts (@arr_artic) {
        # Creacion del objeto Artic.
        my $artic_obj = Artic->new(...)
                        || print(lib_lang::get_interpolated_message('global_error_artic_init', $Artic::ERR) . "\n");

        # En la instruccion siguiente al constructor, revisamos si se creo el objeto y en caso negativo
        # hacemos un next al siguiente articulo
        next if (!ref($artic_obj));

        # Si el artic se instancio OK, realizo el resto de las operaciones, por ejemplo,
        # reparsearlo en todas las vistas a partir de los datos del XML:
        # Para la vista ppal:
        $artic_obj->generar_vista_art('') || print $Artic::ERR;

        # Para vistas secundarias, agregar:
        foreach my $mv (keys %{$prontus_varglb::CONFIG{$this->{prontus_id}}{'MULTIVISTA'}}) {
            $artic_obj->generar_vista_art($mv) || print $Artic::ERR;
        };

    };

=for comment
# ---------------------------------------------------------------

=head2 CONSTRUCTOR

=over 8

=item B<new()>

Devuelve un nuevo objeto Artic. Los parametros son los indicados en la sinopsis.
Es importante considerar que los campos pasados en el hash 'campos' al constructor deben tener
los nombres correspondientes a los campos del XML del artic.
para un ejemplo sobre como pasar pasarle este hash al objeto a fin de ingresar un nuevo articulo,
revisar la lib_artic.pm

B<Parametros>

$clase

  la referencia a usar para $this.

$artic

  referencia a hash con los campos del artículo.

B<Retorna>

  Referencia al objeto artic.

=for comment
# ---------------------------------------------------------------

=head2 VARIABLES DE CLASE ESTATICAS

=over 8

=item C<$Artic::XML_BASE>

Contiene el XML matriz de los articulos prontus:

    <?xml version="1.0" encoding="UTF-8"?>
    <artic_data>
    <_private>
    <_txt_titular></_txt_titular>
    <_slug></_slug>
    <_custom_slug></_custom_slug>
    <_users_id></_users_id>
    <_fid></_fid>
    <_plt></_plt>
    <_fechap></_fechap>
    <_horap></_horap>
    <_fechae></_fechae>
    <_horae></_horae>
    <_soloportadas></_soloportadas>
    <publicarvistas></publicarvistas>
    <_seccion1></_seccion1>
    <_tema1></_tema1>
    <_subtema1></_subtema1>
    <_seccion2></_seccion2>
    <_tema2></_tema2>
    <_subtema2></_subtema2>
    <_seccion3></_seccion3>
    <_tema3></_tema3>
    <_subtema3></_subtema3>
    <_nom_seccion1></_nom_seccion1>
    <_nom_tema1></_nom_tema1>
    <_nom_subtema1></_nom_subtema1>
    <_nom_seccion2></_nom_seccion2>
    <_nom_tema2></_nom_tema2>
    <_nom_subtema2></_nom_subtema2>
    <_nom_seccion3></_nom_seccion3>
    <_nom_tema3></_nom_tema3>
    <_nom_subtema3></_nom_subtema3>
    <_autor></_autor>
    <_txt_bajada></_txt_bajada>
    <_alta></_alta>
    <_indexar></_indexar>
    <_tax></_tax>
    <_tags></_tags>
    <_tagnames></_tagnames>
    <_multitag_seccion></_multitag_seccion>
    <_multitag_tema></_multitag_tema>
    <_multitag_subtema></_multitag_subtema>
    <_galeria_prontus_conf></_galeria_prontus_conf>
    <_galeria_flag_imagen_temporal></_galeria_flag_imagen_temporal>
    <_galeria_prontus_str></_galeria_prontus_str>
    <_txt_galeria_description_total></_txt_galeria_description_total>
    <_txt_galeria_credito_total></_txt_galeria_credito_total>
    <_modtime></_modtime>
    </_private>
    <_public>
    </_public>
    </artic_data>

=item C<$Artic::ERR>

Variable que se carga con el texto descriptivo del error en caso de que un metodo retorne 0.
Este texto esta adecuado para ser presentado directamente al usuario, ya que el texto completo
del error, incluyendo paths y backtrace es enviado al STDERR.

=back

=for comment
# ---------------------------------------------------------------

=head2 ATRIBUTOS

Luego de creado el objeto, se puede acceder a una serie de atributos con informacion del articulo.
Estos atributos se acceden de la forma:

    my $attr = $artic_obj->{nombre_atributo};

Los atributos son:

B<ts>

    timestamp del articulo, por ejemplo '20081023172849'

B<fechac>

    string correspondiente a la fecha de creacion del artic,
    por ejemplo '20081023'

B<fullpath_xml>

    path completo al XML del articulo.
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/xml/20081023172849.xml'

B<prontus_id>

    nombre del publicador al que pertenece el articulo,
    por ejemplo 'prontus_noticias'

B<public_server_name>

    nombre del servidor donde estara publicado el articulo,
    por ejemplo 'www.misitio.cl'

B<cpan_server_name>

    nombre del servidor desde donde se publicara el articulo,
    usualmente es el mismo que public_server_name

B<document_root>

    document root asociado al server del articulo,
    por ejemplo '/sites/misitio/web'

B<campos>

    Hash con todos los campos pasados al constructor.
    Para acceder a alguno en particular:
    my $plt = $artic_obj->{campos}->{'_plt'};

B<dst_base>

    path completo del directorio donde se ubican los archivos del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023'

B<dst_pags>

    path completo del directorio donde se ubica el html del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/pags'

B<dst_xml>

    path completo del directorio donde se ubica el xml del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/xml'

B<dst_asocfile>

    path completo del directorio donde se ubican los asocfile del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/asocfile/20081023172849'

B<dst_foto>

    path completo del directorio donde se ubican las imag del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/imag'

B<dst_multimedia>

    path completo del directorio donde se ubican los mmedia del articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/mmedia'

B<lnk_multimedia>

    path relativo al doc. root del directorio donde se ubican los mmedia del articulo,
    por ejemplo: '/prontus_noticias/site/artic/20081023/mmedia'

B<pathdir_plt>

    path completo del directorio donde se ubican las plantillas de articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/plantillas/artic/fecha'

B<pathdir_plt_macros>

    path completo del directorio donde se ubican las macros de articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/plantillas/artic/fecha/macros'

B<pathdir_plt_pags>

    path completo del directorio donde se ubican las plantillas de la vista default de articulo,
    por ejemplo: '/sites/misitio.cl/web/prontus_noticias/plantillas/artic/fecha/pags'



=for comment
# ---------------------------------------------------------------

=head2 METODOS PUBLICOS

=for comment
# ---------------------------------------------------------------

=over 8

=item B<generar_vista_art($mv, $plt, $is_paralela)>

Genera HTML (o lo que corresponda a la plantilla) del articulo y graba el archivo.

Toma como entrada los datos del XML, no los que vengan en el hash $this->{campos}.

B<Parametros>

$mv

  nombre de la vista en la que se desea generar el html articulo.
  para la vista default, poner ''. Para otras vistas poner por ejemplo: 'pda'

$plt

  nombre de la plantilla de la vista a generar

$is_paralela

  Flag para indicar si es una plantilla paralela

$only_pproc

  Flag para indicar si está corriendo por postprocesos.

B<Retorna>

1 | 0 (exito o error).
En caso de error, ademas setea la variable de clase $Artic::ERR con el texto correspondiente, y hace print STDERR de ese error.

=for comment
# ---------------------------------------------------------------

=item B<generar_xml_artic()>

Genera la estructura de datos esencial para guardar en XML, y escribe a disco. Los datos derivados que se calculan u obtienen en base a los esenciales no se almacenan aqui.

B<Parametros>

void

B<Retorna>

1 | 0 (exito o error) directo de _flush_xml()

=for comment
# ---------------------------------------------------------------

=item B<get_fullpath_artic($plt, $mv, $is_paralela)>

Obtiene el path completo al html del artic., por ejemplo:
'/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/pags/20081023172849.html'

B<Parametros>

$plt

  nombre del archivo de plantilla de artic.
  Puede obtenerse de los campos recibidos del form, o bien,
  desde el xml, invocando antes a get_xml_content()

$mv

  nombre de la vista en la que se desea generar el html articulo.
  para la vista default, poner ''. Para otras vistas poner
  por ejemplo: 'pda'

$is_paralela
  Indica si se esta procesando una plantilla paralela o no



B<Retorna>

El path completo al html del artic., por ejemplo:
'/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/pags/20081023172849.html'

=for comment
# ---------------------------------------------------------------

=item B<get_xml_content()>

Obtiene un hash con los campos del XML del artic.
Cada entrada es del tipo nom_campo => valor_campo
Ya vienen eliminados los bloques CDATA, en caso de haberlos.

Uso tipico:

  my %campos_xml = $artic_obj->get_xml_content();

B<Retorna>

Hash con los campos del XML del artic.

=for comment
# ---------------------------------------------------------------

=item B<parse_artic_data($fullpath_vista, $buffer, \%vars_adicionales, $use_xml_tax)>

Parsea todas las marcas del XML mas algunas derivadas de estas en el buffer dado.
Este metodo NO elimina las marcas que queden sin sustituir.

Uso tipico:

  $buffer = $artic_obj->parse_artic_data($buffer, \%vars_adicionales);

B<Parametros>

$buffer

  string, normalmente html, en donde se deben parsear los datos del articulo

\%vars_adicionales

  Hash con campos custom que se deseen parsear en el buffer.

B<Retorna>

String con buffer parseado


=for comment
# ---------------------------------------------------------------

=item B<get_nom_artic($plt)>

Obtiene el nombre del archivo del articulo, por ejemplo:
'20081023172849.html'

B<Parametros>

$plt

  nombre del archivo de plantilla de artic.
  Puede obtenerse de los campos recibidos del form, o bien,
  desde el xml, invocando antes a get_xml_content():

  my %campos_xml = $artic_obj->get_xml_content();
  my $plt = $campos_xml{'_plt'};

  o bien,desde el mismo hash de campos pasado al constructor:

  my $plt = $artic_obj->{campos}->{'_plt'};

B<Retorna>:

El path completo al html del artic., por ejemplo:
'/sites/misitio.cl/web/prontus_noticias/site/artic/20081023/pags/20081023172849.html'

=for comment
# ---------------------------------------------------------------

=item B<art_insert_bd($base)>

Crea articulo en base de datos prontus a partir de info del xml.

Uso tipico:

  # Conectar a BD Prontus
  my ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();
  $id = $ARTIC_OBJ->art_insert_bd($base);

  my ($this, $base) = @_; # basicamente apara regeneracion de tabla de arts.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

Id del articulo.

=for comment
# ---------------------------------------------------------------

=item B<exec_post_proceso_art($pathArticulo, $isNew)>

Una vez escrito el articulo, ubica el nombre del script de post-proceso (con extensión y con path absoluto completo) y lo ejecuta.
El script de postproceso es optativo y puede ir en cualquier parte del articulo. Pueden ser varios.
Usa $this->{post_proceso_lista}, una lista de postprocesos separada por \t

B<Parametros>

$pathArticulo:

  el path completo al artículo.

$isNew N o E (nuevo o edición)

B<Retorna>

  void

=for comment
# ---------------------------------------------------------------

=item B<art_update_bd($base)>

Actualiza base de datos prontus a partir de info del xml

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

ID del artículo en base de datos, o 0.

=for comment
# ---------------------------------------------------------------

=item B<guardar_pub_exp_artic($base, $comando)>

Recibe una ref a la base de datos, y un comando (pub|exp).
hace un insert o update de los datos en la tabla art_publicar o art_despublicar, según el comando pasado.
Si la fecha pasada es anterior a la fecha actual, intenta hacer un delete de la misma tabla.

B<Parametros>

$base

  handler de la BD Prontus

$comando

  'pub' o 'exp' para especificar publicar o despublicar.

B<Retorna>

  1 en caso de éxito, 0 de otro modo. Setea $Artic::Err y hace print STDERR del eventual error.

=for comment
# ---------------------------------------------------------------

=item B<borrar_pub_exp_artic($base, $comando)>

Recibe una ref a la base de datos, y un comando (pub|exp).
borra todos los registros en la tabla art_publicar o art_despublicar con fecha anterior a la actual, según el comando pasado.

B<Parametros>

$base

  handler de la BD Prontus

$comando

  'pub' o 'exp' para especificar publicar o despublicar.

B<Retorna>

  1 en caso de éxito, 0 de otro modo. Setea $Artic::Err y hace print STDERR del eventual error.

=for comment
# ---------------------------------------------------------------

=item B<tags2bd($base)>

Guarda tags en base de datos. Se invoca al guardar un artículo.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

1 si todo anduvo bien, 0 de otro modo. Hace print a STDERR del eventual error.

=for comment
# ---------------------------------------------------------------

=item B<multitags2bd($base)>

Puebla tabla de multitags en base de datos. Se invoca al guardar un artículo.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

1 si todo anduvo bien, 0 de otro modo. Hace print a STDERR del eventual error.

=for comment
# ---------------------------------------------------------------

=item B<check_publish_art()>

Verifica si el articulo es visible. El criterio de publicación es
- si es una preview, es visible. De otro modo, deben cumplirse:
- alta = 1
- fecha de publicacion es anterior a la fecha actual
- fecha de expiracion es posterior a la fecha actual
- si esta marcado "solo despublicar de portadas" el articulo es visible

B<Parametros>

  void

B<Retorna>

1 si se cumplen los criterios detallados, 0 de otro modo.

=for comment
# ---------------------------------------------------------------

=item B<borra_pags_artic($base)>

Invocada al eliminar vistas de artículos para el cron fechas.
Elimina todos los archivos de vista.
Elimina archivos de friendly URL.
No elimina XML ni archivos asociados.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

''

=for comment
# ---------------------------------------------------------------

=item B<borra_artic($base, $no_run_regen, $list_procs)>

Elimina un artículo con todos sus datos asociados.

B<Parametros>

$base

  handler de la BD Prontus

$no_run_regen

  flag para evitar regenerar tagonómicas.

$list_procs

  referencia a hash de lista de procesos a ejecutar como parte de la eliminación.

B<Retorna>

''

=for comment
# ---------------------------------------------------------------

=item B<borrar_artic_files($dir_aux)>

Elimina los archivos y directorios asociados a un artículo dentro del directorio especificado.

B<Parametros>

$dir_aux

  el directorio que contiene subdirectorios y/o archivos a borrar. ej: dir de imágenes, de multimedia, de thumbnails

B<Retorna>

  void

=for comment
# ---------------------------------------------------------------

=item B<art_delete_bd($base)>

Borra la fila de un artículo en la tabla ART de la base de datos.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

1 si logró borrar, 0 de otro modo. Hace print a STDERR del eventual error.

=for comment
# ---------------------------------------------------------------

=item B<art_update_bd($base)>

Actualiza el registro de un artículo en la tabla ART de la base de datos.

B<Parametros>

$base

  handler de la BD Prontus

B<Retorna>

Id del artículo en db, 0 de otro modo. setea $Artic::ERR con el eventual error.

=for comment
# ---------------------------------------------------------------

=item B<ajusta_campos_art4bd($base, %refhash_campos)>

Limpia y formatea los campos a ingresar en base de datos.

TODO: hacer que reciba y retorne una referencia, o que reciba y retorne un hash.

B<Parametros>

$refhash_campos

  referencia a hash con los campos del artículo.

B<Retorna>

Hash con los campos formateados correctamente.

=for comment
# ---------------------------------------------------------------

=item B<set_preview_artic()>

Establece que el ts para todos los efectos es 'preview'. Usar al generar preview de artículos.

B<Parametros>

  void

B<Retorna>

  void

=for comment
# ---------------------------------------------------------------

=for comment
# ---------------------------------------------------------------

=back

=for comment
# ---------------------------------------------------------------

=head2 METODOS PRIVADOS

=for comment
# ---------------------------------------------------------------

=over 8

=item B<_flush_xml()>

Escribe a disco el XML del artículo.

Usada en generar_xml_artic()

B<Parametros>

void

B<Retorna>

1 (éxito), 0 (error). En caso de error setea $Artic::ERR y hace print STDERR.

=for comment
# ---------------------------------------------------------------

=item B<_guarda_recursos($type)>

Guarda en xml y disco los recursos de un cierto tipo (FOTO, ASOCFILE, MULTIMEDIA) asociados al articulo

Usada en generar_xml_artic()

B<Parametros>

$type

  /^(foto|asocfile|multimedia)$/

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_get_newnom_arch($type, $nom_campo, $val_campo, $arch_existente, $nom_foto_orig)>

Obtiene un nuevo nombre para el recurso del tipo dado.

Usada en _guarda_recursos()

B<Parametros>

$type

  /^(foto|asocfile|multimedia)$/

$nom_campo

  el nombre del campo a procesar

$val_campo

  el valor contenido en $nom_campo

$arch_existente

  el archivo existente

$nom_foto_orig

  el archivo de foto original

B<Retorna>

El nuevo nombre.

=for comment
# ---------------------------------------------------------------

=item B<_get_nom_asocfile($texto, $af_existente)>

Retorna nombre de archivo para asocfile_i, con extension y sin path

Usada en _get_newnom_arch()

B<Parametros>

$texto

  el texto original del nombre de archivo

$af_existente

  el archivo original

B<Retorna>

El nuevo nombre.

=for comment
# ---------------------------------------------------------------

=item B<_artic_ajusta_horas()>

Ajusta horas desde los valores de entrada antes de guardarlos en el XML

Usada en generar_xml_artic()

B<Parametros>

void

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_get_fulldir_vista($mv)>

Lee el path en this->dst_pags, retorna el mismo path pero adaptado a la vista pasada.

Ejemplo: my $fulldir_vista = $this->_get_fulldir_vista('movil');

Usada en generar_vista_art() y get_fullpath_artic()

B<Parametros>

$mv

  nombre de la vista: '', 'movil', 'en', ...

B<Retorna>

El path contenido en this->dst_pags adaptado según la vista.

=for comment
# ---------------------------------------------------------------

=item B<_set_dirs()>

Configura directorios de trabajo del articulo.

Usada en new()

B<Parametros>

void

B<Retorna>

1

=for comment
# ---------------------------------------------------------------

=item B<_parsing_mmedia($nom_campo, $val_campo, $buffer)>

Parsea en vista del artic los recursos de un cierto tipo (ASOCFILE, MULTIMEDIA) asociados al mismo

Usada en parse_artic_data()

B<Parametros>

$nom_campo

  ASOCFILE_xx, MULTIMEDIA_xx

$val_campo

  el valor del campo $nom_campo

$buffer

  el buffer del artículo

B<Retorna>

El buffer procesado.

=for comment
# ---------------------------------------------------------------

=item B<_parsing_fotos($nom_campo, $val_campo, $buffer)>

Parsea en vista del artic las fotos (FOTOFIJA) asociadas al mismo

Usada en parse_artic_data()

B<Parametros>

$nom_campo

  FOTOFIJA_xx

$val_campo

  el valor del campo $nom_campo

$buffer

  el buffer del artículo

B<Retorna>

El buffer procesado.

=for comment
# ---------------------------------------------------------------

=item B<_parsing_vtxt($buffer, $nom_campo, $val_campo)>

Parsea en vista del artic los campos vtxt asociadas al mismo.

Usada en parse_artic_data()

B<Parametros>

$buffer

  el buffer del artículo

$nom_campo

  el nombre del campo a procesar, debe ser un vtxt.

$val_campo

  el valor del campo $nom_campo

B<Retorna>

El buffer procesado.

=for comment
# ---------------------------------------------------------------

=item B<_get_data4subtit($buffer, $nom_campo)>

Parsea el buffer de artículo buscando campos de subtítulo.

Obtiene un template de indice de titulos, y busca las marcas _SUBTIT_HTML_xx, devuelve el loop repetido para cada título y el valor de cada título.

Usada en _parsing_vtxt().

B<Parametros>

$buffer

  el buffer del artículo

$nom_campo

  el nombre del campo a buscar

B<Retorna>

Array con el segmento de código y el título correspondientes.

=for comment
# ---------------------------------------------------------------

=item B<_add_foto_prontus_xml($wfoto, $hfoto, $nom_arch, $size, $original)>

Parsea los datos de una foto y los guarda al xml de artículo.

Usada en _guarda_recursos(), _add_foto_sitiolocal_articulolocal(), _add_foto_sitiolocal_articuloexterno(), _add_foto_filesystem()

B<Parametros>

$wfoto

  el ancho de la foto

$hfoto

  El alto de la foto

$nom_arch

  el nombre del archivo
$size

  tamaño de archivo en bytes

$original

  indica si la foto es original, fue subida por el usuario o no (fue recortada o generada por fotofija)

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_genera_fotofija($nom_foto, $fotofija, $maxw, $maxh, $cuadrar)>

Genera una fotofija con nuevas dimensiones a partir de los datos pasados.
Valida datos y llama a $this->_add_foto_sitiolocal_articulo_local()

Usada en CGIs pese a llamarse con _. cgi-cpn/prontus_art_sbmit.cgi, cgi-cpn/prontus_art_sbmit_ajax.cgi

B<Parametros>

$nom_foto

  El nombre de la foto_n. Ej: foto_134520110619122632.jpg

$fotofija
  El nombre de la foto fija (nombre marca marca)

$maxw

  El ancho maximo

$maxh

  El alto maximo

$cuadrar

  Si se debe cuadrar la imagen o no: si|<anything>

B<Retorna>

Exito:

  Array ( '', #mensaje de error
    $final_dimx, # dimensión X
    $final_dimy, # dimensión Y
    $final_nomfoto, # nombre final
    $final_relfoto # path relativo
    )

Error:
  Array( Mensaje de error, '', '', '','' )

=for comment
# ---------------------------------------------------------------

=item B<_get_nom_foto($path_foto, $nom_foto_orig)>
Obtiene nombre de la siguiente foto que toca para este articulo. Dicho nombre comenzara con el correlativo sgte al ultimo ingresado

Aplica un valor random para minimizar que el nombre no se repita

Procesa el nombre original de la foto para agregarlo al path final

Usada en _add_foto_filesystem(), _guarda_binfoto_prontus(), _get_newnom_arch()

B<Parametros>

$path_foto

  el path a la imagen

$nom_foto_orig

  Nombre original de la foto. si se pasa este parametro, se usa para extraer la extensión, se limpia el nombre y se agrega al final del nuevo nombre.

B<Retorna>

El nuevo nombre a usar.

=for comment
# ---------------------------------------------------------------

=item B<_guarda_fotosfijas()>

Guarda imágenes externas o locales al prontus. Delega el trabajo a subs como _add_foto_sitioexterno().

Usada en generar_xml_artic(),

B<Parametros>

void

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_guarda_fotos_posting($nom_campo_foto, nom_foto)>

Guarda imágenes enviadas via form posting.

Usada en _guarda_recursos()

B<Parametros>

$nom_campo_foto

  nombre del campo xml para la imagen. Usa el match /^foto_posting_.*?$/

$nom_foto

  nombre del archivo

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_guarda_binfoto_prontus($val_campo, $final_dimx, $final_dimy, $binfoto, $arch_existente, $nom_foto_orig)>

Genera un archivo binario a partir de una foto. En caso de que el original sea un GIF, genera un archivo PNG.
Usada en _add_foto_sitiolocal_articuloexterno() y en add_foto_sitiolocal_articulolocal()

B<Parametros>

$val_campo

  path al archivo

$final_dimx

  dimension horizontal final

$final_dimy

  dimensión vertical final

$binfoto

  archivo de foto

$arch_existente

  archivo original con la foto.

$nom_foto_orig

  nombre inicial del archivo de la foto.

B<Retorna>

El nombre del archivo guardado, o '' en caso de error en parámetros.

=for comment
# ---------------------------------------------------------------

=item B<_add_foto_filesystem($path_foto, $nom_foto_orig)>

Copia un archivo de foto, y agrega los datos de la foto (nombre, dimensiones) al xml del artículo.

A pesar de llamarse con _, es usada solamente en cgis. (prontus_art_sbmit*.cgi, prontus_art_upload.cgi, prontus_editor_imag_guardar.cgi)

B<Parametros>

$path_foto

  el path al archivo sin document_root

$nom_foto_orig

  El path a la foto original

B<Retorna>

El nombre nuevo de la foto.

=for comment
# ---------------------------------------------------------------

=item B<_add_foto_sitioexterno($nom_campo, $val_campo)>

Carga en el objeto xml_data los datos de una foto externa a Prontus.

Usada en _guarda_fotosfijas()

B<Parametros>

$nom_campo

  El nombre del campo a procesar

$val_campo

  El valor de $nom_campo

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_add_foto_sitiolocal_articulolocal($nom_campo, $val_campo, $maxw, $maxh, $dx, $dy, $cuadrar, $nom_campo_orig, $nom_foto_orig)>

Guarda una foto del artículo basada en otra foto del mismo artículo

Usada en _guarda_fotosfijas(), _guarda_fotos_posting(), _genera_fotofija()

B<Parametros>

$nom_campo

  nombre del campo a procesar.

$val_campo

  valor del campo $nom_campo

$maxw

  ancho maximo

$maxh

  alto maximo

$dx

  dimension x

$dy

  dimensión y

$cuadrar

  flag para indicar si debe cuadrarse la imagen

$nom_campo_orig

  nombre del campo del que proviene la foto

$nom_foto_orig

  nombre del archivo de foto original

B<Retorna>

Array (
    $final_dimx,
    $final_dimy,
    $nom_arch,
    $val_campo,
)

=for comment
# ---------------------------------------------------------------

=item B<_add_foto_sitiolocal_articuloexterno($nom_campo, $val_campo, $maxw, $maxh, $dx, $dy, $cuadrar)>

Guarda una foto externa al articulo pero local al prontus

Si la foto externa es mas grande de lo que debe ser, se redimensiona. Requiera o no ser redimensionada, siempre se genera una nueva foto prontus para el articulo y luego esta nueva foto, ya internalizada, se asigna a la fotofija.

Usada en _guarda_fotosfijas()

B<Parametros>

$nom_campo

  nombre del campo a procesar.

$val_campo

  valor del campo $nom_campo

$maxw

  ancho maximo

$maxh

  alto maximo

$dx

  dimension x

$dy

  dimensión y

$cuadrar

  flag para indicar si debe cuadrarse la imagen

B<Retorna>

void

=for comment
# ---------------------------------------------------------------

=item B<_check_ext_foto($path_foto)>

Valida que el nombre de archivo pasado termine en una extensión de imagen válida ( jpe?g | gif | png )

Usada en _guarda_binfoto_prontus(), _get_newnom_arch()

B<Parametros>

$path_foto

  el path al archivo

B<Retorna>

1 en caso de ser una extensión válida, 0 de otro modo

=for comment
# ---------------------------------------------------------------

=item B<_get_new_ts()>

Asigna un nuevo timestamp único para el artículo

Usada en new()

B<Parametros>

void

B<Retorna>

$ts, el nuevo timestamp del artículo (yyyymmddhhmmss)

=for comment
# ---------------------------------------------------------------

=item B<_check_artic_dirs()>

Revisa la existencia de todos los directorios necesarios para la creación del artículo.

Setea la variable global $ERR, TODO: probablemente hay que unificar esto con $Artic::ERR para hacer el código más legible.

Usada en _set_dirs()

B<Parametros>

void

B<Retorna>

1 (éxito), 0 (error)

=for comment
# ---------------------------------------------------------------
=item B<get_friendly_url()>

Obtiene la URL friendly del artículo según la configuración de Prontus.

Usada en prontus_art_sbmit.cgi para pasar la URL friendly a Twitter.

Usa la sub lib_prontus::parse_filef().

B<Parametros>

void

B<Retorna>

Una string con la URL friendly (o la URL común).

=for comment
# ---------------------------------------------------------------


=item B<_genera_lista_post_procesos()>

Parsea el buffer de artículo buscando tags de postproceso.
Agrega cada postproceso encontrado a la variable Artic->post_proceso_lista.

B<Parametros>

$buffer, string, el buffer de plantilla.

B<Retorna>

el buffer de plantilla, limpio.

=for comment
# ---------------------------------------------------------------


=back

=head1 AUTHORS

Por favor reportar bugs usando el formulario de contacto en https://www.prontus.cl/

Alfredo Luco G., E<lt>aluco@altavoz.netE<gt>

Pierina Riffo B., E<lt>priffo@altavoz.netE<gt>

Yerko Chapanoff,

César Vasquez I.,

Eduardo Alarcón G., E<lt>ealarcon@altavoz.net<gt>

Jorge Olivares R., E<lt>jolivares@altavoz.net<gt>

Sebastián Castillo T., E<lt>scastillo@altavoz.net<gt>

Adriano Varoli Piazza G., E<lt>avaroli@altavoz.net<gt>

Manuel Tabilo C.,

Ricardo Vivanco, E<lt>rvivanco@altavoz.net<gt>

Daniel García O.,

Anderson Bracho D.,

Ignacio Nicovani C.,

Waldo Morales A.,

Hector Cortés,

Nicolás Albornoz R.,

Manuel Pedraza G.,

=cut
